﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System;
using System.Collections.Generic;

namespace BeamColumnWeld107
{
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private ConnectionComponent m_comp = null;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine = Vec3.ZERO;

        private BeamPlanes m_mainPlanes = null;

        private Beam m_subPart = null;

        private Vec3 m_subLine = Vec3.ZERO;

        private BeamPlanes m_subPlanes = null;

        private Plane m_faceSubNearPlane = null;

        private Plane m_faceSubFarPlane = null;
        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 SubLine { get => m_subLine; set => m_subLine = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane FaceSubNearPlane { get => m_faceSubNearPlane; set => m_faceSubNearPlane = value; }
        public Plane FaceSubFarPlane { get => m_faceSubFarPlane; set => m_faceSubFarPlane = value; }
        public Plane SubAlignPlane { get; set; }
        public List<PartBase> Stiffs { get; set; }
        public List<PartBase> DoublePlate { get; set; }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (comp.Subs == null)
                return false;
            Beam subPart = comp.Subs.First() as Beam;
            if (mainPart == null || subPart == null)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);
            m_subPart = subPart;
            m_subPlanes = BeamPlanes.CreateFromBeam(subPart);
            //验证主零件
            if (m_mainPlanes.BackWeb == null || m_mainPlanes.FrontWeb == null ||
                m_mainPlanes.TopFlangeInside == null || MainPlanes.BottomFlangeInside == null
                || m_mainPlanes.TopFlange == null || m_mainPlanes.BottomFlange == null)
                return false;
            //验证次零件
            if (m_subPlanes.TopFlange == null || m_subPlanes.BottomFlange == null)
                return false;
            if (!DefineSubLine(subPart))
                return false;
            this.ParseRefPlane();
            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var dir = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!dir.HasValue)
                return false;
            m_mainLine = dir.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 调整次零件的构建方向
        /// </summary>
        private bool DefineSubLine(Beam subPart)
        {
            //调整为靠近主零件的控制点为第一控制点
            var point1 = subPart.CtrlPoints?.Last()?.WorldPos;
            var point2 = subPart.CtrlPoints?.First()?.WorldPos;
            var dir = point1 - point2;
            if (!dir.HasValue)
                return false;
            Vec3 origin = m_mainPart.CtrlPoints.First().WorldPos;
            var dir1 = point1.Value - origin;
            var dir2 = point2.Value - origin;
            double dist1 = dir1.Cross(m_mainLine).Length;
            double dist2 = dir2.Cross(m_mainLine).Length;
            if (dist1 > dist2)
                m_subLine = dir.Value.Normalize();
            else
                m_subLine = -dir.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 解析主零件面向次零件的面
        /// </summary>
        private bool ParseRefPlane()
        {
            //主零件面向次零件较近的那一侧为近侧 反之为远侧
            var flag1 = m_subLine.Dot(m_mainPlanes.BackWeb.Normal.Normalize());
            var flag2 = m_subLine.Dot(m_mainPlanes.TopFlange.Normal.Normalize());
            if (Math.Abs(flag1) > Math.Abs(flag2))
            {
                if(flag1 > 0)
                {
                    m_faceSubNearPlane = m_mainPlanes.BackWeb;
                    m_faceSubFarPlane = m_mainPlanes.FrontWeb;
                }
                else
                {
                    m_faceSubNearPlane = m_mainPlanes.FrontWeb;
                    m_faceSubFarPlane = m_mainPlanes.BackWeb;
                }
            }
            else
            {
                if(flag2 > 0)
                {
                    m_faceSubNearPlane = m_mainPlanes.TopFlange;
                    m_faceSubFarPlane = m_mainPlanes.BottomFlange;
                }
                else
                {
                    m_faceSubNearPlane = m_mainPlanes.BottomFlange;
                    m_faceSubFarPlane = m_mainPlanes.TopFlange;
                }
            }
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
            m_subPart = null;
            m_subPlanes = null;
        }
    }
}
