﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using LoongCAD.STC;
using System;

namespace BeamColumnWeld107
{
    
    public class DoubleHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private PlateProperties m_plateProps = new PlateProperties();
        private PartBase nsPlate = null;
        private PartBase fsPlate = null;
        private double m_height = 0.0;
        //解析面
        private Plane m_nsWeb = null;
        private Plane m_fsWeb = null;
        private Plane m_nsFlange = null;
        private Plane m_fsFlange = null;
        private Plane m_subRefPlane = null;
        //孔属性
        private List<double> m_horiStride = new List<double>();
        private List<double> m_vertStride = new List<double>();
        private double m_horiSum = 0.0;
        private double m_vertSum = 0.0;
        private double m_upGap = double.MaxValue;
        private double m_downGap = double.MaxValue;
        private double m_leftGap = double.MaxValue;
        private double m_rightGap = double.MaxValue;
        private double m_holeDiameter = 0.0;
        private double m_flangeGap = double.MaxValue;
        private double m_bottomGap = double.MaxValue;
        private double m_soltWidth = 0.0;
        private double m_soltHeight = 0.0;
        public DoubleHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;
            int platePos = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_CREATION_MODE);
            if (platePos < 2)
                return false;
            bool nsEnabled = false;
            bool fsEnabled = false;
            switch (platePos)
            {
                case 2:
                    fsEnabled = true;
                    break;
                case 3:
                    nsEnabled = true;
                    break;
                case 4:
                    fsEnabled = true;
                    nsEnabled = true;
                    break;
                default:
                    break;
            }
            //基于右手坐标系确定向量旋转方向
            var normal = m_context.TargetComp.LocalMatrix.Z;
            m_context.DoublePlate = new List<PartBase>();
            if (nsEnabled)
            {
                nsPlate = CreatePlate(m_nsWeb);
                if(nsPlate != null)
                {
                    AdaptPlateShape(nsPlate, m_nsWeb, normal);
                    CreateCutter(nsPlate, m_nsWeb);
                    m_context.DoublePlate.Add(nsPlate);
                }
            }
            if (fsEnabled)
            {
                fsPlate = CreatePlate(m_fsWeb);
                if(fsPlate != null)
                {
                    AdaptPlateShape(fsPlate, m_fsWeb, normal);
                    CreateCutter(fsPlate, m_fsWeb);
                    m_context.DoublePlate.Add(fsPlate);
                }
            }
            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.DOUBLE_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);

            m_leftGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_LEFT_MARGIN);
            m_rightGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_RIGHT_MARGIN);
            m_upGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_TOP_MARGIN);
            m_downGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_BOTTOM_MARGIN);
            m_holeDiameter = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_DIAMETER);
            m_flangeGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_FLANGE_GAP);
            m_bottomGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_BOTTOM_GAP);
            m_soltWidth = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_SLOT_WIDTH);
            m_soltHeight = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_SLOT_HEIGHT);

            int horiNum = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_HOLE_HORI_COUNT);
            var arrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.DOUBLE_HOLE_HORI_STRIDE);
            m_horiStride = HandlerStride(arrY, horiNum);
            int vertNum = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_HOLE_VERT_COUNT);
            var arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.DOUBLE_HOLE_VERT_STRIDE);
            m_vertStride = HandlerStride(arrX, vertNum);
            m_horiSum = SumArray(m_horiStride);
            m_vertSum = SumArray(m_vertStride);
            if (!ParsePlaneProps())
                return false;
            return true;
        }
        private bool ParsePlaneProps()
        {
            //验证主次零件相对位置是否正确
            Vec3 dir = m_context.TargetComp.LocalMatrix.Z;
            var mainPlanes = m_context.MainPlanes;
            bool flag1 = Math.Abs(dir.Dot(mainPlanes.BackBound.Normal)) > 1e-3;
            bool flag2 = Math.Abs(dir.Dot(mainPlanes.TopBound.Normal)) > 1e-3;
            //次零件翼缘面向我们不创建组件
            if (flag1 || flag2)
                return false;
            var subPlanes = m_context.SubPlanes;
            //基于冒尖方向确定次零件bottom翼缘面
            Plane bottomPlane;
            if (subPlanes.TopBound.Normal.Dot(dir) < 0)
                bottomPlane = subPlanes.TopBound;
            else
                bottomPlane = subPlanes.BottomBound;
            var doubleDir = m_paramsHandler.GetIntValue(ParamNames.STIFF_DIR);
            //由加劲肋方向控制参照面是否跟随翼缘面方向
            if (doubleDir == 2)
            {
                var point = bottomPlane.Intersect(m_context.SubAlignPlane, subPlanes.BackBound);
                m_subRefPlane = new Plane(point, dir).Move(m_bottomGap);
            }
            else
            {
                m_subRefPlane = bottomPlane.Invert().Move(m_bottomGap);
            }
            //主零件靠近次零件的那一侧为近侧 -- 左右近侧
            //主零件面向我们的那一侧为近侧 -- 前后近侧
            Vec3 nsDir = m_context.FaceSubNearPlane.Normal;
            Plane nearFlange, farFlange;
            if (Math.Abs(mainPlanes.TopBound.Normal.Dot(nsDir)) < 1e-3)
            {
                m_nsWeb = m_context.FaceSubNearPlane;
                m_fsWeb = m_context.FaceSubFarPlane;
                //基于世界坐标系方向确定前后近侧面
                var worldDir = ParseWorldDir(mainPlanes.TopBound.Normal);
                if(worldDir.Dot(mainPlanes.TopBound.Normal) > 0)
                {
                    nearFlange = mainPlanes.BottomBound;
                    farFlange = mainPlanes.TopBound;
                }
                else
                {
                    nearFlange = mainPlanes.TopBound;
                    farFlange = mainPlanes.BottomBound;
                }
            }
            else
            {
                if(mainPlanes.TopBound.Normal.Dot(nsDir) > 0)
                {
                    nearFlange = mainPlanes.TopBound;
                    farFlange = mainPlanes.BottomBound;
                }
                else
                {
                    nearFlange = mainPlanes.BottomBound;
                    farFlange = mainPlanes.TopBound;
                }
                //基于世界坐标系方向确定前后近侧面
                var worldDir = ParseWorldDir(mainPlanes.BackWeb.Normal);
                if (worldDir.Dot(mainPlanes.BackWeb.Normal) > 0)
                {
                    m_nsWeb = mainPlanes.FrontWeb;
                    m_fsWeb = mainPlanes.BackWeb;
                }
                else
                {
                    m_nsWeb = mainPlanes.BackWeb;
                    m_fsWeb = mainPlanes.FrontWeb;
                }
            }
            if(m_flangeGap != double.MaxValue)
            {
                m_fsFlange = farFlange.Move(-m_flangeGap);
                m_nsFlange = nearFlange.Move(-m_flangeGap);
            }
            else
            {
                var dist = nearFlange.Dist(farFlange.Origin);
                var offSet = m_leftGap + m_rightGap + m_horiSum;
                m_fsFlange = farFlange.Move(-dist / 2 + offSet / 2);
                m_nsFlange = nearFlange.Move(-dist / 2 +offSet / 2);
            }
            return true;
        }
        /// <summary>
        /// 创建板
        /// </summary>
        /// <param name="webPlane">腹板面</param>
        private PartBase CreatePlate(Plane webPlane)
        {
            if (m_plateProps.Thickness <= 0 || m_plateProps.Height <= 0)
                return null;
            List<Plane> plsSide = new List<Plane>();
            plsSide.Add(m_subRefPlane);
            plsSide.Add(m_fsFlange);
            if(m_plateProps.Height != double.MaxValue)
            {
                m_height = m_plateProps.Height;
                plsSide.Add(m_subRefPlane.Move(m_height));
            }
            else
            {
                m_height = m_upGap + m_downGap + m_vertSum;
                plsSide.Add(m_subRefPlane.Move(m_height));
            }
            plsSide.Add(m_nsFlange);
            var retPlate = m_context.TargetComp.CreateSteelPlate(webPlane,
                m_plateProps.Thickness, plsSide.ToArray(), true);
            if (retPlate == null)
                return null;
            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);
            CreateHoles(webPlane, m_height, retPlate);
            return retPlate;
        }
        private bool CreateHoles(Plane webPlane, double height, PartBase retPlate)
        {
            if (m_holeDiameter <= 0 || m_horiStride == null || m_vertStride == null)
                return false;
            var width = m_fsFlange.Dist(m_nsFlange.Origin);
            if (width - m_leftGap - m_rightGap < m_horiSum 
                || height - m_upGap - m_downGap < m_vertSum)
                return false;
            Vec3 boltX = -m_context.TargetComp.LocalMatrix.Z;
            Vec3 boltY = m_nsFlange.Normal.Normalize();
            Vec3 boltZ = boltX.Cross(boltY);
            Vec3 startPos = m_subRefPlane.Move(height).Intersect(webPlane, m_fsFlange);
            Vec3 endPos = startPos + boltX * height;
            BoltGroup curHoles = m_context.TargetComp.CreateBoltHoles(null, retPlate, startPos, endPos, true);
            if (curHoles == null)
                return false;
            curHoles.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            curHoles.Rotation = 0;
            curHoles.RotationSide = Define.ObjectSide.SIDE_TOP;
            curHoles.BoltSize = m_holeDiameter;
            curHoles.HoleX = m_soltHeight;
            curHoles.HoleY = m_soltWidth;
            curHoles.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
            curHoles.OverSizeHoleEnable0 = true;
            curHoles.OverSizeHoleEnable1 = true;
            var xGap = m_upGap;
            if (height - m_upGap - m_downGap > m_vertSum)
                xGap = height / 2 - m_vertSum / 2;
            var yGap = m_leftGap;
            if (width - m_leftGap - m_rightGap > m_horiSum)
                yGap = width / 2 - m_horiSum / 2;
            curHoles.CutLength = m_plateProps.Thickness * 2.0 + 1.0;
            curHoles.Shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_LIST;
            var strideY = GetHoleStride(m_horiStride, yGap);
            var strideX = GetHoleStride(m_vertStride, xGap);
            var listX = HoleStragger(m_subRefPlane.Move(height).Invert(), 
                m_fsFlange.Invert(), startPos, strideX, strideY);
            if (listX == null)
                return false;
            var listY = new List<double>();
            double sum = 0;
            for(int i = 0; i < strideY.Length; i++)
            {
                sum += strideY[i];
                for (int j = 0; j < strideX.Length; j++)
                    listY.Add(sum);
            }
            curHoles.ListX = listX;
            curHoles.ListY = listY.ToArray();
            curHoles.Refresh();
            return true;
        }
        /// <summary>
        /// 得到螺栓组的位置间距值
        /// </summary>
        /// <param name="stride">螺栓组间距</param>
        /// <param name="gap">螺栓组与板边缘间距</param>
        /// <returns></returns>
        private double[] GetHoleStride(List<double> stride, double gap)
        {
            List<double> list = new List<double>();
            list.Add(gap);
            if (stride == null || stride.Count < 1)
                return list.ToArray();
            foreach (var item in stride)
                list.Add(item);
            return list.ToArray();
        }
        /// <summary>
        /// 螺栓交错
        /// </summary>
        /// <param name="xPlane">X间距螺栓参照面</param>
        /// <param name="yPlane">Y间距螺栓参照面</param>
        /// <param name="refPlane">螺栓组起点</param>
        /// <param name="strideX">X方向螺栓组间距</param>
        /// <param name="strideY">Y方向螺栓组间距</param>
        /// <returns></returns>
        private double[] HoleStragger(Plane xPlane, Plane yPlane, Vec3 startPos, double[] strideX, double[] strideY)
        {
            int lenX = strideX.Length;
            int lenY = strideY.Length;
            if (lenX < 1 || lenY < 1 || xPlane == null || yPlane == null)
                return null;
            Plane refPlane = new Plane(startPos, -m_context.TargetComp.LocalMatrix.Z);
            List<double> offset = new List<double>();
            var offsetYPlane = new Plane(yPlane);
            for (int i = 0; i < lenY; i++)
            {
                offsetYPlane.MoveThis(strideY[i]);
                var offsetXPlane = new Plane(xPlane);
                for (int j = 0; j < lenX; j++)
                {
                    offsetXPlane.MoveThis(strideX[j]);
                    var point = m_context.MainPlanes.BackBound.Intersect(offsetYPlane, offsetXPlane);
                    offset.Add(refPlane.SignedDist(point));
                }
            }
            return offset.ToArray();
        }
        /// <summary>
        /// 调整板边缘形状
        /// </summary>
        /// <param name="retPlate">板</param>
        /// <param name="webPlane">腹板面</param>
        /// <param name="normal">旋转向量参照的轴向量</param>
        private bool AdaptPlateShape(PartBase retPlate, Plane webPlane, Vec3 normal)
        {
            int plateEdgeShape = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_PLATE_EDGE_SHAPE);
            if (plateEdgeShape == 2)
                return false;
            double bevelAngle = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_PLATE_BEVEL_ANGLE);
            if (bevelAngle <= 0 || bevelAngle >= 90)
                return false;
            var ctrlPoints = retPlate.CtrlPoints;
            if (ctrlPoints.Length < 3)
                return false;
            var point1 = ctrlPoints[0].WorldPos;
            Vec3 point2;
            //找到板控制点中不都在normal方向的两个点
            if (normal.IsParallelTo(point1 - ctrlPoints[1].WorldPos))
                point2 = ctrlPoints[2].WorldPos;
            else
                point2 = ctrlPoints[1].WorldPos;
            //对齐面1
            var dir1 = PlateTools.RotateVectorInPlane(-webPlane.Normal.Normalize(), normal, bevelAngle);
            Plane alignPlane1 = new Plane(point1, dir1);
            m_context.TargetComp.CreateAlignPlane(retPlate, alignPlane1, true);
            //对齐面2
            var dir2 = PlateTools.RotateVectorInPlane(-webPlane.Normal.Normalize(), -normal, bevelAngle);
            Plane alignPlane2 = new Plane(point2, dir2);
            m_context.TargetComp.CreateAlignPlane(retPlate, alignPlane2, true);
            return true;
        }
        /// <summary>
        /// 板切割 防止与零件碰撞
        /// </summary>
        private void CreateCutter(PartBase retPlate, Plane webPlane)
        {
            int plateCut = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_PLATE_CUT);
            if (plateCut != 2)
                return;
            double eps = 1.0;
            double offSet = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_PLATE_CUT_LENGTH);
            var dir = m_context.TargetComp.LocalMatrix.Z;
            var mainPlanes = m_context.MainPlanes;
            Plane fsBound, fsInside, nsBound, nsInside;
            if(mainPlanes.BottomBound.Normal.Dot(m_fsFlange.Normal) > 0)
            {
                fsBound = mainPlanes.BottomBound;
                fsInside = mainPlanes.BottomFlangeInside;
                nsBound = mainPlanes.TopBound;
                nsInside = mainPlanes.TopFlangeInside;
            }
            else
            {
                fsBound = mainPlanes.TopBound;
                fsInside = mainPlanes.TopFlangeInside;
                nsBound = mainPlanes.BottomBound;
                nsInside = mainPlanes.BottomFlangeInside;
            }
            var fsSide = new List<Plane>();
            fsSide.Add(webPlane.Move(-eps));
            fsSide.Add(fsBound.Move(eps));
            var fsPoint1 = webPlane.Intersect(fsInside.Move(offSet), m_subRefPlane);
            var fspoint2 = fsPoint1 - fsInside.Normal.Normalize() * offSet
                + webPlane.Normal.Normalize() * offSet;
            fsSide.Add(new Plane(fsPoint1, fspoint2 - fsPoint1, dir));

            var nsSide = new List<Plane>();
            nsSide.Add(webPlane.Move(-eps));
            nsSide.Add(nsBound.Move(eps));
            var nsPoint1 = webPlane.Intersect(nsInside.Move(offSet), m_subRefPlane);
            var nspoint2 = nsPoint1 - nsInside.Normal.Normalize() * offSet
                + webPlane.Normal.Normalize() * offSet;
            nsSide.Add(new Plane(nsPoint1, nspoint2 - nsPoint1, dir));
            var zTop = m_subRefPlane.Move(m_height);
            var zBottom = m_subRefPlane.Invert();
            m_context.TargetComp.CreatePlateCutter(retPlate, zBottom, zTop, fsSide.ToArray(), true);
            m_context.TargetComp.CreatePlateCutter(retPlate, zBottom, zTop, nsSide.ToArray(), true);
        }
        private List<double> HandlerStride(double[] arr, int num)
        {
            var list = new List<double>();
            if (num <= 0)
            {
                list = null;
            }
            else if(num == 1)
            {
                list.Add(0.0);
            }
            else
            {
                int len = arr.Length - 1;
                for (int i = 0; i < num - 1; i++)
                {
                    if (i > len)
                        list.Add(arr[len]);
                    else
                        list.Add(arr[i]);
                }
            }
            return list;
        }
        private Vec3 ParseWorldDir(Vec3 normal)
        {
            var dir = normal.Normalize();
            var m_comp = m_context.TargetComp;
            var axis = new Vec3[] {
                m_comp.WorkAxis.X, m_comp.WorkAxis.Y, m_comp.WorkAxis.Z
            };
            double max = 0.0;
            int index = 0;
            for (int i = 0; i < axis.Length; i++)
            {
                var temp = Math.Abs(dir.Dot(axis[i]));
                if (temp > max)
                {
                    max = temp;
                    index = i;
                }
            }
            return axis[index];
        }
        /// <summary>
        /// 计算给定数组的元素绝对值之和
        /// </summary>
        private double SumArray(List<double> arr)
        {
            if (arr == null || arr.Count == 0)
                return 0.0;
            double sum = 0.0;
            foreach (var item in arr)
                sum += Math.Abs(item);
            return sum;
        }
    }
}
