﻿using System;
using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;

namespace BeamColumnWeld107
{
    public class GraphicHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private double m_gap = 0.0;
        private Plane m_mainNearPlane = null;
        private BeamPlanes m_mainPlanes = null;
        private BeamPlanes m_subPlanes = null;
        private Plane m_subTopPlane = null;
        private Plane m_subBottomPlane = null;
        public GraphicHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            this.ParseProps();
            if (!SubEndCut())
                return false;
            SetBeamChamfer();
            return true;
        }
        private bool ParseProps()
        {
            m_gap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_PARTS_GAP);
            m_mainNearPlane = m_context.FaceSubNearPlane;
            m_mainPlanes = m_context.MainPlanes;
            m_subPlanes = m_context.SubPlanes;
            if(m_mainPlanes.TopBound.Normal.Dot(m_subPlanes.BackBound.Normal) > 0)
            {
                m_subTopPlane = m_subPlanes.BackBound;
                m_subBottomPlane = m_subPlanes.FrontBound;
            }
            else
            {
                m_subTopPlane = m_subPlanes.FrontBound;
                m_subBottomPlane = m_subPlanes.BackBound;
            }
            return true;
        }
        /// <summary>
        /// 次零件端部切割
        /// </summary>
        private bool SubEndCut()
        {
            if (m_mainNearPlane == null)
                return false;
            var alignPlane = m_mainNearPlane.Move(m_gap);
            m_context.TargetComp.CreateAlignPlane(m_context.SubPart, alignPlane, true);
            m_context.SubAlignPlane = alignPlane;
            return true;
        }
        /// <summary>
        /// 设置次零件翼缘折角
        /// </summary>
        private void SetBeamChamfer()
        {
            //主零件的腹板要面向次零件
            if (Math.Abs(m_mainNearPlane.Normal.Dot(m_mainPlanes.BackWeb.Normal)) < 1e-3)
                return;
            //次零件的腹板要面向我们
            var dir = m_context.TargetComp.LocalMatrix.Z;
            if (dir.Dot(m_mainPlanes.TopFlange.Normal.Normalize()) > 1e-3)
                return;
            int cutType = m_paramsHandler.GetIntValue(ParamNames.PIC_BEAM_CUT_TYPE);
            var cutX = m_paramsHandler.GetDoubleValue(ParamNames.PIC_BEAM_CUT_X);
            var cutY = m_paramsHandler.GetDoubleValue(ParamNames.PIC_BEAM_CUT_Y);
            if (cutType < 2 || cutX <= 0 || cutY <= 0)
                return;
            if (cutType > 2)
                cutY = cutX;
            //top切角
            var mainTopPlane = m_mainPlanes.TopFlangeInside;
            var pls1 = GetPlsPlane(mainTopPlane, m_subTopPlane);
            HandlerChamfer(pls1, cutType, cutX, cutY);
            //bottom切角
            var mainBottomPlane = m_mainPlanes.BottomFlangeInside;
            var pls2 = GetPlsPlane(mainBottomPlane, m_subBottomPlane);
            HandlerChamfer(pls2, cutType, cutX, cutY);
        }
        private void HandlerChamfer(Plane pls, int cutType, double cutX, double cutY)
        {
            if (pls == null)
                return;
            var plsSide = new List<Plane>();
            plsSide.Add(pls.Move(cutY + 1.0));
            plsSide.Add(m_mainNearPlane.Move(m_gap + cutX));
            plsSide.Add(pls);
            plsSide.Add(m_mainNearPlane.Move(-1.0));
            Plane zTop = m_subPlanes.TopBound.Move(5.0);
            Plane zBottom = m_subPlanes.BottomBound.Move(5.0);
            var cutter = m_context.TargetComp.CreatePlateCutter(m_context.SubPart,
                zBottom, zTop, plsSide.ToArray(), true);
            if (cutter == null)
                return;
            int num = GetChamferPoint(cutter, pls);
            int len = cutter.CtrlPoints.Length;
            var xPoint = cutter.CtrlPoints[num - 1 < 0 ? len - 1 : num - 1].WorldPos;
            var point = cutter.CtrlPoints[num].WorldPos;
            var lenX = cutX;
            var lenY = cutY;
            //若折角点的X切割方向与pls的法线方向一致 调整cutX 与 cutY的切割方向
            if (pls.Normal.IsParallelTo(xPoint - point))
            {
                lenX = cutY;
                lenY = cutX;
            }
            SetChamfer(cutter.CtrlPoints[num] as ChamferCtrlPoint,
                cutType, lenX, lenY, cutter);
        }
        private int GetChamferPoint(PartBase cutter, Plane pls)
        {
            var ctrlPoints = cutter.CtrlPoints;
            int len = ctrlPoints.Length;
            double dist1 = double.MinValue;
            double dist2 = double.MinValue;
            int num = 0;
            for(int i = 0; i < len; i++)
            {
                var point = ctrlPoints[i].WorldPos;
                var temp1 = pls.Dist(point);
                var temp2 = m_mainNearPlane.Dist(point);
                //到主零件的距离最大，到次零件的距离最大
                if(temp1 >= dist1 && temp2 >= dist2)
                {
                    dist1 = temp1;
                    dist2 = temp2;
                    num = i;
                }
            }
            return num;
        }
        /// <summary>
        /// 得到切割体距离主零件内翼缘的间距
        /// </summary>
        private double GetPartsGap()
        {
            //槽口尺寸测量方式
            int sizeMode = m_paramsHandler.GetIntValue(ParamNames.NOTCH_SIZE_MODE);
            double notchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_SIZE_Y); //纵向尺寸
            if (sizeMode == 2)
            {
                //主零件翼缘厚度
                var mainFlangeT = m_mainPlanes.TopFlange.Dist(m_mainPlanes.TopFlangeInside.Origin);
                notchSize -= mainFlangeT;
            }
            //槽口舍入尺寸
            double sizeBase = notchSize;
            int sizeRounding = m_paramsHandler.GetIntValue(ParamNames.NOTCH_SIZE_ROUNDING);
            if (sizeRounding == 2)
            {
                sizeBase = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_ROUNDING_BASE_Y);
            }
            if (sizeBase > notchSize)
                notchSize = sizeBase;
            else if(sizeBase < notchSize)
            {
                var size = sizeBase;
                while (true)
                {
                    sizeBase += size;
                    if(sizeBase > notchSize)
                    {
                        notchSize = sizeBase;
                        break;
                    }
                }
            }
            return notchSize;
        }
        /// <summary>
        /// 得到切割体的侧面
        /// </summary>
        /// <param name="mainPlane">主零件内翼缘面</param>
        /// <param name="subPlane">次零件包围面</param>
        /// <returns></returns>
        private Plane GetPlsPlane(Plane mainPlane, Plane subPlane)
        {
            var gap = GetPartsGap();
            Plane pls;
            if (mainPlane.Normal.IsParallelTo(subPlane.Normal))
            {
                var dist = mainPlane.SignedDist(subPlane.Origin);
                if (dist > 0 && dist > gap)
                {
                    pls = subPlane.Move(1.0).Invert();
                }
                else
                {
                    pls = mainPlane.Move(gap - 1.0);
                }
            }
            else
            {
                pls = mainPlane.Move(gap - 1.0);
            }
            return pls;
        }
        private void SetChamfer(ChamferCtrlPoint point, int type, double lenX, double lenY, PartBase retPlate)
        {
            if (retPlate == null)
                return;
            Define.CornerCutType chamferType;
            switch (type)
            {
                case 4:
                    chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    break;
                case 3:
                    chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    break;
                default:
                    chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    break;
            }
            if (point != null)
            {
                point.CutX = lenX;
                point.CutY = lenY;
                point.CutType = chamferType;
                point.Flush();
                retPlate.Flush();
            }
        }
    }
}
