﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;
using System;
using System.Collections.Generic;

namespace BeamColumnWeld107
{
    public class StiffHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private PlateProperties m_topNs = new PlateProperties();
        private PlateProperties m_topFs = new PlateProperties();
        private PlateProperties m_bottomNs = new PlateProperties();
        private PlateProperties m_bottomFs = new PlateProperties();

        private Vec3 m_subLine = Vec3.ZERO;
        private Plane m_mainNearWeb = null;
        private Plane m_mainFarWeb = null;
        private Plane m_mainNearFlange = null;
        private Plane m_mainFarFlange = null;
        private Plane m_subTopPlane = null;
        private Plane m_subBottomPlane = null;
        private double m_flangeGap = double.MaxValue;
        private double m_boundGap = double.MaxValue;
        private List<PartBase> stiffs = new List<PartBase>();
        public StiffHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!CheckTarget())
                return false;
            CreateStiff();
            SetChamfer();
            m_context.Stiffs = stiffs;
            return true;
        }
        private bool CheckTarget()
        {
            var stiffEnabled = m_paramsHandler.GetIntValue(ParamNames.STIFF_ENABLED);
            if (stiffEnabled == 1)
                return false;
            var subPlanes = m_context.SubPlanes;
            //次零件翼缘不朝向主零件的构建方向 不创建加劲肋
            var flag = Math.Abs(subPlanes.TopFlange
                .Normal.Normalize().Dot(m_context.MainLine));
            if (flag < 1e-3)
                return false;
            DObject topNs = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_TOP_NS_PROPS);
            if (topNs == null)
                return false;
            m_topNs.SetProps(topNs);
            DObject topFs = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_TOP_FS_PROPS);
            if (topFs == null)
                return false;
            m_topFs.SetProps(topFs);
            DObject bottomNs = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_BOTTOM_NS_PROPS);
            if (bottomNs == null)
                return false;
            m_bottomNs.SetProps(bottomNs);
            DObject bottomFs = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_BOTTOM_FS_PROPS);
            if (bottomFs == null)
                return false;
            m_bottomFs.SetProps(bottomFs);
            m_boundGap = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_FLANGE_GAP_Y);
            m_flangeGap = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_FLANGE_GAP_X);
            m_subLine = m_context.SubLine;

            Vec3 dir = m_context.TargetComp.LocalMatrix.Z;
            //解析加劲肋位置  位于空间中次零件上下包围面处
            Plane topPlane, bottomPlane;
            if (dir.Dot(subPlanes.TopBound.Normal) > 0)
            {
                topPlane = subPlanes.TopBound;
                bottomPlane = subPlanes.BottomBound;
            }
            else
            {
                topPlane = subPlanes.BottomBound;
                bottomPlane = subPlanes.TopBound;
            }
            var stiffDir = m_paramsHandler.GetIntValue(ParamNames.STIFF_DIR);
            if (stiffDir == 2)
            {
                var point1 = topPlane.Intersect(m_context.SubAlignPlane, subPlanes.BackBound);
                var point2 = bottomPlane.Intersect(m_context.SubAlignPlane, subPlanes.BackBound);
                m_subTopPlane = new Plane(point1, dir);
                m_subBottomPlane = new Plane(point2, -dir);
            }
            else
            {
                m_subTopPlane = topPlane;
                m_subBottomPlane = bottomPlane;
            }
            InitData();
            return true;
        }
        private bool InitData()
        {
            //解析主零件的远近腹板 翼板面
            var mainPlanes = m_context.MainPlanes;
            Vec3 worldDir;
            if(Math.Abs(m_context.FaceSubNearPlane.Normal
                .Dot(mainPlanes.BackWeb.Normal)) > 0)
            {
                //当主零件腹板面向次零件时 -- 腹板近与远相对于次零件的位置
                //翼缘近与远相对于主零件面向我们的近侧与远侧
                m_mainNearWeb = m_context.FaceSubNearPlane;
                m_mainFarWeb = m_context.FaceSubFarPlane;
                //以世界坐标系为参照
                var normal = mainPlanes.TopFlange.Normal;
                worldDir = ParseWorldDir(normal);
                if(normal.Dot(worldDir) > 0)
                {
                    m_mainNearFlange = mainPlanes.BottomFlangeInside;
                    m_mainFarFlange = mainPlanes.TopFlangeInside;
                }
                else
                {
                    m_mainNearFlange = mainPlanes.TopFlangeInside;
                    m_mainFarFlange = mainPlanes.BottomFlangeInside;
                }
            }
            else
            {
                //当主零件翼缘面向次零件时 -- 翼缘近与远相对于次零件的位置
                //腹板近与远相对于主零件面向我们的近侧与远侧
                if (mainPlanes.TopFlange.Normal
                    .Dot(m_context.FaceSubNearPlane.Normal) > 0)
                {
                    m_mainNearFlange = mainPlanes.TopFlangeInside;
                    m_mainFarFlange = mainPlanes.BottomFlangeInside;
                }
                else
                {
                    m_mainNearFlange = mainPlanes.BottomFlangeInside;
                    m_mainFarFlange = mainPlanes.TopFlangeInside;
                }
                var normal = mainPlanes.BackWeb.Normal;
                worldDir = ParseWorldDir(normal);
                if(normal.Dot(worldDir) > 0)
                {
                    m_mainNearWeb = mainPlanes.FrontWeb;
                    m_mainFarWeb = mainPlanes.BackWeb;
                }
                else
                {
                    m_mainNearWeb = mainPlanes.BackWeb;
                    m_mainFarWeb = mainPlanes.FrontWeb;
                }
            }
            //topNs -- bottomNs  bottomNs未输入的情况
            if (m_bottomNs.Thickness == double.MaxValue)
                m_bottomNs.Thickness = m_topNs.Thickness;
            if (m_topNs.Height != double.MaxValue && m_bottomNs.Height == double.MaxValue)
                m_bottomNs.Height = m_topNs.Height;
            if (m_topNs.Width != double.MaxValue && m_bottomNs.Height == double.MaxValue)
                m_bottomNs.Width = m_topNs.Width;

            //topNs -- topFs  topFs未输入的情况
            if (m_topFs.Thickness == double.MaxValue)
                m_topFs.Thickness = m_topNs.Thickness;
            if (m_topNs.Height != double.MaxValue && m_topFs.Height == double.MaxValue)
                m_topFs.Height = m_topNs.Height;
            if (m_topNs.Width != double.MaxValue && m_topFs.Height == double.MaxValue)
                m_topFs.Width = m_topNs.Width;

            //topFs -- bottomFs  BottomFs未输入的情况
            if (m_bottomFs.Thickness == double.MaxValue)
                m_bottomFs.Thickness = m_topFs.Thickness;
            if (m_topFs.Height != double.MaxValue && m_bottomFs.Height == double.MaxValue)
                m_bottomFs.Height = m_topFs.Height;
            if (m_topFs.Width != double.MaxValue && m_bottomFs.Height == double.MaxValue)
                m_bottomFs.Width = m_topFs.Width;
            stiffs.Add(null);
            stiffs.Add(null);
            stiffs.Add(null);
            stiffs.Add(null);
            return true;
        }
        private bool CreateStiff()
        {
            var m_webGap = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_WEB_GAP);
            double width = m_context.MainPlanes.BackBound.Dist(m_context.MainPlanes
                .FrontBound.Origin) / 2 - m_mainFarWeb.Dist(m_mainNearWeb.Origin) / 2;
            double height = m_mainNearFlange.Dist(m_mainFarFlange.Origin);
            if (m_webGap >= width || m_boundGap >= width || m_flangeGap >= height)
                return false;
            m_mainNearWeb.MoveThis(m_webGap);
            m_mainFarWeb.MoveThis(m_webGap);
            var m_comp = m_context.TargetComp;
            do
            {
                //topNs
                if (m_topNs.Thickness <= 0.0 || m_topNs.Height <= 0 || m_topNs.Width <= 0)
                    break;
                var planes = GetBuildPlanes(m_topNs, m_mainNearWeb, width, height);
                Plane nearBound = planes[0];
                Plane nearFlange = planes[1];
                Plane farFlange = planes[2];
                double offset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_TOP_NS_OFFSET);
                var refPlane = AdaptRefPos(m_mainNearWeb, farFlange, m_subTopPlane, m_topNs.Thickness);
                if (refPlane == null)
                    break;
                refPlane.MoveThis(offset);
                var points = new Vec3[4];
                points[0] = m_mainNearWeb.Intersect(farFlange, refPlane);
                points[1] = m_mainNearWeb.Intersect(nearFlange, refPlane);
                points[2] = nearBound.Intersect(nearFlange, refPlane);
                points[3] = nearBound.Intersect(farFlange, refPlane);
                AdjustRefPlane(points, new Plane[] { farFlange, nearFlange,
                    nearFlange, farFlange}, refPlane, m_topNs.Thickness);
                var topNs = m_comp.CreateSteelPlate(points, m_topNs.Thickness, Define.ObjectSide.SIDE_LEFT, true);
                if (topNs == null)
                    break;
                topNs.Class = m_comp.CompClass;
                m_topNs.ApplyToPart(topNs);
                stiffs[0] = topNs;
            } while (false);
            do
            {
                //topFs
                if (m_topFs.Thickness <= 0.0 || m_topFs.Height <= 0 || m_topFs.Width <= 0)
                    break;
                var planes = GetBuildPlanes(m_topFs, m_mainFarWeb, width, height);
                Plane farBound = planes[0];
                Plane nearFlange = planes[1];
                Plane farFlange = planes[2];
                double offset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_TOP_FS_OFFSET);
                var refPlane = AdaptRefPos(m_mainFarWeb, farFlange, m_subTopPlane, m_topFs.Thickness);
                if (refPlane == null)
                    break;
                refPlane.MoveThis(offset);
                var points = new Vec3[4];
                points[0] = m_mainFarWeb.Intersect(farFlange, refPlane);
                points[1] = m_mainFarWeb.Intersect(nearFlange, refPlane);
                points[2] = farBound.Intersect(nearFlange, refPlane);
                points[3] = farBound.Intersect(farFlange, refPlane);
                AdjustRefPlane(points, new Plane[] { farFlange, nearFlange,
                    nearFlange, farFlange}, refPlane.Invert(), m_topFs.Thickness);
                var topFs = m_comp.CreateSteelPlate(points, m_topFs.Thickness, Define.ObjectSide.SIDE_LEFT, true);
                if (topFs == null)
                    break;
                topFs.Class = m_comp.CompClass;
                m_topFs.ApplyToPart(topFs);
                stiffs[1] = topFs;
            } while (false);
            do
            {
                //bottomNs
                if (m_bottomNs.Thickness <= 0.0 || m_bottomNs.Height <= 0 || m_bottomNs.Width <= 0)
                    break;
                var planes = GetBuildPlanes(m_bottomNs, m_mainNearWeb, width, height);
                Plane nearBound = planes[0];
                Plane nearFlange = planes[1];
                Plane farFlange = planes[2];
                double offset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_BOTTOM_NS_OFFSET);
                var refPlane = AdaptRefPos(m_mainNearWeb, farFlange, m_subBottomPlane, m_bottomNs.Thickness);
                if (refPlane == null)
                    break;
                refPlane.MoveThis(-offset);
                var points = new Vec3[4];
                points[0] = m_mainNearWeb.Intersect(farFlange, refPlane);
                points[1] = m_mainNearWeb.Intersect(nearFlange, refPlane);
                points[2] = nearBound.Intersect(nearFlange, refPlane);
                points[3] = nearBound.Intersect(farFlange, refPlane);
                AdjustRefPlane(points, new Plane[] { farFlange, nearFlange,
                    nearFlange, farFlange}, refPlane.Invert(), m_bottomNs.Thickness);
                var bottomNs = m_comp.CreateSteelPlate(points, m_bottomNs.Thickness, Define.ObjectSide.SIDE_LEFT, true);
                if (bottomNs == null)
                    break;
                bottomNs.Class = m_comp.CompClass;
                m_bottomNs.ApplyToPart(bottomNs);
                stiffs[2] = bottomNs;
            } while (false);
            do
            {
                //bottomFs
                if (m_bottomFs.Thickness <= 0.0 || m_bottomFs.Height <= 0 || m_bottomFs.Width <= 0)
                    break;
                var planes = GetBuildPlanes(m_bottomFs, m_mainFarWeb, width, height);
                Plane farBound = planes[0];
                Plane nearFlange = planes[1];
                Plane farFlange = planes[2];
                double offset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_BOTTOM_FS_OFFSET);
                var refPlane = AdaptRefPos(m_mainFarWeb, farFlange, m_subBottomPlane, m_bottomFs.Thickness);
                if (refPlane == null)
                    break;
                refPlane.MoveThis(-offset);
                var points = new Vec3[4];
                points[0] = m_mainFarWeb.Intersect(farFlange, refPlane);
                points[1] = m_mainFarWeb.Intersect(nearFlange, refPlane);
                points[2] = farBound.Intersect(nearFlange, refPlane);
                points[3] = farBound.Intersect(farFlange, refPlane);
                AdjustRefPlane(points, new Plane[] { farFlange, nearFlange,
                    nearFlange, farFlange}, refPlane, m_bottomFs.Thickness);
                var bottomFs = m_comp.CreateSteelPlate(points, m_bottomFs.Thickness, Define.ObjectSide.SIDE_LEFT, true);
                if (bottomFs == null)
                    break;
                bottomFs.Class = m_comp.CompClass;
                m_bottomFs.ApplyToPart(bottomFs);
                stiffs[3] = bottomFs;
            } while (false);
            return true;
        }
        private bool SetChamfer()
        {
            var enabled = m_paramsHandler.GetIntValue(ParamNames.STIFF_CHAMFER_ENABLED);
            if (enabled == 1)
                return false;

            //内侧折角
            var insideType = m_paramsHandler.GetIntValue(ParamNames.STIFF_INSIDE_CHAMFER_TYPE);
            double insideX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_INSIDE_CHAMFER_X);
            double insideY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_INSIDE_CHAMFER_Y);
            if (insideType > 2)
                insideY = insideX;

            //外侧折角
            var chamferType = m_paramsHandler.GetIntValue(ParamNames.STIFF_CHAMFER_TYPE);
            double chamferX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_CHAMFER_X);
            double chamferY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_CHAMFER_Y);
            if (chamferType > 2)
                chamferY = chamferX;
            foreach(var stiff in stiffs)
            {
                if (stiff == null)
                    continue;
                var point1 = stiff.CtrlPoints[0] as ChamferCtrlPoint;
                var point2 = stiff.CtrlPoints[1] as ChamferCtrlPoint;
                var point3 = stiff.CtrlPoints[2] as ChamferCtrlPoint;
                var point4 = stiff.CtrlPoints[3] as ChamferCtrlPoint;
                PlateTools.SetChamfer(point1, insideType, insideX, insideY, stiff);
                PlateTools.SetChamfer(point2, insideType, insideY, insideX, stiff);
                PlateTools.SetChamfer(point3, chamferType, chamferX, chamferY, stiff);
                PlateTools.SetChamfer(point4, chamferType, chamferY, chamferX, stiff);
            }
            return true;
        }
        /// <summary>
        /// 调整加劲肋控制点的位置，避免与主零件发生碰撞
        /// </summary>
        private void AdjustRefPlane(Vec3[] points, Plane[] planes, Plane refPlane, double thickness)
        {
            var dir = refPlane.Normal.Normalize();
            for(int i = 0; i < points.Length; i++)
            {
                var point = points[i] - dir * thickness;
                if (planes[i].SignedDist(point) >= 0)
                    continue;
                var normal = planes[i].Normal.Normalize();
                var line = (point - points[i]).Normalize();
                double sinus = Math.Abs(normal.Dot(line));
                int symbol = m_subLine.Dot(planes[i].Normal) > 0 ? 1 : -1;
                points[i] = points[i] + symbol * m_subLine * Math.Tan(Math.Asin(sinus)) * thickness;
            }
        }
        /// <summary>
        /// 解析加劲肋的构件平面
        /// </summary>
        private Plane[] GetBuildPlanes(PlateProperties props, Plane webPlane, double width, double height)
        {
            Plane[] planes = new Plane[3];
            if (props.Width != double.MaxValue)
                planes[0] = webPlane.Move(props.Width - m_boundGap);
            else
                planes[0] = webPlane.Move(-m_boundGap + width);
            if (props.Height != double.MaxValue)
            {
                planes[2] = m_mainFarFlange.Move(height / 2 - props.Height / 2 + m_flangeGap / 2);
                planes[1] = m_mainNearFlange.Move(height / 2 - props.Height / 2 + m_flangeGap / 2);
            }
            else
            {
                planes[2] = m_mainFarFlange.Move(m_flangeGap);
                planes[1] = m_mainNearFlange.Move(m_flangeGap);
            }
            return planes;
        }
        private Plane AdaptRefPos(Plane webPlane, Plane flangePlane, Plane refPlane, double offset)
        {
            if (webPlane == null || flangePlane == null || refPlane == null)
                return null;
            var axisX = flangePlane.Normal.Normalize();
            var axisY = webPlane.Normal.Normalize();
            var axisZ = axisX.Cross(axisY);
            var dir = refPlane.Normal.Normalize();
            if (dir.Dot(axisZ) > 0)
                return new Plane(refPlane.Move(-offset));
            return new Plane(refPlane);
        }
        private Vec3 ParseWorldDir(Vec3 normal)
        {
            var m_comp = m_context.TargetComp;
            var axis = new Vec3[] {
                m_comp.WorkAxis.X, m_comp.WorkAxis.Y, m_comp.WorkAxis.Z
            };
            double max = 0.0;
            int index = 0;
            for (int i = 0; i < axis.Length; i++)
            {
                var temp = Math.Abs(normal.Dot(axis[i]));
                if (temp > max)
                {
                    max = temp;
                    index = i;
                }
            }
            return axis[index];
        }
    }
}
