﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System;
using System.Collections.Generic;

namespace BeamSideEndPlate209
{
    public class ComponentInstanceContext
    {
        //目标组件类型--节点
        private ConnectionComponent m_comp = null;

        //主零件参数
        private Beam m_mainPart = null;
        private Vec3 m_mainLine = Vec3.ZERO;
        private BeamPlanes m_mainPlanes = null;
        private Plane m_mainNearPlane = null;
        private Plane m_mainTopPlane = null;
        private Plane m_mainBottomPlane = null;

        //次零件参数
        private Beam m_subPart = null;
        private Vec3 m_subLine = Vec3.ZERO;
        private BeamPlanes m_subPlanes = null;
        private Plane m_subTopPlane = null;
        private Plane m_subBottomPlane = null;
        private Plane m_subUpPlane = null;
        private Plane m_subDownPlane = null;

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }

        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (comp.Subs == null)
                return false;
            Beam subPart = comp.Subs.First() as Beam;
            if (mainPart == null || subPart == null)
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);
            m_subPart = subPart;
            m_subPlanes = BeamPlanes.CreateFromBeam(subPart);

            //判定主零件是否符合组件构建要求
            if (m_mainPlanes.BackWeb == null || m_mainPlanes.FrontWeb == null ||
                m_mainPlanes.BottomFlange == null || m_mainPlanes.BottomFlangeInside == null
                || m_mainPlanes.TopFlange == null || m_mainPlanes.TopFlangeInside == null)
                return false;
            //此组件次零件不限制 -- 至少可以创建出组件
            if (!AdjustSubLine(subPart, mainPart))
                return false;

            if (!ParseMainNearPlane())
                return false;

            this.PrepareBuildData();
            return true;
        }

        /// <summary>
        /// 调整次零件的构建方向
        /// </summary>
        /// <param name="subPart">次零件</param>
        /// <param name="mainPart">主零件</param>
        /// <returns></returns>
        private bool AdjustSubLine(Beam subPart, Beam mainPart)
        {
            //先解析主零件方向，若不存在不创建组件
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;
            //如果主零件存在，定义方向为从第一控制点指向第二控制点
            var mainLine = line.Value.Normalize();

            var point1 = subPart.CtrlPoints?.Last()?.WorldPos;
            var point2 = subPart.CtrlPoints?.First()?.WorldPos;
            var dir = point1 - point2;
            if (!dir.HasValue)
                return false;
            Vec3 origin = m_mainPart.CtrlPoints.First().WorldPos;
            var dir1 = point1.Value - origin;
            var dir2 = point2.Value - origin;
            //由叉乘的几何意义 -- 向量所围成的平行四边形面积作比较
            double dist1 = dir1.Cross(mainLine).Length;
            double dist2 = dir2.Cross(mainLine).Length;
            //取一边为单位向量，那么面积就可以代表点到直线的距离
            if (dist1 > dist2)
                m_subLine = dir.Value.Normalize();
            else
                m_subLine = -dir.Value.Normalize();
            return true;
        }

        /// <summary>
        /// 由主次零件的相对位置解析主零件near包围面
        /// </summary>
        private bool ParseMainNearPlane()
        {
            //主零件前后腹板面
            var backWeb = m_mainPlanes.BackWeb;
            var frontWeb = m_mainPlanes.FrontWeb;

            //若主零件腹板完全不贴近次零件 不创建组件
            if (Math.Abs(m_subLine.Dot(backWeb.Normal)) < 1e-3 &&
                Math.Abs(m_subLine.Dot(frontWeb.Normal)) < 1e-3)
                return false;

            //筛选符合的包围面
            var planes = new Plane[] {
                m_mainPlanes.BottomBound, m_mainPlanes.TopBound,
                m_mainPlanes.FrontBound, m_mainPlanes.BackBound
            };
            int mark = 0;
            double maxDot = -2;
            for(int i = 0; i < planes.Length; i++)
            {
                double dot = planes[i].Normal.Normalize().Dot(m_subLine);
                if(dot > maxDot)
                {
                    maxDot = dot;
                    mark = i;
                }
            }
            m_mainNearPlane = planes[mark];

            //筛选出的包围面不与腹板平行 组件创建错误
            if (!m_mainNearPlane.Normal.IsParallelTo(backWeb.Normal))
                return false;

            return true;
        }

        /// <summary>
        /// 准备后续构件板的初始平面数据
        /// </summary>
        private void PrepareBuildData()
        {
            //定义前方向
            var dir = Vec3.ZERO;
            var worldDir = new Vec3[] {
                m_comp.LocalMatrix.X, m_comp.LocalMatrix.Y, m_comp.LocalMatrix.Z
            };
            //若组件帽尖方向垂直于主零件翼缘与腹板的法线
            //取前方向为翼缘法线对应的世界方向
            if (Math.Abs(worldDir[2].Dot(m_mainPlanes.BackWeb.Normal.Normalize())) < 1e-3
                && Math.Abs(worldDir[2].Dot(m_mainPlanes.TopFlange.Normal.Normalize())) < 1e-3)
            {
                if (worldDir[0].IsParallelTo(m_mainPlanes.TopFlange.Normal))
                {
                    dir = worldDir[0];
                }else{
                    dir = worldDir[1];
                }
            }//否则为组件帽尖方向
            else{
                dir = worldDir[2];
            }

            //调整主零件方向为世界方向
            var mainLine = (m_mainPart.CtrlPoints.Last().WorldPos
                - m_mainPart.CtrlPoints.First().WorldPos).Normalize();
            foreach(var item in worldDir)
            {
                if(Math.Abs(mainLine.Dot(item)) > 1e-1)
                {
                    mainLine = new Vec3(item);
                    break;
                }
            }
            m_mainLine = mainLine;

            //筛选主零件前后方向面 原则：通过前方向dir判断
            m_mainTopPlane = new Plane(m_mainPlanes.TopBound);
            m_mainBottomPlane = new Plane(m_mainPlanes.BottomBound);
            if (dir.Dot(m_mainPlanes.TopBound.Normal) < 0)
            {
                //topPlane的法线方向要与dir相同
                this.SwapVar(ref m_mainTopPlane, ref m_mainBottomPlane);
            }

            //筛选次零件前后方向面
            var dot1 = m_subPlanes.BottomBound.Normal.Normalize().Dot(dir);
            var dot2 = m_subPlanes.BackBound.Normal.Normalize().Dot(dir);
            if(Math.Abs(dot1) > Math.Abs(dot2))
            {
                //前后方向
                m_subTopPlane = new Plane(m_subPlanes.BottomBound);
                m_subBottomPlane = new Plane(m_subPlanes.TopBound);
                if (dot1 < 0)
                {
                    //topPlane的法线方向要与dir相同
                    this.SwapVar(ref m_subTopPlane, ref m_subBottomPlane);
                }

                //上下方向
                m_subUpPlane = m_subPlanes.BackBound;
                m_subDownPlane = m_subPlanes.FrontBound;
                if (m_subPlanes.BackBound.Normal.Dot(mainLine) > 0)
                {
                    //upPlane的法线方向要与调整后的主零件方向相同
                    this.SwapVar(ref m_subUpPlane, ref m_subDownPlane);
                }
            }
            else {
                //前后方向
                m_subTopPlane = m_subPlanes.BackBound;
                m_subBottomPlane = m_subPlanes.FrontBound;
                if (dot2 > 0)
                {
                    //topPlane的法线方向要与dir相同
                    this.SwapVar(ref m_subTopPlane, ref m_subBottomPlane);
                }

                //上下方向
                m_subUpPlane = m_subPlanes.TopBound;
                m_subDownPlane = m_subPlanes.BottomBound;
                if (m_subPlanes.TopBound.Normal.Dot(mainLine) > 0)
                {
                    //upPlane的法线方向要与调整后的主零件方向相同
                    this.SwapVar(ref m_subUpPlane, ref m_subDownPlane);
                }
            }
        }

        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
            m_subPart = null;
            m_subPlanes = null;
        }

        /// <summary>
        /// 交换两个面的属性信息
        /// </summary>
        /// <param name="plane1">面1</param>
        /// <param name="plane2">面2</param>
        private void SwapVar(ref Plane plane1, ref Plane plane2)
        {
            var temp = plane1;
            plane1 = plane2;
            plane2 = temp;
        }

        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 SubLine { get => m_subLine; set => m_subLine = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane MainNearPlane { get => m_mainNearPlane; set => m_mainNearPlane = value; }
        public Plane MainTopPlane { get => m_mainTopPlane; set => m_mainTopPlane = value; }
        public Plane MainBottomPlane { get => m_mainBottomPlane; set => m_mainBottomPlane = value; }
        public Plane SubTopPlane { get => m_subTopPlane; set => m_subTopPlane = value; }
        public Plane SubBottomPlane { get => m_subBottomPlane; set => m_subBottomPlane = value; }
        public Plane SubUpPlane { get => m_subUpPlane; set => m_subUpPlane = value; }
        public Plane SubDownPlane { get => m_subDownPlane; set => m_subDownPlane = value; }
        public Plane SubAlignPlane { get; set; }
        public SteelBeam EndPlate { get; set; }
        public SteelBeam FrontPlate { get; set; }
        public PartBase ConnPlate { get; set; }
        public PartBase StiffPlate { get; set; }
    }
}
