﻿using System.Collections.Generic;
using System.Linq;

namespace BeamSideEndPlate209
{
    /// <summary>
    /// 螺栓参数类
    /// </summary>
    public class BoltData
    {
        public int VertOffsetMode { get; set; }
        public double VertOffset { get; set; }
        public int HoriOffsetMode { get; set; }
        public double HoriOffset { get; set; }
        public double UpGap { get; set; }
        public double DownGap { get; set; }
        public double LeftGap { get; set; }
        public double RightGap { get; set; }
        public double[] VertStride { get; set; }
        public double[] HoriStride { get; set; }
        public int[] IgnoreBolt { get; set; }
        public BoltsConfig BoltsCfg { get; set; } = new BoltsConfig();
    }

    /// <summary>
    /// 解析螺栓参数
    /// </summary>
    public class BoltParse
    {
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();

        public BoltParse(ParamsHandler paramsHandler)
        {
            m_paramsHandler = paramsHandler;
        }

        public BoltData GetBoltParams()
        {
            if (!ParseProps())
                return null;
            return m_boltData;
        }

        private bool ParseProps()
        {
            m_boltData.BoltsCfg.CopyFromBasicProps
               (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS));
            m_boltData.BoltsCfg.CopyFromHoleProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS));
            m_boltData.BoltsCfg.CopyFromContentProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS));
            m_boltData.BoltsCfg.CopyFromExtendProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS));
            m_boltData.BoltsCfg.CopyFromShapeProps
                (m_paramsHandler.GetIntValue(ParamNames.BOLT_SHAPE));

            m_boltData.VertOffsetMode = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_OFFSET_MODE);
            m_boltData.VertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET);
            m_boltData.HoriOffsetMode = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_OFFSET_MODE);
            m_boltData.HoriOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET);
            m_boltData.UpGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_UP_GAP);
            m_boltData.DownGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_DOWN_GAP);
            m_boltData.LeftGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_LEFT_GAP);
            m_boltData.RightGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_RIGHT_GAP);

            int vertNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
            var arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
            m_boltData.VertStride = HandlerStride(arrX, vertNum, 60.0);
            int horiNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
            var arrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_HORI_STRIDE);
            m_boltData.HoriStride = HandlerStride(arrY, horiNum, 80.0);

            var ignoreArr = m_paramsHandler.GetIntArrayValue(ParamNames.BOLT_IGNORE);
            m_boltData.IgnoreBolt = HandlerIgnore(ignoreArr, vertNum, horiNum);

            return true;
        }
        
        /// <summary>
        /// 处理螺栓间隔
        /// </summary>
        /// <param name="arr">螺栓间隔数组</param>
        /// <param name="num">螺栓数量</param>
        /// <param name="stride">默认间隔</param>
        /// <returns></returns>
        private double[] HandlerStride(double[] arr, int num, double stride)
        {
            //螺栓数目小于零 不创建螺栓
            if (num <= 0)
                return null;
            if (num == 1)
                return new double[] { 0 };

            var ret = new List<double>();
            if (arr == null || arr.Length == 0)
            {
                //数组不存在，采取默认值
                ret.AddRange(Enumerable.Repeat(stride, num - 1));
                return ret.ToArray();
            }

            //数组存在
            for (int i = 0; i < num - 1; i++)
            {
                //数组长度小于螺栓数目 添加尾元素扩充数组
                if (i > arr.Length - 1)
                    ret.Add(arr[arr.Length - 1]);
                else
                    ret.Add(arr[i]);
            }
            return ret.ToArray();
        }

        /// <summary>
        /// 处理螺栓忽略
        /// </summary>
        /// <param name="arr">要忽略的螺栓序列</param>
        /// <param name="vertNum">纵向螺栓数量</param>
        /// <param name="horiNum">横向螺栓数量</param>
        /// <returns></returns>
        private int[] ReverseIgnore(int[] arr, int vertNum, int horiNum)
        {
            if (vertNum <= 0 || horiNum <= 0 || arr == null || arr.Length == 0)
                return null;
            var boltSeq = new int[vertNum, horiNum]; //螺栓序列
            var index = 1;
            //初始化螺栓序列  按列方向
            for (int i = 0; i < horiNum; i++)
            {
                for (int j = 0; j < vertNum; j++)
                {
                    boltSeq[j, i] = index;
                    index++;
                }
            }
            int num = vertNum * horiNum;

            //拿取数据 -- 起到排序的作用
            var seq = new int[num];
            index = 0;
            for (int i = 0; i < vertNum; i++)
            {
                for (int j = 0; j < horiNum; j++)
                {
                    seq[index] = boltSeq[i, j];
                    index++;
                }
            }

            //输出
            List<int> nums = new List<int>();
            foreach (var item in arr)
            {
                if (item > 0 && item <= num)
                    nums.Add(seq[item - 1]);
            }
            if (nums.Count < 1)
                return null;
            return nums.ToArray();
        }

        /// <summary>
        /// 处理螺栓忽略
        /// </summary>
        /// <param name="arr">要忽略的螺栓序列</param>
        /// <param name="vertNum">纵向螺栓数量</param>
        /// <param name="horiNum">横向螺栓数量</param>
        /// <returns></returns>
        private int[] HandlerIgnore(int[] arr, int vertNum, int horiNum)
        {
            if (vertNum <= 0 || horiNum <= 0 || arr == null || arr.Length == 0)
                return new int[] { };

            int num = vertNum * horiNum;

            //输出
            List<int> nums = new List<int>();
            foreach (var item in arr)
            {
                if (item > 0 && item <= num)
                    nums.Add(item);
            }
            if (nums.Count < 1)
                return new int[] { };
            return nums.ToArray();
        }
    }
}
