﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using System.Linq;
using LoongCAD.STC;

namespace BeamSideEndPlate209
{
    public class HoleHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private string m_holeStandard = "TS10.9";
        private double m_upDiameter = double.MaxValue;
        private double m_downDiameter = double.MaxValue;
        private int m_holeNumber = 0;

        private SteelBeam m_endPlate = null;
        private double m_thickness = 0d;

        public HoleHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            this.ParseProps();

            this.AutoHoleDiameter();

            this.HandlerHole();
        }

        private bool ParseProps()
        {
            int standard = m_paramsHandler.GetIntValue(ParamNames.HOLE_STANDARD);
            switch (standard)
            {
                case 1:
                    m_holeStandard = "A";
                    break;
                case 2:
                    m_holeStandard = "B";
                    break;
                case 3:
                    m_holeStandard = "HS4.6";
                    break;
                case 4:
                    m_holeStandard = "HS8.8";
                    break;
                case 5:
                    m_holeStandard = "HS10.9";
                    break;
                case 6:
                    m_holeStandard = "TS8.8";
                    break;
                case 7:
                default:
                    m_holeStandard = "TS10.9";
                    break;
            }
            m_holeNumber = m_paramsHandler.GetIntValue(ParamNames.HOLE_NUMBER);

            //得到板厚度 -- 用于切割长度
            var props = m_paramsHandler.GetDObjectValue(ParamNames.PART_END_PROPS);
            if (props == null)
                return false;
            var m_props = new PlateProperties();
            m_props.SetProps(props);
            m_thickness = m_props.Thickness;

            m_endPlate = m_context.EndPlate;
            if (m_endPlate == null)
                return false;

            return true;
        }

        private void AutoHoleDiameter()
        {
            m_upDiameter = m_paramsHandler.GetDoubleValue(ParamNames.HOLE_UP_DIAMETER);
            m_downDiameter = m_paramsHandler.GetDoubleValue(ParamNames.HOLE_DOWN_DIAMETER);

            if(m_upDiameter == double.MaxValue && m_downDiameter == double.MaxValue)
            {
                //用户未输入 采取默认值
                m_upDiameter = 20.0;
                m_downDiameter = 20.0;
            }

            if (m_upDiameter != double.MaxValue && m_downDiameter != double.MaxValue)
                return;

            if(m_upDiameter == double.MaxValue)
            {
                m_upDiameter = m_downDiameter;
            }else{
                m_downDiameter = m_upDiameter;
            }
        }

        private void HandlerHole()
        {
            //无孔
            if (m_holeNumber < 2)
                return;

            var m_upHoleVertGap = m_paramsHandler.GetDoubleValue(ParamNames.HOLE_UP_VERT_GAP);
            var m_upHoleHoriGap = m_paramsHandler.GetDoubleValue(ParamNames.HOLE_UP_HORI_GAP);
            var m_downHoleVertGap = m_paramsHandler.GetDoubleValue(ParamNames.HOLE_DOWN_VERT_GAP);
            var m_downHoleHoriGap = m_paramsHandler.GetDoubleValue(ParamNames.HOLE_DOWN_HORI_GAP);

            //自适应孔偏移
            if(m_upHoleHoriGap == double.MaxValue && m_downHoleHoriGap == double.MaxValue)
            {
                m_upHoleHoriGap = 15.0;
                m_downHoleHoriGap = 15.0;
            }else if(m_upHoleHoriGap == double.MaxValue || m_downHoleHoriGap == double.MaxValue){
                if (m_downHoleHoriGap == double.MaxValue)
                    m_downHoleHoriGap = m_upHoleHoriGap;
                if (m_upHoleHoriGap == double.MaxValue)
                    m_upHoleHoriGap = m_downHoleHoriGap;
            }

            if(m_upHoleVertGap == double.MaxValue && m_downHoleVertGap == double.MaxValue)
            {
                m_upHoleVertGap = 37.5;
                m_downHoleVertGap = 37.5;
            }else if(m_upHoleVertGap == double.MaxValue || m_downHoleVertGap == double.MaxValue)
            {
                if (m_upHoleVertGap == double.MaxValue)
                    m_upHoleVertGap = m_downHoleVertGap;
                if (m_downHoleVertGap == double.MaxValue)
                    m_downHoleVertGap = m_upHoleVertGap;
            }

            switch (m_holeNumber)
            {
                case 2:
                    //一个孔
                    this.CreateHole(m_upDiameter, m_upHoleHoriGap, m_upHoleVertGap, new double[] { 0 });
                    break;
                case 3:
                    //两个孔
                    this.CreateHole(m_upDiameter, m_upHoleHoriGap, m_upHoleVertGap, new double[] { 0 });
                    this.CreateHole(m_upDiameter, -m_downHoleHoriGap, -m_downHoleVertGap, new double[] { 0 });
                    break;
                default:
                    //四个孔
                    this.CreateHole(m_upDiameter, -m_upHoleHoriGap, m_upHoleVertGap, new double[] { 2 * m_upHoleHoriGap });
                    this.CreateHole(m_upDiameter, -m_downHoleHoriGap, -m_downHoleVertGap, new double[] { 2 * m_downHoleHoriGap });
                    break;
            }
        }

        /// <summary>
        /// 创建洞
        /// </summary>
        /// <param name="diameter"></param>
        /// <param name="vertOffset"></param>
        /// <param name="horiOffset"></param>
        private void CreateHole(double diameter, double horiOffset, double vertOffset, double[] strideX)
        {
            if (diameter <= 0)
                return;

            Vec3 boltX = m_context.MainLine;
            Vec3 boltY = m_context.MainTopPlane.Normal.Normalize();
            Vec3 boltZ = boltX.Cross(boltY);

            var startPos = m_endPlate.CtrlPoints.First().WorldPos / 2
                + m_endPlate.CtrlPoints.Last().WorldPos / 2;
            var endPos = startPos + boltX * 150;

            BoltGroup curHoles = m_context.TargetComp.CreateBoltHoles(null, m_endPlate, startPos, endPos, true);
            if (curHoles == null)
                return;
            curHoles.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            curHoles.Rotation = 0;
            curHoles.RotationSide = Define.ObjectSide.SIDE_TOP;
            curHoles.BoltSize = diameter;
            curHoles.BoltStandard = m_holeStandard;
            curHoles.HoleX = 0.0;
            curHoles.HoleY = 0.0;
            curHoles.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
            curHoles.OverSizeHoleEnable0 = false;
            curHoles.OverSizeHoleEnable1 = false;
            curHoles.ArrayStrideX = strideX;
            curHoles.ArrayStrideY = new double[] { 0 };
            curHoles.HeadOffsetX = horiOffset;
            curHoles.VerticalOffset = vertOffset;
            curHoles.CutLength = m_thickness * 2.0 + 1.0;
            curHoles.Refresh();
        }
    }
}
