﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace BeamSideEndPlate209
{
    /// <summary>
    /// 零件页处理类
    /// </summary>
    public class PartHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();
        private Plane m_horiRefPlane = null;
        private Plane m_vertRefPlane = null;

        //工作坐标系
        private Vec3 m_axisX = Vec3.ZERO;
        private Vec3 m_axisY = Vec3.ZERO;
        private Vec3 m_axisZ = Vec3.ZERO;

        //主零件参数
        private Plane m_mainTopPlane = null;
        private Plane m_mainBottomPlane = null;

        //次零件参数
        private Plane m_subTopPlane = null;
        private Plane m_subBottomPlane = null;
        private Plane m_subUpPlane = null;
        private Plane m_subDownPlane = null;
        private Plane m_subAlignPlane = null;

        //端板
        private PlateProperties m_endProps = new PlateProperties();
        private double endWidth = 0.0;  //端板宽度
        //适应X向螺栓组得到的板长度
        private double adaptLength = double.MaxValue;
        private double m_plateGap = double.MaxValue;
        //控制点
        private Vec3 endPos0 = Vec3.ZERO;
        private Vec3 endPos1 = Vec3.ZERO;

        //前板
        private PlateProperties m_frontProps = new PlateProperties();

        //垫板
        private PlateProperties m_padProps1 = new PlateProperties();
        private PlateProperties m_padProps2 = new PlateProperties();
        private PlateProperties m_padProps3 = new PlateProperties();
        private int m_padNum1 = 0;
        private int m_padNum2 = 0;
        private int m_padNum3 = 0;
        private double m_padHoriMargin = double.MaxValue;
        private double m_padVertMargin = double.MaxValue;
        private double m_boltVertLength = 0d;
        private double m_boltHoriLength = 0d;

        //零件
        private SteelBeam m_endPlate = null;
        private SteelBeam m_frontPlate = null;
        private List<SteelBeam> m_subPlates = new List<SteelBeam>();

        private Dictionary<int, Vec3> m_boltMap = new Dictionary<int, Vec3>();
        private double m_boltSize = 0d;

        public PartHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            if (!ParseProps())
                return false;

            if (!HandlerEndPlate())
                return false;

            if (!HandlerFrontPlate())
                return false;

            this.HandlerPadPlate();

            this.CreateBolt();

            this.HandlerPadNotch();

            this.AlignSubBeam();

            return true;
        }

        private bool ParseProps()
        {
            //端板
            if (!ParseParam(m_endProps, ParamNames.PART_END_PROPS))
                return false;
            //前板
            if (!ParseParam(m_frontProps, ParamNames.PART_FRONT_PROPS))
                return false;

            //检测组件创建板梁厚度要求
            if(m_endProps.Thickness < 1e-6 || m_frontProps.Thickness < 1e-6)
                    return false;

            //得到螺栓属性
            BoltParse boltParse = new BoltParse(m_paramsHandler);
            m_boltData = boltParse.GetBoltParams();
            if (m_boltData == null)
                return false;
            m_boltHoriLength = SumArray(m_boltData.HoriStride);
            m_boltVertLength = SumArray(m_boltData.VertStride);
            adaptLength = m_boltData.UpGap + m_boltVertLength + m_boltData.DownGap;

            //垫板属性处理
            ParseParam(m_padProps1, ParamNames.PART_PAD1_PROPS);
            ParseParam(m_padProps2, ParamNames.PART_PAD2_PROPS);
            ParseParam(m_padProps3, ParamNames.PART_PAD3_PROPS);
            m_padNum1 = m_paramsHandler.GetIntValue(ParamNames.PART_PAD1_COUNT);
            m_padNum2 = m_paramsHandler.GetIntValue(ParamNames.PART_PAD2_COUNT);
            m_padNum3 = m_paramsHandler.GetIntValue(ParamNames.PART_PAD3_COUNT);
            m_plateGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FRONT_TO_END_GAP);
            m_padHoriMargin = m_paramsHandler.GetDoubleValue(ParamNames.PARAM_PAD_BOLT_HORI_MARGIN);
            m_padVertMargin = m_paramsHandler.GetDoubleValue(ParamNames.PARAM_PAD_BOLT_VERT_MARGIN);

            //次零件对齐面与主零件的距离
            double moveDist = m_plateGap + m_padProps1.Thickness * m_padNum1
                + m_padProps2.Thickness * m_padNum2 + m_padProps3.Thickness * m_padNum3
                + m_endProps.Thickness + m_frontProps.Thickness;
            m_subAlignPlane = m_context.MainNearPlane.Move(moveDist);

            //初始化零件面信息
            m_mainTopPlane = m_context.MainTopPlane;
            m_mainBottomPlane = m_context.MainBottomPlane;
            m_subTopPlane = m_context.SubTopPlane;
            m_subBottomPlane = m_context.SubBottomPlane;
            m_subUpPlane = m_context.SubUpPlane;
            m_subDownPlane = m_context.SubDownPlane;

            m_axisX = m_mainTopPlane.Normal.Normalize();
            m_axisY = m_context.MainLine;
            m_axisZ = m_subAlignPlane.Normal.Normalize();

            return true;
        }

        /// <summary>
        /// 处理端板
        /// </summary>
        private bool HandlerEndPlate()
        {
            if (m_endProps.Thickness <= 0)
                return false;

            //得到适应螺栓的板宽度
            endWidth = m_boltData.LeftGap + m_boltData.RightGap + m_boltHoriLength;
            if (endWidth <= 0)
                return false;

            //解析端板控制点的横向参照平面 -- 通过适应螺栓组的横向测量方式
            m_horiRefPlane = this.GetHoriRefPlane(m_subAlignPlane);
            if (m_horiRefPlane == null)
                return false;

            //解析端板控制点的横向参照平面 -- 通过适应螺栓组的纵向测量方式
            Plane zCrlPlane = m_subAlignPlane.Move(-m_endProps.Thickness); //控制点所在面
            var dir = m_axisX; //控制点移动方向
            m_vertRefPlane = this.GetVertRefPlane(zCrlPlane, dir);
            if (m_vertRefPlane == null)
                return false;

            //解析端板控制点的纵向参平面
            double endUpGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_END_TO_SUB_BOUND_UP_GAP);
            double endDownGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_END_TO_SUB_BOUND_DOWN_GAP);

            //规则：间距选项中有一个选项填值时 未填值的选项也要获取到该值
            if(endUpGap == double.MaxValue)
            {
                if (endDownGap != double.MaxValue)
                    endUpGap = endDownGap;
            }
            if(endDownGap == double.MaxValue)
            {
                if (endUpGap != double.MaxValue)
                    endDownGap = endUpGap;
            }

            //端板控制点
            Vec3 pos0, pos1;
            if (endUpGap != double.MaxValue)
            {
                //与次零件的上方间距控制
                pos0 = m_horiRefPlane.Intersect(zCrlPlane, m_subTopPlane);
                pos0 = pos0 - dir * endUpGap;
            }else {
                //适应螺栓组的纵向测量方式
                pos0 = m_horiRefPlane.Intersect(zCrlPlane, m_vertRefPlane);
                pos0 = pos0 + dir * m_boltData.UpGap;
            }

            //得到适应螺栓的板长度
            adaptLength = m_boltVertLength + m_boltData.UpGap + m_boltData.DownGap;
            if (endDownGap != double.MaxValue)
            {
                pos1 = m_horiRefPlane.Intersect(zCrlPlane, m_subBottomPlane);
                pos1 = pos1 + dir * endDownGap;
            }else {
                pos1 = pos0 - dir * adaptLength;
            }

            string profile = "PL" + m_endProps.Thickness + "*" + endWidth;
            m_endPlate = m_context.TargetComp.CreateSteelBeam(profile, pos0, pos1 ,true);
            if (m_endPlate == null)
                return false;

            //工作坐标系
            m_endPlate.WorkAxis = new Mat43(m_axisX, m_axisY, m_axisZ, Vec3.ZERO);
            m_endPlate.Rotation = 0;
            m_endPlate.RotationSide = Define.ObjectSide.SIDE_FRONT;
            m_endPlate.VertOffsetSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            m_endPlate.HorizonOffset = m_boltData.RightGap / 2 - m_boltData.LeftGap / 2;
            m_endPlate.Class = m_context.TargetComp.CompClass;
            m_endPlate.Refresh();
            m_endProps.ApplyToPart(m_endPlate);

            endPos0 = pos0;
            endPos1 = pos1;
            m_context.EndPlate = m_endPlate;

            return true;
        }

        /// <summary>
        /// 处理前板
        /// </summary>
        private bool HandlerFrontPlate()
        {
            if (m_frontProps.Thickness <= 0)
                return false;

            var frontWidth = endWidth;
            if (m_frontProps.Width != double.MaxValue)
                frontWidth = m_frontProps.Width;
            if (frontWidth <= 0)
                return false;

            var frontUpGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FRONT_TO_MAIN_BOUND_UP_GAP);
            var frontDownGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FRONT_TO_MAIN_BOUND_DOWN_GAP);

            //规则：间距选项中有一个选项填值时 未填值的选项也要获取到该值
            if(frontUpGap == double.MaxValue)
            {
                if (frontDownGap != double.MaxValue)
                    frontUpGap = frontDownGap;
            }
            if(frontDownGap == double.MaxValue)
            {
                if (frontUpGap != double.MaxValue)
                    frontDownGap = frontUpGap;
            }

            //前板控制点
            Vec3 pos0 = m_context.MainNearPlane.Intersect(m_mainTopPlane.Move(- 5.0), m_horiRefPlane);
            Vec3 pos1 = m_context.MainNearPlane.Intersect(m_mainBottomPlane.Move(-5.0), m_horiRefPlane);
            var vertDir = m_axisX; //控制点纵向移动方向
            var horiDir = m_axisZ; //控制点横向移动方向

            double moveDist = m_plateGap + m_padProps1.Thickness * m_padNum1
                + m_padProps2.Thickness * m_padNum2 
                + m_padProps3.Thickness * m_padNum3 + m_frontProps.Thickness;
            var newPos0 = endPos0 - horiDir * moveDist;
            var newPos1 = endPos1 - horiDir * moveDist;

            if (frontUpGap != double.MaxValue)
            {
                //由主零件的上方间距控制
                pos0 = m_context.MainNearPlane.Intersect(m_mainTopPlane.Move(-frontUpGap), m_horiRefPlane);
            }else if (vertDir.Dot(newPos0 - pos0) > 0)
            {
                //适应端板
                pos0 = newPos0;
            }

            if(frontDownGap != double.MaxValue)
            {
                pos1 = m_context.MainNearPlane.Intersect(m_mainBottomPlane.Move(-frontDownGap), m_horiRefPlane);
            }else if(vertDir.Dot(newPos1 - pos1) < 0)
            {
                pos1 = newPos1;
            }

            string profile = "PL" + m_frontProps.Thickness + "*" + frontWidth;
            m_frontPlate = m_context.TargetComp.CreateSteelBeam(profile, pos0, pos1, true);
            if (m_frontPlate == null)
                return false;
            //工作坐标系
            m_frontPlate.WorkAxis = new Mat43(m_axisX, m_axisY, m_axisZ, Vec3.ZERO);
            m_frontPlate.Rotation = 0;
            m_frontPlate.RotationSide = Define.ObjectSide.SIDE_FRONT;
            m_frontPlate.VertOffsetSide = Define.ObjectSide.SIDE_FRONT;
            m_frontPlate.HorizonOffset = m_boltData.RightGap / 2 - m_boltData.LeftGap / 2;
            m_frontPlate.Class = m_context.TargetComp.CompClass;
            m_frontPlate.Refresh();
            m_frontProps.ApplyToPart(m_frontPlate);
            m_context.FrontPlate = m_frontPlate;

            return true;
        }

        /// <summary>
        /// 处理垫板
        /// </summary>
        private bool HandlerPadPlate()
        {
            m_subPlates.Add(m_endPlate);

            //垫板1
            var dir = m_axisZ; //控制点横向移动方向
            Vec3 pos0 = m_vertRefPlane.Intersect(m_horiRefPlane, m_subAlignPlane.Move(-m_endProps.Thickness));
            for (int i = 1; i <= m_padNum1; i++)
            {
                pos0 = pos0 - dir * m_padProps1.Thickness;
                this.CreatePadPlate(new Vec3(pos0), m_padProps1);
            }

            //垫板2
            for(int i = 1; i <= m_padNum2; i++)
            {
                pos0 = pos0 - dir * m_padProps2.Thickness;
                this.CreatePadPlate(new Vec3(pos0), m_padProps2);
            }

            //垫板3
            for(int i = 1; i <= m_padNum3; i++)
            {
                pos0 = pos0 - dir * m_padProps3.Thickness;
                this.CreatePadPlate(new Vec3(pos0), m_padProps3);
            }

            return true;
        }

        /// <summary>
        /// 创建垫板
        /// </summary>
        /// <param name="pos0">起始控制点</param>
        /// <param name="m_props">垫板属性集</param>
        private void CreatePadPlate(Vec3 pos0, PlateProperties m_props)
        {
            if (m_props.Thickness <= 1e-6)
                return;

            double width = m_boltHoriLength + 2 * m_padHoriMargin;
            double height = m_boltVertLength + 2 * m_padVertMargin;
            if (width <= 1e-6 || height <= 1e-6)
                return;

            pos0 = pos0 + m_axisX * m_padVertMargin;
            var pos1 = pos0 - m_axisX * height;
            string profile = "PL" + m_props.Thickness + "*" + width;

            var m_padPlate = m_context.TargetComp.CreateSteelBeam(profile, pos0, pos1, true);
            if (m_padPlate == null)
                return;

            //工作坐标系
            m_padPlate.WorkAxis = new Mat43(m_axisX, m_axisY, m_axisZ, Vec3.ZERO);
            m_padPlate.Rotation = 0;
            m_padPlate.RotationSide = Define.ObjectSide.SIDE_FRONT;
            m_padPlate.VertOffsetSide = Define.ObjectSide.SIDE_FRONT;
            m_padPlate.Class = m_context.TargetComp.CompClass;
            m_padPlate.Refresh();
            m_props.ApplyToPart(m_padPlate);
            m_subPlates.Add(m_padPlate);
        }

        /// <summary>
        /// 得到螺栓组控制点的横向参照面
        /// </summary>
        /// <param name="alignPlane">次零件的对齐面</param>
        private Plane GetHoriRefPlane(Plane alignPlane)
        {
            Plane refPlane = null;
            if (alignPlane == null || m_subUpPlane == null ||
                m_subDownPlane == null || m_subTopPlane == null)
                return refPlane;

            //得到对齐面与次零件上下方向面的交点
            var upPoint = m_subUpPlane.Intersect(alignPlane, m_subTopPlane);
            var downPoint = m_subDownPlane.Intersect(alignPlane, m_subTopPlane);
            double moveDist = m_boltData.HoriOffset + m_boltHoriLength / 2;

            //螺栓组的横向测量方式
            switch (m_boltData.HoriOffsetMode)
            {
                case 1:
                    //从次零件的左边缘到最左边的螺栓
                    refPlane = new Plane(upPoint, m_context.MainLine);
                    refPlane.MoveThis(-moveDist);
                    break;
                case 3:
                    //从次零件的右边缘到最右边的螺栓
                    refPlane = new Plane(downPoint, m_context.MainLine);
                    refPlane.MoveThis(moveDist);
                    break;
                case 2:
                default:
                    //从次零件的中心线到螺栓的中心线
                    //upPoint与downPoint的中点在所求中心面上
                    refPlane = new Plane(upPoint/2 + downPoint/2, m_context.MainLine);
                    refPlane.MoveThis(-m_boltData.HoriOffset);
                    break;
            }

            return refPlane;
        }

        /// <summary>
        /// 得到螺栓组控制点的纵向参照面
        /// </summary>
        /// <param name="zPlane">控制点所在面</param>
        /// <param name="normal">参照面法线为定义的组件前方向</param>
        private Plane GetVertRefPlane(Plane zPlane, Vec3 normal)
        {
            Plane refPlane = null;
            if (zPlane == null || m_subUpPlane == null ||
                m_subBottomPlane == null || m_subTopPlane == null)
                return refPlane;

            //得到对齐面与次零件前后方向面的交点
            var topPoint = m_subTopPlane.Intersect(zPlane, m_subUpPlane);
            var BottomPoint = m_subBottomPlane.Intersect(zPlane, m_subUpPlane);
            double moveDist = m_boltData.VertOffset;

            //螺栓组的纵向测量方式
            switch (m_boltData.VertOffsetMode)
            {
                case 1:
                    //从次零件的上边缘到最上方的螺栓
                    refPlane = new Plane(topPoint, normal);
                    refPlane.MoveThis(-moveDist);
                    break;
                case 3:
                    //从次零件的下边缘到最下方的螺栓
                    refPlane = new Plane(BottomPoint, normal);
                    moveDist += m_boltVertLength;
                    refPlane.MoveThis(moveDist);
                    break;
                case 2:
                default:
                    //从螺栓的中心线到次零件的中心线
                    //topPoint与bottomPoint的中点在所求中心面上
                    refPlane = new Plane(topPoint / 2 + BottomPoint / 2, normal);
                    moveDist += m_boltVertLength / 2;
                    refPlane.MoveThis(moveDist);
                    break;
            }
            return refPlane;
        }

        /// <summary>
        /// 创建螺栓组
        /// </summary>
        private bool CreateBolt()
        {
            //若构件中的主次零件不存在 不创建螺栓组
            if (m_endPlate == null || m_frontPlate == null)
                return false;
            //若螺栓组的数目或间距不符合现实 不创建螺栓组
            if (m_boltData.VertStride == null || m_boltData.HoriStride == null)
                return false;

            var boltX = m_mainBottomPlane.Normal.Normalize();
            var boltY = m_context.MainLine;
            var boltZ = boltX.Cross(boltY);

            //改变螺栓方向
            int boltDir = m_paramsHandler.GetIntValue(ParamNames.PARAM_BOLT_DIR);
            if(boltDir == 2)
            {
                //反向
                boltZ = -boltZ;
            }
            
            Vec3 startPos = m_vertRefPlane.Intersect(m_horiRefPlane, m_subAlignPlane.Move(-m_endProps.Thickness))
                + m_subAlignPlane.Normal.Normalize() * m_endProps.Thickness;
            //10.0为了使错误情况下螺栓也能创建出来
            Vec3 endPos = startPos + boltX * (adaptLength + 10.0);

            BoltGroup bolts = m_context.TargetComp.CreateBoltGroup(m_frontPlate,
                m_subPlates.ToArray(), startPos, endPos, true);
            if (bolts == null)
                return false;
            bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
            bolts.Rotation = 0;
            m_boltData.BoltsCfg.FillBasicProps(bolts, true);
            m_boltData.BoltsCfg.FillHoleProps(bolts);
            m_boltData.BoltsCfg.FillContentProps(bolts);
            m_boltData.BoltsCfg.FillExtendProps(bolts);
            m_boltData.BoltsCfg.FillShape(bolts);
            bolts.ArrayStrideX = m_boltData.VertStride;
            bolts.ArrayStrideY = m_boltData.HoriStride;
            bolts.HeadOffsetX = 0;
            bolts.VerticalOffset = 0;
            var cutLength =  m_plateGap + m_padProps1.Thickness * m_padNum1
                + m_padProps2.Thickness * m_padNum2 + m_padProps3.Thickness * m_padNum3
                + m_endProps.Thickness + m_frontProps.Thickness;
            bolts.CutLength = 2 * cutLength;
            bolts.IgnoreIndices = m_boltData.IgnoreBolt;
            bolts.Refresh();

            //记录螺栓坐标
            if(bolts.Bolts != null && bolts.Bolts.Length > 0)
            {
                foreach (var item in bolts.Bolts)
                    m_boltMap.Add(item.ShapeIndex, item.WorldMatrix.Pos);
            }
            m_boltSize = bolts.BoltSize;

            return true;
        }

        private void HandlerPadNotch()
        {

            var notchType = m_paramsHandler.GetIntValue(ParamNames.PARAM_PAD_SHAPE);

            //无槽口
            if (notchType < 2)
                return;
            //没有螺栓
            if (m_boltMap.Count < 1)
            {
                m_boltMap = null;
                return;
            }
            //没有垫板
            if (m_subPlates.Count < 2)
                return;
            
            int vertNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
            int horiNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
            //一方螺栓数目不符合实际
            if (vertNum <= 0 || horiNum <= 0)
                return;

            int totalNum = vertNum * horiNum;
            var ignoreHash = new HashSet<int>(m_boltData.IgnoreBolt); //使用hash便于搜索

            //得到未删除的螺栓索引序列
            List<int> boltList = new List<int>();
            for (int i = 0; i < totalNum; i++)
            {
                if (!ignoreHash.Contains(i + 1))
                    boltList.Add(i);
            }
            if (boltList.Count < 1)
                return;

            int count = 0; //记录槽口序号
            var boltArr = boltList.ToArray();
            //螺栓朝向
            int boltDir = m_paramsHandler.GetIntValue(ParamNames.PARAM_BOLT_DIR);
            var yDir = Vec3.ZERO;
            if(boltDir == 2)
            {
                //反向
                yDir = new Vec3(-m_axisY); //螺栓y方向发生改变
            }else {
                yDir = new Vec3(m_axisY);
            }

            //水平槽
            if (notchType == 2)
            {
                //筛选螺栓坐标
                var horiLocation = new List<Vec3>();
                //槽口尺寸
                var notchLength = new List<double>();
                while(count < vertNum)
                {
                    int index = -1; //索引
                    int num = 0; //横向间距索引
                    var length = m_boltHoriLength + m_padHoriMargin;
                    for(int i = count; i < totalNum; i = i + vertNum)
                    {
                        //从未删除的螺栓序列中查找索引
                        index = this.BinarySearchIndex(boltArr, i);
                        if(index != -1)
                        {
                            horiLocation.Add(m_boltMap[index]); //保存槽口构建的基坐标
                            notchLength.Add(length); //保存槽口尺寸
                            break;
                        }
                        length -= m_boltData.HoriStride[num];
                        num++; //跳转到下一个间距值
                    }
                    count++; //螺栓索引步长
                }

                //检查槽口基坐标数量
                if(horiLocation.Count < 1)
                    return;
                //检查槽口尺寸序列数量
                if (notchLength.Count < 1 && notchLength.Count != horiLocation.Count)
                    return;

                this.CreatePadNotch(horiLocation.ToArray(), yDir, m_axisX, -m_axisZ, 
                    m_subPlates.ToArray(), notchLength.ToArray(), boltDir);
            }

            //垂直槽
            if (notchType == 3)
            {
                //筛选螺栓坐标
                var vertLocation = new List<Vec3>();
                //槽口尺寸
                var notchLength = new List<double>();
                while (count < totalNum)
                {
                    int index = -1; //索引
                    int num = 0; //横向间距索引
                    var length = m_boltVertLength + m_padVertMargin;
                    for (int i = count; i < totalNum; i++)
                    {
                        //从未删除的螺栓序列中查找索引
                        index = this.BinarySearchIndex(boltArr, i);
                        if (index != -1)
                        {
                            vertLocation.Add(m_boltMap[index]);
                            notchLength.Add(length);
                            break;
                        }
                        length -= m_boltData.VertStride[num];
                        num++;
                    }
                    count += vertNum; //螺栓索引步长
                }

                //检查槽口基坐标数量
                if (vertLocation.Count < 1)
                    return;
                //检查槽口尺寸序列数量
                if (notchLength.Count < 1 && notchLength.Count != vertLocation.Count)
                    return;

                this.CreatePadNotch(vertLocation.ToArray(), -m_axisX, yDir, -m_axisZ,
                    m_subPlates.ToArray(), notchLength.ToArray(), boltDir);
            }
        }

        /// <summary>
        /// 二分法查找数组中的索引
        /// </summary>
        /// <param name="arr"></param>
        /// <param name="target"></param>
        private int BinarySearchIndex(int[] arr, int target)
        {
            if (arr == null || arr.Length < 1)
                return -1;

            int left = 0;
            int right = arr.Length - 1;

            while(left <= right)
            {
                int mid = left + (right - left) / 2;

                if (arr[mid] == target)
                {
                    //找到了
                    return mid;
                }else if(arr[mid] < target)
                {
                    left = mid + 1; //在右边
                }else {
                    right = mid - 1; //在左边
                }
            }

            //没有查找到
            return -1;
        }

        /// <summary>
        /// 创建垫板的指状槽口
        /// </summary>
        /// <param name="boltPos">槽口基坐标(需变换)</param>
        /// <param name="xDir">坐标系X反向</param>
        /// <param name="yDir">坐标系Y反向</param>
        /// <param name="zDir">坐标系Z反向</param>
        /// <param name="pads">垫板集合</param>
        /// <param name="notchLength">槽口尺寸集合</param>
        private void CreatePadNotch(Vec3[] boltPos, Vec3 xDir, Vec3 yDir, Vec3 zDir, SteelBeam[] pads, double[] notchLength, int boltDir)
        {
            
            var tolerance = m_paramsHandler.GetDoubleValue(ParamNames.PARAM_TOLERANCE); //容许量
            var size = tolerance / 2 + m_boltSize / 2;
            var eps = 1.0; //消除切割探测误差

            var zTopPlane = m_subAlignPlane.Move(-m_endProps.Thickness);
            //映射基坐标
            for (int i  = 0; i < boltPos.Length; i++)
            {
                boltPos[i] = boltPos[i].ProjectTo(zTopPlane);
            }
            
            for(int i = 1; i < pads.Length; i++)
            {

                //受先前定义螺栓构建影响 i = 0时为端板 排除掉
                for(int j = 0; j < boltPos.Length; j++)
                {
                    var plsSide = new List<Plane>();
                    plsSide.Add(new Plane(boltPos[j] + yDir * size, yDir));
                    plsSide.Add(new Plane(boltPos[j] - xDir * size, xDir));
                    plsSide.Add(new Plane(boltPos[j] - yDir * size, yDir));
                    plsSide.Add(new Plane(boltPos[j] + xDir * (notchLength[j] + eps), xDir));

                    //得到顶底面间距 并作为得到下一块垫板基坐标的移动距离
                    var pos = pads[i].CtrlPoints.First().WorldPos;
                    var moveDist = zTopPlane.Dist(pos);
                    var zBottomPlane = zTopPlane.Invert().Move(moveDist);

                    var cutter = m_context.TargetComp.CreatePlateCutter(pads[i], 
                        zBottomPlane, zTopPlane, plsSide.ToArray(), true);

                    //向前移动
                    boltPos[j].MoveThis(zDir, moveDist);

                    if (cutter == null)
                        continue;
                    //设置为凸弧折角
                    //迫于螺栓反向后切割体控制点没有改变位置 所做的适应性操作
                    ChamferCtrlPoint chamferPoint0, chamferPoint1;
                    if(boltDir == 2)
                    {
                        //反向
                        chamferPoint0 = cutter.CtrlPoints[2] as ChamferCtrlPoint;
                        chamferPoint1 = cutter.CtrlPoints[3] as ChamferCtrlPoint;
                    }
                    else {
                        chamferPoint0 = cutter.CtrlPoints[0] as ChamferCtrlPoint;
                        chamferPoint1 = cutter.CtrlPoints[1] as ChamferCtrlPoint;
                    }
                    PlateTools.SetChamfer(chamferPoint0, Define.CornerCutType.CORNER_CUT_ROUND, size, size, cutter);
                    PlateTools.SetChamfer(chamferPoint1, Define.CornerCutType.CORNER_CUT_ROUND, size, size, cutter);
                } 
            }
        }

        /// <summary>
        /// 对齐次零件
        /// </summary>
        private void AlignSubBeam()
        {
            //定义端板和次梁之间间隙的限制值
            var gapSize = m_paramsHandler.GetDoubleValue(ParamNames.PARAM_GAP_SIZE);

            var point1 = m_subAlignPlane.Intersect(m_subTopPlane, m_subUpPlane);
            var point2 = m_subAlignPlane.Intersect(m_subTopPlane, m_subDownPlane);
            var point3 = m_subAlignPlane.Intersect(m_subBottomPlane, m_subUpPlane);
            var point4 = m_subAlignPlane.Intersect(m_subBottomPlane, m_subDownPlane);
            Plane refPlane = GetDirTopPlane(
                new Plane(point1, m_context.SubLine), new Plane(point2, m_context.SubLine),
                new Plane(point3, m_context.SubLine), new Plane(point4, m_context.SubLine));
            if(refPlane == null)
            {
                m_context.TargetComp.CreateAlignPlane(m_context.SubPart, m_subAlignPlane, true);
                return;
            }

            var pos1 = refPlane.Intersect(m_subTopPlane, m_subUpPlane);
            var pos2 = refPlane.Intersect(m_subTopPlane, m_subDownPlane);
            var pos3 = refPlane.Intersect(m_subBottomPlane, m_subUpPlane);
            var pos4 = refPlane.Intersect(m_subBottomPlane, m_subDownPlane);

            double realGap = this.MaxArray(
                m_subAlignPlane.Dist(pos1), m_subAlignPlane.Dist(pos2),
                m_subAlignPlane.Dist(pos3), m_subAlignPlane.Dist(pos4));
            //规则：如果实际间隙小于此值，则梁末端保持平直
            //如果实际间隙大于此值，则梁末端接合到端板上
            if (realGap < gapSize)
            {
                m_context.TargetComp.CreateAlignPlane(m_context.SubPart, refPlane, true);
            }
            else {
                m_context.TargetComp.CreateAlignPlane(m_context.SubPart, m_subAlignPlane, true);
            }

            m_context.SubAlignPlane = m_subAlignPlane;
        }

        /// <summary>
        /// 获取零件的属性对象并设置到对象字典中
        /// </summary>
        /// <param name="m_props">对象字典</param>
        /// <param name="paramName">零件名称</param>
        /// <returns></returns>
        private bool ParseParam(PlateProperties m_props, string paramName)
        {
            DObject plateProps =  m_paramsHandler.GetDObjectValue(paramName);
            if (plateProps == null)
                return false;
            m_props.SetProps(plateProps);

            if (m_props.Thickness <= 0)
                m_props.Thickness = 0;
            return true;
        }

        /// <summary>
        /// 计算给定数组的元素绝对值之和
        /// </summary>
        private double SumArray(double[] arr)
        {
            if (arr == null || arr.Length == 0)
                return 0.0;
            double sum = 0.0;
            foreach (var item in arr)
                sum += Math.Abs(item);
            return sum;
        }

        /// <summary>
        /// 计算给定序列中最大的元素
        /// </summary>
        /// <param name="arr">给定序列</param>
        /// <returns></returns>
        private double MaxArray(params double[] arr)
        {
            if (arr == null || arr.Length == 0)
                return 0d;

            double max = arr[0];
            for(int i = 1; i < arr.Length; i++)
            {
                if (max < arr[i])
                    max = arr[i];
            }
            return max;
        }

        /// <summary>
        /// 得到方向上靠前的面
        /// </summary>
        /// <param name="planes">法线方向相同的面集合</param>
        /// <returns></returns>
        private Plane GetDirTopPlane(params Plane[] planes)
        {
            if (planes == null || planes.Length == 0)
                return null;

            var refPlane = planes[0];
            for(int i = 1; i < planes.Length; i++)
            {
                //表明planes[i] 更靠前
                if (refPlane.SignedDist(planes[i].Origin) > 0)
                    SwapVar(ref refPlane, ref planes[i]);
            }
            return refPlane;
        }

        /// <summary>
        /// 交换两个面的属性信息
        /// </summary>
        /// <param name="plane1">面1</param>
        /// <param name="plane2">面2</param>
        private void SwapVar(ref Plane plane1, ref Plane plane2)
        {
            var temp = plane1;
            plane1 = plane2;
            plane2 = temp;
        }
    }
}
