﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace BeamSideEndPlate209
{
    public class StiffHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PlateProperties m_plateProps = new PlateProperties();
        private int m_chamferType = 0;
        private double m_chamferX = double.MaxValue;
        private double m_chamferY = double.MaxValue;
        private double m_gap = double.MaxValue;
        private Plane zCtrlPlane = null;

        //主零件参数
        private Plane m_connRefPlane = null;
        private Plane m_stiffRefPlane = null;
        private Plane m_insideFlange1 = null;
        private Plane m_insideFlange2 = null;

        public StiffHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            if (!ParseProps())
                return;

            if (!ParseMainPlane())
                return;

            if (!CreateConnectPlate())
                return;
            this.CreateStiff();
        }

        private bool ParseProps()
        {
            if (!ParseParam(m_plateProps, ParamNames.PART_CONNECT_PROPS))
                return false;

            m_chamferType = m_paramsHandler.GetIntValue(ParamNames.PARAM_CHAMFER_TYPE);
            m_chamferX = m_paramsHandler.GetDoubleValue(ParamNames.PARAM_CHAMFER_X);
            m_chamferY = m_paramsHandler.GetDoubleValue(ParamNames.PARAM_CHAMFER_Y);

            //迫于定位控制点折角 X与Y方向不定 对于弧形折角尺寸的适应性修改
            if (m_chamferType > 2)
                // type > 2时 为凸弧或凹弧折角
                m_chamferY = m_chamferX;

            m_gap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_STIFF_TO_MAIN_GAP);

            return true;
        }

        /// <summary>
        /// 解析构件板的主零件面信息
        /// </summary>
        private bool ParseMainPlane()
        {
            var mainPlanes = m_context.MainPlanes;
            if (mainPlanes.TopFlangeInside == null || mainPlanes.BottomFlangeInside == null)
                return false;
            if (mainPlanes.BackWeb == null || mainPlanes.FrontWeb == null)
                return false;

            //若腹板的法线方向与靠近次零件的包围面法线方向相同
            //则此腹板为构建连接板的平面 否则为构建加劲肋的平面
            if(m_context.MainNearPlane.Normal.Dot(mainPlanes.BackWeb.Normal) > 0)
            {
                m_connRefPlane = mainPlanes.BackWeb;
                m_stiffRefPlane = mainPlanes.FrontWeb;
            }else {
                m_connRefPlane = mainPlanes.FrontWeb;
                m_stiffRefPlane = mainPlanes.BackWeb;
            }

            //筛选内侧翼缘 原则：定义flange1的法线方向与世界方向相反
            var insideFlange = new Plane[] { 
                new Plane(mainPlanes.TopFlangeInside), 
                new Plane(mainPlanes.BottomFlangeInside)
            };
            if(insideFlange[0].Normal.Dot(m_context.MainTopPlane.Normal) > 0)
            {
                SwapVar(ref insideFlange[0], ref insideFlange[1]);
            }

            m_insideFlange1 = insideFlange[0].Move(m_gap);
            if (m_plateProps.Height != double.MaxValue)
            {
                m_insideFlange2 = m_insideFlange1.Move(m_plateProps.Height);
            }else {
                //此时板的长度由gap控制
                //若gap的间距2倍大于dist 此时板的长度小于等于0 不创建板
                double dist = mainPlanes.TopFlangeInside.Dist(mainPlanes.BottomFlangeInside.Origin);
                if (m_gap >= dist / 2)
                    return false;

                m_insideFlange2 = insideFlange[1].Move(m_gap);
            }
            return true;
        }

        private bool CreateConnectPlate()
        {
            //解析zCtrlPlane
            if (m_context.SubAlignPlane == null)
                return false;
            var alignPlane = m_context.SubAlignPlane;

            //得到对齐面与次零件上下方向面的交点
            var upPoint = m_context.SubUpPlane.Intersect(alignPlane, m_context.SubTopPlane);
            var downPoint = m_context.SubDownPlane.Intersect(alignPlane, m_context.SubTopPlane);
            //upPoint与downPoint的中点在连接板顶底面的中心面上
            zCtrlPlane = new Plane(upPoint / 2 + downPoint / 2, m_context.MainLine);

            var plsSide = new List<Plane>();
            plsSide.Add(m_connRefPlane);
            plsSide.Add(m_insideFlange1);

            //根据板属性集是否填写处理板宽度
            if(m_plateProps.Width != double.MaxValue)
            {
                plsSide.Add(m_connRefPlane.Move(m_plateProps.Width));
            }else {
                //TODO:有待进一步的确定这2.5的距离规律
                plsSide.Add(m_context.MainNearPlane.Move(-2.5));
            }
            plsSide.Add(m_insideFlange2);

            //迫于不能固定主零件的构建方向 先从中心面zCtrlPlane出发创建板的控制点
            var ctrlPoints = this.GetCtrlPoints(plsSide.ToArray(), zCtrlPlane);
            if (ctrlPoints.Length < 3)
                return false;
            //再根据板的创建方向进行调整
            this.AdjustCtrlPoints(ctrlPoints, m_plateProps.Thickness);

            PartBase connPlate = m_context.TargetComp.CreateSteelPlate(ctrlPoints, 
                m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (connPlate == null)
                return false;
            connPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(connPlate);

            //设置折角
            var chamferPoint1 = connPlate.CtrlPoints.First() as ChamferCtrlPoint;
            PlateTools.SetChamfer(chamferPoint1, m_chamferType, m_chamferY, m_chamferX, connPlate);

            if(m_plateProps.Height == double.MaxValue)
            {
                //此时创建折角 否则不创建
                var chamferPoint2 = connPlate.CtrlPoints.Last() as ChamferCtrlPoint;
                PlateTools.SetChamfer(chamferPoint2, m_chamferType, m_chamferX, m_chamferY, connPlate);
            }
            m_context.ConnPlate = connPlate;

            return true;
        }

        /// <summary>
        /// 创建加劲肋
        /// </summary>
        private void CreateStiff()
        {
            int stiffEnabled = m_paramsHandler.GetIntValue(ParamNames.PARAM_STIFF_ENABLED);
            // stiffEnabled = 2 时创建加劲肋
            if (stiffEnabled != 2)
                return;

            var plsSide = new List<Plane>();
            plsSide.Add(m_stiffRefPlane);
            plsSide.Add(m_insideFlange1);

            //根据板属性集是否填写处理板宽度
            if (m_plateProps.Width != double.MaxValue)
            {
                plsSide.Add(m_stiffRefPlane.Move(m_plateProps.Width));
            }else {
                //筛选与m_stiffRefPlane平行且法线同向的包围面
                if(m_context.MainPlanes.BackBound.Normal.Dot(m_stiffRefPlane.Normal) > 0)
                {
                    plsSide.Add(m_context.MainPlanes.BackBound);
                }else {
                    plsSide.Add(m_context.MainPlanes.FrontBound);
                }
            }
            plsSide.Add(m_insideFlange2);

            //迫于不能固定主零件的构建方向 先从中心面zCtrlPlane出发创建板的控制点
            var ctrlPoints = this.GetCtrlPoints(plsSide.ToArray(), zCtrlPlane);
            if (ctrlPoints.Length < 3)
                return;
            //再根据板的创建方向进行调整
            this.AdjustCtrlPoints(ctrlPoints, m_plateProps.Thickness);

            PartBase stiffPlate = m_context.TargetComp.CreateSteelPlate(ctrlPoints,
                m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (stiffPlate == null)
                return;
            stiffPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(stiffPlate);

            //设置折角
            var chamferPoint1 = stiffPlate.CtrlPoints.First() as ChamferCtrlPoint;
            PlateTools.SetChamfer(chamferPoint1, m_chamferType, m_chamferY, m_chamferX, stiffPlate);

            if (m_plateProps.Height == double.MaxValue)
            {
                //此时创建折角 否则不创建
                var chamferPoint2 = stiffPlate.CtrlPoints.Last() as ChamferCtrlPoint;
                PlateTools.SetChamfer(chamferPoint2, m_chamferType, m_chamferX, m_chamferY, stiffPlate);
            }
            m_context.StiffPlate = stiffPlate;
        }

        /// <summary>
        /// 获取零件的属性对象并设置到对象字典中
        /// </summary>
        /// <param name="m_props">对象字典</param>
        /// <param name="paramName">零件名称</param>
        /// <returns></returns>
        private bool ParseParam(PlateProperties m_props, string paramName)
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(paramName);
            if (plateProps == null)
                return false;
            m_props.SetProps(plateProps);

            //板的属性值不能小于0
            if (m_props.Thickness <= 0 || m_props.Width <= 0 || m_props.Height <= 0)
                return false;

            return true;
        }

        /// <summary>
        /// 交换两个面的属性信息
        /// </summary>
        /// <param name="plane1">面1</param>
        /// <param name="plane2">面2</param>
        private void SwapVar(ref Plane plane1, ref Plane plane2)
        {
            var temp = plane1;
            plane1 = plane2;
            plane2 = temp;
        }

        /// <summary>
        /// 得到板的控制点
        /// </summary>
        /// <param name="plsSide">侧面数组</param>
        /// <param name="refPlane">控制点所在顶面</param>
        /// <returns></returns>
        private Vec3[] GetCtrlPoints(Plane[] plsSide, Plane refPlane)
        {
            if (plsSide == null || plsSide.Length < 2 || refPlane == null)
                return null;

            int len = plsSide.Length;
            var ctrlPoints = new List<Vec3>();
            for(int i = 0; i < len; i++)
            {
                var point = refPlane.Intersect(plsSide[i], plsSide[ (i + 1) % len]);
                ctrlPoints.Add(point);
            }

            //构件平面的点数量不能小于3
            if (ctrlPoints.Count < 3)
                return null;
            return ctrlPoints.ToArray();
        }

        private void AdjustCtrlPoints(Vec3[] ctrlPoints, double thickness)
        {
            //原则：根据创建板的法线方向 往反方向移动 板厚度 / 2
            var axisX = (ctrlPoints[1] - ctrlPoints[0]).Normalize();
            var axisY = (ctrlPoints.Last() - ctrlPoints[0]).Normalize();
            var normal = axisX.Cross(axisY);

            var moveDist = -thickness / 2;
            for(int i = 0; i < ctrlPoints.Length; i++)
            {
                ctrlPoints[i] = ctrlPoints[i] + normal * moveDist;
            }
        }
    }
}
