﻿using System;
using BoltedGusset_11_.Main;
using BoltedGusset_11_.Tool;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;

namespace BoltedGusset_11_.Running
{
    public class BuildConnPlate : RunningBaseClass
    {

        public BuildConnPlate(ComponentInstanceContext context, ParserSubPart parserSubPart, ParserGussetBolts parserGussetBolts, BuildGussetPlate buildGussetPlate) : base(context)
        {
            ParserSubPart = parserSubPart;
            ParserGussetBolts = parserGussetBolts;
            BuildGussetPlate = buildGussetPlate;
        }

        private readonly ParserSubPart ParserSubPart;
        private readonly ParserGussetBolts ParserGussetBolts;
        private readonly BuildGussetPlate BuildGussetPlate;

        private ParamData AllParamData { get;} = new ParamData();

        private class ParamData 
        {
            /// <summary>
            /// 连接板的厚度,沿着组件z方向
            /// </summary>
            public double Thickness { get; set; }
            /// <summary>
            /// 连接板的宽度,沿着组件x方向
            /// </summary>
            public double Width { get; set; }
            /// <summary>
            /// 连接板的高度,沿着组件y方向
            /// </summary>
            public double Height{ get; set; }
            /// <summary>
            /// 连接部分与节点板顶底的偏移
            /// </summary>
            public double TopOffSet { get; set; }
        }

        #region 连接板相关
        /// <summary>
        /// 截面类型
        /// </summary>
        private string Profile { get; set; } = "";

        /// <summary>
        /// Y方向远端的构建点
        /// </summary>
        internal Vec3 m_topBuildP;
        /// <summary>
        /// Y方向近端的构建点
        /// </summary>
        internal Vec3 m_btmBuildP;

        /// <summary>
        /// 创建好的连接板实例
        /// </summary>
        public SteelBeam ConnectionPlate { get; private set; }

        /// <summary>
        /// 能否创建位置一的情况
        /// </summary>
        internal bool m_bCanBuildSide1;
        /// <summary>
        /// 能否创建位置二的情况
        /// </summary>
        internal bool m_bCanBuildSide2 = false;
        /// <summary>
        /// 相对节点板的位置
        /// </summary>
        private int m_position;
        #endregion

        public override bool Init()
        {
            if (!InitParamData()) return false;
            return base.Init();
        }

        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.Thickness = m_pub.ConnectionPlatePro.Thickness;
            AllParamData.Width = m_pub.ConnectionPlatePro.Width;
            AllParamData.TopOffSet = handler.GetDoubleValue (ParamNames.PIC_TOP_DISTANCE_BETWEEN_CONNECTION_PART_AND_GUSSET_PLATE_EDGE);
            
            return true;
        }

        private void PrepareTopOffSet(double dist)
        {
            if (Equals(AllParamData.TopOffSet, double.MaxValue)) {
                AllParamData.TopOffSet = BuildGussetPlate.m_distTopBtm/2 - dist/2;
            }
        }

        public override bool Prepare()
        {
            var select = m_pub.SelectConnectionPlateOrLProfile;
            m_bCanBuildSide1 = select == 2 || select == 3;
            if (AllParamData.Thickness <= 0) {
                m_bCanBuildSide1 = false;
            }
            m_position = select;

            var gBolts= ParserGussetBolts;
            //处理宽度未填写的情况，其数值等于螺栓在x方向的长度
            if (Equals(AllParamData.Width, double.MaxValue)) {
                AllParamData.Width = gBolts.GetConnectionPlateWidth();
                gBolts.HandlerWightNoFill();
            }
            else {
                gBolts.HandlerWightHasFill();
            }
            AllParamData.Height = gBolts.GetConnectionPlateHeight(); 
            Profile = "PL" + AllParamData.Thickness + "*" + AllParamData.Width;
            
            //处理构建点
            ParserGussetBolts.CalculateBoltBuildPoint(AllParamData.Width);
            m_topBuildP = ParserGussetBolts.TopBoltBuild; 
            m_btmBuildP= ParserGussetBolts.BtmBoltBuild; 
            //额外自适应处理TopOffSet
            PrepareTopOffSet(m_topBuildP.Dist(m_btmBuildP));
            //处理偏移
            PointHelper.MoveRef(ref m_topBuildP,-ParserSubPart.AxisY,AllParamData.TopOffSet);
            PointHelper.MoveRef(ref m_btmBuildP,-ParserSubPart.AxisY,AllParamData.TopOffSet);
            if (m_position == 2)//前侧
            {
                var frontPl = new Plane(ParserSubPart.NearBuildPlane.Move(AllParamData.Thickness / 2));
                PointHelper.MoveToRef(ref m_topBuildP,frontPl);
                PointHelper.MoveToRef(ref m_btmBuildP,frontPl);
            }
            else if (m_position == 3) {//后侧
                var backPl = new Plane(ParserSubPart.FarBuildPlane.Move(AllParamData.Thickness / 2));
                PointHelper.MoveToRef(ref m_topBuildP,backPl);
                PointHelper.MoveToRef(ref m_btmBuildP,backPl);
            }
            return true;
        }

        public override bool Execute()
        {
            var gusset = BuildGussetPlate;

            if (!m_bCanBuildSide1)
            {
                return true;
            }
            var target = m_context.TargetComp;
            
            ConnectionPlate = target.CreateSteelBeam(Profile, m_btmBuildP, m_topBuildP, true);
            if (ConnectionPlate == null)
            {
                Console.WriteLine(@"create steelBeam fail");
                return false;
            }
            ConnectionPlate.WorkAxis = new Mat43(ParserSubPart.AxisX, ParserSubPart.AxisY, ParserSubPart.AxisZ, Vec3.ZERO);
            //处理旋转
            var side = gusset.GussetPlate.OffsetSide;
            ConnectionPlate.RotationSide = side == Define.ObjectSide.SIDE_RIGHT ? Define.ObjectSide.SIDE_FRONT : Define.ObjectSide.SIDE_BACK;
            ConnectionPlate.Flush();
            m_pub.ConnectionPlatePro.ApplyToPart(ConnectionPlate);
            
            m_context.BuildWeld.BuildWeldForConnPlate(ParserSubPart,BuildGussetPlate,ConnectionPlate,m_btmBuildP,m_topBuildP
                ,AllParamData.Thickness,AllParamData.Width,AllParamData.Height);
            return true;
        }
    }
}