﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using BoltedGusset_11_.Main;
using BoltedGusset_11_.Tool;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;

namespace BoltedGusset_11_.Running
{
    public class SubData
    {
        /// <summary>
        /// 顶底的高度值
        /// </summary>
        public double TopBottomDist;
        /// <summary>
        /// 前后距离
        /// </summary>
        public double FrontBackDist;
        /// <summary>
        /// 支撑的厚度，不同截面类型的含义不同
        /// </summary>
        public double Thickness;
        /// <summary>
        /// 离主零件近的支撑控制点
        /// </summary>
        public Vec3 NearPoint = Vec3.ZERO;
        /// <summary>
        /// 离主零件远的支撑控制点
        /// </summary>
        public Vec3 FarPoint = Vec3.ZERO;
        /// <summary>
        /// NearPoint指向FarPoint
        /// </summary>
        public Vec3 NearToFarLine = Vec3.ZERO;
        /// <summary>
        /// TopPlane与AlignNearPlane以及节点板平面GussetPlateBuildPlane的交点
        /// </summary>
        public Vec3 TopNearAlignPoint = Vec3.ZERO;
        /// <summary>
        /// TopPlane与AlignFarPlane以及节点板平面GussetPlateBuildPlane的交点
        /// </summary>
        public Vec3 TopFarAlignPoint = Vec3.ZERO;
        /// <summary>
        /// BottomPlane与AlignNearPlane以及节点板平面GussetPlateBuildPlane的交点
        /// </summary>
        public Vec3 BottomNearAlignPoint = Vec3.ZERO;
        /// <summary>
        /// BottomPlane与AlignFarPlane以及节点板平面GussetPlateBuildPlane的交点
        /// </summary>
        public Vec3 BottomFarAlignPoint = Vec3.ZERO;
        /// <summary>
        /// 支撑的解析
        /// </summary>
        public Beam Beam;
        /// <summary>
        /// 按梁解析后的解析体
        /// </summary>
        public BeamPlanes BeamPlanes;
        /// <summary>
        /// 支撑头尾面中离主零件近的面
        /// </summary>
        public Plane NearPlane;
        /// <summary>
        /// 支撑头尾面中离主零件远的面
        /// </summary>
        public Plane FarPlane;
        /// <summary>
        /// 次零件和Y轴最同向的面
        /// </summary>
        public Plane TopPlane;
        /// <summary>
        /// TopPlane的背离面
        /// </summary>
        public Plane BottomPlane;
        /// <summary>
        /// 次零件和Z轴最同向的面
        /// </summary>
        public Plane FrontPlane;
        /// <summary>
        /// FrontPlane的背离面
        /// </summary>
        public Plane BackPlane;
        /// <summary>
        /// 支撑最终的对齐面,法线应该朝向支撑
        /// </summary>
        public Plane NearAlignPlane { get; private set; }
        /// <summary>
        /// 支撑与节点板的对齐面，法线背离主零件，朝向支撑
        /// 与AlignPlaneOutSide的距离为BoltAreaLength
        /// </summary>
        public Plane FarAlignPlane { get; private set; }
        // NOTE - 获取螺栓区域的长度
        /// <summary>
        /// 螺栓区域的长度值
        /// </summary>
        public double BoltAreaLength = 150.0d;
        /// <summary>
        /// TopFarAlignPoint的偏移距离，沿着法线
        /// </summary>
        public double TopFarAlignPlaneOffset;
        /// <summary>
        /// BottomFarAlignPoint的偏移距离，沿着法线
        /// </summary>
        public double BottomFarAlignPlaneOffset;
        /// <summary>
        /// 该支撑的截面类型值
        /// </summary>
        public Profile Profile;
        
        public void UpdateFarAlignPlane(Vec3 adjustP)
        {
            FarAlignPlane = new Plane(adjustP, FarAlignPlane.Normal);
            NearAlignPlane = FarAlignPlane.Move(-BoltAreaLength);
        }
        public void UpdateNearAlignPlane(Vec3 adjustP)
        {
            NearAlignPlane = new Plane(adjustP, NearAlignPlane.Normal);
            FarAlignPlane = NearAlignPlane.Move(BoltAreaLength);
        }
        
        public void UpdateNearAlignPlane(Plane adjustPl)
        {
            NearAlignPlane = new Plane(adjustPl);
            FarAlignPlane = NearAlignPlane.Move(BoltAreaLength);
        }

        public void UpdateNearAlignPlane(double adjustDist)
        {
            NearAlignPlane.MoveThis(adjustDist);
            FarAlignPlane = NearAlignPlane.Move(BoltAreaLength);
        }
    }
    
    /// <summary>
    /// 子零件解析、准备数据
    /// </summary>
    public class ParserSubPart:RunningBaseClass
    {
        public ParserSubPart(ComponentInstanceContext context, ParserBraceBolts braceBolts) : base(context)
        {
            m_braceBolts = braceBolts;
        }

        private readonly ParserBraceBolts m_braceBolts;

        /// <summary>
        /// 子零件的数组,下标为选取的顺序
        /// </summary>
        internal List<SubData> SubData { get; } = new List<SubData>();
        /// <summary>
        /// 子零件的数组，下标为沿着mainLine方向顺序，最近的为0
        /// </summary>
        internal List<SubData> SubDataSort { get; } = new List<SubData>();
        /// <summary>
        /// 子零件的数目
        /// </summary>
        internal int m_subNum;
        /// <summary>
        /// 主零件
        /// </summary>
        internal Beam m_mainPart;
        /// <summary>
        /// 实际的节点板在主零件上构件点的参考面，尚未偏移
        /// </summary>
        internal Plane FaceBuildPlane
        { get => new Plane(m_faceBuildPlane); private set => m_faceBuildPlane = new Plane(value);}
        /// <summary>
        /// 主零件朝向支撑侧的面
        /// </summary>
        public Plane FaceToSubPlane 
        { get => new Plane(m_faceToSubPlane); private set => m_faceToSubPlane =new Plane(value);}
        /// <summary>
        /// 主零件背朝次零件的面
        /// </summary>
        internal Plane BackToSubPlane
        { get => new Plane(m_backToSubPlane); private set => m_backToSubPlane =new Plane(value);}
        /// <summary>
        /// 节点板的构建平面,与Z轴接近同向
        /// </summary>
        internal Plane RefSubBuildPlane
        { get => new Plane(m_refSubBuildPlane); private set => m_refSubBuildPlane =new Plane(value);}
        /// <summary>
        /// 组件的构建坐标系X,FaceToSubPlane的法线方向
        /// </summary>
        internal Vec3 AxisX { get; private set; }
        /// <summary>
        /// 组件的构建坐标系Y,主零件从创建起始点指向创建终止点的线,方向为第一个的选择的sub指向最后一个
        /// </summary>
        internal Vec3 AxisY { get; private set; }
        /// <summary>
        /// 组件的构建坐标系Z,根据x,y确定下的右手坐标系
        /// </summary>
        internal Vec3 AxisZ { get; private set; }
        /// <summary>
        /// 参考的子零件
        /// </summary>
        internal SubData m_subPartsRef;
        private Plane m_faceToSubPlane;
        /// <summary>
        /// 主零件背朝次零件的面
        /// </summary>
        private Plane m_backToSubPlane;
        private Plane m_refSubBuildPlane;
        private Plane m_faceBuildPlane;
        /// <summary>
        /// 是否处于增长的特殊模式
        /// </summary>
        public bool m_bExtensionMode;
        /// <summary>
        /// 主零件的起始点指向终点的方向,单位向量
        /// </summary>
        private Vec3 MainLine;
        private Plane m_nearBuildPlane;
        private Plane m_farBuildPlane;
        private Plane m_centreBuildPlane;
        /// <summary>
        /// 构建点底部的延长控制点
        /// </summary>
        internal Vec3 m_btmExtendedPt = Vec3.ZERO;
        /// <summary>
        /// 构建点顶部的延长控制点
        /// </summary>
        internal Vec3 m_topExtendedPt = Vec3.ZERO;
        /// <summary>
        /// 离z轴远的节点板平面,与NearBuildPlane法线相反
        /// </summary>
        public Plane FarBuildPlane
        { get => new Plane(m_farBuildPlane); private set => m_farBuildPlane = new Plane(value); }
        /// <summary>
        /// 离z轴近的节点板平面,与Z轴接近同向
        /// </summary>
        public Plane NearBuildPlane
        { get => new Plane(m_nearBuildPlane); private set => m_nearBuildPlane = new Plane(value); }
        /// <summary>
        /// 节点板厚度侧的中心平面
        /// 法线同NearBuildPlane
        /// </summary>
        public Plane CentreBuildPlane
        { get => new Plane(m_centreBuildPlane); set => m_centreBuildPlane = new Plane(value); }

        /// <summary>
        /// 所有的与面板相关的值
        /// </summary>
        internal ParamData AllParamData { get; } = new ParamData();
        /// <summary>
        /// 面板中的值。
        /// </summary>
        internal class ParamData 
        {
            /// <summary>
            /// 主零件面朝次零件的面偏移值 
            /// </summary>
            public double OffSetForFaceBuildPlane { get; set; }
            /// <summary>
            /// 主零件与支撑的距离，影响对齐面
            /// </summary>
            public double DistanceBetweenMainPartAndBrace { get; set; }
            /// <summary>
            /// 支撑之间的距离
            /// </summary>
            public double MinDistanceBetweenBrace { get; set; }
            #region 节点板拐角处长度
            /// <summary>
            /// 节点板拐角边缘长度，顶部支撑外侧
            /// </summary>
            public double LengthEdgeOfGussetTopOut { get; set; } 
            /// <summary>
            /// 节点板拐角边缘长度，顶部支撑内侧
            /// </summary>
            public double LengthEdgeOfGussetTopIn  { get; set; }
            /// <summary>
            /// 节点板拐角边缘长度，底部支撑外侧
            /// </summary>
            public double LengthEdgeOfGussetBottomOut { get; set; }
            /// <summary>
            /// 节点板拐角边缘长度，底部支撑内侧
            /// </summary>
            public double LengthEdgeOfGussetBottomIn  { get; set; }
            #endregion
            /// <summary>
            /// 节点板顶部拐角
            /// </summary>
            public double GussetCornerAngleTop { get; set; }
            /// <summary>
            /// 节点板底部拐角
            /// </summary>
            public double GussetCornerAngleBottom { get; set; }
            /// <summary>
            /// 节点板在支撑的位置选择
            /// </summary>
            public int PosOnBraceSelect { get; set; }
            /// <summary>
            /// 节点板的Z轴偏移
            /// </summary>
            internal double GussetPlateZOffset { get;set; }
        }

        public override bool Prepare()
        {
            return PrepareData();
        }

        public override bool Init()
        {
            if (!InitMain()) return false;
            if (!InitSubDataList()) return false;
            if (!InitBaseData()) return false;
            if (!InitParamData()) return false;
            return true;
        }

        /// <summary>
        /// 准备剩余数据
        /// </summary>
        /// <returns></returns>
        private bool PrepareData()
        {
            PrepareNearFarCentreBuildPlane();
            //处理螺栓区域的长度
            PrepareBoltsData();
            PrepareSubDataWithBraceBolts();
            // NOTE 处理支撑对齐面
            //第一次处理，直接对齐到主零件的offsetFaceInSub
            PrepareAlignPlaneOffset();
            PrepareAlignPoint();
            //第二次处理，处理相邻支撑间的距离
            PrepareMinDistanceBetweenBrace();
            # region 处理凸包
            if (m_pub.ShapeSelect != 3) {
                PrepareBraceBoundary();
                PrepareConvexHull();
            }
            #endregion
            //无视之前所有处理，强制对其支撑


            var handler = m_context.ParamsHandler;
            var offList = new List<double>()
            {
                handler.GetDoubleValue(ParamNames.PIC_BRACE_1_ALIGN_DIST),
                handler.GetDoubleValue(ParamNames.PIC_BRACE_2_ALIGN_DIST),
                handler.GetDoubleValue(ParamNames.PIC_BRACE_3_ALIGN_DIST),
            };

            for (var i = 0; i < SubDataSort.Count; i++)
            {
                var sub = SubDataSort[i];
                var off = offList[i % 3];
                sub.UpdateNearAlignPlane(off);
                UpdateAlignPoint(sub);
            }

            return true;
        }

        /// <summary>
        /// 处理构件的近远平面
        /// </summary>
        private void PrepareNearFarCentreBuildPlane()
        {
            var thickness = m_pub.GussetPlatePro.Thickness;
            NearBuildPlane = RefSubBuildPlane.Move(thickness/2);
            FarBuildPlane = RefSubBuildPlane.Move(-thickness/2);
            FarBuildPlane = new Plane(FarBuildPlane.Origin,-NearBuildPlane.Normal);
            CentreBuildPlane = PlaneHelper.GetMidPlane(NearBuildPlane, FarBuildPlane);   
        }
        /// <summary>
        /// 准备螺栓相关数据
        /// </summary>
        private void PrepareBoltsData()
        {
            var subs = SubData;
            for (int i = 0; i < subs.Count; ++i)
            {
                var index = i % 3;
                m_braceBolts.PrepareBoltsData(subs[i],index);
            }
        }

        #region 凸包相关 
        /// <summary>
        /// 处理相邻支撑之间的边界线
        /// </summary>
        private void PrepareBraceBoundary()
        {
            for (var i = 0; i < SubDataSort.Count - 1; ++i)
            {
                var cur = SubDataSort[i];
                var next = SubDataSort[i + 1];
                var curBoundary = GetCurBorderPlane(cur);
                var nextBoundary = GetNextBorderPlane(next);
                //处理平行
                if(curBoundary.Normal.IsParallelTo(nextBoundary.Normal))
                    continue;
                //边界的交点，不会改变
                var boundaryP = RefSubBuildPlane.Intersect(curBoundary, nextBoundary);
                var nextDist = GetConvexPlane(next).SignedDist(boundaryP);
                //边界交点在对齐面的左侧，需要移动
                if (nextDist > 0) {
                   //UpdateFarAlignPlane(next,boundaryP); 
                   next.UpdateFarAlignPlane(boundaryP);
                   UpdateAlignPoint(next);
                }
                var curDist = GetConvexPlane(cur).SignedDist(boundaryP);
                //边界交点在对齐面的左侧，需要移动
                if (curDist > 0) {
                   //UpdateFarAlignPlane(cur,boundaryP); 
                   cur.UpdateFarAlignPlane(boundaryP);
                   UpdateAlignPoint(cur);
                }
            }
        }
        /// <summary>
        ///处理凸包 
        /// </summary>
        private void PrepareConvexHull()
        {
            //按照排序的subData检查相邻的支撑的对齐面是不是交于支撑之间。
            for (int i = 0; i < SubDataSort.Count - 1; ++i)
            {
                var cur = SubDataSort[i];
                var next = SubDataSort[i + 1];
                var ret = CheckIntersection(cur,next);
                //交点在上边界外
                //可以往后移动next支撑，不需要回溯
                if (ret < 0) {
                    //cur的对齐面，cur的边界的交点
                    var adjustP = RefSubBuildPlane.Intersect(GetConvexPlane(cur), GetCurBorderPlane(cur));
                    //UpdateFarAlignPlane(next,adjustP);
                    next.UpdateFarAlignPlane(adjustP);
                    UpdateAlignPoint(next);
                }
                //此时调整过后的交点一定在上边界内，
                //二次检查有没有在下边界外
                ret = CheckIntersection(cur,next);
                //交点在下边界外
                //需要往后移动cur支撑，需要回溯
                if (ret > 0)
                {
                    //next的对齐面，next的边界的交点
                    var adjustP = RefSubBuildPlane.Intersect(GetConvexPlane(next), GetNextBorderPlane(next));
                    //UpdateFarAlignPlane(cur,adjustP);
                    cur.UpdateFarAlignPlane(adjustP);
                    UpdateAlignPoint(cur);
                    int k = i-1;//回到上一组
                    //反向循环处理，一直到某一组支撑不需要调整cur支撑
                    while (k >= 0)
                    {
                        cur = SubDataSort[k];
                        next = SubDataSort[k + 1];
                        ret = CheckIntersection(cur, next);
                        if (ret > 0)
                        {
                            //next的对齐面，next的边界的交点
                            adjustP = RefSubBuildPlane.Intersect(GetConvexPlane(next), GetNextBorderPlane(next));
                            //UpdateFarAlignPlane(cur,adjustP);
                            cur.UpdateFarAlignPlane(adjustP);
                            UpdateAlignPoint(cur);
                            --k;
                            continue;
                        }
                        break;
                    }
                }
            }
            
        }

#if false
        private void UpdateFarAlignPlane(SubData subData,Vec3 adjustP)
        {
            subData.FarAlignPlane = new Plane(adjustP, subData.FarAlignPlane.Normal);
            subData.NearAlignPlane = subData.FarAlignPlane.Move(-subData.BoltAreaLength);
        }
        private void UpdateNearAlignPlane(SubData subData,Vec3 adjustP)
        {
            subData.NearAlignPlane = new Plane(adjustP, subData.NearAlignPlane.Normal);
            subData.FarAlignPlane = subData.NearAlignPlane.Move(subData.BoltAreaLength);
        }
#endif

        /// <summary>
        /// 检查cur,next的交点位置。
        /// </summary>
        /// <param name="cur"></param>
        /// <param name="next"></param>
        /// <returns>小于0说明到cur边界的距离小于0，大于0说明到next边界距离小于0</returns>
        private int CheckIntersection(SubData cur, SubData next)
        {
            return CheckIntersection(GetConvexPlane(cur), GetConvexPlane(next), GetCurBorderPlane(cur),
                GetNextBorderPlane(next));
        }

        /// <summary>
        /// 检查交点位于那一侧
        /// </summary>
        /// <param name="curRefPl"></param>
        /// <param name="nextRefPl"></param>
        /// <param name="curBorderPl"></param>
        /// <param name="nextBorderPl"></param>
        /// <returns>小于0说明到cur边界的距离小于0，大于0说明到next边界距离小于0</returns>
        private int CheckIntersection(Plane curRefPl, Plane nextRefPl, Plane curBorderPl, Plane nextBorderPl) {
            //得到此时的交点
            var refP = RefSubBuildPlane.Intersect(curRefPl, nextRefPl);
            //与cur边界的距离
            var curDist = curBorderPl.SignedDist(refP);
            //与next边界的距离
            var nextDist = nextBorderPl.SignedDist(refP);
            if (curDist < 0) {
                return -1;
            }
            if (nextDist < 0) {
                return 1;
            }
            return 0;
        }

        /// <summary>
        /// 获取边界
        /// </summary>
        /// <param name="next"></param>
        /// <returns></returns>
        private Plane GetNextBorderPlane(SubData next) {
            return new Plane(next.TopPlane).Move(next.TopFarAlignPlaneOffset);
        }
        /// <summary>
        /// 获取边界
        /// </summary>
        /// <param name="cur"></param>
        /// <returns></returns>
        private Plane GetCurBorderPlane(SubData cur) {
            return new Plane(cur.BottomPlane).Move(cur.BottomFarAlignPlaneOffset);
        }
        /// <summary>
        /// 获取凸面
        /// </summary>
        /// <param name="subData"></param>
        /// <returns></returns>
        private Plane GetConvexPlane(SubData subData)
        {
            return new Plane(subData.FarAlignPlane);
        }

        #endregion

        /// <summary>
        /// 处理螺栓区域的长度
        /// </summary>
        private void PrepareSubDataWithBraceBolts() {
            for (int i = 0; i < SubData.Count; ++i) {
                var index = i % 3;
                m_braceBolts.SetUpAreaLength(SubData[i], index);
            }
        }

        /// <summary>
        /// 处理支撑之间的最小距离
        /// </summary>
        private void PrepareMinDistanceBetweenBrace()
        {
            var min = AllParamData.MinDistanceBetweenBrace;
            for (var i = 0; i < SubDataSort.Count - 1; ++i) {
                var cur = SubDataSort[i];
                var next = SubDataSort[i + 1];
                var nextDist = cur.BottomPlane.SignedDist(next.TopNearAlignPoint);
                //NOTE 如果相邻两个支撑之间是平行的，则不做处理
                if (!cur.BottomPlane.Normal.IsParallelTo(next.TopPlane.Normal) 
                    && nextDist < min)
                {
                    var distRefPlane = cur.BottomPlane.Move(min); //距离参考面
                    var refPoint = RefSubBuildPlane.Intersect(distRefPlane, next.TopPlane);
                    next.UpdateNearAlignPlane(refPoint);
                    //next.NearAlignPlane = new Plane(refPoint, next.NearAlignPlane.Normal);
                    //next.FarAlignPlane = next.NearAlignPlane.Move(next.BoltAreaLength);
                    UpdateAlignPoint(next); //更新对应的对齐点
                }

                var curDist = next.TopPlane.SignedDist(cur.BottomNearAlignPoint);
                if (!next.TopPlane.Normal.IsParallelTo(cur.BottomPlane.Normal) && curDist < min)
                {
                    var refPlane = next.TopPlane.Move(min);
                    var refPoint = RefSubBuildPlane.Intersect(refPlane, cur.BottomPlane);
                    next.UpdateNearAlignPlane(refPoint);
                    //cur.NearAlignPlane = new Plane(refPoint, cur.NearAlignPlane.Normal);
                    //cur.FarAlignPlane = cur.NearAlignPlane.Move(cur.BoltAreaLength);
                    UpdateAlignPoint(cur);
                }
            }
        }

        /// <summary>
        /// 准备节点板于支撑侧对齐面的折角偏移
        /// </summary>
        private void PrepareAlignPlaneOffset() {
            for (var i = 1; i < SubDataSort.Count - 1; i++) {
                var sub = SubDataSort[i];
                sub.TopFarAlignPlaneOffset = AllParamData.LengthEdgeOfGussetTopIn;
                sub.BottomFarAlignPlaneOffset = AllParamData.LengthEdgeOfGussetBottomIn;
            }
            var first = SubDataSort.First();
            first.TopFarAlignPlaneOffset = AllParamData.LengthEdgeOfGussetTopOut;
            first.BottomFarAlignPlaneOffset = AllParamData.LengthEdgeOfGussetTopIn;
            var last = SubDataSort.Last();
            last.TopFarAlignPlaneOffset = AllParamData.LengthEdgeOfGussetBottomIn;
            last.BottomFarAlignPlaneOffset = AllParamData.LengthEdgeOfGussetBottomOut;
            //对面进行偏移
            foreach (var sub in SubDataSort) {
                //sub.NearAlignPlane = CalculateAlignPlane(sub.NearPlane, sub.TopPlane, sub.BottomPlane);
                var adjustPl = CalculateAlignPlane(sub.NearPlane, sub.TopPlane, sub.BottomPlane);
                sub.UpdateNearAlignPlane(adjustPl);
            }
        }

        /// <summary>
        /// 处理支撑对齐点
        /// </summary>
        private void PrepareAlignPoint()
        {
            // NOTE 处理支撑对齐点
            foreach (var sub in SubDataSort) {
                UpdateAlignPoint(sub);
            }
        }

        private void UpdateAlignPoint(SubData sub)
        {
            sub.TopNearAlignPoint = RefSubBuildPlane.Intersect(sub.TopPlane, sub.NearAlignPlane);
            sub.BottomNearAlignPoint = RefSubBuildPlane.Intersect(sub.BottomPlane, sub.NearAlignPlane);
            sub.TopFarAlignPoint = RefSubBuildPlane.Intersect(sub.TopPlane, sub.FarAlignPlane);
            sub.BottomFarAlignPoint = RefSubBuildPlane.Intersect(sub.BottomPlane, sub.FarAlignPlane);
        }

        /// <summary>
        /// 准备排序好的支撑数组
        /// </summary>
        /// <returns></returns>
        private bool InitSubDataSort() {
            // NOTE 沿着mainLine方向，给支撑数组排序
            SubDataSort.AddRange(SubData);
            SubDataSort.Sort ((sub1, sub2) =>
            {
                var p1 = sub1.NearPlane.Normal.Normalize();
                var p2 = sub2.NearPlane.Normal.Normalize();
                var dot1 = p1.Dot(AxisY);
                var dot2 = p2.Dot(AxisY);
                return dot1.CompareTo(dot2);
            });
#if false
            // NOTE 排序好的支撑可能存在，倾斜角度正确但是相对高度越界的情况
            // 不考虑做处理

            for (int i = 0; i < SubDataSort.Count-1; i++)
            {
                var cur = SubDataSort[i];
                var next = SubDataSort[i + 1];
                var curFarPoint = cur.FarPoint;

                var refPlane = new Plane(curFarPoint, AxisX);//以当前点，建立和Y轴平行的面
                //由于所有支撑不允许平行于Y，所有一定有交点
                var subNextPoint = GussetPlateBuildPlane.Intersect(refPlane,next.TopPlane);
                
                if (PointHelper.ComparePointsByProjection(curFarPoint, subNextPoint, AxisY) > 0)
                { continue; }
                var tmp = SubDataSort[i];
                SubDataSort[i] = SubDataSort[i + 1];
                SubDataSort[i + 1] = tmp;
                --i;
            }
#endif
            
            return true;
        }

        /// <summary>
        /// 计算对齐面，不考虑支撑间的距离
        /// </summary>
        /// <param name="subNearPlane"></param>
        /// <param name="subTopPlane"></param>
        /// <param name="subBottomPlane"></param>
        private Plane CalculateAlignPlane(Plane subNearPlane, Plane subTopPlane, Plane subBottomPlane)
        {
            // NOTE 处理主零件和节点板的距离偏移
            var refPlane = FaceToSubPlane.Move(AllParamData.DistanceBetweenMainPartAndBrace);

            //支撑不允许平行于主零件
            var pointTop = RefSubBuildPlane.Intersect(subTopPlane, refPlane);
            var pointBottom= RefSubBuildPlane.Intersect(subBottomPlane, refPlane);

            var distTop = subNearPlane.SignedDist(pointTop);
            var distBottom = subNearPlane.SignedDist(pointBottom);

            var offset = Math.Min(distTop, distBottom);
            var alignPlane = subNearPlane.Move(offset);
            alignPlane.InvertThis();
            
            return alignPlane;
        }

        /// <summary>
        /// 获取面板中的参数
        /// </summary>
        /// <returns></returns>
        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.OffSetForFaceBuildPlane = handler.GetDoubleValue
                (ParamNames.PIC_GAP_BETWEEN_GUSSET_PLATE_AND_MAIN_PART);
            AllParamData.DistanceBetweenMainPartAndBrace = handler.GetDoubleValue
                (ParamNames.PIC_DISTANCE_BETWEEN_MAIN_PART_AND_BRACE);
            AllParamData.MinDistanceBetweenBrace = handler.GetDoubleValue
                (ParamNames.PIC_MIN_DISTANCE_BETWEEN_THE_BRACES);
            if (SubData.Count <= 1)//只有一个支撑的时候不存在支撑间间距的概念
                AllParamData.MinDistanceBetweenBrace = 0.0d;
            # region 处理节点板折角距离
            {
                var length = handler.GetDoubleValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_TOP_INSIDE);
                AdaptiveLengthForEdgeOfGussetPlate(ref length);
                AllParamData.LengthEdgeOfGussetTopIn = length;
                length = handler.GetDoubleValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_TOP_OUTSIDE);
                AdaptiveLengthForEdgeOfGussetPlate(ref length);
                AllParamData.LengthEdgeOfGussetTopOut = length;
                length = handler.GetDoubleValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_BOTTOM_INSIDE);
                AdaptiveLengthForEdgeOfGussetPlate(ref length);
                AllParamData.LengthEdgeOfGussetBottomIn = length;
                length = handler.GetDoubleValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_BOTTOM_OUTSIDE);
                AdaptiveLengthForEdgeOfGussetPlate(ref length);
                AllParamData.LengthEdgeOfGussetBottomOut = length;
            }
            void AdaptiveLengthForEdgeOfGussetPlate(ref double edgeLength) {
                if (Equals(edgeLength, double.MaxValue))
                    edgeLength = AllParamData.MinDistanceBetweenBrace / 2;
            }
            #endregion

            #region 处理节点板拐角角度
            {
                AllParamData.GussetCornerAngleTop = handler.GetDoubleValue
                    (ParamNames.PIC_THE_GUSSET_PLATE_CORNER_ANGLE_TOP);
                AllParamData.GussetCornerAngleBottom= handler.GetDoubleValue
                    (ParamNames.PIC_THE_GUSSET_PLATE_CORNER_ANGLE_BOTTOM);
            }
            #endregion
            AllParamData.PosOnBraceSelect = handler.GetIntValue(ParamNames.GUSSET_GUSSET_PLATE_POSITION_ON_BRACE_SELECT);
            AllParamData.GussetPlateZOffset = handler.GetDoubleValue(ParamNames.GUSSET_GUSSET_PLATE_Z_DIRECTION_OFFSET);
            return true;
        }

        /// <summary>
        /// 返回在FacePlane上的构建点,板形状1
        /// </summary>
        /// <returns></returns>
        internal List<Vec3> GetBuildPointsInFacePlaneForShape0()
        {
            var points = new List<Vec3>();
            //var topPt = RefSubBuildPlane.Intersect(offsetFaceToSubPlane, retPl);
            var offsetFaceToSubPlane = FaceBuildPlane.Move(AllParamData.OffSetForFaceBuildPlane);
            
            var first = SubDataSort.First();
            var firstPt = PointHelper.Move(first.TopFarAlignPoint, first.TopPlane.Normal, first.TopFarAlignPlaneOffset);
            var angleTop = AllParamData.GussetCornerAngleTop;
            var topAnglePl = HandlerGussetCornerAngle(first,firstPt,-angleTop);
            var topAnglePt = RefSubBuildPlane.Intersect(offsetFaceToSubPlane,topAnglePl);

            var last = SubDataSort.Last();
            var lastPt = PointHelper.Move(last.BottomFarAlignPoint, last.BottomPlane.Normal, last.BottomFarAlignPlaneOffset);
            var angleBtm= AllParamData.GussetCornerAngleBottom;
            var btmAnglePl = HandlerGussetCornerAngle(last, lastPt, angleBtm);
            var btmAnglePt = RefSubBuildPlane.Intersect(offsetFaceToSubPlane,btmAnglePl);
            
            //可能存在 nearAlignPoint更低或更高的情况
            var topNearPl = new Plane(first.TopPlane);
            var topNearPt = topNearPl.Intersect(RefSubBuildPlane,offsetFaceToSubPlane);
            var btmNearPl = new Plane(last.BottomPlane);
            var btmNearPt = btmNearPl.Intersect(RefSubBuildPlane,offsetFaceToSubPlane);

            var top = PointHelper.MaxPointsByProjection(topAnglePt, topNearPt, AxisY);
            var btm = PointHelper.MaxPointsByProjection(btmAnglePt,btmNearPt,-AxisY);

            var topPt = top.Item1;
            var topPl = top.Item2 ? new Plane(topAnglePl) : new Plane(topNearPl);
            var btmPt = btm.Item1;
            var btmPl = btm.Item2 ? new Plane(btmAnglePl) : new Plane(btmNearPl);

            var head = m_context.MainPlanes.HeadBound;
            var tail = m_context.MainPlanes.TailBound;
            //检查btm和top是否在头尾。
            HandlerOverHeadTail(ref topPt,ref m_topExtendedPt, head, tail,topPl);
            HandlerOverHeadTail(ref btmPt,ref m_btmExtendedPt, head, tail,btmPl);
            
            //返回：底，顶
            points.Add(btmPt);
            points.Add(topPt);
            
            return points;
        }

        private void HandlerOverHeadTail(ref Vec3 refPt, ref Vec3 extendedPt, Plane head, Plane tail, Plane extendedPl)
        {
            HandlerOverThrough(ref refPt,ref extendedPt,head,extendedPl);
            HandlerOverThrough(ref refPt,ref extendedPt,tail,extendedPl);
        }
        private void HandlerOverThrough(ref Vec3 refPt,ref Vec3 extendedPt,Plane refPl,Plane extendedPl)
        {
            if(refPl == null)return;
            var dist = refPl.SignedDist(refPt);
            if (dist > 0.0d)
            {
                extendedPt = RefSubBuildPlane.Intersect(extendedPl, refPl); 
                PointHelper.MoveToRef(ref refPt,refPl);
            }
        }


        /// <summary>
        /// 处理节点板拐角
        /// </summary>
        /// <param name="sub">顶部或底部支撑</param>
        /// <param name="refPt">顶部或底部偏移后的FarAlignPoint</param>
        /// <param name="angle">旋转的角度，绕Z轴以右手定理</param>
        /// <returns></returns>
        private Plane HandlerGussetCornerAngle(SubData sub,Vec3 refPt,double angle)
        {
            var offsetFaceToSubPlane = FaceBuildPlane.Move(AllParamData.OffSetForFaceBuildPlane);
            var retPl = Plane.NULL;
            if (!Equals(Math.Abs(angle), double.MaxValue)) {
                if (angle == 0.0d) {
                    retPl = new Plane(refPt, sub.TopPlane.Normal);
                }
                else if (angle <= 90.0d && angle >= -90.0d) {
                    var line = new Vec3(sub.TopPlane.Normal);
                    var plNormal = PointHelper.RotateVectorInPlane(line, AxisZ, angle);
                    retPl = new Plane(refPt, plNormal);
                }
            }
            else {
                retPl = new Plane(refPt, AxisY);
            }

            //处理平行的情况
            if (retPl.Normal.IsParallelTo(offsetFaceToSubPlane.Normal))
            {
                retPl = new Plane(refPt, AxisY);
            }
            return retPl;
        }
        /// <summary>
        /// 解析基础数据，无需等待Param的解析
        /// </summary>
        /// <returns></returns>
        private bool InitBaseData()
        {
            if (!InitAxisY()) return false;
            if (!InitSubFarNearPoint()) return false;
            if (!InitSubFarNearPlane()) return false;
            if (!InitFaceToBackToSubPlane()) return false;
            if (!InitAxisX()) return false;
            if (!InitGussetPlateBuild()) return false;
            if (!InitSubTopBtmPlane()) return false;
            if (!InitSubThickness()) return false;
            if (!InitSubDataSort()) return false;
            
            return true;
        }
        /// <summary>
        /// 初始化SubDataList
        /// </summary>
        /// <returns></returns>
        private bool InitSubDataList()
        {
            var subParts = m_context.TargetComp.Subs;
            if (subParts == null) return false;
            m_subNum = subParts.Length;
            if (m_subNum == 0) return false;
            for (var i = 0; i < m_subNum; ++i) {
                if (subParts[i] is Beam subBeam) {
                    var subData = new SubData {
                        Beam = subBeam,
                        BeamPlanes = BeamPlanes.CreateFromBeam(subBeam),
                        Profile = subBeam.Profile,
                    };
                    SubData.Add(subData);
                }
                else {
                    return false;
                }
            }
            if (SubData.Count != m_subNum) return false;
            return true;
        }

        /// <summary>
        /// 处理支撑的厚度值
        /// </summary>
        /// <returns></returns>
        private bool InitSubThickness()
        {
            var refPlane = RefSubBuildPlane;
            var refNormal = refPlane.Normal;
            foreach (var sub in SubData)
            {
                var beamPlanes = sub.BeamPlanes;
                sub.Thickness = PlaneHelper.GetSameNormalFlangeOrWebThickness(beamPlanes, refNormal);
            }
            return true;
        }

        /// <summary>
        /// 准备节点板的构建数据
        /// </summary>
        /// <returns></returns>
        private bool InitGussetPlateBuild()
        {
            var gstTypeList = PlaneHelper.BOUND_PLANE_TBFB;
            var type = PlaneHelper.DotMax(m_subPartsRef.BeamPlanes, gstTypeList, AxisZ);
            RefSubBuildPlane = PlaneHelper.GetBeamPlane(type,m_subPartsRef.BeamPlanes);
            if (RefSubBuildPlane == null) {
                Console.WriteLine(@"RefSubBuildPlane is null");
                return false;
            }
            // NOTE 迫于流程控制，将选择项插入此处
            var dist = 0.0d;
            if (type == EBeamPlaneType.BackBound || type == EBeamPlaneType.FrontBound) {
                dist = PlaneHelper.GetTwoPlaneDist(EBeamPlaneType.FrontBound, EBeamPlaneType.BackBound,m_subPartsRef.BeamPlanes);
            }else if (type == EBeamPlaneType.BottomBound || type == EBeamPlaneType.TopBound) {
                dist = PlaneHelper.GetTwoPlaneDist(EBeamPlaneType.TopBound, EBeamPlaneType.BottomBound, m_subPartsRef.BeamPlanes);
            }

            //拿到的type为与Z轴最同向的面
            InitRefBuildPlanePosOnBrace(dist);
            //处理Z轴的额外偏移
            m_refSubBuildPlane.MoveThis(AllParamData.GussetPlateZOffset);
            
            
            // NOTE 二次处理FaceBuildPlane
            var typeList = new List<EBeamPlaneType>()
            {
                EBeamPlaneType.TopBound, EBeamPlaneType.BottomBound,
                EBeamPlaneType.FrontBound, EBeamPlaneType.BackBound,
            };
            type = PlaneHelper.DotMax(m_context.MainPlanes,
                typeList,AxisZ);
            EBeamPlaneType type2 = PlaneHelper.GetOppositeBoundType(type);
            
            var nearPlane = PlaneHelper.GetBeamPlane(type, m_context.MainPlanes);
            var farPlane  = PlaneHelper.GetBeamPlane(type2, m_context.MainPlanes);
            if (nearPlane == null) {
                Console.WriteLine(@"MainPlanes "+type+ @" is null");
                return false;
            }
            if (farPlane == null) {
                Console.WriteLine(@"MainPlanes "+type2+ @" is null");
                return false;
            }
            
            var rerPoint = RefSubBuildPlane.Intersect(FaceToSubPlane,m_subPartsRef.TopPlane);
            dist = nearPlane.SignedDist(rerPoint == Vec3.MAX ? //nearPlane和GussetPlateBuildPlane平行
                RefSubBuildPlane.Origin : rerPoint);
            var dist2 = farPlane.SignedDist(rerPoint == Vec3.MAX ? //nearPlane和GussetPlateBuildPlane平行
                RefSubBuildPlane.Origin : rerPoint);
            if (dist >= 0 || dist2 >= 0)
            {
                m_bExtensionMode = true;
                if (!PlaneHelper.CheckTypeList(m_context.MainPlanes, typeList)) {
                    return false;
                }
                FaceBuildPlane = PlaneHelper.GetSameNormalPlane(m_context.MainPlanes,typeList,BackToSubPlane.Normal);
                FaceBuildPlane.InvertThis();
                FaceToSubPlane = PlaneHelper.GetSameNormalPlane(m_context.MainPlanes, typeList, FaceToSubPlane.Normal);
                BackToSubPlane = PlaneHelper.GetSameNormalPlane(m_context.MainPlanes, typeList, BackToSubPlane.Normal);
            }
            else
            {
                FaceBuildPlane = FaceToSubPlane;
                m_bExtensionMode = false;
            }
            return true;
        }

        private void InitRefBuildPlanePosOnBrace(double dist)
        {
            var select = AllParamData.PosOnBraceSelect;
            var thickness = m_pub.GussetPlatePro.Thickness;
            if (select == 1) {//居中布置
                m_refSubBuildPlane.MoveThis(-dist/2);
            }else if (select == 3) {//远离Z轴布置
                m_refSubBuildPlane.MoveThis(-dist);
                m_refSubBuildPlane.MoveThis(-thickness / 2);
            }else if (select == 2) {//默认为靠近Z轴布置
                m_refSubBuildPlane.MoveThis(thickness / 2);
            }else if (select == 0)
            {
                m_refSubBuildPlane.MoveThis(thickness / 2);
                //TODO 不同截面类型的自适应
            }
        }


        /// <summary>
        /// 准备子零件的头尾的近远面
        /// </summary>
        /// <returns></returns>
        private bool InitSubFarNearPlane()
        {
            foreach (var sub in SubData) {
                var dist1 = sub.BeamPlanes.HeadBound.Dist(sub.NearPoint);
                var dist2 = sub.BeamPlanes.TailBound.Dist(sub.NearPoint);
                sub.NearPlane = new Plane(sub.BeamPlanes.HeadBound);
                sub.FarPlane = new Plane(sub.BeamPlanes.TailBound);
                if (dist1 > dist2) {
                    sub.NearPlane = new Plane(sub.BeamPlanes.TailBound);
                    sub.FarPlane = new Plane(sub.BeamPlanes.HeadBound);
                }
            }
            return true;
        }

        private bool InitAxisX() {
            AxisX = FaceBuildPlane.Normal.Normalize();
            if (!InitAxisZ()) return false;
            return true;
        }

        private bool InitAxisY()
        {
            var first = SubData.First();
            var last = SubData.Last();

            var pA = first.FarPoint;
            var pB = last.FarPoint;
            var lineAb = pB - pA;
            var dotN = MainLine.Dot(lineAb);
            if (m_subNum == 1) {
                var line = SubData.First().NearToFarLine;
                dotN = MainLine.Dot(line);
            }
            
            if (dotN >= 0) {
                AxisY = MainLine;
            }else if (dotN < 0) {
                AxisY = -MainLine;
            }
            return true;
        }

        private bool InitAxisZ() {
            AxisZ = AxisX.Cross(AxisY).Normalize();
            return true;
        }

        /// <summary>
        /// 主备主零件的参照面
        /// </summary>
        /// <returns></returns>
        private bool InitFaceToBackToSubPlane()
        {
            var list = new List<EBeamPlaneType>() {
                EBeamPlaneType.FrontWeb, EBeamPlaneType.BackWeb,
                EBeamPlaneType.TopFlange, EBeamPlaneType.BottomFlange,
            };
            //参考的支撑
            m_subPartsRef= SubData.First();
            //主零件的mainPlanes
            var mainPlanes = m_context.MainPlanes;
            var line = m_subPartsRef.FarPoint - m_subPartsRef.NearPoint;
            var type = PlaneHelper.DotMax(mainPlanes, list, line);
            Plane face = null;
            Plane back = null;
            if (type == EBeamPlaneType.FrontWeb) {
                face = mainPlanes.FrontWeb;
                back = mainPlanes.BackWeb;
            }else if (type == EBeamPlaneType.BackWeb) {
                face = mainPlanes.BackWeb;
                back = mainPlanes.FrontWeb;
            }else if (type == EBeamPlaneType.TopFlange) {
                face = mainPlanes.TopFlange;
                back = mainPlanes.BottomFlange;
            } else {
                face = mainPlanes.BottomFlange;
                back = mainPlanes.TopFlange;
            }
            if (face == null) return false;
            if (back== null) return false;
            FaceToSubPlane = face;
            BackToSubPlane = back;
            FaceBuildPlane = FaceToSubPlane;

            
            return true;
        }


        /// <summary>
        /// 准备主零件的起点到终点的线
        /// </summary>
        /// <returns></returns>
        private bool InitMain()
        {
            m_mainPart = m_context.MainPart;
            var line = m_mainPart.CtrlPoints?.Last()?.WorldPos -
                       m_mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue) return false;
            MainLine = line.Value.Normalize();
            return true;
        }

        /// <summary>
        /// 准备支撑的面数据
        /// </summary>
        /// <returns></returns>
        private bool InitSubTopBtmPlane()
        {
            var list = new List<EBeamPlaneType>()
            {
                EBeamPlaneType.TopBound, EBeamPlaneType.BottomBound,
                EBeamPlaneType.FrontBound, EBeamPlaneType.BackBound,
            };
            foreach (var sub in SubData)
            {
                //Log.Instance.LogInfo("rotationSide:" + rotationSide);
                var type = PlaneHelper.DotMax(sub.BeamPlanes, list,AxisY);
                sub.TopPlane = SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.FrontBound,
                    EBeamPlaneType.BackBound, true,AxisY);
                sub.BottomPlane= SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.FrontBound,
                    EBeamPlaneType.BackBound, false,AxisY);
                if (type == EBeamPlaneType.TopBound || type == EBeamPlaneType.BottomBound)
                {
                    sub.TopPlane = SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.TopBound,
                        EBeamPlaneType.BottomBound, true,AxisY);
                    sub.BottomPlane= SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.TopBound,
                        EBeamPlaneType.BottomBound, false,AxisY);
                }
                sub.TopBottomDist = Math.Abs(sub.TopPlane.Dist(sub.BottomPlane.Origin));
                
                //获取最和Z轴同向的面
                //type = PlaneHelper.DotMax(sub.BeamPlanes, list, AxisZ);
                sub.FrontPlane = SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.TopBound,
                    EBeamPlaneType.BottomBound, true,AxisZ);
                sub.BackPlane= SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.TopBound,
                    EBeamPlaneType.BottomBound, false,AxisZ);
                if (type == EBeamPlaneType.TopBound || type == EBeamPlaneType.BottomBound)
                {
                    sub.FrontPlane = SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.FrontBound,
                        EBeamPlaneType.BackBound, true,AxisZ);
                    sub.BackPlane= SelectPlaneBetween(sub.BeamPlanes, EBeamPlaneType.FrontBound,
                        EBeamPlaneType.BackBound, false,AxisZ);
                }
                if (sub.FrontPlane == null || sub.BackPlane == null) { return false;}
                sub.FrontBackDist = Math.Abs(sub.FrontPlane.Dist(sub.BackPlane.Origin));

            }
            return true;
        }


        /// <summary>
        /// 从type1 和 type2中选择和MainLine点乘结果为正数的面,
        /// 点乘结果为0时返回null
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <param name="type1"></param>
        /// <param name="type2"></param>
        /// <param name="than0"></param>
        /// <param name="refDir"></param>
        /// <returns></returns>
        private Plane SelectPlaneBetween(BeamPlanes beamPlanes,EBeamPlaneType type1,EBeamPlaneType type2,bool than0,Vec3 refDir)
        {
            var plane1= PlaneHelper.GetBeamPlane(type1,beamPlanes);
            var plane2= PlaneHelper.GetBeamPlane(type2,beamPlanes);
            if (plane1 == null || plane2 == null)
                return null;
            var dot = refDir.Dot(plane1.Normal.Normalize());
            if (dot == 0)
                return null;
            // 如果than0为真且点积大于0，或者than0为假且点积小于0，返回plane1
            if ((than0 && dot > 0) || (!than0 && dot < 0))
                return new Plane(plane1);
            // 否则返回plane2
            return new Plane(plane2);
        }
        
        

        /// <summary>
        /// 解析子零件的控制点
        /// </summary>
        /// <returns></returns>
        private bool InitSubFarNearPoint()
        {

            var linePoint = m_mainPart.CtrlPoints.First().WorldPos;
            var lineDirection = m_mainPart.CtrlPoints.Last().WorldPos -
                                m_mainPart.CtrlPoints.First().WorldPos;
            foreach (var subData in SubData)
            {
                var firstCtrlPoint = subData.Beam?.CtrlPoints?.First()?.WorldPos;
                var lastCtrlPoint = subData.Beam?.CtrlPoints?.Last()?.WorldPos;
                if (!firstCtrlPoint.HasValue || !lastCtrlPoint.HasValue) 
                    return false;
                var dist1 = PointHelper.PointToLineDistance(firstCtrlPoint.Value, linePoint, lineDirection);
                var dist2 = PointHelper.PointToLineDistance(lastCtrlPoint.Value, linePoint, lineDirection);
                subData.NearPoint = firstCtrlPoint.Value;
                subData.FarPoint = lastCtrlPoint.Value;
                if (dist1 > dist2) {
                    subData.NearPoint = lastCtrlPoint.Value;
                    subData.FarPoint = firstCtrlPoint.Value;
                }

                //近指向远
                var line = subData.FarPoint - subData.NearPoint;
                if (line.IsParallelTo(AxisY))
                    return false;
                subData.NearToFarLine = line;
            }
            
            return true;
        }

    }
}