﻿using System;
using System.Collections.Generic;
using LoongCAD.Geometry;

namespace BoltedGusset_11_.Tool
{
    public static class PointHelper
    {
        /// <summary>
        /// 让点朝着某个方向移动一定距离
        /// </summary>
        /// <param name="point">要移动的点</param>
        /// <param name="direction">移动的方向</param>
        /// <param name="length">移动的距离</param>
        /// <returns>移动后点的位置</returns>
        public static Vec3 Move(Vec3 point, Vec3 direction, double length)
        {
            var dir =direction.Normalize();
            var movement = dir * length;
            return new Vec3(point + movement);
        }

        /// <summary>
        /// 让点朝着某个方向移动一定距离
        /// </summary>
        /// <param name="point">要移动的点，传入初始位置，传出移动后的位置</param>
        /// <param name="direction">移动的方向</param>
        /// <param name="length">移动的距离</param>
        public static void MoveRef(ref Vec3 point, Vec3 direction, double length)
        {
            var dir = direction.Normalize();
            var movement = dir * length;
            point = point + movement;
        }

        /// <summary>
        /// 把一个点移动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="targetPlane"></param>
        public static Vec3 MoveTo(Vec3 point, Plane targetPlane)
        {
            var dist = targetPlane.SignedDist(point);
            return Move(point, targetPlane.Normal, -dist);
        }

        /// <summary>
        /// 依次将点挪动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="planes"></param>
        /// <returns></returns>
        public static Vec3 MoveTo(Vec3 point, List<Plane> planes)
        {
            foreach (var plane in planes)
            {
                point = MoveTo(point, plane);
            }
            return point;
        }

        /// <summary>
        /// 把一个点移动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="targetPlane"></param>
        public static void MoveToRef(ref Vec3 point, Plane targetPlane)
        {
            var dist = targetPlane.SignedDist(point);
            MoveRef(ref point, targetPlane.Normal, -dist);
        }
        
        /// <summary>
        /// 依次将点挪动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="planes"></param>
        /// <returns></returns>
        public static void MoveToRef(ref Vec3 point, List<Plane> planes)
        {
            foreach (var plane in planes)
            {
                MoveToRef(ref point, plane);
            }
        }
        /// <summary>
        /// 在给定平面内旋转一个三维向量指定的角度。遵循右手定理
        /// </summary>
        /// <param name="vector">要旋转的向量。</param>
        /// <param name="planeNormal">定义旋转平面的法线向量。</param>
        /// <param name="angleDegrees">向量在平面内旋转的角度，单位为度。</param>
        /// <returns>旋转后的向量。</returns>
        public static Vec3 RotateVectorInPlane(Vec3 vector, Vec3 planeNormal, double angleDegrees)
        {
            // 将平面法线标准化，确保它是单位向量
            Vec3 normalizedPlaneNormal = planeNormal.Normalize();

            // 将角度从度转换为弧度
            double angleRadians = Math.PI / 180 * angleDegrees;

            // 根据罗德里格斯旋转公式计算旋转的各个分量
            Vec3 rotatedVector = vector * Math.Cos(angleRadians) // 向量与余弦角度的乘积
                                    + normalizedPlaneNormal.Cross(vector) * Math.Sin(angleRadians) // 法线与向量的叉乘与正弦角度的乘积
                                    + normalizedPlaneNormal * normalizedPlaneNormal.Dot(vector) * (1 - Math.Cos(angleRadians)); // 法线与法线与向量点乘的乘积，再乘以1减去余弦角度
            return rotatedVector; // 返回旋转后的向量
        }

        /// <summary>
        /// 返回a,b点的中点
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static Vec3 Midpoint(Vec3 a, Vec3 b)
        {
            var x = (a.X + b.X) / 2;
            var y = (a.Y + b.Y) / 2;
            var z = (a.Z + b.Z) / 2;
            return new Vec3(x, y, z);
        }

        /// <summary>
        /// 计算两个点之间的距离，以给定方向为基准,即两点之间的距离在这个方向上的投影
        /// </summary>
        /// <param name="pointA"></param>
        /// <param name="pointB"></param>
        /// <param name="direction">给定方向</param>
        /// <returns></returns>
        public static double DistInDir(Vec3 pointA, Vec3 pointB, Vec3 direction)
        {
            Vec3 ab = new Vec3(pointB.X - pointA.X, pointB.Y - pointA.Y, pointB.Z - pointA.Z);
            Vec3 directionNormalized = direction.Normalize();
            return ab.Dot(directionNormalized);
        }

        /// <summary>
        /// 计算一个点到一条线的垂直距离
        /// </summary>
        /// <param name="point">需要计算的点</param>
        /// <param name="linePoint">线上的一点</param>
        /// <param name="lineDirection">线的方向</param>
        /// <returns></returns>
        public static double PointToLineDistance(Vec3 point,Vec3 linePoint,Vec3 lineDirection)
        {
            //线上的点指向传入点
            Vec3 pq = point - linePoint;
            Vec3 n = pq.Cross(lineDirection.Normalize());
            return n.Length;
        }
        
        /// <summary>
        /// 比较两个点在特定方向上的投影
        /// </summary>
        /// <param name="p1"></param>
        /// <param name="p2"></param>
        /// <param name="direction"></param>
        /// <returns>大于0表示p1比p2在这个方向上更远，等于为相同，小于0则表示p1比p2在这个方向更近</returns>
        internal static int ComparePointsByProjection(Vec3 p1, Vec3 p2, Vec3 direction)
        {
            double projection1 = p1.Dot(direction);
            double projection2 = p2.Dot(direction);
            return projection1.CompareTo(projection2);
        }

        /// <summary>
        /// 如果p1比p2在dir方向远则返回p1,否则返回p2
        /// </summary>
        /// <param name="p1"></param>
        /// <param name="p2"></param>
        /// <param name="dir"></param>
        /// <returns></returns>
        internal static Tuple<Vec3,bool> MaxPointsByProjection(Vec3 p1, Vec3 p2, Vec3 dir)
        {
            if (ComparePointsByProjection(p1, p2, dir) > 0)
            {
                return new Tuple<Vec3, bool>(p1,true);
            }
            else
            {
                return new Tuple<Vec3, bool>(p2,false);
            }
        }
    }
}