﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace ColdRolledOverlap1
{
    class ChamferControl
    {
        public CtrlPoint[] ctrlPointList; //实际需要进行折角的点
        public Vec3[] comparePointList; //折角点的位置
        public int[] placeMap; //输出chamferCtrlPointList中的点对应哪一个cut_type，这个列表存着cut_typeList的对应下标
        //public DetailComponent cmp;
        public bool isSameOrient = true; //控制点方向与折角点位置方向是否一致

        /// <summary>
        /// 设置折角点位置（通过面）
        /// </summary>
        /// <param name="sp"></param>
        /// <param name="nearPlaneList"></param>
        public ChamferControl(SteelPlate sp, Plane[] nearPlaneList)
        {
            this.ctrlPointList = sp.CtrlPoints;
            Plane ctrlPlane = GetPlane(sp.CtrlPoints);
            List<Vec3> comparePointList = new List<Vec3>(); //比较控制点与这些点的距离，确定正确的对应关系
            comparePointList.Add(ctrlPlane.Intersect(nearPlaneList[nearPlaneList.Length - 1], nearPlaneList[0])); //先确定第一个点
            for (int i = 1; i < nearPlaneList.Length; i++)
            {
                Vec3 point = ctrlPlane.Intersect(nearPlaneList[i - 1], nearPlaneList[i]);
                comparePointList.Add(point);
            }
            this.comparePointList = comparePointList.ToArray();
            FindCtrlPointPlace();
        }

        /// <summary>
        /// 设置折角点位置（通过点）
        /// </summary>
        /// <param name="sp"></param>
        /// <param name="comparePointList"></param>
        public ChamferControl(SteelPlate sp, Vec3[] comparePointList)
        {
            this.ctrlPointList = sp.CtrlPoints;
            this.comparePointList = comparePointList;
            FindCtrlPointPlace();
        }

        //寻找控制点位置
        private void FindCtrlPointPlace()
        {
            placeMap = new int[comparePointList.Length];
            int[] mark = new int[comparePointList.Length];
            for (int i = 0; i < comparePointList.Length; i++)
            {
                Vec3 comparePoint = comparePointList[i];
                double distance = double.MaxValue;
                int pos = 0;
                for (int j = 0; j < ctrlPointList.Length; j++) //遍历得到最接近comparePoint的控制点
                {
                    if (mark[j] == 1)
                        continue;
                    double nowDistance = ctrlPointList[j].WorldPos.Dist(comparePoint);
                    if (nowDistance < distance)
                    {
                        distance = nowDistance;
                        pos = j;
                    }
                }
                placeMap[pos] = i;
                mark[pos] = 1; //对已经记录位置的控制点做标记
            }
            Plane p1 = GetPlane(this.comparePointList);
            Plane p2 = GetPlane(this.ctrlPointList);
            isSameOrient = p1.Normal.Dot(p2.Normal) > 0;
        }

        //通过点列表（相对位置）获取创建面
        private static Plane GetPlane(Vec3[] ctrlPointList)
        {
            if (ctrlPointList.Length < 3)
                return null;
            Vec3 v0 = ctrlPointList[0];
            Vec3 v1 = ctrlPointList[1];
            Vec3 v2 = ctrlPointList[2];
            Plane p = new Plane(v0, v1 - v0, v2 - v0);
            return p;
        }
        private static Plane GetPlane(CtrlPoint[] ctrlPointList)
        {
            if (ctrlPointList.Length < 3)
                return null;
            Vec3 v0 = ctrlPointList[0].WorldPos;
            Vec3 v1 = ctrlPointList[1].WorldPos;
            Vec3 v2 = ctrlPointList[2].WorldPos;
            Plane p = new Plane(v0, v1 - v0, v2 - v0);
            return p;
        }

        /// <summary>
        /// 设置折角参数
        /// </summary>
        /// <param name="cut_xList"></param>
        /// <param name="cut_yList"></param>
        /// <param name="cut_typeList">折角类型，需对应折角点位置的输入</param>
        public void ChamferAllPoints(List<double> cut_xList, List<double> cut_yList, List<Define.CornerCutType> cut_typeList)
        {
            if (ctrlPointList.Length < 3)
                return;
            //让输入的XY顺序与实际一致
            if(!isSameOrient)
            {
                List<double> temp = cut_xList;
                cut_xList = cut_yList;
                cut_yList = temp;
            }
            for (int i = 0; i < ctrlPointList.Length; i++)
            {
                int index = placeMap[i];
                ChamferCtrlPoint chamferCtrlPoint = ctrlPointList[i] as ChamferCtrlPoint;
                chamferCtrlPoint.CutType = cut_typeList[index];
                chamferCtrlPoint.CutX = cut_xList[index];
                chamferCtrlPoint.CutY = cut_yList[index];
                chamferCtrlPoint.Flush();
            }
        }

        public void ChamferOnePoint(double cutX, double cutY, Define.CornerCutType cutType, Vec3 comparePoint)
        {
            if (!isSameOrient)
            {
                double temp = cutX;
                cutX = cutY;
                cutY = temp;
            }
            double distance = double.MaxValue;
            int pos = 0;
            for (int j = 0; j < ctrlPointList.Length; j++) //遍历得到最接近comparePoint的控制点
            {
                double nowDistance = ctrlPointList[j].WorldPos.Dist(comparePoint);
                if (nowDistance < distance)
                {
                    distance = nowDistance;
                    pos = j;
                }
            }
            ChamferCtrlPoint chamferCtrlPoint = ctrlPointList[pos] as ChamferCtrlPoint;
            chamferCtrlPoint.CutType = cutType;
            chamferCtrlPoint.CutX = cutX;
            chamferCtrlPoint.CutY = cutY;
            chamferCtrlPoint.Flush();
        }

    }
}
