﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using LoongCAD.STC.Plugins;
using LoongCAD.STC;

namespace ColdRolledOverlap1
{
    class StiffenerWeld
    {
        private ConnectionComponent comp;
        private PartBase mainPart, subPart;
        private List<Plane> surroundPlaneList;
        public List<Vec3> surroundPointList = new List<Vec3>();
        private List<WeldProperties> weldParamList = new List<WeldProperties>();
        private List<CtrlPoint> ctrlPointList;
        private double eps = 1e-3;
        private int modn = 1;

        public StiffenerWeld(ConnectionComponent comp, PartBase mainPart, PartBase subPart)
        {
            this.comp = comp;
            this.mainPart = mainPart;
            this.subPart = subPart;
        }

        //按surroundPlane顺序添加焊缝，weldParams对应项为空值代表不焊接
        public void SetSurroundPlane(List<Plane> surroundPlane, List<WeldProperties> weldParams)
        {
            modn = surroundPlane.Count;
            //计算板的创建面的法向量
            CtrlPoint[] ctrlPoints = subPart.CtrlPoints;
            Vec3 ctrlPoint1 = ctrlPoints[0].WorldPos;
            Vec3 ctrlPoint2 = ctrlPoints[1].WorldPos;
            Vec3 ctrlPoint3 = ctrlPoints[2].WorldPos;
            Plane p = new Plane(ctrlPoint1, ctrlPoint2 - ctrlPoint1, ctrlPoint3 - ctrlPoint1);
            //获取所有包围面的交点
            for (int i = 0; i < surroundPlane.Count; i++)
            {
                Vec3 point = surroundPlane[MOD(i - 1)].Intersect(p, surroundPlane[i]);
                surroundPointList.Add(point);
            }
            //将所有包围面的法向量统一朝内
            Vec3 normal = (surroundPointList[1] - surroundPointList[0]).Cross(surroundPointList[2] - surroundPointList[0]);
            for (int i = 0; i < surroundPlane.Count; i++)
            {
                Vec3 orient = normal.Cross(surroundPointList[MOD(i + 1)] - surroundPointList[i]);
                if (surroundPlane[i].Normal.Dot(orient) < 0)
                    surroundPlane[i] = surroundPlane[i].Invert();
            }
            //
            this.weldParamList = weldParams;
            this.surroundPlaneList = surroundPlane;
            this.ctrlPointList = new List<CtrlPoint>(ctrlPoints);
        }

        public int MOD(int m)
        {
            if (m > 0)
                return m % modn;
            else
                return (m + modn) % modn;
        }

        //weldParams的某个元素是否为空判断是否添加(折角处的)焊缝，1,2,3代表前、后、两者
        public void CreateSideWeld(Plane frontPlane, Plane backPlane, int createMode)
        {
            if (mainPart == null || subPart == null)
                return;
            Vec3[][] frontPoints = { null, null, null, null }; //frontPlane面的侧边点（一个侧边两个ctrlPoint点）
            Vec3[][] backPoints = { null, null, null, null };
            for (int i = 0; i < surroundPlaneList.Count; i++)
            {
                if (weldParamList[i] == null) //选择了不焊接
                    continue;
                List<int> nearPlanePoint = FindNearCtrlPoints(i);
                if (nearPlanePoint.Count != 2) //有两个点与平面接触才能焊接
                    continue;
                //
                Vec3 auxOrient = backPlane.Origin - frontPlane.Origin; //front面的原点指向back面的原点，用于辅助确定方向
                Vec3 normal = frontPlane.Normal.Normalize();
                if (normal.Dot(auxOrient) < 0)
                    normal *= -1; //normal是一个从front面指向back面的单位向量
                double thickness = frontPlane.Dist(backPlane.Origin);
                //折角移动点
                Vec3 point1 = MovePointByChamfer(nearPlanePoint[0], surroundPlaneList[i]);
                Vec3 point2 = MovePointByChamfer(nearPlanePoint[1], surroundPlaneList[i]);
                //控制点位置
                frontPoints[i] = new Vec3[] { point1, point2 };
                backPoints[i] = new Vec3[] { point1, point2 };
                if (backPlane.Dist(ctrlPointList[nearPlanePoint[0]].WorldPos) < eps) //控制点在back面上
                {
                    frontPoints[i][0] -= normal * thickness;
                    frontPoints[i][1] -= normal * thickness;
                }
                else
                {
                    backPoints[i][0] += normal * thickness;
                    backPoints[i][1] += normal * thickness;
                }
            }
            //创建焊缝
            for(int i = 0; i < surroundPlaneList.Count; i++)
            {
                //侧边
                if(createMode != 2 && frontPoints[i] != null) //前
                {
                    CreateSideWeld(i, frontPoints[i]);
                }
                if (createMode != 1 && backPoints[i] != null) //后
                {
                    CreateSideWeld(i, backPoints[i]);
                }
            }
            //底部
            if(frontPoints[0] == null && frontPoints[1] != null)
            {
                Vec3[] weldPoints = new Vec3[] { frontPoints[1][0], backPoints[1][0] };
                CreateWeld(weldPoints, weldParamList[0]);
            }
            //顶部
            if (frontPoints[2] == null && frontPoints[1] != null)
            {
                Vec3[] weldPoints = new Vec3[] { frontPoints[1][1], backPoints[1][1] };
                CreateWeld(weldPoints, weldParamList[2]);
            }
        }

        //创建某一个面上的多边形包围焊缝
        public void CreateSurroundWeld(Plane ctrlPlane, Plane[] sidePlanes)
        {
            //得到交点
            List<Vec3> pointList = new List<Vec3>();
            for(int i = 0; i < sidePlanes.Length; i++)
            {
                Vec3 intersectPoint = ctrlPlane.Intersect(sidePlanes[i], sidePlanes[(i + 1) % sidePlanes.Length]);
                pointList.Add(intersectPoint);
            }
            //包围焊缝由内部点连线和边线组成（如果有超出部分）
            //List<Vec3> weldPointList = new List<Vec3>();
            //焊接
            Weld weld = comp.CreatePolyWeld(mainPart, subPart, pointList.ToArray(), true);
            weldParamList[0].ApplyToWeld(weld);
            weld.Flush();
        }

        public bool IsInsideBound(Vec3 point, ref Plane intersectPoint)
        {
            for(int i = 0; i < surroundPlaneList.Count; i++)
            {
                if(surroundPlaneList[i].SignedDist(point) < 0)
                {
                    //intersectPoint;
                    return false;
                }
            }
            return true;
        }

        public static Vec3 PlaneLineIntersect(Plane p, Vec3 v1, Vec3 v2)
        {
            Vec3 v = v2 - v1; //v1指向v2的向量
            Vec3 normal = p.Normal.Normalize();
            if (Math.Abs(normal.Dot(v)) < 1e-6)
                return Vec3.ZERO;
            double dist1 = p.SignedDist(v1);
            double dist2 = p.SignedDist(v2);
            if (dist1 == 0)
                return v1;
            if (dist2 == 0)
                return v2;
            if(dist1 * dist2 < 0) //在异侧
            {
                normal.Along(v);
                dist1 = Math.Abs(dist1);
                dist2 = Math.Abs(dist2);
                Vec3 v1Proj = v1 + normal * dist1; //v1点在平面上的投影点
                Vec3 v2Proj = v2 - normal * dist2;
                Vec3 horiVec = (v2Proj - v1Proj).Normalize(); //v1投影点指向v2投影点的单位向量
                double percentage = dist1 / (dist1 + dist2);
                return v1Proj + horiVec * v1Proj.Dist(v2Proj) * percentage;
            }
            else //在同侧
            {
                if (dist1 > 0) //normal方向改为点指向平面的方向
                    normal *= -1;
                dist1 = Math.Abs(dist1);
                dist2 = Math.Abs(dist2);
                Vec3 v1Proj = v1 + normal * dist1; //v1点在平面上的投影点
                Vec3 v2Proj = v2 + normal * dist2;
                if(dist1 > dist2) //保证v1投影点靠近交点
                {
                    double temp = dist1;
                    dist1 = dist2;
                    dist2 = temp;
                    Vec3 tmp = v1Proj;
                    v1Proj = v2Proj;
                    v2Proj = tmp;
                }
                if (v1Proj.Dist(v2Proj) < 1e-6)
                    return v1Proj;
                Vec3 horiVec = (v1Proj - v2Proj).Normalize(); //v2投影点指向v1投影点的单位向量
                double percentage = dist1 / (dist2 - dist1);
                return v1Proj + horiVec * v1Proj.Dist(v2Proj) * percentage;
            }
        }

        public static Vec3 ProjectPoint(Plane p, Vec3 v)
        {
            double dist = p.SignedDist(v);
            Vec3 normal = p.Normal.Normalize();
            return v - normal * dist;
        }

        //判断焊接点是否在包围面内部，并创建焊缝
        private void CreateSideWeld(int i, Vec3[] weldPoints)
        {
            Plane lastPlane = surroundPlaneList[MOD(i - 1)];
            Plane nextPlane = surroundPlaneList[MOD(i + 1)];
            Vec3 lastIntersectPoint = PlaneLineIntersect(lastPlane, weldPoints[0], weldPoints[1]);
            Vec3 nextIntersectPoint = PlaneLineIntersect(nextPlane, weldPoints[0], weldPoints[1]);
            if (lastPlane.SignedDist(weldPoints[0]) < 0)
                weldPoints[0] = lastIntersectPoint;
            if (nextPlane.SignedDist(weldPoints[0]) < 0)
                weldPoints[0] = nextIntersectPoint;
            if (lastPlane.SignedDist(weldPoints[1]) < 0)
                weldPoints[1] = lastIntersectPoint;
            if (nextPlane.SignedDist(weldPoints[1]) < 0)
                weldPoints[1] = nextIntersectPoint;
            CreateWeld(weldPoints, weldParamList[i]);
        }

        private void CreateWeld(Vec3[] weldPoints, WeldProperties weldProps)
        {
            Weld weld = comp.CreatePolyWeld(mainPart, subPart, weldPoints, true);
            weldProps.ApplyToWeld(weld);
            weld.Flush();
        }

        public Vec3 MovePointByChamfer(int index, Plane plane) 
        {
            Vec3 point = ctrlPointList[index].WorldPos;
            Vec3 lastPoint = ctrlPointList[(index - 1 + ctrlPointList.Count) % ctrlPointList.Count].WorldPos;
            Vec3 nextPoint = ctrlPointList[(index + 1) % ctrlPointList.Count].WorldPos;
            Vec3 xVec = Vec3.ZERO, yVec = Vec3.ZERO; //x_cut,y_cut的方向
            if (plane.Dist(lastPoint) < eps)
                xVec = lastPoint - point;
            else
                yVec = nextPoint - point;
            ChamferCtrlPoint chamferCtrlPoint = ctrlPointList[index] as ChamferCtrlPoint;
            if(chamferCtrlPoint.CutType != Define.CornerCutType.CORNER_CUT_NONE)
            {
                point += xVec.Normalize() * chamferCtrlPoint.CutX;
                point += yVec.Normalize() * chamferCtrlPoint.CutY;
            }
            return point;
        }

        public List<int> FindNearCtrlPoints(int index)
        {
            List<int> re = new List<int>();
            for(int i = 0; i < ctrlPointList.Count; i++)
            {
                Plane nearPlane = surroundPlaneList[index];
                if (nearPlane.Dist(ctrlPointList[i].WorldPos) < eps)
                    re.Add(i); //将下标存入数组并返回
            }
            if(re.Count == 2)
            {
                Vec3 lastPoint = ctrlPointList[re[0]].WorldPos; //返回的第一个点应该在第二个点前面（按传入平面的顺序）
                Vec3 nextPoint = ctrlPointList[re[1]].WorldPos;
                Vec3 orient = surroundPointList[MOD(index + 1)] - surroundPointList[index];
                if((nextPoint - lastPoint).Dot(orient) < 0)
                {
                    int tmp = re[0];
                    re[0] = re[1];
                    re[1] = tmp;
                }
            }
            return re;
        }

    }
}
