﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ColumnWithStiffeners182
{
    class Util
    {
        public static void CreateSteelBySymmetry(ConnectionComponent cmp, Plane zplane, double thickness, Plane[] sidePlanes, bool bRuntime, Plane plane)
        {
            Plane symmetryCenterPlane = plane.Clone();
            Plane zCtrlPlane = zplane.Clone();
            cmp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, bRuntime);
            if (symmetryCenterPlane != null)
            {
                double dist = zCtrlPlane.SignedDist(symmetryCenterPlane.Origin);
                zCtrlPlane.Move(2 * dist - thickness); //移动板平面到对应位置
                cmp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, bRuntime);
            }
            cmp.Flush();
        }

        public static Plane CreatePlaneBySymmetry(Plane zplane, Plane plane)
        {
            Plane symmetryCenterPlane = plane.Clone();
            Plane zCtrlPlane = zplane.Clone();
            double dist = zCtrlPlane.SignedDist(symmetryCenterPlane.Origin);
            return zCtrlPlane.Move(2 * dist).Invert(); //移动板平面到对应位置
        }

        public static Vec3 SymmetryPoint(Plane plane, Vec3 point)
        {
            double dist = plane.SignedDist(point);
            point = point - 2 * dist * plane.Normal.Normalize();
            return point;
        }

        public static Vec3 PlaneLineIntersect(Plane plane, Vec3 p1, Vec3 p2)
        {
            double op;
            plane.IntersectRay(p1, p2 - p1, out op);
            Vec3 vec = p2 - p1;
            return p1 + op * vec;
        }

        /// <summary>
        /// 两点的中心点
        /// </summary>
        public static Vec3 CenterPoint(Vec3 p1, Vec3 p2)
        {
            double x = (p1.X + p2.X) / 2;
            double y = (p1.Y + p2.Y) / 2;
            double z = (p1.Z + p2.Z) / 2;
            return new Vec3(x, y, z);
        }

        /// <summary>
        /// 两个面的中心面
        /// </summary>
        public static Plane CenterPlane(Plane p1, Plane p2)
        {
            Vec3 origin = CenterPoint(p1.Origin, p2.Origin); //未做方向校验
            return new Plane(origin, p1.Normal);
        }



        /// <summary>
        /// 设置多个默认值，排在前面的优先赋值
        /// </summary>
        /// <param name="value"></param>
        /// <param name="setValueList"></param>
        public static void SetDefaultValue<T>(ref T value, params T[] setValueList)
        {
            double limit = 1e6; //一个正常的区间（<1km）
            if (!GetObjectVaule(value, out _)) //不能转为数值，选取第一个非空的值
            {
                for (int i = 0; i < setValueList.Length; i++)
                {
                    if (value != null)
                        break;
                    value = setValueList[i];
                }
                return;
            }
            for (int i = 0; i < setValueList.Length; i++)
            {
                GetObjectVaule(value, out double nowValue);
                if (nowValue > -limit && nowValue < limit)
                    break;
                value = setValueList[i];
            }
        }
        private static bool GetObjectVaule<T>(T obj, out double convertValue)
        {
            if (obj is double)
            {
                convertValue = (double)(Object)obj;
                return true;
            }
            else if (obj is int)
            {
                convertValue = (int)(Object)obj;
                return true;
            }
            else if (obj is long)
            {
                convertValue = (long)(Object)obj;
                return true;
            }
            else
            {
                convertValue = 0;
                return false;
            }
        }

        public static bool PlatePropIsValid(PlateProperties props)
        {
            if (props.Thickness <= 0 || props.Width <= 0 || props.Height <= 0)
                return false;
            return true;
        }

        /// <summary>
        /// 过一条直线做某个平面的垂直面
        /// </summary>
        /// <param name="plane"></param>
        /// <param name="line"></param>
        /// <param name="vec">指定面的朝向</param>
        /// <returns></returns>
        public static Plane VerticalPlane(Plane plane, Line line, Vec3 vec)
        {
            Vec3 normal = plane.Normal.Cross(line.Dir);
            normal = normal.Along(vec);
            return new Plane(line.V0, normal);
        }
        /// <summary>
        /// 将vertPlane变为与horiPlane垂直的面
        /// </summary>
        /// <param name="horiPlane">参考面</param>
        /// <param name="vertPlane">要旋转的面</param>
        /// <returns></returns>
        public static Plane VerticalPlane(Plane horiPlane, Plane vertPlane)
        {
            if (vertPlane.Normal.IsVerticalTo(horiPlane.Normal))
                return vertPlane.Clone();
            return VerticalPlane(horiPlane, horiPlane.Intersect(vertPlane), vertPlane.Normal);
        }
        /// <summary>
        /// 两个面相交，根据方向指定新面的方向
        /// </summary>
        /// <param name="plane1"></param>
        /// <param name="plane2"></param>
        /// <param name="vec"></param>
        /// <returns></returns>
        public static Plane OrientPlane(Plane plane1, Plane plane2, Vec3 vec)
        {
            return new Plane(plane1.Intersect(plane2).V0, vec);
        }

        public static double VectorAngle(Vec3 v1, Vec3 v2)
        {
            double dot = v1.Dot(v2);
            double cosValue = dot / (v1.Length * v2.Length);
            return Math.Acos(cosValue);
        }

        public static Plane CreatePlaneBy2Line(Line line1, Line line2)
        {
            Vec3 normal = line1.Dir.Cross(line2.Dir);
            if (normal.Dist(Vec3.ZERO) > 1e-4)
                return new Plane(line1.V0, normal);
            normal = line1.Dir.Cross(line2.V0 - line1.V0);
            if (normal.Dist(Vec3.ZERO) > 1e-4)
                return new Plane(line1.V0, normal);
            return Plane.IDENTITY; //未判断异面直线
        }

        public static Plane CreatePlaneBy3Point(Vec3 point1, Vec3 point2, Vec3 point3)
        {
            return new Plane(point2, point1 - point2, point3 - point2);
        }

        /// <summary>
        /// 平面绕线旋转
        /// </summary>
        /// <param name="plane">要旋转的面</param>
        /// <param name="start">旋转中心线的起点</param>
        /// <param name="end">旋转中心线的终点</param>
        /// <param name="rad">旋转角度</param>
        public static void RotatePlane(ref Plane plane, Vec3 start, Vec3 end, double rad)
        {
            Vec3 normal = plane.Normal; 
            Vec3 origin = plane.Origin; 
            Rotate(ref normal, end - start, rad, Vec3.ZERO);
            Rotate(ref origin, end - start, rad, start);
            plane = new Plane(origin, normal);
        }

        /// <summary>
        /// 旋转一个点或向量
        /// </summary>
        /// <param name="target">要旋转的点或向量</param>
        /// <param name="vecUp">旋转方向</param>
        /// <param name="rad">旋转角度</param>
        /// <param name="o">旋转点时填vecUp起点，用于确定旋转中心。旋转向量时填Vec3.ZERO</param>
        public static void Rotate(ref Vec3 target, Vec3 vecUp, double rad, Vec3 o)
        {
            Vec3 vecn = vecUp.Normalize();
            target -= o;

            double _cos = System.Math.Cos(rad);
            double _sin = System.Math.Sin(rad);
            double _oneMinusCos = 1.0 - System.Math.Cos(rad);

            target = target * _cos + vecn.Cross(target) * _sin + vecn * vecn.Dot(target) * _oneMinusCos;
            target += o;
        }

        /// <summary>
        /// 返回梁的起始点
        /// </summary>
        /// <param name="beam"></param>
        /// <returns>Vec3[]{ startPos, endPos}</returns>
        public static Vec3[] StartEndPoint(Beam beam)
        {
            Vec3[] points = new Vec3[2];
            BeamPlanes planes = BeamPlanes.CreateFromBeam(beam);
            Plane centerPlane1 = planes.TopBound.Move(planes.TopBound.SignedDist(planes.BottomBound.Origin) / 2);
            Plane centerPlane2 = planes.FrontBound.Move(planes.FrontBound.SignedDist(planes.BackBound.Origin) / 2);
            Vec3 start = planes.HeadBound.Intersect(centerPlane1, centerPlane2);
            Vec3 end = planes.TailBound.Intersect(centerPlane1, centerPlane2);
            points[0] = start;
            points[1] = end;
            return points;
        }

        public static Vec3 ProjPoint(Plane plane, Vec3 point)
        {
            return point - plane.Normal * plane.SignedDist(point);
        }

        public static void Swap<T>(ref T x, ref T y)
        {
            T temp = x;
            x = y;
            y = temp;
        }

        public static void Along(ref Plane p, Vec3 orient)
        {
            if (p.Normal.Dot(orient) < 0)
                p.InvertThis();
        }

        public static void MovePlane(Plane plane, Vec3 orient, double dist)
        {
            orient.NormalizeThis();
            plane.Origin = plane.Origin + orient * dist;
        }
        public static void MovePlanes(Plane[] planes, Vec3 orient, double dist)
        {
            for(int i = 0; i < planes.Length; i++)
            {
                MovePlane(planes[i], orient, dist);
            }
        }

        public static void IngoreListToMatrix(List<int> inputDeleteList, int xCount, int yCount, ref bool[][] deleteMatrix)
        {
            if (deleteMatrix == null)
                deleteMatrix = InitailArray(yCount + 1, xCount + 1);
            foreach (int deleteIndex in inputDeleteList)
            {
                if (deleteIndex > xCount * yCount)
                    continue;
                //不处理时，水平方向(x)上第a个螺栓，垂直方向(-y)上第b个螺栓
                int a = (deleteIndex - 1) % xCount + 1;
                int b = (deleteIndex - 1) / xCount + 1;
                deleteMatrix[b][a] = true;
            }
        }
        public static List<int> IngoreMatrixToList(bool[][] deleteMatrix)
        {
            List<int> deleteIndexList = new List<int>();
            return deleteIndexList;
        }
        public static int[] ReverseIngoreList(int[] inputDeleteList, int xCount, int yCount)
        {
            List<int> deleteIndexList = new List<int>();
            if (inputDeleteList == null || inputDeleteList.Length == 0)
                return deleteIndexList.ToArray();
            foreach (int deleteIndex in inputDeleteList)
            {
                if (deleteIndex > xCount * yCount)
                    continue;
                //不处理时，水平方向(x)上第a个螺栓，垂直方向(-y)上第b个螺栓
                int a = (deleteIndex - 1) % xCount + 1;
                int b = (deleteIndex - 1) / xCount + 1;
                //删除螺栓对称到上侧
                b = yCount + 1 - b;
                int delete = xCount * (b - 1) + a;
                deleteIndexList.Add(delete);
            }
            return deleteIndexList.ToArray();
        }

        private static bool[][] InitailArray(int m, int n)
        {
            if (m == 0 || n == 0 || m == double.MaxValue || n == double.MaxValue)
                return null;
            bool[][] re = new bool[m][];
            for (int i = 0; i < m; i++)
            {
                re[i] = new bool[n];
                for (int j = 0; j < n; j++)
                {
                    re[i][j] = false;
                }
            }
            return re;
        }
        public static bool[][] TMatrix(bool[][] oriMatrix)
        {
            int m = oriMatrix.Length - 1, n = oriMatrix[0].Length - 1;
            bool[][] matrix = InitailArray(n + 1, m + 1);
            for(int i = 1; i <= m; i++)
                for(int j = 1; j <= n; j++)
                    matrix[j][i] = oriMatrix[i][j];
            return matrix;
        }

        public static Plane[] Copy(Plane[] planes)
        {
            List<Plane> re = new List<Plane>();
            foreach (var item in planes)
            {
                re.Add(item.Clone());
            }
            return re.ToArray();
        }
        public static PlateProperties Copy(PlateProperties props)
        {
            PlateProperties re_props = new PlateProperties();
            re_props.Height = props.Height;
            re_props.Width = props.Width;
            re_props.Thickness = props.Thickness;
            return re_props;
        }
        public static WeldProperties Copy(WeldProperties props)
        {
            WeldProperties re_props = new WeldProperties();
            //
            re_props.EdgeAround = props.EdgeAround;
            re_props.RefText = props.RefText;
            re_props.StitchWeld = props.StitchWeld;
            re_props.MadeType = props.MadeType;
            //
            re_props.SizeDown = props.SizeDown;
            re_props.TypeDown = props.TypeDown;
            re_props.RootGapDown = props.RootGapDown;
            re_props.ThroatDown = props.ThroatDown;
            re_props.RootFaceDown = props.RootFaceDown;
            re_props.PitchDown = props.PitchDown;
            re_props.LengthDown = props.LengthDown;
            re_props.FinishDown = props.FinishDown;
            re_props.GrooveAngleDown = props.GrooveAngleDown;
            re_props.ContourDown = props.ContourDown;
            //
            re_props.SizeUp = props.SizeUp;
            re_props.TypeUp = props.TypeUp;
            re_props.RootGapUp = props.RootGapUp;
            re_props.ThroatUp = props.ThroatUp;
            re_props.RootFaceUp = props.RootFaceUp;
            re_props.PitchUp = props.PitchUp;
            re_props.LengthUp = props.LengthUp;
            re_props.FinishUp = props.FinishUp;
            re_props.GrooveAngleUp = props.GrooveAngleUp;
            re_props.ContourUp = props.ContourUp;

            return re_props;
        }
    }
}
