﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ColumnWithStiffeners182
{
    
    public class DoubleHandler
    {

        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PlateProperties m_plateProps = new PlateProperties();

        private bool m_creatingNearPlate = false;
        private bool m_creatingFarPlate = false;
        
        private bool m_cuttingPlateEdge = false;
        private double m_cuttingAngle = double.MaxValue;

        private double m_holeDiameter = double.MaxValue;
        private double m_slotWidth = double.MaxValue;
        private double m_slotHeight = double.MaxValue;

        private int m_holeHoriCount = 0;
        private int m_holeVertCount = 0;
        private List<double> m_holeHoriStride = new List<double>();
        private List<double> m_holeVertStride = new List<double>();
        private BoltLocation boltLocationHori, boltLocationVert;
        private double m_flangeGap = double.MaxValue;
        private double m_bottomGap = double.MaxValue;
        Plane nsWebPlane;
        Plane fsWebPlane;

        private Plane m_leftPlane;
        private Plane m_rightPlane;
        private Plane m_topPlane;
        private Plane m_bottomPlane;

        // 主零件长度方向的下方向
        private Vec3 m_downNormal = -Vec3.AXIS_Z;

        public DoubleHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            if (!this.ParseProps())
                return;
            if (!this.ParseSidePlanes())
                return;

            // 解析主零件远近腹板面
            m_context.ParseAwaySidePlane(
                m_context.MainPlanes.FrontWeb, m_context.MainPlanes.BackWeb,
                out nsWebPlane, out fsWebPlane);

            Plane[] sidePlanes = new Plane[] { m_leftPlane, m_bottomPlane, m_rightPlane, m_topPlane };
            
            // 创建双板
            PartBase nsPlate = null;
            PartBase fsPlate = null;
            if (m_creatingNearPlate) {
                nsPlate = m_context.Comp.CreateSteelPlate(nsWebPlane, nsWebPlane.Move(m_plateProps.Thickness), sidePlanes, true);
                m_context.NearDoublePlate = nsPlate;
            }
            if (m_creatingFarPlate) {
                fsPlate = m_context.Comp.CreateSteelPlate(fsWebPlane, fsWebPlane.Move(m_plateProps.Thickness), sidePlanes, true);
                m_context.FarDoublePlate = fsPlate;
            }

            // 分析主零件连接面处是否需要为连接板留出间隙
            const double eps = 1e-3;
            if (m_context.MainJointPlane.Normal.IsParallelTo(nsWebPlane.Normal) &&
                m_context.MainJointPlane.Dist(nsWebPlane.Origin) < eps && nsPlate != null) {
                m_context.MainJointMargin = m_plateProps.Thickness;
            }
            else if (m_context.MainJointPlane.Normal.IsParallelTo(fsWebPlane.Normal) &&
                m_context.MainJointPlane.Dist(fsWebPlane.Origin) < eps && fsPlate != null) {
                m_context.MainJointMargin = m_plateProps.Thickness;
            }

            //
            Plane plateCenterPlane = m_topPlane.Move(-0.5 * m_topPlane.Dist(m_bottomPlane.Origin));
            this.CutPlateEdges(nsPlate, nsWebPlane, plateCenterPlane, new Plane[] { m_leftPlane, m_rightPlane});
            this.CutPlateEdges(fsPlate, fsWebPlane, plateCenterPlane, new Plane[] { m_leftPlane, m_rightPlane });
            this.CreateHoles(nsPlate, fsPlate);
        }

        private bool ParseProps()
        {
            //
            {
                LoongCAD.Util.DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.DOUBLE_PLATE_PROPS);
                m_plateProps.SetProps(plateProps);
            }

            //
            {
                int createIndex = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_CREATION_MODE);
                switch (createIndex) {
                case 1: { // None
                        m_creatingNearPlate = false;
                        m_creatingFarPlate = false;
                    } break;
                case 2: {
                        m_creatingNearPlate = false;
                        m_creatingFarPlate = true;
                    } break;
                case 3: {
                        m_creatingNearPlate = true;
                        m_creatingFarPlate = false;
                    } break;
                case 4: {
                        m_creatingNearPlate = true;
                        m_creatingFarPlate = true;
                    } break;
                case 0:
                default: {
                        m_creatingNearPlate = false;
                        m_creatingFarPlate = false;
                        System.Diagnostics.Debug.Assert(false, "无效的双板创建模式");
                    } break;
                }
            }

            //
            {
                int shapeIndex = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_PLATE_EDGE_SHAPE);
                switch (shapeIndex) {
                case 1: {
                        m_cuttingPlateEdge = true;
                    } break;
                case 2: {
                        m_cuttingPlateEdge = false;
                    } break;
                case 0:
                default: {
                        m_cuttingPlateEdge = false;
                        System.Diagnostics.Debug.Assert(false, "无效的双板边缘形状选项");
                    } break;
                }
                m_cuttingAngle = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_PLATE_BEVEL_ANGLE);
            }

            //
            m_holeDiameter = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_DIAMETER);
            m_slotWidth = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_SLOT_WIDTH);
            m_slotHeight = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_SLOT_HEIGHT);

            m_flangeGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_FLANGE_GAP);
            m_bottomGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_BOTTOM_GAP);

            double m_holeLeftMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_LEFT_MARGIN);
            double m_holeRightMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_RIGHT_MARGIN);
            double m_holeTopMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_TOP_MARGIN);
            double m_holeBottomMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_BOTTOM_MARGIN);

            m_holeHoriCount = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_HOLE_HORI_COUNT);
            double[] horiStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.DOUBLE_HOLE_HORI_STRIDE);
            m_holeVertCount = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_HOLE_VERT_COUNT);
            double[] vertStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.DOUBLE_HOLE_VERT_STRIDE);
            double[] defaultStride = { 60 };
            boltLocationHori = new BoltLocation(40, 40, 2, defaultStride);
            boltLocationVert = new BoltLocation(40, 40, 2, defaultStride);
            //水平长度
            double horiLength = double.MaxValue;
            if (m_flangeGap != double.MaxValue)
                horiLength = m_context.MainPlanes.TopBound.Dist(m_context.MainPlanes.BottomBound.Origin) - 2 * m_flangeGap;
            boltLocationHori.centerMode = true;
            boltLocationHori.SetBoltLocation(m_holeLeftMargin, m_holeRightMargin, m_holeHoriCount, horiStride, horiLength);
            //垂直长度
            boltLocationHori.centerMode = true;
            boltLocationVert.SetBoltLocation(m_holeBottomMargin, m_holeTopMargin, m_holeVertCount, vertStride, m_plateProps.Height);

            return true;
        }

        private bool ParseSidePlanes()
        {
            // 解析上下侧面
            Plane topPlane = null;
            Plane bottomPlane = null;
            do {
                // 解析次零件相对场景在下方的边界面
                Plane subBottomBound = null;
                {
                    double thresh = Math.Cos(89.0 * Math.PI / 180.0);
                    Vec3 refBottomDir;
                    m_downNormal = -m_context.MainDirection.Normalize();
                    // 主零件不是水平放置
                    // 以场景Z轴负方向为下方
                    if (Math.Abs(m_context.MainDirection.Normalize().Dot(Vec3.AXIS_Z)) > thresh) {
                        refBottomDir = -Vec3.AXIS_Z;
                        if (m_downNormal.Dot(Vec3.AXIS_Z) > 0.0) {
                            m_downNormal = -m_downNormal;
                        }
                    }
                    // 主零件水平放置
                    // 以主零件长度方向负方向为下方
                    else {
                        refBottomDir = -m_context.MainDirection;
                    }

                    // 分析次零件下边界面
                    double maxDot = 0.0;
                    // 顶边界面
                    {
                        double curDot = m_context.SubPlanes.TopBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.TopBound;
                        }
                    }
                    // 底边界面
                    {
                        double curDot = m_context.SubPlanes.BottomBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.BottomBound;
                        }
                    }
                    // 前边界面
                    {
                        double curDot = m_context.SubPlanes.FrontBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.FrontBound;
                        }
                    }
                    // 后边界面
                    {
                        double curDot = m_context.SubPlanes.BackBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.BackBound;
                        }
                    }
                }
                if (subBottomBound == null)
                    break;

                // 底侧面
                int stiffOrientMode = m_paramsHandler.GetIntValue(ParamNames.STIFF_ORIENTATION);
                bottomPlane = subBottomBound.Clone();
                if (stiffOrientMode != 1)
                    bottomPlane = Util.VerticalPlane(m_context.MainNearBound, bottomPlane);
                bottomPlane.MoveThis(-m_bottomGap);

                // 顶侧面
                topPlane = bottomPlane.Invert();
                topPlane.MoveThis(boltLocationVert.length);

            } while (false);
            if (topPlane == null || bottomPlane == null)
                return false;

            // 解析左右侧面
            Plane leftPlane = m_context.MainPlanes.TopBound.Invert();
            Plane rightPlane = m_context.MainPlanes.BottomBound.Invert();
            if (leftPlane.Normal.Dot(m_context.SubDirection) < 0)
                Util.Swap(ref leftPlane, ref rightPlane);
            if (m_flangeGap != double.MaxValue) //填了板边距，按边距定位板
            {
                m_leftPlane = leftPlane.Move(m_flangeGap).Invert();
                m_rightPlane = rightPlane.Move(m_flangeGap).Invert();
            }
            else //否则螺栓居中，螺栓定位板
            {
                Plane centerPlane = Util.CenterPlane(leftPlane, rightPlane);
                m_leftPlane = centerPlane.Invert().Move(boltLocationHori.CenterMargin1);
                m_rightPlane = centerPlane.Move(boltLocationHori.CenterMargin2);
            }

            m_topPlane = topPlane;
            m_bottomPlane = bottomPlane;
            return true;
        }

        private void CreateHoles(PartBase nsPlate, PartBase fsPlate)
        {
            if (m_topPlane == null || m_bottomPlane == null) {
                return;
            }

            Plane xAxisPlane = m_bottomPlane.Move(-boltLocationVert.margin1);
            Plane yAxisPlane = m_leftPlane.Move(-boltLocationHori.margin1);
            Vec3 xAxis = -m_leftPlane.Normal.Normalize();

            // 解析螺栓坐标系
            Vec3 nsStartPos = xAxisPlane.Intersect(yAxisPlane, nsWebPlane.Move(m_plateProps.Thickness));
            Vec3 nsEndPos = nsStartPos + xAxis * boltLocationHori.length;
            // 工作坐标系
            Vec3 nsX = xAxis;
            Vec3 nsZ = nsWebPlane.Normal.Normalize()
                .Along(m_context.SubDirection).Along(m_context.SubDirection.Cross(m_context.Comp.WorldMatrix.Z));
            Vec3 nsY = nsZ.Cross(nsX);
            Mat43 nsBoltAxis = new Mat43(nsX, nsY, nsZ, Vec3.ZERO);
            //
            double offsetAngle = Util.VectorAngle(nsX, m_context.SubDirection);
            Vec3 bottomOrient = m_context.MainDirection.Along(m_bottomPlane.Normal);
            if (m_context.SubDirection.Dot(bottomOrient) > 0)
                offsetAngle *= -1;
            if (!m_context.IsMainRotate)
                offsetAngle = 0;
            GetXYList(out List<double> xList, out List<double> yList, offsetAngle);

            //
            CreateHole(xList, yList, nsPlate, nsStartPos, nsEndPos, nsBoltAxis);
            CreateHole(xList, yList, fsPlate, nsStartPos, nsEndPos, nsBoltAxis);
        }

        private void CreateHole(List<double> xList, List<double> yList, PartBase plate, Vec3 start, Vec3 end, Mat43 axis)
        {
            if (plate == null)
            {
                return;
            }

            BoltGroup curHoles = m_context.Comp.CreateBoltHoles(
                (PartBase)null, plate, start, end, true);
            if (curHoles == null)
                return;

            /// 创建螺栓之后需要首先设置其WorkAxis属性(若不为单位矩阵)再设置其他属性
            const double eps = 1.0;
            curHoles.WorkAxis = axis;
            curHoles.Shape = LoongCAD.STC.Define.BoltGroupShape.BOLT_GROUPSHAPE_LIST;
            curHoles.ListX = xList.ToArray();
            curHoles.ListY = yList.ToArray();
            curHoles.ArrayStrideX = m_holeHoriStride.ToArray();
            curHoles.ArrayStrideY = m_holeVertStride.ToArray();
            curHoles.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_TOP;
            curHoles.BoltSize = m_holeDiameter;
            curHoles.HoleX = m_slotWidth;
            curHoles.HoleY = m_slotHeight;
            curHoles.HoleType = LoongCAD.STC.Define.BoltHoleType.BOLT_HOLE_SLOTTED;
            curHoles.OverSizeHoleEnable0 = true;
            curHoles.OverSizeHoleEnable1 = true;
            curHoles.CutLength = 2.0 * (m_context.MainJointThickness + m_plateProps.Thickness * 2.0) + eps;
            curHoles.Refresh();
        }

        private void CutPlateEdges(PartBase targetPlate, Plane mainWebPlane, Plane plateCenterPlane, Plane[] edgePlanes)
        {
            if (targetPlate == null || !m_cuttingPlateEdge)
                return;
            const double eps = 1e-3;
            if (m_cuttingAngle < eps || m_cuttingAngle > (90 - eps))
                return;

            //
            double theta = Math.PI * m_cuttingAngle / 180.0;
            Vec3 webNormal = mainWebPlane.Normal.Normalize();
            for (int i=0; i<edgePlanes.Length; ++i) {
                Vec3 edgeNormal = edgePlanes[i].Normal.Normalize();
                Vec3 edgeMiddlePt = edgePlanes[i].Intersect(plateCenterPlane, mainWebPlane);
                if (edgeMiddlePt == Vec3.MAX)
                    break;
                Vec3 cutNormal = Math.Sin(theta) * edgeNormal + Math.Cos(theta) * webNormal;
                Plane cutPlane = new Plane(edgeMiddlePt, -cutNormal);
                m_context.Comp.CreateCutPlane(targetPlate, cutPlane, true);
            }
        }

        private void GetXYList(out List<double> xList, out List<double> yList, double offsetAngle)
        {
            //求各个点的坐标
            xList = new List<double>();
            yList = new List<double>();
            List<double> yStartLocationList = new List<double>();
            { //确定每一列第一个点的纵坐标
                double yLocation = 0;
                for (int x = 0; ;)
                {
                    yStartLocationList.Add(yLocation);
                    if (x == boltLocationHori.stride.Count)
                        break;
                    yLocation += boltLocationHori.stride[x++] * Math.Tan(offsetAngle);
                }
            }
            {
                double xLocation = 0;
                for (int x = 0; ;) //遍历列
                {
                    double yLocation = yStartLocationList[x];
                    for (int y = 0; ;) //遍历行
                    {
                        xList.Add(xLocation);
                        yList.Add(yLocation);
                        if (y == boltLocationVert.stride.Count)
                            break;
                        yLocation += boltLocationVert.stride[y++] / Math.Cos(offsetAngle);
                    }
                    if (x == boltLocationHori.stride.Count)
                        break;
                    xLocation += boltLocationHori.stride[x++];//移动至后一列
                }
            }
        }
    }

}
