﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;

namespace ColumnWithStiffeners182
{
    public class EndHandler
    {

        private ComponentInstanceContext m_context = null;

        private ParamsHandler m_paramsHandler = null;

        private EndCutMode m_cutMode = EndCutMode.DEFAULT;

        private FlangeEndCutMode m_flangeCutMode = FlangeEndCutMode.DEFAULT;

        private double m_endCutGap = Double.MaxValue;

        private double m_flangeCutGap = Double.MaxValue;

        public EndHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            if (!this.ParseProps()) {
                return;
            }
            this.CutPart();
        }

        private bool ParseProps()
        {
            // 端部切割方式, pic_end_cut
            {
                // 解析属性值
                int modeIdx = m_paramsHandler.GetIntValue(ParamNames.PIC_END_CUT);
                switch (modeIdx) {
                case 1: { // 自适应切割模式
                        double angleThresh = 10.0;
                        double dotThresh = Math.Cos(Math.PI * (90.0 - angleThresh) / 180.0);
                        double partDot = m_context.MainDirection.Normalize().Dot(m_context.SubDirection.Normalize());
                        // 次零件相对主零件垂直方向倾斜角度小于10度，则自适应为方形切割
                        if (Math.Abs(partDot) < dotThresh) {
                            m_cutMode = EndCutMode.SQUARE;
                        }
                        // 否则为斜切
                        else {
                            m_cutMode = EndCutMode.BEVEL;
                        }
                    } break;
                case 2:
                    m_cutMode = EndCutMode.SQUARE;
                    break;
                case 3:
                default:
                    m_cutMode = EndCutMode.BEVEL;
                    break;
                }
            }
            // 端部翼板切割方式，pic_flange_cut
            {
                int modeIdx = m_paramsHandler.GetIntValue(ParamNames.PIC_FLANGE_CUT);
                switch (modeIdx) {
                case 2:
                    m_flangeCutMode = FlangeEndCutMode.SQUARE;
                    break;
                case 1:
                default:
                    m_flangeCutMode = FlangeEndCutMode.BEVEL;
                    break;
                }
            }
            // 端部切割间隙
            m_endCutGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_END_GAP);
            // 翼缘切割间隙
            m_flangeCutGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FLANGE_GAP);

            return true;
        }

        private void CutPart()
        {
            // 检查需要的参数是否有效
            if (m_context.MainPlanes == null || m_context.MainJointPlane == null ||
                m_context.MainPlanes.FrontBound == null || m_context.MainPlanes.TopBound == null)
                return;
            if (m_context.SubNearBound == null ||
                m_context.SubPlanes.TopBound == null || m_context.SubPlanes.BottomBound == null ||
                m_context.SubPlanes.FrontWeb == null || m_context.SubPlanes.BackWeb == null)
                return;

            // 计算端部切割的最小间隙
            // 对于连接到腹板面的情况，最小间隙为腹板面到主零件近边界面的有效距离；
            // 其他情况为0
            double minEndGap = m_context.MainNearBound.SignedDist(m_context.MainJointPlane.Origin);
            m_endCutGap = Math.Max(minEndGap, m_endCutGap);

            // 解析用于对齐次零件的对齐辅助面
            Plane alignHelperPlan;
            BeamAlignPlane alignMode = (BeamAlignPlane)m_paramsHandler.GetIntValue(ParamNames.BEAM_ALIGN_PLANE);
            ConnPlateShape plateShape = (ConnPlateShape)m_paramsHandler.GetIntValue(ParamNames.PLATE_SHAPE);
            {
                Vec3 mainJointNormal = m_context.MainJointPlane.Normal;
                if (Math.Abs(mainJointNormal.Dot(m_context.MainPlanes.FrontBound.Normal)) >
                    Math.Abs(mainJointNormal.Dot(m_context.MainPlanes.TopBound.Normal)))
                {
                    alignHelperPlan = m_context.MainNearBound.Move(m_endCutGap);
                    if (alignMode == BeamAlignPlane.ALIGN_WEB && plateShape == ConnPlateShape.SQUARE)
                        alignHelperPlan = m_context.MainNearWeb.Move(m_endCutGap);
                }
                else
                {
                    alignHelperPlan = m_context.MainJointPlane.Move(m_endCutGap);
                }
            }

            //
            var cp = m_context.Comp;
            switch (m_cutMode)
            {
                case EndCutMode.SQUARE: { // 方形切割
                        List<Line> subSideLines = this.ParseBeamSideLines(m_context.SubPlanes);
                        if (subSideLines.Count == 0)
                            break;
                        double minDist = double.MaxValue;
                        Vec3 alignOrigin = Vec3.MAX;
                        double t = 0.0;
                        for (int i=0; i<subSideLines.Count; ++i) {
                            if (!alignHelperPlan.IntersectRay(
                                subSideLines[i].V0, subSideLines[i].Dir, out t)) {
                                continue;
                            }
                            Vec3 pt0 = subSideLines[i].Sample(t);
                            double curDist = m_context.SubNearBound.SignedDist(pt0);
                            if (curDist < minDist) {
                                minDist = curDist;
                                alignOrigin = pt0;
                            }
                        }
                        if (alignOrigin == Vec3.MAX) {
                            break;
                        }
                        m_context.SubAlignPlane = new Plane(alignOrigin, -m_context.SubNearBound.Normal);

                        //
                        cp.CreateAlignPlane(m_context.SubPart, m_context.SubAlignPlane, true);
                    } break;
                case EndCutMode.BEVEL: { // 斜角切割
                        m_context.SubAlignPlane = alignHelperPlan;
                        cp.CreateAlignPlane(m_context.SubPart, m_context.SubAlignPlane, true);
                        if (m_flangeCutMode == FlangeEndCutMode.SQUARE) {
                            this.CutPartFlange();
                        }
                    } break;
#if false
                case EndCutMode.SQUARE_AND_CLOSER: {

                    } break;
                case EndCutMode.CLIPPED_FLANGE: {

                    }
                    break;
#endif
                default:
                    break;
            }
        }

        private void CutPartFlange()
        {
            const double FLANGE_CUT_MARGIN = 10.0;

            // 计算次零件对齐面与其前后腹板面、中心面的交点
            Vec3 frontPoint = Vec3.MAX;
            Vec3 backPoint = Vec3.MAX;
            {
                Plane subCenterPlane = m_context.SubPlanes.TopBound.Move(
                    -0.5 * m_context.SubPlanes.TopBound.Dist(m_context.SubPlanes.BottomBound.Origin));
                frontPoint = m_context.SubAlignPlane.Intersect(
                    subCenterPlane, m_context.SubPlanes.FrontWeb);
                frontPoint = m_context.SubAlignPlane.Intersect(
                    subCenterPlane, m_context.SubPlanes.BackWeb);
            }
            if (frontPoint == Vec3.MAX || backPoint == Vec3.MAX)
                return;

            // 计算前后交点与次零件边界面的距离
            // 距离差小于1，则无需创建翼缘分切割体
            double frontDist = m_context.SubNearBound.SignedDist(frontPoint);
            double backDist = m_context.SubNearBound.SignedDist(backPoint);
            if (Math.Abs(frontDist - backDist) < 1) {
                return;
            }

            var cp = m_context.Comp;
            // 切割翼板
            do {
                // 仅当腹板法线与主零件长度方向平行时切割翼板
                if (!m_context.MainDirection.IsParallelTo(m_context.SubPlanes.FrontWeb.Normal)) {
                    break;
                }
                Plane leftPlane = m_context.SubNearBound.Clone();
                Plane rightPlane = leftPlane.Invert();
                if (frontDist > backDist) {
                    leftPlane.MoveThis(frontDist);
                    rightPlane.MoveThis(-backDist);
                } else {
                    leftPlane.MoveThis(backDist);
                    rightPlane.MoveThis(-frontDist);
                }
                rightPlane.MoveThis(m_flangeCutGap);
                //
                Plane zBottomPlane = m_context.SubPlanes.FrontBound.Move(FLANGE_CUT_MARGIN);
                Plane zTopPlane = m_context.SubPlanes.BackBound.Move(FLANGE_CUT_MARGIN);
                // 切割上翼板
                {
                    Plane topPlane = m_context.SubPlanes.TopFlange.Move(FLANGE_CUT_MARGIN);
                    Plane bottomPlane = m_context.SubPlanes.TopFlangeInside.Move(FLANGE_CUT_MARGIN);
                    Plane[] sidePlanes = { leftPlane, bottomPlane, rightPlane, topPlane };
                    cp.CreateCutGeo(m_context.SubPart, zBottomPlane, zTopPlane, sidePlanes, true);
                }
                // 切割下翼板
                {
                    Plane topPlane = m_context.SubPlanes.BottomFlange.Move(FLANGE_CUT_MARGIN);
                    Plane bottomPlane = m_context.SubPlanes.BottomFlangeInside.Move(FLANGE_CUT_MARGIN);
                    Plane[] sidePlanes = { leftPlane, bottomPlane, rightPlane, topPlane };
                    cp.CreateCutGeo(m_context.SubPart, zBottomPlane, zTopPlane, sidePlanes, true);
                }
            } while (false);

            // 切割腹板及突出的翼板部分
            do
            {
                Plane zTopPlane = m_context.SubPlanes.TopFlange.Move(FLANGE_CUT_MARGIN);
                Plane zBottomPlane = m_context.SubPlanes.BottomFlange.Move(FLANGE_CUT_MARGIN);
                Plane cutPlane0 = m_context.SubAlignPlane.Move(FLANGE_CUT_MARGIN);
                Plane cutPlane1 = null;
                Plane cutPlane2 = null;
                if (frontDist > backDist)
                {
                    cutPlane1 = m_context.SubPlanes.FrontBound.Move(FLANGE_CUT_MARGIN);
                    cutPlane2 = m_context.SubNearBound.Invert();
                    cutPlane2.MoveThis(-backDist);
                    cutPlane2.MoveThis(FLANGE_CUT_MARGIN);
                }
                else
                {
                    cutPlane1 = m_context.SubPlanes.BackBound.Move(FLANGE_CUT_MARGIN);
                    cutPlane2 = m_context.SubNearBound.Invert();
                    cutPlane2.MoveThis(-frontDist);
                    cutPlane2.MoveThis(FLANGE_CUT_MARGIN);
                }
                Plane[] sidePlanes = { cutPlane0, cutPlane1, cutPlane2 };
                cp.CreateCutGeo(m_context.SubPart, zBottomPlane, zTopPlane, sidePlanes, true);
            } while (false);
        }

        private List<Line> ParseBeamSideLines(BeamPlanes partPlanes)
        {
            List<Line> sideLines = new List<Line>();

            Line curLine = partPlanes.FrontBound.Intersect(partPlanes.TopBound);
            if (curLine.Length > 0.0)
            {
                sideLines.Add(curLine);
            }
            curLine = partPlanes.FrontBound.Intersect(partPlanes.BottomBound);
            if (curLine.Length > 0.0)
            {
                sideLines.Add(curLine);
            }
            curLine = partPlanes.BackBound.Intersect(partPlanes.TopBound);
            if (curLine.Length > 0.0)
            {
                sideLines.Add(curLine);
            }
            curLine = partPlanes.BackBound.Intersect(partPlanes.BottomBound);
            if (curLine.Length > 0.0)
            {
                sideLines.Add(curLine);
            }

            return sideLines;
        }

    }
}
