﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners182
{
    
    public class WeldHandler
    {

        private ComponentInstanceContext m_context;

        private ParamsHandler m_paramsHandler = null;

        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i=0; i<ParamNames.WELD_PROPS_LIST.Length; ++i) {
                m_weldProps.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            if (!this.ParseProps())
                return;

            this.CreateWelds();
        }

        private bool ParseProps()
        {
            for (int i=0; i<ParamNames.WELD_PROPS_LIST.Length; ++i) {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }

        private void CreateWelds()
        {
            //
            m_context.ParseAwaySidePlane(
                    m_context.MainPlanes.FrontWeb, m_context.MainPlanes.BackWeb,
                    out Plane nsMainWeb, out Plane fsMainWeb);

            //
            Vec3 xOrient = m_context.MainNearBound.Normal;
            Vec3 zOrient = m_context.MainDirection.Along(m_context.Comp.WorldMatrix.Z);
            Vec3 yOrient = zOrient.Cross(xOrient);
            Mat43 workAxis = new Mat43(xOrient, yOrient, zOrient, Vec3.ZERO);
            // 1.腋板与次零件之间的焊接
            do {
                if (m_weldProps[0].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                if (m_context.HaunchPlates.Count == 0)
                    break;
                foreach (var curPlate in m_context.HaunchPlates) {
                    if (curPlate == null)
                        continue;

                    //
                    Vec3 partCenter = Vec3.ZERO;
                    var partCtrls = curPlate.CtrlPoints;
                    if (partCtrls == null || partCtrls.Length < 0)
                        continue;
                    for (int i=0; i<partCtrls.Length; ++i) {
                        partCenter += partCtrls[i].WorldPos;
                    }
                    partCenter /= (double)partCtrls.Length;

                    //
                    Plane nearPlane = m_context.SubPlanes.TopFlange;
                    if (nearPlane.SignedDist(partCenter) < 0.0) {
                        nearPlane = m_context.SubPlanes.BottomFlange;
                    }
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, nearPlane);
                    if (weldPoints == null)
                        continue;
                    Weld curWeld = m_context.Comp.CreatePolyWeld(m_context.SubPart, curPlate, weldPoints.ToArray(), true);
                    m_weldProps[0].ApplyToWeld(curWeld);
                }
            } while (false);
            // 2.连接板与主零件之间的焊接
            do {
                if (m_weldProps[1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                for (int i = 0; i < m_context.ShearPlates.Count; ++i) {
                    var curPlate = m_context.ShearPlates[i];
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, m_context.ShearWeldPlanes[0]);
                    if (weldPoints == null)
                        break;
                    Weld curWeld;
                    if (!m_context.IsMainRotate && m_context.NearDoublePlate != null)
                    {
                        curWeld = m_context.Comp.CreateAutoWeld(curPlate, m_context.NearDoublePlate, LoongCAD.STC.Define.SpaceAxis.AXIS_NX, true);
                        curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                        curWeld.WorkAxis = workAxis;
                    }
                    else
                        curWeld = m_context.Comp.CreatePolyWeld(m_context.MainPart, curPlate, weldPoints.ToArray(), true);
                    m_weldProps[1].ApplyToWeld(curWeld);
                }
            } while (false);
            // 3/8.次梁和下/上加劲的焊接
            do {
                double gap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_END_GAP);
                if (gap > 5)
                    break;
                string[] plateKeys = { StiffPlanes.BOTTOM_NS_KEY, StiffPlanes.TOP_NS_KEY };
                int[] weldProps = { 2, 7 };
                Plane[] OutsideCutPlane = { m_context.SubPlanes.TopFlange, m_context.SubPlanes.BottomFlange };
                Plane[] InsideCutPlane = { m_context.SubPlanes.TopFlangeInside, m_context.SubPlanes.BottomFlangeInside };
                //柱的翼缘面朝向次梁
                WeldReadyMode weldReady = (WeldReadyMode)m_paramsHandler.GetIntValue(ParamNames.WELD_READY_MODE);
                if (m_context.IsMainRotate)
                {
                    //计算柱和梁的前后面、上下面
                    Plane columnFront = m_context.MainPlanes.FrontBound;
                    Plane columnBack = m_context.MainPlanes.BackBound;
                    Plane beamFront, beamBack, beamTop, beamBottom;
                    if (m_context.IsSubRotate)
                    {
                        beamFront = m_context.SubPlanes.TopBound.Clone();
                        beamBack = m_context.SubPlanes.BottomBound.Clone();
                        beamTop = m_context.SubPlanes.FrontBound.Clone();
                        beamBottom = m_context.SubPlanes.BackBound.Clone();
                    }
                    else
                    {
                        beamFront = m_context.SubPlanes.FrontBound.Clone();
                        beamBack = m_context.SubPlanes.BackBound.Clone();
                        beamTop = m_context.SubPlanes.TopBound.Clone();
                        beamBottom = m_context.SubPlanes.BottomBound.Clone();
                    }
                    if (beamFront.Normal.Dot(columnFront.Normal) < 0)
                        Util.Swap(ref beamFront, ref beamBack);
                    if (beamTop.Normal.Dot(zOrient) < 0)
                        Util.Swap(ref beamTop, ref beamBottom);
                    //焊接位置计算
                    List<Vec3> topWeldPoint = new List<Vec3>();
                    List<Vec3> bottomWeldPoint = new List<Vec3>();
                    if (!m_context.IsSubRotate)
                    {
                        if (beamFront.SignedDist(columnFront.Origin) > 0)
                        {
                            topWeldPoint.Add(beamFront.Intersect(beamTop, m_context.SubAlignPlane));
                            bottomWeldPoint.Add(beamFront.Intersect(beamBottom, m_context.SubAlignPlane));
                        }
                        else
                        {
                            topWeldPoint.Add(columnFront.Intersect(beamTop, m_context.SubAlignPlane));
                            bottomWeldPoint.Add(columnFront.Intersect(beamBottom, m_context.SubAlignPlane));
                        }
                        if (beamBack.SignedDist(columnBack.Origin) > 0)
                        {
                            topWeldPoint.Add(beamBack.Intersect(beamTop, m_context.SubAlignPlane));
                            bottomWeldPoint.Add(beamBack.Intersect(beamBottom, m_context.SubAlignPlane));
                        }
                        else
                        {
                            topWeldPoint.Add(columnBack.Intersect(beamTop, m_context.SubAlignPlane));
                            bottomWeldPoint.Add(columnBack.Intersect(beamBottom, m_context.SubAlignPlane));
                        }
                    }
                    else
                    {
                        if (beamFront.SignedDist(columnFront.Origin) >= -1e-4)
                        {
                            topWeldPoint.Add(beamFront.Intersect(beamTop, m_context.SubAlignPlane));
                            topWeldPoint.Add(beamFront.Intersect(beamBottom, m_context.SubAlignPlane));
                        }
                        if (beamBack.SignedDist(columnBack.Origin) >= -1e-4)
                        {
                            bottomWeldPoint.Add(beamBack.Intersect(beamTop, m_context.SubAlignPlane));
                            bottomWeldPoint.Add(beamBack.Intersect(beamBottom, m_context.SubAlignPlane));
                        }
                    }
                    //根据板的切割体重新计算焊接控制点
                    if (topWeldPoint.Count == 2)
                    {
                        foreach (var cutPlane in m_context.TopWeldPlacePlane)
                        {
                            Vec3 intersect = Util.PlaneLineIntersect(cutPlane, topWeldPoint[0], topWeldPoint[1]);
                            if (cutPlane.SignedDist(topWeldPoint[0]) > 0)
                                topWeldPoint[0] = intersect;
                            else
                                topWeldPoint[1] = intersect;
                        }
                    }
                    if (bottomWeldPoint.Count == 2)
                    {
                        foreach (var cutPlane in m_context.BottomWeldPlacePlane)
                        {
                            Vec3 intersect = Util.PlaneLineIntersect(cutPlane, bottomWeldPoint[0], bottomWeldPoint[1]);
                            if (cutPlane.SignedDist(bottomWeldPoint[0]) > 0)
                                bottomWeldPoint[0] = intersect;
                            else
                                bottomWeldPoint[1] = intersect;
                        }
                    }
                    //创建焊缝
                    List<Vec3>[] weldPosition = { bottomWeldPoint, topWeldPoint };
                    for (int i = 0; i < 2; i++)
                    {
                        if (m_weldProps[weldProps[i]].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                            continue;
                        //开坡口
                        if (weldReady == WeldReadyMode.OPEN || weldReady == WeldReadyMode.AUTO)
                        {
                            WeldPrepareCut(m_weldProps[weldProps[i]], OutsideCutPlane[i], InsideCutPlane[i], m_context.SubAlignPlane.Invert(),
                            m_context.SubPlanes.FrontBound, m_context.SubPlanes.BackBound, m_context.SubPart);
                        }
                        if (weldPosition[i].Count < 2)
                            continue;
                        Weld curWeld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.SubPart, weldPosition[i].ToArray(), true);
                        //Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, m_context.SubPart, spaceAxis[i], true);
                        m_weldProps[weldProps[i]].ApplyToWeld(curWeld);
                    }
                }
                //柱的腹板面朝向次梁
                else
                {
                    for (int i = 0; i < 2; i++)
                    {
                        if (m_weldProps[weldProps[i]].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                            continue;
                        //开坡口
                        if (weldReady == WeldReadyMode.OPEN)
                        {
                            WeldPrepareCut(m_weldProps[weldProps[i]], OutsideCutPlane[i], InsideCutPlane[i], m_context.SubAlignPlane.Invert(),
                                m_context.SubPlanes.FrontBound, m_context.SubPlanes.BackBound, m_context.SubPart);
                        }
                        if (!m_context.StiffMap.ContainsKey(plateKeys[i]))
                        {
                            continue;
                        }
                        PartBase curPlate = m_context.StiffMap[plateKeys[i]];
                        if (curPlate != null)
                        {
                            Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.SubPart, curPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                            curWeld.WorkAxis = workAxis;
                            curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                            m_weldProps[weldProps[i]].ApplyToWeld(curWeld);
                        }
                    }
                }
            } while (false);

            //加劲肋的焊接
            Vec3 stiffWeldX = nsMainWeb.Normal;
            Vec3 topVec = m_context.TopNearStiffPlanes != null ? 
                m_context.TopNearStiffPlanes.ZTopPlane.Normal : m_context.MainDirection.Along(m_context.Comp.WorldMatrix.Z);
            Vec3 stiffWeldY = topVec.Cross(m_context.MainPlanes.BottomFlangeInside.Normal).Cross(topVec);
            Mat43 stiffWeldAxis = new Mat43(stiffWeldX, stiffWeldY, stiffWeldX.Cross(stiffWeldY), Vec3.ZERO);
            // 4.上加劲和柱翼缘焊接
            do
            {
                WeldProperties weldProps = m_weldProps[3];
                if (weldProps.TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                string[] plateKeys = { StiffPlanes.TOP_NS_KEY, StiffPlanes.TOP_FS_KEY };
                for(int plateIndex = 0; plateIndex < 2; plateIndex++)
                {
                    if (!m_context.StiffMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKeys[plateIndex]];
                    if (m_context.StiffPlanesMap.ContainsKey(plateKeys[plateIndex]))
                    // 上翼板处焊接
                    {
                        Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, curPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_Y, true);
                        curWeld.WorkAxis = stiffWeldAxis;
                        curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Y;
                        weldProps.ApplyToWeld(curWeld);
                    }
                    // 下翼板处焊接
                    {
                        Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, curPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_NY, true);
                        curWeld.WorkAxis = stiffWeldAxis;
                        curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                        weldProps.ApplyToWeld(curWeld);
                    }
                    if (m_context.StiffPlanesMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        StiffFlangeCutter(m_context.StiffPlanesMap[plateKeys[plateIndex]], weldProps);
                    }
                }
            } while (false);
            // 5.上加劲和柱腹板焊接
            do
            {
                WeldProperties weldProps = m_weldProps[4];
                if (weldProps.TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                string[] plateKeys = { StiffPlanes.TOP_NS_KEY, StiffPlanes.TOP_FS_KEY };
                LoongCAD.STC.Define.SpaceAxis[] spaceAxis = { LoongCAD.STC.Define.SpaceAxis.AXIS_NX, LoongCAD.STC.Define.SpaceAxis.AXIS_X };
                for (int plateIndex = 0; plateIndex < 2; plateIndex++)
                {
                    if (!m_context.StiffMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKeys[plateIndex]];
                    {
                        Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, curPlate, spaceAxis[plateIndex], true);
                        weldProps.ApplyToWeld(curWeld);
                        curWeld.WorkAxis = stiffWeldAxis;
                        curWeld.AutoWeldPosition = spaceAxis[plateIndex];
                        curWeld.Refresh();
                    }
                    if (m_context.StiffPlanesMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        StiffWebCutter(m_context.StiffPlanesMap[plateKeys[plateIndex]], weldProps);
                    }
                }
            } while (false);
            // 6.下加劲和柱翼缘焊接
            do
            {
                WeldProperties weldProps = m_weldProps[5];
                if (weldProps.TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                string[] plateKeys = { StiffPlanes.BOTTOM_NS_KEY, StiffPlanes.BOTTOM_FS_KEY };
                for (int plateIndex = 0; plateIndex < 2; plateIndex++)
                {
                    if (!m_context.StiffMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKeys[plateIndex]];
                    // 上翼板处焊接
                    {
                        Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, curPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_Y, true);
                        curWeld.WorkAxis = stiffWeldAxis;
                        curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Y;
                        weldProps.ApplyToWeld(curWeld);
                    }
                    // 下翼板处焊接
                    {
                        Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, curPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_NY, true);
                        curWeld.WorkAxis = stiffWeldAxis;
                        curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                        weldProps.ApplyToWeld(curWeld);
                    }
                    if (m_context.StiffPlanesMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        StiffFlangeCutter(m_context.StiffPlanesMap[plateKeys[plateIndex]], weldProps);
                    }
                }
            } while (false);
            // 7.下加劲和柱腹板焊接
            do
            {
                WeldProperties weldProps = m_weldProps[6];
                if (weldProps.TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                string[] plateKeys = { StiffPlanes.BOTTOM_NS_KEY, StiffPlanes.BOTTOM_FS_KEY };
                LoongCAD.STC.Define.SpaceAxis[] spaceAxis = { LoongCAD.STC.Define.SpaceAxis.AXIS_NX, LoongCAD.STC.Define.SpaceAxis.AXIS_X };
                for (int plateIndex = 0; plateIndex < 2; plateIndex++)
                {
                    if (!m_context.StiffMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKeys[plateIndex]];
                    {
                        Weld curWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, curPlate, spaceAxis[plateIndex], true);
                        curWeld.WorkAxis = stiffWeldAxis;
                        curWeld.AutoWeldPosition = spaceAxis[plateIndex];
                        weldProps.ApplyToWeld(curWeld);
                    }
                    if (m_context.StiffPlanesMap.ContainsKey(plateKeys[plateIndex]))
                    {
                        StiffWebCutter(m_context.StiffPlanesMap[plateKeys[plateIndex]], weldProps);
                    }
                }
            } while (false);

            // 9.连接板和上、下加劲的焊接
            do {
                if (m_context.IsMainRotate)
                    break;
                //获取上、下加劲
                PartBase[] stiffs = { null, null};
                if (m_context.StiffMap.ContainsKey(StiffPlanes.TOP_NS_KEY))
                    stiffs[0] = m_context.StiffMap[StiffPlanes.TOP_NS_KEY];
                if (m_context.StiffMap.ContainsKey(StiffPlanes.BOTTOM_NS_KEY))
                    stiffs[1] = m_context.StiffMap[StiffPlanes.BOTTOM_NS_KEY];
                //上下焊缝焊缝的探测方向不一样
                LoongCAD.STC.Define.SpaceAxis[] spaceAxis = { LoongCAD.STC.Define.SpaceAxis.AXIS_Z, LoongCAD.STC.Define.SpaceAxis.AXIS_NZ };
                for (int i = 0; i < m_context.ShearPlates.Count; ++i)
                {
                    var connPlate = m_context.ShearPlates[i];
                    for (int j = 0; j < 2; j++)
                    {
                        if (m_weldProps[8].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                            continue;
                        PartBase stiff = stiffs[j];
                        if (stiff != null && connPlate != null)
                        {
                            Weld curWeld = m_context.Comp.CreateAutoWeld(stiff, connPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                            curWeld.WorkAxis = workAxis;
                            curWeld.AutoWeldPosition = spaceAxis[j];
                            m_weldProps[8].ApplyToWeld(curWeld);
                        }
                    }
                }
            } while (false);
            // 10.双板和柱的焊接
            do
            {
                if (m_weldProps[9].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                List<PartBase> DoublePlates = new List<PartBase>() { m_context.NearDoublePlate, m_context.FarDoublePlate };
                if (DoublePlates.Count == 0)
                    break;

                foreach (var curPlate in DoublePlates)
                {
                    if (curPlate == null)
                        continue;

                    var partCtrls = curPlate.CtrlPoints;
                    if (partCtrls == null || partCtrls.Length < 2)
                        continue;
                    for (int i = 0; i < partCtrls.Length; ++i)
                    {
                        Vec3 start = partCtrls[i].WorldPos;
                        Vec3 end = partCtrls[(i + 1) % partCtrls.Length].WorldPos;
                        Weld curWeld = m_context.Comp.CreatePolyWeld(m_context.MainPart, curPlate, new Vec3[] { start, end }, true);
                        m_weldProps[9].ApplyToWeld(curWeld);
                    }
                }
            } while (false);
        }

        private void StiffWebCutter(StiffPlanes stiffPlanes, WeldProperties weldProps)
        {
            if (stiffPlanes == null || stiffPlanes.StiffKey == "")
                return;
            if (!m_context.StiffMap.ContainsKey(stiffPlanes.StiffKey))
                return;
            PartBase cutPart = m_context.StiffMap[stiffPlanes.StiffKey];
            //内侧
            Plane ctrlTopPlane = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[0]);
            Plane ctrlBottomPlane = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[2]);
            //靠近近侧腹板面的加劲肋侧面
            Plane webSidePlane1 = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[1]);
            Plane webSidePlane2 = Util.VerticalPlane(stiffPlanes.ZBottomPlane, stiffPlanes.RefPlanes[1]);
            Plane webSidePlane = webSidePlane1.SignedDist(webSidePlane2.Origin) > 0 ? webSidePlane1 : webSidePlane2;
            WeldPrepareCut(weldProps, stiffPlanes.ZTopPlane, stiffPlanes.ZBottomPlane, webSidePlane,
                ctrlTopPlane, ctrlBottomPlane, cutPart);
        }

        private void StiffFlangeCutter(StiffPlanes stiffPlanes, WeldProperties weldProps)
        {
            if (stiffPlanes == null || stiffPlanes.StiffKey == "")
                return;
            if (!m_context.StiffMap.ContainsKey(stiffPlanes.StiffKey))
                return;
            PartBase cutPart = m_context.StiffMap[stiffPlanes.StiffKey];
            //
            Plane ctrlTopPlane = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[1]);
            Plane ctrlBottomPlane = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[3]);
            //靠近上翼缘的加劲肋侧面
            Plane topFlangeSidePlane1 = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[0]);
            Plane topFlangeSidePlane2 = Util.VerticalPlane(stiffPlanes.ZBottomPlane, stiffPlanes.RefPlanes[0]);
            Plane topFlangeSidePlane = topFlangeSidePlane1.SignedDist(topFlangeSidePlane2.Origin) > 0 ? topFlangeSidePlane1 : topFlangeSidePlane2;
            //靠近下翼缘的加劲肋侧面
            Plane bottomFlangeSidePlane1 = Util.VerticalPlane(stiffPlanes.ZTopPlane, stiffPlanes.RefPlanes[2]);
            Plane bottomFlangeSidePlane2 = Util.VerticalPlane(stiffPlanes.ZBottomPlane, stiffPlanes.RefPlanes[2]);
            Plane bottomFlangeSidePlane = bottomFlangeSidePlane1.SignedDist(bottomFlangeSidePlane2.Origin) > 0 ? bottomFlangeSidePlane1 : bottomFlangeSidePlane2;
            //创建
            WeldPrepareCut(weldProps, stiffPlanes.ZTopPlane, stiffPlanes.ZBottomPlane, topFlangeSidePlane,
                ctrlTopPlane, ctrlBottomPlane, cutPart);
            WeldPrepareCut(weldProps, stiffPlanes.ZTopPlane, stiffPlanes.ZBottomPlane, bottomFlangeSidePlane,
                ctrlTopPlane, ctrlBottomPlane, cutPart);
        }

        /// <summary>
        /// 用于创建焊接准备中的切割体
        /// </summary>
        /// <param name="weldProps">焊缝属性</param>
        /// <param name="topPlane">上侧坡口的侧面</param>
        /// <param name="bottomPlane">上侧坡口的侧面</param>
        /// <param name="frontPlane">板、梁需要创建切割体的那一端</param>
        /// <param name="ctrlTopPlane">切割体上侧控制面</param>
        /// <param name="ctrlBottomPlane">切割体下侧控制面</param>
        /// <param name="part">被切割的零件</param>
        private void WeldPrepareCut(WeldProperties weldProps, Plane topPlane, Plane bottomPlane, Plane frontPlane,
            Plane ctrlTopPlane, Plane ctrlBottomPlane, PartBase part)
        {
            Define.WeldType typeUp = weldProps.TypeUp;
            Define.WeldType typeDown = weldProps.TypeDown;
            Define.WeldContour contourUp = weldProps.ContourUp;
            Define.WeldContour contourDown = weldProps.ContourDown;
            double vertSideUp = weldProps.SizeUp, vertSideDown = weldProps.SizeDown, horiSideUp = vertSideUp, horiSideDown = vertSideDown;

            //有效判断
            bool upVaild = ((typeUp != Define.WeldType.WELD_TYPE_FILLET && typeUp != Define.WeldType.WELD_TYPE_NONE) || contourUp != Define.WeldContour.WELD_CONTOUR_NONE);
            bool downVaild = ((typeDown != Define.WeldType.WELD_TYPE_FILLET && typeDown != Define.WeldType.WELD_TYPE_NONE) || contourDown != Define.WeldContour.WELD_CONTOUR_NONE);
            if (!upVaild)
                vertSideUp = 0;
            if (!downVaild)
                vertSideDown = 0;
            if (upVaild && typeDown == Define.WeldType.WELD_TYPE_NONE && contourDown == Define.WeldContour.WELD_CONTOUR_NONE)
                vertSideUp *= 2;
            if (!upVaild && downVaild && weldProps.SizeDown != double.MaxValue)
                vertSideDown *= 2;

            //尺寸处理
            if (weldProps.SizeUp != double.MaxValue)
                vertSideUp = weldProps.SizeUp;
            else
                weldProps.SizeUp = 6;
            if (weldProps.SizeDown != double.MaxValue)
                vertSideDown = weldProps.SizeDown;
            else
                weldProps.SizeDown = 6;

            //角度处理
            if (weldProps.GrooveAngleUp > 0 && weldProps.GrooveAngleUp < 90)
                horiSideUp = Math.Tan(weldProps.GrooveAngleUp * Math.PI / 180) * vertSideUp;
            else
                horiSideUp = vertSideUp;
            if (weldProps.GrooveAngleDown > 0 && weldProps.GrooveAngleDown < 90)
                horiSideDown = Math.Tan(weldProps.GrooveAngleDown * Math.PI / 180) * vertSideDown;
            else
                horiSideDown = vertSideDown;

            //执行切割操作
            double[] horiCutSize = { horiSideUp, horiSideDown };
            double[] vertCutSize = { vertSideUp, vertSideDown };
            Plane[] triSidePlane = { topPlane, bottomPlane };
            bool[] isCreate = { upVaild, downVaild };
            for (int i = 0; i < 2; i++)
            {
                if (horiCutSize[i] <= 0 || vertCutSize[i] <= 0 || !isCreate[i])
                    continue;
                //三角形切割体斜面的创建
                Vec3 tempVec = triSidePlane[i].Normal.Cross(frontPlane.Normal);
                Vec3 acute1MoveVector = tempVec.Cross(frontPlane.Normal).Normalize();
                Vec3 acute2MoveVector = triSidePlane[i].Normal.Cross(tempVec).Normalize();
                Vec3 rightPoint = ctrlBottomPlane.Intersect(triSidePlane[i], frontPlane);
                Vec3 acutePoint1 = rightPoint + acute1MoveVector * horiCutSize[i]; //side是直角三角形的直角边长
                Vec3 acutePoint2 = rightPoint + acute2MoveVector * vertCutSize[i];
                Plane sidePlane3 = new Plane(acutePoint1, acutePoint2 - acutePoint1, ctrlBottomPlane.Normal);
                //
                PartBase cutPartBase = m_context.Comp.CreatePlateCutter(part, ctrlBottomPlane.Move(0.1), ctrlTopPlane.Move(0.1), new Plane[] { triSidePlane[i], frontPlane, sidePlane3 }, true);
                cutPartBase.Flush();
            }
        }

    }

}
