﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ColumnWithStiffeners186
{
    
    public class DoubleHandler
    {

        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PlateProperties m_plateProps = new PlateProperties();

        private bool m_creatingNearPlate = false;
        private bool m_creatingFarPlate = false;
        
        private bool m_cuttingPlateEdge = false;
        private double m_cuttingAngle = double.MaxValue;

        private double m_holeDiameter = double.MaxValue;
        private double m_slotWidth = double.MaxValue;
        private double m_slotHeight = double.MaxValue;

        private int m_holeHoriCount = 0;
        private int m_holeVertCount = 0;
        private List<double> m_holeHoriStride = new List<double>();
        private List<double> m_holeVertStride = new List<double>();
        private double m_holeLeftMargin = double.MaxValue;
        private double m_holeRightMargin = double.MaxValue;
        private double m_holeTopMargin = double.MaxValue;
        private double m_holeBottomMargin = double.MaxValue;
        private double m_flangeGap = double.MaxValue;
        private double m_bottomGap = double.MaxValue;

        private Plane m_leftPlane;
        private Plane m_rightPlane;
        private Plane m_topPlane;
        private Plane m_bottomPlane;

        // 主零件长度方向的下方向
        private Vec3 m_downNormal = -Vec3.AXIS_Z;

        public DoubleHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            if (!this.ParseProps())
                return;
            if (!this.ParseSidePlanes())
                return;

            // 解析主零件远近腹板面
            Plane nsWebPlane;
            Plane fsWebPlane;
            m_context.ParseAwaySidePlane(
                m_context.MainPlanes.FrontWeb, m_context.MainPlanes.BackWeb,
                out nsWebPlane, out fsWebPlane);

            Plane[] sidePlanes = new Plane[] { m_leftPlane, m_bottomPlane, m_rightPlane, m_topPlane };
            
            // 创建双板
            PartBase nsPlate = null;
            PartBase fsPlate = null;
            if (m_creatingNearPlate) {
                nsPlate = m_context.Comp.GetOrCreateSteelPlate(ObjectUsage.NEAR_DOUBLE_PLATE,
                    nsWebPlane, nsWebPlane.Move(m_plateProps.Thickness), sidePlanes);
                if (nsPlate != null) {
                    m_context.DoublePlates.Add(nsPlate);
                }
            }
            if (m_creatingFarPlate) {
                fsPlate = m_context.Comp.GetOrCreateSteelPlate(ObjectUsage.FAR_DOUBLE_PLATE,
                    fsWebPlane, fsWebPlane.Move(m_plateProps.Thickness), sidePlanes);
                if (fsPlate != null) {
                    m_context.DoublePlates.Add(fsPlate);
                }
            }

            // 分析主零件连接面处是否需要为连接板留出间隙
            const double eps = 1e-3;
            if (m_context.MainJointPlane.Normal.IsParallelTo(nsWebPlane.Normal) &&
                m_context.MainJointPlane.Dist(nsWebPlane.Origin) < eps && nsPlate != null) {
                m_context.MainJointMargin = m_plateProps.Thickness;
            }
            else if (m_context.MainJointPlane.Normal.IsParallelTo(fsWebPlane.Normal) &&
                m_context.MainJointPlane.Dist(fsWebPlane.Origin) < eps && fsPlate != null) {
                m_context.MainJointMargin = m_plateProps.Thickness;
            }

            //
            Plane plateCenterPlane = m_topPlane.Move(-0.5 * m_topPlane.Dist(m_bottomPlane.Origin));
            this.CutPlateEdges(nsPlate, nsWebPlane, plateCenterPlane, new Plane[] { m_leftPlane, m_rightPlane});
            this.CutPlateEdges(fsPlate, fsWebPlane, plateCenterPlane, new Plane[] { m_leftPlane, m_rightPlane });
            this.CreateHoles(nsPlate, fsPlate);
        }

        private bool ParseProps()
        {
            //
            {
                LoongCAD.Util.DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.DOUBLE_PLATE_PROPS);
                m_plateProps.SetProps(plateProps);
            }

            //
            {
                int createIndex = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_CREATION_MODE);
                switch (createIndex) {
                case 1: { // None
                        m_creatingNearPlate = false;
                        m_creatingFarPlate = false;
                    } break;
                case 2: {
                        m_creatingNearPlate = false;
                        m_creatingFarPlate = true;
                    } break;
                case 3: {
                        m_creatingNearPlate = true;
                        m_creatingFarPlate = false;
                    } break;
                case 4: {
                        m_creatingNearPlate = true;
                        m_creatingFarPlate = true;
                    } break;
                case 0:
                default: {
                        m_creatingNearPlate = false;
                        m_creatingFarPlate = false;
                        System.Diagnostics.Debug.Assert(false, "无效的双板创建模式");
                    } break;
                }
            }

            //
            {
                int shapeIndex = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_PLATE_EDGE_SHAPE);
                switch (shapeIndex) {
                case 1: {
                        m_cuttingPlateEdge = true;
                    } break;
                case 2: {
                        m_cuttingPlateEdge = false;
                    } break;
                case 0:
                default: {
                        m_cuttingPlateEdge = false;
                        System.Diagnostics.Debug.Assert(false, "无效的双板边缘形状选项");
                    } break;
                }
                m_cuttingAngle = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_PLATE_BEVEL_ANGLE);
            }

            //
            m_holeDiameter = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_DIAMETER);
            m_slotWidth = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_SLOT_WIDTH);
            m_slotHeight = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_SLOT_HEIGHT);

            m_flangeGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_FLANGE_GAP);
            m_bottomGap = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_BOTTOM_GAP);
            m_holeLeftMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_LEFT_MARGIN);
            m_holeRightMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_RIGHT_MARGIN);
            m_holeTopMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_TOP_MARGIN);
            m_holeBottomMargin = m_paramsHandler.GetDoubleValue(ParamNames.DOUBLE_HOLE_BOTTOM_MARGIN);

            // 解析孔排布信息
            do {
                m_holeHoriCount = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_HOLE_HORI_COUNT);
                m_holeVertCount = m_paramsHandler.GetIntValue(ParamNames.DOUBLE_HOLE_VERT_COUNT);
                if (m_holeHoriCount == 0 || m_holeVertCount == 0)
                    break;

                //
                {
                    double[] valList = m_paramsHandler.GetDoubleArrayValue(ParamNames.DOUBLE_HOLE_HORI_STRIDE);
                    if (m_holeHoriCount > 1 && valList != null && valList.Length > 0) {
                        for (int i = 1; i < m_holeHoriCount; ++i) {
                            m_holeHoriStride.Add(valList[Math.Min(i - 1, valList.Length - 1)]);
                        }
                    } else {
                        m_holeHoriStride.Add(0.0);
                    }
                }

                //
                {
                    double[] valList = m_paramsHandler.GetDoubleArrayValue(ParamNames.DOUBLE_HOLE_VERT_STRIDE);
                    if (m_holeVertCount > 1 && valList != null && valList.Length > 0) {
                        for (int i = 1; i < m_holeVertCount; ++i) {
                            m_holeVertStride.Add(valList[Math.Min(i - 1, valList.Length - 1)]);
                        }
                    } else {
                        m_holeVertStride.Add(0.0);
                    }
                }
            } while (false);
            
            // 应用自适应高度
            if (m_plateProps.Height == double.MaxValue) {
                m_plateProps.Height = m_holeTopMargin + m_holeBottomMargin;
                foreach (var curGap in m_holeVertStride) {
                    m_plateProps.Height += curGap;
                }
            }

            return true;
        }

        private bool ParseSidePlanes()
        {
            // 解析上下侧面
            Plane topPlane = null;
            Plane bottomPlane = null;
            do {
                // 解析次零件相对场景在下方的边界面
                Plane subBottomBound = null;
                {
                    double thresh = Math.Cos(89.0 * Math.PI / 180.0);
                    Vec3 refBottomDir;
                    m_downNormal = -m_context.MainDirection.Normalize();
                    // 主零件不是水平放置
                    // 以场景Z轴负方向为下方
                    if (Math.Abs(m_context.MainDirection.Normalize().Dot(Vec3.AXIS_Z)) > thresh) {
                        refBottomDir = -Vec3.AXIS_Z;
                        if (m_downNormal.Dot(Vec3.AXIS_Z) > 0.0) {
                            m_downNormal = -m_downNormal;
                        }
                    }
                    // 主零件水平放置
                    // 以主零件长度方向负方向为下方
                    else {
                        refBottomDir = -m_context.MainDirection;
                    }

                    // 分析次零件下边界面
                    double maxDot = 0.0;
                    // 顶边界面
                    {
                        double curDot = m_context.SubPlanes.TopBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.TopBound;
                        }
                    }
                    // 底边界面
                    {
                        double curDot = m_context.SubPlanes.BottomBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.BottomBound;
                        }
                    }
                    // 前边界面
                    {
                        double curDot = m_context.SubPlanes.FrontBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.FrontBound;
                        }
                    }
                    // 后边界面
                    {
                        double curDot = m_context.SubPlanes.BackBound.Normal.Normalize().Dot(refBottomDir);
                        if (curDot > maxDot) {
                            maxDot = curDot;
                            subBottomBound = m_context.SubPlanes.BackBound;
                        }
                    }
                }
                if (subBottomBound == null)
                    break;

                // 解析主零件两侧翼板之间的中心面
                Plane mainCenterPlane = m_context.MainPlanes.TopBound.Invert();
                mainCenterPlane.MoveThis(0.5 * mainCenterPlane.Dist(m_context.MainPlanes.BottomBound.Origin));
                Vec3 bottomOrigin = m_context.MainPlanes.FrontWeb.Intersect(
                    subBottomBound, mainCenterPlane);
                if (bottomOrigin == Vec3.MAX)
                    break;

                // 底侧面
                bottomPlane = new Plane(bottomOrigin, m_downNormal);
                bottomPlane.MoveThis(-m_bottomGap);

                // 顶侧面
                topPlane = bottomPlane.Invert();
                topPlane.MoveThis(m_plateProps.Height);
            } while (false);
            if (topPlane == null || bottomPlane == null)
                return false;

            // 解析左右侧面
            Plane leftPlane = null;
            Plane rightPlane = null;
            do {
                double xBoltSize = 0.0;
                for (int i=0; i<m_holeHoriStride.Count; ++i) {
                    xBoltSize += m_holeHoriStride[i];
                }
                //
                leftPlane = m_context.MainPlanes.TopFlange.Clone();
                rightPlane = m_context.MainPlanes.BottomFlange.Clone();
                Vec3 leftPt = topPlane.Intersect(leftPlane, m_context.MainPlanes.FrontWeb);
                Vec3 rightPt = topPlane.Intersect(rightPlane, m_context.MainPlanes.FrontWeb);
                //
                double flangeDist = (leftPt - rightPt).Length;
                // 自适应边宽度
                if (m_flangeGap == double.MaxValue) {
                    double curOffset = (xBoltSize - flangeDist) * 0.5;
                    leftPlane.MoveThis(curOffset);
                    rightPlane.MoveThis(curOffset);
                    leftPlane.MoveThis(m_holeLeftMargin);
                    rightPlane.MoveThis(m_holeRightMargin);
                } else {
                    leftPlane.MoveThis(-m_flangeGap);
                    rightPlane.MoveThis(-m_flangeGap);
                    m_holeLeftMargin = (flangeDist - 2.0 * m_flangeGap - xBoltSize) * 0.5;
                    m_holeRightMargin = m_holeLeftMargin;
                }
            } while (false);
            if (leftPlane == null || rightPlane == null) {
                m_leftPlane = null;
                m_rightPlane = null;
                return false;
            }

            m_leftPlane = leftPlane;
            m_rightPlane = rightPlane;
            m_topPlane = topPlane;
            m_bottomPlane = bottomPlane;
            return true;
        }

        private void CreateHoles(PartBase nsPlate, PartBase fsPlate)
        {
            if (m_topPlane == null || m_bottomPlane == null) {
                return;
            }
            // 计算螺栓Y轴面
            Plane yHolePlane = null;
            {
                Plane holeBottomPlane = m_bottomPlane.Move(-m_holeBottomMargin);
                Plane holeTopPlane = m_topPlane.Move(-m_holeTopMargin);
                yHolePlane = holeTopPlane.Move(-0.5 * holeTopPlane.Dist(holeBottomPlane.Origin));
            }

            // 解析螺栓坐标系
            Mat43 nsBoltAxis = Mat43.NULL;
            Mat43 fsBoltAxis = Mat43.NULL;
            Vec3 nsStartPos, nsEndPos;
            Vec3 fsStartPos, fsEndPos;
            {
                Vec3 boltZ = m_context.MainPlanes.BackWeb.Normal.Normalize();
                Vec3 boltY = boltZ.Cross(m_context.MainPlanes.BottomFlange.Normal).Normalize();
                Vec3 boltX = boltY.Cross(boltZ);
                if (boltY.Dot(m_downNormal) > 0) {
                    boltY *= -1.0;
                    boltZ *= -1.0;
                }
                double boltSizeX = 0.0;
                foreach (double curGap in m_holeHoriStride) {
                    boltSizeX += curGap;
                }
                //
                Plane holeLeftPlane = m_leftPlane.Move(-m_holeLeftMargin);
                nsStartPos = m_context.MainPlanes.BackWeb.Intersect(holeLeftPlane, yHolePlane);
                if (nsStartPos != Vec3.MAX) {
                    nsBoltAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
                }

                //
                double ctrlDistance = boltSizeX != 0.0 ? boltSizeX : Math.Max(1.0, m_holeDiameter);
                nsEndPos = nsStartPos + boltX * ctrlDistance;

                //
                fsStartPos = m_context.MainPlanes.BackWeb.Intersect(holeLeftPlane, yHolePlane);
                if (fsStartPos != Vec3.MAX) {
                    fsBoltAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
                }
                fsEndPos = fsStartPos + boltX * ctrlDistance;
            }

            //
            do {
                if (nsPlate == null || nsBoltAxis == Mat43.NULL) {
                    break;
                }

                BoltGroup curHoles = m_context.Comp.GetOrCreateBoltHoles(
                    ObjectUsage.NEAR_DOUBLE_PLATE_HOLES,
                    (PartBase)null, nsPlate, nsStartPos, nsEndPos);
                if (curHoles == null)
                    break;

                /// 创建螺栓之后需要首先设置其WorkAxis属性(若不为单位矩阵)再设置其他属性
                const double eps = 1.0;
                curHoles.WorkAxis = nsBoltAxis;
                curHoles.Shape = LoongCAD.STC.Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                curHoles.ArrayStrideX = m_holeHoriStride.ToArray();
                curHoles.ArrayStrideY = m_holeVertStride.ToArray();
                curHoles.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_TOP;
                curHoles.BoltSize = m_holeDiameter;
                curHoles.HoleX = m_slotWidth;
                curHoles.HoleY = m_slotHeight;
                curHoles.HoleType = LoongCAD.STC.Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                curHoles.OverSizeHoleEnable0 = true;
                curHoles.OverSizeHoleEnable1 = true;
                curHoles.CutLength = 2.0 * (m_context.MainJointThickness + m_plateProps.Thickness * 2.0) + eps;
                curHoles.Refresh();
            } while (false);

            //
            do {
                if (fsPlate == null || fsBoltAxis == Mat43.NULL) {
                    break;
                }
                BoltGroup curHoles = m_context.Comp.GetOrCreateBoltHoles(
                    ObjectUsage.FAR_DOUBLE_PLATE_HOLES,
                    (PartBase)null, fsPlate, fsStartPos, fsEndPos);
                if (curHoles == null)
                    break;

                /// 创建螺栓之后需要首先设置其WorkAxis属性(若不为单位矩阵)再设置其他属性
                const double eps = 1.0;
                curHoles.WorkAxis = fsBoltAxis;
                curHoles.Shape = LoongCAD.STC.Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                curHoles.ArrayStrideX = m_holeHoriStride.ToArray();
                curHoles.ArrayStrideY = m_holeVertStride.ToArray();
                curHoles.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_TOP;
                curHoles.BoltSize = m_holeDiameter;
                curHoles.HoleX = m_slotWidth;
                curHoles.HoleY = m_slotHeight;
                curHoles.HoleType = LoongCAD.STC.Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                curHoles.OverSizeHoleEnable0 = true;
                curHoles.OverSizeHoleEnable1 = true;
                curHoles.CutLength = 2.0 * (m_context.MainJointThickness + m_plateProps.Thickness * 2.0) + eps;
                curHoles.Refresh();
            } while (false);
        }

        private void CutPlateEdges(PartBase targetPlate, Plane mainWebPlane, Plane plateCenterPlane, Plane[] edgePlanes)
        {
            if (targetPlate == null || !m_cuttingPlateEdge)
                return;
            const double eps = 1e-3;
            if (m_cuttingAngle < eps || m_cuttingAngle > (90 - eps))
                return;

            //
            double theta = Math.PI * m_cuttingAngle / 180.0;
            Vec3 webNormal = mainWebPlane.Normal.Normalize();
            for (int i=0; i<edgePlanes.Length; ++i) {
                Vec3 edgeNormal = edgePlanes[i].Normal.Normalize();
                Vec3 edgeMiddlePt = edgePlanes[i].Intersect(plateCenterPlane, mainWebPlane);
                if (edgeMiddlePt == Vec3.MAX)
                    break;
                Vec3 cutNormal = Math.Cos(theta) * edgeNormal + Math.Sin(theta) * webNormal;
                Plane cutPlane = new Plane(edgeMiddlePt, -cutNormal);
                m_context.Comp.CreateCutPlane(targetPlate, cutPlane, true);
            }
        }

    }

}
