﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners186
{

    public class HaunchHandler
    {

        private const double HAUNCH_BACKUP_WIDTH = 50.0;
        private const double HAUNCH_BOLT_MARGIN = 10.0;
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private bool m_creatingTopHaunch = false;
        private bool m_creatingBottomHaunch = false;
        private double m_topChamferX = double.MaxValue;
        private double m_topChamferY = double.MaxValue;
        private double m_bottomChamferX = double.MaxValue;
        private double m_bottomChamferY = double.MaxValue;
        private PlateProperties m_topPlateProps = new PlateProperties();
        private PlateProperties m_bottomPlateProps = new PlateProperties();
        private double m_haunchEndMargin = 0.0;

        public HaunchHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute(BoltParameter boltParams)
        {
            if (!this.ParseProps(boltParams))
                return;

            if (m_creatingTopHaunch) {
                this.CreateHaunchPlate(ObjectUsage.TOP_HAUNCH, m_topPlateProps, boltParams,
                m_context.SubPlanes.FrontWeb, m_context.SubPlanes.TopFlange,
                m_topChamferX, m_topChamferY);
            }
            if (m_creatingBottomHaunch) {
                this.CreateHaunchPlate(ObjectUsage.BOTTOM_HAUNCH, m_bottomPlateProps, boltParams,
                m_context.SubPlanes.FrontWeb, m_context.SubPlanes.BottomFlange,
                m_bottomChamferX, m_bottomChamferY);
            }
        }

        private bool ParseProps(BoltParameter boltParams)
        {
            int createMode = m_paramsHandler.GetIntValue(ParamNames.HAUNCH_CREATION);
            switch (createMode) {
            // 根据螺栓位置自适应创建腋板
            // 若存在螺栓超出次零件上翼板面，则开启上腋板;
            // 若存在螺栓超出次零件下翼板面，则开启下腋板
            case 1: {
                    if (boltParams == null)
                        break;
                    if (boltParams.HoriCount == 0 || boltParams.VertCount == 0)
                        break;

                    // FIXME: 有待根据次零件在场景中的方位分析相对上下翼板面
                    Plane subTopFlange = m_context.SubPlanes.TopFlange;
                    Plane subBottomFlange = m_context.SubPlanes.BottomFlange;

                    //
                    List<Vec3> boltCornerPts = new List<Vec3>();
                    boltCornerPts.Add(boltParams.BoltPos);
                    boltCornerPts.Add(boltParams.BoltPos + boltParams.BoltDirX * boltParams.BoltSizeX);
                    boltCornerPts.Add(boltCornerPts[1] + boltParams.BoltDirY * boltParams.BoltSizeY);
                    boltCornerPts.Add(boltCornerPts[0] + boltParams.BoltDirY * boltParams.BoltSizeY);
                    for (int i=0; i<boltCornerPts.Count; ++i) {
                        if (subTopFlange != null &&
                            subTopFlange.SignedDist(boltCornerPts[i]) > 0.0) {
                            m_creatingTopHaunch = true;
                        }
                        if (subBottomFlange != null && 
                            subBottomFlange.SignedDist(boltCornerPts[i]) > 0.0) {
                            m_creatingBottomHaunch = true;
                        }
                    }
                }
                break;
            case 2: {
                    m_creatingTopHaunch = true;
                    m_creatingBottomHaunch = true;
                } break;
            default:
                break;
            }
            if (!m_creatingTopHaunch && !m_creatingBottomHaunch)
                return false;

            m_topChamferX = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_TOP_CHAMFER_X);
            m_topChamferY = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_TOP_CHAMFER_Y);
            m_bottomChamferX = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_BOTTOM_CHAMFER_X);
            m_bottomChamferY = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_BOTTOM_CHAMFER_Y);
            this.ParsePlateProps(boltParams);

            // 自适应折角大小
            if (m_topChamferX == double.MaxValue) {
                m_topChamferX = m_topPlateProps.Width * 0.25;
            }
            if (m_topChamferY == double.MaxValue) {
                m_topChamferY = m_topPlateProps.Width * 0.25;
            }
            if (m_bottomChamferX == double.MaxValue) {
                m_bottomChamferX = m_bottomPlateProps.Width * 0.25;
            }
            if (m_bottomChamferY == double.MaxValue) {
                m_bottomChamferY = m_topPlateProps.Width * 0.25;
            }

            //
            this.ParseEndMargin(boltParams);
            return true;
        }

        private bool ParsePlateProps(BoltParameter boltParams)
        {
            DObject topProps = m_paramsHandler.GetDObjectValue(ParamNames.HAUNCH_TOP_PROPS);
            DObject bottomProps = m_paramsHandler.GetDObjectValue(ParamNames.HAUNCH_BOTTOM_PROPS);

            if (topProps != null) {
                m_topPlateProps.SetProps(topProps);
                
                // 自适应宽度
                // 与次零件长度方向垂直的板边距
                if (m_topPlateProps.Width == double.MaxValue) {
                    double maxBoltDist = -double.MaxValue;
                    foreach (var curPoint in boltParams.CornerPoints) {
                        maxBoltDist = Math.Max(maxBoltDist,
                            m_context.SubPlanes.TopFlange.SignedDist(curPoint));
                    }
                    maxBoltDist += boltParams.BoltSize + boltParams.HoleTol + HAUNCH_BOLT_MARGIN;
                    m_topPlateProps.Width = Math.Max(maxBoltDist, HAUNCH_BACKUP_WIDTH);
                }
            } else {
                return false;
            }

            if (bottomProps != null) {
                m_bottomPlateProps.SetProps(bottomProps);

                // 自适应宽度
                // 与次零件长度方向垂直的板边距
                if (m_bottomPlateProps.Width == double.MaxValue) {
                    double maxBoltDist = -double.MaxValue;
                    foreach (var curPoint in boltParams.CornerPoints) {
                        maxBoltDist = Math.Max(maxBoltDist,
                            m_context.SubPlanes.BottomFlange.SignedDist(curPoint));
                    }
                    maxBoltDist += boltParams.BoltSize + boltParams.HoleTol + HAUNCH_BOLT_MARGIN;
                    m_bottomPlateProps.Width = Math.Max(maxBoltDist, HAUNCH_BACKUP_WIDTH);
                }
            } else {
                return false;
            }

            return true;
        }

        private void ParseEndMargin(BoltParameter boltParams)
        {
            Plane leftPlane = m_context.SubAlignPlane;
            Vec3 pt0 = m_context.SubPlanes.FrontWeb.Intersect(leftPlane, m_context.SubPlanes.TopFlange);
            Vec3 pt1 = m_context.SubPlanes.FrontWeb.Intersect(leftPlane, m_context.SubPlanes.BottomFlange);

            // 将腋板右侧面移动到与最右侧螺栓齐平
            Plane rightPlane = m_context.SubNearBound.Invert();
            {
                double maxDist = -double.MaxValue;
                foreach (var curPoint in boltParams.CornerPoints) {
                    maxDist = Math.Max(maxDist, rightPlane.SignedDist(curPoint));
                }
                rightPlane.MoveThis(maxDist);
            }

            m_haunchEndMargin = Math.Max(rightPlane.Dist(pt0), rightPlane.Dist(pt1));
            m_haunchEndMargin += boltParams.BoltSize + boltParams.HoleTol + HAUNCH_BOLT_MARGIN;
        }

        private void CreateHaunchPlate(string plateUsage,
            PlateProperties plateProps, BoltParameter boltParam, 
            Plane zTopPlane, Plane subJointPlane, double chamferX, double chamferY)
        {
            if (zTopPlane == null || subJointPlane == null)
                return;

            //
            Plane zBottomPlane = zTopPlane.Invert();
            zBottomPlane.MoveThis(plateProps.Thickness);
            Plane leftPlane = m_context.SubAlignPlane.Clone();
            Plane rightPlane = m_context.SubNearBound.Invert();
            Plane vertPlane0 = subJointPlane.Invert();
            Plane vertPlane1 = vertPlane0.Invert();
            vertPlane1 = vertPlane1.Move(plateProps.Width);
            
            // 调整腋板右侧面
            {
                if (plateProps.Height == double.MaxValue) {
                    Vec3 leftPt = m_context.SubPlanes.FrontWeb.Intersect(
                        leftPlane, subJointPlane);
                    rightPlane.MoveThis(
                        rightPlane.SignedDist(leftPt) + m_haunchEndMargin);
                } else {
                    Vec3 leftPt0 = m_context.SubPlanes.FrontWeb.Intersect(leftPlane, vertPlane0);
                    Vec3 leftPt1 = m_context.SubPlanes.FrontWeb.Intersect(
                        leftPlane, vertPlane1);
                    double dist0 = rightPlane.SignedDist(leftPt0);
                    double dist1 = rightPlane.SignedDist(leftPt1);
                    if (dist0 > dist1) {
                        rightPlane.MoveThis(plateProps.Height + dist1);
                    } else {
                        rightPlane.MoveThis(plateProps.Height + dist0);
                    }
                }
            }

            // 创建板
            PartBase retPlate = m_context.Comp.GetOrCreateSteelPlate(
                plateUsage, zTopPlane, zBottomPlane,
                new Plane[] { leftPlane, vertPlane0, rightPlane, vertPlane1 });
            if (retPlate != null) {
                m_context.HaunchPlates.Add(retPlate);
            } else {
                return;
            }

            // 根据组件通用属性设置板的级别
            retPlate.Class = m_context.Comp.CompClass;

            // 设置板切角
            ChamferCtrlPoint chamferPoint = PlateHelper.FindNearCtrlPoint(
                retPlate, rightPlane, vertPlane1);
            if (chamferPoint != null) {
                chamferPoint.CutX = chamferX;
                chamferPoint.CutY = chamferY;
                chamferPoint.CutType = Define.CornerCutType.CORNER_CUT_LINE;
                chamferPoint.Flush();
                retPlate.Flush();
            }
        }

    }

}
