﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ColumnWithStiffeners186
{
    
    public class ShearHandler
    {

        private const double SHEAR_CUTTER_SIDE_MARGIN = 1.0;
        private const double SHEAR_CUTTER_THICKNESS_MARGIN = 5.0;
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;

        private PlateProperties m_plateProps = new PlateProperties();
        private BoltParameter m_boltParam = new BoltParameter();

        private bool m_creatingFrontShear = false;
        private Plane m_frontShearInside = null;
        private Plane m_frontShearLeft = null;
        private Plane m_frontShearRight = null;
        private Plane m_frontShearTop = null;
        private Plane m_frontShearBottom = null;
        private PartBase m_frontPlate = null;

        private bool m_creatingBackShear = false;
        private Plane m_backShearInside = null;
        private Plane m_backShearLeft = null;
        private Plane m_backShearRight = null;
        private Plane m_backShearTop = null;
        private Plane m_backShearBottom = null;
        private PartBase m_backPlate = null;

        private Define.CornerCutType m_chamferType = Define.CornerCutType.CORNER_CUT_NONE;
        private double m_chamferX = double.MaxValue;
        private double m_chamferY = double.MaxValue;
        /// <summary>
        /// 板朝向
        /// 0 - 默认
        /// 1 - 自动
        /// 2 - 横侧面平行于次零件，纵侧面垂直于主零件
        /// 3 - 横纵侧面都垂直于主零件
        /// 4 - 横侧面平行于次零件，左侧面垂直于主零件，右侧面垂直于次零件
        /// </summary>
        private int m_plateDir = 0;
        private double m_plateGap = 0.0;

        public ShearHandler(ComponentInstanceContext ctx, ParamsHandler paramsHandler)
        {
            m_context = ctx;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            if (!this.ParseProps()) {
                return false;
            }
            if (!this.ParseShearPlanes())
                return false;

            this.CreateShearPlates();
            this.CreateHaunchPlates();
            this.CreateShearChamfers();
            this.CreateCuttersByShear();
            this.CreateBoltGroups();
            return true;
        }

        private bool ParseProps()
        {
            this.ParsePlateProps();
            this.ParseBoltProps();

            //
            {
                int chamferIndex = m_paramsHandler.GetIntValue(ParamNames.PLATE_CHAMFER_TYPE);
                switch (chamferIndex) {
                case 2:
                    m_chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    break;
                case 3:
                    m_chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    break;
                case 4:
                    m_chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    break;
                case 1:
                default:
                    m_chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                }
            }
            m_chamferX = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_X);
            m_chamferY = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_Y);
            if (m_chamferX == double.MaxValue) {
                m_chamferX = 0.0;
            }
            if (m_chamferY == double.MaxValue) {
                m_chamferY = 0.0;
            }

            //
            {
                Plane subFrontWeb = m_context.SubPlanes.FrontWeb;
                Plane subBackWeb = m_context.SubPlanes.BackWeb;
                Vec3 subFrontNormal = m_context.SubPlanes.FrontWeb.Normal;
                Vec3 subBackNormal = m_context.SubPlanes.BackWeb.Normal;
                int creatingIndex = m_paramsHandler.GetIntValue(ParamNames.PLATE_CREATION_MODE);
                Plane nsWebPlane, fsWebPlane;
                m_context.ParseAwaySidePlane(subFrontWeb, subBackWeb, out nsWebPlane, out fsWebPlane);
                bool bFarFront = fsWebPlane.Normal.Dot(subFrontNormal) >
                    fsWebPlane.Normal.Dot(subBackNormal);

                //
                switch (creatingIndex) {
                case 0: // Default
                    /// FIXME: 有待实现基于组件安装方向确定创建哪一侧连接板的规则
                case 1: { // Auto
                        Vec3 xCompDir = m_context.Comp.WorldMatrix.X;
                        double frontDot = m_context.MainDirection.Dot(xCompDir);
                        double backDot = m_context.MainDirection.Dot(xCompDir);
                        if (frontDot <= backDot) {
                            m_creatingFrontShear = true;
                        } else {
                            m_creatingBackShear = true;
                        }
                    } break;
                case 2: // Far
                    if (bFarFront) {
                        m_creatingFrontShear = true;
                        m_creatingBackShear = false;
                    } else {
                        m_creatingFrontShear = false;
                        m_creatingBackShear = true;
                    }
                    break;
                case 3: // Both
                    m_creatingFrontShear = true;
                    m_creatingBackShear = true;
                    break;
                case 4: // Near
                    if (bFarFront) {
                        m_creatingFrontShear = false;
                        m_creatingBackShear = true;
                    } else {
                        m_creatingFrontShear = true;
                        m_creatingBackShear = false;
                    }
                    break;
                }
            }

            //
            if (m_creatingFrontShear && m_creatingBackShear) {
                m_plateGap = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_GAP);
            } else {
                m_plateGap = 0.0;
            }

            //
            m_plateDir = m_paramsHandler.GetIntValue(ParamNames.PLATE_ORIENTATION);
            return true;
        }

        private void ParsePlateProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_PROPS);
            m_plateProps.SetProps(plateProps);
        }

        private bool ParseBoltProps()
        {
            {
                DObject basicProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS);
                DObject holeProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS);
                DObject contentProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS);
                DObject extendProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS);

                m_boltParam.CopyFromBasicProps(basicProps);
                m_boltParam.CopyFromHoleProps(holeProps);
                m_boltParam.CopyFromContentProps(contentProps);
                m_boltParam.CopyFromExtendProps(extendProps);
            }

            //
            m_boltParam.BoltTopMargin = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_TOP_MARGIN);
            m_boltParam.BoltBottomMargin = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_BOTTOM_MARGIN);
            m_boltParam.BoltRightMargin = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_RIGHT_MARGIN);
            
            // 螺栓组坐标系横向平面
            double horiOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET);
            Plane hBoltPlane = m_context.SubAlignPlane.Move(horiOffset);

            // 螺栓组纵向平面
            int vertOffsetMode = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_OFFSET_MODE);
            Plane vBoltPlane = null;
            {
                double vertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET);
                switch (vertOffsetMode) {
                case 1: { // 相对顶翼缘偏移
                        if (vertOffset == double.MaxValue) {
                            // 自适应下翼缘偏移: 上翼板厚度 + 螺栓上部间距 + 10
                            double flangeThick = m_context.SubPlanes.TopFlange.Dist(
                                m_context.SubPlanes.TopFlangeInside.Origin);
                            vertOffset = flangeThick + m_boltParam.BoltTopMargin + 10.0;
                        }
                        vBoltPlane = m_context.SubPlanes.TopFlange.Move(-vertOffset);
                    } break;
                case 2: { // 相对中心面偏移
                        
                        if (vertOffset == double.MaxValue) {
                            vertOffset = 0.0;
                        }
                        Vec3 topBoltPos = m_context.SubPlanes.FrontWeb.Intersect(
                            m_context.SubPlanes.TopFlange, hBoltPlane);
                        Vec3 bottomBoltPos = m_context.SubPlanes.FrontWeb.Intersect(
                            m_context.SubPlanes.BottomFlange, hBoltPlane);
                        vBoltPlane = new Plane(
                            (topBoltPos + bottomBoltPos) * 0.5,
                            m_context.SubPlanes.TopBound.Normal);
                        vBoltPlane.MoveThis(vertOffset);
                    } break;
                case 3: { // 相对底翼缘偏移
                        if (vertOffset == double.MaxValue) {
                            // 自适应下翼缘偏移: 下翼板厚度 + 螺栓下部间距 + 10
                            double flangeThick = m_context.SubPlanes.BottomFlange.Dist(
                                m_context.SubPlanes.BottomFlangeInside.Origin);
                            vertOffset = flangeThick + m_boltParam.BoltBottomMargin + 10.0;
                        }
                        vBoltPlane = m_context.SubPlanes.BottomFlange.Move(-vertOffset);
                    } break;
                default:
                    break;
                }
            }
            if (vBoltPlane == null)
                return false;

            // 解析螺栓横纵间隔
            do {
                m_boltParam.StrideX.Clear();
                m_boltParam.StrideY.Clear();

                int hCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
                int vCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
                double[] hStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_HORI_STRIDE);
                double[] vStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
                if (hCount == 0 || vCount == 0) {
                    break;
                }

                //
                if (hCount > 1 && hStride.Length > 0) {
                    for (int i = 1; i < hCount; ++i) {
                        m_boltParam.StrideX.Add(hStride[Math.Min(i - 1, hStride.Length - 1)]);
                    }
                } else {
                    m_boltParam.StrideX.Add(0.0);
                }

                //
                if (vCount != int.MaxValue) {
                    if (vCount > 1 && vStride.Length > 0) {
                        for (int i = 1; i < vCount; ++i) {
                            m_boltParam.StrideY.Add(vStride[Math.Min(i - 1, vStride.Length - 1)]);
                        }
                    } else {
                        m_boltParam.StrideY.Add(0.0);
                    }
                } else { // 自适应纵向螺栓个数
                    // FIXME: 有待实现纵向自适应螺栓个数
                    m_boltParam.StrideY.AddRange(vStride);
                }
            } while (false);

            //
            for (int i=0; i<m_boltParam.StrideX.Count; ++i) {
                m_boltParam.BoltSizeX += m_boltParam.StrideX[i];
            }
            for (int i = 0; i < m_boltParam.StrideY.Count; ++i) {
                m_boltParam.BoltSizeY += m_boltParam.StrideY[i];
            }

            //
            m_boltParam.BoltPos = m_context.SubPlanes.FrontWeb.Intersect(hBoltPlane, vBoltPlane);
            if (m_boltParam.BoltPos == Vec3.MAX)
                return false;

            //
            {
                int shapeIndex = m_paramsHandler.GetIntValue(ParamNames.BOLT_SHAPE);
                switch (shapeIndex) {
                case 2:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_ODD;
                    break;
                case 3:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_EVEN;
                    break;
                case 4:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_ODD;
                    break;
                case 5:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_EVEN;
                    break;
                case 1:
                default:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                    break;
                }
            }

            //
            {
                m_boltParam.BoltOrientation = m_paramsHandler.GetIntValue(ParamNames.BOLT_ORIENTATION);
                if (m_boltParam.BoltOrientation == 1) {
                    /// FIXME: 自动确定螺栓朝向，目前固定为平行于次零件
                    m_boltParam.BoltOrientation = 4;
                }
                Vec3 boltDirZ = m_context.SubPlanes.FrontWeb.Normal;
                switch (m_boltParam.BoltOrientation) {
                case 2: { // 交错
                        m_boltParam.BoltDirX = m_context.SubDirection;
                        Vec3 refDirX = boltDirZ.Cross(m_context.MainJointPlane.Normal).Cross(boltDirZ);
                        m_boltParam.BoltDirY = boltDirZ.Cross(refDirX);
                    } break;
                case 3: { // 垂直于主零件
                        m_boltParam.BoltDirX = boltDirZ.Cross(m_context.MainJointPlane.Normal).Cross(boltDirZ);
                        m_boltParam.BoltDirY = boltDirZ.Cross(m_boltParam.BoltDirX);
                    } break;
                case 4: { // 平行于次零件
                        m_boltParam.BoltDirX = m_context.SubDirection;
                        m_boltParam.BoltDirY = m_context.SubPlanes.TopBound.Normal;
                    } break;
                }
                m_boltParam.BoltDirX = m_boltParam.BoltDirX.Normalize();
                m_boltParam.BoltDirY = m_boltParam.BoltDirY.Normalize();

                //
                if (m_boltParam.BoltDirY.Dot(Vec3.AXIS_Z) < 0.0) {
                    m_boltParam.BoltDirY *= -1.0;
                }
            }

            // 根据纵向偏移模式调整螺栓位置
            switch (vertOffsetMode) {
            case 1: { // 相对顶部偏移
                    m_boltParam.BoltPos -= m_boltParam.BoltDirY * m_boltParam.BoltSizeY;
                }
                break;
            case 2: { // 相对中间偏移
                    m_boltParam.BoltPos -= m_boltParam.BoltDirY * m_boltParam.BoltSizeY * 0.5;
                }
                break;
            case 3:
            default:
                break;
            }

            //
            {
                int[] valArray = m_paramsHandler.GetIntArrayValue(ParamNames.BOLT_IGNORE);
                if (valArray != null && valArray.Length > 0) {
                    m_boltParam.BoltIgnores.AddRange(valArray);
                } else {
                    m_boltParam.BoltIgnores.Clear();
                }
            }
            

            m_boltParam.UpdateCornerPoints();
            return true;
        }

        private bool ParseShearPlanes()
        {
            m_frontShearInside = m_context.SubPlanes.FrontWeb.Move(m_plateGap);
            m_backShearInside = m_context.SubPlanes.BackWeb.Move(m_plateGap);
            m_frontShearInside = m_frontShearInside.Invert();
            m_backShearInside = m_backShearInside.Invert();

            bool ret = this.ParseShearPlanes(m_frontShearInside,
                out m_frontShearLeft, out m_frontShearRight,
                out m_frontShearTop, out m_frontShearBottom);
            ret = ret && this.ParseShearPlanes(m_backShearInside,
                out m_backShearLeft, out m_backShearRight,
                out m_backShearTop, out m_backShearBottom);

            // 根据板高度调整板上下侧面
            if (m_plateProps.Height != double.MaxValue && m_plateProps.Height > 0.0) {
                double planeOffset = (m_plateProps.Height - m_frontShearTop.Dist(m_frontShearBottom.Origin)) * 0.5;
                m_frontShearTop.MoveThis(planeOffset);
                m_frontShearBottom.MoveThis(planeOffset);

                planeOffset = (m_plateProps.Height - m_backShearTop.Dist(m_backShearBottom.Origin)) * 0.5;
                m_backShearTop.MoveThis(planeOffset);
                m_backShearBottom.MoveThis(planeOffset);
            }

            // 根据板宽度调整板右侧面
            do {
                if (m_plateProps.Width == double.MaxValue || m_plateProps.Width <= 0.0) {
                    break;
                }
                Plane subCenterPlane = m_context.SubPlanes.TopBound.Move(
                    m_context.SubPlanes.TopBound.Dist(m_context.SubPlanes.BottomBound.Origin) * 0.5);
                // 前连接板
                {
                    Vec3 pt0 = m_frontShearRight.Intersect(m_context.SubPlanes.FrontWeb, subCenterPlane);
                    if (pt0 != Vec3.MAX) {
                        double planeOffset = m_plateProps.Width - m_frontShearLeft.Dist(pt0);
                        m_frontShearRight.MoveThis(planeOffset);
                    }
                }
                // 后连接板
                {
                    Vec3 pt0 = m_backShearRight.Intersect(m_context.SubPlanes.BackWeb, subCenterPlane);
                    if (pt0 != Vec3.MAX) {
                        double planeOffset = m_plateProps.Width - m_backShearLeft.Dist(pt0);
                        m_backShearRight.MoveThis(planeOffset);
                    }
                }
            } while (false);

            return ret;
        }

        private bool ParseShearPlanes(Plane insidePlane, 
            out Plane leftPlane, out Plane rightPlane, out Plane topPlane, out Plane bottomPlane)
        {
            leftPlane = null;
            rightPlane = null;
            topPlane = null;
            bottomPlane = null;

            //
            Vec3 zDir = -insidePlane.Normal;
            Plane outsidePlane = insidePlane.Invert();
            outsidePlane.MoveThis(m_plateProps.Thickness);

            // 左侧面，与主零件连接面相邻的侧面
            Vec3 leftNormal;
            {
                leftNormal = -m_context.MainJointPlane.Normal;
                leftNormal = zDir.Cross(leftNormal).Normalize().Cross(zDir);
                Vec3 pt0 = m_context.MainJointPlane.Intersect(
                    outsidePlane, m_context.SubPlanes.TopFlange);
                if (pt0 == Vec3.MAX)
                    return false;
                leftPlane = new Plane(pt0, leftNormal.Normalize());
                leftPlane.MoveThis(-m_context.MainJointMargin);
            }

            // 右侧面
            Vec3 rightNormal;
            {
                //
                switch (m_plateDir) {
                case 4: {
                        rightNormal = -m_context.SubNearBound.Normal;
                    } break;
                case 2:
                case 3:
                default: {
                        rightNormal = -leftNormal;
                    } break;
                }

                //
                rightPlane = new Plane(m_boltParam.BoltPos, rightNormal.Normalize());
                if (m_boltParam.CornerPoints.Count > 0) {
                    double maxDist = -double.MaxValue;
                    for (int i=0; i<m_boltParam.CornerPoints.Count; ++i) {
                        maxDist = Math.Max(maxDist, rightPlane.SignedDist(m_boltParam.CornerPoints[i]));
                    }
                    rightPlane.MoveThis(maxDist);
                }
                rightPlane.MoveThis(m_boltParam.BoltRightMargin);
            }

            // 上侧面
            Vec3 topNormal;
            {
                switch (m_plateDir) {
                case 3: {
                        topNormal = zDir.Cross(rightNormal);
                    } break;
                case 2:
                case 4:
                default:
                    topNormal = zDir.Cross(m_context.SubDirection);
                    break;
                }

                topPlane = new Plane(m_boltParam.BoltPos, topNormal.Normalize());
                if (m_boltParam.CornerPoints.Count > 0) {
                    double maxDist = -double.MaxValue;
                    for (int i = 0; i < m_boltParam.CornerPoints.Count; ++i) {
                        maxDist = Math.Max(maxDist, topPlane.SignedDist(m_boltParam.CornerPoints[i]));
                    }
                    topPlane.MoveThis(maxDist);
                }
                topPlane.MoveThis(m_boltParam.BoltTopMargin);
            }

            // 下侧面
            {
                Vec3 bottomNorml = -topNormal;
                bottomPlane = new Plane(m_boltParam.BoltPos, bottomNorml.Normalize());
                if (m_boltParam.CornerPoints.Count > 0) {
                    double maxDist = -double.MaxValue;
                    for (int i = 0; i < m_boltParam.CornerPoints.Count; ++i) {
                        maxDist = Math.Max(maxDist, bottomPlane.SignedDist(m_boltParam.CornerPoints[i]));
                    }
                    bottomPlane.MoveThis(maxDist);
                }
                bottomPlane.MoveThis(m_boltParam.BoltBottomMargin);
            }

            return true;
        }

        private void CreateShearPlates()
        {
            if (m_creatingFrontShear) {
                Plane[] sidePlanes = new[]
                {
                    m_frontShearLeft, m_frontShearBottom, m_frontShearRight, m_frontShearTop
                };

                m_frontPlate = m_context.Comp.GetOrCreateSteelPlate(
                    ObjectUsage.FRONT_SHEAR,
                    m_frontShearInside.Invert(), m_plateProps.Thickness, sidePlanes);
                if (m_frontPlate != null) {
                    m_context.ShearPlates.Add(m_frontPlate);
                    m_context.ShearWeldPlanes.Add(m_frontShearLeft);
                }
            }
            if(m_creatingBackShear) {
                Plane[] sidePlanes = new[]
                {
                    m_backShearLeft, m_backShearBottom, m_backShearRight, m_backShearTop
                };

                m_backPlate = m_context.Comp.GetOrCreateSteelPlate(
                    ObjectUsage.BACK_SHEAR,
                    m_backShearInside.Invert(), m_plateProps.Thickness, sidePlanes);
                if (m_backPlate != null) {
                    m_context.ShearPlates.Add(m_backPlate);
                    m_context.ShearWeldPlanes.Add(m_backShearLeft);
                }
            }

            //
            if(m_frontPlate != null && m_backPlate != null) {
                m_context.ShearFrontPlane = m_frontShearInside.Invert();
                m_context.ShearFrontPlane.MoveThis(m_plateProps.Thickness);

                m_context.ShearBackPlane = m_backShearInside.Invert();
                m_context.ShearBackPlane.MoveThis(m_plateProps.Thickness);
            }
            else if(m_frontPlate != null) {
                m_context.ShearFrontPlane = m_frontShearInside.Invert();
                m_context.ShearFrontPlane.MoveThis(m_plateProps.Thickness);
                m_context.ShearBackPlane = m_frontShearInside.Clone();
            }
            else if(m_backPlate != null) {
                m_context.ShearBackPlane = m_backShearInside.Invert();
                m_context.ShearBackPlane.MoveThis(m_plateProps.Thickness);
                m_context.ShearFrontPlane = m_backShearInside.Clone();
            }

            //
            foreach (PartBase curPart in m_context.ShearPlates) {
                curPart.NumberPrefix = m_plateProps.NoPrefix;
                curPart.NumberStart = m_plateProps.NoStart;
                curPart.NumberPostfix = m_plateProps.NoPostfix;
                curPart.PrebuiltNumberPrefix = m_plateProps.AssNoPrefix;
                curPart.PrebuiltNumberStart = m_plateProps.AssNoStart;
                curPart.PrebuiltNumberPostfix = m_plateProps.AssNoPostfix;

                curPart.Material = m_plateProps.Material;
                curPart.Name = m_plateProps.Name;
                curPart.Class = m_context.Comp.CompClass;
                curPart.Flush();
            }
        }

        private void CreateBoltGroups()
        {
            if (m_context.ShearPlates.Count < 0)
                return;

            //
            Vec3 boltStart, boltEnd;
            {
                boltStart = m_boltParam.BoltPos + 0.5 * m_boltParam.BoltSizeY * m_boltParam.BoltDirY;
                if (m_boltParam.BoltSizeX != 0.0) {
                    boltEnd = boltStart + m_boltParam.BoltSizeX * m_boltParam.BoltDirX;
                } else {
                    boltEnd = boltStart + m_boltParam.BoltSize * m_boltParam.BoltDirX;
                }
            }

            //
            uint sectFlags = 0;
            {
                switch (m_boltParam.SlotIn) {
                case BoltParameter.SoltInMode.BEAM:
                    sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE0;
                    break;
                case BoltParameter.SoltInMode.PLATE: {
                        sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE1;
                        sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE2;
                    } break;
                case BoltParameter.SoltInMode.BOTH: {
                        sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE0;
                        sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE1;
                        sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE2;
                    } break;
                default:
                    break;
                }
                if (m_boltParam.PartWasherUp) {
                    sectFlags = sectFlags | Define.BOLT_SECT_WASHER_UP;
                }
                if (m_boltParam.PartWasherDown0) {
                    sectFlags = sectFlags | Define.BOLT_SECT_WASHER_DOWN0;
                }
                if (m_boltParam.PartWasherDown1) {
                    sectFlags = sectFlags | Define.BOLT_SECT_WASHER_DOWN1;
                }
                if (m_boltParam.PartNut0) {
                    sectFlags = sectFlags | Define.BOLT_SECT_NUT0;
                }
                if (m_boltParam.PartNut1) {
                    sectFlags = sectFlags | Define.BOLT_SECT_NUT1;
                }
            }

            // 1. 其中一个连接板为螺栓的主零件
            // 2. 次梁为螺栓的第一次零件
            // 3. 另一个连接板为螺栓的后续次零件
            PartBase boltMain = m_context.ShearPlates.First();
            List<PartBase> boltSubs = new List<PartBase>() { m_context.SubPart };
            if (m_context.ShearPlates.Count > 1) {
                for (int i=1; i<m_context.ShearPlates.Count; ++i) {
                    boltSubs.Add(m_context.ShearPlates[i]);
                }
            }

            //
            BoltGroup boltGroup = m_context.Comp.GetOrCreateBoltGroup(
                ObjectUsage.SHEAR_BOLT,
                boltMain, boltSubs.ToArray(), boltStart, boltEnd);
            boltGroup.WorkAxis = new Mat43(
                m_boltParam.BoltDirX, m_boltParam.BoltDirY, 
                m_boltParam.BoltDirX.Cross(m_boltParam.BoltDirY),
                Vec3.ZERO
                );
            boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;

            //
            boltGroup.BoltStandard = m_boltParam.BoltStandard;
            boltGroup.BoltSize = m_boltParam.BoltSize;
            boltGroup.Shape = m_boltParam.BoltShape;
            boltGroup.SectionFlags = sectFlags;
            boltGroup.HoleTol = m_boltParam.HoleTol;
            boltGroup.ArrayStrideX = m_boltParam.StrideX.ToArray();
            boltGroup.ArrayStrideY = m_boltParam.StrideY.ToArray();
            boltGroup.MadeType = m_boltParam.MadeType0;
            boltGroup.ThreadInMtrl = m_boltParam.ThreadInMaterial;

            //
            boltGroup.HoleType = m_boltParam.HoleType;
            boltGroup.HoleX = m_boltParam.HoleX;
            boltGroup.HoleY = m_boltParam.HoleY;
            boltGroup.RotateSlot = m_boltParam.RotSlot;

            //
            double cutLength = (double)m_context.ShearPlates.Count * (m_plateProps.Thickness + m_plateGap) + m_context.SubWebThickness;
            cutLength *= 2.0;
            cutLength += 1.0; // Extra margin.
            boltGroup.CutLength = cutLength;

            //
            boltGroup.ExtLength = m_boltParam.ExtLength;
            boltGroup.IgnoreIndices = m_boltParam.BoltIgnores.ToArray();

            //
            boltGroup.Refresh();
        }

        private void CreateHaunchPlates()
        {
            HaunchHandler haunchHandler = new HaunchHandler(m_context, m_paramsHandler);
            haunchHandler.Execute(m_boltParam);
        }

        private void CreateShearChamfers()
        {
            if (m_chamferType == Define.CornerCutType.CORNER_CUT_NONE)
                return;
            switch (m_chamferType) {
            case Define.CornerCutType.CORNER_CUT_LINE: {
                    PlateHelper.SetChamfer(
                    m_frontPlate, m_frontShearLeft, m_chamferType, m_chamferX, m_chamferY);
                    PlateHelper.SetChamfer(
                        m_backPlate, m_backShearLeft, m_chamferType, m_chamferX, m_chamferY);
                } break;
            default: {
                    PlateHelper.SetChamfer(
                    m_frontPlate, m_frontShearLeft, m_chamferType, m_chamferX, m_chamferX);
                    PlateHelper.SetChamfer(
                        m_backPlate, m_backShearLeft, m_chamferType, m_chamferX, m_chamferX);
                } break;
            }
        }

        private void CreateCuttersByShear()
        {
            if (m_context.ShearPlates.Count == 0)
                return;

            // 切割体左侧面
            Plane leftCutPlane;
            {
                leftCutPlane = m_frontShearLeft;
                if (leftCutPlane.SignedDist(m_backShearLeft.Origin) > 0.0) {
                    leftCutPlane = m_backShearLeft;
                }
                leftCutPlane = leftCutPlane.Move(SHEAR_CUTTER_SIDE_MARGIN);
            }

            // 切割体右侧面
            Plane rightCutPlane;
            {
                rightCutPlane = m_frontShearRight;
                if (rightCutPlane.SignedDist(m_backShearRight.Origin) > 0.0) {
                    rightCutPlane = m_backShearRight;
                }
                double flangeCutGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FLANGE_GAP);
                rightCutPlane.MoveThis(flangeCutGap);
            }

            // 切割体上侧面
            Plane topCutPlane;
            {
                topCutPlane = m_frontShearTop;
                if (topCutPlane.SignedDist(m_backShearTop.Origin) > 0.0) {
                    topCutPlane = m_backShearTop;
                }
                topCutPlane = topCutPlane.Move(SHEAR_CUTTER_SIDE_MARGIN);
            }

            // 切割体下侧面
            Plane bottomCutPlane;
            {
                bottomCutPlane = m_frontShearBottom;
                if (bottomCutPlane.SignedDist(m_backShearBottom.Origin) > 0.0) {
                    bottomCutPlane = m_backShearBottom;
                }
                bottomCutPlane = bottomCutPlane.Move(SHEAR_CUTTER_SIDE_MARGIN);
            }

            // 分析切割体在次零件腹板法线方向上的厚度
            double cutterThickness;
            {
                cutterThickness = Math.Max(
                    m_context.SubPlanes.FrontWeb.Dist(m_context.SubPlanes.FrontBound.Origin),
                    m_context.SubPlanes.BackWeb.Dist(m_context.SubPlanes.BackBound.Origin));
                cutterThickness += SHEAR_CUTTER_THICKNESS_MARGIN;
            }

            // 若连接板超出次零件腹板，则强制创建两侧切割体
            bool bForceBoth = false;
            do {
                //
                Plane topSide = m_frontShearTop;
                Plane bottomSide = m_frontShearBottom;
                Vec3 subTopNormal = m_context.SubPlanes.TopBound.Normal;
                if (topSide.Normal.Dot(subTopNormal) < bottomSide.Normal.Dot(subTopNormal)) {
                    Plane tmpSide = topSide;
                    topSide = bottomSide;
                    bottomSide = tmpSide;
                }
                //
                Vec3 pt0 = topSide.Intersect(m_frontShearRight, m_frontShearInside);
                if (pt0 != Vec3.MAX &&m_context.SubPlanes.TopFlangeInside.SignedDist(pt0) < 0.0) {
                    bForceBoth = true;
                    break;
                }
                Vec3 pt1 = bottomSide.Intersect(m_frontShearRight, m_frontShearInside);
                if (pt1 != Vec3.MAX && m_context.SubPlanes.BottomFlangeInside.SignedDist(pt1) < 0.0) {
                    bForceBoth = true;
                    break;
                }
            } while (false);

            //
            Plane[] sidePlanes = new Plane[] { leftCutPlane, bottomCutPlane, rightCutPlane, topCutPlane };
            if (m_frontPlate != null || bForceBoth) {
                Plane zInsidePlane = m_frontShearInside;
                Plane zOutsidePlane = zInsidePlane.Invert().Move(cutterThickness);
                m_context.Comp.CreatePlateCutter(
                    m_context.SubPart, zInsidePlane, zOutsidePlane, sidePlanes, true);
            }
            if (m_backPlate != null || bForceBoth) {
                Plane zInsidePlane = m_backShearInside;
                Plane zOutsidePlane = zInsidePlane.Invert().Move(cutterThickness);
                m_context.Comp.CreatePlateCutter(
                    m_context.SubPart, zInsidePlane, zOutsidePlane, sidePlanes, true);
            }
        }

    }

}
