﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners186
{
    public class StiffHandler
    {

        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        
        private PlateProperties m_topNSProps = new PlateProperties();
        private PlateProperties m_topFSProps = new PlateProperties();
        private PlateProperties m_bottomNSProps = new PlateProperties();
        private PlateProperties m_bottomFSProps = new PlateProperties();

        private bool m_creatingStiffs = false;
        private bool m_haveChamfer = false;
        // 1 - 平行于次零件
        // 2 - 垂直与主零件
        private int m_stiffDir = 1;

        private Define.CornerCutType m_chamferType = Define.CornerCutType.CORNER_CUT_LINE;
        private double m_chamferX = double.MaxValue;
        private double m_chamferY = double.MaxValue;

        private double m_webGap = double.MaxValue;
        private double m_topNSOffset = double.MaxValue;
        private double m_topFSOffset = double.MaxValue;
        private double m_bottomNSOffset = double.MaxValue;
        private double m_bottomFSOffset = double.MaxValue;
        private double m_nsFlangeGapX = double.MaxValue;
        private double m_nsFlangeGapY = double.MaxValue;
        private double m_fsFlangeGapX = double.MaxValue;
        private double m_fsFlangeGapY = double.MaxValue;

        //
        private Plane m_zTopPlane = null; // 顶部加劲肋的原始顶面
        private Plane m_zBottomPlane = null; // 底部加劲肋的原始底面
        private double m_topAutoOffset = 0.0;
        private double m_bottomAutoOffset = 0.0;

        public StiffHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            if (!this.ParseProps())
                return;
            List<StiffPlanes> stiffPlaneList = this.ParseStiffPlanes();
            if (stiffPlaneList.Count != 4)
                return;
            if (m_topNSOffset == double.MaxValue) {
                m_topNSOffset = m_topAutoOffset;
            }
            if (m_topFSOffset == double.MaxValue) {
                m_topFSOffset = m_topAutoOffset;
            }
            if (m_bottomNSOffset == double.MaxValue) {
                m_bottomNSOffset = m_bottomAutoOffset;
            }
            if (m_bottomFSOffset == double.MaxValue) {
                m_bottomFSOffset = m_bottomAutoOffset;
            }

            // 创建板
            List<Plane> chamferPlanes = new List<Plane>();
            for (int i=0; i<stiffPlaneList.Count; ++i) {
                if (stiffPlaneList[i].SidePlanes.Count == 0)
                    continue;
                StiffPlanes curStiffPlanes = stiffPlaneList[i];
                PartBase curPlate = m_context.Comp.GetOrCreateSteelPlate(
                    curStiffPlanes.StiffUsage,
                    curStiffPlanes.ZTopPlane, curStiffPlanes.ZBottomPlane, 
                    curStiffPlanes.SidePlanes.ToArray());
                if (curPlate == null)
                    continue;
                m_context.StiffPlates.Add(curPlate);
                m_context.StiffMap[curStiffPlanes.StiffKey] = curPlate;
                chamferPlanes.Add(curStiffPlanes.SidePlanes[1]);
            }

            //
            foreach (var kv in m_context.StiffMap) {
                switch (kv.Key) {
                case StiffPlanes.TOP_NS_KEY:
                    this.ApplyPlateParameters(kv.Value, m_topNSProps);
                    break;
                case StiffPlanes.TOP_FS_KEY:
                    this.ApplyPlateParameters(kv.Value, m_topFSProps);
                    break;
                case StiffPlanes.BOTTOM_NS_KEY:
                    this.ApplyPlateParameters(kv.Value, m_bottomNSProps);
                    break;
                case StiffPlanes.BOTTOM_FS_KEY:
                    this.ApplyPlateParameters(kv.Value, m_bottomFSProps);
                    break;
                }
            }

            // 创建折角
            do {
                if (!m_haveChamfer || m_chamferType == Define.CornerCutType.CORNER_CUT_NONE) {
                    break;
                }
                for (int idx = 0; idx < m_context.StiffPlates.Count; ++idx) {
                    PartBase curPlate = m_context.StiffPlates[idx];
                    Plane insidePlane = chamferPlanes[idx];
                    var targetPoints = PlateHelper.FindNearCtrlPoints(curPlate, insidePlane);
                    switch (m_chamferType) {
                    case Define.CornerCutType.CORNER_CUT_LINE:
                        foreach (var curPoint in targetPoints) {
                            curPoint.CutType = m_chamferType;
                            curPoint.CutX = m_chamferX;
                            curPoint.CutY = m_chamferY;
                            curPoint.Flush();
                        }
                        break;
                    default:
                        foreach (var curPoint in targetPoints) {
                            curPoint.CutType = m_chamferType;
                            curPoint.CutX = m_chamferY;
                            curPoint.CutY = m_chamferY;
                            curPoint.Flush();
                        }
                        break;
                    }
                    curPlate.Flush();
                }
            } while (false);
        }

        private bool ParseProps()
        {
            //
            {
                int stiffCreation = m_paramsHandler.GetIntValue(ParamNames.STIFF_CREATION);
                switch (stiffCreation) {
                case 1: {
                        // TODO: 有待实现自适应创建加劲肋的规则
                    }
                    break;
                case 3: {
                        m_creatingStiffs = true;
                    }
                    break;

                default:
                    m_creatingStiffs = false;
                    break;
                }
            }
            if (!m_creatingStiffs)
                return false;

            // 板属性
            {
                DObject topNSProps = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_TOP_NS_PROPS);
                DObject topFSProps = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_TOP_FS_PROPS);
                DObject bottomNSProps = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_BOTTOM_NS_PROPS);
                DObject bottomFSProps = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_BOTTOM_FS_PROPS);

                m_topNSProps.SetProps(topNSProps);
                m_topFSProps.SetProps(topFSProps);
                m_bottomNSProps.SetProps(bottomNSProps);
                m_bottomFSProps.SetProps(bottomFSProps);
            }

            //
            m_stiffDir = m_paramsHandler.GetIntValue(ParamNames.STIFF_ORIENTATION);

            //
            {
                int stiffShape = m_paramsHandler.GetIntValue(ParamNames.STIFF_SHAPE);
                switch (stiffShape) {
                case 1: {
                        // TODO: 有待实现自适应创建加劲肋的规则
                        m_haveChamfer = true;
                    }break;
                case 3: {
                        m_haveChamfer = true;
                    } break;
                default:
                    m_haveChamfer = false;
                    break;
                }
            }

            //
            {
                int chamferIndex = m_paramsHandler.GetIntValue(ParamNames.STIFF_CHAMFER_TYPE);
                switch (chamferIndex) {
                case 1: {
                        m_chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    } break;
                case 2: {
                        m_chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    } break;
                case 3: {
                        m_chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    } break;
                case 4: {
                        m_chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    } break;
                default:
                    m_chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                }
            }

            //
            m_chamferX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_CHAMFER_X);
            m_chamferY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_CHAMFER_Y);

            //
            {
                m_webGap = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_WEB_GAP);
                m_topNSOffset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_TOP_NS_OFFSET);
                m_topFSOffset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_TOP_FS_OFFSET);
                m_bottomNSOffset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_BOTTOM_NS_OFFSET);
                m_bottomFSOffset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_BOTTOM_FS_OFFSET);
            }

            //
            {
                m_nsFlangeGapX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_NS_FLANGE_GAP_X);
                m_nsFlangeGapY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_NS_FLANGE_GAP_Y);
                m_fsFlangeGapX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_FS_FLANGE_GAP_X);
                m_fsFlangeGapY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_FS_FLANGE_GAP_Y);
            }

            return true;
        }

        private List<StiffPlanes> ParseStiffPlanes()
        {
            List<StiffPlanes> stiffPlaneList = new List<StiffPlanes>();
            if (!this.ParseZPlanes())
                return stiffPlaneList;

            // 分析近端、远端对应的主零件腹板面、翼板侧面
            Plane nsWebPlane;
            Plane fsWebPlane;
            Plane nsFlangeSide;
            Plane fsFlangeSide;
            {
                m_context.ParseAwaySidePlane(
                    m_context.MainPlanes.FrontWeb, m_context.MainPlanes.BackWeb,
                    out nsWebPlane, out fsWebPlane);
                if (nsWebPlane.Normal.Dot(m_context.MainPlanes.FrontWeb.Normal) > 0.0) {
                    nsFlangeSide = m_context.MainPlanes.TopFlangeFront;
                    fsFlangeSide = m_context.MainPlanes.TopFlangeBack;
                } else {
                    nsFlangeSide = m_context.MainPlanes.TopFlangeBack;
                    fsFlangeSide = m_context.MainPlanes.TopFlangeFront;
                }
            }
            
            // 上加劲肋
            {
                // 近端
                {
                    StiffPlanes stiffPlanes = new StiffPlanes();
                    stiffPlanes.StiffKey = StiffPlanes.TOP_NS_KEY;
                    stiffPlanes.StiffUsage = ObjectUsage.TOP_NEAR_STIFF;

                    // 加劲肋上下面
                    stiffPlanes.ZTopPlane = m_zTopPlane.Move(m_topNSOffset);
                    stiffPlanes.ZBottomPlane = stiffPlanes.ZTopPlane.Invert();
                    stiffPlanes.ZBottomPlane.MoveThis(m_topNSProps.Thickness);

                    // 连接上翼板的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.TopFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_nsFlangeGapX);

                    // 连接近端腹板面的侧面
                    stiffPlanes.RefPlanes.Add(nsWebPlane.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_webGap);

                    // 连接下翼板面的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.BottomFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_nsFlangeGapX);

                    // 与近端翼板侧面重合的侧面
                    stiffPlanes.RefPlanes.Add(nsFlangeSide);
                    stiffPlanes.RefPlaneOffsets.Add(m_nsFlangeGapY);

                    // 根据属性计算板侧面
                    stiffPlanes.ParseSidePlanes(m_stiffDir, m_topNSProps);
                    stiffPlaneList.Add(stiffPlanes);
                }
                // 远端
                {
                    StiffPlanes stiffPlanes = new StiffPlanes();
                    stiffPlanes.StiffKey = StiffPlanes.TOP_FS_KEY;
                    stiffPlanes.StiffUsage = ObjectUsage.TOP_FAR_STIFF;

                    // 加劲肋上下面
                    stiffPlanes.ZTopPlane = m_zTopPlane.Move(m_topFSOffset);
                    stiffPlanes.ZBottomPlane = stiffPlanes.ZTopPlane.Invert();
                    stiffPlanes.ZBottomPlane.MoveThis(m_topFSProps.Thickness);

                    // 连接上翼板面的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.TopFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_fsFlangeGapX);

                    // 连接到远端腹板面的侧面
                    stiffPlanes.RefPlanes.Add(fsWebPlane.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_webGap);

                    // 连接下翼板面的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.BottomFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_fsFlangeGapX);

                    // 有远端翼板侧面重合的侧面
                    stiffPlanes.RefPlanes.Add(fsFlangeSide);
                    stiffPlanes.RefPlaneOffsets.Add(m_fsFlangeGapY);

                    // 根据属性计算板侧面
                    stiffPlanes.ParseSidePlanes(m_stiffDir, m_topFSProps);
                    stiffPlaneList.Add(stiffPlanes);

                }
            }

            // 下加劲肋
            {
                // 近端
                {
                    StiffPlanes stiffPlanes = new StiffPlanes();
                    stiffPlanes.StiffKey = StiffPlanes.BOTTOM_NS_KEY;
                    stiffPlanes.StiffUsage = ObjectUsage.BOTTOM_NEAR_STIFF;

                    // 加劲肋上下面
                    stiffPlanes.ZTopPlane = m_zBottomPlane.Move(m_bottomNSOffset);
                    stiffPlanes.ZBottomPlane = stiffPlanes.ZTopPlane.Invert();
                    stiffPlanes.ZBottomPlane.MoveThis(m_bottomNSProps.Thickness);

                    // 连接上翼板的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.TopFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_nsFlangeGapX);

                    // 连接近端腹板面的侧面
                    stiffPlanes.RefPlanes.Add(nsWebPlane.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_webGap);

                    // 连接下翼板面的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.BottomFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_nsFlangeGapX);

                    // 与近端翼板侧面重合的侧面
                    stiffPlanes.RefPlanes.Add(nsFlangeSide);
                    stiffPlanes.RefPlaneOffsets.Add(m_nsFlangeGapY);

                    // 根据属性计算板侧面
                    stiffPlanes.ParseSidePlanes(m_stiffDir, m_bottomNSProps);
                    stiffPlaneList.Add(stiffPlanes);
                }
                // 远端
                {
                    StiffPlanes stiffPlanes = new StiffPlanes();
                    stiffPlanes.StiffKey = StiffPlanes.BOTTOM_FS_KEY;
                    stiffPlanes.StiffUsage = ObjectUsage.BOTTOM_FAR_STIFF;

                    // 加劲肋上下面
                    stiffPlanes.ZTopPlane = m_zBottomPlane.Move(m_bottomFSOffset);
                    stiffPlanes.ZBottomPlane = stiffPlanes.ZTopPlane.Invert();
                    stiffPlanes.ZBottomPlane.MoveThis(m_bottomFSProps.Thickness);

                    // 连接上翼板面的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.TopFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_fsFlangeGapX);

                    // 连接到远端腹板面的侧面
                    stiffPlanes.RefPlanes.Add(fsWebPlane.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_webGap);

                    // 连接下翼板面的侧面
                    stiffPlanes.RefPlanes.Add(m_context.MainPlanes.BottomFlangeInside.Invert());
                    stiffPlanes.RefPlaneOffsets.Add(m_fsFlangeGapX);

                    // 有远端翼板侧面重合的侧面
                    stiffPlanes.RefPlanes.Add(fsFlangeSide);
                    stiffPlanes.RefPlaneOffsets.Add(m_fsFlangeGapY);

                    // 根据属性计算板侧面
                    stiffPlanes.ParseSidePlanes(m_stiffDir, m_bottomFSProps);
                    stiffPlaneList.Add(stiffPlanes);

                }
            }

            return stiffPlaneList;
        }

        private bool ParseZPlanes()
        {
            if (m_context.MainJointPlane == null)
                return false;

            // 解析次零件朝向主零件上下方向的边界面
            Plane subUpBound = null;
            Plane subDownBound = null;
            {
                //
                Vec3 mainUpDir = m_context.MainUpDirection.Normalize();
                
                //
                double maxDot = m_context.SubPlanes.TopBound.Normal.Dot(mainUpDir);
                subUpBound = m_context.SubPlanes.TopBound;
                subDownBound = m_context.SubPlanes.BottomBound;
                //
                {
                    double curDot = m_context.SubPlanes.BottomBound.Normal.Dot(mainUpDir);
                    if (curDot > maxDot) {
                        maxDot = curDot;
                        subUpBound = m_context.SubPlanes.BottomBound;
                        subDownBound = m_context.SubPlanes.TopBound;
                    }
                }
                //
                {
                    double curDot = m_context.SubPlanes.FrontBound.Normal.Dot(mainUpDir);
                    if (curDot > maxDot) {
                        maxDot = curDot;
                        subUpBound = m_context.SubPlanes.FrontBound;
                        subDownBound = m_context.SubPlanes.BackBound;
                    }
                }
                //
                {
                    double curDot = m_context.SubPlanes.BackBound.Normal.Dot(mainUpDir);
                    if (curDot > maxDot) {
                        maxDot = curDot;
                        subUpBound = m_context.SubPlanes.BackBound;
                        subDownBound = m_context.SubPlanes.FrontBound;
                    }
                }
            }

            //
            m_zTopPlane = subUpBound.Clone();
            m_zBottomPlane = subDownBound.Clone();
            Vec3 mainJointNormal = m_context.MainJointPlane.Normal.Normalize();

            // 计算与主零件长度方向垂直的加劲肋上下面，用于做后续调整
            Vec3 mainTopDir = Vec3.MAX;
            Vec3 mainBottomDir = Vec3.MAX;
            {
                Vec3 topDir = m_zTopPlane.Normal.Normalize();
                mainTopDir = mainJointNormal.Cross(topDir).Cross(mainJointNormal).Normalize();
                Vec3 bottomDir = m_zBottomPlane.Normal.Normalize();
                mainBottomDir = mainJointNormal.Cross(bottomDir).Cross(mainJointNormal).Normalize();
            }

            // 加劲肋与主零件平行时，调整上下面方向
            // 根据加劲肋原始上下面与主梁近边界面、连接面交线中点确定新的加劲板上下面
            do {
                if (2 != m_stiffDir) {
                    break;
                }
                
                // 主零件与其连接面垂直的中心面
                Plane mainCenterPlane = null;
                if (Math.Abs(mainJointNormal.Dot(m_context.MainPlanes.FrontWeb.Normal)) >
                    Math.Abs(mainJointNormal.Dot(m_context.MainPlanes.BackWeb.Normal))) {
                    mainCenterPlane = m_context.MainPlanes.TopBound.Invert();
                    mainCenterPlane.MoveThis(
                        0.5 * mainCenterPlane.Dist(m_context.MainPlanes.BottomBound.Origin));
                } else {
                    mainCenterPlane = m_context.MainPlanes.FrontBound.Invert();
                    mainCenterPlane.MoveThis(
                        0.5 * mainCenterPlane.Dist(m_context.MainPlanes.BackBound.Origin));
                }

                //
                do {
                    Vec3 pt0 = m_zTopPlane.Intersect(m_context.MainJointPlane, mainCenterPlane);
                    Vec3 pt1 = m_zTopPlane.Intersect(m_context.MainNearBound, mainCenterPlane);
                    if (pt0 == Vec3.MAX)
                        break;
                    m_zTopPlane = new Plane(pt0, mainTopDir);
                    if (pt1 == Vec3.MAX)
                        break;
                    double dist1 = m_zTopPlane.SignedDist(pt1);
                    if (dist1 > 0.0) {
                        m_zTopPlane.MoveThis(dist1);
                    }
                } while (false);

                //
                {
                    Vec3 pt0 = m_zBottomPlane.Intersect(m_context.MainJointPlane, mainCenterPlane);
                    Vec3 pt1 = m_zBottomPlane.Intersect(m_context.MainNearBound, mainCenterPlane);
                    if (pt0 == Vec3.MAX)
                        break;
                    m_zBottomPlane = new Plane(pt1, mainBottomDir);
                    double dist1 = m_zBottomPlane.SignedDist(pt1);
                    if (dist1 > 0.0) {
                        m_zBottomPlane.MoveThis(dist1);
                    }
                }
            } while (false);

            // 计算自动缩进值
            do {

                // 主零件连接面不在其边界内，无需根据连接板、腋板调整加劲肋缩进值
                {
                    const double eps = -1e3;
                    if (m_context.MainNearBound.Normal.IsParallelTo(mainJointNormal) &&
                    m_context.MainNearBound.SignedDist(m_context.MainJointPlane.Origin) > eps) {
                        break;
                    }
                }

                // 根据连接板、腋板计算加劲肋的上下自动缩进值
                do {
                    List<Vec3> skipPoints = new List<Vec3>();

                    // 腋板
                    {
                        // 筛选距离主零件近边界面比较近的点
                        foreach (var curHaunch in m_context.HaunchPlates) {
                            CtrlPoint[] curCtrls = curHaunch.CtrlPoints;
                            if (curCtrls.Length < 2)
                                continue;

                            for (int idx0 = 0; idx0 < curCtrls.Length; ++idx0) {
                                //
                                Vec3 pt0 = curCtrls[idx0].WorldPos;
                                double dist0 = m_context.MainNearBound.SignedDist(pt0);
                                if (dist0 < 0.0) {
                                    skipPoints.Add(pt0);
                                }
                                //
                                int idx1 = (idx0 + 1) % curCtrls.Length;
                                Vec3 pt1 = curCtrls[idx1].WorldPos;
                                double dist1 = m_context.MainNearBound.SignedDist(pt1);
                                if (dist1 < 0.0) {
                                    skipPoints.Add(pt1);
                                }
                                // 两个控制点在主零件近边界面的同侧
                                if (0 <= dist0 * dist1) {
                                    continue;
                                }
                                
                                //
                                double curFactor = 0.0;
                                if (!m_context.MainNearBound.IntersectRay(pt0, pt1 - pt0, out curFactor)) {
                                    continue;
                                }
                                Line helperLine = new Line(pt0, pt1);
                                skipPoints.Add(helperLine.Sample(curFactor));
                            }
                        }
                    }

                    // 连接板
                    if (m_context.ShearPlates.Count > 0) {
                        const double eps = 1.0;
                        PartBase curShear = m_context.ShearPlates[0];
                        CtrlPoint[] shearCtrls = curShear.CtrlPoints;
                        foreach (var curCtrl in shearCtrls) {
                            if (m_context.MainJointPlane.Dist(curCtrl.WorldPos) < eps) {
                                skipPoints.Add(curCtrl.WorldPos);
                            }
                        }
                    }
                } while (false);

                // 根据板厚度计算缩进值
                do {
                    //
                    double nearDist = 0;
                    Vec3 topPoint = m_context.SubAlignPlane.Intersect(
                        m_context.SubPlanes.FrontWeb, m_context.SubPlanes.TopFlange);
                    Vec3 bottomPoint = m_context.SubAlignPlane.Intersect(
                        m_context.SubPlanes.FrontWeb, m_context.SubPlanes.BottomFlange);
                    if (topPoint != Vec3.MAX) {
                        nearDist = Math.Min(m_context.MainNearBound.SignedDist(topPoint), nearDist);
                    }
                    const double eps = 1e-3;
                    if (nearDist > eps)
                        break;

                    //
                    m_topAutoOffset += Math.Max(
                        m_topNSProps.Thickness, m_topFSProps.Thickness);
                    m_bottomAutoOffset += Math.Max(
                        m_bottomNSProps.Thickness, m_bottomFSProps.Thickness);
                } while (false);
            } while (false);

            // 根据主零件边界点限制加劲肋上下面的缩进值
            {
                
                List<Vec3> mainBoundPoints = new List<Vec3>();
                {
                    Plane[] sidePlanes = new Plane[]
                    {
                        m_context.MainPlanes.FrontBound,
                        m_context.MainPlanes.BottomBound,
                        m_context.MainPlanes.BackBound,
                        m_context.MainPlanes.TopBound
                    };
                    for (int i=0; i< sidePlanes.Length; ++i) {
                        Vec3 pt0 = m_context.MainPlanes.HeadBound.Intersect(
                            sidePlanes[i], sidePlanes[(i + 1) % sidePlanes.Length]);
                        if (pt0 != Vec3.MAX) {
                            mainBoundPoints.Add(pt0);
                        }
                        Vec3 pt1 = m_context.MainPlanes.TailBound.Intersect(
                            sidePlanes[i], sidePlanes[(i + 1) % sidePlanes.Length]);
                        if (pt1 != Vec3.MAX) {
                            mainBoundPoints.Add(pt1);
                        }
                    }
                }

                //
                double maxTopMargin = 0.0;
                double maxBottomMargin = 0.0;
                foreach (Vec3 curPoint in mainBoundPoints) {
                    maxTopMargin = Math.Max(m_zTopPlane.SignedDist(curPoint), maxTopMargin);
                    maxBottomMargin = Math.Max(m_zBottomPlane.SignedDist(curPoint), maxBottomMargin);
                }

                m_topAutoOffset = Math.Min(maxTopMargin, m_topAutoOffset);
                m_bottomAutoOffset = Math.Min(maxBottomMargin, m_bottomAutoOffset);
            }

            return true;
        }

        private void ApplyPlateParameters(PartBase targetPlate, PlateProperties plateProps)
        {
            targetPlate.NumberPrefix = plateProps.NoPrefix;
            targetPlate.NumberStart = plateProps.NoStart;
            targetPlate.NumberPostfix = plateProps.NoPostfix;
            targetPlate.Material = plateProps.Material;
            targetPlate.Name = plateProps.Name;
            targetPlate.Class = m_context.Comp.CompClass;
        }

    }

}
