﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace ConnectingPlates14
{

    public class ComponentInstanceContext
    {
        private ConnectionComponent m_comp;

        // 主零件
        public Beam m_mainPart;

        private Plane m_mainNearCenterPlane; //主零件靠近中心(靠近次零件)的面
        private Plane m_mainAwayCenterPlane; //主零件远离中心的面

        private Plane m_mainFrontBound; 
        private Plane m_mainBackBound; 
        private Plane m_mainTopBound; 
        private Plane m_mainBottomBound; 
        private Plane m_mainTopFlangeInside; 
        private Plane m_mainBottomFlangeInside; 
        private Plane m_mainHCenterPlane; //梁中心面（法线为Top方向）

        private Plane m_mainFrontWeb; //前腹板面
        private Plane m_mainBackWeb; //后腹板面
        private Plane m_mainWebCenterPlane; //腹板中心面(法线为front方向)

        private double m_mainHeight; //高度
        private double m_mainWidth; //宽度
        private double m_mainWebThickness; //腹板厚度
        private double m_mainFlangeThickness; //翼缘厚度


        //次零件
        private Beam m_subPart;

        private Plane m_subNearCenterPlane;
        private Plane m_subAwayCenterPlane;

        private Plane m_subFrontBound; 
        private Plane m_subBackBound; 
        private Plane m_subTopBound; 
        private Plane m_subBottomBound;
        private Plane m_subTopFlangeInside; 
        private Plane m_subBottomFlangeInside; 
        private Plane m_subHCenterPlane;

        private Plane m_subFrontWeb;
        private Plane m_subBackWeb;
        private Plane m_subWebCenterPlane; 

        private double m_subHeight;
        private double m_subWidth; 
        private double m_subWebThickness;
        private double m_subFlangeThickness; 
        private double m_gap; //梁间距
        private Plane m_alignPlane;
        //
        private BoltLocation m_endPlateBoltLocationHori;
        private BoltLocation m_endPlateBoltLocationVert;
        private Vec3 m_plateFrontOrient;
        private Vec3 m_plateTopOrient;
        private Plane m_plateTopPlane;
        private Plane m_plateBottomPlane;
        private Plane m_mainPlateCtrlPlane;
        private Plane m_subPlateCtrlPlane;
        private Plane[] m_subSidePlanes;
        private Plane[] m_mainSidePlanes;
        private Plane[] m_shimSidePlanes;
        private double m_mainEndPlateThickness;
        private double m_subEndPlateThickness;
        private SteelPlate m_mainEndPlate;
        private SteelPlate m_subEndPlate;
        private SteelPlate m_frontEarPlate;
        private SteelPlate m_backEarPlate;
        private SteelPlate m_mainStiff;
        private SteelPlate m_subStiff;
        private List<SteelPlate> m_shimPlateList = new List<SteelPlate>();
        private Dictionary<String, List<HaunchHelper>> m_haunchDict = new Dictionary<string, List<HaunchHelper>>();

        ComponentInstanceContext(){}

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp))
            {
                compContext.Clear();
                return null;
            }
            return compContext;
        }

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            Beam subPart = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            if (mainPart == null || subPart == null)
                return false;
            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart;

            BeamPlanes mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            BeamPlanes subPlanes = BeamPlanes.CreateFromBeam(m_subPart);


            /*
             * 设置主次梁靠近中心的面
             */
            Plane mainPlane1 = mainPlanes.HeadBound.Clone();
            Plane mainPlane2 = mainPlanes.TailBound.Clone();
            Plane subPlane1 = subPlanes.HeadBound.Clone();
            Plane subPlane2 = subPlanes.TailBound.Clone();
            Vec3 mainPoint = mainPlanes.TopBound.Intersect(mainPlanes.FrontBound, mainPlanes.HeadBound); //主梁上一点，用于比较次梁的哪个面离主梁更近
            Vec3 subPoint = subPlanes.TopBound.Intersect(subPlanes.FrontBound, subPlanes.HeadBound);
            //主梁近心面
            if (mainPlane1.Dist(subPoint) <= mainPlane2.Dist(subPoint))
            {//距离subPoint距离小的较近
                MainNearCenterPlane = mainPlane1;
                MainAwayCenterPlane = mainPlane2;
            }
            else
            {
                MainNearCenterPlane = mainPlane2;
                MainAwayCenterPlane = mainPlane1;
            }
            //次梁近心面
            if (subPlane1.Dist(mainPoint) <= subPlane2.Dist(mainPoint))
            {
                SubNearCenterPlane = subPlane1;
                SubAwayCenterPlane = subPlane2;
            }
            else
            {
                SubNearCenterPlane = subPlane2;
                SubAwayCenterPlane = subPlane1;
            }


            /*
             * 设置上下面
             */
            m_mainTopBound = mainPlanes.TopBound.Clone();
            m_mainBottomBound = mainPlanes.BottomBound.Clone();
            SetInsidePlane(ref m_mainTopFlangeInside, m_mainTopBound.Invert(), mainPlanes.TopFlangeInside);
            SetInsidePlane(ref m_mainBottomFlangeInside, m_mainBottomBound.Invert(), mainPlanes.BottomFlangeInside);
            m_subTopBound = subPlanes.TopBound.Clone();
            m_subBottomBound = subPlanes.BottomBound.Clone();
            SetInsidePlane(ref m_subTopFlangeInside, m_subTopBound.Invert(), subPlanes.TopFlangeInside);
            SetInsidePlane(ref m_subBottomFlangeInside, m_subBottomBound.Invert(), subPlanes.BottomFlangeInside);
            if (m_mainTopBound.Normal.Dot(m_subTopBound.Normal) < 0) //统一主次零件上下面(以主零件为准)
            {
                Swap(ref m_subTopBound, ref m_subBottomBound);
                Swap(ref m_subTopFlangeInside, ref m_subBottomFlangeInside);
            }


            /*
             * 设置前后面(与主次顺序有关)，Front = MainNearCenter × MainTop
             */
            Vec3 frontOrient = m_mainNearCenterPlane.Normal.Cross(m_mainTopBound.Normal);
            //主
            m_mainFrontBound = mainPlanes.FrontBound.Clone();
            m_mainBackBound = mainPlanes.BackBound.Clone();
            m_mainFrontWeb = mainPlanes.FrontWeb.Clone();
            m_mainBackWeb = mainPlanes.BackWeb.Clone();
            if (m_mainFrontBound.Normal.Dot(frontOrient) < 0)
            {
                Swap(ref m_mainFrontBound, ref m_mainBackBound);
                Swap(ref m_mainFrontWeb, ref m_mainBackWeb);
            }
            //次
            m_subFrontBound = subPlanes.FrontBound.Clone();
            m_subBackBound = subPlanes.BackBound.Clone();
            m_subFrontWeb = subPlanes.FrontWeb.Clone();
            m_subBackWeb = subPlanes.BackWeb.Clone();
            if (m_subFrontBound.Normal.Dot(frontOrient) < 0)
            {
                Swap(ref m_subFrontBound, ref m_subBackBound);
                Swap(ref m_subFrontWeb, ref m_subBackWeb);
            }

            /*
             * 梁参数
             */
            //
            m_mainWidth = m_mainFrontBound.Dist(m_mainBackBound.Origin);
            m_mainHeight = m_mainBottomBound.Dist(m_mainTopBound.Origin);
            m_mainWebThickness = m_mainFrontWeb.Dist(m_mainBackWeb.Origin);
            m_mainHCenterPlane = m_mainTopBound.Move(-m_mainHeight / 2);
            m_mainWebCenterPlane = m_mainFrontWeb.Move(-m_mainWebThickness / 2);
            m_mainFlangeThickness = mainPlanes.TopFlange.Dist(mainPlanes.TopFlangeInside.Origin);
            //
            m_subWidth = m_subFrontBound.Dist(m_subBackBound.Origin);
            m_subHeight = m_subBottomBound.Dist(m_subTopBound.Origin);
            m_subWebThickness = m_subFrontWeb.Dist(m_subBackWeb.Origin);
            m_subHCenterPlane = m_subTopBound.Move(-m_subHeight / 2);
            m_subWebCenterPlane = m_subFrontWeb.Move(-m_subWebThickness / 2);
            m_subFlangeThickness = subPlanes.TopFlange.Dist(subPlanes.TopFlangeInside.Origin);

            /*
             * 对齐面获取
             */
            m_plateFrontOrient = m_mainFrontBound.Normal.Normalize() + m_subFrontBound.Normal.Normalize(); //端板、衬板的前面（前侧面，与零件的前面一个方向）
            m_plateTopOrient = m_mainTopBound.Normal.Normalize() + m_subTopBound.Normal.Normalize(); //端板、衬板的顶面
            if (m_mainNearCenterPlane.Normal.Cross(m_subNearCenterPlane.Normal).Dist(Vec3.ZERO) < 1e-4) //正常情况，两个梁创建方向相同
                m_alignPlane = m_mainNearCenterPlane;
            else if (m_mainFrontWeb.Normal.Cross(m_subFrontWeb.Normal).Dist(Vec3.ZERO) < 1e-4) //主次梁腹板面平行，使用上（顶面）方向确定对齐面
            {
                Line line = m_mainHCenterPlane.Intersect(m_subHCenterPlane);
                m_alignPlane = new Plane(line.V0, line.V1 - line.V0, m_plateTopOrient);
            }
            else if (m_mainTopBound.Normal.Cross(m_subTopBound.Normal).Dist(Vec3.ZERO) < 1e-4) //主次梁翼缘面平行，使用前面方向确定对齐面
            {
                Line line = m_mainWebCenterPlane.Intersect(m_subWebCenterPlane);
                m_alignPlane = new Plane(line.V0, line.V1 - line.V0, m_plateFrontOrient);
            }
            else //两个方向上都不平行
            {
                Line line1 = m_mainHCenterPlane.Intersect(m_subHCenterPlane);
                //Line line2 = m_mainWebCenterPlane.Intersect(m_subWebCenterPlane);
                m_alignPlane = new Plane(line1.V0, line1.V1 - line1.V0, m_plateTopOrient.Normalize() + m_plateFrontOrient.Normalize());
            }
            if (m_alignPlane.Normal.Dot(m_mainNearCenterPlane.Normal) < 0)
            {
                m_alignPlane.InvertThis();
            }

            return true;
        }

        public bool IsSameOrient(Plane p1, Plane p2)
        {
            return p1.Normal.Dot(p2.Normal) > 0;
        }

        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;

            m_mainNearCenterPlane = null;
            m_mainAwayCenterPlane = null;
            m_subNearCenterPlane = null;
            m_subAwayCenterPlane = null;
        }

        public static void Swap(ref Plane p1, ref Plane p2)
        {
            Plane p = p1;
            p1 = p2;
            p2 = p;
        }

        public Plane[] CopyPlaneList(Plane[] planes)
        {
            List<Plane> re = new List<Plane>();
            foreach (var item in planes)
            {
                re.Add(item.Clone());
            }
            return re.ToArray();
        }

        public void SetInsidePlane(ref Plane p, Plane defaultPlane, Plane setPlane)
        {
            if (setPlane == null)
                p = defaultPlane;
            else
                p = setPlane.Clone();
        }

        public void test(Plane p)
        {
            Comp.CreateQuad(p, 4000, 4000, true);
        }
        public void test(Vec3 p)
        {
            Comp.CreatePoint(p, true);
        }
        public void test(Vec3 v1, Vec3 v2)
        {
            Comp.CreateLine(v1, v2, 1000, Color.Red, true);
        }
        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }

        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }

        public Beam SubPart { get => m_subPart; set => m_subPart = value; }

        public Plane MainTopBound { get => m_mainTopBound; set => m_mainTopBound = value; }

        public Plane MainBottomBound { get => m_mainBottomBound; set => m_mainBottomBound = value; }

        public Plane MainFrontBound { get => m_mainFrontBound; set => m_mainFrontBound = value; }

        public Plane MainBackBound { get => m_mainBackBound; set => m_mainBackBound = value; }

        public Plane MainFrontWeb { get => m_mainFrontWeb; set => m_mainFrontWeb = value; }

        public Plane MainBackWeb { get => m_mainBackWeb; set => m_mainBackWeb = value; }

        /// <summary>
        /// 主零件靠近中心(靠近次零件)的面
        /// </summary>
        public Plane MainNearCenterPlane { get => m_mainNearCenterPlane; set => m_mainNearCenterPlane = value; }

        /// <summary>
        /// 主零件远离中心(远离次零件)的面
        /// </summary>
        public Plane MainAwayCenterPlane { get => m_mainAwayCenterPlane; set => m_mainAwayCenterPlane = value; }

        /// <summary>
        /// 主梁腹板中心面(法线为front方向)
        /// </summary>
        public Plane MainWebCenterPlane { get => m_mainWebCenterPlane; set => m_mainWebCenterPlane = value; }

        /// <summary>
        /// 主梁中心面（法线为Top方向）
        /// </summary>
        public Plane MainHCenterPlane { get => m_mainHCenterPlane; set => m_mainHCenterPlane = value; }

        public Plane SubTopBound { get => m_subTopBound; set => m_subTopBound = value; }

        public Plane SubBottomBound { get => m_subBottomBound; set => m_subBottomBound = value; }

        public Plane SubFrontBound { get => m_subFrontBound; set => m_subFrontBound = value; }

        public Plane SubBackBound { get => m_subBackBound; set => m_subBackBound = value; }

        public Plane SubFrontWeb { get => m_subFrontWeb; set => m_subFrontWeb = value; }

        public Plane SubBackWeb { get => m_subBackWeb; set => m_subBackWeb = value; }

        /// <summary>
        /// 次零件靠近中心(靠近主零件)的面
        /// </summary>
        public Plane SubNearCenterPlane { get => m_subNearCenterPlane; set => m_subNearCenterPlane = value; }

        /// <summary>
        /// 次零件远离中心(远离主零件)的面
        /// </summary>
        public Plane SubAwayCenterPlane { get => m_subAwayCenterPlane; set => m_subAwayCenterPlane = value; }

        /// <summary>
        /// 次梁腹板中心面(法线为front方向)
        /// </summary>
        public Plane SubWebCenterPlane { get => m_subWebCenterPlane; set => m_subWebCenterPlane = value; }

        /// <summary>
        /// 次梁中心面（法线为Top方向）
        /// </summary>
        public Plane SubHCenterPlane { get => m_subHCenterPlane; set => m_subHCenterPlane = value; }

        /// <summary>
        /// 主次梁的对齐面，方向从主梁指向次梁
        /// </summary>
        public Plane AlignPlane { get => m_alignPlane; set => m_alignPlane = value; }

        /// <summary>
        /// 端板螺栓组的水平定位
        /// </summary>
        public BoltLocation EndPlateBoltLocationHori { get => m_endPlateBoltLocationHori; set => m_endPlateBoltLocationHori = value; }

        /// <summary>
        /// 端板螺栓组的垂直定位
        /// </summary>
        public BoltLocation EndPlateBoltLocationVert { get => m_endPlateBoltLocationVert; set => m_endPlateBoltLocationVert = value; }

        public double Gap { get => m_gap; set => m_gap = value; }
        public double SubHeight { get => m_subHeight; set => m_subHeight = value; }
        public double SubFlangeThickness { get => m_subFlangeThickness; set => m_subFlangeThickness = value; }
        public double MainHeight { get => m_mainHeight; set => m_mainHeight = value; }
        public double MainWidth { get => m_mainWidth; set => m_mainWidth = value; }
        public double MainFlangeThickness { get => m_mainFlangeThickness; set => m_mainFlangeThickness = value; }

        /// <summary>
        /// 端板、衬板的前面（前侧面，与零件的前面一个方向）
        /// </summary>
        public Vec3 PlateFrontOrient { get => m_plateFrontOrient; set => m_plateFrontOrient = value; }


        /// <summary>
        /// 端板、衬板的顶面
        /// </summary>
        public Vec3 PlateTopOrient { get => m_plateTopOrient; set => m_plateTopOrient = value; }
        public List<SteelPlate> ShimPlateList { get => m_shimPlateList; set => m_shimPlateList = value; }
        public SteelPlate MainEndPlate { get => m_mainEndPlate; set => m_mainEndPlate = value; }
        public SteelPlate SubEndPlate { get => m_subEndPlate; set => m_subEndPlate = value; }
        public SteelPlate FrontEarPlate { get => m_frontEarPlate; set => m_frontEarPlate = value; }
        public SteelPlate BackEarPlate { get => m_backEarPlate; set => m_backEarPlate = value; }
        public SteelPlate MainStiff { get => m_mainStiff; set => m_mainStiff = value; }
        public SteelPlate SubStiff { get => m_subStiff; set => m_subStiff = value; }

        /// <summary>
        /// 主端板边面(frontPlane, topPlane, backPlane, bottomPlane)
        /// </summary>
        public Plane[] MainSidePlanes { get => m_mainSidePlanes; set => m_mainSidePlanes = value; }

        /// <summary>
        /// 次端板边面(frontPlane, topPlane, backPlane, bottomPlane)
        /// </summary>
        public Plane[] SubSidePlanes { get => m_subSidePlanes; set => m_subSidePlanes = value; }

        /// <summary>
        /// 衬板边面(frontPlane, topPlane, backPlane, bottomPlane)
        /// </summary>
        public Plane[] ShimSidePlanes { get => m_shimSidePlanes; set => m_shimSidePlanes = value; }
        public double MainEndPlateThickness { get => m_mainEndPlateThickness; set => m_mainEndPlateThickness = value; }
        public double SubEndPlateThickness { get => m_subEndPlateThickness; set => m_subEndPlateThickness = value; }


        public Plane MainTopFlangeInside { get => m_mainTopFlangeInside; set => m_mainTopFlangeInside = value; }
        public Plane SubTopFlangeInside { get => m_subTopFlangeInside; set => m_subTopFlangeInside = value; }
        public Plane MainBottomFlangeInside { get => m_mainBottomFlangeInside; set => m_mainBottomFlangeInside = value; }
        public Plane SubBottomFlangeInside { get => m_subBottomFlangeInside; set => m_subBottomFlangeInside = value; }

        /// <summary>
        /// 端板顶面（marginTop=0时的顶面）
        /// </summary>
        public Plane PlateTopPlane { get => m_plateTopPlane; set => m_plateTopPlane = value; }

        /// <summary>
        /// 端板底面（marginBottom=0时的底面）
        /// </summary>
        public Plane PlateBottomPlane { get => m_plateBottomPlane; set => m_plateBottomPlane = value; }

        /// <summary>
        /// 主端板靠近主梁的那个面（正常与MainNearCenterPlane重合），方向指向次零件
        /// </summary>
        public Plane MainCtrlPlane { get => m_mainPlateCtrlPlane; set => m_mainPlateCtrlPlane = value; }

        /// <summary>
        /// 次端板靠近次梁的那个面（正常与SubNearCenterPlane重合），方向指向主零件
        /// </summary>
        public Plane SubCtrlPlane { get => m_subPlateCtrlPlane; set => m_subPlateCtrlPlane = value; }

        /// <summary>
        /// 保存腋板
        /// </summary>
        public Dictionary<String, List<HaunchHelper>> HaunchDict { get => m_haunchDict; set => m_haunchDict = value; }

    }
}
