﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ConnectingPlates14
{
    class BoltHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;

        private Mat43 workAxis; //螺栓和螺栓孔位置类似，共用一个工作坐标系

        public BoltHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {

            /*
             * 创建准备
             */
            //端板的各个面
            Plane frontPlane = m_context.SubSidePlanes[0];
            Plane backPlane = m_context.SubSidePlanes[2];
            Plane bottomPlane = m_context.SubSidePlanes[3];
            Plane horiCenterPlane = bottomPlane.Move(-m_context.EndPlateBoltLocationVert.margin1 - m_context.EndPlateBoltLocationVert.stride.Sum() / 2);
            double horiOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET); //板的水平偏移（由螺栓中心偏移引起）
            Vec3 subTopCenterPoint = m_context.SubTopBound.Intersect(m_context.SubNearCenterPlane, m_context.SubWebCenterPlane);
            Plane vertCenterPlane = new Plane(subTopCenterPoint, -m_context.PlateFrontOrient).Move(horiOffset);
            //工作坐标系（公用）
            Vec3 xVec = frontPlane.Normal;
            Vec3 yVec = bottomPlane.Normal;
            workAxis = new Mat43(xVec, yVec, xVec.Cross(yVec), Vec3.ZERO);
            //梁的中心面
            Plane mainWebCenterPlane = m_context.MainWebCenterPlane;
            Plane mainHCenterPlane = m_context.MainHCenterPlane;
            Plane subWebCenterPlane = m_context.SubWebCenterPlane;
            Plane subHCenterPlane = m_context.SubHCenterPlane;
            //要删除的螺栓
            int[] inputDeleteList = m_paramsHandler.GetIntArrayValue(ParamNames.BOLT_DELETE_INDICES);
            int xCount = m_context.EndPlateBoltLocationHori.count;
            int yCount = m_context.EndPlateBoltLocationVert.count;
            List<int> deleteIndexList = new List<int>();
            bool[][] deleteMatrix = InitailArray(yCount + 1, xCount + 1);
            foreach (int deleteIndex in inputDeleteList)
            {
                //不处理时，水平方向(x)上第a个螺栓，垂直方向(-y)上第b个螺栓
                int a = (deleteIndex - 1) % xCount + 1;
                int b = (deleteIndex - 1) / xCount + 1;
                //删除螺栓对称到上侧
                b = yCount + 1 - b;
                int delete = xCount * (b - 1) + a;
                deleteIndexList.Add(delete);
                deleteMatrix[b][a] = true;
            }


            /*
             * 连接端板、衬板的螺栓组
             */
            //参数解析
            DObject basicProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS);
            DObject holeProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS);
            DObject contentProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS);
            DObject extendProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS);
            int shapeIndex = m_paramsHandler.GetIntValue(ParamNames.BOLT_SHAPE);
            BoltParameter boltParameter = new BoltParameter();
            boltParameter.CopyFromBasicProps(basicProps);
            boltParameter.CopyFromHoleProps(holeProps);
            boltParameter.CopyFromContentProps(contentProps);
            boltParameter.CopyFromExtendProps(extendProps);
            boltParameter.SetShape(shapeIndex);
            //螺栓位置设置
            PartBase mainPart = m_context.MainEndPlate;
            List<PartBase> subPartList = new List<PartBase>() { m_context.SubEndPlate };
            subPartList.AddRange(m_context.ShimPlateList);
            //起始点
            Vec3 start = m_context.MainCtrlPlane.Intersect(vertCenterPlane.Move(m_context.EndPlateBoltLocationHori.stride.Sum() / 2), horiCenterPlane);
            Vec3 end = m_context.MainCtrlPlane.Intersect(frontPlane, horiCenterPlane);
            //创建螺栓（需要两边的数量不为0）
            if(xCount > 0 && yCount > 0)
            {
                BoltGroup boltGroup = m_context.Comp.CreateBoltGroup(mainPart, subPartList.ToArray(), start, end, true);
                boltGroup.IgnoreIndices = deleteIndexList.ToArray();
                //boltGroup.ThreadInMtrl = true;
                boltParameter.StrideX = m_context.EndPlateBoltLocationHori.stride;
                boltParameter.StrideY = m_context.EndPlateBoltLocationVert.stride;
                boltGroup.CutLength = 2 * m_context.MainCtrlPlane.Dist(m_context.SubCtrlPlane.Origin); //切割长度
                SetCommonProps(boltGroup, boltParameter);
                boltGroup.Refresh();
            }


            /*
             * 衬板螺栓槽
             */
            ShimSlotMode slotMode = (ShimSlotMode)m_paramsHandler.GetIntValue(ParamNames.SHIM_SLOT_MODE);
            double shimMarginTopBottom = m_context.ShimSidePlanes[3].Dist(m_context.SubSidePlanes[3].Origin); //顶底方向的边距
            double shimMarginFrontBack = m_context.ShimSidePlanes[0].Dist(m_context.SubSidePlanes[0].Origin); //前后方向的边距
            //求水平/垂直方向有几个螺栓，用以确定开槽长度
            List<int> horiArrangeCount = new List<int>(); //开槽朝下，槽沿水平排列。每一列有几个螺栓
            List<int> vertArrangeCount = new List<int>(); //开槽朝右，槽沿垂直排列。每一行有几个螺栓
            for (int x = 1; x <= xCount; x++) //水平方向
            {
                int count = yCount;
                while(count >= 1) //垂直方向
                {
                    if (!deleteMatrix[count][x]) //若不删除螺栓，停止循环
                        break;
                    count--;

                }
                horiArrangeCount.Add(count);
            }
            for (int y = 1; y <= yCount; y++)
            {
                int delete = 0; //删除的螺栓数目
                while(delete < xCount)
                {
                    if (!deleteMatrix[y][delete + 1])
                        break;
                    delete++;
                }
                vertArrangeCount.Add(xCount - delete);
            }
            //开槽宽度
            double holeTol = m_paramsHandler.GetDoubleValue(ParamNames.PAD_BOLT_SIZE);
            if (holeTol == double.MaxValue)
                holeTol = boltParameter.HoleTol; 
            double slotWidth = boltParameter.BoltSize + holeTol;
            //水平/垂直方向的开槽长度
            List<double> horiArrangeLength = new List<double>(); //开槽朝下，槽沿水平排列。每一列的长度
            List<double> vertArrangeLength = new List<double>(); //开槽朝右，槽沿垂直排列。每一行的长度
            foreach(int count in horiArrangeCount)
            {
                double length = 0;
                if(count != 0)
                {
                    List<double> stride = m_context.EndPlateBoltLocationVert.stride.GetRange(0, count - 1);
                    length = m_context.EndPlateBoltLocationVert.margin1 + stride.Sum() + slotWidth / 2 + shimMarginTopBottom;
                }
                horiArrangeLength.Add(length);
            }
            foreach (int count in vertArrangeCount)
            {
                double length = 0;
                if (count != 0)
                {
                    List<double> stride = m_context.EndPlateBoltLocationHori.stride.GetRange(0, count - 1);
                    length = m_context.EndPlateBoltLocationHori.margin2 + stride.Sum() + slotWidth / 2 + shimMarginFrontBack;
                }
                vertArrangeLength.Add(length);
            }
            //水平排列的槽
            if(slotMode == ShimSlotMode.VERT_SLOT)
            {
                Plane cutBottomPlane = m_context.MainCtrlPlane.Move(m_context.MainEndPlateThickness - 0.1);
                Plane cutTopPlane = m_context.SubCtrlPlane.Move(m_context.SubEndPlateThickness - 0.1);
                Plane vertMovePlane = m_context.SubSidePlanes[2].Invert().Move(m_context.EndPlateBoltLocationHori.margin1); //用于确定切割体垂直方向的中心面
                Plane sideBottomPlane = m_context.ShimSidePlanes[3];
                for (int i = 0; i < xCount; i++)
                {
                    Plane sideBackPlane = vertMovePlane.Move(-slotWidth / 2);
                    Plane sideFrontPlane = vertMovePlane.Move(slotWidth / 2);
                    if (i != xCount - 1) // 已经到了最后一个螺栓间距
                        vertMovePlane.MoveThis(m_context.EndPlateBoltLocationHori.stride[i]); //移到下一列螺栓中间
                    if (horiArrangeLength[i] == 0) //这一列长度为0，跳过
                        continue;
                    Plane sideTopPlane = sideBottomPlane.Invert().Move(horiArrangeLength[i]);
                    Plane[] cutSidePlanes = { sideTopPlane, sideFrontPlane, sideBottomPlane, sideBackPlane };
                    Vec3 chamferPoint1 = cutBottomPlane.Intersect(sideTopPlane, sideFrontPlane);
                    Vec3 chamferPoint2 = cutBottomPlane.Intersect(sideTopPlane, sideBackPlane);
                    for (int j = 0; j < m_context.ShimPlateList.Count; j++)
                    {
                        PartBase cutPartBase = m_context.Comp.CreatePlateCutter(m_context.ShimPlateList[j], cutBottomPlane, cutTopPlane, cutSidePlanes, true);
                        ChamferControl chamferControl = new ChamferControl(cutPartBase, cutSidePlanes);
                        chamferControl.ChamferOnePoint(slotWidth / 2, slotWidth / 2, Define.CornerCutType.CORNER_CUT_ROUND, chamferPoint1);
                        chamferControl.ChamferOnePoint(slotWidth / 2, slotWidth / 2, Define.CornerCutType.CORNER_CUT_ROUND, chamferPoint2);
                        cutPartBase.Flush();
                    }
                }
            }
            //垂直排列的槽
            if(slotMode == ShimSlotMode.HORI_SLOT)
            {
                Plane cutBottomPlane = m_context.MainCtrlPlane.Move(m_context.MainEndPlateThickness - 0.1);
                Plane cutTopPlane = m_context.SubCtrlPlane.Move(m_context.SubEndPlateThickness - 0.1);
                Plane horiMovePlane = m_context.SubSidePlanes[3].Invert().Move(m_context.EndPlateBoltLocationVert.margin1); //用于确定切割体垂直方向的中心面
                Plane sideBottomPlane = m_context.ShimSidePlanes[0];
                for (int i = 0; i < yCount; i++)
                {
                    Plane sideBackPlane = horiMovePlane.Move(-slotWidth / 2);
                    Plane sideFrontPlane = horiMovePlane.Move(slotWidth / 2);
                    if (i != yCount - 1) // 已经到了最后一个螺栓间距
                        horiMovePlane.MoveThis(m_context.EndPlateBoltLocationVert.stride[i]); //移到下一列螺栓中间
                    if (vertArrangeLength[i] == 0) //这一列长度为0，跳过
                        continue;
                    Plane sideTopPlane = sideBottomPlane.Invert().Move(vertArrangeLength[i]);
                    Plane[] cutSidePlanes = { sideTopPlane, sideFrontPlane, sideBottomPlane, sideBackPlane };
                    Vec3 chamferPoint1 = cutBottomPlane.Intersect(sideTopPlane, sideFrontPlane);
                    Vec3 chamferPoint2 = cutBottomPlane.Intersect(sideTopPlane, sideBackPlane);
                    for (int j = 0; j < m_context.ShimPlateList.Count; j++)
                    {
                        PartBase cutPartBase = m_context.Comp.CreatePlateCutter(m_context.ShimPlateList[j], cutBottomPlane, cutTopPlane, cutSidePlanes, true);
                        ChamferControl chamferControl = new ChamferControl(cutPartBase, cutSidePlanes);
                        chamferControl.ChamferOnePoint(slotWidth / 2, slotWidth / 2, Define.CornerCutType.CORNER_CUT_ROUND, chamferPoint1);
                        chamferControl.ChamferOnePoint(slotWidth / 2, slotWidth / 2, Define.CornerCutType.CORNER_CUT_ROUND, chamferPoint2);
                        cutPartBase.Flush();
                    }
                }
            }



            /*
            * 端板螺栓孔
            */
            //主
            {
            //螺栓孔位置（相对于梁中心点的偏移）获取
            double horiOffsetTop = m_paramsHandler.GetDoubleValue(ParamNames.END1_HOLE_TW_MARGIN); //上侧螺栓孔与腹板中心的间距
            double vertOffsetTop = m_paramsHandler.GetDoubleValue(ParamNames.END1_HOLE_TH_MARGIN); //上侧螺栓孔与水平中心的间距
            double horiOffsetBottom = m_paramsHandler.GetDoubleValue(ParamNames.END1_HOLE_BW_MARGIN);
            double vertOffsetBottom = m_paramsHandler.GetDoubleValue(ParamNames.END1_HOLE_BH_MARGIN);
            //螺栓属性
            //标准
            BoltParameter boltParameter1 = new BoltParameter();
            boltParameter1.BoltStandard = m_paramsHandler.GetStringValue(ParamNames.MAIN_END_PLATE_BOLT_STANDARD);
            //尺寸
            double topBoltSize = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_END_PLATE_BOLT_SIZE_TOP);
            double bottomBoltSize = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_END_PLATE_BOLT_SIZE_BOTTOM);
            boltParameter1.BoltSize = topBoltSize;
            //工厂/工地
            boltParameter1.MadeType = BoltParameter.ParseMadeType(m_paramsHandler.GetIntValue(ParamNames.MAIN_END_PLATE_BOLT_TYPE));
            EndPlateHoleMode mainHoleMode = (EndPlateHoleMode)m_paramsHandler.GetIntValue(ParamNames.END1_PLATE_HOLE_MODE);
            Vec3 topStart; //顶端螺栓孔起点
            Vec3 bottomStart; //底端螺栓孔起点
            switch (mainHoleMode)
            {
                case EndPlateHoleMode.UP_ONE:
                    Util.SetDefaultValue(ref vertOffsetTop, 0);
                    Util.SetDefaultValue(ref horiOffsetTop, 0);
                    topStart = mainHCenterPlane.Move(vertOffsetTop).Intersect(mainWebCenterPlane.Move(horiOffsetTop), m_context.MainCtrlPlane);
                    CreateBoltHole(topStart, m_context.MainEndPlate, boltParameter1);
                    break;
                case EndPlateHoleMode.TWO:
                    SetSameValue(ref horiOffsetTop, ref horiOffsetBottom, 20);
                    SetSameValue(ref vertOffsetTop, ref vertOffsetBottom, 70);
                    //
                    topStart = mainHCenterPlane.Move(vertOffsetTop).Intersect(mainWebCenterPlane.Move(horiOffsetTop), m_context.MainCtrlPlane);
                    CreateBoltHole(topStart, m_context.MainEndPlate, boltParameter1);
                    //
                    boltParameter1.BoltSize = bottomBoltSize;
                    bottomStart = mainHCenterPlane.Move(-vertOffsetBottom).Intersect(mainWebCenterPlane.Move(-horiOffsetBottom), m_context.MainCtrlPlane);
                    CreateBoltHole(bottomStart, m_context.MainEndPlate, boltParameter1);
                    break;
                case EndPlateHoleMode.FOUR:
                    SetSameValue(ref horiOffsetTop, ref horiOffsetBottom, 20);
                    SetSameValue(ref vertOffsetTop, ref vertOffsetBottom, 70);
                    //上
                    topStart = mainHCenterPlane.Move(vertOffsetTop).Intersect(mainWebCenterPlane, m_context.MainCtrlPlane);
                    boltParameter1.StrideY = new List<double>() { horiOffsetTop * 2 };
                    CreateBoltHole(topStart, m_context.MainEndPlate, boltParameter1);
                    //下
                    boltParameter1.BoltSize = bottomBoltSize;
                    bottomStart = mainHCenterPlane.Move(-vertOffsetBottom).Intersect(mainWebCenterPlane, m_context.MainCtrlPlane);
                    boltParameter1.StrideY = new List<double>() { horiOffsetBottom * 2 };
                    CreateBoltHole(bottomStart, m_context.MainEndPlate, boltParameter1);
                    break;
                default:
                    break;
            }
            }
            //次
            {
                //螺栓孔位置（相对于梁中心点的偏移）获取
                double horiOffsetTop = m_paramsHandler.GetDoubleValue(ParamNames.END2_HOLE_TW_MARGIN); //上侧螺栓孔与腹板中心的间距
                double vertOffsetTop = m_paramsHandler.GetDoubleValue(ParamNames.END2_HOLE_TH_MARGIN); //上侧螺栓孔与水平中心的间距
                double horiOffsetBottom = m_paramsHandler.GetDoubleValue(ParamNames.END2_HOLE_BW_MARGIN);
                double vertOffsetBottom = m_paramsHandler.GetDoubleValue(ParamNames.END2_HOLE_BH_MARGIN);
                //螺栓属性
                //标准
                BoltParameter boltParameter2 = new BoltParameter();
                boltParameter2.BoltStandard = m_paramsHandler.GetStringValue(ParamNames.SUB_END_PLATE_BOLT_STANDARD);
                //尺寸
                double topBoltSize = m_paramsHandler.GetDoubleValue(ParamNames.SUB_END_PLATE_BOLT_SIZE_TOP);
                double bottomBoltSize = m_paramsHandler.GetDoubleValue(ParamNames.SUB_END_PLATE_BOLT_SIZE_BOTTOM);
                boltParameter2.BoltSize = topBoltSize;
                //工厂/工地
                boltParameter2.MadeType = BoltParameter.ParseMadeType(m_paramsHandler.GetIntValue(ParamNames.SUB_END_PLATE_BOLT_TYPE));
                EndPlateHoleMode subHoleMode = (EndPlateHoleMode)m_paramsHandler.GetIntValue(ParamNames.END2_PLATE_HOLE_MODE);
                Vec3 topStart; //顶端螺栓孔起点
                Vec3 bottomStart; //底端螺栓孔起点
                switch (subHoleMode)
                {
                    case EndPlateHoleMode.UP_ONE:
                        Util.SetDefaultValue(ref vertOffsetTop, 0);
                        Util.SetDefaultValue(ref horiOffsetTop, 0);
                        topStart = subHCenterPlane.Move(vertOffsetTop).Intersect(subWebCenterPlane.Move(horiOffsetTop), m_context.SubCtrlPlane);
                        CreateBoltHole(topStart, m_context.SubEndPlate, boltParameter2);
                        break;
                    case EndPlateHoleMode.TWO:
                        SetSameValue(ref horiOffsetTop, ref horiOffsetBottom, 20);
                        SetSameValue(ref vertOffsetTop, ref vertOffsetBottom, 70);
                        //
                        topStart = subHCenterPlane.Move(vertOffsetTop).Intersect(subWebCenterPlane.Move(horiOffsetTop), m_context.SubCtrlPlane);
                        CreateBoltHole(topStart, m_context.SubEndPlate, boltParameter2);
                        //
                        boltParameter2.BoltSize = bottomBoltSize;
                        bottomStart = subHCenterPlane.Move(-vertOffsetBottom).Intersect(subWebCenterPlane.Move(-horiOffsetBottom), m_context.SubCtrlPlane);
                        CreateBoltHole(bottomStart, m_context.SubEndPlate, boltParameter2);
                        break;
                    case EndPlateHoleMode.FOUR:
                        SetSameValue(ref horiOffsetTop, ref horiOffsetBottom, 20);
                        SetSameValue(ref vertOffsetTop, ref vertOffsetBottom, 70);
                        //上
                        topStart = subHCenterPlane.Move(vertOffsetTop).Intersect(subWebCenterPlane, m_context.SubCtrlPlane);
                        boltParameter2.StrideY = new List<double>() { horiOffsetTop * 2 };
                        CreateBoltHole(topStart, m_context.SubEndPlate, boltParameter2);
                        //下
                        boltParameter2.BoltSize = bottomBoltSize;
                        bottomStart = subHCenterPlane.Move(-vertOffsetBottom).Intersect(subWebCenterPlane, m_context.SubCtrlPlane);
                        boltParameter2.StrideY = new List<double>() { horiOffsetBottom * 2 };
                        CreateBoltHole(bottomStart, m_context.SubEndPlate, boltParameter2);
                        break;
                    default:
                        break;
                }
            }
        }

        //如果一个值设置了，另一个跟随
        public void SetSameValue(ref double v1, ref double v2, double defaultValue)
        {
            if (v1 == double.MaxValue)
                v1 = v2;
            if (v2 == double.MaxValue)
                v2 = v1;
            if (v1 == double.MaxValue)
                v1 = defaultValue;
            if (v2 == double.MaxValue)
                v2 = defaultValue;
        }

        public void CreateBoltHole(Vec3 point, PartBase part, BoltParameter boltParameter)
        {
            BoltGroup bg = m_context.Comp.CreateBoltHoles(null, part, point, point + workAxis.Y * 20, true);
            boltParameter.SectionFlags = 0;
            SetCommonProps(bg, boltParameter);
            bg.Refresh();
        }

        //设置基础属性
        public void SetCommonProps(BoltGroup boltGroup, BoltParameter boltParameter)
        {
            boltGroup.WorkAxis = workAxis;
            if (boltParameter == null)
                boltParameter = new BoltParameter();
            if(boltParameter.StrideX.Count != 0)
                boltGroup.ArrayStrideX = boltParameter.StrideX.ToArray();
            if(boltParameter.StrideY.Count != 0)
                boltGroup.ArrayStrideY = boltParameter.StrideY.ToArray();
            boltGroup.BoltStandard = boltParameter.BoltStandard;
            boltGroup.BoltSize = boltParameter.BoltSize;
            boltGroup.HoleTol = boltParameter.HoleTol; //孔容差范围
            boltGroup.ThreadInMtrl = boltParameter.ThreadInMaterial; //是否包含螺纹 
            boltGroup.MadeType = boltParameter.MadeType; //螺栓制作类型(工厂/工地) 
            boltGroup.HoleX = boltParameter.HoleX; //孔X方向扩大尺寸
            boltGroup.HoleY = boltParameter.HoleY;
            boltGroup.HoleType = boltParameter.HoleType; //孔类型
            boltGroup.RotateSlot = boltParameter.RotateSlot; //孔开槽旋转方式 
            boltGroup.SectionFlags = boltParameter.SectionFlags; //螺栓部件开关标志位
            boltGroup.ExtLength = boltParameter.ExtLength; // 附加长度 
            boltGroup.RotationSide = Define.ObjectSide.SIDE_BOTTOM; //螺栓朝向
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;
        }
    
        public bool[][] InitailArray(int m, int n)
        {
            if (m == 0 || n == 0 || m == double.MaxValue || n == double.MaxValue)
                return null;
            bool[][] re = new bool[m][];
            for(int i = 0; i < m; i++)
            {
                re[i] = new bool[n];
                for(int j = 0; j < n; j++)
                {
                    re[i][j] = false;
                }
            }
            return re;
        }
    }
}
