﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using CornerBoltedGusset57.Handlers;
using System.Collections.Generic;

namespace CornerBoltedGusset57.Common
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private ConnectionComponent m_comp;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine;

        private Beam m_subPart = null;

        private Vec3 m_subLine;

        private BeamPlanes m_mainPlanes;

        private BeamPlanes m_subPlanes;

        private List<BraceData> m_subData = new List<BraceData>();

        private List<BraceData> m_braceData = new List<BraceData>();

        private List<Vec3> m_gussetWeldPoints = new List<Vec3>();

        private List<Vec3> m_squarePoints = new List<Vec3>();

        private List<Vec3> m_stiffener1 = new List<Vec3>();

        private List<Vec3> m_stiffener2 = new List<Vec3>();

        private bool m_enableNotch = false;

        private bool m_enableSpecialNotch = false;

        private double m_thickness = double.MaxValue;
        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 SubLine { get => m_subLine; set => m_subLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public List<BraceData> SubData { get => m_subData; set => m_subData = value; }
        public List<BraceData> BraceData { get => m_braceData; set => m_braceData = value; }
        public Plane MainFaceBracePlane { get; set; }
        public Plane SubFaceBracePlane { get; set; }
        public Plane BaseGussetPlane { get; set; }
        /// <summary>
        /// 支撑的节点板构建选择平面1
        /// </summary>
        public Plane BracePlane { get; set; }
        /// <summary>
        /// 支撑的节点板构建选择平面2
        /// </summary>
        public Plane OppositeBracePlane { get; set; }
        /// <summary>
        /// T型 I型支撑构建节点板的偏移平面1
        /// </summary>
        public Plane BraceOffsetPlane { get; set; }
        /// <summary>
        /// T型 I型支撑构建节点板的偏移平面2
        /// </summary>
        public Plane OppositeBraceOffsetPlane { get; set; }
        public PartBase RetPlate { get; set; }
        public List<Vec3> GussetWeldPoints { get => m_gussetWeldPoints; set => m_gussetWeldPoints = value; }
        public List<Vec3> SquarePoints { get => m_squarePoints; set => m_squarePoints = value; }
        public List<Vec3> Stiffener1 { get => m_stiffener1; set => m_stiffener1 = value; }
        public List<Vec3> Stiffener2 { get => m_stiffener2; set => m_stiffener2 = value; }
        public bool EnableNotch { get => m_enableNotch; set => m_enableNotch = value; }
        public bool EnableSpecialNotch { get => m_enableSpecialNotch; set => m_enableSpecialNotch = value; }
        public double Thickness { get => m_thickness; set => m_thickness = value; }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (mainPart == null || comp.Subs == null || comp.Subs.Length < 2)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;

            Beam subPart = comp.Subs.Length > 0 ? comp.Subs.Last() as Beam : null;
            if (subPart == null)
                return false;
            if (!PrepareSubLine(mainPart, subPart, m_mainLine))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);
            m_subPlanes = BeamPlanes.CreateFromBeam(subPart);

            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue) 
                return false;
            m_mainLine = line.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 准备次零件的构建方向
        /// </summary>
        private bool PrepareSubLine(Beam mainPart, Beam subPart, Vec3 mainLine)
        {
            var linePoint = mainPart.CtrlPoints.First().WorldPos;
            Vec3 firstCtrlPoint = subPart.CtrlPoints.First().WorldPos;
            Vec3 lastCtrlPoint = subPart.CtrlPoints.Last().WorldPos;
            if (firstCtrlPoint == null || lastCtrlPoint == null)
                return false;
            double dist1 = GenericTools.PointToLineDistance(firstCtrlPoint - linePoint, mainLine);
            double dist2 = GenericTools.PointToLineDistance(lastCtrlPoint - linePoint, mainLine);
            var line = (lastCtrlPoint - firstCtrlPoint).Normalize();
            m_subLine = dist1 > dist2 ? -line : line;
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;
            m_mainPlanes = null;
            m_subPlanes = null;
        }
    }
}
