﻿using System.Collections.Generic;
using LoongCAD.Geometry;
using CornerBoltedGusset57.Common;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using System.Linq;
using System;

namespace CornerBoltedGusset57.Handlers
{
    public class GussetHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private PlateProperties m_plateProps = new PlateProperties();
        private Plane m_BaseGussetPlane;

        private Vec3 m_axisX;
        private Vec3 m_axisY;
        private Vec3 m_axisZ;

        private List<BraceData> m_braceData = new List<BraceData>();
        private int m_braceDataLength = 0;
        private double[] m_bracePlateLengthArr;
        private double m_thickness = double.MaxValue;

        //图形页参数
        private double m_firstBraceGussetPlateEdgeLength = double.MaxValue;
        private double m_braceBetweenGussetPlateEdgeLength = double.MaxValue;
        private double m_lastBraceGussetPlateEdgeLength = double.MaxValue;

        //节点板页面参数
        private int m_gussetPosition = 0;
        private double m_gussetMoveZ = double.MaxValue;
        private int m_gussetPlateShape = 0;
        private int m_braceNotchType = 0;
        private double m_halfVertNotch = double.MaxValue;
        private double m_notchAngle = double.MaxValue;
        private double m_vertNotch = double.MaxValue;
        private double m_horiNotch = double.MaxValue;

        public GussetHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            //建立坐标系
            m_axisY = m_context.MainLine;
            m_axisX = m_context.MainFaceBracePlane.Normal.Normalize();
            m_axisZ = m_axisX.Cross(m_axisY);

            //获取支撑数据
            m_braceData = m_context.BraceData;
            m_braceDataLength = m_braceData.Count;
            if (!ParseProps())
                return false;
            this.AdjustGussetPlaneDepth();
            this.MoveGussetZ();
            //初始化上下文参数
            m_context.BaseGussetPlane = m_BaseGussetPlane;

            //开槽
            if(m_braceNotchType == 3)
                m_context.EnableNotch = true;
            //开特殊槽口
            if (m_braceNotchType == 2)
                m_context.EnableSpecialNotch = true;

            //初始化螺栓水平距离
            m_bracePlateLengthArr = GetBracePlateLengths(m_braceDataLength);
            if (m_bracePlateLengthArr == null)
                return false;
            //改变节点板形状
            this.ChangeGussetPlateShape();
            //创建槽口
            this.CreateNotch();
            return true;
        }
        //解析属性
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.GUSSET_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);
            m_thickness = m_plateProps.Thickness;
            if (m_thickness == 0)
                return false;

            //自适应边距
            m_firstBraceGussetPlateEdgeLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_LENGTH_FIRST_BRACE);
            m_braceBetweenGussetPlateEdgeLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_LENGTH_BETWEEN_BRACE);
            m_lastBraceGussetPlateEdgeLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_LENGTH_LAST_BRACE);
            Dictionary<BraceSpacingType, double> map = new Dictionary<BraceSpacingType, double>();
            map.Add(BraceSpacingType.FIRST, m_firstBraceGussetPlateEdgeLength);
            map.Add(BraceSpacingType.BETWEEN, m_braceBetweenGussetPlateEdgeLength);
            map.Add(BraceSpacingType.LAST, m_lastBraceGussetPlateEdgeLength);
            double value = GussetPlateTools.AutoAdaptParams(map);
            if (value != double.MaxValue)
            {
                m_firstBraceGussetPlateEdgeLength = m_firstBraceGussetPlateEdgeLength
                    == double.MaxValue ? value : m_firstBraceGussetPlateEdgeLength;
                m_braceBetweenGussetPlateEdgeLength = m_braceBetweenGussetPlateEdgeLength
                    == double.MaxValue ? value : m_braceBetweenGussetPlateEdgeLength;
                m_lastBraceGussetPlateEdgeLength = m_lastBraceGussetPlateEdgeLength
                    == double.MaxValue ? value : m_lastBraceGussetPlateEdgeLength;
            }

            m_gussetPosition = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_POSITION_BRACE);
            m_gussetMoveZ = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_MOVE_DISTANCE_Z);
            m_gussetPlateShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_SHAPE);
            m_braceNotchType = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BRACE_NOTCH);
            m_horiNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_HORIZONTAL_NOTCH_DIMENSION);
            m_vertNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_VERTICAL_NOTCH_DIMENSION_FULL_NOTCH);

            m_halfVertNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_VERTICAL_NOTCH_DIMENSION_HALF_NOTCH);
            m_notchAngle = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_NOTCH_ANGLE);
            return true;
        }
        //调整节点板在支撑上的位置
        private void AdjustGussetPlaneDepth()
        {
            Plane plane1 = m_context.BracePlane;
            Plane plane2 = m_context.OppositeBracePlane;
            //判断标准
            bool flag = plane1.Normal.Normalize().Dot(m_axisZ) > 0;

            switch (m_gussetPosition)
            {
                case 2:
                    //向Z轴负方向移动
                    m_BaseGussetPlane = flag ? plane2 : plane1;
                    break;
                case 3:
                    Plane offsetPlane = m_context.BraceOffsetPlane;
                    Plane oppositeOffsetPlane = m_context.OppositeBraceOffsetPlane;
                    if(offsetPlane != null && oppositeOffsetPlane != null)
                        //有腹板截面节点板偏移处理
                        m_BaseGussetPlane = flag ? oppositeOffsetPlane : offsetPlane;
                    else
                    {
                        var dist = plane1.Dist(plane2.Origin);
                        var thickness = m_plateProps.Thickness;
                        m_BaseGussetPlane = plane1.Move(-dist / 2 - thickness / 2);
                    }
                    break;
                case 4:
                    //向Z轴正方向移动
                    m_BaseGussetPlane = flag ? plane1 : plane2;
                    break;
                case 1:
                default:
                    m_BaseGussetPlane = plane1;
                    break;
            }
        }
        //在Z轴方向移动节点板
        private void MoveGussetZ()
        {
            var dot = m_axisZ.Dot(m_BaseGussetPlane.Normal);
            if(dot > 0)
            {
                m_BaseGussetPlane.MoveThis(m_gussetMoveZ);
            }
            else
            {
                m_BaseGussetPlane.MoveThis(-m_gussetMoveZ);
            }
        }
        //调整节点板形状
        private void ChangeGussetPlateShape()
        {
            switch (m_gussetPlateShape)
            {
                case 3:
                    GetBracePlateCtrlPoints();
                    break;
                case 0:
                default:
                    GetBracePlateCtrlPoints();
                    PrepareBraceBoundary();
                    PrepareConvexHull();
                    HandlerAlign();
                    break;
            }
        }
        //创建槽口
        private void CreateNotch()
        {
            if (m_braceNotchType < 2)
                return;
            switch (m_braceNotchType)
            {
                case 3:
                    //得到顶面
                    Plane plTop = m_BaseGussetPlane.Move(0);
                    foreach (var brace in m_context.BraceData)
                    {
                        //得到侧面
                        List<Plane> plsSide = new List<Plane>();
                        plsSide.Add(brace.TopPlane);
                        plsSide.Add(brace.AlignPlane.Invert().Move(5.0));
                        plsSide.Add(brace.BottomPlane);
                        plsSide.Add(brace.GussetBorderPlane.Move(m_horiNotch));
                        m_context.TargetComp.CreatePlateCutter(brace.Beam, plTop.Move(m_thickness + m_vertNotch),
                            plTop.Invert().Move(m_vertNotch), plsSide.ToArray(), true);
                    }
                    break;
                default:
                    foreach(var brace in m_braceData)
                    {
                        this.CreateHalfNotch(brace);
                    }
                    break;
            }
        }
        //创建特殊槽口
        private void CreateHalfNotch(BraceData brace)
        {
            var type = GussetPlateTools.ParseBraceProfileType(brace.Beam);
            double moveLen; //切割长度
            double t;       //切割厚度
            switch (type)
            {
                case BraceProfileType.T:
                case BraceProfileType.I:
                    t = GussetPlateTools.GetTCutLength(brace.TopPlane, brace.BeamPlanes);
                    if (brace.BeamPlanes.BackWeb.Normal.IsParallelTo(m_BaseGussetPlane.Normal))
                    {
                        moveLen = brace.BackPlane.Dist(m_BaseGussetPlane.Origin);
                        HandlerHalfCutter(brace, t, moveLen);
                    }
                    else
                    {
                        moveLen = brace.BackPlane.Dist(brace.FrontPlane.Origin) / 2;
                        HandlerHalfCutter(brace, t, moveLen);
                    }
                    break;
                case BraceProfileType.RECTHOLLOW:
                    {
                        var eps = 5.0;
                        //得到切割体的侧面
                        List<Plane> rectSide = new List<Plane>();
                        rectSide.Add(brace.AlignPlane.Invert().Move(eps)) ;
                        rectSide.Add(brace.TopPlane.Move(eps));
                        rectSide.Add(brace.GussetBorderPlane.Move(m_horiNotch));
                        rectSide.Add(brace.BottomPlane.Move(eps));
                        //得到顶面
                        Plane rectTop = m_BaseGussetPlane.Move(0);
                        m_context.TargetComp.CreatePlateCutter(brace.Beam, rectTop.Move(m_thickness + m_vertNotch),
                            rectTop.Invert().Move(m_vertNotch), rectSide.ToArray(), true);
                    }
                    break;
                case BraceProfileType.BAN:
                case BraceProfileType.HQ:
                case BraceProfileType.WELDBOX:
                    moveLen = brace.BackPlane.Dist(brace.FrontPlane.Origin) / 2;
                    t = GussetPlateTools.GetBANCutLength(brace.TopPlane, brace.BeamPlanes);
                    HandlerHalfCutter(brace, t, moveLen);
                    break;
                default:
                    moveLen = brace.BackPlane.Dist(brace.FrontPlane.Origin) / 2;
                    t = GussetPlateTools.GetLCutLength(brace.TopPlane, brace.BeamPlanes);
                    HandlerHalfCutter(brace, t, moveLen);
                    break;
            }
        }
        //处理特殊槽口切割体
        private void HandlerHalfCutter(BraceData brace, double t, double moveLen)
        {
            var eps = 5.0;
            var len = brace.BackPlane.Dist(brace.FrontPlane.Origin);
            Plane cTop1 = brace.TopPlane.Move(eps);
            Plane cBottom1 = brace.TopPlane.Invert().Move(eps + t);
            Plane cTop2 = brace.BottomPlane.Move(eps);
            Plane cBottom2 = brace.BottomPlane.Invert().Move(eps + t);
            //进行切割 Z轴负方向
            {
                if (m_BaseGussetPlane.Dist(brace.BackPlane.Origin) >= len
                    || m_BaseGussetPlane.Dist(brace.FrontPlane.Origin) >= len)
                    return;
                List<Plane> cSide = new List<Plane>();
                cSide.Add(brace.AlignPlane.Invert().Move(eps));
                cSide.Add(brace.BackPlane.Move(eps));
                cSide.Add(brace.GussetBorderPlane.Move(m_horiNotch));
                cSide.Add(brace.BackPlane.Invert().Move(moveLen));
                m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop1, cBottom1, cSide.ToArray(), true);
                m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop2, cBottom2, cSide.ToArray(), true);
            }
            //折角操作
            {
                if (m_notchAngle <= 0 || m_notchAngle >= 90 || m_halfVertNotch >= len - moveLen)
                    return;
                //切割长度
                var chamferLength = len - moveLen  - m_halfVertNotch;
                //转化为弧度
                var radian = m_notchAngle * Math.PI / 180;
                //顶
                {
                    var point = brace.AlignPlane.Intersect(brace.TopPlane, brace.FrontPlane);
                    //沿着X轴正方向
                    Vec3 pointX = point + brace.braceLine.Normalize() * chamferLength * Math.Tan(radian);
                    //沿着Z轴负方向
                    Vec3 pointZ = point - brace.FrontPlane.Normal.Normalize() * chamferLength;
                    //定义Y轴方向
                    Vec3 pointY = pointZ + brace.BottomPlane.Normal.Normalize() * t;
                    List<Plane> plsSide = new List<Plane>();
                    plsSide.Add(new Plane(pointX, pointX - pointZ).Move(2 * eps));

                    var normal_1 = (pointY - pointZ).Normalize().Cross((pointX - pointZ).Normalize());
                    var normal_2 = -normal_1;

                    var normal = normal_1.Dot(brace.braceLine) > 0 ? normal_1 : normal_2;
                    plsSide.Add(new Plane(pointZ, normal));

                    plsSide.Add(plsSide[0].Invert().Move(chamferLength / Math.Cos(radian) + 4 * eps));
                    plsSide.Add(plsSide[1].Invert().Move(chamferLength / Math.Sin(radian)));
                    m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop1, cBottom1, plsSide.ToArray(), true);
                }
                //底
                {
                    var point = brace.AlignPlane.Intersect(brace.BottomPlane, brace.FrontPlane);
                    //沿着X轴正方向
                    Vec3 pointX = point + brace.braceLine.Normalize() * chamferLength * Math.Tan(radian);
                    //沿着Z轴负方向
                    Vec3 pointZ = point - brace.FrontPlane.Normal.Normalize() * chamferLength;
                    //定义Y轴方向
                    Vec3 pointY = pointZ + brace.TopPlane.Normal.Normalize() * t;
                    List<Plane> plsSide = new List<Plane>();
                    plsSide.Add(new Plane(pointX, pointX - pointZ).Move(2 * eps));

                    var normal_1 = (pointY - pointZ).Normalize().Cross((pointX - pointZ).Normalize());
                    var normal_2 = -normal_1;

                    var normal = normal_1.Dot(brace.braceLine) > 0 ? normal_1 : normal_2;
                    plsSide.Add(new Plane(pointZ, normal));

                    plsSide.Add(plsSide[0].Invert().Move(chamferLength / Math.Cos(radian) + 4 * eps));
                    plsSide.Add(plsSide[1].Invert().Move(chamferLength / Math.Sin(radian)));
                    m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop2, cBottom2, plsSide.ToArray(), true);
                }
            }
        }
        //得到节点板在支撑上的长度
        private double[] GetBracePlateLengths(int num)
        {
            BoltParse boltParse = new BoltParse(m_context, m_paramsHandler);
            if (!boltParse.Execute()) return null;
            List<BoltParse.ParamData> BoltParams = boltParse.ParamDataMap;
            var list = new List<double>();
            int len = BoltParams.Count;
            for (int i = 0; i < num; i++)
            {
                BoltParse.ParamData boltGroup = BoltParams[i % len];
                var sum = boltGroup.VerNearEdgeDist + boltGroup.VerFarEdgeDist;
                double[] X_Spacing = boltGroup.HorBoltsSpace;
                sum += GenericTools.SumArray(X_Spacing);
                list.Add(sum);
            }
            return list.ToArray();
        }
        //得到节点板在支撑附近的控制点
        private void GetBracePlateCtrlPoints()
        {
            var braces = m_braceData.ToArray();
            double lastSpacing;
            double firstSpacing = m_braceBetweenGussetPlateEdgeLength;

            for (int i = 0; i < m_braceDataLength; i++)
            {
                lastSpacing = i == 0 ?
                    m_lastBraceGussetPlateEdgeLength : m_braceBetweenGussetPlateEdgeLength;
                if (i == m_braceDataLength - 1)
                    firstSpacing = m_firstBraceGussetPlateEdgeLength;
                var brace = braces[i];
                //对齐面平移距离
                var length = m_bracePlateLengthArr[brace.ClickNumber];
                var borderPlane = brace.AlignPlane.Move(length);
                brace.TopPoint = brace.TopPlane.Move(lastSpacing)
                    .Intersect(m_BaseGussetPlane, borderPlane);
                brace.BottomPoint = brace.BottomPlane.Move(firstSpacing)
                    .Intersect(m_BaseGussetPlane, borderPlane);
                brace.GussetBorderPlane = borderPlane;
            }
        }
        /// <summary>
        /// 处理相邻支撑之间的边界线
        /// </summary>
        private void PrepareBraceBoundary()
        {
            for (var i = 0; i < m_braceData.Count - 1; ++i)
            {
                var cur = m_braceData[i];
                var next = m_braceData[i + 1];
                var curBoundary = GetCurBorderPlane(cur);
                var nextBoundary = GetNextBorderPlane(next);
                //处理平行
                if (curBoundary.Normal.IsParallelTo(nextBoundary.Normal))
                    continue;
                //边界的交点，不会改变
                var boundaryP = m_BaseGussetPlane.Intersect(curBoundary, nextBoundary);
                var nextDist = GetConvexPlane(next).SignedDist(boundaryP);
                //边界交点在对齐面的左侧，需要移动
                if (nextDist > 0)
                {
                    UpdateFarAlignPlane(next, boundaryP);
                    UpdateAlignPoint(next);
                }
                var curDist = GetConvexPlane(cur).SignedDist(boundaryP);
                //边界交点在对齐面的左侧，需要移动
                if (curDist > 0)
                {
                    UpdateFarAlignPlane(cur, boundaryP);
                    UpdateAlignPoint(cur);
                }
            }
        }
        /// <summary>
        /// 获取下一个支撑的边界
        /// </summary>
        /// <param name="next"></param>
        /// <returns></returns>
        private Plane GetNextBorderPlane(BraceData next)
        {
            return new Plane(next.TopPlane).Move(m_braceBetweenGussetPlateEdgeLength);
        }
        /// <summary>
        /// 获取当前支撑的边界
        /// </summary>
        /// <param name="cur"></param>
        /// <returns></returns>
        private Plane GetCurBorderPlane(BraceData cur)
        {
            return new Plane(cur.BottomPlane).Move(m_braceBetweenGussetPlateEdgeLength);
        }
        /// <summary>
        /// 获取凸面
        /// </summary>
        /// <param name="subData"></param>
        /// <returns></returns>
        private Plane GetConvexPlane(BraceData braceData)
        {
            return new Plane(braceData.GussetBorderPlane);
        }
        /// <summary>
        /// 更新节点板边界面
        /// </summary>
        /// <param name="brace"></param>
        /// <param name="point"></param>
        private void UpdateFarAlignPlane(BraceData brace,Vec3 point)
        {
            brace.GussetBorderPlane =  new Plane(point, brace.braceLine);
        }
        /// <summary>
        /// 更新支撑构建节点板的控制点
        /// </summary>
        /// <param name="brace"></param>
        private void UpdateAlignPoint(BraceData brace)
        {
            if(brace.SortNumber == 0)
            {
                brace.TopPoint = brace.GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, brace.TopPlane.Move(m_lastBraceGussetPlateEdgeLength));
                brace.BottomPoint = brace.GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, brace.BottomPlane.Move(m_braceBetweenGussetPlateEdgeLength));
            }else if(brace.SortNumber == m_braceDataLength - 1)
            {
                brace.TopPoint = brace.GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, brace.TopPlane.Move(m_braceBetweenGussetPlateEdgeLength));
                brace.BottomPoint = brace.GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, brace.BottomPlane.Move(m_firstBraceGussetPlateEdgeLength));
            }
            else
            {
                brace.TopPoint = brace.GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, brace.TopPlane.Move(m_braceBetweenGussetPlateEdgeLength));
                brace.BottomPoint = brace.GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, brace.BottomPlane.Move(m_braceBetweenGussetPlateEdgeLength));
            }
        }
        /// <summary>
        ///处理凸包 
        /// </summary>
        private void PrepareConvexHull()
        {
            //按照排序的subData检查相邻的支撑的对齐面是不是交于支撑之间。
            for (int i = 0; i < m_braceData.Count - 1; ++i)
            {
                var cur = m_braceData[i];
                var next = m_braceData[i + 1];
                var ret = CheckIntersection(cur, next);
                //交点在上边界外
                //可以往后移动next支撑，不需要回溯
                if (ret < 0)
                {
                    //cur的对齐面，cur的边界的交点
                    var adjustP = m_BaseGussetPlane.Intersect(GetConvexPlane(cur), GetCurBorderPlane(cur));
                    UpdateFarAlignPlane(next, adjustP);
                    UpdateAlignPoint(next);
                }
                //此时调整过后的交点一定在上边界内，
                //二次检查有没有在下边界外
                ret = CheckIntersection(cur, next);
                //交点在下边界外
                //需要往后移动cur支撑，需要回溯
                if (ret > 0)
                {
                    //next的对齐面，next的边界的交点
                    var adjustP = m_BaseGussetPlane.Intersect(GetConvexPlane(next), GetNextBorderPlane(next));
                    UpdateFarAlignPlane(cur, adjustP);
                    UpdateAlignPoint(cur);
                    int k = i - 1;//回到上一组
                    //反向循环处理，一直到某一组支撑不需要调整cur支撑
                    while (k >= 0)
                    {
                        cur = m_braceData[k];
                        next = m_braceData[k + 1];
                        ret = CheckIntersection(cur, next);
                        if (ret > 0)
                        {
                            //next的对齐面，next的边界的交点
                            adjustP = m_BaseGussetPlane.Intersect(GetConvexPlane(next), GetNextBorderPlane(next));
                            UpdateFarAlignPlane(cur, adjustP);
                            UpdateAlignPoint(cur);
                            --k;
                            continue;
                        }
                        break;
                    }
                }
            }
        }
        /// <summary>
        /// 检查cur,next的交点位置。
        /// </summary>
        /// <param name="cur"></param>
        /// <param name="next"></param>
        /// <returns>小于0说明到cur边界的距离小于0，大于0说明到next边界距离小于0</returns>
        private int CheckIntersection(BraceData cur, BraceData next)
        {
            return CheckIntersection(GetConvexPlane(cur), GetConvexPlane(next), GetCurBorderPlane(cur),
                GetNextBorderPlane(next));
        }

        /// <summary>
        /// 检查交点位于那一侧
        /// </summary>
        /// <param name="curRefPl"></param>
        /// <param name="nextRefPl"></param>
        /// <param name="curBorderPl"></param>
        /// <param name="nextBorderPl"></param>
        /// <returns>小于0说明到cur边界的距离小于0，大于0说明到next边界距离小于0</returns>
        private int CheckIntersection(Plane curRefPl, Plane nextRefPl, Plane curBorderPl, Plane nextBorderPl)
        {
            //得到此时的交点
            var refP = m_BaseGussetPlane.Intersect(curRefPl, nextRefPl);
            //与cur边界的距离
            var curDist = curBorderPl.SignedDist(refP);
            //与next边界的距离
            var nextDist = nextBorderPl.SignedDist(refP);
            if (curDist < 0)
            {
                return -1;
            }
            if (nextDist < 0)
            {
                return 1;
            }
            return 0;
        }
        //更新对齐面
        private void HandlerAlign()
        {
            foreach(var brace in m_braceData)
            {
                var length = m_bracePlateLengthArr[brace.ClickNumber];
                brace.AlignPlane = brace.GussetBorderPlane.Move(-length);
            }
        }
    }
}
