﻿using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;
using CornerBoltedGusset57.Common;

namespace CornerBoltedGusset57.Handlers
{
    public class GussetPlateTools
    {
        /// <summary>
        /// 自适应调整支撑附近节点板边缘长度
        /// </summary>
        /// <param name="dictionary"></param>
        /// <returns></returns>
        public static double AutoAdaptParams(Dictionary<BraceSpacingType, double> dictionary)
        {
            double[] items = dictionary.Values.ToArray();
            int num = 0;
            foreach (var item in items)
            {
                if (item == double.MaxValue)
                    num++;
            }
            double paramValue = double.MaxValue;
            switch (num)
            {
                case 3:
                    paramValue = 10.0;
                    break;
                case 2:
                    foreach (var value in items)
                    {
                        if (value != double.MaxValue)
                            paramValue = value;
                    }
                    break;
                case 1:
                    paramValue = dictionary[BraceSpacingType.FIRST] == double.MaxValue ?
                        dictionary[BraceSpacingType.LAST] : dictionary[BraceSpacingType.FIRST];
                    break;
                default:
                    break;

            }
            return paramValue;
        }
        public static BraceProfileType ParseBraceProfileType(Beam beam)
        {
            if (beam == null)
                return BraceProfileType.OTHER;
            var type = beam.Profile.GetType();
            if (type == typeof(LProfile))
                return BraceProfileType.L;
            else if (type == typeof(TProfile))
                return BraceProfileType.T;
            else if (type == typeof(IProfile))
                return BraceProfileType.I;
            else if (type == typeof(CProfile))
                return BraceProfileType.C;
            else if (type == typeof(UProfile))
                return BraceProfileType.U;
            else if (type == typeof(CCProfile))
                return BraceProfileType.CC;
            else if (type == typeof(RectHollowProfile))
                return BraceProfileType.RECTHOLLOW;
            else if (type == typeof(PlateProfile))
                return BraceProfileType.BAN;
            else if (type == typeof(WQProfile))
                return BraceProfileType.HQ;
            else if (type == typeof(WeldedBoxProfile))
                return BraceProfileType.WELDBOX;
            return BraceProfileType.OTHER;
        }
        /// <summary>
        /// 得到螺栓的切割长度
        /// </summary>
        public static double GetSectionThickness(BraceProfileType type, BeamPlanes beamPlanes, Plane plane)
        {
            double cutLength;
            switch (type)
            {
                case BraceProfileType.I:
                case BraceProfileType.T:
                    cutLength = GetTCutLength(plane, beamPlanes);
                    break;
                case BraceProfileType.BAN:
                case BraceProfileType.HQ:
                case BraceProfileType.WELDBOX:
                    cutLength = GetBANCutLength(plane, beamPlanes);
                    break;
                default:
                    cutLength = GetLCutLength(plane, beamPlanes);
                    break;
            }
            return cutLength;
        }
        /// <summary>
        /// 得到L型 C型 U型 CC型 []型截面的切割厚度
        /// </summary>
        public static double GetLCutLength(Plane plane, BeamPlanes beamPlanes)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BottomFlange.Normal.Normalize(),
                beamPlanes.BackWeb.Normal.Normalize()
            };
            int num = GenericTools.DotAbsMax(normals, plane.Normal.Normalize());
            if (num == 0)
            {
                CutLength = beamPlanes.BottomFlange.Dist(beamPlanes.BottomFlangeInside.Origin);
                return CutLength;
            }
            CutLength = beamPlanes.BackWeb.Dist(beamPlanes.BackWebInside.Origin);
            return CutLength;
        }
        /// <summary>
        /// 得到T型 I型截面的切割厚度
        /// </summary>
        public static double GetTCutLength(Plane plane, BeamPlanes beamPlanes)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BackWeb.Normal.Normalize(),
                beamPlanes.TopFlange.Normal.Normalize(),
            };
            int num = GenericTools.DotAbsMax(normals, plane.Normal.Normalize());
            if (num == 0)
            {
                CutLength = beamPlanes.BackBound.Dist(beamPlanes.FrontBound.Origin) / 2;
                return CutLength;
            }
            CutLength = beamPlanes.TopFlange.Dist(beamPlanes.TopFlangeInside.Origin);
            return CutLength;
        }
        /// <summary>
        /// 得到板 HQ型 焊接箱型截面的切割厚度
        /// </summary>
        /// <param name="flag">true为Z轴方向，false为Y轴方向</param>
        public static double GetBANCutLength(Plane plane, BeamPlanes beamPlanes)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BackBound.Normal.Normalize(),
                beamPlanes.TopBound.Normal.Normalize(),
            };
            int num = GenericTools.DotAbsMax(normals, plane.Normal.Normalize());
            if (num == 0)
            {
                CutLength = beamPlanes.BackBound.Dist(beamPlanes.FrontBound.Origin) / 2;
                return CutLength;
            }
            CutLength = beamPlanes.TopBound.Dist(beamPlanes.BottomBound.Origin) / 2;
            return CutLength;
        }
        /// <summary>
        /// 调整L型支撑螺栓组纵向偏移值
        /// <summary>
        /// param name="type">纵向测量类型</param>
        /// <param name="dist">支撑截面高度</param>
        /// <param name="NonMiddle">纵向螺栓组总间距</param>
        /// <param name="PanelValue">螺栓面板填写值</param>
        public static double AdjustLVertOffset(int type, double dist, double NonMiddle, double PanelValue)
        {
            if (type == 2 && PanelValue == double.MaxValue)
            {
                PanelValue = 0;
            }
            else if(type != 2 && PanelValue == double.MaxValue)
            {
                PanelValue = 40;
            }
            double VertOffset;
            switch (type)
            {
                case 1:
                    VertOffset = dist / 2 - NonMiddle / 2 - PanelValue;
                    break;
                case 2:
                    VertOffset = PanelValue;
                    break;
                case 0:
                default:
                    VertOffset = -dist / 2 + NonMiddle / 2 + PanelValue;
                    break;

            }
            return VertOffset;
        }
        /// <summary>
        /// 调整L型以外支撑螺栓组纵向偏移值
        /// </summary>
        /// <param name="type">纵向测量类型</param>
        /// <param name="dist">支撑截面高度</param>
        /// <param name="NonMiddle">纵向螺栓组总间距</param>
        /// <param name="PanelValue">螺栓面板填写值</param>
        public static double AdjustOtherVertOffset(int type, double dist, double NonMiddle, double PanelValue)
        {
            if((type == 1 || type == 3) && PanelValue == double.MaxValue)
            {
                PanelValue = 40;
            }
            else if(type != 1 && type != 3 && PanelValue == double.MaxValue)
            {
                PanelValue = 0;
            }
            double VertOffset;
            switch (type)
            {
                case 1:
                    VertOffset = -dist / 2 + NonMiddle / 2 + PanelValue;
                    break;
                case 3:
                    VertOffset = dist / 2 - NonMiddle / 2 - PanelValue;
                    break;
                case 0:
                default:
                    VertOffset = PanelValue;
                    break;

            }
            return VertOffset;
        }
    }
}
