﻿using System.Collections.Generic;
using System.Linq;
using CornerBoltedGusset57.Common;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System;

namespace CornerBoltedGusset57.Handlers
{
    /// <summary>
    /// 梁零件处理
    /// </summary>
    public class PartsHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        /// <summary>
        /// 支撑的集合 按照选取支撑的顺序
        /// </summary>
        private List<BraceData> m_subData = new List<BraceData>();
        /// <summary>
        /// 排序后的支撑 从靠近次零件的开始算
        /// </summary>
        private List<BraceData> m_braceData = new List<BraceData>();
        private int m_braceLength;
        private Plane m_mainFaceBraceplane;
        private Plane m_subFaceBraceplane;
        private Plane m_baseGussetPlane;

        Vec3 m_axisX;
        Vec3 m_axisY;
        Vec3 m_axisZ;

        //图形页面参数
        private double s_lastBraceGap = double.MaxValue;
        private double m_braceGap = double.MaxValue;
        private double m_firstBraceGap = double.MaxValue;
        public PartsHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            //解析支撑数据
            if (!ParseBraceData())
                return false;
            if (!ParseBraceCtrlPoints())
                return false;
            GetBraceFarPlane();
            if (!PrepareBracePlanes())
                return false;
            m_braceData = SortBraceData(m_subData, m_context.SubLine);
            m_context.BraceData = m_braceData;

            //解析主次零件
            if (!PrepareMainPlane())
                return false;
            m_mainFaceBraceplane = m_context.MainFaceBracePlane;

            if (!PrepareSubPlane())
                return false;
            m_subFaceBraceplane = m_context.SubFaceBracePlane;

            //支撑与主次零件平行时不创建组件
            foreach (var brace in m_braceData)
            {
                var line = brace.braceLine;
                if (line.IsParallelTo(m_context.MainLine) || line.IsParallelTo(m_context.SubLine))
                    return false;
            }
            //建立坐标系
            m_axisX = m_context.MainFaceBracePlane.Normal.Normalize();
            m_axisY = m_context.MainLine;
            m_axisZ = m_axisX.Cross(m_axisY);
            //解析支撑的前后面
            if (!PrepareBraceZPlanes())
                return false;
            //准备节点板的构建平面
            if (!ParseBraceType())
                return false;
            if (!GetBaseGussetPlane())
                return false;
            m_baseGussetPlane = m_context.BracePlane;

            this.ParseProps();
            if (!GetBraceAlignPlanes())
                return false;
            return true;
        }
        //解析支撑
        private bool ParseBraceData()
        {
            m_braceLength = m_context.TargetComp.Subs.Length - 1;
            for (int i = 0; i < m_braceLength; i++)
            {
                var item = m_context.TargetComp.Subs[i];
                if (item != null)
                {
                    Beam beam = item as Beam;
                    if (beam == null)
                        return false;
                    var braceData = new BraceData
                    {
                        Beam = beam,
                        BeamPlanes = BeamPlanes.CreateFromBeam(beam),
                        ClickNumber = i
                    };
                    m_subData.Add(braceData);
                }
            }
            if (m_subData == null || m_subData.Count != m_braceLength)
                return false;
            m_context.SubData = m_subData;
            return true;
        }
        //解析支撑控制点
        private bool ParseBraceCtrlPoints()
        {
            var linePoint = m_context.MainPart.CtrlPoints.First().WorldPos;
            foreach (var subData in m_subData)
            {
                var firstCtrlPoint = subData.Beam?.CtrlPoints?.First()?.WorldPos;
                var lastCtrlPoint = subData.Beam?.CtrlPoints?.Last()?.WorldPos;
                if (!firstCtrlPoint.HasValue || !lastCtrlPoint.HasValue)
                    return false;
                double dist1 = GenericTools.PointToLineDistance(firstCtrlPoint.Value - linePoint, m_context.MainLine);
                double dist2 = GenericTools.PointToLineDistance(lastCtrlPoint.Value - linePoint, m_context.MainLine);
                subData.farPoint = dist1 < dist2 ? lastCtrlPoint.Value : firstCtrlPoint.Value;
                var line = (lastCtrlPoint.Value - firstCtrlPoint.Value).Normalize();
                subData.braceLine = dist1 < dist2 ? line : -line;
            }
            return true;
        }
        //得到支撑离主零件较远的面
        private void GetBraceFarPlane()
        {
            foreach (var subData in m_subData)
            {
                var line = subData.braceLine;
                var TailBound = subData.BeamPlanes.TailBound;
                var HeadBound = subData.BeamPlanes.HeadBound;
                if (line.Dot(TailBound.Normal) > 0)
                    subData.FarPlane = TailBound;
                else
                    subData.FarPlane = HeadBound;
            }
        }
        /// <summary>
        /// 准备支撑的顶底面数据
        /// </summary>
        private bool PrepareBracePlanes()
        {
            foreach (var sub in m_subData)
            {
                //先得到top bottom
                Plane dotMaxPlane = GenericTools.DotExtremeBoundPlane(sub.BeamPlanes, m_context.MainLine, true);
                if (dotMaxPlane == null) return false;
                Plane oppositeplane = GenericTools.GetOppositePlane(dotMaxPlane, sub.BeamPlanes);
                if (oppositeplane == null)
                    return false;
                //再判断哪个是top
                double dist1 = m_context.SubLine.Dot(dotMaxPlane.Normal.Normalize());
                double dist2 = m_context.SubLine.Dot(oppositeplane.Normal.Normalize());
                sub.TopPlane = dist1 > dist2 ? dotMaxPlane : oppositeplane;
                sub.BottomPlane = dist1 > dist2 ? oppositeplane : dotMaxPlane;
            }
            return true;
        }
        /// <summary>
        /// 准备支撑的前后面数据
        /// </summary>
        private bool PrepareBraceZPlanes()
        {
            foreach (var sub in m_subData)
            {
                //先得到front back
                Plane dotMinPlane = GenericTools.DotExtremeBoundPlane(sub.BeamPlanes, m_context.MainLine, false);
                if (dotMinPlane == null) return false;
                Plane oppositeplane = GenericTools.GetOppositePlane(dotMinPlane, sub.BeamPlanes);
                if (oppositeplane == null)
                    return false;
                double dot1 = dotMinPlane.Normal.Normalize().Dot(m_axisZ);
                double dot2 = oppositeplane.Normal.Normalize().Dot(m_axisZ);
                //再判断哪个是front
                sub.BackPlane = dot1 < 0 ? dotMinPlane : oppositeplane;
                sub.FrontPlane = dot2 > 0 ? oppositeplane : dotMinPlane;

            }
            return true;
        }
        /// <summary>
        /// 准备主零件的参照面
        /// </summary>
        private bool PrepareMainPlane()
        {
            //适用于I型 []型 L型截面 其余看包围面
            var type = GussetPlateTools.ParseBraceProfileType(m_context.MainPart);
            var line = m_braceData.Last().braceLine;
            Plane plane = GenericTools.DotExtremePlane(m_context.MainPlanes, line, type, true);
            if (plane == null)
                return false;
            m_context.MainFaceBracePlane = plane;
            return true;
        }
        /// <summary>
        /// 准备次零件的参照面
        /// </summary>
        private bool PrepareSubPlane()
        {
            //适用于I型截面 []型 L型截面 其余看包围面
            var type = GussetPlateTools.ParseBraceProfileType(m_context.SubPart);
            var line = m_braceData.First().braceLine;
            Plane plane = GenericTools.DotExtremePlane(m_context.SubPlanes, line, type, true);
            if (plane == null)
                return false;
            m_context.SubFaceBracePlane = plane;
            return true;
        }
        /// <summary>
        /// 解析支撑的截面类型
        /// </summary>
        private bool ParseBraceType()
        {
            foreach (var brace in m_braceData)
            {
                var type = GussetPlateTools.ParseBraceProfileType(brace.Beam);
                if (type == BraceProfileType.OTHER)
                    return false;
                brace.ProfileType = type;
            }
            return true;
        }
        //得到节点板的构建平面
        private bool GetBaseGussetPlane()
        {
            var brace1 = m_braceData.First();
            var brace2 = m_braceData.Last();
            //节点板的构建面取定参照支撑
            //主零件构建方向与支撑调整后的构建方向点积最大的那个支撑
            var line_first = brace1.braceLine;
            var line_last = brace2.braceLine;
            double Dot1 = line_first.Dot(m_axisY);
            double Dot2 = line_last.Dot(m_axisY);
            var beam = Dot1 > Dot2 ? brace1.Beam : brace2.Beam;
            var beamPlanes = Dot1 > Dot2 ? brace1.BeamPlanes : brace2.BeamPlanes;
            var type = GussetPlateTools.ParseBraceProfileType(beam);
            if (type == BraceProfileType.OTHER)
                return false;

            var planes = GenericTools.ParseBasePlane(type, m_axisZ, beamPlanes);
            if (planes == null)
                return false;
            m_context.BracePlane = planes[0];
            m_context.OppositeBracePlane = planes[1];

            //处理I型 T型的腹板问题
            if (type != BraceProfileType.T && type != BraceProfileType.I)
                return true;
            var flag = beamPlanes.BackWeb.Normal.Normalize().Dot(m_axisZ);
            if (flag > 1e-6)
            {
                m_context.BraceOffsetPlane = beamPlanes.BackWeb;
                m_context.OppositeBraceOffsetPlane = beamPlanes.BackWebInside;
            }
            else if (Math.Abs(flag) > 1e-6)
            {
                m_context.BraceOffsetPlane = beamPlanes.BackWebInside;
                m_context.OppositeBraceOffsetPlane = beamPlanes.BackWeb;
            }
            return true;
        }
        private void ParseProps()
        {
            //默认值属性
            s_lastBraceGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SECONDARY_PART_DISTANCE_LAST_BRACE);
            m_braceGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_BRACE_DISTANCE);
            m_firstBraceGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_MAIN_PART_DISTANCE_FIRST_BRACE);
        }
        //得到支撑的对齐面
        private bool GetBraceAlignPlanes()
        {
            switch (m_braceLength)
            {
                case 1:
                    if (!CreateAlignPlane()) return false;
                    break;
                default:
                    this.AdjustBraceAndPart();
                    this.AdjustBraceAndBrace();
                    break;
            }
            return true;
        }
        //单个支撑情况创建对齐面
        private bool CreateAlignPlane()
        {
            Vec3 point;
            var brace = m_braceData.First();
            List<Plane> planes = new List<Plane>();
            point = m_subFaceBraceplane.Move(s_lastBraceGap)
                    .Intersect(brace.TopPlane, m_baseGussetPlane);
            planes.Add(new Plane(point, brace.braceLine));

            point = m_mainFaceBraceplane.Move(m_firstBraceGap)
                    .Intersect(brace.BottomPlane, m_baseGussetPlane);
            planes.Add(new Plane(point, brace.braceLine));
            Plane alignPlane = GenericTools.PlanesDistMin(planes.ToArray(), brace.FarPlane);
            if (alignPlane == null) return false;
            brace.AlignPlane = alignPlane;
            return true;
        }
        //调整支撑和主次零件的距离
        private void AdjustBraceAndPart()
        {
            foreach(var brace in m_braceData)
            {
                var point1 = m_mainFaceBraceplane.Move(m_firstBraceGap)
                    .Intersect(m_baseGussetPlane, brace.TopPlane);
                Plane alignPlane1 = new Plane(point1, brace.braceLine);
                var point2 = m_mainFaceBraceplane.Move(m_firstBraceGap)
                    .Intersect(m_baseGussetPlane, brace.BottomPlane);
                Plane alignPlane2 = new Plane(point2, brace.braceLine);
                brace.AlignPlane = GenericTools.PlaneDistMin(alignPlane1, alignPlane2, brace.FarPlane);
            }
            var item = m_braceData.First();
            var point = m_subFaceBraceplane.Move(s_lastBraceGap)
                .Intersect(m_baseGussetPlane, item.TopPlane);
            Plane alignPlane = new Plane(point, item.braceLine);
            item.AlignPlane = GenericTools.PlaneDistMin(item.AlignPlane, alignPlane, item.FarPlane);
        }
        //调整支撑之间的距离
        private void AdjustBraceAndBrace()
        {
            Vec3 point;
            Plane alignPlane;
            point = m_braceData[1].TopPlane.Move(m_braceGap)
                    .Intersect(m_braceData[0].BottomPlane, m_baseGussetPlane);
            alignPlane = new Plane(point, m_braceData[0].braceLine);
            m_braceData[0].AlignPlane = GenericTools.PlaneDistMin(alignPlane, 
                m_braceData[0].AlignPlane, m_braceData[0].FarPlane);
            for (int i = 1; i < m_braceLength - 1; i++)
            {
                List<Plane> planes = new List<Plane>();
                point = m_braceData[i - 1].BottomPlane.Move(m_braceGap)
                        .Intersect(m_braceData[i].TopPlane, m_baseGussetPlane);
                planes.Add(new Plane(point, m_braceData[i].braceLine));
                point = m_braceData[i + 1].TopPlane.Move(m_braceGap)
                        .Intersect(m_braceData[i].BottomPlane, m_baseGussetPlane);
                planes.Add(new Plane(point, m_braceData[i].braceLine));
                planes.Add(m_braceData[i].AlignPlane);
                alignPlane = GenericTools.PlanesDistMin(planes.ToArray(), m_braceData[i].FarPlane);
                m_braceData[i].AlignPlane = alignPlane;
            }
            var last = m_braceData.Last();
            point = m_braceData[m_braceLength - 2].BottomPlane.Move(m_braceGap)
                    .Intersect(last.TopPlane, m_baseGussetPlane);
            alignPlane = new Plane(point, last.braceLine);
            last.AlignPlane = GenericTools.PlaneDistMin(alignPlane, last.AlignPlane, last.FarPlane);
        }
        /// <summary>
        /// 对支撑组按照给定的方向向量进行排序，
        /// 排序原则为与方向向量的夹角从小到大的顺序
        /// </summary>
        private List<BraceData> SortBraceData(List<BraceData> m_braceData, Vec3 line)
        {
            List<BraceData> braceData = new List<BraceData>();
            int len = m_braceData.Count;
            //记录的数组
            int[] sortArr = new int[len];
            for (int i = 0; i < len; i++)
            {
                sortArr[i] = i;
            }
            //排序
            for (int i = 0; i < len - 1; i++)
            {
                for (int j = 0; j < len - i - 1; j++)
                {
                    double value1 = m_braceData[sortArr[j]].braceLine.Dot(line);
                    double value2 = m_braceData[sortArr[j + 1]].braceLine.Normalize().Dot(line);
                    if (value1 < value2)
                    {
                        //值交换
                        sortArr[j] = sortArr[j] ^ sortArr[j + 1];
                        sortArr[j + 1] = sortArr[j] ^ sortArr[j + 1];
                        sortArr[j] = sortArr[j] ^ sortArr[j + 1];
                    }
                }
            }
            for (int i = 0; i < len; i++)
            {
                BraceData brace = m_braceData[sortArr[i]];
                brace.SortNumber = i;
               
                braceData.Add(brace);
            }
            return braceData;
        }
    }
}
