﻿using System;
using System.Collections.Generic;
using System.Linq;
using CornerBoltedGusset57.Common;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;

namespace CornerBoltedGusset57.Handlers
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private PlateProperties m_plateProps = new PlateProperties();
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        private List<Vec3> m_gussetWeldPoints = new List<Vec3>();
        private PartBase m_retPlate;
        private List<BraceData> m_braceData = new List<BraceData>();
        private Plane m_baseGussetPlane;
        private Beam WeldPart;

        private Vec3 m_axisX;
        private Vec3 m_axisY;
        private Vec3 m_axisZ;

        //图形页参数
        private double m_plateDistanceToMain = double.MaxValue;
        private double m_plateDistanceToSub = double.MaxValue;

        //节点板页参数
        private int m_plateShape = 0;
        private int m_gussetWeldType = 0;
        private double m_horiNotch = double.MaxValue;
        private double m_vertNotch = double.MaxValue;

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            if (!ParseProps())
                return;
            //建立坐标系
            m_axisX = m_context.SubLine;
            m_axisY = m_context.SubFaceBracePlane.Normal.Normalize();
            m_axisZ = m_axisX.Cross(m_axisY);

            m_gussetWeldPoints = m_context.GussetWeldPoints;
            m_retPlate = m_context.RetPlate;

            this.CreatePartWelds();

            //创建支撑的焊缝
            if (m_weldProps[1].SizeUp > 0 || m_weldProps[1].SizeDown > 0)
            {
                m_baseGussetPlane = m_context.BaseGussetPlane;
                m_braceData = m_context.BraceData;
                this.CreateWelds();
            }   
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.GUSSET_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);
            m_plateDistanceToMain = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_GAP_MAIN_PART);
            m_plateDistanceToSub = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_GAP_SECONDARY_PART);

            m_plateShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_SHAPE);
            m_gussetWeldType = m_paramsHandler.GetIntValue(ParamNames.GUSSET_WELDING);
            m_horiNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_HORIZONTAL_NOTCH_DIMENSION);
            m_vertNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_VERTICAL_NOTCH_DIMENSION_FULL_NOTCH);

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }

            if (m_gussetWeldType == 3 || m_gussetWeldType == 2)
                WeldPart = m_context.MainPart;
            else
                WeldPart = m_context.SubPart;
            return true;
        }
        private void CreateWelds()
        {
            if (m_weldProps[1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                return;
                
            foreach (var brace in m_braceData)
            {
                var planes = GenericTools.GetBracePlane(brace.BeamPlanes, brace.ProfileType);
                if (planes == null)
                    return;
                if (FindWeldPlane(planes, m_baseGussetPlane) != null)
                {
                    //有一个面贴合节点板的情形
                    if (m_plateShape == 2)
                        WeldSquarePlate(brace);
                    else
                        WeldingBraceAndPlate(brace);
                    continue;
                }

                var dist = GenericTools.ArrayMax(GetPlaneDist(m_baseGussetPlane, brace.BackPlane, brace.FrontPlane));
                if (dist > brace.BackPlane.Dist(brace.FrontPlane.Origin))
                    continue;
                //节点板在支撑中间的情形
                if (m_context.EnableNotch)
                {
                    WeldingNotchBrace(brace, planes);
                }
                else if (m_context.EnableSpecialNotch)
                {
                    WeldingSpecialNotchBrace(brace, planes);
                }
            }
        }
        /// <summary>
        /// 确定焊接面
        /// </summary>
        private Plane FindWeldPlane(Plane[] planes, Plane plane)
        {
            var eps = 1e-3;//阈值
            var line = plane.Normal;
            foreach (var pls in planes)
            {
                if (pls.Normal.IsParallelTo(line) && pls.Dist(plane.Origin) < eps)
                    return pls;
            }
            return null;
        }
        private Plane[] FindNotchWeldPlane(Plane[] planes, Plane plane)
        {
            var line = plane.Normal;
            var list = new List<Plane>();
            foreach (var pls in planes)
            {
                if (Math.Abs(pls.Normal.Dot(line)) < 1e-3)
                    list.Add(pls);
            }
            return list.ToArray();
        }
        /// <summary>
        /// 焊接方形节点板
        /// </summary>
        private void WeldSquarePlate(BraceData brace)
        {
            var Target = m_context.TargetComp;
            var points = m_context.SquarePoints.ToArray();
            var point = points[2];
            var plane1 = new Plane(points[1], points[3] - point);
            var plane2 = new Plane(points[3], point - points[1]);

            List<Vec3> weldPoints = new List<Vec3>();
            if (brace.TopPlane.SignedDist(point) < 0 && brace.BottomPlane.SignedDist(point) < 0)
                weldPoints.Add(point);
            if (brace.TopPlane.SignedDist(point) < 0)
                weldPoints.Add(plane1.Intersect(brace.TopPlane, m_baseGussetPlane));
            else
                weldPoints.Add(plane2.Intersect(brace.TopPlane, m_baseGussetPlane));
            weldPoints.Add(brace.AlignPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
            weldPoints.Add(brace.AlignPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
            if (brace.BottomPlane.SignedDist(point) > 0)
                weldPoints.Add(plane1.Intersect(brace.BottomPlane, m_baseGussetPlane));
            else
                weldPoints.Add(plane2.Intersect(brace.BottomPlane, m_baseGussetPlane));
            for (int i = 0; i < weldPoints.Count; i++)
            {
                Weld curWeld = Target.CreatePolyWeld(WeldPart, brace.Beam,
                    new Vec3[] { weldPoints[i], weldPoints[(i + 1) % weldPoints.Count] }, true);
                m_weldProps[1].ApplyToWeld(curWeld);
            }
        }
        /// <summary>
        /// 创建支撑和节点板的焊缝
        /// </summary>
        private void WeldingBraceAndPlate(BraceData brace)
        {
            List<Vec3> weldPoints = new List<Vec3>();
            var Target = m_context.TargetComp;
            weldPoints.Add(brace.AlignPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
            weldPoints.Add(brace.GussetBorderPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
            weldPoints.Add(brace.GussetBorderPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
            weldPoints.Add(brace.AlignPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
            for (int i = 0; i < weldPoints.Count; i++)
            {
                Weld curWeld = Target.CreatePolyWeld(WeldPart, brace.Beam,
                    new Vec3[] { weldPoints[i], weldPoints[(i + 1) % weldPoints.Count] }, true);
                m_weldProps[1].ApplyToWeld(curWeld);
            }
        }
        /// <summary>
        /// 对开槽口的支撑进行焊接
        /// </summary>
        private void WeldingNotchBrace(BraceData brace, Plane[] planes)
        {
            var weldPlanes = FindNotchWeldPlane(planes, m_baseGussetPlane);
            var Target = m_context.TargetComp;
            var zBottom = m_baseGussetPlane.Move(m_context.Thickness);
            if (m_horiNotch == 0.0)
            {
                //焊接水平方向
                foreach (var pls in weldPlanes)
                {
                    var weldPoint1 = pls.Intersect(m_baseGussetPlane, brace.GussetBorderPlane);
                    var weldPoint2 = pls.Intersect(brace.GussetBorderPlane, zBottom);
                    Weld curWeld = Target.CreatePolyWeld(WeldPart, brace.Beam,
                    new Vec3[] { weldPoint1, weldPoint2 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld);
                }
            }

            if (m_vertNotch == 0.0)
            {
                //焊接垂直方向
                foreach (var pls in weldPlanes)
                {
                    var weldPoint1 = pls.Intersect(m_baseGussetPlane, brace.GussetBorderPlane);
                    var weldPoint2 = pls.Intersect(m_baseGussetPlane, brace.AlignPlane);
                    Weld curWeld1 = Target.CreatePolyWeld(WeldPart, brace.Beam,
                    new Vec3[] { weldPoint1, weldPoint2 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld1);

                    var weldPoint3 = pls.Intersect(zBottom, brace.GussetBorderPlane);
                    var weldPoint4 = pls.Intersect(zBottom, brace.AlignPlane);
                    Weld curWeld2 = Target.CreatePolyWeld(WeldPart, brace.Beam,
                    new Vec3[] { weldPoint3, weldPoint4 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld2);
                }
            }
        }
        /// <summary>
        /// 焊接特殊槽口的支撑
        /// </summary>
        private void WeldingSpecialNotchBrace(BraceData brace, Plane[] planes)
        {
            var weldPlanes = FindNotchWeldPlane(planes, m_baseGussetPlane);
            var Target = m_context.TargetComp;
            var zBottom = m_baseGussetPlane.Move(m_context.Thickness);

            double dist1 = brace.FrontPlane.Dist(m_baseGussetPlane.Origin);
            double dist2 = brace.FrontPlane.Dist(zBottom.Origin);

            //沿着Z轴负方向
            var plane = dist1 > dist2 ? m_baseGussetPlane : zBottom;
            //焊接垂直方向
            foreach (var pls in weldPlanes)
            {
                var weldPoint1 = pls.Intersect(plane, brace.GussetBorderPlane);
                var weldPoint2 = pls.Intersect(plane, brace.AlignPlane);
                Weld curWeld = Target.CreatePolyWeld(WeldPart, brace.Beam,
                new Vec3[] { weldPoint1, weldPoint2 }, true);
                m_weldProps[1].ApplyToWeld(curWeld);
            }

            if (m_horiNotch == 0.0)
            {
                //焊接水平方向
                foreach (var pls in weldPlanes)
                {
                    var weldPoint1 = pls.Intersect(m_baseGussetPlane, brace.GussetBorderPlane);
                    var weldPoint2 = pls.Intersect(brace.GussetBorderPlane, zBottom);
                    Weld curWeld = Target.CreatePolyWeld(WeldPart, brace.Beam,
                    new Vec3[] { weldPoint1, weldPoint2 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld);
                }
            }
        }
        //创建主次零件焊缝
        private void CreatePartWelds()
        {
            switch (m_gussetWeldType)
            {
                case 1:
                    if (m_weldProps[3].SizeUp > 0 || m_weldProps[3].SizeDown > 0)
                        WeldingSubGusset();
                    break;
                case 3:
                    if (m_weldProps[0].SizeUp > 0 || m_weldProps[0].SizeDown > 0)
                        WeldingMainGusset();
                    if (m_weldProps[3].SizeUp > 0 || m_weldProps[3].SizeDown > 0)
                        WeldingSubGusset();
                    break;
                case 0:
                default:
                    if (m_weldProps[0].SizeUp > 0 || m_weldProps[0].SizeDown > 0)
                        WeldingMainGusset();
                    break;
            }
        }
        //焊接主零件和节点板
        private bool WeldingMainGusset()
        {
            if (m_weldProps[0].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE || m_plateDistanceToMain > 0)
                return false;
            Vec3[] weldPoints = new Vec3[] { m_gussetWeldPoints[0], m_gussetWeldPoints.Last() };
            Weld curWeld1 = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, m_retPlate, weldPoints, true);
            m_weldProps[0].ApplyToWeld(curWeld1);

            Vec3 vertOffset;
            if (m_axisZ.Dot(m_context.BaseGussetPlane.Normal) > 0)
                vertOffset = m_axisZ;
            else
                vertOffset = -m_axisZ;
            Weld curWeld2 = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, m_retPlate, new Vec3[] {
                m_gussetWeldPoints[0] + vertOffset * m_plateProps.Thickness,
                m_gussetWeldPoints.Last() + vertOffset * m_plateProps.Thickness
            }, true);
            m_weldProps[0].ApplyToWeld(curWeld2);
            return true;
        }
        //焊接次零件和节点板
        private bool WeldingSubGusset()
        {
            if (m_weldProps[3].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE || m_plateDistanceToSub > 0)
                return false;
            if (m_gussetWeldPoints[2].Equals(m_gussetWeldPoints.Last()))
                return false;
            Vec3[] weldPoints = new Vec3[] { m_gussetWeldPoints[1], m_gussetWeldPoints[2] };
            Vec3 vertOffset;
            if (m_axisZ.Dot(m_context.BaseGussetPlane.Normal) > 0)
                vertOffset = m_axisZ;
            else
                vertOffset = -m_axisZ;

            var WeldMain = m_context.SubPart;
            var WeldSub = m_retPlate;
            PlateProperties flag = new PlateProperties();
            flag.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.STIFFENER_PROPS1));
            if (m_gussetWeldType == 3 || flag.Thickness > 0)
            {
                WeldMain = m_context.MainPart;
                WeldSub = m_context.SubPart;
            }
                
            Weld curWeld = m_context.TargetComp.CreatePolyWeld(WeldMain, WeldSub, weldPoints, true);
            Weld curWeld2 = m_context.TargetComp.CreatePolyWeld(WeldMain, WeldSub, new Vec3[] {
                m_gussetWeldPoints[1] + vertOffset * m_plateProps.Thickness,
                m_gussetWeldPoints[2] + vertOffset * m_plateProps.Thickness
            }, true);
            m_weldProps[3].ApplyToWeld(curWeld);
            m_weldProps[3].ApplyToWeld(curWeld2);
            return true;
        }
        /// <summary>
        /// 得到面集合中所有面与给定面的距离（均相互平行）
        /// </summary>
        private double[] GetPlaneDist(Plane plane, params Plane[] planes)
        {
            List<double> dis = new List<double>();
            foreach (var pls in planes)
            {
                if (pls == null)
                    return null;
                dis.Add(plane.Dist(pls.Origin));
            }
            return dis.ToArray(); ;
        }
    }
}
