﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using System;

namespace DiagonalJoint53
{
    public class PartsTools
    {
        /// <summary>
        /// 得到梁零件与给定向量符合点积条件的包围面组
        /// 不限定截面类型
        /// <param name="flag">true时为点积绝对值最大，false时为点积绝对值最小</param>
        /// </summary>
        /// <returns></returns>
        public static Plane[] DotExtremeBoundPlanes(BeamPlanes planes, Vec3 mainLine, bool flag)
        {
            if (planes == null)
                return null;
            var dot1 = Math.Abs(planes.TopBound.Normal.Normalize().Dot(mainLine));
            var dot2 = Math.Abs(planes.FrontBound.Normal.Normalize().Dot(mainLine));
            switch (dot1 > dot2 == flag)
            {
                case true:
                    var topDot = planes.TopBound.Normal.Dot(mainLine);
                    if(topDot > 0)
                        return new Plane[] { planes.TopBound, planes.BottomBound };
                    return new Plane[] { planes.BottomBound, planes.TopBound };
                default:
                    var backDot = planes.BackBound.Normal.Dot(mainLine);
                    if(backDot > 0)
                        return new Plane[] { planes.BackBound, planes.FrontBound };
                    return new Plane[] { planes.FrontBound, planes.BackBound };
            }
        }
        /// <summary>
        /// 得到平行的面集合中距离参照点最近的面
        /// </summary>
        /// <param name="point">参照点</param>
        /// <param name="planes">面集合</param>
        /// <returns></returns>
        public static Plane GetDistMinPlane(Vec3 point, params Plane[] planes)
        {
            int num = -1;
            double min = double.MaxValue;
            for (int i = 0; i < planes.Length; i++)
            {
                var temp = planes[i].Dist(point);
                if (temp < min)
                {
                    min = temp;
                    num = i;
                }
            }
            return planes[num];
        }
        /// <summary>
        /// 求给定数组中全部元素之和
        /// </summary>
        public static double SumArray(double[] arr)
        {
            if (arr == null)
                return 0.0;
            if (arr.Length == 0)
                return 0.0;
            var sum = 0.0;
            foreach (var item in arr)
            {
                sum += item;
            }
            return sum;
        }
        /// <summary>
        /// 在给定平面内旋转一个三维向量指定的角度。
        /// </summary>
        /// <param name="vector">要旋转的向量。</param>
        /// <param name="planeNormal">定义旋转平面的法线向量。</param>
        /// <param name="angleDegrees">向量在平面内旋转的角度，单位为度。</param>
        /// <returns>旋转后的向量。</returns>
        public static Vec3 RotateVectorInPlane(Vec3 vector, Vec3 planeNormal, double angleDegrees)
        {
            // 将平面法线标准化，确保它是单位向量
            Vec3 normalizedPlaneNormal = planeNormal.Normalize();

            // 将角度从度转换为弧度
            double angleRadians = Math.PI / 180 * angleDegrees;

            // 根据罗德里格斯旋转公式计算旋转的各个分量
            Vec3 rotatedVector = vector * Math.Cos(angleRadians) // 向量与余弦角度的乘积
                                    + normalizedPlaneNormal.Cross(vector) * Math.Sin(angleRadians) // 法线与向量的叉乘与正弦角度的乘积
                                    + normalizedPlaneNormal * normalizedPlaneNormal.Dot(vector) * (1 - Math.Cos(angleRadians)); // 法线与法线与向量点乘的乘积，再乘以1减去余弦角度
            return rotatedVector; // 返回旋转后的向量
        }
    }
}
