﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using System.Collections.Generic;
using System.Linq;

namespace EndSealingPlate1002
{
    public class GraphicHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private SteelBeam retBeam;
        private Vec3 pos0;
        private Vec3 pos1;

        private Vec3 axisX;
        private Vec3 axisY;
        private Vec3 axisZ;

        private PlateProperties m_plateProps = new PlateProperties();
        private double m_upGap = double.MaxValue;
        private double m_downGap = double.MaxValue;
        private double m_leftGap = double.MaxValue;
        private double m_rightGap = double.MaxValue;
        public GraphicHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;

            var upDir = m_context.TargetComp.CompDirUp;
            switch (upDir)
            {
                case LoongCAD.STC.Define.SpaceAxis.AXIS_NONE:
                    if (!CreateAutoPlate())
                        return false;
                    break;
                default:
                    if (!CreatePlate())
                        return false; 
                    break;
            }

            //存储上下文参数
            SaveParams();
            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.PIC_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);

            m_upGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_GAP);
            m_downGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_GAP);
            m_leftGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_GAP);
            m_rightGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_GAP);

            return true;
        }
        /// <summary>
        /// 在向上方向为非自动时创建端板
        /// </summary>
        private bool CreatePlate()
        {
            if (m_plateProps.Thickness == 0 || m_plateProps.Width == 0 || m_plateProps.Height == 0)
                return false;

            //建立坐标系
            var Target = m_context.TargetComp;
            axisX = Target.LocalMatrix.Z.Normalize();
            if (axisX.IsParallelTo(m_context.MainLine))
                return false;
            var dirs = GetAlignDir();
            axisZ = dirs.First().Normalize();
            axisY = dirs.Last().Normalize();
            var point = m_context.TargetComp.CtrlPoints[0].WorldPos;

            var basePlane = new Plane(point, axisZ);
            //进行对齐
            Target.CreateAlignPlane(m_context.MainPart, basePlane, true);

            //计算构建板型梁的控制点
            var mainPlanes = m_context.MainPlanes;
            var points = GetCtrlPoints(axisZ, mainPlanes, basePlane);
            var pos = points.Last() / 2 - points.First() / 2;
            if(m_plateProps.Height == double.MaxValue)
            {
                var dot = axisX.Dot(pos.Normalize());
                //沿着X方向的为pos0
                if(dot > 0)
                {
                    pos0 = points.Last() - axisX * m_upGap;
                    pos1 = points.First() + axisX * m_downGap;
                }
                else
                {
                    pos0 = points.First() - axisX * m_upGap;
                    pos1 = points.Last() + axisX * m_downGap;
                }
            }
            else
            {
                pos0 = pos + axisX * m_plateProps.Height / 2;
                pos1 = pos - axisX * m_plateProps.Height / 2;
            }
            
            double main_b = mainPlanes.FrontBound.Dist(mainPlanes.BackBound.Origin);
            var b = main_b;
            if (m_plateProps.Width == double.MaxValue)
            {
                b = b - m_leftGap - m_rightGap;
                pos0 = pos0 + axisY * m_rightGap / 2 - axisY * m_leftGap / 2;
                pos1 = pos1 + axisY * m_rightGap / 2 - axisY * m_leftGap / 2;
            }
            else
                b = m_plateProps.Width;
            string profile = "PL" + m_plateProps.Thickness + "*" + b;
            retBeam = m_context.TargetComp.CreateSteelBeam(profile, pos0, pos1, true);
            if (retBeam == null)
                return false;

            retBeam.WorkAxis = new Mat43(axisX, axisY, axisZ, Vec3.ZERO);
            retBeam.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            retBeam.Rotation = 0;
            retBeam.VertOffsetSide = LoongCAD.STC.Define.ObjectSide.SIDE_BACK;
            retBeam.Class = m_context.TargetComp.CompClass;
            retBeam.Refresh();
            m_plateProps.ApplyToPart(retBeam);
            return true;
        }
        /// <summary>
        /// 在向上方向为自动时创建端板
        /// </summary>
        private bool CreateAutoPlate()
        {
            if (m_plateProps.Thickness == 0 || m_plateProps.Width == 0 || m_plateProps.Height == 0)
                return false;

            var point = m_context.TargetComp.CtrlPoints[0].WorldPos;
            axisZ = m_context.MainLine;
            var basePlane = new Plane(point, axisZ);
            //进行对齐
            m_context.TargetComp.CreateAlignPlane(m_context.MainPart, basePlane, true);

            var mainPlanes = m_context.MainPlanes;
            double main_b = mainPlanes.FrontBound.Dist(mainPlanes.BackBound.Origin);
            var middlePlane = mainPlanes.BackBound.Move(- main_b / 2);
            if(m_plateProps.Height == double.MaxValue)
            {
                pos0 = basePlane.Intersect(middlePlane, mainPlanes.TopBound.Move(- m_upGap));
                pos1 = basePlane.Intersect(middlePlane, mainPlanes.BottomBound.Move(- m_downGap));
            }
            else
            {
                double main_h = mainPlanes.TopBound.Dist(mainPlanes.BottomBound.Origin);
                var hPlane = mainPlanes.TopBound.Move(- main_h / 2);
                pos0 = basePlane.Intersect(hPlane.Move(m_plateProps.Height / 2), middlePlane);
                pos1 = basePlane.Intersect(hPlane.Move(-m_plateProps.Height / 2), middlePlane);
            }
            var b = main_b;
            if (m_plateProps.Width != double.MaxValue)
                b = m_plateProps.Width;
            else
            {
                b = b - m_leftGap - m_rightGap;
                pos0 = pos0 - mainPlanes.BackBound.Normal.Normalize() * m_rightGap / 2
                    - mainPlanes.FrontBound.Normal.Normalize() * m_leftGap / 2;
                pos1 = pos1 - mainPlanes.BackBound.Normal.Normalize() * m_rightGap / 2
                    - mainPlanes.FrontBound.Normal.Normalize() * m_leftGap / 2;
            } 
            string profile = "PL" + m_plateProps.Thickness + "*" + b;

            retBeam = m_context.TargetComp.CreateSteelBeam(profile, pos0, pos1, true);
            if (retBeam == null)
                return false;

            axisY = mainPlanes.TopBound.Normal.Normalize();
            axisX = mainPlanes.FrontBound.Normal.Normalize();
            retBeam.WorkAxis = new Mat43(axisX, axisY, axisZ, Vec3.ZERO);
            retBeam.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            retBeam.Rotation = 0;
            retBeam.VertOffsetSide = LoongCAD.STC.Define.ObjectSide.SIDE_BACK;
            retBeam.Class = m_context.TargetComp.CompClass;
            retBeam.Refresh();
            m_plateProps.ApplyToPart(retBeam);
            return true;
        }
        private void SaveParams()
        {
            m_context.RetBeam = retBeam;
            m_context.X = axisX;
            m_context.Y = axisY;
            m_context.Z = axisZ;
            m_context.Pos0 = pos0;
        }
        /// <summary>
        /// 得到向上方向非自动时端板的构建平面法线
        /// </summary>
        private List<Vec3> GetAlignDir()
        {
            var Target = m_context.TargetComp;
            Vec3[] normals = new Vec3[]
            {
                Target.LocalMatrix.X,
                Target.LocalMatrix.Y,
                - Target.LocalMatrix.X,
                - Target.LocalMatrix.Y
            };
            int num = -1;
            double max = -2;
            for(int i = 0; i < normals.Length; i++)
            {
                var temp = normals[i].Dot(m_context.MainLine);
                if(temp > max)
                {
                    max = temp;
                    num = i;
                }
            }
            List<Vec3> dir = new List<Vec3>();
            dir.Add(normals[num]);
            if (num == 0 || num == 2)
                dir.Add(normals[1]);
            else
                dir.Add(normals[0]);
            return dir;
        }
        /// <summary>
        /// 得到向上方向非自动时端板的控制点
        /// </summary>
        private Vec3[] GetCtrlPoints(Vec3 line, BeamPlanes mainPlanes, Plane plane)
        {
            List<Vec3> points = new List<Vec3>();
            var planes = new Plane[]
            {
                mainPlanes.BottomBound,
                mainPlanes.TopBound,
                mainPlanes.FrontBound,
                mainPlanes.BackBound
            };
            int num = -1;
            double max = -2;
            for(int i = 0; i < planes.Length; i++)
            {
                var temp = planes[i].Normal.Normalize().Dot(line);
                if(temp > max)
                {
                    max = temp;
                    num = i;
                }
            }
            Plane middlePlane, bottomPlane;
            if(num == 0 || num == 1)
            {
                var dist = planes[2].Dist(planes[3].Origin);
                middlePlane = planes[2].Move(- dist / 2);
                bottomPlane = num == 0 ? planes[1] : planes[0];
            }
            else
            {
                var dist = planes[0].Dist(planes[1].Origin);
                middlePlane = planes[0].Move(- dist / 2);
                bottomPlane = num == 2 ? planes[3] : planes[2];
            }
            var topPlane = planes[num];
            points.Add(topPlane.Intersect(plane, middlePlane));
            points.Add(bottomPlane.Intersect(plane, middlePlane));
            return points.ToArray();
        }
    }
}
