﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System;
using System.Collections.Generic;

namespace ExtendedTrapezoidalPlate103
{
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private ConnectionComponent m_comp = null;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine = Vec3.ZERO;

        private BeamPlanes m_mainPlanes = null;

        private Beam m_subPart = null;

        private Vec3 m_subLine = Vec3.ZERO;

        private BeamPlanes m_subPlanes = null;

        private Vec3 m_worldDir = Vec3.ZERO;
        //远近决定于次零件面向我们的相对位置
        private Plane m_subNearPlane = null;

        private Plane m_subFarPlane = null;

        private Plane m_mainNearPlane = null;

        private Plane m_mainFarPlane = null;
        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 SubLine { get => m_subLine; set => m_subLine = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane SubAlignPlane { get; set; }
        public Plane SubNearPlane { get => m_subNearPlane; set => m_subNearPlane = value; }
        public Plane SubFarPlane { get => m_subFarPlane; set => m_subFarPlane = value; }
        public Plane MainNearPlane { get => m_mainNearPlane; set => m_mainNearPlane = value; }
        public Plane MainFarPlane { get => m_mainFarPlane; set => m_mainFarPlane = value; }
        //主零件近侧包围面移动对齐距离得到的面
        public Plane VertAlignPlane { get; set; } 
        public List<PartBase> Plates { get; set; }
        public List<PartBase> Stiffs { get; set; }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (comp.Subs == null)
                return false;
            Beam subPart = comp.Subs.First() as Beam;
            if (mainPart == null || subPart == null)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);
            m_subPart = subPart;
            m_subPlanes = BeamPlanes.CreateFromBeam(subPart);
            //验证主零件
            if (m_mainPlanes.TopFlangeInside == null || m_mainPlanes.BottomFlangeInside == null
                || m_mainPlanes.FrontWeb == null || m_mainPlanes.BackWeb == null)
                return false;
            //验证次零件
            if (m_subPlanes.BackWeb == null || m_subPlanes.FrontWeb == null
                || m_subPlanes.FrontWebInside == null || m_subPlanes.BackWebInside == null)
                return false;
            //开槽口需要
            if (m_subPlanes.TopFlange == null || m_subPlanes.BottomFlange == null)
                return false;
            if (!DefineSubLine(subPart))
                return false;
            //主零件和次零件平行时不创建组件
            if (m_mainLine.IsParallelTo(m_subLine))
                return false;
            ParseWorldDir();
            if (!ParseSubWeb())
                return false;
            if (!ParseBuildPlane())
                return false;
            return true;
        }
        private void ParseWorldDir()
        {
            var axis = new Vec3[] {
                m_comp.WorkAxis.X, m_comp.WorkAxis.Y, m_comp.WorkAxis.Z
            };
            double max = 0.0;
            int index = 0;
            //基于构件连接板的腹板面出发
            var normal = m_subPlanes.BackWeb.Normal.Normalize();
            for(int i = 0; i < axis.Length; i++)
            {
                var temp = Math.Abs(normal.Dot(axis[i]));
                if(temp > max)
                {
                    max = temp;
                    index = i;
                }
            }
            m_worldDir = axis[index];
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var dir = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!dir.HasValue)
                return false;
            m_mainLine = dir.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 调整次零件的构建方向
        /// </summary>
        private bool DefineSubLine(Beam subPart)
        {
            //调整为靠近主零件的控制点为第一控制点
            var point1 = subPart.CtrlPoints?.Last()?.WorldPos;
            var point2 = subPart.CtrlPoints?.First()?.WorldPos;
            var dir = point1 - point2;
            if (!dir.HasValue)
                return false;
            Vec3 origin = m_mainPart.CtrlPoints.First().WorldPos;
            var dir1 = point1.Value - origin;
            var dir2 = point2.Value - origin;
            double dist1 = dir1.Cross(m_mainLine).Length;
            double dist2 = dir2.Cross(m_mainLine).Length;
            if (dist1 > dist2)
                m_subLine = dir.Value.Normalize();
            else
                m_subLine = - dir.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 基于组件冒尖方向确定次零件的上下内翼缘面 
        /// 基于世界坐标系确定次零件的远近腹板面
        /// 远面指的是背向我们的那一侧
        /// </summary>
        private bool ParseSubWeb()
        {
            //腹板面
            if (m_subPlanes.BackWeb == null ||
                m_subPlanes.FrontWeb == null)
                return false;
            if(m_subPlanes.BackWeb.Normal.Dot(m_worldDir) > 0)
            {
                m_subFarPlane = m_subPlanes.BackWeb;
                m_subNearPlane = m_subPlanes.FrontWeb;
            }
            else
            {
                m_subFarPlane = m_subPlanes.FrontWeb;
                m_subNearPlane = m_subPlanes.BackWeb;
            }
            return true;
        }
        private bool ParseBuildPlane()
        {
            //靠近主零件且面向次零件的那一侧为近侧 与之相反为远测
            if (m_mainPlanes.TopFlange == null || m_mainPlanes.BottomFlange == null)
                return false;
            var flag1 = m_subLine.Dot(m_mainPlanes.BackWeb.Normal.Normalize());
            var flag2 = m_subLine.Dot(m_mainPlanes.TopFlange.Normal.Normalize());
            do
            {
                if (Math.Abs(flag1) < Math.Abs(flag2))
                    break;
                if (flag1 > 0)
                {
                    m_mainNearPlane = m_mainPlanes.BackWeb;
                    m_mainFarPlane = m_mainPlanes.FrontWeb;
                }
                else
                {
                    m_mainNearPlane = m_mainPlanes.FrontWeb;
                    m_mainFarPlane = m_mainPlanes.BackWeb;
                } 
            } while (false);

            do
            {
                if (Math.Abs(flag1) > Math.Abs(flag2))
                    break;
                if (flag2 > 0)
                {
                    m_mainNearPlane = m_mainPlanes.TopFlange;
                    m_mainFarPlane = m_mainPlanes.BottomFlange;
                }
                else
                {
                    m_mainNearPlane = m_mainPlanes.BottomFlange;
                    m_mainFarPlane = m_mainPlanes.TopFlange;
                }   
            } while (false);
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
            m_subPart = null;
            m_subPlanes = null;
            m_mainNearPlane = null;
            m_mainFarPlane = null;
            m_subNearPlane = null;
            m_subFarPlane = null;
        }
    }
}
