﻿using LoongCAD.Geometry;
using System.Linq;
using System;

namespace ExtendedTrapezoidalPlate103
{
    public class EndCutHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private double m_alignGap = 0.0;
        private Plane m_nearPlane = null;
        private Plane m_subTailPlane = null;  //底面
        private Plane m_subUpPlane = null;
        private Plane m_subDownPlane = null;
        private Plane m_subRefPlane = null;
        public EndCutHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            if (!ParseProps())
                return;
            ParseMainNearPlane();
            m_context.VertAlignPlane = m_nearPlane.Move(m_alignGap);
            BeamEndCut();
        }
        private bool ParseProps()
        {
            Vec3 dir = m_context.MainLine;
            var subPlanes = m_context.SubPlanes;
            var dot1 = subPlanes.TopBound.Normal.Normalize().Dot(dir);
            var dot2 = subPlanes.BackBound.Normal.Normalize().Dot(dir);
            if(Math.Abs(dot1) > Math.Abs(dot2))
            {
               //up 与down可以不按照空间的位置区分
                m_subUpPlane = subPlanes.TopBound;
                m_subDownPlane = subPlanes.BottomBound;
                //ref只是为了三面定点
                m_subRefPlane = subPlanes.BackBound;
            }
            else
            {
                m_subUpPlane = subPlanes.BackBound;
                m_subDownPlane = subPlanes.FrontBound;
                m_subRefPlane = subPlanes.TopBound;
            }
            m_alignGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_PARTS_CUT_GAP);
            //对齐后要避免次零件与主零件碰撞 为此解析次零件的底面
            var point1 = m_context.SubPart.CtrlPoints.First().WorldPos;
            var point2 = m_context.SubPart.CtrlPoints.Last().WorldPos;
            if (m_context.SubLine.Dot(point2 - point1) > 0)
                m_subTailPlane = new Plane(point2, m_context.SubLine);
            else
                m_subTailPlane = new Plane(point1, m_context.SubLine);
            return true;
        }
        /// <summary>
        /// 梁端切割
        /// </summary>
        private void BeamEndCut()
        {
            int endCutType = m_paramsHandler.GetIntValue(ParamNames.PIC_BEAM_END_CUT);
            Plane alignPlane;
            if(endCutType == 2)
            {
                //正方形
                var refPlane = m_nearPlane.Move(m_alignGap);
                Vec3 point1 = refPlane.Intersect(m_subUpPlane, m_subRefPlane);
                Vec3 point2 = refPlane.Intersect(m_subDownPlane, m_subRefPlane);
                Plane plane1 = new Plane(point1, m_context.SubLine);
                Plane plane2 = new Plane(point2, m_context.SubLine);
                //避免次零件与主零件碰撞
                alignPlane = GetPlaneDistMin(plane1, plane2, m_subTailPlane);
            }
            else
            {
                //斜角
                alignPlane = m_nearPlane.Move(m_alignGap);
            }
            m_context.TargetComp.CreateAlignPlane(m_context.SubPart, alignPlane, true);
            m_context.SubAlignPlane = alignPlane;
        }
        private Plane GetPlaneDistMin(Plane plane1, Plane plane2, Plane refPlane)
        {
            if (plane1 == null || plane2 == null || refPlane == null)
                return null;
            var dist1 = plane1.Dist(refPlane.Origin);
            var dist2 = plane2.Dist(refPlane.Origin);
            return dist1 < dist2 ? plane1 : plane2;
        }
        private void ParseMainNearPlane()
        {
            //near 就是主零件面向次零件较近的侧面
            var mainPlanes = m_context.MainPlanes;
            Vec3 subLine = m_context.SubLine;
            var flag1 = subLine.Dot(mainPlanes.BackBound.Normal.Normalize());
            var flag2 = subLine.Dot(mainPlanes.TopBound.Normal.Normalize());
            do
            {
                if (Math.Abs(flag1) < Math.Abs(flag2))
                    break;
                if (flag1 > 0)
                    m_nearPlane = mainPlanes.BackBound;
                else
                    m_nearPlane = mainPlanes.FrontBound;
            } while (false);

            do
            {
                if (Math.Abs(flag1) > Math.Abs(flag2))
                    break;
                if (flag2 > 0)
                    m_nearPlane = mainPlanes.TopBound;
                else
                    m_nearPlane = mainPlanes.BottomBound;
            } while (false);
        }
    }
}
