﻿using LoongCAD.STC;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;

namespace ExtendedTrapezoidalPlate103
{
    public class NotchHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        private AutomaticNotchSettings m_autoSettings = null;
        private ManualNotchSettings m_manualSettings = null;

        public NotchHandler(ComponentInstanceContext ctx, ParamsHandler paramsHandler)
        {
            m_context = ctx;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            if (!this.ParseProps())
                return false;
            if (m_context.MainPart == null || m_context.SubPart == null ||
                m_context.SubAlignPlane == null)
                return false;

            AutomaticNotchCreator autoCreator = new AutomaticNotchCreator(m_autoSettings, m_context.TargetComp);
            autoCreator.CreateNotches(m_context.MainPart, m_context.SubPart, m_context.SubAlignPlane);

            ManualNotchCreator manualCreator = new ManualNotchCreator(m_manualSettings, m_context.TargetComp);
            manualCreator.CreateNotches(m_context.MainPart, m_context.SubPart, m_context.SubAlignPlane);
            return true;
        }

        private bool ParseProps()
        {
            this.ParseAutoNotchProps();
            this.ParseManualNotchProps();
            return true;
        }

        private void ParseAutoNotchProps()
        {
            m_autoSettings = new AutomaticNotchSettings();
            // 槽口形状
            {
                int notchShape = m_paramsHandler.GetIntValue(ParamNames.NOTCH_SHAPE);
                switch (notchShape)
                {
                    case 2:
                        m_autoSettings.NotchShape = AutomaticNotch.Shape.SQUARE_TO_SECONDARY;
                        break;
                    case 3:
                        m_autoSettings.NotchShape = AutomaticNotch.Shape.HIBIRD;
                        break;
                    case 4:
                        m_autoSettings.NotchShape = AutomaticNotch.Shape.NONE;
                        break;
                    case 1:
                    default:
                        m_autoSettings.NotchShape = AutomaticNotch.Shape.SQUARE_TO_MAIN;
                        break;
                }
            }
            // 翼板切割形状
            {
                int flangeCutShape = m_paramsHandler.GetIntValue(ParamNames.NOTCH_FLANGE_CUT_SHAPE);
                switch (flangeCutShape)
                {
                    case 2:
                        m_autoSettings.FlangeCutShape = AutomaticNotch.FlangeCutShape.SQUARE_TO_SECONDARY;
                        break;
                    case 1:
                    default:
                        m_autoSettings.FlangeCutShape = AutomaticNotch.FlangeCutShape.PARALLEL_TO_MAIN;
                        break;
                }
            }
            // 槽口位置
            {
                int notchPos = m_paramsHandler.GetIntValue(ParamNames.NOTCH_POSITION);
                switch (notchPos)
                {
                    case 2:
                        m_autoSettings.NotchPosition = AutomaticNotch.Position.ABOVE_FLANGE;
                        break;
                    case 1:
                    default:
                        m_autoSettings.NotchPosition = AutomaticNotch.Position.BELOW_FLANGE;
                        break;
                }
            }
            // 槽口尺寸
            {
                // 尺寸测量方式
                {
                    int sizeMode = m_paramsHandler.GetIntValue(ParamNames.NOTCH_SIZE_MODE);
                    switch (sizeMode)
                    {
                        case 2:
                            m_autoSettings.SizeMode = AutomaticNotch.SizeMode.FROM_WEB_AND_FLANGE;
                            break;
                        case 1:
                        default:
                            m_autoSettings.SizeMode = AutomaticNotch.SizeMode.FROM_INSIDE_AND_EDGE;
                            break;
                    }
                }
                // 槽口尺寸
                double xNotchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_SIZE_X);
                double yNotchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_SIZE_Y);
                m_autoSettings.NotchSize = new Vec2(xNotchSize, yNotchSize);
            }
            // 槽口尺寸舍入
            do
            {
                //
                {
                    int sizeRounding = m_paramsHandler.GetIntValue(ParamNames.NOTCH_SIZE_ROUNDING);
                    switch (sizeRounding)
                    {
                        case 2:
                            m_autoSettings.SizeRounding = true;
                            break;
                        case 1:
                        default:
                            m_autoSettings.SizeRounding = false;
                            break;
                    }
                }
                if (!m_autoSettings.SizeRounding)
                {
                    break;
                }

                //
                double xRoundingBase = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_ROUNDING_BASE_X);
                double yRoundingBase = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_ROUNDING_BASE_Y);
                m_autoSettings.RoundingBase = new Vec2(xRoundingBase, yRoundingBase);
            } while (false);
            // 槽口折角
            do
            {
                // 折角类型
                {
                    int chamferType = m_paramsHandler.GetIntValue(ParamNames.NOTCH_CHAMFER_TYPE);
                    switch (chamferType)
                    {
                        case 2:
                            m_autoSettings.NotchChamfer = Define.CornerCutType.CORNER_CUT_LINE;
                            break;
                        case 3:
                            m_autoSettings.NotchChamfer = Define.CornerCutType.CORNER_CUT_ROUND;
                            break;
                        case 1:
                        default:
                            m_autoSettings.NotchChamfer = Define.CornerCutType.CORNER_CUT_NONE;
                            break;
                    }
                }
                if (m_autoSettings.NotchChamfer == Define.CornerCutType.CORNER_CUT_NONE)
                    break;

                // 折角半径
                m_autoSettings.NotchChamferRadius = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_CHAMFER_RADIUS);
            } while (false);
        }

        private void ParseManualNotchProps()
        {
            m_manualSettings = new ManualNotchSettings();
            // 上翼板处的手动槽口属性
            do
            {
                // 槽口侧
                {
                    int notchSide = m_paramsHandler.GetIntValue(ParamNames.NOTCH_MANUAL_TOP_SIDE);
                    switch (notchSide)
                    {
                        case 1:
                            { // 自动
                              // FIXME: 有待完善自动槽口侧逻辑
                                m_manualSettings.TopNotchSide = Define.AwaySide.BOTH;
                            }
                            break;
                        case 3:
                            m_manualSettings.TopNotchSide = Define.AwaySide.NEAR;
                            break;
                        case 4:
                            m_manualSettings.TopNotchSide = Define.AwaySide.FAR;
                            break;
                        case 2:
                        default:
                            m_manualSettings.TopNotchSide = Define.AwaySide.BOTH;
                            break;
                    }
                }
                if (m_manualSettings.TopNotchSide == Define.AwaySide.NONE)
                    break;

                // 槽口形状
#if false
                bool bAutoShape = false;
#endif
                {
                    int notchShape = m_paramsHandler.GetIntValue(ParamNames.NOTCH_MANUAL_TOP_SHAPE);
                    switch (notchShape)
                    {
                        case 1:
                            {
                                //bAutoShape = true;
                                m_manualSettings.TopNotchShape = ManualNotch.Shape.FULL;
                            }
                            break;
                        case 2:
                            m_manualSettings.TopNotchShape = ManualNotch.Shape.CHAMFER;
                            break;
                        case 4:
                            m_manualSettings.TopNotchShape = ManualNotch.Shape.NONE;
                            break;
                        case 5:
                            m_manualSettings.TopNotchShape = ManualNotch.Shape.HALF;
                            break;
                        case 6:
                            m_manualSettings.TopNotchShape = ManualNotch.Shape.SPECIFIED;
                            break;
                        case 3:
                        default:
                            m_manualSettings.TopNotchShape = ManualNotch.Shape.FULL;
                            break;
                    }
                }
                if (m_manualSettings.TopNotchShape == ManualNotch.Shape.NONE)
                    break;

                // 槽口深度
                {
                    // 深度模式
                    {
                        int depthMode = m_paramsHandler.GetIntValue(ParamNames.NOTCH_MANUAL_TOP_DEPTH_MODE);
                        switch (depthMode)
                        {
                            case 2:
                                m_manualSettings.TopDepthMode = ManualNotch.DepthMode.FROM_WEB;
                                break;
                            case 1:
                            default:
                                m_manualSettings.TopDepthMode = ManualNotch.DepthMode.FROM_FLANGE;
                                break;
                        }
                    }
                    // 深度值
                    m_manualSettings.TopNotchDepth = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MANUAL_TOP_DEPTH);
                }

                // 槽口尺寸
                {
                    double xNotchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MANUAL_TOP_SIZE_X);
                    double yNotchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MANUAL_TOP_SIZE_Y);
                    if (xNotchSize == double.MaxValue)
                    {
                        xNotchSize = 0.0;
                    }
                    if (yNotchSize == double.MaxValue)
                    {
                        var subTopThickness = 0.0;
                        if (m_context.SubPlanes.TopFlange != null 
                            && m_context.SubPlanes.TopFlangeInside != null)
                            subTopThickness = m_context.SubPlanes.TopFlange
                                .Dist(m_context.SubPlanes.TopFlangeInside.Origin);
                        yNotchSize = subTopThickness * 2.0 + 5.0;
                    }
                    m_manualSettings.TopNotchSize = new Vec2(xNotchSize, yNotchSize);
                }
            } while (false);

            // 下翼板处的手动槽口属性
            do
            {
                // 槽口侧
                {
                    int notchSide = m_paramsHandler.GetIntValue(ParamNames.NOTCH_MANUAL_BOTTOM_SIDE);
                    switch (notchSide)
                    {
                        case 1:
                            { // 自动
                              // FIXME: 有待完善自动槽口侧逻辑
                                m_manualSettings.BottomNotchSide = Define.AwaySide.BOTH;
                            }
                            break;
                        case 3:
                            m_manualSettings.BottomNotchSide = Define.AwaySide.NEAR;
                            break;
                        case 4:
                            m_manualSettings.BottomNotchSide = Define.AwaySide.FAR;
                            break;
                        case 2:
                        default:
                            m_manualSettings.BottomNotchSide = Define.AwaySide.BOTH;
                            break;
                    }
                }
                if (m_manualSettings.BottomNotchSide == Define.AwaySide.NONE)
                    break;

                // 槽口形状
#if false
                bool bAutoShape = false;
#endif
                {
                    int notchShape = m_paramsHandler.GetIntValue(ParamNames.NOTCH_MANUAL_BOTTOM_SHAPE);
                    switch (notchShape)
                    {
                        case 1:
                            {
                                //bAutoShape = true;
                                m_manualSettings.BottomNotchShape = ManualNotch.Shape.FULL;
                            }
                            break;
                        case 2:
                            m_manualSettings.BottomNotchShape = ManualNotch.Shape.CHAMFER;
                            break;
                        case 4:
                            m_manualSettings.BottomNotchShape = ManualNotch.Shape.NONE;
                            break;
                        case 5:
                            m_manualSettings.BottomNotchShape = ManualNotch.Shape.HALF;
                            break;
                        case 6:
                            m_manualSettings.BottomNotchShape = ManualNotch.Shape.SPECIFIED;
                            break;
                        case 3:
                        default:
                            m_manualSettings.BottomNotchShape = ManualNotch.Shape.FULL;
                            break;
                    }
                }
                if (m_manualSettings.BottomNotchShape == ManualNotch.Shape.NONE)
                    break;

                // 槽口深度
                {
                    // 深度模式
                    {
                        int depthMode = m_paramsHandler.GetIntValue(ParamNames.NOTCH_MANUAL_BOTTOM_DEPTH_MODE);
                        switch (depthMode)
                        {
                            case 2:
                                m_manualSettings.BottomDepthMode = ManualNotch.DepthMode.FROM_WEB;
                                break;
                            case 1:
                            default:
                                m_manualSettings.BottomDepthMode = ManualNotch.DepthMode.FROM_FLANGE;
                                break;
                        }
                    }
                    // 深度值
                    m_manualSettings.BottomNotchDepth = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MANUAL_BOTTOM_DEPTH);
                }

                // 槽口尺寸
                {
                    double xNotchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MANUAL_BOTTOM_SIZE_X);
                    double yNotchSize = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MANUAL_BOTTOM_SIZE_Y);
                    if (xNotchSize == double.MaxValue)
                    {
                        xNotchSize = 0.0;
                    }
                    if (yNotchSize == double.MaxValue)
                    {
                        var subBottomThickness = 0.0;
                        if (m_context.SubPlanes.BottomFlange != null && 
                            m_context.SubPlanes.BottomFlangeInside != null)
                            subBottomThickness = m_context.SubPlanes.BottomFlange
                                .Dist(m_context.SubPlanes.BottomFlangeInside.Origin);
                        yNotchSize = subBottomThickness * 2.0 + 5.0;
                    }
                    m_manualSettings.BottomNotchSize = new Vec2(xNotchSize, yNotchSize);
                }
            } while (false);
        }
    }
}
