﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace ExtendedTrapezoidalPlate103
{
    public class PlateHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();
        private Vec3 m_subLine = Vec3.ZERO;
        private Vec3 m_dir = Vec3.ZERO;
        private Plane m_mainNearPlane = null;
        private Plane m_subFarPlane = null;
        private Plane m_subNearPlane = null;
        private Plane m_subUpPlane = null;
        private Plane m_subDownPlane = null;
        private Plane m_vertAlignPlane = null;

        //图形页参数
        private double m_upChamferHeight = double.MaxValue;
        private double m_upChamferGap = double.MaxValue;
        private double m_downChamferHeight = double.MaxValue;
        private double m_downChamferGap = double.MaxValue;

        private double horiLen = 0.0;  //螺栓面板水平方向长度
        private double maxLen = 0.0;  //连接板最大长度
        private List<PartBase> plates = new List<PartBase>();
        private PlateProperties m_plateProps = new PlateProperties();
        private int m_plateDir = 0;
        private int m_platePos = 0;
        private double m_plateGap = 0.0;
        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;
            if (!CreatePlate())
                return false;
            CreateBolts();
            BeamFilletCut();
            m_context.Plates = plates;
            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);
            //得到螺栓属性
            BoltParse boltParse = new BoltParse(m_paramsHandler);
            m_boltData = boltParse.GetBoltParams();
            if (m_boltData == null)
                return false;
            //初始化上下文属性
            m_subLine = m_context.SubLine;
            m_mainNearPlane = m_context.MainNearPlane;
            m_subNearPlane = m_context.SubNearPlane;
            m_subFarPlane = m_context.SubFarPlane;
            //翼缘面
            var subPlanes = m_context.SubPlanes;
            if (subPlanes.TopFlangeInside == null ||
                subPlanes.BottomFlangeInside == null)
                return false;
            m_dir = m_context.TargetComp.LocalMatrix.Z;
            //空间位置中的上下内翼缘面
            if (subPlanes.TopFlangeInside.Normal.Dot(m_dir) > 0)
            {
                m_subDownPlane = subPlanes.TopFlangeInside;
                m_subUpPlane = subPlanes.BottomFlangeInside;
            }
            else
            {
                m_subUpPlane = subPlanes.TopFlangeInside;
                m_subDownPlane = subPlanes.BottomFlangeInside;
            }
            m_vertAlignPlane = m_context.VertAlignPlane;
            AutoBoltNum();
            m_plateDir = m_paramsHandler.GetIntValue(ParamNames.PLATE_DIR);
            m_platePos = m_paramsHandler.GetIntValue(ParamNames.PLATE_POSITION);
            m_plateGap = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_GAP);
            m_upChamferHeight = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_CHAMFER_PART_HEIGHT);
            m_downChamferHeight = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_CHAMFER_PART_HEIGHT);
            return true;
        }
        //根梁宽自适应纵向螺栓数量
        private void AutoBoltNum()
        {
            int vertCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
            var arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
            if (vertCount == int.MaxValue)
            {
                vertCount = 2;
                var subWide = m_subUpPlane.Dist(m_subDownPlane.Origin);
                //70.5 = 10.5 + 60 -- 10.5为板与上翼缘内侧间距  60为初始螺栓间距
                //80.0 = 40.0 + 40.0 -- 40.0为初始螺栓组与板上下间距
                var len = subWide - 70.5 - 80.0;
                len -= 76.2;
                while(len > 0)
                {
                    vertCount++;
                    len -= 76.2;
                }
                m_boltData.VertStride = HandlerStrideX(arrX, vertCount);
            }
            else
            {
                m_boltData.VertStride = HandlerStrideX(arrX, vertCount);
            }
            var ignore = m_paramsHandler.GetIntArrayValue(ParamNames.BOLT_IGNORE);
            m_boltData.IgnoreBolt = HandlerIgnore(ignore, vertCount, m_boltData.HoriCount);
        }
        private bool CreatePlate()
        {
            if (m_plateProps.Thickness <= 0.0 || m_plateProps.Width <= 0.0)
                return false;
            Plane refPlane; 
            if(m_plateDir == 3)
            {
                //方形
                var point = m_mainNearPlane.Intersect(AdjustBoltVertMode(), m_subFarPlane);
                refPlane = new Plane(point, -m_dir);
            }
            else
            {
                //倾斜
                refPlane = AdjustBoltVertMode();
            }
            var parts = new List<PartBase>();
            switch (m_platePos)
            {
                case 3:
                    parts.Add(CreatePolyPlate(refPlane, m_subNearPlane.Move(m_plateGap)));
                    parts.Add(CreatePolyPlate(refPlane, m_subFarPlane.Move(m_plateGap)));
                    break;
                case 4:
                    parts.Add(CreatePolyPlate(refPlane, m_subNearPlane));
                    break;
                default:
                    parts.Add(CreatePolyPlate(refPlane, m_subFarPlane));
                    break;
            }
            if (parts == null || parts.Count < 1)
                return false;
            int chamferType = m_paramsHandler.GetIntValue(ParamNames.PLATE_CHAMFER_TYPE);
            var chamferX = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_X);
            var chamferY = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_Y);
            foreach (var item in parts)
            {
                if(item != null)
                {
                    item.Class = m_context.TargetComp.CompClass;
                    m_plateProps.ApplyToPart(item);
                    plates.Add(item);
                    PlateTools.SetChamfer(item.CtrlPoints.First() as ChamferCtrlPoint,
                        chamferType, chamferX, chamferY, item);
                }
            }
            return true;
        }
        /// <summary>
        /// 创建多边形板
        /// </summary>
        /// <param name="refPlane">板与主零件贴合的面</param>
        /// <param name="subPlane">次零件的远近腹板面</param>
        private PartBase CreatePolyPlate(Plane refPlane, Plane subPlane)
        {
            if (refPlane == null)
                return null;
            if (m_plateProps.Width != double.MaxValue)
            {
                horiLen = m_plateProps.Width;
            }
            else
            {
                var alignDist = m_vertAlignPlane.Dist(m_mainNearPlane.Origin);
                horiLen = PlateTools.SumArray(m_boltData.HoriStride) +
                m_boltData.LeftGap + m_boltData.RightGap + alignDist;
            }
            int plateShape = m_paramsHandler.GetIntValue(ParamNames.PLATE_SHAPE);
            maxLen = m_upChamferHeight + m_downChamferHeight + m_boltData.UpGap
                + PlateTools.SumArray(m_boltData.VertStride) + m_boltData.DownGap;
            double vertLen = maxLen;
            if (plateShape == 1)
            {
                //方板
                m_upChamferGap = horiLen;
                m_downChamferGap = horiLen;
            }
            else
            {
                //带斜角的板
                m_upChamferGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_CHAMFER_PART_GAP);
                m_downChamferGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_CHAMFER_PART_GAP);
                vertLen -= m_downChamferHeight;
            }
            var points = new List<Vec3>();
            points.Add(refPlane.Intersect(m_mainNearPlane, subPlane));
            points.Add(subPlane.Intersect(m_mainNearPlane, refPlane.Move(maxLen)));
            points.Add(subPlane.Intersect(m_mainNearPlane
                .Move(m_downChamferGap), refPlane.Move(maxLen)));
            Plane alignPlane;
            if (m_plateDir == 4)
            {
                //修改的倾斜
                var point = m_mainNearPlane.Move(horiLen)
                    .Intersect(m_subUpPlane.Move(10.5), m_subFarPlane);
                alignPlane = new Plane(point, -m_subLine);
            }
            else
            {
                alignPlane = m_mainNearPlane.Move(horiLen);
            }
            points.Add(subPlane.Intersect(alignPlane, refPlane.Move(vertLen)));
            points.Add(subPlane.Intersect(alignPlane, refPlane.Move(m_upChamferHeight)));
            points.Add(subPlane.Intersect(m_mainNearPlane
                .Move(m_upChamferGap), refPlane));
            var ctrlPoints = AdjustCtrlPoints(points.ToArray(), subPlane);
            var retPlate = m_context.TargetComp.CreateSteelPlate(ctrlPoints, 
                m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            return retPlate;
        }
        private bool CreateBolts()
        {
            if (m_boltData.VertStride == null || m_boltData.HoriStride == null)
                return false;
            Vec3 boltX;
            if(m_boltData.BoltOrientation == 4)
                //倾斜
                boltX = m_subUpPlane.Normal.Normalize();
            else
                //方形
                boltX = -m_dir;
            Vec3 boltY = m_subLine;
            Vec3 boltZ = boltX.Cross(boltY);
            var platePlane = AdjustBoltVertMode();
            //startPos取第一列螺栓所在的线上
            Vec3 startPos = m_vertAlignPlane.Move(m_boltData.LeftGap).Intersect(m_subFarPlane, platePlane);
            var endPos = startPos + boltX * (maxLen + 25.0);
            BoltGroup bolts = m_context.TargetComp.CreateBoltGroup(
                m_context.SubPart, plates.ToArray(), startPos, endPos, true);
            if (bolts == null)
                return false;
            bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
            bolts.Rotation = 0;
            m_boltData.BoltsCfg.FillBasicProps(bolts, true);
            m_boltData.BoltsCfg.FillHoleProps(bolts);
            m_boltData.BoltsCfg.FillContentProps(bolts);
            m_boltData.BoltsCfg.FillExtendProps(bolts);
            m_boltData.BoltsCfg.FillShape(bolts);
            //次零件腹板厚度
            var webThickness = m_subFarPlane.Dist(m_subNearPlane.Origin);
            if (m_platePos == 3)
                bolts.CutLength = 1.0 + 2 * (webThickness +
                    m_plateGap + m_plateProps.Thickness);
            else
                bolts.CutLength = 2 * (m_plateProps.Thickness + webThickness) + 1.0;
            if(m_boltData.BoltOrientation == 2)
            {
                //交错
                bolts.Shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_LIST;
                var strideX = MergeArray(m_boltData.VertStride, m_boltData.UpGap);
                var strideY = MergeArray(m_boltData.HoriStride, m_boltData.LeftGap);
                var refPlane = new Plane(startPos, boltX);
                var listX = BoltStragger(platePlane, m_vertAlignPlane, refPlane, strideX, strideY);
                if (listX == null)
                    return false;
                var listY = new List<double>();
                var sumY = 0.0;
                //由于startPos在第一列螺栓形成的连线上，所以strideY第一个元素赋值0
                strideY[0] = 0.0;
                for(int i = 0; i < strideY.Length; i++)
                {
                    sumY += strideY[i];
                    for(int j = 0; j < strideX.Length; j++)
                    {
                        listY.Add(sumY);
                    }
                }
                //处理螺栓忽略
                if(m_boltData.IgnoreBolt != null)
                {
                    foreach (var item in m_boltData.IgnoreBolt)
                    {
                        listX.RemoveAt(item - 1);
                        listY.RemoveAt(item - 1);
                    }
                }
                bolts.ListX = listX.ToArray();
                bolts.ListY = listY.ToArray();
            }
            else
            {
                bolts.ArrayStrideX = m_boltData.VertStride;
                bolts.ArrayStrideY = m_boltData.HoriStride;
                bolts.HeadOffsetX = m_boltData.UpGap + m_upChamferHeight;
                bolts.VerticalOffset = PlateTools.SumArray(m_boltData.HoriStride) / 2;
                bolts.IgnoreIndices = m_boltData.IgnoreBolt;
            }
            bolts.Refresh();
            return true;
        }
        /// <summary>
        /// 螺栓交错
        /// </summary>
        /// <param name="xPlane">X方向螺栓所在面</param>
        /// <param name="yPlane">Y方向螺栓所在面</param>
        /// <param name="refPlane">螺栓组起始点所在平面</param>
        /// <param name="strideX">X方向螺栓组间距</param>
        /// <param name="strideY">Y方向螺栓组间距</param>
        /// <returns></returns>
        private List<double> BoltStragger(Plane xPlane, Plane yPlane, Plane refPlane, double[] strideX, double[] strideY)
        {
            int lenX = strideX.Length;
            int lenY = strideY.Length;
            if (lenX < 1 || lenY < 1 || xPlane == null || yPlane == null || refPlane == null)
                return null;
            List<double> offset = new List<double>();
            var offsetYPlane = new Plane(yPlane);
            for(int i = 0; i < lenY; i++)
            {
                offsetYPlane.MoveThis(strideY[i]);
                var offsetXPlane = new Plane(xPlane);
                for (int j = 0; j < lenX; j++)
                {
                    offsetXPlane.MoveThis(strideX[j]);
                    var point = m_subFarPlane.Intersect(offsetYPlane, offsetXPlane);
                    offset.Add(refPlane.SignedDist(point));
                }
            }
            return offset;
        }
        /// <summary>
        /// 次零件圆角槽口
        /// </summary>
        private void BeamFilletCut()
        {
            if (plates == null || plates.Count < 1)
                return;
            var cutEnabled = m_paramsHandler.GetIntValue(ParamNames.PIC_FILLET_NOTCH_ENABLED);
            var cutLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FILLET_NITCH_LENGTH);
            if (cutEnabled == 3 || cutLength <= 0)
                return;
            var refPoint = plates.First().CtrlPoints[1].WorldPos;
            if (m_subDownPlane.SignedDist(refPoint) > -1e-3)
                return;
            var subPlanes = m_context.SubPlanes;
            var webThickness = Math.Min(
                m_subDownPlane.Dist(subPlanes.TopBound.Origin),
                m_subDownPlane.Dist(subPlanes.BottomBound.Origin)
                ); //下翼缘厚度
            var points = new List<Vec3>();
            var ctrlPoints = plates.First().CtrlPoints;
            Plane bottomPlane = m_subDownPlane.Invert().Move(webThickness);
            Plane topPlane = bottomPlane.Move(-cutLength).Invert();
            //业务规则 从第2个控制点开始算 找符合条件的板线段
            int len = ctrlPoints.Length - 1;
            for (int i = 1; i < len; i++)
            {
                //该点超出次零件边界且下一个点未超出次零件边界
                Vec3 pos1 = ctrlPoints[i].WorldPos;
                Vec3 pos2 = ctrlPoints[i + 1].WorldPos;
                var dist1 = m_subDownPlane.SignedDist(pos1);
                var dist2 = m_subDownPlane.SignedDist(pos2);
                if(dist1 < 0 && dist2 > 0)
                {
                    //更新板线段
                    if(topPlane.SignedDist(pos2) < 0 && i + 2 <= len)
                    {
                        points.Add(pos2);
                        points.Add(ctrlPoints[i + 2].WorldPos);
                    }
                    else
                    {
                        points.Add(pos1);
                        points.Add(pos2);
                    }
                    break;
                }
            }
            //计算开槽参照面
            var point1 = points.First();
            var point2 = points.Last();
            
            var platePlane = new Plane(point1, point2 - point1, m_subFarPlane.Normal.Normalize());
            Vec3 point = platePlane.Intersect(topPlane, m_subFarPlane);
            double notchLen = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FLANGE_CUT_SIZE);

            //创建板切割体
            double eps = 10.0;
            List<Plane> plsSide = new List<Plane>();
            plsSide.Add(topPlane);
            plsSide.Add(m_mainNearPlane);
            plsSide.Add(bottomPlane.Move(eps));
            plsSide.Add(new Plane(point, m_subLine).Move(notchLen));

            var zTop = subPlanes.BackBound.Move(eps);
            var zBottom = subPlanes.FrontBound.Move(eps);
            var cutter = m_context.TargetComp.CreatePlateCutter(m_context.SubPart, 
                zBottom, zTop, plsSide.ToArray(), true);
            if (cutter == null)
                return;
            var radius = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FILLET_NOTCH_RADIUS);
            ChamferCtrlPoint pos = null;
            foreach (var ctrlPoint in cutter.CtrlPoints)
            {
                //找到圆角的控制点
                if (m_subDownPlane.SignedDist(ctrlPoint.WorldPos) > 0
                    && m_mainNearPlane.Dist(ctrlPoint.WorldPos) > 1e-1)
                    pos = ctrlPoint as ChamferCtrlPoint;
            }
            if (pos == null)
                return;
            PlateTools.SetChamfer(pos, 3, radius, radius, cutter);
        }
        private Plane AdjustBoltVertMode()
        {
            if (m_boltData.VertOffset == double.MaxValue)
                return m_subUpPlane.Move(10.5);
            var subPlanes = m_context.SubPlanes;
            var dist1 = m_subUpPlane.Dist(subPlanes.TopBound.Origin);
            var dist2 = m_subUpPlane.Dist(subPlanes.BottomBound.Origin);
            double thickness = Math.Min(dist1, dist2);  //上翼缘厚度
            switch (m_boltData.VertOffsetMode)
            {
                case 2:
                    //中间
                    double width = subPlanes.TopBound.Dist(subPlanes.BottomBound.Origin) - 2 * thickness;
                    return m_subDownPlane.Move(width / 2 + m_boltData.UpGap + m_boltData.VertOffset
                        + PlateTools.SumArray(m_boltData.VertStride) / 2 + m_upChamferHeight).Invert();
                case 3:
                    //底部
                    return m_subDownPlane.Move(PlateTools.SumArray(m_boltData.VertStride)
                        + m_boltData.UpGap + m_boltData.VertOffset + m_upChamferHeight - thickness).Invert();
                default:
                    //顶部
                    return m_subUpPlane.Move(-thickness - m_boltData.UpGap
                        + m_boltData.VertOffset - m_upChamferHeight);
            }
        }
        private Vec3[] AdjustCtrlPoints(Vec3[] oldPoints, Plane plane)
        {
            if (oldPoints == null || oldPoints.Length < 3)
                return null;
            //先去重
            var newPoints = new List<Vec3>();
            foreach(var item in oldPoints)
            {
                if (Contains(newPoints, item))
                    continue;
                newPoints.Add(item);
            }
            var points = newPoints.ToArray();
            var len = points.Length;
            var dir = (points[1] - points[0]).Cross(points[len - 1] - points[0]);
            Vec3 normal = plane.Normal.Normalize();
            if (normal.Dot(dir) > 0)
                return points;
            //调整
            List<Vec3> ctrlPoints = new List<Vec3>();
            for(int i = 0; i < len; i++)
                ctrlPoints.Add(points[i] + normal * m_plateProps.Thickness);
            return ctrlPoints.ToArray();
        }
        /// <summary>
        /// 根据螺栓数目调整螺栓X方向间距
        /// </summary>
        private double[] HandlerStrideX(double[] arr, int num)
        {
            if (num <= 0)
                return null;
            if (num == 1)
                return new double[] { 0 };
            var ret = new List<double>();
            if (arr == null || arr.Length == 0)
            {
                ret.Add(60.0);
                ret.AddRange(Enumerable.Repeat(76.2, num - 2));
                return ret.ToArray();
            }
            for (int i = 0; i < num - 1; i++)
            {
                if (i > arr.Length - 1)
                    ret.Add(arr[arr.Length - 1]);
                else
                    ret.Add(arr[i]);
            }
            return ret.ToArray();
        }
        private int[] HandlerIgnore(int[] arr, int vertNum, int horiNum)
        {
            if (vertNum <= 0 || horiNum <= 0 || arr == null || arr.Length == 0)
                return null;
            var boltSeq = new int[vertNum, horiNum]; //螺栓序列
            var index = 1;
            for(int i = 0; i < horiNum; i++)
            {
                for(int j = 0; j < vertNum; j++)
                {
                    boltSeq[j, i] = index;
                    index++;
                }
            }
            int num = vertNum * horiNum;
            var seq = new int[num];
            index = 0;
            for(int i = 0; i < vertNum; i++)
            {
                for (int j = 0; j < horiNum; j++)
                {
                    seq[index] = boltSeq[i, j];
                    index++;
                }
            }
            List<int> nums = new List<int>();
            foreach (var item in arr)
            {
                if (item > 0 && item <= num)
                    nums.Add(seq[item - 1]);
            }
            if (nums.Count < 1)
                return null;
            return nums.ToArray();
        }
        private double[] MergeArray(double[] arr, double data)
        {
            var list = new List<double>();
            list.Add(data);
            if (arr == null || arr.Length == 0)
                return list.ToArray();
            foreach (var item in arr)
                list.Add(item);
            return list.ToArray();
        }
        private bool Contains(List<Vec3> points, Vec3 item)
        {
            foreach(var pos in points)
            {
                var line = item - pos;
                if (line.Length < 1e-3)
                    return true;
            }
            return false;
        }
    }
}
