﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using System.Linq;
using LoongCAD.Geometry;
using System.Collections.Generic;

namespace HorizontalStiffener1017
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private DetailComponent m_comp;

        private Beam m_mainPart = null;

        private BeamPlanes m_mainPlanes = null;

        private Vec3 m_mainLine = Vec3.ZERO;

        private Plane m_upFlangePlane = null;

        private Plane m_downFlangePlane = null;

        public static ComponentInstanceContext CreateInstance(DetailComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }

        private bool Initialize(DetailComponent comp)
        {
            if (comp == null)
                return false;

            Beam mainPart = comp.Main as Beam;
            if (mainPart == null)
                return false;
            if (!PrepareMainLine(mainPart, comp))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);

            if (!CheckPart(comp, m_mainPlanes))
                return false;

            return true;
        }

        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart, DetailComponent comp)
        {
            if (mainPart == null || comp == null)
                return false;

            var point = comp.CtrlPoints?.First()?.WorldPos;  //细部控制点
            //细部控制点不存在 不创建组件
            if (!point.HasValue)
                return false;

            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;

            var plane = new Plane(point.Value, line.Value);
            var dist1 = plane.Dist(mainPart.CtrlPoints.First().WorldPos);
            var dist2 = plane.Dist(mainPart.CtrlPoints.Last().WorldPos);

            if(dist1 > dist2)
            {
                //细部控制点离主零件第二控制点近
                m_mainLine = -line.Value.Normalize();
            }else {
                //细部控制点离主零件第一控制点近
                m_mainLine = line.Value.Normalize();
            }

            return true;
        }

        private bool CheckPart(DetailComponent comp, BeamPlanes mainPlanes)
        {
            if (comp == null || mainPlanes == null)
                return false;

            //检查此零件是否有腹板
            if (mainPlanes.BackWeb == null || mainPlanes.FrontWeb == null)
                return false;

            //检查此零件是否有翼缘 一些属性的默认值是基于零件翼缘厚度
            if (mainPlanes.TopFlange == null || mainPlanes.TopFlangeInside == null ||
                mainPlanes.BottomFlange == null || mainPlanes.BottomFlangeInside == null)
                return false;

            //基于组件帽尖方向确定上下翼缘面
            var dir = comp.LocalMatrix.Z.Normalize();
            m_upFlangePlane = mainPlanes.TopFlange;
            m_downFlangePlane = mainPlanes.BottomFlange;
            if(dir.Dot(m_upFlangePlane.Normal) < 0)
            {
                this.SwapVar(ref m_upFlangePlane, ref m_downFlangePlane);
            }

            return true;
        }

        /// <summary>
        /// 交换两个面的属性信息
        /// </summary>
        /// <param name="plane1">面1</param>
        /// <param name="plane2">面2</param>
        private void SwapVar(ref Plane plane1, ref Plane plane2)
        {
            var temp = plane1;
            plane1 = plane2;
            plane2 = temp;
        }

        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
        }

        //开发组件需要的上下文
        public DetailComponent TargetComp { get => m_comp; set => m_comp = value; }
        //组件的主要部分
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public Plane UpFlangePlane { get => m_upFlangePlane; set => m_upFlangePlane = value; }
        public Plane DownFlangePlane { get => m_downFlangePlane; set => m_downFlangePlane = value; }
        public List<PartBase> Parts { get; set; }
    }
}
