﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace HorizontalStiffener1017
{
    public class PlateHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PlateProperties m_upProps = new PlateProperties();
        private PlateProperties m_middleProps = new PlateProperties();
        private PlateProperties m_downProps = new PlateProperties();

        private Plane m_upFlangePlane = null;
        private Plane m_downFlangePlane = null;

        //零件
        private List<PartBase> parts = new List<PartBase>();

        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            if (!ParseProps())
                return false;

            if (!HandlerStiff())
                return false;

            //设置折角
            if(parts.Count > 0)
            {
                var chamferType = m_paramsHandler.GetIntValue(ParamNames.PART_PLATE_CHAMFER_TYPE);
                double chamferX = m_paramsHandler.GetDoubleValue(ParamNames.PART_PLATE_CHAMFER_X);
                double chamferY = m_paramsHandler.GetDoubleValue(ParamNames.PART_PLATE_CHAMFER_Y);
                //适应凹弧或凸弧折角
                if (chamferType > 2)
                    chamferY = chamferX;
                
                foreach(var part in parts)
                {
                    var chamferPoint1 = part.CtrlPoints[0] as ChamferCtrlPoint;
                    var chamferPoint2 = part.CtrlPoints[1] as ChamferCtrlPoint;
                    this.SetChamfer(chamferPoint1, chamferType, chamferX, chamferY, part);
                    this.SetChamfer(chamferPoint2, chamferType, chamferY, chamferX, part);
                }
            }

            m_context.Parts = parts;

            return true;
        }

        private bool ParseProps()
        {
            var m_mainPlanes = m_context.MainPlanes;
            //计算腹板厚度
            var webThickness = m_mainPlanes.BackWeb.Dist(m_mainPlanes.FrontWeb.Origin);

            //计算翼缘宽度
            var flangeWidth = m_mainPlanes.BackBound.Dist(m_mainPlanes.FrontBound.Origin);

            //初始化上下文参数
            m_upFlangePlane = m_context.UpFlangePlane;
            m_downFlangePlane = m_context.DownFlangePlane;

            //计算翼缘厚度
            var flangeThickness = Math.Min(
                m_upFlangePlane.Dist(m_mainPlanes.TopFlangeInside.Origin),
                m_upFlangePlane.Dist(m_mainPlanes.BottomFlangeInside.Origin));

            //上加劲板
            if (!ParseParam(ref m_upProps, ParamNames.PART_UP_STIFF_PROPS, 
                0.5 * (flangeWidth - webThickness), flangeThickness))
                return false;
            //中间加劲板
            if (!ParseParam(ref m_middleProps, ParamNames.PART_MIDDLE_STIFF_PROPS,
                0.5 * (flangeWidth - webThickness), flangeThickness))
                return false;
            //下加劲板
            if (!ParseParam(ref m_downProps, ParamNames.PART_DOWN_STIFF_PROPS,
                0.5 * (flangeWidth - webThickness), flangeThickness))
                return false;

            return true;
        }

        private bool HandlerStiff()
        {
            //计算梁高  垂直距离默认为 0.25 * 梁高
            var _h = m_upFlangePlane.Dist(m_downFlangePlane.Origin);

            //细部控制点
            var point = m_context.TargetComp.CtrlPoints.First().WorldPos;
            var refPlane = new Plane(point, m_context.MainLine);

            //记录垂直距离
            List<double> vertOffset = new List<double>();
            //记录底边偏移
            List<double> horiOffset = new List<double>();
            //记录要创建板的板属性集合
            List<PlateProperties> props = new List<PlateProperties>();

            //上加劲板
            var upHoriOffset = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_HORI_DIST);
            var upVertOffset = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_VERT_DIST);
            var downHoriOffset = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_HORI_DIST);

            if (upVertOffset == double.MaxValue)
                upVertOffset = _h / 4;

            //至少有一个加劲板
            vertOffset.Add(upVertOffset);
            horiOffset.Add(upHoriOffset);
            props.Add(m_upProps);

            StiffNumber stiffNumber = (StiffNumber)m_paramsHandler.GetIntValue(ParamNames.PIC_PLATE_NUMBER);
            switch (stiffNumber)
            {
                case StiffNumber.ONE:
                    break;
                case StiffNumber.THREE:
                    this.HandlerStride(_h, ref vertOffset);
                    horiOffset.Add(m_paramsHandler.GetDoubleValue(ParamNames.PIC_MIDDLE_HORI_DIST));
                    horiOffset.Add(downHoriOffset);
                    props.Add(m_middleProps);
                    props.Add(m_downProps);
                    break;
                case StiffNumber.TWO:
                default:
                    //下加劲板
                    var downVertOffset = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_VERT_DIST);
                    if (downVertOffset == double.MaxValue)
                        downVertOffset = _h / 4;
                    vertOffset.Add(_h -downVertOffset- upVertOffset);
                    horiOffset.Add(downHoriOffset);
                    props.Add(m_downProps);
                    break;
            }

            if (!CreateStiff(m_upFlangePlane.Invert(), refPlane, vertOffset.ToArray(),
                horiOffset.ToArray(), props.ToArray(), ref parts))
                return false;

            if (parts.Count < 2)
                return false;

            return true;
        }

        /// <summary>
        /// 创建加劲板
        /// </summary>
        /// <param name="zPlane">控制点所在平面 要求法线方向为沿着组件帽尖反方向</param>
        /// <param name="bottomPlane">b边平面</param>
        /// <param name="vertOffset">垂直偏移</param>
        /// <param name="horiOffset">水平偏移</param>
        /// <param name="props">板属性集合</param>
        /// <param name="parts">存储创建的加劲板</param>
        private bool CreateStiff(Plane zPlane, Plane bottomPlane, double[] vertOffset, double[] horiOffset, PlateProperties[] props, ref List<PartBase> parts)
        {
            if (zPlane == null || bottomPlane == null)
                return false;
            if (vertOffset.Length != horiOffset.Length || horiOffset.Length != props.Length)
                return false;

            int len = vertOffset.Length;
            var webPlane1 = m_context.MainPlanes.BackWeb;
            var webPlane2 = m_context.MainPlanes.FrontWeb;

            for(int i = 0; i < len; i++)
            {
                var thickness = props[i].Thickness;
                var width = props[i].Width;
                var height = props[i].Height;
                if (thickness < 1e-3 || height < 1e-3 || width < 1e-3)
                    continue;

                var plsSide = new List<Plane>();
                var pls1 = bottomPlane.Move(horiOffset[i]);
                var pls2 = bottomPlane.Move(horiOffset[i] + height);
                plsSide.Add(pls1);
                plsSide.Add(webPlane1);
                plsSide.Add(pls2);
                plsSide.Add(webPlane1.Move(width));
                zPlane.MoveThis(vertOffset[i] - thickness / 2);

                var plate1 = m_context.TargetComp.CreateSteelPlate(zPlane, thickness, plsSide.ToArray(), true);
                if (plate1 == null)
                    return false;

                plate1.Class = m_context.TargetComp.CompClass;
                props[i].ApplyToPart(plate1);

                //对称侧
                plsSide.Clear();

                plsSide.Add(pls1);
                plsSide.Add(webPlane2);
                plsSide.Add(pls2);
                plsSide.Add(webPlane2.Move(width));

                var plate2 = m_context.TargetComp.CreateSteelPlate(zPlane.Move(thickness).Invert(), thickness, plsSide.ToArray(), true);
                if (plate2 == null)
                    return false;

                plate2.Class = m_context.TargetComp.CompClass;
                props[i].ApplyToPart(plate2);

                //存储板数据
                parts.Add(plate1);
                parts.Add(plate2);

                //移动zPlane至下一位置
                zPlane.MoveThis(thickness / 2);
            }

            return true;
        }

        /// <summary>
        /// 获取零件的属性对象并设置到对象字典中
        /// </summary>
        /// <param name="m_props">对象字典</param>
        /// <param name="paramName">零件名称</param>
        /// <param name="width">默认宽度</param>
        /// <param name="thickness">默认厚度</param>
        /// <returns></returns>
        private bool ParseParam(ref PlateProperties m_props, string paramName, double width, double thickness)
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(paramName);
            if (plateProps == null)
                return false;
            m_props.SetProps(plateProps);

            //默认状况下厚度等于梁翼缘厚度
            if (m_props.Thickness == double.MaxValue)
                m_props.Thickness = thickness;

            //默认状况下宽度为0.5*(翼缘宽度-腹板厚度)
            if (m_props.Width == double.MaxValue)
                m_props.Width = width;

            return true;
        }

        /// <summary>
        /// 处理多个加劲板时板间距的逻辑
        /// </summary>
        /// <param name="_h">梁高</param>
        private void HandlerStride(double _h, ref List<double> offset)
        {
            var arr = m_paramsHandler.GetDoubleArrayValue(ParamNames.PIC_PLATE_STRIDE);
            int num = 2; //此组件最多3快加劲板 第一块间距已经自适应

            if(arr == null || arr.Length == 0)
            {
                double value = (_h - _h / 4) / (num + 1);
                for(int i = 0; i < num; i++)
                {
                    offset.Add(value);
                }
                return;
            }

            var len = arr.Length - 1;
            for(int i = 0; i < num; i++)
            {
                if(i > len)
                {
                    offset.Add(arr[len]);
                }else {
                    offset.Add(arr[i]);
                }
            }
        }

        private void SetChamfer(ChamferCtrlPoint point, int type, double lenX, double lenY, PartBase retPlate)
        {
            if (retPlate == null)
                return;
            Define.CornerCutType chamferType;
            switch (type)
            {
                case 1:
                    //无折角
                    chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                case 3:
                    //凸弧折角
                    chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    break;
                case 4:
                    //凹弧折角
                    chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    break;
                case 0:
                default:
                    //线折角
                    chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    break;
            }

            if (point != null)
            {
                point.CutX = lenX;
                point.CutY = lenY;
                point.CutType = chamferType;
                point.Flush();
                retPlate.Flush();
            }
        }
    }
}
