﻿using LoongCAD.Geometry;
using LoongCAD.Util;
using System.Collections.Generic;
using LoongCAD.STC.Plugins;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using System.Linq;
using System;

namespace InnerDiaphragm1059
{
    public class GraphicHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private PartBase m_retPlate;
        BeamPlanes m_mainPlanes;
        private Vec3 m_point;
        private Plane m_basePlane;
        private Vec3 m_centerPoint;

        private PlateProperties m_plateProps = new PlateProperties();
        private double m_plateGap = double.MaxValue;
        private int m_chamferType = 0;
        private double m_chamferLength = double.MaxValue;
        private double m_centerHoleDiameter = double.MaxValue;
        private double m_aroundHoleDiameter = double.MaxValue;
        private double m_holeGap = double.MaxValue;
        private int m_platePosition = 0;
        private enum ProfileType
        {
            CIRCULAR = 0,
            SQUARE = 1,
            WELDBOX = 2,
            SPD = 3,
            OTHER = 4,
        }
        private ProfileType m_partType;
        private double m_vertOffset = double.MaxValue;
        private Vec3 m_axisX;
        private Vec3 m_axisY;
        private Vec3 m_axisZ;
        public GraphicHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;
            if (!ParsePart())
                return false;
            //根据通用性调节内膜板构建坐标系
            var upDir = m_context.TargetComp.CompDirUp;
            if(upDir == Define.SpaceAxis.AXIS_NONE)
            {
                m_axisX = m_context.MainPlanes.TopBound.Normal.Normalize();
                m_axisY = m_context.MainPlanes.FrontBound.Normal.Normalize();
                m_axisZ = m_context.MainLine;
            }
            else
            {
                var dirs = GetBuildPlaneDir();
                m_axisX = m_context.TargetComp.LocalMatrix.Z.Normalize();
                m_axisY = dirs.Last().Normalize();
                m_axisZ = dirs.First().Normalize();
            }
            //板法线与主零件构建方向垂直
            //不创建组件
            if (Math.Abs(m_axisZ.Dot(m_context.MainLine)) < 1e-3)
                return false;
            m_mainPlanes = m_context.MainPlanes;
            //细部控制点
            m_point = m_context.TargetComp.CtrlPoints[0].WorldPos;
            m_basePlane = new Plane(m_point, m_axisZ);
            if(m_partType != ProfileType.WELDBOX)
            {
                //计算板的中心点
                var h = m_mainPlanes.TopBound.Dist(m_mainPlanes.BottomBound.Origin);
                var b = m_mainPlanes.FrontBound.Dist(m_mainPlanes.BackBound.Origin);
                m_centerPoint = m_basePlane.Intersect(m_mainPlanes.TopBound.Move(-h / 2),
                    m_mainPlanes.FrontBound.Move(- b / 2));
            }
            if (m_partType == ProfileType.CIRCULAR)
                CreateCircularPlate();
            else if (m_partType == ProfileType.SQUARE)
                CreateSquarePlate();
            else if (m_partType == ProfileType.SPD)
                CreateSpdPlate();
            else
                CreateBoxPlate();
            CreateHole();
            m_context.RetPlate = m_retPlate;
            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);

            m_plateGap = m_paramsHandler.GetDoubleValue(ParamNames.TWO_PLATES_GAP);
            m_chamferType = m_paramsHandler.GetIntValue(ParamNames.PLATE_CHAMFER_TYPE);
            m_chamferLength = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_LENGTH);
            m_centerHoleDiameter = m_paramsHandler.GetDoubleValue(ParamNames.PLATR_CENTER_HOLE_DIAMETER);
            m_aroundHoleDiameter = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_AROUND_HOLES_DIAMETER);
            m_holeGap = m_paramsHandler.GetDoubleValue(ParamNames.AROUND_HOLES_CENTER_TO_PLATE_EDGE_DISTANCE);
            m_platePosition = m_paramsHandler.GetIntValue(ParamNames.PLATE_POSITION_TYPE);
            //初始化内膜板控制点纵向偏移值
            switch (m_platePosition)
            {
                case 2:
                    m_vertOffset = 0.0;
                    break;
                case 3:
                    m_vertOffset = - m_plateProps.Thickness;
                    break;
                case 0:
                default:
                    m_vertOffset = - m_plateProps.Thickness / 2;
                    break;
            }
            return true;
        }
        private bool ParsePart()
        {
            var type = m_context.MainPart.Profile.GetType();
            if (type == typeof(CircularHollowProfile))
                m_partType = ProfileType.CIRCULAR;
            else if (type == typeof(RectHollowProfile))
                m_partType = ProfileType.SQUARE;
            else if (type == typeof(WeldedBoxProfile))
                m_partType = ProfileType.WELDBOX;
            else if (type == typeof(SPDProfile))
                m_partType = ProfileType.SPD;
            else
                m_partType = ProfileType.OTHER;
            if (m_partType == ProfileType.OTHER)
                return false;
            return true;
        }
        private bool CreateCircularPlate()
        {
            CircularHollowProfile profile = m_context.MainPart.Profile as CircularHollowProfile;
            var subType = profile.SubType;
            //自适应变截面的高度和宽度
            double partH, partB, partT;
            var pls = new Plane(m_point, m_context.MainLine);
            double lmd = pls.Dist(m_context.MainPart.CtrlPoints.First().WorldPos);
            var len = m_mainPlanes.HeadBound.Dist(m_mainPlanes.TailBound.Origin);
            switch (subType)
            {
                case "circular":
                    partH = profile.Diameter;
                    partB = partH;
                    break;
                case "circular_tapered":
                    var d = profile.Diameter1;
                    var r = profile.Diameter2;
                    partH = d - lmd / len * (d - r);
                    partB = partH;
                    break;
                default:
                    var d1 = profile.Diameter1;
                    var d2 = profile.Diameter2;
                    var r1 = profile.MinorDiameter1;
                    var r2 = profile.MinorDiameter2;
                    partH = d1 - lmd / len * (d1 - d2);
                    partB = r1 - lmd / len * (r1 - r2);
                    break;
            }
            partT = profile.PlateThickness;
            var plateH = partH - 2 * partT - 2 * m_plateGap;
            var plateB = partB - 2 * partT - 2 * m_plateGap;
            if (plateH <= 1e-6 || plateB <= 1e-6)
                return false;
            var points = GetCtrlPoints(plateH, plateB);
            var ctrlPoints = GetWeldsPoints(points);
            if (ctrlPoints == null)
                return false;
            PartBase retPlate = m_context.TargetComp
                .CreateSteelPlate(ctrlPoints, m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (retPlate == null)
                return false;
            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);
            //将矩形板切成圆形板
            for (int i = 0; i < retPlate.CtrlPoints.Length; i += 2)
            {
                ChamferCtrlPoint chamferPoint = retPlate.CtrlPoints[i] as ChamferCtrlPoint;
                if (chamferPoint != null)
                {
                    chamferPoint.CutType = Define.CornerCutType.CORNER_CUT_ARC;
                    chamferPoint.Flush();
                    retPlate.Flush();
                }
            }
            m_retPlate = retPlate;
            CreateAroundHoles(AutoHoleGap(plateH), AutoHoleGap(plateB));
            //存储焊接点
            m_context.WeldPoints = ctrlPoints;
            return true;
        }
        private bool CreateSquarePlate()
        {
            var profile = m_context.MainPart.Profile as RectHollowProfile;
            var subType = profile.SubType;
            //自适应变截面的高度和宽度
            double partH, partB, partT;
            switch (subType)
            {
                case "square":
                    partH = profile.Height;
                    partB = partH;
                    break;
                case "rectangular":
                    partH = profile.Height;
                    partB = profile.Width;
                    break;
                default:
                    var h1 = profile.Height1;
                    var h2 = profile.Height2;
                    var b1 = profile.Width1;
                    var b2 = profile.Width2;
                    var pls = new Plane(m_point, m_context.MainLine);
                    double lmd = pls.Dist(m_context.MainPart.CtrlPoints.First().WorldPos);
                    var len = m_mainPlanes.HeadBound.Dist(m_mainPlanes.TailBound.Origin);
                    partH = h1 - lmd / len * (h1 - h2);
                    partB = b1 - lmd / len * (b1 - b2);
                    break;
            }
            partT = profile.PlateThickness;

            var plateH = partH - 2 * partT - 2 * m_plateGap;
            var plateB = partB - 2 * partT - 2 * m_plateGap;
            if (plateH <= 1e-6 || plateB <= 1e-6)
                return false;
            var points = GetCtrlPoints(plateH, plateB);
            PartBase retPlate = m_context.TargetComp
                .CreateSteelPlate(points, m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (retPlate == null)
                return false;
            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);
            //自适应折角长度
            if (m_chamferLength == double.MaxValue)
                m_chamferLength = 2 * partT;
            setChamfer(retPlate, m_chamferLength);
            m_retPlate = retPlate;
            CreateAroundHoles(AutoHoleGap(plateH), AutoHoleGap(plateB));
            //存储焊接点
            var ctrlPoints = GetWeldsPoints(points);
            m_context.WeldPoints = ctrlPoints;
            return true;
        }
        private bool CreateSpdPlate()
        {
            var profile = m_context.MainPart.Profile as SPDProfile;
            var subType = profile.SubType;
            //自适应变截面的高度和宽度
            double partH, partB, partT;
            var pls = new Plane(m_point, m_context.MainLine);
            double lmd = pls.Dist(m_context.MainPart.CtrlPoints.First().WorldPos);
            var len = m_mainPlanes.HeadBound.Dist(m_mainPlanes.TailBound.Origin);
            switch (subType)
            {
                case "circular":
                    partH = profile.Diameter;
                    partB = partH;
                    break;
                case "circular_tapered":
                    var d = profile.Diameter1;
                    var r = profile.Diameter2;
                    partH = d - lmd / len * (d - r);
                    partB = partH;
                    break;
                default:
                    var d1 = profile.Diameter1;
                    var d2 = profile.Diameter2;
                    var r1 = profile.MinorDiameter1;
                    var r2 = profile.MinorDiameter2;
                    partH = d1 - lmd / len * (d1 - d2);
                    partB = r1 - lmd / len * (r1 - r2);
                    break;
            }
            partT = profile.PlateThickness;
            var plateH = partH - 2 * partT - 2 * m_plateGap;
            var plateB = partB - 2 * partT - 2 * m_plateGap;
            if (plateH <= 1e-6 || plateB <= 1e-6)
                return false;
            var points = GetCtrlPoints(plateH, plateB);
            var ctrlPoints = GetWeldsPoints(points);
            if (ctrlPoints == null)
                return false;
            PartBase retPlate = m_context.TargetComp
                .CreateSteelPlate(ctrlPoints, m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (retPlate == null)
                return false;
            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);
            //将矩形板切成圆形板
            for (int i = 0; i < retPlate.CtrlPoints.Length; i += 2)
            {
                ChamferCtrlPoint chamferPoint = retPlate.CtrlPoints[i] as ChamferCtrlPoint;
                if (chamferPoint != null)
                {
                    chamferPoint.CutType = Define.CornerCutType.CORNER_CUT_ARC;
                    chamferPoint.Flush();
                    retPlate.Flush();
                }
            }
            m_retPlate = retPlate;
            CreateAroundHoles(AutoHoleGap(plateH), AutoHoleGap(plateB));
            //存储焊接点
            m_context.WeldPoints = ctrlPoints;
            return true;
        }
        private bool CreateBoxPlate()
        {
            var profile = m_context.MainPart.Profile as WeldedBoxProfile;
            double partH = m_mainPlanes.TopBound.Dist(m_mainPlanes.BottomBound.Origin);
            var subType = profile.SubType;
            double partT1, partT2, partB, maxB;
            switch (subType)
            {
                case "symmetrical":
                    partT1 = profile.FlangeThickness;
                    partT2 = partT1;
                    partB = profile.Width;
                    maxB = partB;
                    break;
                default:
                    partT1 = profile.FlangeThickness1;
                    partT2 = profile.FlangeThickness2;
                    partB = profile.FlangeWidth2;
                    maxB = Math.Max(profile.FlangeWidth1, profile.FlangeWidth2);
                    break;
            }
            double c = profile.Cantilever;
            double s = profile.WebThickness;
            var plateH = partH - partT1 - partT2 - 2 * m_plateGap;
            var plateB = partB - 2 * c - 2 * s - 2 * m_plateGap;
            if (plateH <= 1e-6 || plateB <= 1e-6)
                return false;
            //计算内膜板的中心点
            m_centerPoint = m_basePlane.Intersect(m_mainPlanes.FrontBound.Move(- maxB / 2),
                m_mainPlanes.TopBound.Move(- partT1 - plateH / 2 - m_plateGap));
            var points = GetCtrlPoints(plateH, plateB);
            PartBase retPlate = m_context.TargetComp
                .CreateSteelPlate(points, m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (retPlate == null)
                return false;
            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);
            //自适应折角长度
            if (m_chamferLength == double.MaxValue)
                m_chamferLength = 10.0;
            setChamfer(retPlate, m_chamferLength);
            m_retPlate = retPlate;
            CreateAroundHoles(AutoHoleGap(plateH), AutoHoleGap(plateB));
            //存储焊接点
            var ctrlPoints = GetWeldsPoints(points);
            m_context.WeldPoints = ctrlPoints;
            return true;
        }
        private double AutoHoleGap(double len)
        {
            //自适应周围孔到板边缘的距离
            if(m_holeGap != double.MaxValue)
            {
                var gap = len - 2 * m_holeGap;
                return gap > 0 ? gap : len - gap;
            }

            if (len >= 100)
                return len - 2 * 50.0;
            else
                return 100 - len;
        }
        private void CreateHole()
        {
            if (m_centerHoleDiameter <= 0 || m_retPlate == null)
                return;
            Vec3 boltX = m_axisX;
            Vec3 boltY = m_axisY;
            Vec3 boltZ = m_axisZ;
            var startPos = m_centerPoint;
            var len = m_mainPlanes.TopBound.Dist(m_mainPlanes.BottomBound.Origin);
            var endPos = startPos + boltX * len;
            BoltGroup curHoles = m_context.TargetComp.CreateBoltHoles(null, m_retPlate, startPos, endPos, true);
            if (curHoles == null)
                return;
            curHoles.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            curHoles.Rotation = 0;
            curHoles.RotationSide = Define.ObjectSide.SIDE_TOP;
            curHoles.BoltSize = m_centerHoleDiameter;
            curHoles.HoleX = 0.0;
            curHoles.HoleY = 0.0;
            curHoles.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
            curHoles.OverSizeHoleEnable0 = false;
            curHoles.OverSizeHoleEnable1 = false;
            curHoles.ArrayStrideX = new double[] { 0 };
            curHoles.ArrayStrideY = new double[] { 0 };
            curHoles.HeadOffsetX = 0;
            curHoles.CutLength = m_plateProps.Thickness * 2.0 + 1.0;
            curHoles.Refresh();
        }
        private void CreateAroundHoles(double strideX, double strideY)
        {
            if (m_aroundHoleDiameter <= 0 || m_retPlate == null)
                return;
            Vec3 boltX = m_axisX;
            Vec3 boltY = m_axisY;
            Vec3 boltZ = m_axisZ;
            var startPos = m_centerPoint;
            var len = m_mainPlanes.TopBound.Dist(m_mainPlanes.BottomBound.Origin);
            var endPos = startPos + boltX * len;
            BoltGroup curHoles = m_context.TargetComp.CreateBoltHoles(null, m_retPlate, startPos, endPos, true);
            if (curHoles == null)
                return;
            curHoles.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            curHoles.Rotation = 0;
            curHoles.RotationSide = Define.ObjectSide.SIDE_TOP;
            curHoles.BoltSize = m_aroundHoleDiameter;
            curHoles.HoleX = 0.0;
            curHoles.HoleY = 0.0;
            curHoles.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
            curHoles.OverSizeHoleEnable0 = false;
            curHoles.OverSizeHoleEnable1 = false;
            curHoles.ArrayStrideX = new double[] { strideX };
            curHoles.ArrayStrideY = new double[] { strideY };
            curHoles.HeadOffsetX = - strideX / 2;
            curHoles.CutLength = m_plateProps.Thickness * 2.0 + 1.0;
            curHoles.Refresh();
        }
        private void setChamfer(PartBase retPlate, double len)
        {
            Define.CornerCutType chamferType;
            switch (m_chamferType)
            {
                case 1:
                    chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                case 3:
                    chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    break;
                case 4:
                    chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    break;
                case 0:
                default:
                    chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    break;
            }
            for (int i = 0; i < retPlate.CtrlPoints.Length; i++)
            {
                ChamferCtrlPoint chamferPoint = retPlate.CtrlPoints[i] as ChamferCtrlPoint;
                if (chamferPoint != null)
                {
                    chamferPoint.CutX = len;
                    chamferPoint.CutY = len;
                    chamferPoint.CutType = chamferType;
                    chamferPoint.Flush();
                    retPlate.Flush();
                }
            }
        }
        //得到矩形板的控制点
        private Vec3[] GetCtrlPoints(double plateH, double plateB)
        {
            Vec3 dir;
            if (m_context.TargetComp.WorkAxis.Z.IsParallelTo(m_axisZ))
                dir = m_context.TargetComp.WorkAxis.Z;
            else
                dir = m_axisZ;
            var centerPoint = m_centerPoint + dir * m_vertOffset;
            Vec3[] ctrlPoints = new Vec3[4];
            ctrlPoints[0] = centerPoint + m_axisX * plateH / 2 + m_axisY * plateB / 2;
            ctrlPoints[1] = ctrlPoints[0] - m_axisX * plateH;
            ctrlPoints[2] = ctrlPoints[1] - m_axisY * plateB;
            ctrlPoints[3] = ctrlPoints[2] + m_axisX * plateH;
            var line = (ctrlPoints[1] - ctrlPoints[0]).Normalize()
                .Cross((ctrlPoints[3] - ctrlPoints[0]).Normalize());
            if (line.Dot(dir) > 0)
                return ctrlPoints;
            var points = new List<Vec3>();
            for (int i = ctrlPoints.Length - 1; i >= 0; i--)
                points.Add(ctrlPoints[i]);
            return points.ToArray();
        }
        private Vec3[] GetWeldsPoints(Vec3[] points)
        {
            var len = points.Length;
            if (len < 2)
                return null;
            var ctrlPoints = new Vec3[len];
            for (int i = 0; i < points.Length; i++)
                ctrlPoints[i] = points[i] / 2 + points[(i + 1) % len] / 2;
            return ctrlPoints;
        }
        /// <summary>
        /// 得到向上方向为非自动时内膜板的构建平面法线
        /// </summary>
        private List<Vec3> GetBuildPlaneDir()
        {
            var Target = m_context.TargetComp;
            Vec3[] normals = new Vec3[]
            {
                Target.LocalMatrix.X,
                Target.LocalMatrix.Y,
                - Target.LocalMatrix.X,
                - Target.LocalMatrix.Y
            };
            int num = -1;
            double max = -2;
            for (int i = 0; i < normals.Length; i++)
            {
                var temp = normals[i].Dot(m_context.MainLine);
                if (temp > max)
                {
                    max = temp;
                    num = i;
                }
            }
            List<Vec3> dir = new List<Vec3>();
            dir.Add(normals[num]);
            if (num == 0 || num == 2)
                dir.Add(normals[1]);
            else
                dir.Add(normals[0]);
            return dir;
        }
    }
}
