﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace MultipleSiffeners1064
{
    class Util
    {
        public static void CreateSteelBySymmetry(ConnectionComponent cmp, Plane zplane, double thickness, Plane[] sidePlanes, bool bRuntime, Plane plane)
        {
            Plane symmetryCenterPlane = plane.Clone();
            Plane zCtrlPlane = zplane.Clone();
            cmp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, bRuntime);
            if (symmetryCenterPlane != null)
            {
                double dist = zCtrlPlane.SignedDist(symmetryCenterPlane.Origin);
                zCtrlPlane.Move(2 * dist - thickness); //移动板平面到对应位置
                cmp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, bRuntime);
            }
            cmp.Flush();
        }

        public static Plane CreatePlaneBySymmetry(Plane zplane, Plane plane)
        {
            Plane symmetryCenterPlane = plane.Clone();
            Plane zCtrlPlane = zplane.Clone();
            double dist = zCtrlPlane.SignedDist(symmetryCenterPlane.Origin);
            return zCtrlPlane.Move(2 * dist).Invert(); //移动板平面到对应位置
        }

        public static Vec3 SymmetryPoint(Plane plane, Vec3 point)
        {
            double dist = plane.SignedDist(point);
            point = point - 2 * dist * plane.Normal.Normalize();
            return point;
        }

        public static Vec3 PlaneLineIntersect(Plane p, Vec3 v1, Vec3 v2)
        {
            Vec3 v = v2 - v1; //v1指向v2的向量
            Vec3 normal = p.Normal.Normalize();
            if (Math.Abs(normal.Dot(v)) < 1e-6)
                return Vec3.ZERO;
            double dist1 = p.SignedDist(v1);
            double dist2 = p.SignedDist(v2);
            if (dist1 == 0)
                return v1;
            if (dist2 == 0)
                return v2;
            if (dist1 * dist2 < 0) //在异侧
            {
                normal.Along(v);
                dist1 = Math.Abs(dist1);
                dist2 = Math.Abs(dist2);
                Vec3 v1Proj = v1 + normal * dist1; //v1点在平面上的投影点
                Vec3 v2Proj = v2 - normal * dist2;
                Vec3 horiVec = (v2Proj - v1Proj).Normalize(); //v1投影点指向v2投影点的单位向量
                double percentage = dist1 / (dist1 + dist2);
                return v1Proj + horiVec * v1Proj.Dist(v2Proj) * percentage;
            }
            else //在同侧
            {
                if (dist1 > 0) //normal方向改为点指向平面的方向
                    normal *= -1;
                dist1 = Math.Abs(dist1);
                dist2 = Math.Abs(dist2);
                Vec3 v1Proj = v1 + normal * dist1; //v1点在平面上的投影点
                Vec3 v2Proj = v2 + normal * dist2;
                if (dist1 > dist2) //保证v1投影点靠近交点
                {
                    double temp = dist1;
                    dist1 = dist2;
                    dist2 = temp;
                    Vec3 tmp = v1Proj;
                    v1Proj = v2Proj;
                    v2Proj = tmp;
                }
                if (v1Proj.Dist(v2Proj) < 1e-6)
                    return v1Proj;
                Vec3 horiVec = (v1Proj - v2Proj).Normalize(); //v2投影点指向v1投影点的单位向量
                double percentage = dist1 / (dist2 - dist1);
                return v1Proj + horiVec * v1Proj.Dist(v2Proj) * percentage;
            }
        }

        /// <summary>
        /// 两点的中心点
        /// </summary>
        /// <param name="p1"></param>
        /// <param name="p2"></param>
        /// <returns></returns>
        public static Vec3 CenterPoint(Vec3 p1, Vec3 p2)
        {
            Vec3 v = (p2 - p1).Normalize();
            p1 += v * p1.Dist(p2) / 2;
            return p1;
        }

        /// <summary>
        /// 设置多个默认值，排在前面的优先赋值
        /// </summary>
        /// <param name="value"></param>
        /// <param name="setValueList"></param>
        public static void SetDefaultValue(ref double value, params double[] setValueList)
        {
            for(int i = 0; i < setValueList.Length; i++)
            {
                if (value > -1e10 && value < 1e10) //一个正常的区间
                    break;
                value = setValueList[i];
            }
        }

        public static bool PlatePropIsValid(PlateProperties props)
        {
            if (props.Thickness <= 0 || props.Width <= 0 || props.Height <= 0)
                return false;
            return true;
        }

        /// <summary>
        /// 过一条直线做某个平面的垂直面
        /// </summary>
        /// <param name="plane"></param>
        /// <param name="line"></param>
        /// <param name="vec">指定面的朝向</param>
        /// <returns></returns>
        public static Plane VerticalPlane(Plane plane, Line line, Vec3 vec)
        {
            Vec3 normal = plane.Normal.Cross(line.Dir);
            normal = normal.Along(vec);
            return new Plane(line.V0, normal);
        }

        public static double VectorAngle(Vec3 v1, Vec3 v2)
        {
            double dot = v1.Dot(v2);
            double cosValue = dot / (v1.Length * v2.Length);
            return Math.Acos(cosValue);
        }

        public static Plane CreatePlaneBy2Line(Line line1, Line line2)
        {
            Vec3 normal = line1.Dir.Cross(line2.Dir);
            if (normal.Dist(Vec3.ZERO) > 1e-4)
                return new Plane(line1.V0, normal);
            normal = line1.Dir.Cross(line2.V0 - line1.V0);
            if (normal.Dist(Vec3.ZERO) > 1e-4)
                return new Plane(line1.V0, normal);
            return Plane.NULL; //未判断异面直线
        }
    }
}
