﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace ShortColumn133
{

    public class ComponentInstanceContext
    {
        private ConnectionComponent m_comp;
        public ColumnSplice42.SpliceContext spliceContext = null;

        // 主零件
        public Beam m_mainPart;

        private Plane m_mainNearBoundPlane; //主零件靠近中心(靠近次零件)的边界面（FrontBound或BackBound）
        private Plane m_mainAwayBoundPlane; //主零件远离中心的边界面
        private Plane m_mainNearWebPlane;
        private Plane m_mainAwayWebPlane;

        private Plane m_mainFrontBound; 
        private Plane m_mainBackBound; 
        private Plane m_mainTopBound; 
        private Plane m_mainBottomBound; 
        private Plane m_mainTopFlangeInside; 
        private Plane m_mainBottomFlangeInside; 
        private Plane m_mainHCenterPlane; //梁中心面（法线为Top方向）

        private Plane m_mainFrontWeb; //前腹板面
        private Plane m_mainBackWeb; //后腹板面
        private Plane m_mainWebCenterPlane; //腹板中心面(法线为front方向)

        private double m_mainHeight = 0; //高度
        private double m_mainWidth = 0; //宽度
        private double m_mainWebThickness = 0; //腹板厚度
        private double m_mainFlangeThickness = 0; //翼缘厚度


        //次零件
        private Beam m_subPart;

        private Plane m_subNearCenterPlane;
        private Plane m_subAwayCenterPlane;

        private Plane m_subFrontBound; 
        private Plane m_subBackBound;
        private Plane m_subFrontWeb;
        private Plane m_subBackWeb;
        private Plane m_subTopBound; 
        private Plane m_subBottomBound;
        private Plane m_subTopFlangeInside;
        private Plane m_subBottomFlangeInside;

        private Plane m_subHoriCenterPlane;
        private Plane m_subVertCenterPlane; 

        private Plane m_alignPlane;
        //
        private Plane m_subTopPlane;
        private Plane m_subBottomPlane;
        private Plane m_subFrontPlane;
        private Plane m_subBackPlane;
        private Vec3 m_topOrient;
        private bool m_isSubRotate;
        private bool m_isMainRotate = false;
        //
        private SteelPlate m_topPlate, m_bottomPlate, m_webPlate, m_topBackingBar, m_bottomBackingBar;


        ComponentInstanceContext(){}

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp))
            {
                compContext.Clear();
                return null;
            }
            return compContext;
        }

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            Beam subPart = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            if (mainPart == null || subPart == null)
                return false;
            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart;

            BeamPlanes mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            BeamPlanes subPlanes = BeamPlanes.CreateFromBeam(m_subPart);


            /*
             * 确定上方向：把主零件（柱）摆在左侧，次零件（梁）摆在右侧，上方向为柱的Head或Tail方向（通用性中可设置）
             * 图形页的第1个图的上侧确定为前侧（上×右），第2个图的上侧为节点上侧
             */
            m_topOrient = mainPlanes.HeadBound.Normal;
            if (m_topOrient.Dot(Vec3.AXIS_Z) < 0)
                m_topOrient *= -1;

            /*
             * 主梁（柱）的边界面
             */
            //
            m_mainTopBound = mainPlanes.TopBound.Clone();
            m_mainBottomBound = mainPlanes.BottomBound.Clone();
            m_mainFrontBound = mainPlanes.FrontBound.Clone();
            m_mainBackBound = mainPlanes.BackBound.Clone();
            m_mainFrontWeb = mainPlanes.FrontWeb.Clone();
            m_mainBackWeb = mainPlanes.BackWeb.Clone();
            //
            m_subTopBound = subPlanes.TopBound.Clone();
            m_subBottomBound = subPlanes.BottomBound.Clone();
            m_subFrontBound = subPlanes.FrontBound.Clone();
            m_subBackBound = subPlanes.BackBound.Clone();
            m_subFrontWeb = subPlanes.FrontWeb.Clone();
            m_subBackWeb = subPlanes.BackWeb.Clone();


            /*
             * 设置次梁靠近中心的面
             */
            Vec3[] subStartEndPoints = Util.StartEndPoint(subPart); //获取次梁起始点
            Plane[] subHeadTailPlanes = { subPlanes.HeadBound, subPlanes.TailBound }; //次梁起始边界面
            Plane[] comparePlanes = { mainPlanes.FrontBound, mainPlanes.BackBound, mainPlanes.TopBound, mainPlanes.BottomBound};
            for(int i = 0; i < 4; i++)
            {
                if (GetNearCenterPlane(ref subStartEndPoints, ref subHeadTailPlanes, comparePlanes[i]))
                    break;
            }
            m_subNearCenterPlane = subHeadTailPlanes[0];
            m_subAwayCenterPlane = subHeadTailPlanes[1];


            /*
             * 设置主零件近心面
             */
            //通过柱腹板方向和梁首尾方向的夹角判断柱是否旋转
            Vec3 subOrient = subStartEndPoints[1] - subStartEndPoints[0];
            Vec3 frontBackOrient = mainPlanes.FrontBound.Normal;
            frontBackOrient = frontBackOrient.Along(subOrient);
            double cosAngle = subOrient.Dot(frontBackOrient) / subOrient.Length / frontBackOrient.Length;
            //正常情况
            if (mainPlanes.TopFlangeInside != null)
                m_mainTopFlangeInside = mainPlanes.TopFlangeInside.Clone();
            else
                m_mainTopFlangeInside = mainPlanes.TopBound.Invert();
            if (mainPlanes.BottomFlangeInside != null)
                m_mainBottomFlangeInside = mainPlanes.BottomFlangeInside.Clone();
            else
                m_mainBottomFlangeInside = mainPlanes.BottomBound.Invert();
            //夹角超过了45度，认为旋转了
            if (cosAngle < 0.7071)
            {
                double mainTopFlangeThickness = mainPlanes.TopFlangeInside.Dist(m_mainTopBound.Origin);
                double mainBottomFlangeThickness = mainPlanes.BottomFlangeInside.Dist(m_mainBottomBound.Origin);
                double mainWebThickness = m_mainFrontWeb.Dist(m_mainBackWeb.Origin);
                Util.Swap(ref m_mainTopBound, ref m_mainFrontBound);
                Util.Swap(ref m_mainBottomBound, ref m_mainBackBound);
                m_mainTopFlangeInside = m_mainTopBound.Invert().Move(mainTopFlangeThickness);
                m_mainBottomFlangeInside = m_mainBottomBound.Invert().Move(mainBottomFlangeThickness);
                Plane centerPlane = Util.CenterPlane(m_mainFrontBound, m_mainBackBound);
                m_mainFrontWeb = centerPlane.Move(mainWebThickness / 2);
                m_mainBackWeb = centerPlane.Invert().Move(mainWebThickness / 2);
                m_isMainRotate = true;
            }
            //判断近心面
            if(m_mainFrontWeb.Dist(subStartEndPoints[1]) <= m_mainBackWeb.Dist(subStartEndPoints[1])) 
            {
                m_mainNearBoundPlane = m_mainFrontBound;
                m_mainNearWebPlane = m_mainFrontWeb;
                m_mainAwayBoundPlane = m_mainBackBound;
                m_mainAwayWebPlane = m_mainBackWeb;
            }
            else
            {
                m_mainNearBoundPlane = m_mainBackBound;
                m_mainNearWebPlane = m_mainBackWeb;
                m_mainAwayBoundPlane = m_mainFrontBound;
                m_mainAwayWebPlane = m_mainFrontWeb;
            }
            //判断顶底面（柱的截面的顶底，不是节点顶底方向）
            Vec3 topOrient = m_mainNearBoundPlane.Normal.Cross(m_topOrient);
            if (topOrient.Dot(m_mainTopBound.Normal) < 0)
            {
                Util.Swap(ref m_mainTopBound, ref m_mainBottomBound);
                Util.Swap(ref m_mainTopFlangeInside, ref m_mainBottomFlangeInside);
            }

            /*
             * 设置次梁包围面
             */
            //次梁是否旋转
            Vec3 subTop = m_subTopBound.Normal.Normalize().Along(m_topOrient);
            Vec3 subFront = m_subFrontBound.Normal.Normalize().Along(m_topOrient);
            m_isSubRotate = subTop.Dot(m_topOrient) < subFront.Dot(m_topOrient);
            Util.SetDefaultValue(ref m_subTopFlangeInside, subPlanes.TopFlangeInside);
            Util.SetDefaultValue(ref m_subBottomFlangeInside, subPlanes.BottomFlangeInside);
            if (m_isSubRotate)
            {
                //次梁顶底面
                if (m_subTopBound.Normal.Dot(m_mainBottomBound.Normal) < 0)
                {
                    Util.Swap(ref m_subTopBound, ref m_subBottomBound);
                    Util.Swap(ref m_subTopFlangeInside, ref m_subBottomFlangeInside);
                }
                //次梁前后面
                if (m_subBackBound.Normal.Dot(m_topOrient) < 0)
                {
                    Util.Swap(ref m_subBackBound, ref m_subBackBound);
                    Util.Swap(ref m_subFrontWeb, ref m_subBackWeb);
                }
            }
            else
            {
                //次梁顶底面
                if (m_subTopBound.Normal.Dot(m_topOrient) < 0)
                {
                    Util.Swap(ref m_subTopBound, ref m_subBottomBound);
                    Util.Swap(ref m_subTopFlangeInside, ref m_subBottomFlangeInside);
                }
                //次梁前后面
                if (m_subFrontBound.Normal.Dot(m_mainBottomBound.Normal) < 0)
                {
                    Util.Swap(ref m_subFrontBound, ref m_subBackBound);
                    Util.Swap(ref m_subFrontWeb, ref m_subBackWeb);
                }
            }
            //端板包围面
            Plane[] subSidePlanes = { m_subTopBound, m_subBottomBound, m_subFrontBound, m_subBackBound };
            m_subTopPlane = GetSameOrientPlane(subSidePlanes, m_topOrient);
            m_subBottomPlane = GetSameOrientPlane(subSidePlanes, -m_topOrient);
            m_subFrontPlane = GetSameOrientPlane(subSidePlanes, m_mainBottomBound.Normal);
            m_subBackPlane = GetSameOrientPlane(subSidePlanes, m_mainTopBound.Normal);


            /*
             * 梁参数
             */
            //
            m_mainWidth = m_mainFrontBound.Dist(m_mainBackBound.Origin);
            m_mainHeight = m_mainBottomBound.Dist(m_mainTopBound.Origin);
            m_mainWebThickness = m_mainFrontWeb.Dist(m_mainBackWeb.Origin);
            m_mainHCenterPlane = m_mainTopBound.Move(-m_mainHeight / 2);
            m_mainWebCenterPlane = m_mainNearWebPlane.Move(-m_mainWebThickness / 2);
            m_mainFlangeThickness = mainPlanes.TopFlange.Dist(mainPlanes.TopFlangeInside.Origin);
            //
            //m_subWidth = m_subFrontBound.Dist(m_subBackBound.Origin);
            //m_subHeight = m_subBottomBound.Dist(m_subTopBound.Origin);
            //m_subWebThickness = m_subFrontWeb.Dist(m_subBackWeb.Origin);
            m_subHoriCenterPlane = Util.CenterPlane(m_subTopPlane, m_subBottomPlane);
            m_subVertCenterPlane = Util.CenterPlane(m_subFrontPlane, m_subBackPlane);
            //m_subWebCenterPlane = m_subFrontWeb.Move(-m_subWebThickness / 2);
            //m_subFlangeThickness = subPlanes.TopFlange.Dist(subPlanes.TopFlangeInside.Origin);

            return true;
        }


        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;

            m_subNearCenterPlane = null;
            m_subAwayCenterPlane = null;
        }


        public bool GetNearCenterPlane(ref Vec3[] points, ref Plane[] planes, Plane comparePlane)
        {
            double dist1 = comparePlane.SignedDist(points[0]); //起点距主零件距离
            double dist2 = comparePlane.SignedDist(points[1]);
            //起始点不在面的同一侧
            if (dist1 * dist2 < 0)
                return false;
            //转为正值，方便比较
            dist1 = Math.Abs(dist1);
            dist2 = Math.Abs(dist2);
            //两点连线与面平行
            if (Math.Abs(dist1 - dist2) < 1e-4)
                return false;
            //
            if (dist2 < dist1)
            {
                Util.Swap(ref points[0], ref points[1]);
                Util.Swap(ref planes[0], ref planes[1]);
            }
            return true;
        }
        public Plane GetSameOrientPlane(Plane[] planes, Vec3 orient)
        {
            double dot = double.MinValue;
            Plane rePlane = Plane.IDENTITY;
            for (int i = 0; i < planes.Length; i++)
            {
                Vec3 nowOrient = planes[i].Normal.Normalize();
                double nowDot = nowOrient.Dot(orient);
                if (nowDot > dot)
                {
                    rePlane = planes[i];
                    dot = nowDot;
                }
            }
            return rePlane;
        }

        public void test(Plane p)
        {
            Comp.CreateQuad(p, 4000, 4000, true);
        }
        public void test(Vec3 p)
        {
            Comp.CreatePoint(p, true);
        }
        public void test(Vec3 v1, Vec3 v2)
        {
            Comp.CreateLine(v1, v2, 0, Color.Red, true);
        }
        public void test(Mat43 mat, double len = 1000)
        {
            Comp.CreatePoint(mat.Pos, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.X.Normalize(), 0, Color.Red, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Y.Normalize(), 0, Color.Green, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Z.Normalize(), 0, Color.Purple, true);
        }



        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }

        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }

        public Beam SubPart { get => m_subPart; set => m_subPart = value; }

        public Plane MainTopBound { get => m_mainTopBound; set => m_mainTopBound = value; }

        public Plane MainBottomBound { get => m_mainBottomBound; set => m_mainBottomBound = value; }

        public Plane SubFrontWeb { get => m_subFrontWeb; set => m_subFrontWeb = value; }

        public Plane SubBackWeb { get => m_subBackWeb; set => m_subBackWeb = value; }

        /// <summary>
        /// 主零件靠近中心(靠近次零件)的边界面
        /// </summary>
        public Plane MainNearBoundPlane { get => m_mainNearBoundPlane; set => m_mainNearBoundPlane = value; }

        /// <summary>
        /// 主零件远离中心(远离次零件)的边界面
        /// </summary>
        public Plane MainAwayBoundPlane { get => m_mainAwayBoundPlane; set => m_mainAwayBoundPlane = value; }

        /// <summary>
        /// 主零件靠近中心(靠近次零件)的腹板面
        /// </summary>
        public Plane MainNearWebPlane { get => m_mainNearWebPlane; set => m_mainNearWebPlane = value; }

        /// <summary>
        /// 主零件远离中心(远离次零件)的腹板面
        /// </summary>
        public Plane MainAwayWebPlane { get => m_mainAwayWebPlane; set => m_mainAwayWebPlane = value; }

        /// <summary>
        /// 主梁腹板中心面(法线为front方向)
        /// </summary>
        public Plane MainWebCenterPlane { get => m_mainWebCenterPlane; set => m_mainWebCenterPlane = value; }

        /// <summary>
        /// 主梁中心面（法线为Top方向）
        /// </summary>
        public Plane MainHCenterPlane { get => m_mainHCenterPlane; set => m_mainHCenterPlane = value; }

        public Plane SubTopBound { get => m_subTopBound; set => m_subTopBound = value; }

        public Plane SubBottomBound { get => m_subBottomBound; set => m_subBottomBound = value; }

        public Plane SubFrontBound { get => m_subFrontBound; set => m_subFrontBound = value; }

        public Plane SubBackBound { get => m_subBackBound; set => m_subBackBound = value; }

        /// <summary>
        /// 次零件靠近中心(靠近主零件)的面
        /// </summary>
        public Plane SubNearCenterPlane { get => m_subNearCenterPlane; set => m_subNearCenterPlane = value; }

        /// <summary>
        /// 次零件远离中心(远离主零件)的面
        /// </summary>
        public Plane SubAwayCenterPlane { get => m_subAwayCenterPlane; set => m_subAwayCenterPlane = value; }

        /// <summary>
        /// 次梁中心面(法线为节点top方向)
        /// </summary>
        public Plane SubHoriCenterPlane { get => m_subHoriCenterPlane; set => m_subHoriCenterPlane = value; }

        /// <summary>
        /// 次梁中心面（法线为节点front方向）
        /// </summary>
        public Plane SubVertCenterPlane { get => m_subVertCenterPlane; set => m_subVertCenterPlane = value; }

        /// <summary>
        /// 次梁的对齐面，方向从主梁指向次梁
        /// </summary>
        public Plane AlignPlane { get => m_alignPlane; set => m_alignPlane = value; }

        public double MainHeight { get => m_mainHeight; set => m_mainHeight = value; }
        public double MainWidth { get => m_mainWidth; set => m_mainWidth = value; }
        public double MainFlangeThickness { get => m_mainFlangeThickness; set => m_mainFlangeThickness = value; }

        /// <summary>
        /// 节点上方向
        /// </summary>
        public Vec3 TopOrient { get => m_topOrient; set => m_topOrient = value; }


        public Plane MainTopFlangeInside { get => m_mainTopFlangeInside; set => m_mainTopFlangeInside = value; }
        public Plane MainBottomFlangeInside { get => m_mainBottomFlangeInside; set => m_mainBottomFlangeInside = value; }
        public Plane SubTopFlangeInside { get => m_subTopFlangeInside; set => m_subTopFlangeInside = value; }
        public Plane SubBottomFlangeInside { get => m_subBottomFlangeInside; set => m_subBottomFlangeInside = value; }



        public Plane SubTopPlane { get => m_subTopPlane; set => m_subTopPlane = value; }
        public Plane SubBottomPlane { get => m_subBottomPlane; set => m_subBottomPlane = value; }
        public Plane SubFrontPlane { get => m_subFrontPlane; set => m_subFrontPlane = value; }
        public Plane SubBackPlane { get => m_subBackPlane; set => m_subBackPlane = value; }

        public bool IsSubRotate { get => m_isSubRotate; set => m_isSubRotate = value; }

        public bool IsMainRotate { get => m_isMainRotate; set => m_isMainRotate = value; }

        public SteelPlate TopPlate { get => m_topPlate; set => m_topPlate = value; }
        public SteelPlate BottomPlate { get => m_bottomPlate; set => m_bottomPlate = value; }
        public SteelPlate WebPlate { get => m_webPlate; set => m_webPlate = value; }
        public SteelPlate TopBackingBar { get => m_topBackingBar; set => m_topBackingBar = value; }
        public SteelPlate BottomBackingBar { get => m_bottomBackingBar; set => m_bottomBackingBar = value; }



    }
}
