﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace SpliceConnection77
{
    public class WeldHander
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        private double r1, r2;

        private bool isCreateTopFlangeWeld = false;
        private bool isCreateBottomFlangeWeld = false;

        /*
         * 为了确定衬垫板与主（次）梁的焊缝边缘点，创建一个x轴。其中，
         * 主梁指向次梁为x轴正方向，两梁中间为原点，记录主次梁的边缘点坐标。
         */
        double[] topOutsideLocation; //上翼缘外侧的两个边缘点
        double[] topInsideLocation; //上翼缘内侧的两个边缘点
        double[] bottomOutsideLocation; //下翼缘外侧的两个边缘点
        double[] bottomInsideLocation; //下翼缘内侧的两个边缘点

        public WeldHander(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;

            //初始化焊缝属性
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }

            //初始化边缘点坐标
            double gap = m_context.Gap;
            topOutsideLocation = new double[] { -gap, gap };
            topInsideLocation = new double[] { -gap, gap };
            bottomOutsideLocation = new double[] { -gap, gap };
            bottomInsideLocation = new double[] { -gap, gap };
        }

        public void Execute()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }


            //翼缘焊接x2、腹板焊接（这里假定主次梁截面相同且已经对齐）
            CreateBeamWelds();

            //创建衬垫板的同时创建焊缝
            CreateWeldBacking();

            //焊接出入孔的创建
            r1 = m_paramsHandler.GetDoubleValue(ParamNames.WELD_ACCESS_HOLE_RADIUS_MAIN);
            r2 = m_paramsHandler.GetDoubleValue(ParamNames.WELD_ACCESS_HOLE_RADIUS_SUB);
            if(r1 != 0)
            {
                if(isCreateTopFlangeWeld)
                    CommonCreateHole(m_context.MainTopFlangeInside, m_context.MainNearCenterPlane, m_context.MainFrontWeb, r1, m_context.MainPart);
                if(isCreateBottomFlangeWeld)
                    CommonCreateHole(m_context.MainBottomFlangeInside, m_context.MainNearCenterPlane, m_context.MainFrontWeb, r1, m_context.MainPart);
            }
            if(r2 != 0)
            {
                if (isCreateTopFlangeWeld)
                    CommonCreateHole(m_context.SubTopFlangeInside, m_context.SubNearCenterPlane, m_context.SubFrontWeb, r2, m_context.SubPart);
                if (isCreateBottomFlangeWeld)
                    CommonCreateHole(m_context.SubBottomFlangeInside, m_context.SubNearCenterPlane, m_context.SubFrontWeb, r2, m_context.SubPart);
            }

            //创建楼层板的同时创建焊缝
            CreateFloorPlates();
        }

        private void CreateBeamWelds()
        {
            //翼缘焊接
            Plane frontPlane = m_context.MainFront.SignedDist(m_context.SubFront.Origin) > 0 ? m_context.SubFront : m_context.MainFront;
            Plane backPlane = m_context.MainBack.SignedDist(m_context.SubBack.Origin) > 0 ? m_context.SubBack : m_context.MainBack;
            Plane triCutCtrlPlane = frontPlane.Move(m_context.m_main_t).Invert();
            double triCutLength = frontPlane.Dist(backPlane.Origin) + m_context.m_main_t * 2;
            //上翼缘的焊接准备选项：0，1，2，3——无、两者、主、次
            int topFlangeWeldMode = m_paramsHandler.GetIntValue(ParamNames.WELD_READY_TOP_FLANGE);
            do
            {
                double horiSideUp = m_context.m_main_t / 2;
                double vertSideUp = m_context.m_main_t / 2;
                double horiSideDown = m_context.m_main_t / 2;
                double vertSideDown = m_context.m_main_t / 2;
                TriCutParameters(m_weldProps[0], ref horiSideUp, ref vertSideUp, ref horiSideDown, ref vertSideDown);
                if (topFlangeWeldMode == 0)
                    break;
                if (topFlangeWeldMode != 3) //主梁焊接
                {
                    //线上部分
                    CreateTriCut(m_context.MainNearCenterPlane, m_context.MainTopFlange, m_context.MainPart, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                    //线下部分
                    CreateTriCut(m_context.MainNearCenterPlane, m_context.MainTopFlangeInside, m_context.MainPart, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                    CommonCreateFlangeWeld(m_context.MainTopFlange.Move(-vertSideUp), m_weldProps[0], new PartBase[] {m_context.MainPart, m_context.SubPart });
                    topOutsideLocation[0] -= horiSideUp;
                    topInsideLocation[0] -= horiSideDown;
                }
                if (topFlangeWeldMode != 2) //次梁焊接
                {
                    CreateTriCut(m_context.SubNearCenterPlane, m_context.SubTopFlange, m_context.SubPart, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                    CreateTriCut(m_context.SubNearCenterPlane, m_context.SubTopFlangeInside, m_context.SubPart, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                    CommonCreateFlangeWeld(m_context.SubTopFlange.Move(-vertSideUp), m_weldProps[0], new PartBase[] { m_context.SubPart, m_context.MainPart });
                    topOutsideLocation[1] += horiSideUp;
                    topInsideLocation[1] += horiSideDown;
                }
            } while (false);
            //下翼缘的焊接准备选项：0，1，2，3——无、两者、主、次
            int bottomFlangeWeldMode = m_paramsHandler.GetIntValue(ParamNames.WELD_READY_BOTTOM_FLANGE);
            do
            {
                double horiSideUp = m_context.m_main_t / 2;
                double vertSideUp = m_context.m_main_t / 2;
                double horiSideDown = m_context.m_main_t / 2;
                double vertSideDown = m_context.m_main_t / 2;
                TriCutParameters(m_weldProps[3], ref horiSideDown, ref vertSideDown, ref horiSideUp, ref vertSideUp);
                if (bottomFlangeWeldMode == 0)
                    break;
                if (bottomFlangeWeldMode != 3) //主梁焊接
                {
                    //线上部分
                    CreateTriCut(m_context.MainNearCenterPlane, m_context.MainBottomFlange, m_context.MainPart, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                    //线下部分
                    CreateTriCut(m_context.MainNearCenterPlane, m_context.MainBottomFlangeInside, m_context.MainPart, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                    CommonCreateFlangeWeld(m_context.MainBottomFlange.Move(-vertSideUp), m_weldProps[3], new PartBase[] { m_context.MainPart, m_context.SubPart });
                    bottomOutsideLocation[0] -= horiSideUp;
                    bottomInsideLocation[0] -= horiSideDown;
                }
                if (bottomFlangeWeldMode != 2) //次梁焊接
                {
                    CreateTriCut(m_context.SubNearCenterPlane, m_context.SubBottomFlange, m_context.SubPart, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                    CreateTriCut(m_context.SubNearCenterPlane, m_context.SubBottomFlangeInside, m_context.SubPart, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                    CommonCreateFlangeWeld(m_context.SubBottomFlange.Move(-vertSideUp), m_weldProps[3], new PartBase[] { m_context.SubPart, m_context.MainPart });
                    bottomOutsideLocation[1] += horiSideUp;
                    bottomInsideLocation[1] += horiSideDown;
                }
            } while (false);

            //腹板焊接
            double mainWebCutLength = m_context.m_main_h - 2 * m_context.m_main_t;
            double subWebCutLength = m_context.m_sub_h - 2 * m_context.m_sub_t;
            //腹板的焊接准备选项：0，1，2，3——无、两者、主、次
            int webFlangeWeldMode = m_paramsHandler.GetIntValue(ParamNames.WELD_READY_WEB);
            do
            {
                double horiSideUp = m_context.m_main_s / 2;
                double vertSideUp = m_context.m_main_s / 2;
                double horiSideDown = m_context.m_main_s / 2;
                double vertSideDown = m_context.m_main_s / 2;
                TriCutParameters(m_weldProps[1],ref horiSideUp, ref vertSideUp, ref horiSideDown, ref vertSideDown);
                if (webFlangeWeldMode == 0)
                    break;
                if (webFlangeWeldMode != 3) //主梁焊接
                {
                    //线上部分
                    CreateTriCut(m_context.MainBackWeb, m_context.MainNearCenterPlane, m_context.MainPart, horiSideUp, vertSideUp, m_context.MainTopFlangeInside, mainWebCutLength);
                    //线下部分
                    CreateTriCut(m_context.MainFrontWeb, m_context.MainNearCenterPlane, m_context.MainPart, horiSideDown, vertSideDown, m_context.MainTopFlangeInside, mainWebCutLength);
                    CommonCreateWebWeld(m_context.MainBackWeb.Move(-vertSideUp), new PartBase[] { m_context.MainPart, m_context.SubPart });
                }
                if (webFlangeWeldMode != 2) //次梁焊接
                {
                    CreateTriCut(m_context.SubBackWeb, m_context.SubNearCenterPlane, m_context.SubPart, horiSideUp, vertSideUp, m_context.SubTopFlangeInside, subWebCutLength);
                    CreateTriCut(m_context.SubFrontWeb, m_context.SubNearCenterPlane, m_context.SubPart, horiSideDown, vertSideDown, m_context.SubTopFlangeInside, subWebCutLength);
                    CommonCreateWebWeld(m_context.SubBackWeb.Move(-vertSideUp), new PartBase[] { m_context.SubPart, m_context.MainPart });
                }
            } while (false);

            //腹板与连接板的焊接
            {
                Define.WeldType typeUp = m_weldProps[2].TypeUp;
                Define.WeldType typeDown = m_weldProps[2].TypeDown;
                Define.WeldContour contourUp = m_weldProps[2].ContourUp;
                Define.WeldContour contourDown = m_weldProps[2].ContourDown;
                //有效判断
                bool upVaild = ((typeUp != Define.WeldType.WELD_TYPE_FILLET && typeUp != Define.WeldType.WELD_TYPE_NONE) || contourUp != Define.WeldContour.WELD_CONTOUR_NONE);
                bool downVaild = ((typeDown != Define.WeldType.WELD_TYPE_FILLET && typeDown != Define.WeldType.WELD_TYPE_NONE) || contourDown != Define.WeldContour.WELD_CONTOUR_NONE);
                m_weldProps[2].SizeUp = 6;
                m_weldProps[2].SizeDown = 6;
                //前腹板面与连接板的焊接
                if (upVaild && m_context.WebFrontPlateCreate.mainParts.Count > 0)
                {
                    Vec3 v1 = m_context.WebFrontPlateCreate.frontPlane.Intersect(m_context.MainFrontWeb, m_context.WebFrontPlateCreate.tailPlane);
                    Vec3 v2 = m_context.WebFrontPlateCreate.backPlane.Intersect(m_context.MainFrontWeb, m_context.WebFrontPlateCreate.tailPlane);
                    Weld weld1 = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.WebFrontPlateCreate.mainParts[0], new Vec3[] { v1, v2 }, true);
                    m_weldProps[2].ApplyToWeld(weld1);
                    weld1.Flush();
                    Vec3 v3 = m_context.WebFrontPlateCreate.frontPlane.Intersect(m_context.MainFrontWeb, m_context.WebFrontPlateCreate.headPlane);
                    Vec3 v4 = m_context.WebFrontPlateCreate.backPlane.Intersect(m_context.MainFrontWeb, m_context.WebFrontPlateCreate.headPlane);
                    Weld weld2 = m_context.Comp.CreatePolyWeld(m_context.SubPart, m_context.WebFrontPlateCreate.subParts[0], new Vec3[] { v3, v4 }, true);
                    m_weldProps[2].ApplyToWeld(weld2);
                    weld2.Flush();
                }
                //后腹板面与连接板的焊接
                if(downVaild && m_context.WebBackPlateCreate.mainParts.Count > 0)
                {
                    Vec3 v1 = m_context.WebBackPlateCreate.frontPlane.Intersect(m_context.MainBackWeb, m_context.WebBackPlateCreate.tailPlane);
                    Vec3 v2 = m_context.WebBackPlateCreate.backPlane.Intersect(m_context.MainBackWeb, m_context.WebBackPlateCreate.tailPlane);
                    Weld weld1 = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.WebBackPlateCreate.mainParts[0], new Vec3[] { v1, v2 }, true);
                    m_weldProps[2].ApplyToWeld(weld1);
                    weld1.Flush();
                    Vec3 v3 = m_context.WebBackPlateCreate.frontPlane.Intersect(m_context.MainBackWeb, m_context.WebBackPlateCreate.headPlane);
                    Vec3 v4 = m_context.WebBackPlateCreate.backPlane.Intersect(m_context.MainBackWeb, m_context.WebBackPlateCreate.headPlane);
                    Weld weld2 = m_context.Comp.CreatePolyWeld(m_context.SubPart, m_context.WebBackPlateCreate.subParts[0], new Vec3[] { v3, v4 }, true);
                    m_weldProps[2].ApplyToWeld(weld2);
                    weld2.Flush();
                }
            }
        }

        /// <summary>
        /// 设置焊接准备中三角形切割的参数
        /// </summary>
        private void TriCutParameters(WeldProperties weldProps, ref double horiSideUp, ref double vertSideUp, ref double horiSideDown, ref double vertSideDown) 
        {
            Define.WeldType typeUp = weldProps.TypeUp;
            Define.WeldType typeDown = weldProps.TypeDown;
            Define.WeldContour contourUp = weldProps.ContourUp;
            Define.WeldContour contourDown = weldProps.ContourDown;
            //有效判断
            bool upVaild = ((typeUp != Define.WeldType.WELD_TYPE_FILLET && typeUp != Define.WeldType.WELD_TYPE_NONE) || contourUp != Define.WeldContour.WELD_CONTOUR_NONE);
            bool downVaild = ((typeDown != Define.WeldType.WELD_TYPE_FILLET && typeDown != Define.WeldType.WELD_TYPE_NONE) || contourDown != Define.WeldContour.WELD_CONTOUR_NONE);
            if (!upVaild)
                vertSideUp = 0;
            if (!downVaild)
                vertSideDown = 0;
            if(upVaild && typeDown == Define.WeldType.WELD_TYPE_NONE && contourDown == Define.WeldContour.WELD_CONTOUR_NONE)
                vertSideUp *= 2;
            if (!upVaild && downVaild && weldProps.SizeDown != double.MaxValue)
                vertSideDown *= 2;
            //尺寸处理
            if (weldProps.SizeUp != double.MaxValue)
                vertSideUp = weldProps.SizeUp;
            else
                weldProps.SizeUp = 6;
            if (weldProps.SizeDown != double.MaxValue)
                vertSideDown = weldProps.SizeDown;
            else
                weldProps.SizeDown = 6;
            //角度处理
            if (weldProps.GrooveAngleUp > 0 && weldProps.GrooveAngleUp < 90)
                horiSideUp = Math.Tan(weldProps.GrooveAngleUp * Math.PI / 180) * vertSideUp;
            else
                horiSideUp = vertSideUp;
            if (weldProps.GrooveAngleDown > 0 && weldProps.GrooveAngleDown < 90)
                horiSideDown = Math.Tan(weldProps.GrooveAngleDown * Math.PI / 180) * vertSideDown;
            else
                horiSideDown = vertSideDown;
        }

        /// <summary>
        /// 创建三角形的切割
        /// </summary>
        private void CreateTriCut(Plane sidePlane1, Plane sidePlane2, PartBase part, double side1, double side2, Plane triCutCtrlPlane, double cutLength)
        {
            if (side1 == 0 || side2 == 0)
                return;
            Vec3 tempVec = sidePlane1.Normal.Cross(sidePlane2.Normal);
            Vec3 acute1MoveVector = tempVec.Cross(sidePlane2.Normal).Normalize();
            Vec3 acute2MoveVector = sidePlane1.Normal.Cross(tempVec).Normalize();

            Vec3 rightPoint = triCutCtrlPlane.Intersect(sidePlane1, sidePlane2);
            Vec3 acutePoint1 = rightPoint + acute1MoveVector * side1; //side是直角三角形的直角边长
            Vec3 acutePoint2 = rightPoint + acute2MoveVector * side2;

            Plane sidePlane3 = new Plane(acutePoint1, acutePoint2 - acutePoint1, triCutCtrlPlane.Normal);

            PartBase cutPartBase = m_context.Comp.CreatePlateCutter(part, triCutCtrlPlane, triCutCtrlPlane.Move(cutLength).Invert(), new Plane[] { sidePlane1, sidePlane2, sidePlane3 }, true);
            cutPartBase.Flush();
        }

        /// <summary>
        /// 对梁进行圆角（打孔）
        /// </summary>
        public void CommonCreateHole(Plane top, Plane head, Plane ctrlPlane, double radius, PartBase part)
        {
            Plane topPlane = top.Move(radius);
            Plane bottomPlane = top.Invert();
            Plane headPlane = head;
            Plane tailPlane = head.Move(-radius).Invert();

            Vec3 v1 = ctrlPlane.Intersect(headPlane, bottomPlane);
            Vec3 v2 = ctrlPlane.Intersect(headPlane, topPlane);
            Vec3 v3 = ctrlPlane.Intersect(tailPlane, topPlane);
            Vec3 v4 = ctrlPlane.Intersect(tailPlane, bottomPlane);

            SteelPlate sp = m_context.Comp.CreateSteelPlate(new Vec3[] { v1, v2, v3, v4 }, 100, Define.ObjectSide.SIDE_MIDDLE, true);
            sp.CutTarget = part;

            ChamferCtrlPoint chamferCtrlPoint = sp.CtrlPoints[2] as ChamferCtrlPoint;

            chamferCtrlPoint.CutR = radius;
            chamferCtrlPoint.CutType = Define.CornerCutType.CORNER_CUT_ROUND;

            chamferCtrlPoint.Flush();
            sp.Flush();
        }
        
        /// <summary>
        /// 通用的创建翼缘焊缝方法，需传入焊缝所在翼缘面
        /// </summary>
        private void CommonCreateFlangeWeld(Plane p, WeldProperties weldProperties, PartBase[] partBases)
        {
            Vec3 v1 = m_context.MainFront.Intersect(m_context.MainNearCenterPlane, p);
            Vec3 v2 = m_context.MainBack.Intersect(m_context.MainNearCenterPlane, p);
            Weld weld = m_context.Comp.CreatePolyWeld(partBases[0], partBases[1], new Vec3[] { v1, v2 }, true);
            weldProperties.ApplyToWeld(weld);
            weld.Flush();
        }

        /// <summary>
        /// 通用的创建腹板焊缝方法
        /// </summary>
        private void CommonCreateWebWeld(Plane p, PartBase[] partBases)
        {
            //焊缝从远离翼缘的那个孔边缘开始
            double r = r1 > r2 ? r1 : r2; 

            Vec3 v1 = m_context.MainTopFlangeInside.Move(r).Intersect(m_context.MainNearCenterPlane, m_context.MainFrontWeb);
            Vec3 v2 = m_context.MainBottomFlangeInside.Move(r).Intersect(m_context.MainNearCenterPlane, m_context.MainFrontWeb);
            Weld curWeld = m_context.Comp.CreatePolyWeld(partBases[0], partBases[1], new Vec3[] { v1, v2 }, true);
            curWeld.AutoGrooveUp = Define.WeldAutoGroove.WELD_AUTO_GROOVE_TRUE;
            m_weldProps[1].ApplyToWeld(curWeld);
            curWeld.Flush();
        }

        //创建衬垫板并焊接
        private void CreateWeldBacking()
        {

            Plane zCtrlPlaneTop = null, zCtrlPlaneBottom = null; //上（下）翼缘的衬垫板创建面
            double[] locationTop = null, locationBottom = null; //上（下）翼缘的主次梁左右边缘位置坐标

            /*
             * 衬垫板参数获取
             */
            DObject platePropsD = m_paramsHandler.GetDObjectValue(ParamNames.WELD_BACKING_PROPS);
            PlateProperties plateProps = new PlateProperties();
            plateProps.SetProps(platePropsD);
            int weldBackingPlaceTop = m_paramsHandler.GetIntValue(ParamNames.WELD_BACKING_PLACE_TOP);
            int weldBackingPlaceBottom = m_paramsHandler.GetIntValue(ParamNames.WELD_BACKING_PLACE_BOTTOM);
            switch (weldBackingPlaceTop)
            {
                case 2: //内
                    zCtrlPlaneTop = m_context.MainTopFlangeInside;
                    locationTop = topInsideLocation;
                    isCreateTopFlangeWeld = true;
                    break;
                case 3: //外
                    zCtrlPlaneTop = m_context.MainTopFlange;
                    locationTop = topOutsideLocation;
                    isCreateTopFlangeWeld = true;
                    break;
                default: //不创建
                    isCreateTopFlangeWeld = false;
                    break;
            }
            switch (weldBackingPlaceBottom)
            {
                case 2: //内
                    zCtrlPlaneBottom = m_context.MainBottomFlangeInside;
                    locationBottom = bottomInsideLocation;
                    isCreateBottomFlangeWeld = true;
                    break;
                case 3: //外
                    zCtrlPlaneBottom = m_context.MainBottomFlange;
                    locationBottom = bottomOutsideLocation;
                    isCreateBottomFlangeWeld = true;
                    break;
                default: 
                    isCreateBottomFlangeWeld = false;
                    break;
            }

            double thickness = plateProps.Thickness;
            double width = plateProps.Width; //这里的宽度朝向和连接板一样，都是朝着梁方向的
            double height = m_paramsHandler.GetDoubleValue(ParamNames.WELD_BACKING_H); //这里一般有  width < height


            if (height == double.MaxValue)
                height = m_context.m_main_b1;
            else
            {
                int h_mode = m_paramsHandler.GetIntValue(ParamNames.WELD_BACKING_H_MODE);
                switch (h_mode)
                {
                    case 1: // 直接设置height
                        break;
                    case 2: // 设置衬垫板距外侧距离
                        height = m_context.m_main_b1 + 2 * height;
                        break;
                    default:
                        break;
                }
            }

            double centerOffset = m_paramsHandler.GetDoubleValue(ParamNames.WELD_BACKING_CENTER_OFFSET);

            Plane frontPlane = m_context.MainWebCenter.Move(height / 2);
            Plane backPlane = m_context.MainWebCenter.Move(-height / 2).Invert();
            Plane headPlane = m_context.MainNearCenterPlane.Move(width / 2 + m_context.gap / 2 + centerOffset); //远离主梁的面
            Plane tailPlane = headPlane.Move(-width).Invert();


            /*
             * 衬垫板焊缝参数获取(是否创建主梁/次梁与衬垫板的焊缝)
             */
            bool isCreateMainBackingWeld = false, isCreateSubBackingWeld = false;
            int weldBackingMode = m_paramsHandler.GetIntValue(ParamNames.WELD_READY_BACKING);
            switch (weldBackingMode)
            {
                case 1:
                    isCreateMainBackingWeld = true;
                    break;
                case 2:
                    isCreateSubBackingWeld = true;
                    break;
                default:
                    break;
            }

            //比较衬垫板与梁前面哪个更靠近中心，取靠近中心的来计算焊缝
            Plane mainWeldFrontPlane = frontPlane.SignedDist(m_context.MainFront.Origin) < 0 ? m_context.MainFront : frontPlane;
            Plane subWeldFrontPlane = frontPlane.SignedDist(m_context.SubFront.Origin) < 0 ? m_context.SubFront : frontPlane;
            Plane mainWeldBackPlane = backPlane.SignedDist(m_context.MainBack.Origin) < 0 ? m_context.MainBack : backPlane;
            Plane subWeldBackPlane = backPlane.SignedDist(m_context.SubBack.Origin) < 0 ? m_context.SubBack : backPlane;
            Plane centerPlane = m_context.MainNearCenterPlane.Move(m_context.Gap / 2); //两个梁中心的面（法向量方向指向次梁）
            double backingLeftLocation = centerOffset - height / 2;
            double backingRightLocation = centerOffset + height / 2;

            if (isCreateTopFlangeWeld) //上翼缘衬垫板
            {
                //创建衬垫板
                SteelPlate sp = m_context.Comp.CreateSteelPlate(zCtrlPlaneTop, thickness, new Plane[] { frontPlane, headPlane, backPlane, tailPlane }, true);
                //m_context.ApplyPlateParameters(sp, plateProps);
                plateProps.ApplyToPart(sp);
                sp.Flush();

                //创建焊缝，需要比较衬垫板边缘和主次梁边缘的相对位置
                if (isCreateMainBackingWeld && locationTop[0] > backingLeftLocation)  //主梁和衬垫板的焊缝
                {
                    //前侧
                    {
                        Vec3 v1 = centerPlane.Move(locationTop[0]).Intersect(zCtrlPlaneTop, mainWeldFrontPlane);
                        Vec3 v2 = tailPlane.Intersect(zCtrlPlaneTop, mainWeldFrontPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                    //后侧
                    {
                        Vec3 v1 = centerPlane.Move(locationTop[0]).Intersect(zCtrlPlaneTop, mainWeldBackPlane);
                        Vec3 v2 = tailPlane.Intersect(zCtrlPlaneTop, mainWeldBackPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                }
                if (isCreateSubBackingWeld && backingRightLocation > locationTop[1]) //次梁和衬垫板的焊缝
                {
                    {
                        Vec3 v1 = centerPlane.Move(locationTop[1]).Intersect(zCtrlPlaneTop, subWeldFrontPlane);
                        Vec3 v2 = headPlane.Intersect(zCtrlPlaneTop, subWeldFrontPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                    {
                        Vec3 v1 = centerPlane.Move(locationTop[1]).Intersect(zCtrlPlaneTop, subWeldBackPlane);
                        Vec3 v2 = headPlane.Intersect(zCtrlPlaneTop, subWeldBackPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                }
            }

            if (isCreateBottomFlangeWeld) //下翼缘衬垫板
            {
                //创建衬垫板
                SteelPlate sp = m_context.Comp.CreateSteelPlate(zCtrlPlaneBottom, thickness, new Plane[] { frontPlane, headPlane, backPlane, tailPlane }, true);
                //m_context.ApplyPlateParameters(sp, plateProps);
                plateProps.ApplyToPart(sp);
                sp.Flush();

                if (isCreateMainBackingWeld && locationBottom[0] > backingLeftLocation)  //主梁和衬垫板的焊缝
                {
                    //前侧
                    {
                        Vec3 v1 = centerPlane.Move(locationBottom[0]).Intersect(zCtrlPlaneBottom, mainWeldFrontPlane);
                        Vec3 v2 = tailPlane.Intersect(zCtrlPlaneBottom, mainWeldFrontPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                    //后侧
                    {
                        Vec3 v1 = centerPlane.Move(locationBottom[0]).Intersect(zCtrlPlaneBottom, mainWeldBackPlane);
                        Vec3 v2 = tailPlane.Intersect(zCtrlPlaneBottom, mainWeldBackPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                }
                if (isCreateSubBackingWeld && backingRightLocation > locationBottom[1]) //次梁和衬垫板的焊缝
                {
                    {
                        Vec3 v1 = centerPlane.Move(locationBottom[1]).Intersect(zCtrlPlaneBottom, subWeldFrontPlane);
                        Vec3 v2 = headPlane.Intersect(zCtrlPlaneBottom, subWeldFrontPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                    {
                        Vec3 v1 = centerPlane.Move(locationBottom[1]).Intersect(zCtrlPlaneBottom, subWeldBackPlane);
                        Vec3 v2 = headPlane.Intersect(zCtrlPlaneBottom, subWeldBackPlane);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, sp, new Vec3[] { v1, v2 }, true);
                        weld.Flush();
                    }
                }
            }
        }

        //创建楼层板的同时创建焊缝
        public void CreateFloorPlates()
        {

            //0,1,2,3,4 分别是 不创建、不创建、都创建、创建后面、创建前面
            int placeMode = m_paramsHandler.GetIntValue(ParamNames.FLOOR_PLATE_PLACE);
            if (placeMode == 0 || placeMode == 1)
                return;

            DObject platePropsD = m_paramsHandler.GetDObjectValue(ParamNames.FLOOR_PLATE_PROPS);
            PlateProperties plateProps = new PlateProperties();
            plateProps.SetProps(platePropsD);

            double mainMargin = m_paramsHandler.GetDoubleValue(ParamNames.FLOOR_PLATE_MAIN_CENTER_MARGIN);
            double mainOffset = m_paramsHandler.GetDoubleValue(ParamNames.FLOOR_PLATE_MAIN_SIDE_OFFSET);
            double subMargin = m_paramsHandler.GetDoubleValue(ParamNames.FLOOR_PLATE_SUB_CENTER_MARGIN);
            double subOffset = m_paramsHandler.GetDoubleValue(ParamNames.FLOOR_PLATE_SUB_SIDE_OFFSET);

            Vec3 floorPlateOrient = m_context.MainTopFlange.Normal.Cross(m_context.MainNearCenterPlane.Normal);

            Plane[] zCtrlPlanes = { m_context.MainTopFlange.Invert(), m_context.SubTopFlange.Invert() };
            Plane[] frontPlanes = { m_context.MainFront, m_context.MainBack, m_context.SubFront, m_context.SubBack };
            if (floorPlateOrient.Dot(m_context.MainFront.Normal) < 0)
                frontPlanes = new Plane[] { m_context.MainBack, m_context.MainFront, m_context.SubBack, m_context.SubFront };
            Plane[] headPlanes = { m_context.MainNearCenterPlane, m_context.SubNearCenterPlane };
            Plane[] tailPlanes = { m_context.TopTopPlateCreate.tailPlane, m_context.TopTopPlateCreate.headPlane };

            PartBase[] weldBeams = {m_context.MainPart, m_context.SubPart };

            double height = plateProps.Width; //这里任然以梁方向为width方向，与输入不同
            double thickness = plateProps.Thickness;
            double[] margins = { mainMargin, subMargin };
            double[] offsets = { mainOffset, subOffset };

            for (int i = 0; i < 4; i++)
            {
                //创建选项
                if (placeMode == 3 && i % 2 == 1)
                    continue;
                if (placeMode == 4 && i % 2 == 0)
                    continue;

                Plane zCtralPlane = zCtrlPlanes[i / 2];
                Plane frontPlane = frontPlanes[i];
                Plane backPlane = frontPlane.Move(height).Invert();
                Plane headPlane = headPlanes[i / 2].Move(-margins[i / 2]);
                Plane tailPlane = tailPlanes[i / 2].Move(offsets[i / 2]);

                //创建楼层板
                SteelPlate steelPlate = m_context.Comp.CreateSteelPlate(zCtralPlane, thickness, new Plane[] { frontPlane, headPlane, backPlane, tailPlane }, true);
                //m_context.ApplyPlateParameters(steelPlate, plateProps);
                plateProps.ApplyToPart(steelPlate);
                steelPlate.Flush();

                //创建焊缝
                Vec3 v1 = headPlane.Intersect(zCtralPlane, frontPlane);
                Vec3 v2 = tailPlane.Intersect(zCtralPlane, frontPlane);
                Weld weld = m_context.Comp.CreatePolyWeld(weldBeams[i / 2], steelPlate, new Vec3[] { v1, v2 }, true);
                weld.AutoGrooveUp = Define.WeldAutoGroove.WELD_AUTO_GROOVE_TRUE;
                if (m_weldProps[4].SizeUp == double.MaxValue)
                    m_weldProps[4].SizeUp = 6;
                if (m_weldProps[4].SizeDown == double.MaxValue)
                    m_weldProps[4].SizeDown = 6;
                m_weldProps[4].ApplyToWeld(weld);

                steelPlate.Flush();
                weldBeams[i / 2].Flush();
                weld.Flush();
            }

        }

        

    }
}
