﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System;
using System.Collections.Generic;

namespace StiffenedShearPlate17
{
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private ConnectionComponent m_comp = null;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine = Vec3.ZERO;

        private BeamPlanes m_mainPlanes = null;

        private Plane m_mainTopPlane = null;

        private Plane m_mainBomPlane = null;

        private Beam m_subPart = null;

        private Vec3 m_subLine = Vec3.ZERO;

        private BeamPlanes m_subPlanes = null;

        private Plane m_connectRef = null;

        private Plane m_stiffRef = null;

        private Plane m_subFrontPlane = null;

        private Plane m_subBackPlane = null;
        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Plane MainTopPlane { get => m_mainTopPlane; set => m_mainTopPlane = value; }
        public Plane MainBomPlane { get => m_mainBomPlane; set => m_mainBomPlane = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 SubLine { get => m_subLine; set => m_subLine = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane ConnectRef { get => m_connectRef; set => m_connectRef = value; }
        public Plane StiffRef { get => m_stiffRef; set => m_stiffRef = value; }
        public Plane SubFrontPlane { get => m_subFrontPlane; set => m_subFrontPlane = value; }
        public Plane SubBackPlane { get => m_subBackPlane; set => m_subBackPlane = value; }
        public Plane SubAlignPlane { get; set; }
        public SteelPlate Connect { get; set; }
        public SteelBeam UpSplint { get; set; }
        public SteelBeam DownSplint { get; set; }
        public SteelPlate Stiff { get; set; }
        public List<PartBase> Pads { get; set; }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (comp.Subs == null)
                return false;
            Beam subPart = comp.Subs.First() as Beam;
            if (mainPart == null || subPart == null)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);
            m_subPart = subPart;
            m_subPlanes = BeamPlanes.CreateFromBeam(subPart);

            var dir = comp.LocalMatrix.Z;
            //如果不是腹板对腹板的情况  不创建组件
            if (Math.Abs(dir.Dot(MainPlanes.TopBound.Normal.Normalize())) < 1e-8
                || Math.Abs(m_mainLine.Dot(SubPlanes.FrontBound.Normal.Normalize())) < 1e-8)
                return false;
            if (m_subPlanes.TopFlange == null || m_subPlanes.BottomFlange == null)
                return false;
            //调整次零件构建方向为由主到次
            if (!AdjustSubLine(subPart))
                return false;
            if (!GetMainWebPlane())
                return false;
            if (!GetMainFlangePlane())
                return false;
            if (!GetPlateRefPlane())
                return false;
            
            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;
            m_mainLine = line.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 调整次零件的构建方向
        /// </summary>
        private bool AdjustSubLine(Beam subPart)
        {
            var point1 = subPart.CtrlPoints?.Last()?.WorldPos;
            var point2 = subPart.CtrlPoints?.First()?.WorldPos;
            var dir = point1 - point2;
            if (!dir.HasValue)
                return false;
            Vec3 origin = m_mainPart.CtrlPoints.First().WorldPos;
            var dir1 = point1.Value - origin;
            var dir2 = point2.Value - origin;
            double dist1 = dir1.Cross(m_mainLine).Length;
            double dist2 = dir2.Cross(m_mainLine).Length;
            if (dist1 > dist2)
                m_subLine = dir.Value.Normalize();
            else
                m_subLine = - dir.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 基于腹板的前后得到垫板的参照平面
        /// </summary>
        private bool GetPlateRefPlane()
        {
            if(m_subPlanes.FrontWeb != null && m_subPlanes.FrontWebInside != null)
            {
                m_subFrontPlane = m_subPlanes.FrontWeb;
                m_subBackPlane = m_subPlanes.FrontWebInside;
            }else if(m_subPlanes.BackWeb != null && m_subPlanes.BackWebInside != null)
            {
                m_subFrontPlane = m_subPlanes.BackWebInside;
                m_subBackPlane = m_subPlanes.BackWeb;
            }
            else
            {
                //此时为错误情况
                var dist = m_subPlanes.FrontBound.Dist(m_subPlanes.BackBound.Origin);
                var plane = m_subPlanes.FrontBound.Move(- dist / 2 + 2.5);
                m_subFrontPlane = plane;
                m_subBackPlane = plane.Invert().Move(5.0);
            }

            return true;
        }
        /// <summary>
        /// 基于次零件的构建方向得到连接板和加劲肋的参照平面
        /// </summary>
        private bool GetMainWebPlane()
        {
            Plane refPlane1, refPlane2;
            if (m_mainPlanes.FrontWeb != null && m_mainPlanes.FrontWebInside != null)
            {
                refPlane1 = m_mainPlanes.FrontWeb;
                refPlane2 = m_mainPlanes.FrontWebInside;
            }
            else if (m_mainPlanes.BackWeb != null && m_mainPlanes.BackWebInside != null)
            {
                refPlane1 = m_mainPlanes.BackWebInside;
                refPlane2 = m_mainPlanes.BackWeb;
            }
            else
                return false;
            var dir = refPlane1.Normal.Normalize();
            if(dir.Dot(m_subLine) > 0)
            {
                m_connectRef = refPlane1;
                m_stiffRef = refPlane2;
            }
            else
            {
                m_connectRef = refPlane2;
                m_stiffRef = refPlane1;
            }
            return true;
        }
        /// <summary>
        /// 基于冒尖指向确定主零件的前后翼缘面
        /// </summary>
        private bool GetMainFlangePlane()
        {
            Plane refPlane1, refPlane2;
            if (m_mainPlanes.TopFlangeInside == null
                && m_mainPlanes.BottomFlangeInside == null)
                return false;
            if (m_mainPlanes.TopFlangeInside != null)
                refPlane1 = m_mainPlanes.TopFlangeInside;
            else
                refPlane1 = m_mainPlanes.TopBound;

            if (m_mainPlanes.BottomFlangeInside != null)
                refPlane2 = m_mainPlanes.BottomFlangeInside;
            else
                refPlane2 = m_mainPlanes.BottomBound;
            var dir = m_comp.LocalMatrix.Z;
            if(dir.Dot(refPlane1.Normal.Normalize()) > 0)
            {
                m_mainTopPlane = refPlane2;
                m_mainBomPlane = refPlane1;
            }
            else
            {
                m_mainTopPlane = refPlane1;
                m_mainBomPlane = refPlane2;
            }
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
            m_subPart = null;
            m_subPlanes = null;
            m_connectRef = null;
            m_mainBomPlane = null;
            m_mainTopPlane = null;
            m_stiffRef = null;
            m_subFrontPlane = null;
            m_subBackPlane = null;
        }
    }
}
