﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace StiffenedShearPlate17
{
    public class PlateHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();
        private double conBoltLen = 0.0; //连接板上螺栓面板长度
        private double subBoltLen = 0.0; //次零件上螺栓面板长度
        private Plane m_subFrontPlane = null;
        private Plane m_subAlignPlane = null;
        private Vec3 m_subLine = Vec3.ZERO;
        private Vec3 m_dir = Vec3.ZERO; //帽尖指向
        private BeamPlanes m_mainPlanes = null;
        private BeamPlanes m_subPlanes = null;

        private double connectWidth = 0.0; //连接板宽度
        private double webThickness = 0.0; //次零件腹板厚度
        Plane subUpPlane = null;
        Plane subDownPlane = null;
        private SteelPlate connectPlate = null;
        private SteelBeam upSplint = null;
        private SteelBeam downSplint = null;
        private Vec3 pos0 = Vec3.ZERO;
        private Vec3 pos1 = Vec3.ZERO;
        private List<PartBase> parts = new List<PartBase>();

        //图形页参数
        private double m_splintWebGap = double.MaxValue;
        private double m_connectGap = double.MaxValue;
        private bool posEnabled = false;

        //板页参数
        private PlateProperties m_splintProps = new PlateProperties();
        private PlateProperties m_connectProps = new PlateProperties();
        private PlateProperties m_padProps = new PlateProperties();
        private int m_padPos = 0;
        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;
            //双夹板
            if (!CreateSplintPlate())
                return false;
            //连接板
            if (!CreateConnectPlate())
                return false;
            if (upSplint == null || downSplint == null || connectPlate == null)
                return false;
            //垫板
            CreatePadPlate();

            m_context.Connect = connectPlate;
            m_context.UpSplint = upSplint;
            m_context.DownSplint = downSplint;
            m_context.Pads = parts;
            //创建螺栓组
            CreateSubBolt();
            CreatePlateBolt();
            return true;
        }
        private bool ParseProps()
        {
            DObject connectProps = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_CONNECT_PROPS);
            if (connectProps == null)
                return false;
            m_connectProps.SetProps(connectProps);
            DObject splitProps = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_SPLINT_PROPS);
            if (splitProps == null)
                return false;
            m_splintProps.SetProps(splitProps);
            DObject padProps = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_PAD_PROPS);
            if (padProps == null)
                return false;
            m_padProps.SetProps(padProps);

            //得到螺栓属性
            BoltParse boltParse = new BoltParse(m_paramsHandler);
            m_boltData = boltParse.GetBoltParams();
            if (m_boltData == null)
                return false;
            conBoltLen = SumArray(m_boltData.ConnectHoriStride)
                + m_boltData.ConnectLeftGap + m_boltData.ConnectRightGap;
            subBoltLen = SumArray(m_boltData.SplintHoriStride)
                + m_boltData.SplintLeftGap + m_boltData.SplintRightGap;

            //初始化上下文参数
            m_subFrontPlane = m_context.SubFrontPlane;
            m_subLine = m_context.SubLine;
            m_dir = m_context.TargetComp.LocalMatrix.Z.Normalize();
            m_mainPlanes = m_context.MainPlanes;
            m_subPlanes = m_context.SubPlanes;

            //初始化连接板厚度
            webThickness = PlateTools.GetPlaneDist(m_context.SubBackPlane, m_subFrontPlane);
            if (m_connectProps.Thickness == double.MaxValue)
                m_connectProps.Thickness = webThickness;

            m_splintWebGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SPLINT_WEB_GAP);
            m_padPos = m_paramsHandler.GetIntValue(ParamNames.PLATE_PAD_POSITION);
            if (m_connectProps.Thickness != webThickness && m_padPos == 1)
                posEnabled = true;
            //连接板与次零件对齐面间距
            m_connectGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_CONNECT_SECONDARY_GAP);
            AutoConnectWidth(); //得到连接板宽度
            return true;
        }
        /// <summary>
        /// 自适应连接板宽度
        /// </summary>
        private void AutoConnectWidth()
        {
            //连接板到主零件边界面间距
            var connectBorderGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_CONNECT_BORDER_GAP);
            var connectRef = m_context.ConnectRef;
            //确定连接板宽度
            if (m_connectProps.Width != double.MaxValue)
            {
                connectWidth = m_connectProps.Width;
            }
            else if (connectBorderGap != double.MaxValue)
            {
                var plane = connectRef.Normal.Dot(m_mainPlanes.FrontBound.Normal) > 0
                    ? m_mainPlanes.FrontBound : m_mainPlanes.BackBound;
                var gap = PlateTools.GetPlaneDist(plane, connectRef);
                //与包围面间距不能大于主零件腹板与包围面的距离
                if (connectBorderGap >= gap)
                    connectWidth = 0.0;
                else
                    connectWidth = Math.Max(conBoltLen + m_splintWebGap, gap - connectBorderGap); 
            }
            else
            {
                connectWidth = conBoltLen + m_splintWebGap;
            }
            //切割次零件
            m_subAlignPlane = connectRef.Move(connectWidth + m_connectGap);
            m_context.TargetComp.CreateAlignPlane(m_context.SubPart, m_subAlignPlane, true);
            m_context.SubAlignPlane = m_subAlignPlane;
        }
        /// <summary>
        /// 创建连接板
        /// </summary>
        private bool CreateConnectPlate()
        {
            if (m_connectProps.Thickness <= 0.0 ||
                m_connectProps.Width <= 0.0 ||
                m_connectProps.Height <= 0.0)
                return false;
            var plateT = m_connectProps.Thickness;
            var connectRef = m_context.ConnectRef;
            Plane WebPlane = AdjustRefPlane(connectRef, m_context.MainTopPlane);

            //创建矩形板
            var ctrlPoints = new Vec3[4];
            if (posEnabled)
            {
                ctrlPoints[0] = WebPlane.Intersect(connectRef, m_context.MainTopPlane);
            }
            else
            {
                ctrlPoints[0] = WebPlane.Move(plateT / 2 - webThickness / 2)
                    .Intersect(connectRef, m_context.MainTopPlane);
            }
            Vec3 Line;
            //确定控制点横向参照方向
            if (Math.Abs(m_subLine.Dot(m_context.MainLine)) < 1e-3)
                Line = connectRef.Normal.Normalize();
            else
                Line = m_subLine;
            if (connectWidth == 0)
                return false;
            ctrlPoints[1] = ctrlPoints[0] + Line * connectWidth;
            //连接板到主零件翼缘间距
            var connectFlangeGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_CONNECT_FLANGE_GAP);
            var flangeDist = PlateTools.GetPlaneDist(m_context.MainTopPlane, m_context.MainBomPlane);
            //连接板类型
            int connectType = m_paramsHandler.GetIntValue(ParamNames.PLATE_CONNECT_TYPE);
            //确定连接板长度
            if (m_connectProps.Height != double.MaxValue)
                ctrlPoints[2] = ctrlPoints[1] - m_dir * m_connectProps.Height;
            else if (connectFlangeGap != double.MaxValue)
            {
                //与下翼缘间距不能大于主零件两翼缘的距离
                if (connectFlangeGap >= flangeDist)
                    return false;
                ctrlPoints[2] = ctrlPoints[1] - m_dir * (flangeDist - connectFlangeGap);
            }
            else if (connectType == 0 || connectType == 2)
            {
                var dist = m_context.MainTopPlane.SignedDist(pos1);
                if (dist < 0)
                    return false;
                dist = dist > flangeDist ? flangeDist : dist;
                ctrlPoints[2] = ctrlPoints[1] - m_dir * dist;
            }
            else
            {
                ctrlPoints[2] = ctrlPoints[1] - m_dir * flangeDist;
            }  
            ctrlPoints[3] = ctrlPoints[2] - Line * connectWidth;

            connectPlate = m_context.TargetComp.CreateSteelPlate(ctrlPoints,
                plateT, Define.ObjectSide.SIDE_LEFT, true);
            if (connectPlate == null)
                return false;
            var Z = WebPlane.Normal.Normalize();
            connectPlate.Class = m_context.TargetComp.CompClass;
            connectPlate.WorkAxis = new Mat43(m_subLine, m_dir, Z, Vec3.ZERO);
            //调整偏移基准则
            var dot = (ctrlPoints[1] - ctrlPoints[0]).Normalize()
                .Cross((ctrlPoints[3] - ctrlPoints[0]).Normalize());
            if (dot.Dot(Z) > 0)
                connectPlate.OffsetSide = Define.ObjectSide.SIDE_BACK;
            m_connectProps.Thickness = plateT;
            m_connectProps.ApplyToPart(connectPlate);
            HandlerChamfer(connectType);
            //纵向与横向切割
            do
            {
                if (connectType != 3)
                    break;
                var vertCut = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CONNECT_VERT_CUT);
                var len = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CONNECT_HORI_REMAIN_CUT);
                double horiCut = 0.0;
                if (len != double.MaxValue)
                    horiCut = connectWidth - len;
                if (horiCut <= 0)
                    break;
                var point = connectPlate.CtrlPoints[2] as ChamferCtrlPoint;
                //折角
                PlateTools.SetChamfer(point, 0, vertCut, horiCut, connectPlate);
            } while (false);
            return true;
        }
        /// <summary>
        /// 创建双夹板
        /// </summary>
        private bool CreateSplintPlate()
        {
            var b = m_splintProps.Thickness;
            if (b <= 0.0)
                return false;
            //创建双夹板
            var h = m_connectGap + conBoltLen + subBoltLen;
            string profile = "PL" + b + "*" + h;

            Plane refPlane; //控制点参照面
            if (posEnabled)
                refPlane = m_subFrontPlane;
            else
                refPlane = m_subFrontPlane.Move(m_connectProps.Thickness / 2 - webThickness / 2);

            //基于组件帽尖指向确定次零件的前后包围面
            if (m_dir.Dot(m_subPlanes.TopBound.Normal.Normalize()) > 0)
            {
                subUpPlane = m_subPlanes.TopBound;
                subDownPlane = m_subPlanes.BottomBound;
            }
            else
            {
                subUpPlane = m_subPlanes.BottomBound;
                subDownPlane = m_subPlanes.TopBound;
            }
            var upGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SPLINT_UP_GAP);
            var downGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SPLINT_DOWN_GAP);
            //确定板延申参照方向
            Vec3 Line;
            if (Math.Abs(m_subLine.Dot(m_context.MainLine)) < 1e-8)
                Line = m_context.ConnectRef.Normal.Normalize();
            else
                Line = m_subLine;
            var point = m_context.ConnectRef.Move(connectWidth)
                .Intersect(m_subFrontPlane, m_context.MainTopPlane);
            var boltRefPlane = new Plane(point, Line);
            boltRefPlane.MoveThis(m_connectGap + m_boltData.SplintLeftGap);
            var subBeamLen = PlateTools.GetPlaneDist(m_subPlanes.TopBound, m_subPlanes.BottomBound);
            if (upGap != double.MaxValue)
            {
                //与上翼缘间距不能大于次零件梁宽的一半
                if (upGap >= subBeamLen / 2)
                    return false;
                pos0 = refPlane.Intersect(subUpPlane,boltRefPlane)
                    - m_dir * upGap;
            }
            else
            {
                pos0 = GetBoltCtrlPoint() + m_dir * m_boltData.UpPlateGap;
            }
            var len = m_boltData.SplintLeftGap + m_connectGap + conBoltLen;
            pos0 += m_subLine * (h / 2 - len);
            if (downGap != double.MaxValue)
            {
                //与下翼缘间距不能大于次零件梁宽的一半
                if (downGap >= subBeamLen / 2)
                    return false;
                pos1 = refPlane.Intersect(boltRefPlane, subDownPlane)
                    + m_dir * downGap 
                    + m_subLine * (h / 2 - len);
            }
            else
                pos1 = pos0 - m_dir * (SumArray(m_boltData.VertStride)
                    + m_boltData.UpPlateGap + m_boltData.DownPlateGap);
            //上侧夹板
            upSplint = m_context.TargetComp.CreateSteelBeam(
                profile, pos0, pos1, true
                );
            if (upSplint == null)
                return false;
            var Z = refPlane.Normal.Normalize();
            upSplint.WorkAxis = new Mat43(m_subLine, m_dir, Z, Vec3.ZERO);
            upSplint.Rotation = 0.0;
            upSplint.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            upSplint.VertOffsetSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            upSplint.Class = m_context.TargetComp.CompClass;
            upSplint.Refresh();
            m_splintProps.ApplyToPart(upSplint);
            //下侧夹板
            var downPos0 = pos0 - Z * m_connectProps.Thickness;
            var downPos1 = pos1 - Z * m_connectProps.Thickness;
            downSplint = m_context.TargetComp.CreateSteelBeam(
                profile, downPos0, downPos1, true
                );
            if (downSplint == null)
                return false;
            downSplint.WorkAxis = new Mat43(m_subLine, m_dir, -Z, Vec3.ZERO);
            downSplint.Rotation = 0.0;
            downSplint.RotationSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            downSplint.VertOffsetSide = LoongCAD.STC.Define.ObjectSide.SIDE_FRONT;
            downSplint.Class = m_context.TargetComp.CompClass;
            downSplint.Refresh();
            m_splintProps.ApplyToPart(downSplint);
            return true;
        }
        /// <summary>
        /// 创建垫板
        /// </summary>
        private bool CreatePadPlate()
        {
            if (m_connectProps.Thickness == webThickness ||
                m_padProps.Thickness == 0.0
                || m_padProps.Width == 0.0)
                return false;
            double h; //垫板宽度
            var padMargin = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_PAD_MARGIN);
            //确定板延申参照方向
            Vec3 Line;
            if (Math.Abs(m_subLine.Dot(m_context.MainLine)) < 1e-8)
                Line = m_context.ConnectRef.Normal.Normalize();
            else
                Line = m_subLine;
            var boltPlane = new Plane(connectPlate.CtrlPoints[1].WorldPos, Line);
            boltPlane.MoveThis(m_connectGap);
            if (m_padProps.Width == double.MaxValue)
            {
                h = subBoltLen;   
            }
            else
            {
                h = m_padProps.Width;
            }
            boltPlane.MoveThis(h / 2 + padMargin);
            var upGap = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_PAD_UP_GAP);
            var downGap = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_PAD_DOWN_GAP);
            //确定板梁厚度
            double t = m_connectProps.Thickness / 2 - webThickness / 2;
            if (m_padPos == 1)
                t = 2 * t; 
            if (m_padProps.Thickness > t)
                m_padProps.Thickness = t;
            string profile = "PL" + m_padProps.Thickness + "*" + h;
            var Z = m_subFrontPlane.Normal.Normalize();
            var pos0Plane = new Plane(pos0, m_dir);
            var pos1Plane = new Plane(pos1, m_dir);
            var subBeamLen = PlateTools.GetPlaneDist(m_subPlanes.TopBound, m_subPlanes.BottomBound);
            if (m_padPos == 1)
            {
                Vec3 p0, p1;
                if (upGap != double.MaxValue)
                {
                    //与上翼缘间距不能大于次零件梁宽的一半
                    if (upGap >= subBeamLen / 2)
                        return false;
                    p0 = m_subFrontPlane.Intersect(boltPlane, subUpPlane)
                        - m_dir * upGap - Z * webThickness;
                }
                else
                    p0 = pos0Plane.Intersect(boltPlane, m_subFrontPlane)
                        - Z * webThickness;

                if (downGap != double.MaxValue)
                {
                    //与下翼缘间距不能大于次零件梁宽的一半
                    if (downGap >= subBeamLen / 2)
                        return false;
                    p1 = m_subFrontPlane.Intersect(boltPlane, subDownPlane)
                        + m_dir * downGap - Z * webThickness;
                }
                else
                    p1 = pos1Plane.Intersect(boltPlane, m_subFrontPlane)
                        - Z * webThickness;
                var pad = m_context.TargetComp.CreateSteelBeam(profile, p0, p1, true);
                if (pad == null)
                    return false;
                pad.WorkAxis = new Mat43(m_subLine, m_dir, -Z, Vec3.ZERO);
                pad.Rotation = 0;
                pad.RotationSide = Define.ObjectSide.SIDE_FRONT;
                pad.VertOffsetSide = Define.ObjectSide.SIDE_FRONT;
                pad.Class = m_context.TargetComp.CompClass;
                pad.Refresh();
                m_padProps.ApplyToPart(pad);
                parts.Add(pad);
                return true;
            }
            Vec3 upPos0, upPos1;  //板梁控制点
            if (upGap != double.MaxValue)
            {
                //与上翼缘间距不能大于次零件梁宽的一半
                if (upGap >= subBeamLen / 2)
                    return false;
                upPos0 = m_subFrontPlane.Intersect(boltPlane, subUpPlane)
                    - m_dir * upGap;
            }
            else
            {
                upPos0 = pos0Plane.Intersect(boltPlane, m_subFrontPlane);
            }

            if (downGap != double.MaxValue)
            {
                //与下翼缘间距不能大于次零件梁宽的一半
                if (downGap >= subBeamLen / 2)
                    return false;
                upPos1 = m_subFrontPlane.Intersect(boltPlane, subDownPlane)
                    + m_dir * downGap;
            }
            else
            {
                upPos1 = pos1Plane.Intersect(boltPlane, m_subFrontPlane);
            }
            //上侧垫板
            var upPad = m_context.TargetComp.CreateSteelBeam(profile, upPos0, upPos1, true);
            if (upPad == null)
                return false;
            upPad.WorkAxis = new Mat43(m_subLine, m_dir, Z, Vec3.ZERO);
            upPad.Rotation = 0;
            upPad.RotationSide = Define.ObjectSide.SIDE_FRONT;
            upPad.VertOffsetSide = Define.ObjectSide.SIDE_FRONT;
            upPad.Class = m_context.TargetComp.CompClass;
            upPad.Refresh();
            m_padProps.ApplyToPart(upPad);
            //下侧垫板
            var downPad = m_context.TargetComp.CreateSteelBeam(profile,
                upPos0 - Z * webThickness, upPos1 - Z * webThickness, true);
            if (downPad == null)
                return false;
            downPad.WorkAxis = new Mat43(m_subLine, m_dir, -Z, Vec3.ZERO);
            downPad.Rotation = 0;
            downPad.RotationSide = Define.ObjectSide.SIDE_FRONT;
            downPad.VertOffsetSide = Define.ObjectSide.SIDE_FRONT;
            downPad.Class = m_context.TargetComp.CompClass;
            downPad.Refresh();
            m_padProps.ApplyToPart(downPad);
            parts.Add(upPad);
            parts.Add(downPad);
            return true;
        }
        /// <summary>
        /// 创建次零件上的螺栓组
        /// </summary>
        private bool CreateSubBolt()
        {
            if (m_boltData.SplintHoriStride == null || m_boltData.VertStride == null)
                return false;

            var boltX = - m_dir;
            Vec3 boltY;
            if (Math.Abs(m_subLine.Dot(m_context.MainLine)) < 1e-8)
            {
                boltY = m_context.ConnectRef.Normal.Normalize();
            }
            else
            {
                boltY = m_subLine;
            }
            var boltZ = boltY.Cross(boltX);
            Vec3 startPos;
            if (m_boltData.VertOffset != double.MaxValue)
                startPos = GetBoltCtrlPoint() + boltY * SumArray(m_boltData.SplintHoriStride) / 2;
            else
            {
                var len = conBoltLen + m_connectGap + subBoltLen; //夹板长度
                startPos = pos0 + boltY * (len / 2 - m_boltData.SplintRightGap
                    - SumArray(m_boltData.SplintHoriStride) / 2)
                    - m_dir * m_boltData.UpPlateGap;
            }
            Vec3 endPos = startPos + boltX * 1000;
            var subs = new List<PartBase>();
            subs.Add(upSplint);
            if (parts != null && parts.Count > 0)
                subs.AddRange(parts);
            subs.Add(downSplint);
            BoltGroup bolts = m_context.TargetComp.CreateBoltGroup(
                m_context.SubPart, subs.ToArray(), startPos, endPos, true);
            if (bolts == null)
                return false;

            bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
            bolts.Rotation = 0;
            m_boltData.BoltsCfg.FillBasicProps(bolts, true);
            m_boltData.BoltsCfg.FillHoleProps(bolts);
            m_boltData.BoltsCfg.FillContentProps(bolts);
            m_boltData.BoltsCfg.FillExtendProps(bolts);
            bolts.ArrayStrideX = m_boltData.VertStride;
            bolts.ArrayStrideY = m_boltData.SplintHoriStride.Reverse().ToArray();
            bolts.HeadOffsetX = 0;
            bolts.VerticalOffset = 0;
            bolts.CutLength = m_connectProps.Thickness * 2 + m_splintProps.Thickness * 2 + webThickness * 2;
            bolts.MadeType = m_boltData.SplintBoltMadeType;
            bolts.Refresh();
            return true;
        }
        /// <summary>
        /// 创建连接板上的螺栓组
        /// </summary>
        private bool CreatePlateBolt()
        {
            if (m_boltData.ConnectHoriStride == null || m_boltData.VertStride == null)
                return false;

            var boltX = - m_dir;
            Vec3 boltY;
            if (Math.Abs(m_subLine.Dot(m_context.MainLine)) < 1e-8)
            {
                boltY = m_context.ConnectRef.Normal.Normalize();
            }
            else
            {
                boltY = m_subLine;
            }
            var boltZ = boltX.Cross(boltY);
            Vec3 startPos;
            if(m_boltData.VertOffset != double.MaxValue)
            {
                startPos = GetBoltCtrlPoint() + boltY * (SumArray(m_boltData.ConnectHoriStride) / 2
                    - conBoltLen - m_connectGap - m_boltData.SplintLeftGap + m_boltData.ConnectLeftGap);
            }
            else
            {
                var len = conBoltLen + m_connectGap + subBoltLen; //夹板长度
                startPos = pos0 + boltY * (SumArray(m_boltData.ConnectHoriStride) / 2
                    + m_boltData.ConnectLeftGap - len / 2)
                     - m_dir * m_boltData.UpPlateGap;
            }
            Vec3 endPos = startPos + boltX * 1000; ;
            BoltGroup bolts = m_context.TargetComp.CreateBoltGroup(
                connectPlate, new PartBase[] { upSplint, downSplint}, startPos, endPos, true);
            if (bolts == null)
                return false;

            bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
            bolts.Rotation = 0;
            m_boltData.BoltsCfg.FillBasicProps(bolts, true);
            m_boltData.BoltsCfg.FillHoleProps(bolts);
            m_boltData.BoltsCfg.FillContentProps(bolts);
            m_boltData.BoltsCfg.FillExtendProps(bolts);
            bolts.ArrayStrideX = m_boltData.VertStride;
            bolts.ArrayStrideY = m_boltData.ConnectHoriStride;
            bolts.HeadOffsetX = 0;
            bolts.VerticalOffset = 0;
            bolts.CutLength = m_connectProps.Thickness * 2 + m_splintProps.Thickness * 2 + webThickness * 2;
            bolts.MadeType = m_boltData.ConnectBoltMadeType;
            bolts.Refresh();
            return true;
        }
        private Vec3 GetBoltCtrlPoint()
        {
            Plane refPlane; //控制点参照面
            if (posEnabled)
                refPlane = m_subFrontPlane;
            else
                refPlane = m_subFrontPlane.Move(m_connectProps.Thickness / 2 - webThickness / 2);
            var boltRefPlane = m_subAlignPlane.Move(m_boltData.SplintLeftGap);
            //基于帽尖指向确定主零件的上下包围面
            Plane mainUpPlane, mainDownPlane;
            if (m_dir.Dot(m_mainPlanes.TopBound.Normal.Normalize()) > 0)
            {
                mainUpPlane = m_mainPlanes.TopBound;
                mainDownPlane = m_mainPlanes.BottomBound;
            }
            else
            {
                mainUpPlane = m_mainPlanes.BottomBound;
                mainDownPlane = m_mainPlanes.TopBound;
            }
            Vec3 pos;
            switch (m_boltData.VertOffsetMode)
            {
                case 3:
                    pos = refPlane.Intersect(mainDownPlane, boltRefPlane)
                        + m_dir * (m_boltData.VertOffset + SumArray(m_boltData.VertStride));
                    break;
                case 2:
                    var subBeamLen = PlateTools.GetPlaneDist(m_subPlanes.TopBound, m_subPlanes.BottomBound);
                    var plane = mainUpPlane.Move(- subBeamLen / 2);
                    pos = refPlane.Intersect(plane, boltRefPlane)
                        + m_dir * (m_boltData.VertOffset + SumArray(m_boltData.VertStride) / 2);
                    break;
                case 0:
                default:
                    pos = refPlane.Intersect(mainUpPlane, boltRefPlane)
                       - m_dir * m_boltData.VertOffset;
                    break;
            }
            return pos;
        }
        /// <summary>
        /// 进行折角操作
        /// </summary>
        private void HandlerChamfer(int connectType)
        {
            if (connectPlate == null)
                return;
            var cutX = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CONNECT_CUT_X);
            var cutY = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CONNECT_CUT_Y);
            if((cutX > 0 && cutY == 0.0) || (cutY > 0 && cutX == 0.0))
            {
                cutX = Math.Max(cutY, cutX);
                cutY = cutX;
            }
            if(connectType == 1)
            {
                var point1 = connectPlate.CtrlPoints[1] as ChamferCtrlPoint;
                var point2 = connectPlate.CtrlPoints[2] as ChamferCtrlPoint;
                PlateTools.SetChamfer(point1, 0, cutX, cutY, connectPlate);
                PlateTools.SetChamfer(point2, 0, cutY, cutX, connectPlate);
            }
            else
            {
                var point = connectPlate.CtrlPoints[1] as ChamferCtrlPoint;
                PlateTools.SetChamfer(point, 0, cutX, cutY, connectPlate);
            }
            var chamferX = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CONNECT_CHAMFER_X);
            var chamferY = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CONNECT_CHAMFER_Y);
            chamferX = AutoChamferLength(chamferX);
            chamferY = AutoChamferLength(chamferY);
            int chamferType = m_paramsHandler.GetIntValue(ParamNames.PLATE_CONNECT_CHAMFER_TYPE);
            if (chamferType == 3 || chamferType == 4)
                chamferY = chamferX;
            if(connectType == 1)
            {
                var chamferPoint1 = connectPlate.CtrlPoints[0] as ChamferCtrlPoint;
                var chamferPoint2 = connectPlate.CtrlPoints[3] as ChamferCtrlPoint;
                PlateTools.SetChamfer(chamferPoint1, chamferType, chamferY, chamferX, connectPlate);
                PlateTools.SetChamfer(chamferPoint2, chamferType, chamferX, chamferY, connectPlate);
            }
            else
            {
                var chamferPoint = connectPlate.CtrlPoints[0] as ChamferCtrlPoint;
                PlateTools.SetChamfer(chamferPoint, chamferType, chamferY, chamferX, connectPlate);
            }
        }
        //根据梁宽自适应折角尺寸
        private double AutoChamferLength(double len)
        {
            double beamH = PlateTools.GetPlaneDist(
                m_mainPlanes.TopBound, m_mainPlanes.BottomBound);
            if (len == double.MaxValue)
                return beamH <= 125 ? 15.0 : 25.0;
            return len;
        }
        /// <summary>
        /// 调整创建连接板的参照面
        /// </summary>
        private Plane AdjustRefPlane(Plane connectRef, Plane upPlane)
        {
            var frontWebPlane = m_context.SubFrontPlane;
            var backWebPlane = m_context.SubBackPlane;
            var point1 = frontWebPlane.Intersect(connectRef, upPlane);
            var point2 = backWebPlane.Intersect(connectRef, upPlane);
            Plane plsSide1 = new Plane(point1, m_subLine);
            Plane plsSide2 = new Plane(point2, m_subLine);
            //次零件
            var pos0 = m_context.SubPart.CtrlPoints.First().WorldPos;
            var pos1 = m_context.SubPart.CtrlPoints.Last().WorldPos;
            Plane plane;
            if (m_subLine.Dot(pos1 - pos0) > 0)
                plane = new Plane(pos1, m_subLine);
            else
                plane = new Plane(pos0, m_subLine);
            var dist1 = plsSide1.Dist(plane.Origin);
            var dist2 = plsSide2.Dist(plane.Origin);
            if (dist1 > dist2)
                return backWebPlane;
            return frontWebPlane;
        }
        private double SumArray(double[] arr)
        {
            if (arr == null || arr.Length == 0)
                return 0.0;
            double sum = 0.0;
            foreach (var item in arr)
                sum += Math.Abs(item);
            return sum;
        }
    }
}
