﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;
using System;
using System.Linq;

namespace StiffenedShearPlate17
{
    public class StiffHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private PlateProperties m_stiffProps = new PlateProperties();
        private Vec3 m_subLine = Vec3.ZERO;
        private Vec3 m_dir = Vec3.ZERO; //帽尖指向

        private SteelPlate stiff = null;
        private int m_stiffEnabled = 0;
        private int m_stiffDir = 0;
        private double m_vertOffset = double.MaxValue;

        public StiffHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;
            if (!CreateStiff())
                return false;
            SetChamfer();
            m_context.Stiff = stiff;
            return true;
        }
        private bool ParseProps()
        {
            DObject stiffProps = m_paramsHandler.GetDObjectValue(ParamNames.STIFF_PROPS);
            if (stiffProps == null)
                return false;
            m_stiffProps.SetProps(stiffProps);

            m_subLine = m_context.SubLine;
            m_dir = m_context.TargetComp.LocalMatrix.Z.Normalize();

            m_stiffEnabled = m_paramsHandler.GetIntValue(ParamNames.STIFF_ENABLED);
            m_stiffDir = m_paramsHandler.GetIntValue(ParamNames.STIFF_DIR);
            m_vertOffset = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_VERT_OFFSET);
            return true;
        }
        private bool CreateStiff()
        {
            if (m_stiffEnabled != 0)
                return false;
            if (m_stiffProps.Thickness == 0 ||
                m_stiffProps.Width == 0 ||
                m_stiffProps.Height == 0)
                return false;
            //初始化参数
            var webThickness = PlateTools.GetPlaneDist(m_context.SubFrontPlane, 
                m_context.SubBackPlane);
            var stiffRef = m_context.StiffRef;
            var upPlane = m_context.MainTopPlane;
            Plane WebPlane = AdjustRefPlane(stiffRef, upPlane);
            var refPlane = WebPlane.Move(m_stiffProps.Thickness / 2
                - webThickness / 2 + m_vertOffset);
            Vec3 axisY;
            var flag = Math.Abs(m_context.MainLine.Dot(m_subLine));
            if (m_stiffDir == 1 || flag < 1e-6)
                axisY = stiffRef.Normal.Normalize();
            else
                axisY = -m_subLine;
            Vec3 axisX = m_dir;
            //创建矩形板
            var ctrlPoints = new Vec3[4];
            double vertDist;
            if (m_stiffProps.Height != double.MaxValue)
            {
                vertDist = m_stiffProps.Height;
                ctrlPoints[0] = refPlane.Intersect(upPlane, stiffRef);
            }
            else
            {
                var flangeGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_STIFF_FLANGE_GAP);
                var flangeDist = PlateTools.GetPlaneDist(upPlane, m_context.MainBomPlane);
                //与下翼缘间距的两倍不能大于主零件两翼缘的距离)
                if (flangeGap >= flangeDist / 2)
                    return false;
                vertDist = flangeDist - 2 * flangeGap;
                ctrlPoints[0] = refPlane.Intersect(upPlane.Move(flangeGap), stiffRef);
            }
            ctrlPoints[1] = ctrlPoints[0] - axisX * vertDist;
            if (m_stiffProps.Width != double.MaxValue)
            {
                ctrlPoints[2] = ctrlPoints[1] + axisY * m_stiffProps.Width;
            }
            else
            {
                var borderGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_STIFF_BORDER_GAP);
                var plane = stiffRef.Normal.Dot(m_context.MainPlanes.FrontBound.Normal) > 0
                    ? m_context.MainPlanes.FrontBound : m_context.MainPlanes.BackBound;
                var borderDist = PlateTools.GetPlaneDist(plane, stiffRef);
                //与包围面间距不能大于主零件腹板与包围面的距离
                if (borderGap >= borderDist)
                    return false;
                ctrlPoints[2] = ctrlPoints[1] + axisY * (borderDist - borderGap);
            }
            ctrlPoints[3] = ctrlPoints[2] + axisX * vertDist;
            var axisZ = axisX.Cross(axisY);
            stiff = m_context.TargetComp.CreateSteelPlate(ctrlPoints, m_stiffProps.Thickness, 
                Define.ObjectSide.SIDE_LEFT, true);
            if (stiff == null)
                return false;
            stiff.WorkAxis = new Mat43(axisX, axisY, axisZ, Vec3.ZERO);
            //调整偏移基准则
            var dot = (ctrlPoints[1] - ctrlPoints[0]).Normalize()
                .Cross((ctrlPoints[3] - ctrlPoints[0]).Normalize());
            if (dot.Dot(refPlane.Normal.Normalize()) > 0)
                stiff.OffsetSide = Define.ObjectSide.SIDE_BACK;
            stiff.Class = m_context.TargetComp.CompClass;
            m_stiffProps.ApplyToPart(stiff);
            return true;
        }
        private void SetChamfer()
        {
            if (stiff == null)
                return;
            int len = stiff.CtrlPoints.Length;
            var points = new ChamferCtrlPoint[len];
            for(int i = 0; i < len; i++)
            {
                var point = stiff.CtrlPoints[i] as ChamferCtrlPoint;
                if (point == null)
                    return;
                points[i] = point;
            }

            {
                //设置内折角
                var type = m_paramsHandler.GetIntValue(ParamNames.STIFF_INSIDE_CHAMFER);
                var cutX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_INSIDE_CHAMFER_X);
                var cutY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_INSIDE_CHAMFER_Y);
                cutX = AutoChamferLength(cutX);
                cutY = AutoChamferLength(cutY);
                if (type == 3 || type == 4)
                    cutY = cutX;
                PlateTools.SetChamfer(points[0], type, cutX, cutY, stiff);
                PlateTools.SetChamfer(points[1], type, cutY, cutX, stiff);
            }

            {
                //设置外折角
                var type = m_paramsHandler.GetIntValue(ParamNames.STIFF_CHAMFER);
                var cutX = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_CHAMFER_X);
                var cutY = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_CHAMFER_Y);
                if (type == 3 || type == 4)
                    cutY = cutX;
                PlateTools.SetChamfer(points[2], type, cutX, cutY, stiff);
                PlateTools.SetChamfer(points[3], type, cutY, cutX, stiff);
            }
        }
        /// <summary>
        /// 调整创建加劲肋的参照面
        /// </summary>
        private Plane AdjustRefPlane(Plane stiffRef, Plane upPlane)
        {
            var frontWebPlane = m_context.SubFrontPlane;
            var backWebPlane = m_context.SubBackPlane;
            var point1 = frontWebPlane.Intersect(stiffRef, upPlane);
            var point2 = backWebPlane.Intersect(stiffRef, upPlane);
            Plane plsSide1 = new Plane(point1, m_subLine);
            Plane plsSide2 = new Plane(point2, m_subLine);
            //次零件
            var pos0 = m_context.SubPart.CtrlPoints.First().WorldPos;
            var pos1 = m_context.SubPart.CtrlPoints.Last().WorldPos;
            Plane plane;
            if (m_subLine.Dot(pos1 - pos0) > 0)
                plane = new Plane(pos1, m_subLine);
            else
                plane = new Plane(pos0, m_subLine);
            var dist1 = plsSide1.Dist(plane.Origin);
            var dist2 = plsSide2.Dist(plane.Origin);
            if (dist1 > dist2)
                return frontWebPlane;
            return backWebPlane;
        }
        //根据梁宽自适应折角尺寸
        private double AutoChamferLength(double len)
        {
            double beamH = PlateTools.GetPlaneDist(
                m_context.MainPlanes.TopBound, m_context.MainPlanes.BottomBound);
            if (len == double.MaxValue)
                return beamH <= 125 ? 15.0 : 25.0;
            return len;
        }
    }
}
