﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System;

namespace StiffenedShearPlate17
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        private SteelPlate m_stiff = null;
        private SteelPlate m_conn = null;
        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }
        public void Execute()
        {
            if (!ParseProps())
                return;
            this.CreateStiffWelds();
            if (!CreateConnectWelds())
                return;

            //调整构件关系
            AdjustMember();
        }
        private bool ParseProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }
        private bool CreateStiffWelds()
        {
            m_stiff = m_context.Stiff;
            //加劲肋
            if (m_stiff == null)
                return false;
            var stiffDir = m_paramsHandler.GetIntValue(ParamNames.STIFF_DIR);
            Vec3 axisX = m_context.TargetComp.LocalMatrix.Z;
            Vec3 axisY;
            if (stiffDir == 1)
                axisY = m_context.StiffRef.Normal.Normalize();
            else
                axisY = - m_context.SubLine;
            Vec3 axisZ = axisX.Cross(axisY);
            var pos = m_stiff.CtrlPoints[0].WorldPos;
            Vec3[] vec3s = new Vec3[] { axisX, axisY, axisZ, pos };
            //焊缝4
            CreateAutoWeld(3, m_stiff, vec3s, LoongCAD.STC.Define.SpaceAxis.AXIS_X);
            //焊缝5
            CreateAutoWeld(4, m_stiff, vec3s, LoongCAD.STC.Define.SpaceAxis.AXIS_NX);
            //焊缝6
            CreateAutoWeld(5, m_stiff, vec3s, LoongCAD.STC.Define.SpaceAxis.AXIS_NY);
            return true;
        }
        private bool CreateConnectWelds()
        {
            m_conn = m_context.Connect;
            if (m_conn == null)
                return false;
            Vec3 axisX = m_context.TargetComp.LocalMatrix.Z;
            Vec3 axisY;
            if (Math.Abs(m_context.SubLine.Dot(m_context.MainLine)) < 1e-6)
                axisY = m_context.ConnectRef.Normal.Normalize();
            else
                axisY = m_context.SubLine;
            Vec3 axisZ = axisX.Cross(axisY);
            var pos = m_conn.CtrlPoints[0].WorldPos;
            Vec3[] vec3s = new Vec3[] { axisX, axisY, axisZ, pos };
            //焊缝1
            CreateAutoWeld(0, m_conn, vec3s, LoongCAD.STC.Define.SpaceAxis.AXIS_X);
            //焊缝2
            CreateAutoWeld(1, m_conn, vec3s, LoongCAD.STC.Define.SpaceAxis.AXIS_NX);
            //焊缝3
            CreateAutoWeld(2, m_conn, vec3s, LoongCAD.STC.Define.SpaceAxis.AXIS_NY);
            return true;
        }
        private bool AdjustMember()
        {
            //垫板构件
            int padMember = m_paramsHandler.GetIntValue(ParamNames.PLATE_PAD_MEMBER);
            var pads = m_context.Pads;
            do
            {
                if (padMember == 1 || pads == null || pads.Count < 1)
                    break;
                foreach(var item in pads)
                {
                    m_context.TargetComp.CreatePolyWeld(m_context.SubPart,
                        item, new Vec3[] { item.CtrlPoints[0].WorldPos }, true);
                }
            } while (false);

            //连接板构件
            int connMember = m_paramsHandler.GetIntValue(ParamNames.PLATE_CONNECT_MEMBER);
            var upSplint = m_context.UpSplint;
            var downSplint = m_context.DownSplint;
            var points = GetWeldPoint(upSplint, downSplint); //连接板与夹板焊接点
            if (points == null || points.Length < 2)
                return false;
            switch (connMember)
            {
                case 2:
                    m_context.TargetComp.CreatePolyWeld(m_conn,
                        upSplint, new Vec3[] { points[0] }, true);
                    m_context.TargetComp.CreatePolyWeld(m_conn,
                        downSplint, new Vec3[] { points[1] }, true);
                    break;
                case 4:
                    m_context.TargetComp.CreatePolyWeld(m_conn,
                        upSplint, new Vec3[] { points[0] }, true);
                    m_context.TargetComp.CreatePolyWeld(m_conn,
                        downSplint, new Vec3[] { points[1] }, true);
                    m_context.TargetComp.CreatePolyWeld(m_context.SubPart,
                        upSplint, new Vec3[] { upSplint.CtrlPoints[0].WorldPos }, true);
                    m_context.TargetComp.CreatePolyWeld(m_context.SubPart,
                        downSplint, new Vec3[] { downSplint.CtrlPoints[0].WorldPos }, true);
                    break;
                case 1:
                case 3:
                default:
                    break;
            }
            return true;
        }
        private void CreateAutoWeld(int index, PartBase sub, Vec3[] pos, LoongCAD.STC.Define.SpaceAxis type)
        {
            if (m_weldProps[index].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                return;
            Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                sub, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
            if (pos.Length < 4)
                return;
            curWeld.WorkAxis = new Mat43(pos[0], pos[1], pos[2], pos[3]);
            curWeld.AutoWeldPosition = type;
            m_weldProps[index].ApplyToWeld(curWeld);
        }
        private Vec3[] GetWeldPoint(PartBase part1, PartBase part2)
        {
            BoltParse bolt = new BoltParse(m_paramsHandler);
            var datas = bolt.GetBoltParams();
            if (datas == null)
                return null;
            var pos = part1.CtrlPoints[0].WorldPos;
            var pos1 = part2.CtrlPoints[0].WorldPos;
            var points = new List<Vec3>();
            var offset = datas.ConnectLeftGap + 
                PlateTools.SumArray(datas.ConnectHoriStride) / 2;
            double gap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_CONNECT_SECONDARY_GAP);
            var plateLen = datas.ConnectLeftGap + datas.ConnectRightGap
                + PlateTools.SumArray(datas.ConnectHoriStride)
                + datas.SplintLeftGap + datas.SplintRightGap
                + PlateTools.SumArray(datas.SplintHoriStride) + gap;
            points.Add(pos + m_context.SubLine * (offset - plateLen / 2));
            points.Add(pos1 + m_context.SubLine * (offset - plateLen / 2));
            return points.ToArray();
        }
    }
}
