﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace StubConnection119
{
    //创建腋板
    public class HaunchHelper
    {
        public bool isValid = true; //腋板是否有效（是否创建）
        public bool isVertDist = true; //折角是否为平行移动（是否按按垂直间距计算）
        public bool isPlaneVert = true; //输入的两个板是否垂直（腋板是否为平行四边形）
        public bool isTrianlge = false; //腋板是否为三角形
        public PlateProperties props = new PlateProperties();
        public SteelPlate haunch;
        public Plane topPlane;
        public Plane bottomPlane;
        public Plane insidePlane;
        public Plane outsidePlane;
        public Plane centerPlane;
        public double thickness = 10; //默认的宽、高、厚度
        public double height = 50;
        public double width = 50;
        public Define.CornerCutType insideChamferType = Define.CornerCutType.CORNER_CUT_NONE;
        public double insideCutX = 0, insideCutY = 0;
        public Define.CornerCutType outsideChamferType = Define.CornerCutType.CORNER_CUT_NONE;
        public double outsideCutX = 0, outsideCutY = 0;
        public double angle;
        public ConnectionComponent comp;

        /// <summary>
        /// 初始化位置，腋板有四个方向的边面
        /// </summary>
        /// <param name="plateSidePlane">内侧的边面（端板一侧），法线由端板指向加劲肋</param>
        /// <param name="beamSidePlane">内侧的边面（梁一侧），法线由梁指向加劲肋</param>
        /// <param name="centerPlane">腋板中心面，用于确定创建面</param>
        public HaunchHelper(Plane plateSidePlane, Plane beamSidePlane, Plane centerPlane, PlateProperties props)
        {
            if (plateSidePlane == null || beamSidePlane == null || centerPlane == null
                || props == null || !Util.PlatePropIsValid(props))
            {
                isValid = false;
                return;
            }
            //
            this.insidePlane = plateSidePlane.Clone();
            this.topPlane = beamSidePlane.Clone();
            this.centerPlane = centerPlane.Clone();
            //
            this.props = props;
            //
            isPlaneVert = topPlane.Normal.IsVerticalTo(insidePlane.Normal);
            angle = Util.VectorAngle(topPlane.Normal, insidePlane.Normal);
        }

        /// <summary>
        /// 是否默认跟随端板边缘（不跟随时，需要设置默认值）
        /// </summary>
        public void SetFollowPlane(Plane endPlateSidePlane)
        {
            if (!isValid)
                return;
            this.bottomPlane = endPlateSidePlane;
            Vec3 checkPoint = topPlane.Intersect(insidePlane, centerPlane);
            if (endPlateSidePlane.SignedDist(checkPoint) >= 0)
                isValid = false;
            Util.SetDefaultValue(ref props.Height, endPlateSidePlane.Dist(checkPoint));
            Util.SetDefaultValue(ref props.Width, endPlateSidePlane.Dist(checkPoint));
        }

        /// <summary>
        /// 设置默认宽、高、厚度
        /// </summary>
        public void SetDefaultSize(double width = 80, double height = 50, double thickness = 10)
        {
            this.thickness = thickness;
            this.height = height;
            this.width = width;
            if (bottomPlane != null)
                this.height = bottomPlane.Dist(centerPlane.Intersect(insidePlane, topPlane));
            Util.SetDefaultValue(ref props.Width, this.width);
            Util.SetDefaultValue(ref props.Height, this.height);
        }

        public void SetInsideChamfer(double cutx, double cuty, Define.CornerCutType chamferType)
        {
            //宽
            if (cutx > props.Width) //如果折角过大就取边长
                cutx = props.Width;
            //高
            if (cuty > props.Height)
                cuty = props.Height;
            //倾斜且选择垂直距离的的情况下重新计算折角大小
            if (!isPlaneVert && isVertDist && insideChamferType == Define.CornerCutType.CORNER_CUT_LINE)
            {
                double angle = Util.VectorAngle(topPlane.Normal, insidePlane.Normal);
                cutx /= Math.Cos(angle);
            }
            this.insideCutX = cutx;
            this.insideCutY = cuty;
            this.insideChamferType = chamferType;
        }

        public void SetOutsideChamfer(double cutx, double cuty, Define.CornerCutType chamferType)
        {
            //宽
            if (cutx > props.Width) //如果折角过大就取边长
                cutx = props.Width;
            //高
            if (cuty > props.Height)
                cuty = props.Height;
            //
            this.outsideCutX = cutx;
            this.outsideCutY = cuty;
            this.outsideChamferType = chamferType;
        }

        public bool CreateHaunch(ConnectionComponent comp)
        {
            this.comp = comp;
            if (insideCutX == props.Width && outsideCutX == props.Width && insideCutY == props.Height && outsideCutY == props.Height)
                isValid = false;
            if (!isValid)
                return false;
            Plane ctrlPlane = centerPlane.Move(-props.Thickness / 2);
            //外侧边面
            outsidePlane = insidePlane.Move(props.Width);
            //下侧边面
            Line line = insidePlane.Intersect(topPlane);
            bottomPlane = Util.VerticalPlane(insidePlane, line, topPlane.Normal);
            bottomPlane.MoveThis(props.Height);
            //如果倾斜了，并且使用斜的距离
            if (!isVertDist && !isPlaneVert)
            {
                Vec3 p = insidePlane.Intersect(bottomPlane, ctrlPlane);
                bottomPlane = new Plane(p, topPlane.Normal);
            }
            //是否为三角形腋板
            Plane[] sidePlanes;
            if (isTrianlge)
            {
                bottomPlane = Util.CreatePlaneBy2Line(insidePlane.Intersect(bottomPlane), outsidePlane.Intersect(topPlane)); // 此时bottom面是斜着的
                sidePlanes = new Plane[] { topPlane, bottomPlane, insidePlane};
            }
            else
            {
                sidePlanes = new Plane[]{ topPlane, outsidePlane, bottomPlane, insidePlane };
            }
            //创建
            haunch = comp.CreateSteelPlate(ctrlPlane, props.Thickness, sidePlanes, true);
            if (haunch == null)
                return false;
            //折角
            ChamferControl chamferControl = new ChamferControl(haunch, sidePlanes);
            if (!isTrianlge) //三角形腋板不需要对外侧折角
                chamferControl.ChamferOnePoint(outsideCutY, outsideCutX, outsideChamferType, chamferControl.comparePointList[2]);
            chamferControl.ChamferOnePoint(insideCutY, insideCutX, insideChamferType, chamferControl.comparePointList[0]);
            props.ApplyToPart(haunch);
            haunch.Flush();
            return true;
        }

        public void CreateWeld(WeldProperties weldProperties, PartBase beam, PartBase plate)
        {
            if (!isValid || haunch == null)
                return;
            Vec3 x = -insidePlane.Normal;
            Vec3 y = -topPlane.Normal;
            Mat43 workAxis = new Mat43(x, y, x.Cross(y), Vec3.ZERO);
            do {
                Weld weld = comp.CreateAutoWeld(haunch, plate, Define.SpaceAxis.AXIS_X, true);
                if (weld == null)
                    break;
                weld.WorkAxis = workAxis;
                weld.AutoWeldPosition = Define.SpaceAxis.AXIS_X;
                weldProperties.ApplyToWeld(weld);
                weld.Refresh();
            } while (false);
            do {
                Weld weld = comp.CreateAutoWeld(haunch, beam, Define.SpaceAxis.AXIS_Y, true);
                if (weld == null)
                    break;
                weld.WorkAxis = workAxis;
                weld.AutoWeldPosition = Define.SpaceAxis.AXIS_Y;
                weldProperties.ApplyToWeld(weld);
                weld.Refresh();
            } while (false);

        }
    }
}
