﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace StubConnection119
{
    class WeldHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        private List<WeldProperties> m_weldParams = new List<WeldProperties>();
        private Mat43 workAxis;
        private PartBase topPlate, bottomPlate, webPlate, mainEndPlate, subEndPlate, columnTopStiff, columnBottomStiff, columnCenterStiff;

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }


        public void Execute()
        {
            if (!ParseParameter())
                return;

            //焊缝1：上翼板（原名顶板）与柱的焊接（3条）、下翼板（原名底板）与柱的焊接（3条）、腹板（原名中间板）与柱的焊接（1条）
            {
                WeldColumn1(topPlate, m_weldParams[0]); //上翼板
                WeldColumn1(bottomPlate, m_weldParams[0]); //下翼板
            }

            //焊缝2：上翼板与主端板的焊接（1条）、下翼板与主端板的焊接（1条）
            {
                CreateWeld(mainEndPlate, topPlate, m_weldParams[1], LoongCAD.STC.Define.SpaceAxis.AXIS_Y);
                CreateWeld(mainEndPlate, bottomPlate, m_weldParams[1], LoongCAD.STC.Define.SpaceAxis.AXIS_Y);
            }

            //焊缝3：上侧柱加劲板（左侧上侧加劲肋）与柱的焊接（3条）、下侧柱加劲板（左侧下侧加劲肋）与柱的焊接（3条）
            {
                double margin = m_paramsHandler.GetDoubleValue(ParamNames.LEFT_STIFFENER_VERT_MARGIN_SIDE);
                if (margin < 1e-3)
                {
                    //上翼板
                    WeldColumn3(columnTopStiff, m_weldParams[2]);
                    //下翼板
                    WeldColumn3(columnBottomStiff, m_weldParams[2]);
                }
            }

            //焊缝4：中间柱加劲板与柱的焊接（1条）、中间柱加劲板与上侧柱加劲板的焊接（1条）、中间柱加劲板与下侧柱加劲板的焊接（1条）
            {
                CreateWeld(columnTopStiff, columnCenterStiff, m_weldParams[3], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
                CreateWeld(m_context.MainPart, columnCenterStiff, m_weldParams[3], LoongCAD.STC.Define.SpaceAxis.AXIS_Y);
                CreateWeld(columnBottomStiff, columnCenterStiff, m_weldParams[3], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
            }

            //焊缝5：腹板与上、下翼缘板的焊接（2条）
            {
                CreateWeld(topPlate, webPlate, m_weldParams[4], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
                CreateWeld(bottomPlate, webPlate, m_weldParams[4], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
            }

            //焊缝6：腹板与柱的焊接（1条）
            {
                CreateWeld(m_context.MainPart, webPlate, m_weldParams[5], LoongCAD.STC.Define.SpaceAxis.AXIS_NY);
            }

            //焊缝7：腹板与主端板的焊接（1条）
            {
                CreateWeld(mainEndPlate, webPlate, m_weldParams[6], LoongCAD.STC.Define.SpaceAxis.AXIS_Y);
            }

            //焊缝8：加劲板（原名上下、腋板）与端板焊接（2条）× 4 （主上、主下、次上、次下4个腋板）
             {
                //次端板和次梁上的腋板
                foreach (var haunch in m_context.HaunchDict["sub_haunch"])
                {
                    haunch.CreateWeld(m_weldParams[7], m_context.SubPart, subEndPlate);
                }
                //主端板和上翼板的腋板
                if (m_context.HaunchDict["main_top_haunch"].Count > 0)
                {
                    m_context.HaunchDict["main_top_haunch"][0].CreateWeld(m_weldParams[7], topPlate, mainEndPlate);
                }
                //主端板和下翼板的腋板
                if (m_context.HaunchDict["main_bottom_haunch"].Count > 0)
                {
                    m_context.HaunchDict["main_bottom_haunch"][0].CreateWeld(m_weldParams[7], bottomPlate, mainEndPlate);
                }
                //主端板和腹板的腋板
                foreach (var haunch in m_context.HaunchDict["main_web_haunch"])
                {
                    haunch.CreateWeld(m_weldParams[7], webPlate, mainEndPlate);
                }
            }

            //焊缝9：次梁翼缘与次端板的焊接（2条）
            {
                if (!m_context.IsSubRotate)
                {
                    CreateWeld(m_context.SubPart, subEndPlate, m_weldParams[8], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
                    CreateWeld(m_context.SubPart, subEndPlate, m_weldParams[8], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
                }
                else
                {
                    CreateWeld(m_context.SubPart, subEndPlate, m_weldParams[8], LoongCAD.STC.Define.SpaceAxis.AXIS_X);
                    CreateWeld(m_context.SubPart, subEndPlate, m_weldParams[8], LoongCAD.STC.Define.SpaceAxis.AXIS_NX);
                }

            }

            //焊缝10：次梁腹板与次端板的焊接（1条）
            {
                if (!m_context.IsSubRotate)
                    CreateWeld(m_context.SubPart, subEndPlate, m_weldParams[9], LoongCAD.STC.Define.SpaceAxis.AXIS_X);
                else
                    CreateWeld(m_context.SubPart, subEndPlate, m_weldParams[9], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
            }

            webPlate.PreBuilt.Main = m_context.MainPart;
            webPlate.PreBuilt.Refresh();
        }

        public bool ParseParameter()
        {
            //确定方向，为自动焊缝的工作坐标系做准备
            Vec3 x = -m_context.MainBottomBound.Normal;
            Vec3 y = m_context.MainNearBoundPlane.Normal;
            Vec3 z = m_context.TopOrient;
            Vec3 origin = m_context.MainNearBoundPlane.Intersect(m_context.SubTopPlane.Move(50), m_context.SubVertCenterPlane);
            workAxis = new Mat43(x, y, z, origin);


            //焊缝属性
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                WeldProperties weldProperties = new WeldProperties();
                weldProperties.SetProps(weldProps);
                m_weldParams.Add(weldProperties);
            }


            //获取零件
            if (m_context.PartsDict.ContainsKey("top_plate"))
                topPlate = m_context.PartsDict["top_plate"];
            if (m_context.PartsDict.ContainsKey("bottom_plate"))
                bottomPlate = m_context.PartsDict["bottom_plate"];
            if (m_context.PartsDict.ContainsKey("web_plate"))
                webPlate = m_context.PartsDict["web_plate"];
            if (m_context.PartsDict.ContainsKey("main_end_plate"))
                mainEndPlate = m_context.PartsDict["main_end_plate"];
            if (m_context.PartsDict.ContainsKey("sub_end_plate"))
                subEndPlate = m_context.PartsDict["sub_end_plate"];
            if (m_context.PartsDict.ContainsKey("column_top_stiff"))
                columnTopStiff = m_context.PartsDict["column_top_stiff"];
            if (m_context.PartsDict.ContainsKey("column_bottom_stiff"))
                columnBottomStiff = m_context.PartsDict["column_bottom_stiff"];
            if (m_context.PartsDict.ContainsKey("column_center_stiff"))
                columnCenterStiff = m_context.PartsDict["column_center_stiff"];


            return true;
        }

        public void WeldColumn1(PartBase weldPart, WeldProperties weldProps) 
        {
            PartBase partBase1 = m_context.MainPart;
            CreateWeld(partBase1, weldPart, weldProps, LoongCAD.STC.Define.SpaceAxis.AXIS_X); //与柱的上翼缘焊接
            CreateWeld(partBase1, weldPart, weldProps, LoongCAD.STC.Define.SpaceAxis.AXIS_NX); //与柱的下翼缘焊接
            CreateWeld(partBase1, weldPart, weldProps, LoongCAD.STC.Define.SpaceAxis.AXIS_NY); //与柱的腹板焊接
        }

        public void WeldColumn3(PartBase weldPart, WeldProperties weldProps)
        {
            PartBase partBase1 = m_context.MainPart;
            CreateWeld(partBase1, weldPart, weldProps, LoongCAD.STC.Define.SpaceAxis.AXIS_X); //与柱的上翼缘焊接
            CreateWeld(partBase1, weldPart, weldProps, LoongCAD.STC.Define.SpaceAxis.AXIS_NX); //与柱的下翼缘焊接
            CreateWeld(partBase1, weldPart, weldProps, LoongCAD.STC.Define.SpaceAxis.AXIS_Y); //与柱的腹板焊接
        }

        public void CreateWeld(PartBase mainPart, PartBase subPart, WeldProperties weldProps, LoongCAD.STC.Define.SpaceAxis spaceAxis)
        {
            if (mainPart == null || subPart == null)
                return;
            Weld weld = m_context.Comp.CreateAutoWeld(mainPart, subPart, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
            weldProps.ApplyToWeld(weld);
            weld.WorkAxis = workAxis;
            weld.AutoWeldPosition = spaceAxis;
            weld.Refresh();
        }

        
    }
}
