﻿using System;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    public class BuildExtModeBolts : RunningBaseClass
    {
        public BuildExtModeBolts(ComponentInstanceContext context, ParserSubPart parserSubPart, ParserGussetBolts parserGussetBolts, BuildGussetPlate buildGussetPlate) : base(context)
        {
            ParserSubPart = parserSubPart;
            ParserGussetBolts = parserGussetBolts;
            BuildGussetPlate = buildGussetPlate;
        }

        private readonly ParserSubPart ParserSubPart;
        private readonly ParserGussetBolts ParserGussetBolts;
        private readonly BuildGussetPlate BuildGussetPlate;

        private bool m_canBuild;
        private Vec3 m_buildBtmP;
        private Vec3 m_buildTopP;
        private BoltGroup m_boltGroup;

        private readonly ParamData AllParamData = new ParamData();

        private class ParamData
        {
            
            public double VerTopEdgeDist { get; set; } 
            public int VerBoltsNum { get; set; }
            public double[] VerBoltsSpacing { get; set; }
                
            public double HorFarEdgeDist { get; set; } 
            public int HorBoltsNum { get; set; }
            public double[] HorBoltsSpacing { get; set; }
        }

        private readonly BoltsData BoltsData = new BoltsData();

        public override bool Init()
        {
            if (!InitParamData()) return false;
            InitBoltsData();
            return true;
        }

        private void InitBoltsData()
        {
            BoltsData.ArrayStrideX = BoltsHelp.AdaptBoltSpacing
                (AllParamData.VerBoltsNum, AllParamData.VerBoltsSpacing, 60);
            BoltsData.ArrayStrideY = BoltsHelp.AdaptBoltSpacing
                (AllParamData.HorBoltsNum, AllParamData.HorBoltsSpacing, 60);
        }

        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.VerTopEdgeDist = handler.GetDoubleValue (ParamNames.GUSSET_BOLTS_EXTENDED_VERTICAL_BOLTS_TOP_EDGE_DIST);
            AllParamData.VerBoltsNum= handler.GetIntValue
                (ParamNames.GUSSET_BOLTS_EXTENDED_VERTICAL_BOLTS_NUMBER);
            AllParamData.VerBoltsSpacing = handler.GetDoubleArrayValue
                (ParamNames.GUSSET_BOLTS_EXTENDED_VERTICAL_BOLTS_SPACING);
            
            AllParamData.HorFarEdgeDist = handler.GetDoubleValue (ParamNames.GUSSET_BOLTS_EXTENDED_HORIZONTAL_BOLTS_FAR_EDGE_DIST);
            AllParamData.HorBoltsNum= handler.GetIntValue
                (ParamNames.GUSSET_BOLTS_EXTENDED_HORIZONTAL_BOLTS_NUMBER);
            AllParamData.HorBoltsSpacing = handler.GetDoubleArrayValue
                (ParamNames.GUSSET_BOLTS_EXTENDED_HORIZONTAL_BOLTS_SPACING);
            return true;
        }

        public override bool Prepare()
        {
            if (Equals(AllParamData.VerTopEdgeDist, double.MaxValue))
            {
                AllParamData.VerTopEdgeDist = BuildGussetPlate.m_distTopBtm / 2 - BoltsData.ArrayStrideX.Sum()/2;
            }

            BoltsData.HeadOffsetX = AllParamData.VerTopEdgeDist;
            BoltsData.VerticalOffset = CalculateOffset();
            var subPart = ParserSubPart;
            var axisX = subPart.AxisY;
            var axisY = subPart.AxisX;
            BoltsData.m_workAxis = new Mat43
                (axisX,axisY,axisX.Cross(axisY),Vec3.ZERO);
            BoltsData.RotationSide = Define.ObjectSide.SIDE_TOP;
            PrepareHandlerBuildPoint();
            return true;
        }

        private double CalculateOffset()
        {
            if (Equals(AllParamData.HorFarEdgeDist, double.MaxValue))
            {
                return 0;
            }
            
            var subPart = ParserSubPart;
            var sumL = subPart.FaceToSubPlane.Dist(subPart.BackToSubPlane.Origin);
            var off = AllParamData.HorFarEdgeDist;
            var sumSpacing = BoltsData.ArrayStrideY.Sum();
            var halfSpacing = sumSpacing / 2;
            if (off <= sumL) {
                return sumL / 2 - halfSpacing - off;
            }
            return 0;
        }

        public override bool Execute()
        {
            var subPart = ParserSubPart;
            var gstBolts = ParserGussetBolts;
            m_canBuild = subPart.m_bExtensionMode && 
                         gstBolts.AllParamData.SelectBoltsOnGussetPlate == 2;

            if (!m_canBuild) {
                return true;
            }
            if (AllParamData.HorBoltsNum == 0 || AllParamData.VerBoltsNum == 0) {
                return true;
            }

            var gstPlate = BuildGussetPlate;
            var target = m_context.TargetComp;
            var main = m_context.MainPart;
            var sub = gstPlate.GussetPlate;

            var startP = m_buildTopP;
            var endP= m_buildBtmP ;
            
            m_boltGroup = target.CreateBoltGroup(main,sub,startP,endP,true);
            if (m_boltGroup == null)
            {
                Log.Debug(@"create bolts fail");
                return false;
            }
            gstBolts.AllParamData.BoltsConfig.FillAll(m_boltGroup);
            BoltsData.SetUpWith(m_boltGroup);
            m_boltGroup.Refresh();
            
            return true;
        }

        private void PrepareHandlerBuildPoint()
        {
            var subPart = ParserSubPart;
            var gstBuild = BuildGussetPlate;
            var centPlane = PlaneHelper.GetMidPlane(subPart.FaceToSubPlane, subPart.BackToSubPlane);
            
            m_buildBtmP = subPart.RefSubBuildPlane.Intersect
                (gstBuild.m_btmMainPlane, centPlane);
            m_buildTopP = PointHelper.MoveTo(m_buildBtmP, gstBuild.m_topMainPlane);
        }
    }
}