﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    public class BuildGussetPlate : RunningBaseClass
    {
        public BuildGussetPlate(ComponentInstanceContext context, ParserSubPart subPart) : base(context)
        {
            m_subPart = subPart;
        }

        /// <summary>
        /// 子零件的处理数据
        /// </summary>
        private readonly ParserSubPart m_subPart;
        /// <summary>
        /// 节点板的构建点列表
        /// </summary>
        private readonly List<Vec3> m_buildPoints = new List<Vec3>();
        /// <summary>
        /// 在靠主零件侧的构建点,底，顶
        /// </summary>
        public List<Vec3> m_buildPointsInFacePlane = new List<Vec3>();
        /// <summary>
        /// 在支撑端的构建点
        /// </summary>
        private List<Vec3> m_buildPointsInSub = new List<Vec3>();

        /// <summary>
        /// 节点板实例
        /// </summary>
        internal SteelPlate GussetPlate { get; private set; }

        internal Plane m_topMainPlane;
        internal Plane m_btmMainPlane;
        /// <summary>
        /// 最终构建点的顶底距离
        /// </summary>
        public double m_distTopBtm;


        /// <summary>
        /// 所有的与面板相关的值
        /// </summary>
        internal ParamData AllParamData { get; } = new ParamData();
        /// <summary>
        /// 面板中的值。
        /// </summary>
        internal class ParamData 
        {
            /// <summary>
            /// 节点板的厚度
            /// </summary>
            internal double Thickness { get; set; } = 50.0d;
            /// <summary>
            /// 节点板的Y轴偏移
            /// </summary>
            internal double GussetPlateYOffset { get;set; }
            /// <summary>
            /// 顶部的延迟距离
            /// </summary>
            public double MainExtendedDist { get; set; }
            /// <summary>
            /// 底部的延迟距离
            /// </summary>
            public double TopBottomExtendedDist { get; set; }
            
            /// <summary>
            /// 主零件切割的水平尺寸
            /// </summary>
            public double MainCutHorDist { get; set; }
            /// <summary>
            /// 主零件切割的竖直尺寸
            /// </summary>
            public double MainCutVerDist { get; set; }
        }
        public override bool Init()
        {
            if (!InitParamData()) return false;
            return true;
        }

        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.Thickness = m_pub.GussetPlatePro.Thickness;
            AllParamData.GussetPlateYOffset = handler.GetDoubleValue(ParamNames.GUSSET_GUSSET_PLATE_Y_DIRECTION_OFFSET);
            AllParamData.MainExtendedDist = handler.GetDoubleValue(ParamNames.GUSSET_MAIN_PART_EDGE_EXTENSION_DISTANCE);
            AllParamData.TopBottomExtendedDist= handler.GetDoubleValue(ParamNames.GUSSET_TOP_BOTTOM_EDGE_EXTENSION_DISTANCE);
            AllParamData.MainCutHorDist = handler.GetDoubleValue(ParamNames.GUSSET_HORIZONTAL_SIZE_OF_CUT);
            AllParamData.MainCutVerDist = handler.GetDoubleValue(ParamNames.GUSSET_VERTICAL_SIZE_OF_CUT);
            return true;
        }


        public override bool Prepare()
        {
            //处理节点板的构建点
            PrepareBuildPoints();
            //处理节点板的偏移
            PrepareHandleBuildPointsOffset();
            var top = m_buildPointsInFacePlane.Last();
            var btm = m_buildPointsInFacePlane.First();
            
            m_topMainPlane = new Plane(top, m_subPart.AxisY);
            m_btmMainPlane = new Plane(btm, -m_subPart.AxisY);
            m_distTopBtm = top.Dist(btm);
            return true;
        }



        public override bool Execute()
        {
            var target = m_context.TargetComp;
            if (AllParamData.Thickness <= 0) {
                return false;
            }
            GussetPlate = target.CreateSteelPlate(m_buildPoints.ToArray(), 
                AllParamData.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
            if (GussetPlate == null) {
                Log.Debug(@"create SteelPlate fail");
                return false;
            }
            //设置节点板的其他属性
            ExecuteHandleGussetPlate();
            //配置在主零件上的参照面
            if (m_subPart.AllParamData.OffSetForFaceBuildPlane == 0)
            {
                m_context.BuildWeld.BuildWeldForGussetPlate(m_topMainPlane,m_btmMainPlane,GussetPlate,m_subPart);
            }
            //处理穿过主零件的切割
            ExecuteThroughMainPartCut();
            //ExecuteOverThroughMainHeadTail();
            return true;
        }

        private void ExecuteThroughMainPartCut()
        {
            var cutSide = new List<Plane>();
            var insidePl = m_subPart.FaceBuildPlane.Move(-m_pub.GstPassThroughTheMainPartDist); 
            cutSide.Add(insidePl);
            cutSide.Add(m_subPart.NearBuildPlane.Move(AllParamData.MainCutHorDist));
            cutSide.Add(m_subPart.FaceBuildPlane.Move(1));
            cutSide.Add(m_subPart.FarBuildPlane.Move(AllParamData.MainCutHorDist));
            m_context.TargetComp.CreatePlateCutter(m_context.MainPart,
                m_btmMainPlane.Move(AllParamData.MainCutVerDist),
                m_topMainPlane.Move(AllParamData.MainCutVerDist),
                cutSide.ToArray(),true);
        }

        private void PrepareBuildPoints()
        {
            var select = m_pub.ShapeSelect;
            switch (select)
            {
                case 0:
                    HandleShape0();
                    break;
                case 1:
                    HandleShape0();
                    //HandleShape1();
                    break;
                case 2:
                    HandleShape2();
                    break;
                case 3:
                    HandleShape0();
                    break;
            }
            HandleBottomExtendedDist();
            HandleMainPartExtendedDist();
            m_buildPoints.AddRange(HandlerThroughTheMainPart(m_buildPointsInFacePlane));
            HandleTopExtendedDist();
            m_buildPoints.AddRange(m_buildPointsInSub);
            return;
        }

        private void HandleMainPartExtendedDist()
        {
            if (m_subPart.m_subNum != 1)
                return;
            if (Equals(double.MaxValue, AllParamData.MainExtendedDist))
                return;
            var dist = AllParamData.MainExtendedDist;
            var top = m_buildPointsInFacePlane.Last();
            var btm = m_buildPointsInFacePlane.First();
            var dot = m_subPart.AxisY.Dot(m_subPart.m_subPartsRef.NearToFarLine);
            var dir = top - btm;
            if (dot == 0) {
                return;
            }else if (dot > 0)//btm不动,移动top
            {
                m_buildPointsInFacePlane[1] = PointHelper.Move(btm, dir, dist);
            }else if (dot < 0)//top不懂，移动btm
            {
                m_buildPointsInFacePlane[0] = PointHelper.Move(btm, -dir, dist);
            }
            
        }


        private void HandleBottomExtendedDist()
        {
            if (m_subPart.m_subNum != 1)
                return;
            //通过角度判读
            var dot = m_subPart.AxisY.Dot(m_subPart.m_subPartsRef.NearToFarLine);
            if (dot <= 0) {
                return;
            }
            if (!Equals(double.MaxValue, AllParamData.TopBottomExtendedDist))
            {
                var p = m_buildPointsInFacePlane.First();
                var addP = PointHelper.Move(p, m_subPart.AxisX, AllParamData.TopBottomExtendedDist);
                m_buildPoints.Add(addP);
            }
            else
            {
                if (m_subPart.m_btmExtendedPt != Vec3.ZERO)
                {
                    m_buildPoints.Add(m_subPart.m_btmExtendedPt);
                }
            }
        }

        private void HandleTopExtendedDist()
        {
            if (m_subPart.m_subNum != 1)
                return;
            //通过角度判读
            var dot = m_subPart.AxisY.Dot(m_subPart.m_subPartsRef.NearToFarLine);
            if (dot >= 0) {
                return;
            }
            if (!Equals(double.MaxValue, AllParamData.TopBottomExtendedDist))
            {
                var p = m_buildPointsInFacePlane.Last();
                var addP = PointHelper.Move(p, m_subPart.AxisX, AllParamData.TopBottomExtendedDist);
                m_buildPoints.Add(addP);
            }
            else
            {
                if (m_subPart.m_topExtendedPt != Vec3.ZERO)
                {
                    m_buildPoints.Add(m_subPart.m_topExtendedPt);
                }
            }
        }

        /// <summary>
        /// 获取支撑端最远的平行于face面的参考面
        /// </summary>
        /// <returns></returns>
        private Plane GetMaxFarPlane()
        {
            var inSub = GetBuildPointsInSub();
            var maxFarPoint = m_subPart.m_mainPart.CtrlPoints.First().WorldPos;
            foreach (var subPoint in inSub)
            {
                var ret = PointHelper.ComparePointsByProjection(subPoint, maxFarPoint, m_subPart.AxisX);
                //subPoint比maxFarPoint在AxisX方向更远的时候，更新maxFarPoint的值
                if (ret > 0)
                    maxFarPoint = subPoint;
            }

            var maxFarPlane = new Plane(maxFarPoint, m_subPart.AxisX);
            return maxFarPlane;
        }

        private void HandleShape0()
        {
            m_buildPointsInFacePlane = m_subPart.GetBuildPointsInFaceBuildPlane();
            m_buildPointsInSub = GetBuildPointsInSub();
        }
        
        private void HandleShape1()
        {
            //返回的是底，顶
            m_buildPointsInFacePlane = m_subPart.GetBuildPointsInFaceBuildPlane();
            
            SubData maxFarSub = m_subPart.SubDataSort.First();
            var maxFarPoint = new Vec3(maxFarSub.TopConnFarAlignPoint);
            foreach (var sub in m_subPart.SubDataSort)
            {
                var top = PointHelper.Move
                    (sub.TopConnFarAlignPoint, sub.TopPlane.Normal, sub.TopFarAlignPlaneOffset);
                var bottom = PointHelper.Move
                    (sub.BtmConnFarAlignPoint, sub.BottomPlane.Normal, sub.BottomFarAlignPlaneOffset);
                var ret = PointHelper.ComparePointsByProjection(top, maxFarPoint, m_subPart.AxisX);
                //subPoint比maxFarPoint在AxisX方向更远的时候，更新maxFarPoint的值
                if (ret > 0)
                {
                    maxFarPoint = top;
                    maxFarSub = sub;
                }
                ret = PointHelper.ComparePointsByProjection(bottom, maxFarPoint, m_subPart.AxisX);
                if (ret > 0)
                {
                    maxFarPoint = bottom;
                    maxFarSub = sub;
                }
            }

            var planeTop = new Plane(m_buildPointsInFacePlane.Last(), m_subPart.AxisY);
            var planeBtm= new Plane(m_buildPointsInFacePlane.First(), m_subPart.AxisY);
            var planeMaxFar = new Plane(maxFarPoint, m_subPart.AxisX);
            
            //底,顶
            
            var subTmp = m_subPart.SubDataSort.First();
            var point = m_subPart.RefSubBuildPlane.Intersect(planeTop, subTmp.BottomPlane);
            m_buildPointsInSub.Add(point);

            point = m_subPart.RefSubBuildPlane.Intersect(planeMaxFar, maxFarSub.TopPlane);
            m_buildPointsInSub.Add(point);
            point = m_subPart.RefSubBuildPlane.Intersect(planeMaxFar, maxFarSub.BottomPlane);
            m_buildPointsInSub.Add(point);
            
            subTmp = m_subPart.SubDataSort.Last();
            point = m_subPart.RefSubBuildPlane.Intersect(planeBtm, subTmp.TopPlane);
            m_buildPointsInSub.Add(point);
            
        }

        private void HandleShape2()
        {
            var maxFarPlane = GetMaxFarPlane();
            //返回的是底，顶
            m_buildPointsInFacePlane = m_subPart.GetBuildPointsInFaceBuildPlane();
            var btmPlane = new Plane(m_buildPointsInFacePlane.First(),m_subPart.AxisY);
            var topPlane = new Plane(m_buildPointsInFacePlane.Last(),m_subPart.AxisY);

            var btmFarPoint = m_subPart.RefSubBuildPlane.Intersect(btmPlane, maxFarPlane);
            var topFarPoint = m_subPart.RefSubBuildPlane.Intersect(topPlane, maxFarPlane);
            
            m_buildPointsInSub.Add(topFarPoint);
            m_buildPointsInSub.Add(btmFarPoint);
        }



        /// <summary>
        /// 处理节点板构建点的偏移
        /// </summary>
        private void PrepareHandleBuildPointsOffset()
        {
            var axisY = m_subPart.AxisY;
            var offY = AllParamData.GussetPlateYOffset;
            for (var i = 0; i < m_buildPoints.Count; i++)
            {
                var point = m_buildPoints[i];
                m_buildPoints[i] = PointHelper.Move(point, axisY, offY);
            }
        }

        private List<Vec3> HandlerThroughTheMainPart(List<Vec3> inFace)
        {
            //处理穿过主零件的情况
            if (m_pub.SelectGstPassThroughTheMainPart != 2)
                return inFace;
            var refPl = m_subPart.FaceBuildPlane.Move(-m_pub.GstPassThroughTheMainPartDist);
            var ret = new List<Vec3>();
            ret.Add(PointHelper.MoveTo(inFace.First(),refPl));
            ret.Add( PointHelper.MoveTo(inFace.Last(),refPl));
            return ret;
        }

        /// <summary>
        /// 返回在支撑上节点板的构建点
        /// </summary>
        /// <returns></returns>
        private List<Vec3> GetBuildPointsInSub()
        {
            var points = new List<Vec3>();
            foreach (var sub in m_subPart.SubDataSort)
            {
                var top = PointHelper.Move
                    (sub.TopConnFarAlignPoint,sub.TopPlane.Normal,sub.TopFarAlignPlaneOffset);
                var bottom = PointHelper.Move
                    (sub.BtmConnFarAlignPoint,sub.BottomPlane.Normal,sub.BottomFarAlignPlaneOffset);
                points.Add(top);
                points.Add(bottom);
            }
            return points;
        }

        /// <summary>
        /// 对节点板的后处理
        /// </summary>
        private void ExecuteHandleGussetPlate()
        {
            GussetPlate.WorkAxis = new Mat43(m_subPart.AxisX, m_subPart.AxisY, m_subPart.AxisZ, Vec3.ZERO);
            m_pub.GussetPlatePro.ApplyToPart(GussetPlate);
            GussetPlate.Flush();
        }


    }
}