﻿using System;
using System.Collections.Generic;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    public interface IInit
    {
        /// <summary>
        /// 无依赖性的准备操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Init();
    }
     
    public interface IPrepare
    {
        /// <summary>
        /// 有依赖性的准备操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Prepare();
    }
     
    public interface IExecute
    {
        /// <summary>
        /// 执行操作，在所有准备结束后的操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Execute();
    }
    
    public class RunningBaseClass:IInit,IPrepare,IExecute
    {
        /// <summary>
        /// 缓存上下文
        /// </summary>
        protected readonly ComponentInstanceContext m_context;
        /// <summary>
        /// 公用的ParamData
        /// </summary>
        protected readonly PublicParamData m_pub;
        protected RunningBaseClass(ComponentInstanceContext context) {
            m_context = context;
            m_pub = m_context.PublicParamData;
        }
        public virtual bool Init() {
            return true;
        }

        public virtual bool Prepare() {
            return true;
        }
        public virtual bool Execute() {
            return true;
        }

    }
    
    public class CompositeCommand : IExecute
    {
        private readonly List<IExecute> m_commands = new List<IExecute>();

        public void AddCommand(IExecute command)
        {
            m_commands.Add(command);
        }
        
        public bool Execute()
        {
            foreach (var command in m_commands)
            {
                if (!command.Execute())
                {
                    Log.Debug(command.GetType().Name + @" exe fail");
                    return false;
                }
            }
            return true;
        }
    }
    
    /// <summary>
    /// 汇总所有的派生类执行
    /// </summary>
    public class RunningHandler
    {
        private readonly List<RunningBaseClass> m_run = new List<RunningBaseClass>();
        private readonly List<CompositeCommand> m_execute = new List<CompositeCommand>();
        
        public void Running(ComponentInstanceContext context)
        {
            var publicParamData = new PublicParamData(context);
            var buildWeld = new BuildWeld(context);
            var parserBraceBolts= new ParserBraceBolts(context);
            var parserSubPart = new ParserSubPart(context,parserBraceBolts);
            var buildGussetPlate = new BuildGussetPlate(context,parserSubPart);
            var parserGussetBolts =new ParserGussetBolts(context,parserSubPart,buildGussetPlate);
            var buildConnPart = new BuildConnPart(context, parserSubPart);
            var buildBraceBolts = new BuildBraceBolts(context, parserSubPart, parserBraceBolts, buildGussetPlate,buildConnPart);
            var buildConnPlate=new BuildConnPlate(context, parserSubPart, parserGussetBolts, buildGussetPlate);
            var buildLProfilePlate = new BuildLProfilePlate(context, parserSubPart, buildConnPlate,buildGussetPlate);
            var buildGussetPlateBolts = new BuildGussetPlateBolts(context, parserSubPart, parserGussetBolts,buildGussetPlate,buildConnPlate,buildLProfilePlate);
            var buildExtModeBolts= new BuildExtModeBolts(context, parserSubPart, parserGussetBolts, buildGussetPlate);
            var buildBraceNotch = new BuildBraceNotch(context,parserSubPart);
            
            m_run.Add(publicParamData);
            m_run.Add(buildWeld);
            m_run.Add(parserBraceBolts);
            m_run.Add(parserSubPart);
            m_run.Add(buildConnPart);
            m_run.Add(buildBraceNotch);
            m_run.Add(buildGussetPlate);
            m_run.Add(parserGussetBolts);
            m_run.Add(buildBraceBolts);
            m_run.Add(buildConnPlate);
            m_run.Add(buildLProfilePlate);
            m_run.Add(buildGussetPlateBolts);
            m_run.Add(buildExtModeBolts);
            
            
            var commands = new CompositeCommand();
            commands.AddCommand(buildBraceNotch);
            commands.AddCommand(buildConnPart);
            commands.AddCommand(buildGussetPlate);
            commands.AddCommand(buildBraceBolts);
            commands.AddCommand(buildConnPlate);
            commands.AddCommand(buildLProfilePlate);
            commands.AddCommand(buildGussetPlateBolts);
            commands.AddCommand(buildExtModeBolts);
            commands.AddCommand(buildWeld);
            m_execute.Add(commands);
            
            
            foreach (var run in m_run) {
                if (!run.Init()) {
                    Log.Debug(run.GetType().Name + @" init fail");
                    return;
                }
            }
            Log.Debug(@"all init success");
            foreach (var run in m_run) {
                if (!run.Prepare()) {
                    Log.Debug(run.GetType().Name + @" prepare fail");
                    return;
                }
            }
            Log.Debug(@"all prepare success");
            foreach (var exe in m_execute)
            {
                if (!exe.Execute()) {
                    return;
                }
            }   
            Log.Debug(@"all exe success");
        }
    }

}