﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main.Running;
using U.S.Base_plate_1047_.Running;
using U.S.Base_plate_1047_.Tool;
using TargetComponentType = LoongCAD.STC.Object3D.DetailComponent;


namespace U.S.Base_plate_1047_.Main
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        /// <summary>
        /// 公用面板属性
        /// </summary>
        public PublicParamData PublicParamData { get; internal set; }
        /// <summary>
        /// 目标类型组件的接口
        /// </summary>
        public TargetComponentType TargetComp { get; private set; }
        /// <summary>
        ///组件的主要部分
        /// </summary>
        public Beam MainPart { get; private set; } = null;
        /// <summary>
        ///主构件梁的面分析 
        /// </summary>
        public BeamPlanes MainPlanes { get; private set; } = null;
        /// <summary>
        /// 拿到的面板键值对处理器
        /// </summary>
        public ParamsHandler ParamsHandler { get; set; } = null;
        /// <summary>
        /// 基座
        /// </summary>
        public BuildBasePlate BasePlate { get; set; } = null;

        /// <summary>
        /// 灌浆孔
        /// </summary>
        public BuildGroutHole GroutHole { get; set; } = null;
        /// <summary>
        /// 水平面板,层板
        /// </summary>
        public BuildLevelingPlate LevelingPlate { get; set; } = null;
        /// <summary>
        /// 配件板,衬板
        /// </summary>
        public BuildFittingPlate FittingPlate { get; set; } = null;
        /// <summary>
        /// 抗剪键
        /// </summary>
        public BuildShearKey ShearKey { get; set; } = null;
        /// <summary>
        /// 螺栓
        /// </summary>
        public BuildBolts Bolts { get; set; } = null;
        /// <summary>
        /// 锚钉杆
        /// </summary>
        public BuildAnchorRod AnchorRod{get;set;}=null;
        /// <summary>
        /// 切割柱子
        /// </summary>
        public CutMainColumn CutMainColumn { get; set; } = null;
        /// <summary>
        /// 焊缝
        /// </summary>
        public BuildWeld Weld { get; set; } 
        /// <summary>
        /// 灌浆
        /// </summary>
        public BuildGrouting Grouting { get; set; }
        
        /// <summary>
        /// 组件的坐标轴X,FrontBound，即腹板的法线方向
        /// </summary>
        public Vec3 AxisX{get; private set;}
        /// <summary>
        /// 组件的坐标轴Y,TopBound，即翼缘的法线方向
        /// </summary>
        public Vec3 AxisY{get; private set;}
        /// <summary>
        /// 组件的坐标轴Z
        /// </summary>
        public Vec3 AxisZ{get; private set;}
        /// <summary>
        /// 组件的工作坐标系
        /// </summary>
        public Mat43 WorkMat43 { get; private set; }
        
        /// <summary>
        /// 主零件靠近细部点的控制点
        /// </summary>
        public Vec3 MainCtlNearPt { get; private set; }
        /// <summary>
        /// 主零件远离细部点的控制点
        /// </summary>
        public Vec3 MainCtlFarPt { get; private set; }
        /// <summary>
        /// 主零件的顶底中心面
        /// </summary>
        public Plane MainCentreTopBtmPl { get; private set; }
        /// <summary>
        /// 主零件的前后中心面
        /// </summary>
        public Plane MainCentreFrtBckPl { get; private set; }
        /// <summary>
        /// 细部组件的选择点
        /// </summary>
        public Vec3 DetailPt { get; private set; }
        /// <summary>
        /// 以细部组件的选择点为原点，以AxisZ为法线的面
        /// </summary>
        public Plane DetailPl { get; private set; }
        /// <summary>
        /// 细部组件的中心点
        /// </summary>
        public Vec3 DetailCentrePt { get; private set; }
        /// <summary>
        /// 主零件的截面属性中的宽
        /// </summary>
        public double MainPartWidth { get; private set; }
        /// <summary>
        /// 主零件的截面属性中的高
        /// </summary>
        public double MainPartHeight { get; private set; }
        /// <summary>
        /// 主零件的腹板厚度，如果没有腹板面则为0.
        /// </summary>
        public double MainWebThk { get; private set; }
        /// <summary>
        /// 主零件的底部翼缘厚度，如果没有底部翼缘面则为0.
        /// </summary>
        public double MainBtmFlangeThk { get; private set; }
        /// <summary>
        /// 主零件的顶部翼缘厚度，如果没有顶部翼缘面则为0.
        /// </summary>
        public double MainTopFlangeThk { get; private set; }
        /// <summary>
        /// 主零件的腹板中心面
        /// </summary>
        public Plane MainCtlWebPl { get; private set; }
        /// <summary>
        /// 主零件的底部翼缘中心面
        /// </summary>
        public Plane MainCtlBtmFlangePl { get; private set; }
        /// <summary>
        /// 主零件的顶部翼缘中心面
        /// </summary>
        public Plane MainCtlTopFlangePl { get; private set; }

        
        
        /// <summary>
        /// 利用传过来的TargetComponent初始化上下文
        /// 失败时返回null
        /// </summary>
        /// <param name="comp"></param>
        /// <returns></returns>
        public static ComponentInstanceContext CreatInstance(TargetComponentType comp)
        {
            var compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }

        
        /// <summary>
        /// 上下文的初始化，处理TargetComponent的信息，解析数据。
        /// 如果处理、解析失败，则返回false
        /// </summary>
        /// <param name="comp"></param>
        /// <returns></returns>
        private bool Initialize(TargetComponentType comp)
        {
            if (comp == null)
                return false;
            //把传过来的组件，放到上下文
            TargetComp = comp;
            
            //把目标组件的主构件尝试转化为梁
            if (!(comp.Main is Beam mainPart))
                return false;
            //赋值给组件的主要部分（一般情况是梁）
            MainPart = mainPart;
            if (MainPart.Profile == null) {return false;}
            MainPartHeight = MainPart.Profile.ShapeHeight;
            MainPartWidth = MainPart.Profile.ShapeWidth;
            //解析梁
            MainPlanes = BeamPlanes.CreateFromBeam(MainPart);
            if (IsAnyBoundPlaneNull(MainPlanes)) { return false; }
            
            MainCentreTopBtmPl = PlaneHelper.GetMidPlane(MainPlanes.TopBound, MainPlanes.BottomBound);
            if (MainCentreTopBtmPl == null) {
                return false;
            }
            MainCentreFrtBckPl = PlaneHelper.GetMidPlane(MainPlanes.FrontBound, MainPlanes.BackBound);
            if (MainCentreFrtBckPl == null) {
                return false;
            }
            var pls = MainPlanes;
            MainTopFlangeThk = BeamPlanesHelper.GetTopFlangeThickness(pls);
            MainBtmFlangeThk = BeamPlanesHelper.GetBottomFlangeThickness(pls);
            MainWebThk = BeamPlanesHelper.GetWebThickness(pls);
            MainCtlTopFlangePl = (pls.TopFlange??pls.TopBound).Move(-MainTopFlangeThk / 2);
            MainCtlBtmFlangePl = (pls.BottomFlange??pls.BottomBound).Move(-MainBtmFlangeThk / 2);
            MainCtlWebPl = (pls.FrontWeb??pls.FrontBound).Move(-MainWebThk / 2);


            if (TargetComp.CtrlPoints.First() != null) {
                DetailPt = TargetComp.CtrlPoints.First().WorldPos;
            }
            else {return false;}

            var mainFirst = mainPart.CtrlPoints.First();
            if (mainFirst == null) { return false;}
            var mainLast = mainPart.CtrlPoints.Last();
            if (mainLast == null) { return false;}
            var dist1 = DetailPt.Dist(mainFirst.WorldPos);
            var dist2 = DetailPt.Dist(mainLast.WorldPos);
            MainCtlNearPt = dist1 <= dist2 ? mainFirst.WorldPos : mainLast.WorldPos;
            MainCtlFarPt = dist1 <= dist2 ? mainLast.WorldPos : mainFirst.WorldPos;
            //默认的AxisZ应该是nearPt指向FarPt
            AxisZ = MainCtlFarPt - MainCtlNearPt;
            //TODO 考虑和组件坐标系适配
            AxisX = new Vec3(MainPlanes.FrontBound.Normal.Normalize());
            AxisY = new Vec3(MainPlanes.TopBound.Normal.Normalize());

            AxisX = AxisX.Normalize();
            AxisY = AxisY.Normalize();
            AxisZ = AxisZ.Normalize();

            WorkMat43 = new Mat43(AxisX, AxisY, AxisZ,Vec3.ZERO);

            DetailPl = new Plane(DetailPt, AxisZ);
            DetailCentrePt = DetailPl.Intersect(MainCentreTopBtmPl, MainCentreFrtBckPl);
            if (DetailCentrePt == Vec3.MAX) {
                //Console.WriteLine(@"can't get DetailCentrePt");
                return false;
            }
#if DEBUG
            PrintPlane();
#endif
            return true;
        }


        private void PrintPlane()
        {
            var p = PointHelper.Midpoint(MainCtlFarPt,MainCtlNearPt);
            TargetComp.CreateLine(p, p.ProjectTo(MainPlanes.TopBound), 0.0d, Color.Red, true);
            TargetComp.CreateLine(p, p.ProjectTo(MainPlanes.BottomBound), 0.0d, Color.Blue, true);
            TargetComp.CreateLine(p, p.ProjectTo(MainPlanes.FrontBound), 0.0d, Color.Yellow, true);
            TargetComp.CreateLine(p, p.ProjectTo(MainPlanes.BackBound), 0.0d, Color.White, true);
            PlaneHelper.CheckAllBeamPlane(MainPlanes);
        }



        /// <summary>
        /// 检查面存在
        /// </summary>
        /// <param name="mainPlanes"></param>
        /// <returns></returns>
        private bool IsAnyBoundPlaneNull(BeamPlanes mainPlanes)
        {
            
            return Check(mainPlanes.BackBound) || Check(mainPlanes.FrontBound) ||
                    Check(mainPlanes.BottomBound) || Check(mainPlanes.TopBound) ||
                    Check(mainPlanes.HeadBound) || Check(mainPlanes.TailBound);
                    ;
#if false
            return Check(mainPlanes.BackBound) || Check(mainPlanes.FrontBound) ||
                    Check(mainPlanes.BackWeb) || Check(mainPlanes.FrontWeb) ||
                    Check(mainPlanes.BackWebInside) || Check(mainPlanes.FrontWebInside) ||
                    Check(mainPlanes.BottomFlangeBack) || Check(mainPlanes.TopFlangeFront) ||
                    Check(mainPlanes.BottomBound) || Check(mainPlanes.TopBound) ||
                    Check(mainPlanes.BottomFlange) || Check(mainPlanes.TopFlange) ||
                    Check(mainPlanes.BottomFlangeInside) || Check(mainPlanes.TopFlangeInside) ||
                    Check(mainPlanes.BottomFlangeFront) || Check(mainPlanes.TopFlangeBack) ||
                    Check(mainPlanes.HeadBound) || Check(mainPlanes.TailBound);
                    ;
#endif
            bool Check(Plane plane)
            {
                return plane == null;
            }
        }

        /// <summary>
        /// 返回螺栓需要的所有子零件组
        /// </summary>
        /// <returns></returns>
        public List<PartBase> GetBoltsAllSubPartBases()
        {
            var ret = new List<PartBase>();
            if(LevelingPlate.Plate != null)
                ret.Add(LevelingPlate.Plate);
            foreach (var plates in FittingPlate.m_allFittingPlate)
            {
                ret.AddRange(plates);
            }

            return ret;
        }

        /// <summary>
        /// 返回螺栓需要的CutLength
        /// </summary>
        /// <returns></returns>
        public double GetBoltsCutLength()
        {
            var ret = 0.0d;
            //ret += BasePlate.m_thickness;
            ret += PublicParamData.BasePlateThickness;
            if (LevelingPlate.m_plateData != null)
            {
                ret += LevelingPlate.m_plateData.Thickness;
            }

            double sum = 0;
            foreach (var plates in FittingPlate.m_allFittingPlate)
            {
                foreach (var data in plates)
                {
                    sum += data.Thickness;
                }
            }
            ret += sum;
            return ret*2+1;
        }

        /// <summary>
        /// 清理函数
        /// </summary>
        private void Clear()
        {
            MainPart = null;
            MainPlanes = null;
            BasePlate = null;
            GroutHole = null;
            LevelingPlate = null;
            FittingPlate = null;
            ShearKey = null;
            Bolts = null;
            CutMainColumn = null;
            ParamsHandler = null;
            TargetComp = null;
        }

    }
}
