﻿using System;
using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    public class BuildBasePlate : RunningBaseClass
    {
        public BuildBasePlate(ComponentInstanceContext context) : base(context)
        {
            m_context.BasePlate = this;
        }
        /// <summary>
        /// 基板的宽度,工作平面的x方向
        /// </summary>
        private double m_width = double.MaxValue;
        /// <summary>
        /// 基板的高度，工作平面的y方向
        /// </summary>
        private double m_height= double.MaxValue;
        /// <summary>
        /// 基板的厚度
        /// </summary>
        private double m_thickness= 20.0d;
        /// <summary>
        /// 基板顶面的几何中心点
        /// </summary>
        public Vec3 m_buildTopPoint = Vec3.ZERO;
        
        /// <summary>
        /// 基板的构建基准点，基板的构建面上的几何中点，
        /// 用于计算基板的构建点
        /// </summary>
        public Vec3 m_buildBasePlatePoint = Vec3.ZERO;

        /// <summary>
        /// 创建好之后的基板对象
        /// </summary>
        public SteelPlate m_basePlate = null;

        /// <summary>
        /// 底部折角1的宽度
        /// </summary>
        private double m_width_1 = 0.0f;
        /// <summary>
        /// 底部折角1的高度
        /// </summary>
        private double m_height_1 = 0.0f;
        /// <summary>
        /// 底部折角2的高度
        /// </summary>
        private double m_height_2 = 0.0f;
        /// <summary>
        /// 底部折角2的宽度
        /// </summary>
        private double m_width_2 = 0.0f;
        /// <summary>
        /// 底部折角3的宽度
        /// </summary>
        private double m_width_3 = 0.0f;
        /// <summary>
        /// 底部折角3的高度
        /// </summary>
        private double m_height_3 = 0.0f;
        /// <summary>
        /// 底部折角4的高度
        /// </summary>
        private double m_height_4 = 0.0f;
        /// <summary>
        /// 底部折角4的宽度
        /// </summary>
        private double m_width_4 = 0.0f;

        /// <summary>
        /// 需要进行方切割的底板角点编号集
        /// </summary>
        private HashSet<int> m_squareCutCorners = new HashSet<int>{};
        
        /// <summary>
        /// 底板角点集
        /// </summary>
        private readonly List<Vec3> m_pointsList = new List<Vec3>();

        /// <summary>
        /// 工作坐标轴X,相对于主零件的腹板法线方向
        /// </summary>
        public Vec3 BuildAxisX = Vec3.ZERO;
        /// <summary>
        /// 工作坐标轴Y,相对于主零件的翼缘法线方向
        /// </summary>
        public Vec3 BuildAxisY = Vec3.ZERO;
        /// <summary>
        /// 工作坐标轴Z,同时也是板面的法线
        /// </summary>
        public Vec3 BuildAxisZ = Vec3.ZERO;


        public override bool Init()
        {
            InitBuildAxis();
            InitPlatePro();
            return true;
        }

        /// <summary>
        /// 设置工作坐标系
        /// </summary>
        public void InitBuildAxis()
        {
            BuildAxisX = new Vec3(m_context.AxisX);
            //BuildAxisY = new Vec3(m_context.AxisY);
            BuildAxisZ = new Vec3(m_context.AxisZ);
            BuildAxisY = BuildAxisZ.Cross(BuildAxisX).Normalize();
            //改变工作坐标系
            var angle = m_context.ParamsHandler.GetDoubleValue(ParamNames.PARAMETERS_BOLT_GROUP_ROTATION_ANGLE);
            BuildAxisX = PointHelper.RotateVectorInPlane(BuildAxisX, BuildAxisZ, -angle);
            BuildAxisY = PointHelper.RotateVectorInPlane(BuildAxisY, BuildAxisZ, -angle);
        }

        /// <summary>
        /// 创建板
        /// </summary>
        public void CreateBasePlate()
        {
            HandlerBuildPointsCorner();
            var points = m_pointsList.ToArray();
            m_basePlate = m_context.TargetComp.CreateSteelPlate(points, m_thickness, Define.ObjectSide.SIDE_MIDDLE, true);
            if (m_basePlate == null) {
                return;
            }
            var dot = BuildAxisZ.Dot(m_basePlate.WorldMatrix.Z);
            m_basePlate.OffsetSide = dot >= 0 ? Define.ObjectSide.SIDE_FRONT : Define.ObjectSide.SIDE_BACK;
            m_pub.m_basePlatePro.ApplyToPart(m_basePlate);
        }

        /// <summary>
        /// 处理基板构建点的折角
        /// </summary>
        private void HandlerBuildPointsCorner()
        {
            var cord = new CoordinateSystem(m_buildBasePlatePoint,BuildAxisX,BuildAxisY,BuildAxisZ);
            var pointA = new Vec3(cord.GeneratePoint(-m_width/2 + m_width_1,m_height/2,0.0d));
            var pointB = new Vec3(cord.GeneratePoint(-m_width/2, m_height/2-m_height_1, 0.0d));
            var points1 = HandlerCornerPoints(1,pointA,pointB,cord);
            if (Equals(m_width_1, 0.0d) || Equals(m_height_1, 0.0d)) {
                points1 = new List<Vec3>() { pointA };
            }
            
            pointA = new Vec3(cord.GeneratePoint(-m_width / 2, -m_height / 2 + m_height_2, 0.0d));
            pointB = new Vec3(cord.GeneratePoint(-m_width /2  + m_width_2,-m_height/2, 0.0d));
            var points2 = HandlerCornerPoints(2,pointA,pointB,cord);
            if (Equals(m_width_2, 0.0d) || Equals(m_height_2, 0.0d)) {
                points2 = new List<Vec3>() { pointA };
            }

            pointA = new Vec3(cord.GeneratePoint(m_width / 2 - m_width_3, -m_height / 2, 0.0d));
            pointB = new Vec3(cord.GeneratePoint(m_width / 2, -m_height / 2 + m_height_3, 0.0d));
            var points3 = HandlerCornerPoints(3,pointA,pointB,cord);
            if (Equals(m_width_3, 0.0d) || Equals(m_height_3, 0.0d)) {
                points3 = new List<Vec3>() { pointA };
            }

            pointA = new Vec3(cord.GeneratePoint(m_width / 2, m_height / 2 - m_height_4, 0.0d));
            pointB = new Vec3(cord.GeneratePoint(m_width / 2 - m_width_4, m_height / 2, 0.0d));
            var points4 = HandlerCornerPoints(4,pointA,pointB,cord);
            if (Equals(m_width_4, 0.0d) || Equals(m_height_4, 0.0d)) {
                points4 = new List<Vec3>() { pointA };
            }
            m_pointsList.AddRange(points1);
            m_pointsList.AddRange(points2);
            m_pointsList.AddRange(points3);
            m_pointsList.AddRange(points4);
        }

        /// <summary>
        /// 根据底角标号判断是否需要方切，如果需要则增加生成点，按序生成对应的点列表
        /// </summary>
        /// <param name="cutCornerNum"></param>
        /// <param name="point1"></param>
        /// <param name="point2"></param>
        /// <param name="coord"></param>
        /// <returns></returns>
        private List<Vec3> HandlerCornerPoints(int cutCornerNum, Vec3 point1, Vec3 point2, CoordinateSystem coord)
        {
            Vec3 pointInsert;
            //判断需不需要方角
            if (!m_squareCutCorners.Contains(cutCornerNum)) return new List<Vec3>() { point1, point2 };
            switch (cutCornerNum)
            {
                case 1:
                    pointInsert = 
                        new Vec3(coord.GeneratePoint(-m_width / 2 + m_width_1, m_height / 2 - m_height_1,0.0d));
                    break;
                case 2:
                    pointInsert =
                        new Vec3(coord.GeneratePoint(-m_width / 2 + m_width_2, -m_height / 2 + m_height_2, 0.0d));
                    break;
                case 3:
                    pointInsert =
                        new Vec3(coord.GeneratePoint(m_width / 2 - m_width_3, -m_height / 2 + m_height_3, 0.0d));
                    break;
                case 4:
                    pointInsert =
                        new Vec3(coord.GeneratePoint(m_width / 2 - m_width_4, m_height/2- m_height_4, 0.0d));
                    break;
                default:
                    return new List<Vec3>() { point1, point2 };
            }
            return new List<Vec3>() { point1, pointInsert, point2 };
        }

        public override bool Prepare()
        {
            PrepareData();
            return true;
        }

        /// <summary>
        /// 准备数据,处理基板的构建点、厚度、宽度、高度、折角数据、方切角编号
        /// </summary>
        private void PrepareData()
        {
            var handler = m_context.ParamsHandler;
            var worldMatrix = m_context.MainPart.WorldMatrix;
            m_buildBasePlatePoint = PointHelper.MoveTo(m_context.DetailCentrePt, m_pub.BasePlateBtmPl);
            //处理构建点的x、y偏移
            var buildPointOffSetX = handler.GetDoubleValue(ParamNames.PARAMETERS_BASE_PLATE_HORIZONTAL_OFFSET_FROM_THE_COLUMN_CENTER);
            var buildPointOffSetY = handler.GetDoubleValue(ParamNames.PARAMETERS_BASE_PLATE_VERTICAL_OFFSET_FROM_THE_COLUMN_CENTER);
            PointHelper.MoveRef(ref m_buildBasePlatePoint,-worldMatrix.X,buildPointOffSetX);
            PointHelper.MoveRef(ref m_buildBasePlatePoint,worldMatrix.Y,buildPointOffSetY);
            PointHelper.MoveRef(ref m_buildBasePlatePoint,BuildAxisX,m_pub.BasePlateBoltOffX); 
            PointHelper.MoveRef(ref m_buildBasePlatePoint,BuildAxisY,m_pub.BasePlateBoltOffY); 
            
            m_buildTopPoint = m_buildBasePlatePoint;
            PointHelper.MoveToRef(ref m_buildTopPoint,m_pub.BasePlateTopPl);
            
            //准备基板的折角数据
            m_width_1 = handler.GetDoubleValue(ParamNames.PIC_WIDTH_OF_BASE_PLATE_CHAMFER_1);
            m_width_2 = handler.GetDoubleValue(ParamNames.PIC_WIDTH_OF_BASE_PLATE_CHAMFER_2);
            m_width_3 = handler.GetDoubleValue(ParamNames.PIC_WIDTH_OF_BASE_PLATE_CHAMFER_3);
            m_width_4 = handler.GetDoubleValue(ParamNames.PIC_WIDTH_OF_BASE_PLATE_CHAMFER_4);
            m_height_1 = handler.GetDoubleValue(ParamNames.PIC_HEIGHT_OF_BASE_PLATE_CHAMFER_1);
            m_height_2 = handler.GetDoubleValue(ParamNames.PIC_HEIGHT_OF_BASE_PLATE_CHAMFER_2);
            m_height_3 = handler.GetDoubleValue(ParamNames.PIC_HEIGHT_OF_BASE_PLATE_CHAMFER_3);
            m_height_4 = handler.GetDoubleValue(ParamNames.PIC_HEIGHT_OF_BASE_PLATE_CHAMFER_4);
            //确定需要方切割的底板角编号
            var nums = handler.GetIntArrayValue(ParamNames.PARAMETERS_SQUARE_CUT_CORNERS_1TO4);
            foreach (var num in nums)
            {
                if (num == 1 || num == 2 || num == 3 || num == 4) {
                    m_squareCutCorners.Add(num);
                }
            }
        }

        /// <summary>
        /// 处理板的厚度、宽度、高度
        /// </summary>
        private void InitPlatePro()
        {
            m_thickness = m_pub.BasePlateThickness;//厚度无需自适应
            m_width = m_pub.BasePlateWidth;
            m_height = m_pub.BasePlateHeight;
        }

        /// <summary>
        /// 自适应宽度
        /// </summary>
        /// <returns></returns>
        private double AdaptiveWidth()
        {
            var profile = m_context.MainPart.Profile;
            var max = Math.Max(profile.ShapeWidth, profile.ShapeHeight);
            //return CalculationHelper.CalculationOfBasePlateWidthAndHeight( BeamPlanesHelper.GetColumnCrossSection_Max(m_context.MainPlanes));
            return CalculationHelper.CalculationOfBasePlateWidthAndHeight(max);
        }
    }
}