﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    public class BuildStiffener:RunningBaseClass
    {

        /// <summary>
        /// 加劲肋的数据集
        /// </summary>
        private readonly Dictionary<int, StiffenerData> m_allStiffenerData = new Dictionary<int, StiffenerData>
        {
            {1,StiffenerData.ZERO}, {2,StiffenerData.ZERO}, {3,StiffenerData.ZERO}, {4,StiffenerData.ZERO},
            {5,StiffenerData.ZERO}, {6,StiffenerData.ZERO}, {7,StiffenerData.ZERO}, {8,StiffenerData.ZERO},
        };

        /// <summary>
        /// 能被构建的加劲肋编号
        /// </summary>
        private readonly HashSet<int> m_canBuildStiffenerNums = new HashSet<int> { };

        private PlateProperties m_plateHelper1245;
        private PlateProperties m_plateHelper36;
        private PlateProperties m_plateHelper7;
        private PlateProperties m_plateHelper8;

        public BuildStiffener(ComponentInstanceContext context) : base(context)
        {
        }

        private readonly ParamData AllParam = new ParamData();

        private class ParamData
        {
            public int m_stiffenerNums3;
            public double[] m_stiffenerSpacing3;
            public int m_stiffenerNums6;
            public double[] m_stiffenerSpacing6;
            public int m_stiffenerNums7;
            public double[] m_stiffenerSpacing7;
            public int m_stiffenerNums8;
            public double[] m_stiffenerSpacing8;
        }

        public override bool Init()
        {
            var handler = m_context.ParamsHandler;
            AllParam.m_stiffenerNums3 = handler.GetIntValue(ParamNames.STIFFENER_3_NUMS);
            AllParam.m_stiffenerNums6 = handler.GetIntValue(ParamNames.STIFFENER_6_NUMS);
            AllParam.m_stiffenerNums7 = handler.GetIntValue(ParamNames.STIFFENER_7_NUMS);
            AllParam.m_stiffenerNums8 = handler.GetIntValue(ParamNames.STIFFENER_8_NUMS);
            AllParam.m_stiffenerSpacing3 = handler.GetDoubleArrayValue(ParamNames.STIFFENER_3_SPACING);
            AllParam.m_stiffenerSpacing6 = handler.GetDoubleArrayValue(ParamNames.STIFFENER_6_SPACING);
            AllParam.m_stiffenerSpacing7 = handler.GetDoubleArrayValue(ParamNames.STIFFENER_7_SPACING);
            AllParam.m_stiffenerSpacing8 = handler.GetDoubleArrayValue(ParamNames.STIFFENER_8_SPACING);
            AllParam.m_stiffenerSpacing3 = BoltsHelp.AdaptBoltSpacing(AllParam.m_stiffenerNums3,AllParam.m_stiffenerSpacing3, 50.0d);
            AllParam.m_stiffenerSpacing6 = BoltsHelp.AdaptBoltSpacing(AllParam.m_stiffenerNums6,AllParam.m_stiffenerSpacing6, 50.0d);
            AllParam.m_stiffenerSpacing7 = BoltsHelp.AdaptBoltSpacing(AllParam.m_stiffenerNums7,AllParam.m_stiffenerSpacing7, 50.0d);
            AllParam.m_stiffenerSpacing8 = BoltsHelp.AdaptBoltSpacing(AllParam.m_stiffenerNums8,AllParam.m_stiffenerSpacing8, 50.0d);
            return true;
        }

        public void Running()
        {
            PrepareStiffenerBuildNums();
            //准备好stiffenerHelper
            var stiffenerHelper = new StiffenerHelper();
            //遍历AllStiffenerData，构建加劲肋
            foreach (var stiffenerData in m_allStiffenerData)
            {
                var stiffenerId = stiffenerData.Key;
                //判断是否需要构建
                if (!m_canBuildStiffenerNums.Contains(stiffenerId)) { continue; }
                if (!CheckStiffenerNum(stiffenerId)) { continue; }
                
                PrepareStiffenerChamfer(stiffenerData.Value,stiffenerId);
                if (Equals(stiffenerData.Value.Thickness, 0.0d)){ continue;}
                //获取构建点，默认为编号0的控制点
                var buildPoint = GetBuildPoint(stiffenerId);
                if(buildPoint == Vec3.MAX){continue;}
                //x统一为-Z方向
                var stiffenerX = -m_context.AxisZ;
                
                //处理计算参照点的偏移
                var heightOffset = m_allStiffenerData[stiffenerId].Height;
                PointHelper.MoveRef(ref buildPoint,-stiffenerX,heightOffset);
                
                //处理UI参数的偏移
                var offsetDir = GetStiffenerOffsetDir(stiffenerId);
                var offsetLength = GetStiffenerOffsetLength(stiffenerId);
               PointHelper.MoveRef(ref buildPoint, offsetDir, offsetLength);
                
                //y要加劲肋编号对应面的法线
                var stiffenerY = GetStiffenerY(stiffenerId);
                if (stiffenerY == Vec3.ZERO) {continue;}
                
                //NOTE 增加3、6、7、8、的间距多加劲肋
                var spacings = new double[]{};
                spacings = GetStiffenerSpacings(stiffenerId, spacings);
                CreateStiffenerForSpacing(buildPoint, offsetDir, spacings, stiffenerHelper, stiffenerX, stiffenerY, stiffenerData);
            }
        }

        /// <summary>
        /// 检测对应ID的加劲肋数目
        /// </summary>
        /// <param name="stiffenerId"></param>
        /// <returns></returns>
        private bool CheckStiffenerNum(int stiffenerId)
        {
            var num = 0;
            if (stiffenerId == 3) {
                num = AllParam.m_stiffenerNums3;
            }else if (stiffenerId == 6) {
                num = AllParam.m_stiffenerNums6;
            }else if (stiffenerId == 7) {
                num = AllParam.m_stiffenerNums7;
            }else if (stiffenerId == 8) {
                num = AllParam.m_stiffenerNums8;
            } else {
                num = 1;
            }
            return num > 0;
        }

        private double[] GetStiffenerSpacings(int stiffenerId, double[] spacings)
        {
            if (stiffenerId == 3)
            {
                spacings = AllParam.m_stiffenerSpacing3;
            }else if (stiffenerId == 6)
            {
                spacings = AllParam.m_stiffenerSpacing6;
            }else if (stiffenerId == 7)
            {
                spacings = AllParam.m_stiffenerSpacing7;
            }else if (stiffenerId == 8)
            {
                spacings = AllParam.m_stiffenerSpacing8;
            }

            return spacings;
        }

        private void CreateStiffenerForSpacing(Vec3 buildPoint, Vec3 offsetDir, double[] spacings,
            StiffenerHelper stiffenerHelper, Vec3 stiffenerX, Vec3 stiffenerY, KeyValuePair<int, StiffenerData> stiffenerData)
        {
            PointHelper.MoveRef(ref buildPoint,-offsetDir,spacings.Sum()/2);
            CreateStiffener(stiffenerHelper, stiffenerX, stiffenerY, stiffenerData, buildPoint);
            foreach (var spacing in spacings)
            {
                PointHelper.MoveRef(ref buildPoint,offsetDir,spacing);
                CreateStiffener(stiffenerHelper, stiffenerX, stiffenerY, stiffenerData, buildPoint);
            }
        }

        private void CreateStiffener(StiffenerHelper stiffenerHelper, Vec3 stiffenerX, Vec3 stiffenerY,
            KeyValuePair<int, StiffenerData> stiffenerData, Vec3 buildPoint)
        {
            var steelPlate = m_context.TargetComp.CreateSteelPlate(
                stiffenerHelper.ReturnAllControlPoints(stiffenerX,stiffenerY,stiffenerData.Value,buildPoint,0),
                stiffenerData.Value.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
            if (steelPlate != null) {
                //处理属性值
                ProcessingAttributeValu(steelPlate,stiffenerData.Key);
                //创建焊缝
                m_context.Weld.BuildWeldSeamForStiffener(steelPlate, stiffenerHelper);
                //Console.WriteLine(@"stiffener:" + stiffenerData.Key + @" build success");
            }
        }

        private void ProcessingAttributeValu(SteelPlate steelPlate, int stiffenerDataKey)
        {
            if (stiffenerDataKey == 1 || stiffenerDataKey == 2 || stiffenerDataKey == 4 || stiffenerDataKey == 5)
            {
                m_plateHelper1245.ApplyToPart(steelPlate);
            }
            else if (stiffenerDataKey == 3 || stiffenerDataKey == 6)
            {
                m_plateHelper36.ApplyToPart(steelPlate);
            }else if (stiffenerDataKey == 7)
            {
                m_plateHelper7.ApplyToPart(steelPlate);
            }else if (stiffenerDataKey == 8)
            {
                m_plateHelper8.ApplyToPart(steelPlate);
            }
        }

        /// <summary>
        /// 获取对应编号加劲肋的偏移距离
        /// </summary>
        /// <param name="stiffenerNum"></param>
        /// <returns></returns>
        private double GetStiffenerOffsetLength(int stiffenerNum)
        {
            var holder = m_context.ParamsHandler;
            var ret = 0.0d;
            switch (stiffenerNum)
            {
               case 1:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_1_OFFSET);
                   break;
               case 2:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_2_OFFSET);
                   break;
               case 3:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_3_OFFSET);
                   break;
               case 4:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_4_OFFSET);
                   break;
               case 5:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_5_OFFSET);
                   break;
               case 6:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_6_OFFSET);
                   break;
               case 7:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_7_OFFSET);
                   break;
               case 8:
                   ret = holder.GetDoubleValue(ParamNames.STIFFENER_8_OFFSET);
                   break;
            }
            return ret;
        }

        /// <summary>
        /// 获取对应编号加劲肋的偏移方向
        /// </summary>
        /// <param name="stiffenerNum">加劲肋编号</param>
        /// <returns>偏移方向</returns>
        private Vec3 GetStiffenerOffsetDir(int stiffenerNum)
        {
            var worldMatrix = m_context.MainPart.WorldMatrix;
            var pls = m_context.MainPlanes;
            Plane dirPl = null;
            if(stiffenerNum == 1 ||stiffenerNum == 8 || stiffenerNum == 2)
            {
                dirPl = pls.TopFlange ?? pls.TopBound;
            }else if(stiffenerNum == 4 || stiffenerNum == 5 || stiffenerNum == 7)
            {
                dirPl = pls.BottomFlange ?? pls.BottomBound;
            }else if(stiffenerNum == 6)
            {
                dirPl = pls.FrontWeb ?? pls.FrontBound;
            }else if(stiffenerNum == 3)
            {
                dirPl = pls.BackWeb ?? pls.BackBound;
            }
            return dirPl?.Normal ?? Vec3.ZERO;
        }

        /// <summary>
        /// 准备需要构建的加劲肋的位置数据
        /// </summary>
        private void PrepareStiffenerBuildNums()
        {
            var handler= m_context.ParamsHandler;
            var arrayValue = handler.GetIntArrayValue(ParamNames.STIFFENER_POSITIONS_1TO8);
            foreach (var num in arrayValue)
            {
                m_canBuildStiffenerNums.Add(num);
            }
        }

        /// <summary>
        /// 获取对于编号的加劲肋构建点
        /// </summary>
        /// <param name="stiffenerNum"></param>
        /// <returns></returns>
        private Vec3 GetBuildPoint(int stiffenerNum)
        {
            Debug.Assert(stiffenerNum>0 && stiffenerNum <9,$"error stiffenerNum:{stiffenerNum}");
            var pls = m_context.MainPlanes;
            var ctlTopFlangePl = m_context.MainCtlTopFlangePl;
            var ctlBtmFlangePl = m_context.MainCtlBtmFlangePl;
            var ctlWebPl = m_context.MainCtlWebPl;
            
            var buildPt = Vec3.ZERO;
            var refPl = m_pub.BasePlateTopPl;
            
            switch (stiffenerNum)
            {
               case 1:
                   buildPt = refPl.Intersect(ctlBtmFlangePl, pls.BottomFlangeFront??pls.FrontBound);
                   break;
               case 2:
                   buildPt = refPl.Intersect(ctlTopFlangePl, pls.TopFlangeFront??pls.FrontBound);
                   break;
               case 3:
                   //NOTE 考虑用腹板的中心面还是用前后包围面的中心面。 //此处考虑为腹板中心面
                   buildPt = refPl.Intersect(pls.TopFlange??pls.TopBound, ctlWebPl);
                   break;
               case 4:
                   buildPt = refPl.Intersect(ctlTopFlangePl, pls.TopFlangeBack??pls.BackBound);
                   break;
               case 5:
                   buildPt = refPl.Intersect(ctlBtmFlangePl, pls.BottomFlangeBack??pls.BackBound);
                   break;
               case 6:
                   //NOTE 考虑用腹板的中心面还是用前后包围面的中心面。 //此处考虑为腹板中心面
                   buildPt = refPl.Intersect(pls.BottomBound, ctlWebPl);
                   break;
               case 7:
                   buildPt = refPl.Intersect(m_context.MainCentreTopBtmPl, pls.BackWeb??pls.BackBound);
                   break;
               case 8:
                   buildPt = refPl.Intersect(m_context.MainCentreTopBtmPl, pls.FrontWeb??pls.FrontBound);
                   break;
            }
            return new Vec3(buildPt);
        }

        /// <summary>
        /// 获取对应加劲肋的自身Y轴，也就是对应平面的法线
        /// </summary>
        /// <param name="stiffenerNum"></param>
        /// <returns></returns>
        private Vec3 GetStiffenerY(int stiffenerNum)
        {
            Debug.Assert(stiffenerNum>0 && stiffenerNum <9,$"error stiffenerNum:{stiffenerNum}");
            var planes = m_context.MainPlanes;
            if(planes == null){return Vec3.ZERO;}
            var dirPl = GetStiffenerPlaneY(stiffenerNum,planes);
            return dirPl?.Normal ?? Vec3.ZERO;

            Plane GetStiffenerPlaneY(int num,BeamPlanes pls)
            {
                switch (stiffenerNum)
                {
                    case 1:
                        return pls.BottomFlangeFront??pls.FrontBound;
                    case 2:
                        return pls.TopFlangeFront??pls.FrontBound;
                    case 3:
                        return pls.TopFlange??pls.TopBound;
                    case 4:
                        return pls.TopFlangeBack??pls.BackBound;
                    case 5:
                        return pls.BottomFlangeBack??pls.BackBound;
                    case 6:
                        return pls.BottomFlange??pls.BottomBound;
                    case 7:
                        return pls.BackWeb??pls.BackBound;
                    case 8:
                        return pls.FrontWeb??pls.FrontBound;
                }
                return null;
            }
        }


        ///  <summary>
        /// 自适应厚度
        ///  </summary>
        ///  <param name="stiffenerNum"></param>
        ///  <returns></returns>
        private double AdaptiveHThickness(int stiffenerNum)
        {
            var thickness = Math.Max(m_context.MainTopFlangeThk,m_context.MainBtmFlangeThk);
            thickness = Math.Max(thickness, m_context.MainWebThk);
            if (stiffenerNum == 2 || stiffenerNum == 4 || stiffenerNum == 3)
            {
                thickness = m_context.MainTopFlangeThk;
            }else if (stiffenerNum == 1 || stiffenerNum == 5 || stiffenerNum == 6)
            {
                thickness = m_context.MainBtmFlangeThk;
            }else if (stiffenerNum == 7 || stiffenerNum == 8)
            {
                thickness = m_context.MainWebThk;
            }
            return CalculationHelper.CalculationOfStiffenerThickness(thickness);
        }

        /// <summary>
        /// 自适应宽度
        /// </summary>
        /// <returns></returns>
        private double AdaptiveWidth()
        {
            //找到最小边缘距离
            //var minimumEdgeDistance = m_context.BasePlate.m_width - BeamPlanesHelper.GetColumnCrossSection_Max(m_context.MainPlanes);
            var maxCross = BeamPlanesHelper.GetColumnCrossSection_Max(m_context.MainPlanes);
            if (Equals(maxCross, double.MinValue)) {
                maxCross = Math.Max(m_context.MainPartHeight, m_context.MainPartWidth);
            }
            var minimumEdgeDistance = m_pub.BasePlateWidth - maxCross;
            return CalculationHelper.CalculateTheWidthOfStiffeners(minimumEdgeDistance/2);
        }

        /// <summary>
        /// 自适应高度
        /// </summary>
        /// <param name="stiffenerData"></param>
        /// <returns></returns>
        private double AdaptiveHeight(StiffenerData stiffenerData)
        {
            return 1.75 * stiffenerData.Width;
        }

        /// <summary>
        /// 准备加劲肋的折角数据,和长、宽、厚度。
        /// </summary>
        /// <param name="stiffenerData"></param>
        /// <param name="stiffenerNum"></param>
        private void PrepareStiffenerChamfer(StiffenerData stiffenerData,int stiffenerNum)
        {
            Debug.Assert(stiffenerNum>0 && stiffenerNum <9,$"error stiffenerNum:{stiffenerNum}");
            //需要判断该控件有没有被输入
            var handler = m_context.ParamsHandler;
            m_plateHelper1245 = new PlateProperties();
            m_plateHelper36 = new PlateProperties();
            m_plateHelper7 = new PlateProperties();
            m_plateHelper8 = new PlateProperties();
            var help  = new PlateProperties();
            //var data = handler.GetDObjectValue(ParamNames.PARTS_STIFFENERS_1_2_4_5);
            //plate.SetProps(data);
            if (stiffenerNum == 1 || stiffenerNum == 2 || stiffenerNum == 4 || stiffenerNum == 5)
            {
                m_plateHelper1245.SetProps(handler.GetDObjectValue(ParamNames.PARTS_STIFFENERS_1_2_4_5));
                stiffenerData.TopHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_HORIZONTAL_CHAMFER_FOR_STIFFENERS_1_2_4_5);
                stiffenerData.TopVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_VERTICAL_CHAMFER_FOR_STIFFENERS_1_2_4_5);
                stiffenerData.BottomHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_HORIZONTAL_CHAMFER_FOR_STIFFENERS_1_2_4_5);
                stiffenerData.BottomVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_VERTICAL_CHAMFER_FOR_STIFFENERS_1_2_4_5);
                help = m_plateHelper1245;
            }
            else if(stiffenerNum == 3|| stiffenerNum == 6)
            {
                m_plateHelper36.SetProps(handler.GetDObjectValue(ParamNames.PARTS_STIFFENERS_3_6));
                stiffenerData.TopHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_HORIZONTAL_CHAMFER_FOR_STIFFENERS_3_6);
                stiffenerData.TopVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_VERTICAL_CHAMFER_FOR_STIFFENERS_3_6);
                stiffenerData.BottomHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_HORIZONTAL_CHAMFER_FOR_STIFFENERS_3_6);
                stiffenerData.BottomVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_VERTICAL_CHAMFER_FOR_STIFFENERS_3_6);
                help = m_plateHelper36;
            }else if (stiffenerNum == 7)
            {
                m_plateHelper7.SetProps(handler.GetDObjectValue(ParamNames.PARTS_STIFFENERS_7));
                stiffenerData.TopHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_HORIZONTAL_CHAMFER_FOR_STIFFENERS_7);
                stiffenerData.TopVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_VERTICAL_CHAMFER_FOR_STIFFENERS_7);
                stiffenerData.BottomHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_HORIZONTAL_CHAMFER_FOR_STIFFENERS_7);
                stiffenerData.BottomVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_VERTICAL_CHAMFER_FOR_STIFFENERS_7);
                help = m_plateHelper7;
            }else if (stiffenerNum == 8)
            {
                m_plateHelper8.SetProps(handler.GetDObjectValue(ParamNames.PARTS_STIFFENERS_8));
                stiffenerData.TopHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_HORIZONTAL_CHAMFER_FOR_STIFFENERS_8);
                stiffenerData.TopVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_TOP_VERTICAL_CHAMFER_FOR_STIFFENERS_8);
                stiffenerData.BottomHorizontalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_HORIZONTAL_CHAMFER_FOR_STIFFENERS_8);
                stiffenerData.BottomVerticalChamfer = handler.GetDoubleValue(ParamNames.PIC_BOTTOM_VERTICAL_CHAMFER_FOR_STIFFENERS_8);
                help = m_plateHelper8;
            }
            stiffenerData.Thickness = Equals(help.Thickness, double.MaxValue) ? AdaptiveHThickness(stiffenerNum) : help.Thickness;
            stiffenerData.Width = Equals(help.Width, double.MaxValue) ? AdaptiveWidth() : help.Width;
            stiffenerData.Height = Equals(help.Height, double.MaxValue) ? AdaptiveHeight(stiffenerData) : help.Height;
        }
    }
}