﻿using System;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    public class PublicParamData:RunningBaseClass
    {
        public PublicParamData(ComponentInstanceContext context) : base(context)
        {
            m_context.PublicParamData = this;
        }

        /// <summary>
        /// 焊缝距离，实际上是柱子底部的切割距离
        /// </summary>
        public double m_weldGap = 0.0d;
        /// <summary>
        /// 抗剪键焊接位置
        /// </summary>
        public int parts_shear_key_profile_welded;

        /// <summary>
        /// 是否构件锚钉杆
        /// </summary>
        public bool m_buildAnchor = false;
        /// <summary>
        /// 锚钉杆弯钩的圆半径
        /// </summary>
        public double m_anchorRadius;
        /// <summary>
        /// 垫圈孔隙
        /// </summary>
        public double m_anchorWasherHoleTolerance;
        /// <summary>
        /// 基板的板属性
        /// </summary>
        public readonly PlateProperties m_basePlatePro = new PlateProperties();
        /// <summary>
        /// 灌浆层的厚度
        /// </summary>
        public double m_groutingThickness;
        /// <summary>
        /// 基板的底面
        /// </summary>
        public Plane BasePlateBtmPl { get; private set; }
        /// <summary>
        /// 基板的顶面
        /// </summary>
        public Plane BasePlateTopPl { get; private set; }
        /// <summary>
        /// 基板的厚度
        /// </summary>
        public double BasePlateThickness { get; private set; }
        /// <summary>
        /// 基板的宽度
        /// </summary>
        public double BasePlateWidth{ get; private set; }
        /// <summary>
        /// 基板的高度
        /// </summary>
        public double BasePlateHeight{ get; private set; }
        
        /// <summary>
        /// 基板由于螺栓导致的x方向偏移
        /// 依赖于螺栓的Init
        /// </summary>
        public double BasePlateBoltOffX { get; private set; }
        /// <summary>
        /// 基板由于螺栓导致的y方向偏移
        /// 依赖于螺栓的Init
        /// </summary>
        public double BasePlateBoltOffY { get; private set; }
        
        /// <summary>
        /// 下方垫片板的偏移值d
        /// </summary>
        public double LowerPlateOffsetD { get; private set; }
        
        public BeamProperties AnchorRodBeamPro { get; private set; }
        public double AnchorRodSize { get; private set; }
        
        /// <summary>
        /// 抗剪键的板属性
        /// </summary>
        public BeamProperties ShearKeyPro { get; private set; }
        public bool ShearKeyCanBuild { get; private set;}
        
        /// <summary>
        /// 螺栓的形状
        /// </summary>
        public int SelectBoltShape { get; private set; }

        public override bool Init()
        {
            var handler = m_context.ParamsHandler;
            AnchorRodBeamPro = new BeamProperties();
            AnchorRodBeamPro.SetProps(handler.GetDObjectValue(ParamNames.ANCHOR_ROD_PROFILE));
            AnchorRodSize = AnchorRodBeamPro.Profile == "" ? 20.0d : BuildAnchorRod.TryToAnalyzeTheBeamSection(AnchorRodBeamPro.Profile);
            LowerPlateOffsetD = handler.GetDoubleValue(ParamNames.
                ANCHOR_LENGTH_OF_THE_ANCHOR_ROD_BELOW_THE_EXTRA_PLATE);
            if (Equals(LowerPlateOffsetD, double.MaxValue))
                LowerPlateOffsetD = 2 * AnchorRodSize;
            
            m_weldGap = handler.GetDoubleValue(ParamNames.PIC_WELD_GAP);
            parts_shear_key_profile_welded = handler.GetIntValue(ParamNames.PARTS_SHEAR_KEY_PROFILE_WELDED);
            var select = handler.GetIntValue(ParamNames.ANCHOR_SELECT_CAN_BUILD);
            m_buildAnchor= !(select == 0 || select == 1);
            m_anchorRadius = handler.GetDoubleValue(ParamNames.ANCHOR_RADIUS_OF_THE_HOOK);
            m_anchorWasherHoleTolerance= handler.GetDoubleValue(ParamNames.ANCHOR_WASHER_HOLE_TOLERANCE);
            
            m_basePlatePro.SetProps(handler.GetDObjectValue(ParamNames.PARTS_BASE_PLATE));
            BasePlateThickness= m_basePlatePro.Thickness;//厚度无需自适应
            BasePlateWidth = Equals(m_basePlatePro.Width, double.MaxValue) ? AdaptiveBasePlateWidth() : m_basePlatePro.Width;
            BasePlateHeight = Equals(m_basePlatePro.Height, double.MaxValue) ? BasePlateWidth : m_basePlatePro.Height;
            
            m_groutingThickness = handler.GetDoubleValue(ParamNames.ANCHOR_GROUTING_THICKNESS);

            BasePlateBtmPl = m_context.DetailPl.Move(m_groutingThickness);
            BasePlateTopPl = BasePlateBtmPl.Move(m_basePlatePro.Thickness);
            BasePlateBtmPl = new Plane(BasePlateBtmPl.Origin, -BasePlateBtmPl.Normal);
            
            var data = handler.GetDObjectValue(ParamNames.PARTS_KEY_PROFILE);
            ShearKeyPro = new BeamProperties();
            ShearKeyPro.SetProps(data);
            ShearKeyCanBuild = ShearKeyPro.Profile != "";
            if (ShearKeyPro.Profile == null) { ShearKeyCanBuild = false; }

            SelectBoltShape = handler.GetIntValue(ParamNames.BOLTS_STAGGERING_OF_BOLTS);

            return true;
        }

        
        /// <summary>
        /// 自适应宽度
        /// </summary>
        /// <returns></returns>
        private double AdaptiveBasePlateWidth()
        {
            var profile = m_context.MainPart.Profile;
            var max = Math.Max(profile.ShapeWidth, profile.ShapeHeight);
            //return CalculationHelper.CalculationOfBasePlateWidthAndHeight( BeamPlanesHelper.GetColumnCrossSection_Max(m_context.MainPlanes));
            return CalculationHelper.CalculationOfBasePlateWidthAndHeight(max);
        }


        /// <summary>
        /// 缓存偏移
        /// </summary>
        /// <param name="off"></param>
        public void MoveBuildPoint_X(double off)
        {
            BasePlateBoltOffX += off;
        }
        
        /// <summary>
        /// 缓存偏移
        /// </summary>
        /// <param name="off"></param>
        public void MoveBuildPoint_Y(double off)
        {
            BasePlateBoltOffY += off;
        }

    }
}