﻿using System;
using System.Collections.Generic;
using U.S.Base_plate_1047_.Main;

namespace U.S.Base_plate_1047_.Running
{
    public interface IInit
    {
        /// <summary>
        /// 无依赖性的准备操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Init();
    }

    public interface IPrepare
    {
        /// <summary>
        /// 有依赖性的准备操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Prepare();
    }

    public interface IExecute
    {
        /// <summary>
        /// 执行操作，在所有准备结束后的操作
        /// 负责执行相关的Create函数
        /// </summary>
        /// <returns>是否成功</returns>
        bool Execute();
    }

    public class RunningBaseClass:IInit,IPrepare,IExecute
    {
        /// <summary>
        /// 缓存上下文
        /// </summary>
        protected readonly ComponentInstanceContext m_context;
        /// <summary>
        /// 公用的数据
        /// </summary>
        protected readonly PublicParamData m_pub;
        protected RunningBaseClass(ComponentInstanceContext context)
        {
            m_context = context;
            m_pub = m_context.PublicParamData;
        }

        public virtual bool Init() {
            return true;
        }

        public virtual bool Prepare() {
            return true;
        }
        public virtual bool Execute() {
            return true;
        }

    }

    public class CompositeCommand : IExecute
    {
        private readonly List<IExecute> m_commands = new List<IExecute>();

        public void AddCommand(IExecute command)
        {
            m_commands.Add(command);
        }
        
        public bool Execute()
        {
            foreach (var command in m_commands)
            {
                if (!command.Execute())
                {
                    Console.WriteLine(command.GetType().Name + @" exe fail");
                    return false;
                }
                Console.WriteLine(command.GetType().Name + @" exe Successful");
            }
            return true;
        }
    }


    /// <summary>
    /// 汇总所有的派生类执行
    /// </summary>
    public class RunningHandler
    {
        private readonly List<RunningBaseClass> m_run = new List<RunningBaseClass>();
        private readonly List<CompositeCommand> m_execute = new List<CompositeCommand>();

        public void Running(ComponentInstanceContext context)
        {
            var publicParamData = new PublicParamData(context);
            var buildWeld = new BuildWeld(context); 
            var buildGrouting= new BuildGrouting(context);
            var buildBasePlate = new BuildBasePlate(context);
            var buildBolts = new BuildBolts(context);
            var buildStiffener= new BuildStiffener(context); 
            var buildTemporaryPlateOpening = new BuildTemporaryPlateOpening(context,buildBasePlate);
            m_run.Add(publicParamData);
            m_run.Add(buildWeld);
            m_run.Add(buildStiffener);
            m_run.Add(buildGrouting);
            m_run.Add(buildBasePlate);
            m_run.Add(buildBolts);
            m_run.Add(buildTemporaryPlateOpening);
            RunInitAndPrepare();
            
            var cutMainColumn = new CutMainColumn(context);
            cutMainColumn.Running();
            buildBolts.Running();
            var groutHole = new BuildGroutHole(context);
            groutHole.Running();
            buildStiffener.Running();
            var buildAnchorRod = new BuildAnchorRod(context);
            buildGrouting.Running();
            buildAnchorRod.Running();
            buildTemporaryPlateOpening.Execute();
        }

        private void RunInitAndPrepare()
        {
            foreach (var run in m_run)
            {
                if (run.Init()) continue;
#if DEBUG
                Console.WriteLine(run.GetType().Name + @" init fail");
#endif
                return;
            }
#if DEBUG
            Console.WriteLine(@"all init success");
#endif
            foreach (var run in m_run)
            {
                if (run.Prepare()) continue;
#if DEBUG
                Console.WriteLine(run.GetType().Name + @" prepare fail");
#endif
                return;
            }
#if DEBUG
            Console.WriteLine(@"all prepare success");
#endif
        }
    }

}