﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using LoongCAD.Geometry;

namespace U.S.Base_plate_1047_.Tool
{
    /// <summary>
    /// 锚钉杆数据
    /// </summary>
    public class AnchorRodData
    {
        /// <summary>
        /// 锚钉杆的截面型号
        /// </summary>
        public string RodProFile { get; set; }

        /// <summary>
        /// 锚钉杆的长度
        /// </summary>
        public double LengthOfTheAnchorRod
        {
            get => m_lengthOfTheAnchorRod;
            set
            {
                m_lengthOfTheAnchorRod = value;
                LengthOfTheAnchorRodBelowTheBasePlate =
                    m_lengthOfTheAnchorRod - m_lengthOfTheAnchorRodAboveTheBasePlate;
            }
        }

        /// <summary>
        /// 锚钉杆在底板以上的长度,以底板顶面为参考
        /// </summary>
        public double LengthOfTheAnchorRodAboveTheBasePlate
        {
            get => m_lengthOfTheAnchorRodAboveTheBasePlate;
            set
            {
                m_lengthOfTheAnchorRodAboveTheBasePlate = value;
                m_lengthOfTheAnchorRod += value;
            }
        }

        /// <summary>
        /// 锚钉杆在底板以下的长度,以底板顶面为参考
        /// </summary>
        public double LengthOfTheAnchorRodBelowTheBasePlate { get; private set; }

        /// <summary>
        /// 锚钉杆的水平弯折长度
        /// </summary>
        public double HorizontalBendingDistance { get; set; }
        /// <summary>
        /// 锚钉杆的垂直弯折长度
        /// </summary>
        public double VerticalBendingDistance { get; set; }

        /// <summary>
        /// 锚钉杆的类型
        /// </summary>
        public int RodType {get;set;}
        
        /// <summary>
        /// 锚钉杆弯钩方向的类别
        /// </summary>
        public int HookDirType {get;set;}
        
        /// <summary>
        /// 零值
        /// </summary>
        public static readonly AnchorRodData ZERO = new AnchorRodData();

        private double m_lengthOfTheAnchorRod;
        private double m_lengthOfTheAnchorRodAboveTheBasePlate;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        public AnchorRodData()
        {
            RodProFile = "D20";
            LengthOfTheAnchorRod = 500.0d;
            LengthOfTheAnchorRodAboveTheBasePlate = 50.0d;
            LengthOfTheAnchorRodBelowTheBasePlate = 450.0d;
            HorizontalBendingDistance = 0.0d;
            VerticalBendingDistance = 0.0d;
            RodType = 1;
            HookDirType = 1;
        }
    }
    
    /// <summary>
    /// 锚钉杆生成器
    /// </summary>
    public class AnchorRodHelper
    {

        /// <summary>
        /// 锚钉杆数据
        /// </summary>
        public AnchorRodData m_anchorRodData = null;

        /// <summary>
        /// 锚钉杆控制点
        /// </summary>
        private readonly Dictionary<int, Vec3> m_controlPoints = new Dictionary<int, Vec3>()
        {
            {0,Vec3.ZERO},{1,Vec3.ZERO},{2,Vec3.ZERO},{3,Vec3.ZERO},
        };
        /// <summary>
        /// 0号控制点，底板往上LengthOfTheAnchorRodAboveTheBasePlate 的距离 
        /// </summary>
        public Vec3 ControlPoint0 { get => m_controlPoints[0]; set=>m_controlPoints[0] = value; }
        /// <summary>
        /// 1号控制点，底板往下LengthOfTheAnchorRodBelowTheBasePlate 的距离 
        /// </summary>
        public Vec3 ControlPoint1 { get => m_controlPoints[1]; set=>m_controlPoints[1] = value; }
        /// <summary>
        /// 2号控制点，位底板往下LengthOfTheAnchorRodBelowTheBasePlate 的距离，
        /// 同时往+Y方向偏移距离HorizontalBendingDistance
        /// </summary>
        public Vec3 ControlPoint2 { get => m_controlPoints[2]; set=>m_controlPoints[2] = value; }
        /// <summary>
        /// 3号控制点，位于底板往下LengthOfTheAnchorRodBelowTheBasePlate的距离，
        /// 同时往+Y方向偏移距离HorizontalBendingDistance,
        /// 再往-x方向偏移距离VerticalBendingDistance
        /// </summary>
        public Vec3 ControlPoint3 { get => m_controlPoints[3]; set=>m_controlPoints[3] = value; }
        /// <summary>
        /// 3号控制点，位于底板顶面与锚钉杆相交处，
        /// 软件中就是螺栓的点，便于计算偏移
        /// </summary>
        public Vec3 ControlPoint4 { get => m_controlPoints[4]; set=>m_controlPoints[4] = value; }
        
        /// <summary>
        /// 工作坐标系X,锚钉杆的生成方向
        /// </summary>
        private Vec3 m_axisX = Vec3.ZERO;
        /// <summary>
        /// 工作坐标系Y,锚钉杆的弯折方向
        /// </summary>
        private Vec3 m_axisY = Vec3.ZERO;
        /// <summary>
        /// 偏移量，以0号点为原点。记录每个点到0号点的偏移量。
        /// </summary>
        private readonly Vec3[] m_offSets = new Vec3[6];
        /// <summary>
        /// 构造函数 
        /// </summary>
        public AnchorRodHelper(){}

        /// <summary>
        /// 设置x,y轴的方向，并计算偏移值
        /// </summary>
        /// <param name="xAxisDirection"></param>
        /// <param name="yAxisDirection"></param>
        /// <param name="anchorRodData"></param>
        public void SetAxisDirection_XY(Vec3 xAxisDirection, Vec3 yAxisDirection,AnchorRodData anchorRodData)
        {
            m_axisX = xAxisDirection;
            m_axisY = yAxisDirection;
            var dot = xAxisDirection.Dot(yAxisDirection);
            Debug.Assert(Math.Abs(dot - 0) < 0.05, "Wrong x axis or y axis: the x axis is not perpendicular to the y axis.");

            m_offSets[0] = -m_axisX * anchorRodData.LengthOfTheAnchorRodAboveTheBasePlate;
            m_offSets[1] = m_axisX * anchorRodData.LengthOfTheAnchorRodBelowTheBasePlate;
            m_offSets[2] = m_axisX * anchorRodData.LengthOfTheAnchorRodBelowTheBasePlate + m_axisY*anchorRodData.HorizontalBendingDistance;
            m_offSets[3] = m_axisX * (anchorRodData.LengthOfTheAnchorRodBelowTheBasePlate-anchorRodData.VerticalBendingDistance) + m_axisY*anchorRodData.HorizontalBendingDistance;
            m_offSets[4] = Vec3.ZERO;
        }

        /// <summary>
        /// 计算控制点
        /// </summary>
        /// <param name="axisX">锚钉杆的生成方向</param>
        /// <param name="axisY">锚钉杆的弯折方向</param>
        /// <param name="anchorRodData">锚钉杆的数据</param>
        /// <param name="worldPos">锚钉杆的生成参考点</param>
        /// <param name="pointNum">控制点编号</param>
        public void CalculateControlPoint(Vec3 axisX, Vec3 axisY, AnchorRodData anchorRodData, Vec3 worldPos,int pointNum = 4)
        {
           SetAxisDirection_XY(axisX,axisY,anchorRodData);
           m_anchorRodData = anchorRodData;
           var n = m_offSets.Length;
           for (var i = 0; i < n; ++i)
           {
               var relativeOffset = m_offSets[i] - m_offSets[pointNum];
               var controlPointPos = worldPos + relativeOffset;
               m_controlPoints[i] = controlPointPos;
           }
        }

        /// <summary>
        /// 返回所有控制点
        /// </summary>
        /// <returns></returns>
        public Vec3[] ReturnAllControlPointsOfAnchorRod()
        {
            var type = m_anchorRodData.RodType;
            switch (type)
            {
                case 2:
                    return new[]{ControlPoint0,ControlPoint1,ControlPoint2};
                case 3:
                    return new[]{ControlPoint0,ControlPoint1,ControlPoint2,ControlPoint3};
                default:
                    return new[]{ControlPoint0,ControlPoint1};
            }
        }

        
    }
}