﻿using System;
using System.Diagnostics;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;

namespace U.S.Base_plate_1047_.Tool
{
    public static class BeamPlanesHelper
    {
        
        /// <summary>
        /// 获取腹板的中心面
        /// 腹板面移动腹板厚度的一半
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static Plane GetControlPlaneInWeb(BeamPlanes beamPlanes)
        {
            if (beamPlanes == null)
                return null;
            if (beamPlanes.FrontWeb != null)
            {
                return beamPlanes.FrontWeb.Move(-(GetWebThickness(beamPlanes)/2));
            }
            else
            {
                return PlaneHelper.GetMidPlane(beamPlanes.FrontBound, beamPlanes.BackBound);
            }
            
        }
        
        /// <summary>
        /// 获取顶底翼缘面之间的中心面,
        /// 翼缘面移动顶底距离的一半
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static Plane GetControlPlaneTopToBottom(BeamPlanes beamPlanes)
        {
            if (beamPlanes == null)
                return null;
            if (beamPlanes.TopFlange != null)
            {
                var off = GetColumnCrossSectionTopToBottom_H(beamPlanes);
                if (Equals(off, double.MinValue)) {
                    return PlaneHelper.GetMidPlane(beamPlanes.TopBound, beamPlanes.BottomBound);
                }
                return beamPlanes.TopFlange.Move(-off/2);
            }
            else
            {
                return PlaneHelper.GetMidPlane(beamPlanes.TopBound, beamPlanes.BottomBound);
            }
        }


        /// <summary>
        /// 获取腹板厚度
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static double GetWebThickness(BeamPlanes beamPlanes)
        {
            if (beamPlanes != null) return GetTwoPlaneDist(beamPlanes.FrontWeb, beamPlanes.FrontWebInside);
            Debug.Assert(false,"beamPlanes is bull");
            return 0.0d;
        }
        
        /// <summary>
        /// 获取上翼缘的厚度
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static double GetTopFlangeThickness(BeamPlanes beamPlanes)
        {
            if (beamPlanes != null) return GetTwoPlaneDist(beamPlanes.TopFlange, beamPlanes.TopFlangeInside);
            Debug.Assert(false,"beamPlanes is null");
            return 0.0;
        }

        /// <summary>
        /// 获得下翼缘的厚度
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static double GetBottomFlangeThickness(BeamPlanes beamPlanes)
        {
            if (beamPlanes != null) return GetTwoPlaneDist(beamPlanes.BottomFlange, beamPlanes.BottomFlangeInside);
            Debug.Assert(false,"beamPlanes is null");
             return 0.0;
        }

        /// <summary>
        /// 获得两个面之间的距离，如果两个面不平行则返回距离为0
        /// </summary>
        /// <param name="plane1"></param>
        /// <param name="plane2"></param>
        /// <returns></returns>
        private static double GetTwoPlaneDist(Plane plane1, Plane plane2)
        {
            if (plane1 == null || plane2 == null)
            {
                return 0.0;
            }
            if(plane1.Normal.IsParallelTo(plane2.Normal))
            {
                return plane1.Dist(plane2.Origin);
            }
            return 0.0;
        }

        /// <summary>
        /// 获得尺寸界面的单边最大值。
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        public static double GetColumnCrossSection_Max(BeamPlanes beamPlanes)
        {
            
            var maxB = GetColumnCrossSectionFrontToBack_MaxB(beamPlanes);
            bool isInvalidMaxB = Equals(maxB, double.MinValue);
            var maxH = GetColumnCrossSectionTopToBottom_H(beamPlanes);
            bool isInvalidMaxH = Equals(maxH, double.MinValue);
            if (isInvalidMaxB && isInvalidMaxH)
                return double.MinValue;
            if (isInvalidMaxB)
                return maxH;
            if (isInvalidMaxH)
                return maxB;
            return Math.Max(maxB,maxH);
        }

        /// <summary>
        /// 获得尺寸截面的单边最小值。
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        public static double GetColumnCrossSection_Min(BeamPlanes beamPlanes)
        {
            var minB = GetColumnCrossSectionFrontToBack_MinB(beamPlanes);
            bool isInvalidMinB = Equals(minB, double.MinValue);
            var minH = GetColumnCrossSectionTopToBottom_H(beamPlanes);
            bool isInvalidMinH = Equals(minH, double.MinValue);
            if (isInvalidMinB && isInvalidMinH)
                return double.MinValue;
            if (isInvalidMinB)
                return minH;
            if (isInvalidMinH)
                return minB;
            return Math.Min(minB, minH);
        }

        /// <summary>
        /// 获得顶底的尺寸单边距离,两翼缘的外边缘相隔距离,h
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        public static double GetColumnCrossSectionTopToBottom_H(BeamPlanes beamPlanes)
        {
            if (beamPlanes.TopFlange== null || beamPlanes.BottomFlange == null) {
                return double.MinValue;
            }
            return GetTwoPlaneDist(beamPlanes.TopFlange, beamPlanes.BottomFlange);
        }

        /// <summary>
        /// 获得截面的单边最大尺寸值,Max(b1,b2)
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        public static double GetColumnCrossSectionFrontToBack_MaxB(BeamPlanes beamPlanes)
        {
            var b1 = GetColumnCrossSectionFrontToBackForTopFlange(beamPlanes);
            bool isInvalidB1 = Equals(b1, double.MinValue);
            var b2 = GetColumnCrossSectionFrontToBackForBottomFlange(beamPlanes);
            bool isInvalidB2 = Equals(b2, double.MinValue);
            
            if (isInvalidB1 && isInvalidB2)
                return double.MinValue;
            if (isInvalidB1)
                return b2;
            if (isInvalidB2)
                return b1;
            return Math.Max(b1, b2);
        }

        /// <summary>
        /// 获取截面的单边最小尺寸值,Min(b1,b2)
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        private static double GetColumnCrossSectionFrontToBack_MinB(BeamPlanes beamPlanes)
        {
            var b1 = GetColumnCrossSectionFrontToBackForTopFlange(beamPlanes);
            bool isInvalidB1 = Equals(b1, double.MinValue);
            var b2 = GetColumnCrossSectionFrontToBackForBottomFlange(beamPlanes);
            bool isInvalidB2 = Equals(b2, double.MinValue);
            
            if (isInvalidB1 && isInvalidB2)
                return double.MinValue;
            if (isInvalidB1)
                return b2;
            if (isInvalidB2)
                return b1;
            return Math.Min(b1, b2);
        }

        /// <summary>
        /// 获得顶翼缘的前后尺寸长度,b1
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        private static double GetColumnCrossSectionFrontToBackForTopFlange(BeamPlanes beamPlanes)
        {
            if (beamPlanes.TopFlangeFront == null || beamPlanes.TopFlangeBack == null) {
                return double.MinValue;
            }
            return GetTwoPlaneDist(beamPlanes.TopFlangeFront, beamPlanes.TopFlangeBack);
        }
        
        /// <summary>
        /// 获得底翼缘的前后尺寸长度,b2
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <returns>可能返回double.MinValue</returns>
        private static double GetColumnCrossSectionFrontToBackForBottomFlange(BeamPlanes beamPlanes)
        {
            if (beamPlanes.BottomFlangeFront == null || beamPlanes.BottomFlangeBack == null) {
                return double.MinValue;
            }
            return GetTwoPlaneDist(beamPlanes.BottomFlangeFront, beamPlanes.BottomFlangeBack);
        }
    }
}