﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;

namespace U.S.Base_plate_1047_.Tool
{
    /// <summary>
    /// 解析面的枚举值
    /// </summary>
    public enum EBeamPlaneType 
    {
        HeadBound, TailBound,  
        TopBound,BottomBound,
        FrontBound,BackBound,
        TopFlange,TopFlangeInside,TopFlangeFront,TopFlangeBack,
        BottomFlange,BottomFlangeInside,BottomFlangeFront,BottomFlangeBack,
        FrontWeb,FrontWebInside,BackWeb,BackWebInside,
    }
    public static class PlaneHelper
    {

        public static readonly List<EBeamPlaneType> BOUND_PLANE_TBFB = new List<EBeamPlaneType>()
        {
            EBeamPlaneType.TopBound,EBeamPlaneType.BottomBound,
            EBeamPlaneType.FrontBound,EBeamPlaneType.BackBound,
        };
        
        /// <summary>
        /// 获得两个面之间的距离，如果两个面不平行则返回距离为0
        /// </summary>
        /// <param name="plane1"></param>
        /// <param name="plane2"></param>
        /// <returns></returns>
        public static double GetTwoPlaneDist(Plane plane1, Plane plane2)
        {
            if (plane1 == null || plane2 == null)
            {
                Console.WriteLine(@"plane1 or plane2 is null");
                return 0.0;
            }
            if(plane1.Normal.IsParallelTo(plane2.Normal))
            {
                return plane1.Dist(plane2.Origin);
            }
            Console.WriteLine(@"The two planes are not parallel, so the distance cannot be determined.");
            return 0.0;
        }

        /// <summary>
        /// 返回两个平行面之间的中间面，法线同plane1
        /// </summary>
        /// <param name="plane1"></param>
        /// <param name="plane2"></param>
        /// <returns></returns>
        public static Plane GetMidPlane(Plane plane1, Plane plane2)
        {
            if (plane1 == null )
            {
                if (plane2 == null)
                {
                    Console.WriteLine(@"plane1 and plane2 is null");
                    return null;
                }
                Console.WriteLine(@"plane1 is null");
                return new Plane(plane2);
            }
            if (plane2 == null) {
                Console.WriteLine(@"plane2 is null");
                return new Plane(plane1);
            }
                        
            if(plane1.Normal.IsParallelTo(plane2.Normal))
            {
                var dist =  plane1.SignedDist(plane2.Origin);
                return plane1.Move(dist/2);
            }
                        
            Console.WriteLine(@"The two planes are not parallel, so the distance cannot be determined.");
            return null;
        }
        

        /// <summary>
        /// 获取两个面的距离
        /// </summary>
        /// <param name="type1"></param>
        /// <param name="type2"></param>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static double GetTwoPlaneDist(EBeamPlaneType type1, EBeamPlaneType type2, BeamPlanes beamPlanes)
        {
            return GetTwoPlaneDist(GetBeamPlane(type1, beamPlanes), GetBeamPlane(type2, beamPlanes));
        }


        /// <summary>
        /// 检查解析面的存在性
        /// </summary>
        /// <param name="beamPlanes"></param>
        public static void CheckBeamPlane(BeamPlanes beamPlanes)
        {
            if (beamPlanes == null)
                return;
            foreach (EBeamPlaneType value in Enum.GetValues(typeof(EBeamPlaneType)))
            {
                var plane = GetBeamPlane(value, beamPlanes);
                if (plane != null)
                {
                    PrintPlanes(GetBeamPlane(value,beamPlanes),GetBeamPlaneName(value));
                }
            }   
        }


        /// <summary>
        /// 面解析的枚举值返回字符串
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static string GetBeamPlaneName(EBeamPlaneType type)
        {
            string name;
            switch (type)
            {
                case EBeamPlaneType.HeadBound:
                    name = "HeadBound"; break;
                case EBeamPlaneType.TailBound:
                    name = "TailBound"; break;
                case EBeamPlaneType.TopBound:
                    name = "TopBound"; break;
                case EBeamPlaneType.BottomBound:
                    name = "BottomBound"; break;
                case EBeamPlaneType.FrontBound:
                    name = "FrontBound"; break;
                case EBeamPlaneType.BackBound:
                    name = "BackBound"; break;
                case EBeamPlaneType.TopFlange:
                    name = "TopFlange"; break;
                case EBeamPlaneType.TopFlangeInside:
                    name = "TopFlangeInside"; break;
                case EBeamPlaneType.TopFlangeFront:
                    name = "TopFlangeFront"; break;
                case EBeamPlaneType.TopFlangeBack:
                    name = "TopFlangeBack"; break;
                case EBeamPlaneType.BottomFlange:
                    name = "BottomFlange"; break;
                case EBeamPlaneType.BottomFlangeInside:
                    name = "BottomFlangeInside"; break;
                case EBeamPlaneType.BottomFlangeFront:
                    name = "BottomFlangeFront"; break;
                case EBeamPlaneType.BottomFlangeBack:
                    name = "BottomFlangeBack"; break;
                case EBeamPlaneType.FrontWeb:
                    name = "FrontWeb"; break;
                case EBeamPlaneType.FrontWebInside:
                    name = "FrontWebInside"; break;
                case EBeamPlaneType.BackWeb:
                    name = "BackWeb"; break;
                case EBeamPlaneType.BackWebInside:
                    name = "BackWebInside"; break;
                default:
                    name = ""; break;
            }
            return name;
        }

        /// <summary>
        /// 根据枚举值返回解析好的对应面
        /// </summary>
        /// <param name="type"></param>
        /// <param name="planes"></param>
        /// <returns></returns>
        public static Plane GetBeamPlane(EBeamPlaneType type, BeamPlanes planes)
        {
            if (planes == null)
            {
                Console.WriteLine(@"BeamPlanes is null");
                return null;
            }
            Plane ret;
            switch (type)
            {
                case EBeamPlaneType.HeadBound:
                    ret = planes.HeadBound; break;
                case EBeamPlaneType.TailBound:
                    ret = planes.TailBound; break;
                case EBeamPlaneType.TopBound:
                    ret = planes.TopBound; break;
                case EBeamPlaneType.BottomBound:
                    ret = planes.BottomBound ; break;
                case EBeamPlaneType.FrontBound:
                    ret = planes.FrontBound; break;
                case EBeamPlaneType.BackBound:
                    ret = planes.BackBound ; break;
                case EBeamPlaneType.TopFlange:
                    ret = planes.TopFlange ; break;
                case EBeamPlaneType.TopFlangeInside:
                    ret = planes.TopFlangeInside ; break;
                case EBeamPlaneType.TopFlangeFront:
                    ret = planes.TopFlangeFront ; break;
                case EBeamPlaneType.TopFlangeBack:
                    ret = planes.TopFlangeBack ; break;
                case EBeamPlaneType.BottomFlange:
                    ret = planes.BottomFlange ; break;
                case EBeamPlaneType.BottomFlangeInside:
                    ret = planes.BottomFlangeInside ; break;
                case EBeamPlaneType.BottomFlangeFront:
                    ret = planes.BottomFlangeFront ; break;
                case EBeamPlaneType.BottomFlangeBack:
                    ret = planes.BottomFlangeBack ; break;
                case EBeamPlaneType.FrontWeb:
                    ret = planes.FrontWeb ; break;
                case EBeamPlaneType.FrontWebInside:
                    ret = planes.FrontWebInside; break;
                case EBeamPlaneType.BackWeb:
                    ret = planes.BackWeb; break;
                case EBeamPlaneType.BackWebInside:
                    ret = planes.BackWebInside; break;
                default:
                    return null;
            }
            if (ret == null) {
                Console.WriteLine(GetBeamPlaneName(type)+@" get plane is null");
            }

            return ret == null ? null : new Plane(ret);
        }

        public static void CheckPlansInList(BeamPlanes beamPlanes, List<EBeamPlaneType> list)
        {
            foreach (var type in list)
            {
                var pl = GetBeamPlane(type, beamPlanes);
                PrintPlanes(pl,GetBeamPlaneName(type));
            }
        }
        /// <summary>
        /// 检查解析面的存在性
        /// </summary>
        /// <param name="beamPlanes"></param>
        public static void CheckAllBeamPlane(BeamPlanes beamPlanes)
        {
            if (beamPlanes == null)
                return;
            foreach (EBeamPlaneType value in Enum.GetValues(typeof(EBeamPlaneType)))
            {
                var plane = GetBeamPlane(value, beamPlanes);
                if (plane == null)
                {
                    PrintPlanes(GetBeamPlane(value,beamPlanes),GetBeamPlaneName(value));
                }
            }   
        }

        private static void PrintPlanes(Plane plane,string msg)
        {
            if (plane != null)
                msg += " is valid ";
            else
                msg += " is empty ";
            Console.WriteLine(msg);
        }

        /// <summary>
        /// 打印两个面的距离
        /// </summary>
        /// <param name="planeType1"></param>
        /// <param name="planeType2"></param>
        /// <param name="beamPlanes"></param>
        public static void PrintDist(EBeamPlaneType planeType1, EBeamPlaneType planeType2,BeamPlanes beamPlanes)
        {
            var dist = GetTwoPlaneDist(GetBeamPlane(planeType1, beamPlanes),
                GetBeamPlane(planeType2,beamPlanes));
            var msg = GetBeamPlaneName(planeType1) + " to " + GetBeamPlaneName(planeType2)
                      + " dist: " + dist;
            Console.WriteLine(msg);
        }
        
        /// <summary>
        /// 计算出给定面中，哪个面的法线和指定line的点乘绝对值结果最大
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <param name="types"></param>
        /// <param name="line"></param>
        /// <returns></returns>
        internal static EBeamPlaneType DotAbsMax(BeamPlanes beamPlanes,List<EBeamPlaneType> types,Vec3 line)
        {
            line = line.Normalize();
            double dot = -1;
            var ret = types.First();
            foreach (var type in types)
            {
                var plane = PlaneHelper.GetBeamPlane(type,beamPlanes);
                var tmp= Math.Abs(line.Dot(plane.Normal.Normalize()));
                if (tmp > dot)
                {
                    dot = tmp;
                    ret = type;
                }
            }
            return ret;
        }
        
        /// <summary>
        /// 计算出给定面中，哪个面的法线和指定line的点乘结果最大
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <param name="types"></param>
        /// <param name="line"></param>
        /// <returns></returns>
        internal static EBeamPlaneType DotMax(BeamPlanes beamPlanes,List<EBeamPlaneType> types,Vec3 line)
        {
            line = line.Normalize();
            double dot = -1;
            var ret = types.First();
            foreach (var type in types)
            {
                var plane = PlaneHelper.GetBeamPlane(type,beamPlanes);
                if (plane == null) {
                    continue;
                }
                var tmp= line.Dot(plane.Normal.Normalize());
                if (tmp > dot)
                {
                    dot = tmp;
                    ret = type;
                }
            }
            return ret;
        }

        /// <summary>
        /// 获取同法线方向的厚度值
        /// </summary>
        /// <param name="beamPlanes"></param>
        /// <param name="normal"></param>
        /// <returns></returns>
        internal static double GetSameNormalFlangeOrWebThickness(BeamPlanes beamPlanes, Vec3 normal)
        {
            var types = new List<EBeamPlaneType>()
            {
                EBeamPlaneType.TopFlange,
                EBeamPlaneType.BottomFlange,
                EBeamPlaneType.FrontWeb,
                EBeamPlaneType.BackWeb,
            };
            var refType = types.First();
            foreach (var type in types)
            {
                var plane = GetBeamPlane(type, beamPlanes);
                if(plane == null)
                    continue;
                if (!plane.Normal.IsParallelTo(normal)) continue;
                refType = type;
                break;
            }

            var dist = GetFlangeOrWebThickness(refType, beamPlanes);
            return dist;
        }

        /// <summary>
        /// 获得翼缘或者腹板面的厚度
        /// </summary>
        /// <param name="refType"></param>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        private static double GetFlangeOrWebThickness(EBeamPlaneType refType, BeamPlanes beamPlanes)
        {
            var plane = GetBeamPlane(refType, beamPlanes);
            if (plane == null)
                return 0.0d;
            Plane planeBack = null;
            switch (refType)
            {
                case EBeamPlaneType.TopFlange:
                    planeBack=GetBeamPlane(EBeamPlaneType.TopFlangeInside, beamPlanes); break;
                case EBeamPlaneType.BottomFlange:
                    planeBack=GetBeamPlane(EBeamPlaneType.BottomFlangeInside, beamPlanes); break;
                case EBeamPlaneType.FrontWeb:
                    planeBack=GetBeamPlane(EBeamPlaneType.FrontWebInside, beamPlanes); break;
                case EBeamPlaneType.BackWeb:
                    planeBack=GetBeamPlane(EBeamPlaneType.BackWebInside, beamPlanes); break;
                default:
                    return 0.0d;
            }
            if (planeBack == null)
                return 0.0d;
            
            return plane.Dist(planeBack.Origin);
        }

        /// <summary>
        /// 返回Bound的对应背面
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static EBeamPlaneType GetOppositeBoundType(EBeamPlaneType type)
        {
            switch (type) {
                case EBeamPlaneType.BottomBound:
                    return EBeamPlaneType.TopBound;
                case EBeamPlaneType.TopBound:
                    return EBeamPlaneType.BottomBound;
                case EBeamPlaneType.FrontBound:
                    return EBeamPlaneType.BackBound;
                case EBeamPlaneType.BackBound:
                    return EBeamPlaneType.FrontBound;
                case EBeamPlaneType.HeadBound:
                    return EBeamPlaneType.TailBound;
                case EBeamPlaneType.TailBound:
                    return EBeamPlaneType.HeadBound;
                default:
                    return type;
            }
        }

        public static Plane GetSameNormalPlane(BeamPlanes beamPlanes, List<EBeamPlaneType> typeList, Vec3 normal)
        {
            var type = DotMax(beamPlanes, typeList, normal);
            return GetBeamPlane(type, beamPlanes);
        }

        /// <summary>
        /// 获取一组面在参在面的落点
        /// </summary>
        /// <param name="landingPlane"></param>
        /// <param name="planes"></param>
        /// <returns></returns>
        public static List<Vec3> GetLandingPoint(Plane landingPlane, List<Plane> planes)
        {
            var ret = new List<Vec3>();
            for (int i = 0; i < planes.Count-1; ++i)
            {
                ret.Add(landingPlane.Intersect(planes[i], planes[i + 1]));
            }
            ret.Add(landingPlane.Intersect(planes.Last(),planes.First()));
            return ret;
        }

        public static Vec3 GetIntersectPoint(BeamPlanes beamPlanes, EBeamPlaneType type1, EBeamPlaneType type2,
            Plane pl)
        {
            if (beamPlanes == null){return Vec3.MAX;}
            if (pl == null){return Vec3.MAX;}
            var pl1 = GetBeamPlane(type1, beamPlanes);
            var pl2 = GetBeamPlane(type2, beamPlanes);
            return pl.Intersect(pl1, pl2);
        }

        public static void PrintPlane(Beam beam,ConnectionComponent target)
        {
            if (beam == null) return;
            var p = beam.CtrlPoints.First().WorldPos;
            var l = p - beam.CtrlPoints.Last().WorldPos;
            var msg = beam.Profile.Name;
            BeamPlanes beamPlanes = BeamPlanes.CreateFromBeam(beam);
            if(beamPlanes == null) return;
            var pl = new Plane();
            var colors = new List<Color>()
            {
                Color.Red, Color.Orange, Color.Yellow, Color.Green,Color.Aqua,
                Color.Blue,Color.DarkOrchid,Color.Brown,Color.Honeydew
            };
            var i = 0;
            Console.WriteLine(msg + @"plane check:");
            var boundPls = new HashSet<EBeamPlaneType>()
            {
                EBeamPlaneType.HeadBound, EBeamPlaneType.TailBound,
                EBeamPlaneType.TopBound, EBeamPlaneType.BottomBound,
                EBeamPlaneType.FrontBound, EBeamPlaneType.BackBound,
            };
            var dist = 10;
            foreach (EBeamPlaneType type in Enum.GetValues(typeof(EBeamPlaneType)))
            {
                var name = GetBeamPlaneName(type);
                pl = GetBeamPlane(type, beamPlanes);
                if (pl == null)
                {
                    Console.WriteLine(msg + @"| is empty: " + name);
                    ++i;
                    continue;
                }
                if(boundPls.Contains(type)) {
                    dist = 20;
                }
                var colorI = i / 2;
                PointHelper.MoveRef(ref p,-l,15);
                var pos0 = PointHelper.MoveTo(p, pl);
                var pos1 = PointHelper.MoveTo(p, pl.Move(dist));
                var line = target.CreateLine(pos0, pos1, 0, colors[colorI], true);
                var quad = target.CreateQuad(new Plane(pos1, pl.Normal), 10, 10, true);
                quad.Name = name;
                quad.Flush();
                ++i;
            }

        }
        
    }
}