﻿using System;
using System.Collections.Generic;
using System.Numerics;
using LoongCAD.Geometry;

namespace U.S.Base_plate_1047_.Tool
{
    public static class PointHelper
    {
        /// <summary>
        /// 让点朝着某个方向移动一定距离
        /// </summary>
        /// <param name="point">要移动的点</param>
        /// <param name="direction">移动的方向</param>
        /// <param name="length">移动的距离</param>
        /// <returns>移动后点的位置</returns>
        public static Vec3 Move(Vec3 point, Vec3 direction, double length)
        {
            var dir =direction.Normalize();
            var movement = dir * length;
            return new Vec3(point + movement);
        }

        /// <summary>
        /// 让点朝着某个方向移动一定距离
        /// </summary>
        /// <param name="point">要移动的点，传入初始位置，传出移动后的位置</param>
        /// <param name="direction">移动的方向</param>
        /// <param name="length">移动的距离</param>
        public static void MoveRef(ref Vec3 point, Vec3 direction, double length)
        {
            var dir = direction.Normalize();
            var movement = dir * length;
            point = point + movement;
        }

        public static Vec3 RotateVectorInPlane(Vec3 vector, Plane plane, double angleDegrees)
        {
            return RotateVectorInPlane(vector,plane.Normal,angleDegrees);
        }

        /// <summary>
        /// 在给定平面内旋转一个三维向量指定的角度。
        /// </summary>
        /// <param name="vector">要旋转的向量。</param>
        /// <param name="planeNormal">定义旋转平面的法线向量。</param>
        /// <param name="angleDegrees">向量在平面内旋转的角度，单位为度。</param>
        /// <returns>旋转后的向量。</returns>
        public static Vec3 RotateVectorInPlane(Vec3 vector, Vec3 planeNormal, double angleDegrees)
        {
            // 将平面法线标准化，确保它是单位向量
            Vec3 normalizedPlaneNormal = planeNormal.Normalize();

            // 将角度从度转换为弧度
            double angleRadians = Math.PI / 180 * angleDegrees;

            // 根据罗德里格斯旋转公式计算旋转的各个分量
            Vec3 rotatedVector = vector * Math.Cos(angleRadians) // 向量与余弦角度的乘积
                                    + normalizedPlaneNormal.Cross(vector) * Math.Sin(angleRadians) // 法线与向量的叉乘与正弦角度的乘积
                                    + normalizedPlaneNormal * normalizedPlaneNormal.Dot(vector) * (1 - Math.Cos(angleRadians)); // 法线与法线与向量点乘的乘积，再乘以1减去余弦角度
            return rotatedVector; // 返回旋转后的向量
        }

        /// <summary>
        /// 返回a,b点的中点
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static Vec3 Midpoint(Vec3 a, Vec3 b)
        {
            var x = (a.X + b.X) / 2;
            var y = (a.Y + b.Y) / 2;
            var z = (a.Z + b.Z) / 2;
            return new Vec3(x, y, z);
        }
        
        
        /// <summary>
        /// 计算两个点之间的距离，以给定方向为基准
        /// </summary>
        /// <param name="pointA"></param>
        /// <param name="pointB"></param>
        /// <param name="direction"></param>
        /// <returns></returns>
        public static double DistanceInDirection(Vec3 pointA, Vec3 pointB, Vec3 direction)
        {
            Vec3 ab = new Vec3(pointB.X - pointA.X, pointB.Y - pointA.Y, pointB.Z - pointA.Z);
            Vec3 directionNormalized = direction.Normalize();
            return ab.Dot(directionNormalized);
        }

        /// <summary>
        /// 把一个点移动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="targetPlane"></param>
        public static Vec3 MoveTo(Vec3 point, Plane targetPlane)
        {
            var dist = targetPlane.SignedDist(point);
            return Move(point, targetPlane.Normal, -dist);
        }
        /// <summary>
        /// 依次将点挪动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="planes"></param>
        /// <returns></returns>
        public static Vec3 MoveTo(Vec3 point, List<Plane> planes)
        {
            foreach (var plane in planes)
            {
                point = MoveTo(point, plane);
            }
            return point;
        }

        /// <summary>
        /// 把一个点移动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="targetPlane"></param>
        public static void MoveToRef(ref Vec3 point, Plane targetPlane)
        {
            var dist = targetPlane.SignedDist(point);
            MoveRef(ref point, targetPlane.Normal, -dist);
        }
        
        /// <summary>
        /// 依次将点挪动到平面上
        /// </summary>
        /// <param name="point"></param>
        /// <param name="planes"></param>
        /// <returns></returns>
        public static void MoveToRef(ref Vec3 point, List<Plane> planes)
        {
            foreach (var plane in planes)
            {
                MoveToRef(ref point, plane);
            }
        }
        
    }
}