﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using LoongCAD.Geometry;

namespace U.S.Base_plate_1047_.Tool
{
    /// <summary>
    /// 板通用数据
    /// </summary>
    public class PlateData
    {
        /// <summary>
        /// 板厚度
        /// </summary>
        public double Thickness { get; set; } = 0.0d;
        /// <summary>
        /// 板宽度,Y轴方向
        /// </summary>
        public double Width { get; set; } = 0.0d;
        /// <summary>
        /// 板高度,X轴方向
        /// </summary>
        public double Height { get; set; } = 0.0d;

        /// <summary>
        /// 板空值
        /// </summary>
        public static readonly PlateData ZERO = new PlateData(0, 0, 0);

        /// <summary>
        /// 板构造函数
        /// </summary>
        /// <param name="thickness"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public PlateData(double thickness, double width, double height)
        {
            Thickness = thickness;
            Width = width;
            Height = height;
        }

        /// <summary>
        /// 判断是否有效
        /// </summary>
        /// <returns></returns>
        public bool IsValid()
        {
            return !(Thickness <= 0) && !(Width <= 0) && !(Height <= 0);
        }
    }
    
    
    /// <summary>
    /// 矩形板材构建帮助
    /// 矩形板用多边形板创建，以一个矩形面为基础，矩形左下角为原点。
    /// 所有控制点从左下角起，从0开始，顺时针计数，共计4个控制点。
    /// 额外增加一个中心点。
    /// </summary>
    public class RectangularPlateHelper
    {
        /// <summary>
        /// 用于存储所有控制点的字典集
        /// </summary>
        private readonly Dictionary<int, Vec3> m_controlPoints = new Dictionary<int, Vec3>()
        {
            {0,Vec3.ZERO},
            {1,Vec3.ZERO},
            {2,Vec3.ZERO},
            {3,Vec3.ZERO},
            {4,Vec3.ZERO},
        };
        
        /// <summary>
        /// 0号控制点，位于左下角。
        /// </summary>
        public Vec3 ControlPoint0 { get => m_controlPoints[0]; set=>m_controlPoints[0] = value; }
        /// <summary>
        /// 1号控制点，位于左上角
        /// </summary>
        public Vec3 ControlPoint1 { get => m_controlPoints[1]; set=>m_controlPoints[1] = value; }
        /// <summary>
        /// 2号控制点，位于右上角
        /// </summary>
        public Vec3 ControlPoint2 { get => m_controlPoints[2]; set=>m_controlPoints[2] = value; }
        /// <summary>
        /// 3号控制点，位于右下角。
        /// </summary>
        public Vec3 ControlPoint3 { get => m_controlPoints[3]; set=>m_controlPoints[3] = value; }
        
        /// <summary>
        /// 中心点
        /// </summary>
        public Vec3 CenterPoint { get => m_controlPoints[4];set=>m_controlPoints[4] = value; }
        
        /// <summary>
        /// 矩形板X轴方向
        /// </summary>
        public  Vec3 AxisX { get; set; }
        /// <summary>
        /// 矩形板Y轴方向
        /// </summary>
        public  Vec3 AxisY { get; set; }
        
        /// <summary>
        /// 偏移量，以0号点为原点。记录每个点到0号点的偏移量。
        /// </summary>
        private readonly Vec3[] m_offSets = new Vec3[5];

        /// <summary>
        /// 设置坐标系方向
        /// </summary>
        /// <param name="axisX"></param>
        /// <param name="axisY"></param>
        /// <param name="plateData"></param>
        public void SetAxisDirection_XY(Vec3 axisX, Vec3 axisY, PlateData plateData)
        {
            AxisX = axisX.Normalize();
            AxisY = axisY.Normalize();
            // 0号控制点，位于左下角，没有偏移。
            m_offSets[0] = Vec3.ZERO;
            // 1号控制点，位于左上角
            m_offSets[1] = AxisY * plateData.Width;
            // 2号控制点，位于右上角
            m_offSets[2] = AxisY * plateData.Width + AxisX*plateData.Height;
            // 3号控制点，位于右下角
            m_offSets[3] = AxisX*plateData.Height;
            // 中心点
            m_offSets[4] = AxisX * plateData.Height / 2 + AxisY * plateData.Width/ 2;

        }
        
        /// <summary>
        /// 计算控制点
        /// </summary>
        /// <param name="axisX"></param>
        /// <param name="axisY"></param>
        /// <param name="plateData"></param>
        /// <param name="worldPos"></param>
        /// <param name="pointNumber"></param>
        public void CalculateControlPoint(Vec3 axisX,Vec3 axisY,PlateData plateData, Vec3 worldPos,int pointNumber = 4)
        {
            SetAxisDirection_XY(axisX,axisY,plateData);
            Debug.Assert((pointNumber >= 0 && m_offSets.Length > pointNumber), $"Wrong pointNumber:{pointNumber}");
            for (var i = 0; i < m_offSets.Length; ++i)
            {
                var relativeOffset = m_offSets[i] - m_offSets[pointNumber];
                var controlPointPos = worldPos + relativeOffset;
                m_controlPoints[i] = controlPointPos;
            }
        }
        
        /// <summary>
        /// 重新计算控制点
        /// </summary>
        /// <param name="plateData"></param>
        /// <param name="worldPos"></param>
        /// <param name="pointNumber"></param>
        public void RecalculateControlPoints(PlateData plateData,Vec3 worldPos, int pointNumber = 0)
        {
            CalculateControlPoint(AxisX,AxisY,plateData,worldPos,pointNumber); 
        }

        /// <summary>
        /// 返回所有控制点的坐标数组
        /// </summary>
        /// <returns></returns>
        public Vec3[] ReturnAllControlPoints()
        {
            //因为中心点是构建矩形板不需要的
            return new Vec3[4]
            {
                ControlPoint0,
                ControlPoint1,
                ControlPoint2,
                ControlPoint3,
            };
        }
        
        
    }
}