﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using LoongCAD.Geometry;

namespace U.S.Base_plate_1047_.Tool
{
    
    /// <summary>
    /// 加劲肋用多边形板创建，以一个矩形面为基础，矩形左下角为原点。
    /// 所有控制点从左下角起，从0开始，顺时针计数，共计6个控制点
    /// 顶部水平折角位于左侧，底部水平折角位于右侧。
    /// </summary>
    public class StiffenerData
    {
        /// <summary>
        /// 厚度
        /// </summary>
        public double Thickness { get; set; }
        /// <summary>
        /// 宽度
        /// </summary>
        public double Width { get; set; }
        /// <summary>
        /// 高度
        /// </summary>
        public double Height { get; set; }
        /// <summary>
        /// 顶部水平倒角尺寸
        /// </summary>
        public double TopHorizontalChamfer{ get; set; }
        /// <summary>
        /// 顶部垂直倒角尺寸 
        /// </summary>
        public double TopVerticalChamfer{ get; set; }
        /// <summary>
        /// 底部水平倒角尺寸 
        /// </summary>
        public double BottomHorizontalChamfer{ get; set; }
        /// <summary>
        /// 底部垂直倒角尺寸
        /// </summary>
        public double BottomVerticalChamfer{ get; set; }

        /// <summary>
        /// 默认值
        /// </summary>
        public static readonly StiffenerData ZERO = new StiffenerData();
        
        /// <summary>
        /// 默认构造函数
        /// </summary>
        public StiffenerData()
        {
            Thickness = 20.0d;
            Width = 100.0d;
            Height = 200.0d;
            TopHorizontalChamfer = 50;
            TopVerticalChamfer = 50;
            BottomHorizontalChamfer = 50;
            BottomVerticalChamfer = 50;
        }
        
        /// <summary>
        /// 加劲肋配置的构造函数
        /// </summary>
        /// <param name="thickness">厚度</param>
        /// <param name="width">宽度,y轴方向</param>
        /// <param name="height">高度,x轴方向</param>
        /// <param name="topHorizontalChamfer">顶部水平倒角尺寸</param>
        /// <param name="topVerticalChamfer">顶部垂直倒角尺寸</param>
        /// <param name="bottomHorizontalChamfer">底部水平倒角尺寸</param>
        /// <param name="bottomVerticalChamfer">底部垂直倒角尺寸</param>
        public StiffenerData(double thickness,double width,double height,
            double topHorizontalChamfer,double topVerticalChamfer,
            double bottomHorizontalChamfer,double bottomVerticalChamfer)
        {
            Thickness = thickness;
            Width = width;
            Height = height;
            TopHorizontalChamfer = topHorizontalChamfer;
            TopVerticalChamfer = topVerticalChamfer;
            BottomHorizontalChamfer = bottomHorizontalChamfer;
            BottomVerticalChamfer = bottomVerticalChamfer;
        }
    }
    /// <summary>
    /// 折角位于左上角和右下角
    /// </summary>
    public class StiffenerHelper
    {
        
        /// <summary>
        /// 用于存储所有控制点的字典集
        /// </summary>
        private readonly Dictionary<int, Vec3> m_controlPoints = new Dictionary<int, Vec3>()
        {
            {0,Vec3.ZERO},
            {1,Vec3.ZERO},
            {2,Vec3.ZERO},
            {3,Vec3.ZERO},
            {4,Vec3.ZERO},
            {5,Vec3.ZERO},
        };
        
        /// <summary>
        /// 0号控制点，位于左下角。
        /// </summary>
        public Vec3 ControlPoint0 { get => m_controlPoints[0]; set=>m_controlPoints[0] = value; }
        /// <summary>
        /// 1号控制点，位于左上角往下TopVerticalChamfer的位置。
        /// </summary>
        public Vec3 ControlPoint1 { get => m_controlPoints[1]; set=>m_controlPoints[1] = value; }
        /// <summary>
        /// 2号控制点，位于左上角往右TopHorizontalChamfer的位置。
        /// </summary>
        public Vec3 ControlPoint2 { get => m_controlPoints[2]; set=>m_controlPoints[2] = value; }
        /// <summary>
        /// 3号控制点，位于右上角。
        /// </summary>
        public Vec3 ControlPoint3 { get => m_controlPoints[3]; set=>m_controlPoints[3] = value; }
        /// <summary>
        /// 4号控制点，位于右下角往上BottomVerticalChamfer的位置。
        /// </summary>
        public Vec3 ControlPoint4 { get => m_controlPoints[4]; set=>m_controlPoints[4] = value; }
        /// <summary>
        /// 5号控制点，位于右下角往左BottomHorizontalChamfer的位置。
        /// </summary>
        public Vec3 ControlPoint5 { get => m_controlPoints[5]; set=>m_controlPoints[5] = value; }
        
        /// <summary>
        /// X轴的方向，右方向
        /// </summary>
        public Vec3 AxisDirectionX { get; set; } = Vec3.ZERO;
        /// <summary>
        /// Y轴的方向，上反向
        /// </summary>
        public Vec3 AxisDirectionY { get; set; } = Vec3.ZERO;

        /// <summary>
        /// 偏移量，以0号点为原点。记录每个点到0号点的偏移量。
        /// </summary>
        private readonly Vec3[] m_offSets = new Vec3[6];

        /// <summary>
        /// 设置x,y轴的方向，并计算偏移值
        /// </summary>
        /// <param name="xAxisDirection"></param>
        /// <param name="yAxisDirection"></param>
        /// <param name="stiffenerData"></param>
        public void SetAxisDirection_XY(Vec3 xAxisDirection,Vec3 yAxisDirection,StiffenerData stiffenerData)
        {
            AxisDirectionX = xAxisDirection.Normalize();
            AxisDirectionY = yAxisDirection.Normalize();
            
            // 0号控制点，位于左下角，没有偏移。
            m_offSets[0] = Vec3.ZERO;
            // 1号控制点，位于左上角往下TopVerticalChamfer的位置。
            m_offSets[1] = AxisDirectionY * (stiffenerData.Width - stiffenerData.TopVerticalChamfer);
            // 2号控制点，位于左上角往右TopHorizontalChamfer的位置。
            m_offSets[2] = AxisDirectionY * stiffenerData.Width + AxisDirectionX * stiffenerData.TopHorizontalChamfer;
            // 3号控制点，位于右上角。
            m_offSets[3] = AxisDirectionY * stiffenerData.Width + AxisDirectionX * stiffenerData.Height;
            // 4号控制点，位于右下角往上BottomVerticalChamfer的位置。
            m_offSets[4] = AxisDirectionX * stiffenerData.Height + AxisDirectionY * stiffenerData.BottomVerticalChamfer;
            // 5号控制点，位于右下角往左BottomHorizontalChamfer的位置。
            m_offSets[5] = AxisDirectionX * (stiffenerData.Height - stiffenerData.BottomHorizontalChamfer);
        }

        /// <summary>
        /// 计算所有的控制点
        /// </summary>
        /// <param name="axisX"></param>
        /// <param name="axisY"></param>
        /// <param name="stiffenerData"></param>
        /// <param name="worldPos"></param>
        /// <param name="pointNumber"></param>
        public void CalculateControlPoint(Vec3 axisX,Vec3 axisY,StiffenerData stiffenerData, Vec3 worldPos,int pointNumber = 0)
        {
            SetAxisDirection_XY(axisX,axisY,stiffenerData);
            Debug.Assert((pointNumber >= 0 && 6 > pointNumber), $"Wrong pointNumber:{pointNumber}");
            for (var i = 0; i < 6; ++i)
            {
                var relativeOffset = m_offSets[i] - m_offSets[pointNumber];
                var controlPointPos = worldPos + relativeOffset;
                m_controlPoints[i] = controlPointPos;
            }
        }

        /// <summary>
        /// 重新计算所有控制点，x,y轴不变
        /// </summary>
        /// <param name="stiffenerData"></param>
        /// <param name="worldPos"></param>
        /// <param name="pointNumber"></param>
        public void RecalculateControlPoints(StiffenerData stiffenerData,Vec3 worldPos, int pointNumber = 0)
        {
            CalculateControlPoint(AxisDirectionX,AxisDirectionY,stiffenerData,worldPos,pointNumber); 
        }

        /// <summary>
        /// 返回所有控制点的坐标数组
        /// </summary>
        /// <returns></returns>
        public Vec3[] ReturnAllControlPoints(Vec3 axisX,Vec3 axisY,StiffenerData stiffenerData,Vec3 worldPos,int pointNumber = 0)
        {
            CalculateControlPoint(axisX,axisY,stiffenerData,worldPos,pointNumber);
            var ret = new List<Vec3>();
            ret.Add(ControlPoint0);
            HandlerTopLeftChamfer(ret,stiffenerData);
            ret.Add(ControlPoint3);
            HandlerBtmRightChamfer(ret,stiffenerData);
            return ret.ToArray();
        }

        private void HandlerBtmRightChamfer(List<Vec3> ret, StiffenerData stiffenerData)
        {
            if (stiffenerData.BottomVerticalChamfer == 0) {
                ret.Add(ControlPoint4);
            }
            else {
                if (stiffenerData.BottomHorizontalChamfer != 0) {
                    ret.Add(ControlPoint4);
                    ret.Add(ControlPoint5);
                }
                else {
                    ret.Add(ControlPoint5);
                }
            }
        }

        private void HandlerTopLeftChamfer(List<Vec3> ret, StiffenerData stiffenerData)
        {
            if (stiffenerData.TopVerticalChamfer == 0) {
                ret.Add(ControlPoint1);
            }
            else {
                if (stiffenerData.TopHorizontalChamfer != 0) {
                    ret.Add(ControlPoint1);
                    ret.Add(ControlPoint2);
                }
                else {
                    ret.Add(ControlPoint2);
                }
            }
        }

        /// <summary>
        /// 返回左手坐标系下的Z轴方向
        /// </summary>
        /// <returns></returns>
        public Vec3 LeftHandZAxis()
        {
            return AxisDirectionX.Cross(AxisDirectionY);
        }

        /// <summary>
        /// 返回右手坐标系下的Z轴方向
        /// </summary>
        /// <returns></returns>
        public Vec3 RightHandZAxis()
        {
            return AxisDirectionY.Cross(AxisDirectionX);
        }

    }
}