﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;
using System;
using WindBraceConnection110.Common;

namespace WindBraceConnection110.Handlers
{
    public class BraceTools
    {
        /// <summary>
        /// 得到梁零件与给定向量符合点积条件的包围面组
        /// 不限定截面类型
        /// <param name="flag">true时为点积绝对值最大，false时为点积绝对值最小</param>
        /// </summary>
        /// <returns></returns>
        public static Plane[] DotExtremeBoundPlanes(BeamPlanes planes, Vec3 mainLine, bool flag)
        {
            if (planes == null)
                return null;
            var dot1 = Math.Abs(planes.TopBound.Normal.Normalize().Dot(mainLine));
            var dot2 = Math.Abs(planes.FrontBound.Normal.Normalize().Dot(mainLine));
            switch (dot1 > dot2 == flag)
            {
                case true:
                    return new Plane[] { planes.TopBound, planes.BottomBound };
                default:
                    return new Plane[] { planes.BackBound, planes.FrontBound };
            }
        }
        /// <summary>
        /// 得到梁零件与给定向量符合点积条件的包围面
        /// <param name="flag">true时为点积最大，false时为点积最小</param>
        /// </summary>
        /// <returns></returns>
        public static Plane DotExtremeBoundPlane(BeamPlanes beamPlanes, Vec3 line, bool flag)
        {
            if (beamPlanes == null)
                return null;
            int num = -1;
            double extreme;
            if (flag)
                extreme = double.MinValue;
            else
                extreme = double.MaxValue;
            Plane[] planes = new Plane[]
            {
                beamPlanes.BackBound, beamPlanes.FrontBound,
                beamPlanes.TopBound, beamPlanes.BottomBound
            };
            for(int i = 0; i < planes.Length; i++)
            {
                var temp = planes[i].Normal.Normalize().Dot(line);
                if(temp > extreme == flag)
                {
                    extreme = temp;
                    num = i;
                }
            }
            return planes[num];
        }
        /// <summary>
        /// 解析梁的截面类型
        /// </summary>
        public static BraceProfileType ParseBraceProfileType(Beam beam)
        {
            if (beam == null)
                return BraceProfileType.OTHER;
            var type = beam.Profile.GetType();
            if (type == typeof(LProfile))
                return BraceProfileType.L;
            else if (type == typeof(TProfile))
                return BraceProfileType.T;
            else if (type == typeof(IProfile))
                return BraceProfileType.I;
            else if (type == typeof(CProfile))
                return BraceProfileType.C;
            else if (type == typeof(UProfile))
                return BraceProfileType.U;
            else if (type == typeof(CCProfile))
                return BraceProfileType.CC;
            else if (type == typeof(RectHollowProfile))
                return BraceProfileType.RECTHOLLOW;
            else if (type == typeof(PlateProfile))
                return BraceProfileType.BAN;
            else if (type == typeof(WQProfile))
                return BraceProfileType.HQ;
            else if (type == typeof(WeldedBoxProfile))
                return BraceProfileType.WELDBOX;
            return BraceProfileType.OTHER;
        }
        /// <summary>
        /// 解析满足给定条件的支撑解析面组
        /// </summary>
        public static Plane[] ParseBasePlane(BraceProfileType type, Vec3 line, BeamPlanes beamPlanes)
        {
            if (beamPlanes == null)
                return null;
            Plane[] planes;
            switch (type)
            {
                case BraceProfileType.L:
                    planes = new Plane[]
                    {
                        beamPlanes.BottomFlange,
                        beamPlanes.BackWeb
                    };
                    break;
                case BraceProfileType.C:
                case BraceProfileType.U:
                case BraceProfileType.CC:
                    planes = new Plane[]
                    {
                        beamPlanes.BottomFlange,
                        beamPlanes.BackWeb,
                        beamPlanes.TopFlange
                    };
                    break;
                case BraceProfileType.T:
                    planes = new Plane[]
                    {
                        beamPlanes.TopFlange,
                        beamPlanes.BackBound,
                        beamPlanes.FrontBound
                    };
                    break;
                case BraceProfileType.I:
                case BraceProfileType.RECTHOLLOW:
                case BraceProfileType.BAN:
                case BraceProfileType.HQ:
                case BraceProfileType.WELDBOX:
                    planes = new Plane[]
                    {
                        beamPlanes.TopBound,
                        beamPlanes.BackBound,
                        beamPlanes.FrontBound,
                        beamPlanes.BottomBound
                    };
                    break;
                default:
                    return null;
            }
            double max = -1;
            int num = -1;
            for(int i = 0; i < planes.Length; i++)
            {
                double temp = Math.Abs(planes[i].Normal.Normalize().Dot(line));
                if(temp > max)
                {
                    max = temp;
                    num = i;
                }
            }
            return new Plane[] { planes[num], GetOppositePlane(planes[num], beamPlanes) };
        }
        /// <summary>
        /// 得到与给定的包围面相对应的包围面
        /// </summary>
        /// <param name="plane"></param>
        /// <param name="beamPlanes"></param>
        /// <returns></returns>
        public static Plane GetOppositePlane(Plane plane, BeamPlanes beamPlanes)
        {
            if (plane.Equals(beamPlanes.BackWeb))
                return beamPlanes.FrontBound;
            else if (plane.Equals(beamPlanes.FrontWeb))
                return beamPlanes.BackBound;
            else if (plane.Equals(beamPlanes.BottomFlange))
                return beamPlanes.TopBound;
            else if (plane.Equals(beamPlanes.TopFlange))
                return beamPlanes.BottomBound;
            else if (plane.Equals(beamPlanes.TopBound))
                return beamPlanes.BottomBound;
            else if (plane.Equals(beamPlanes.BottomBound))
                return beamPlanes.TopBound;
            else if (plane.Equals(beamPlanes.FrontBound))
                return beamPlanes.BackBound;
            else
                return beamPlanes.FrontBound;
        }
        /// <summary>
        /// 得到对应类型支撑的有效解析面
        /// </summary>
        public static Plane[] GetBracePlane(BeamPlanes beamPlanes, BraceProfileType type)
        {
            if (beamPlanes == null)
                return null;
            Plane[] planes;
            switch (type)
            {
                case BraceProfileType.L:
                    planes = new Plane[]
                    {
                        beamPlanes.BottomFlange,
                        beamPlanes.BottomFlangeInside,
                        beamPlanes.BackWeb,
                        beamPlanes.BackWebInside
                    };
                    break;
                case BraceProfileType.C:
                case BraceProfileType.U:
                case BraceProfileType.CC:
                    planes = new Plane[]
                    {
                        beamPlanes.TopFlange,
                        beamPlanes.TopFlangeInside,
                        beamPlanes.BottomFlange,
                        beamPlanes.BottomFlangeInside,
                        beamPlanes.BackWeb,
                        beamPlanes.BackWebInside
                    };
                    break;
                case BraceProfileType.I:
                case BraceProfileType.RECTHOLLOW:
                    planes = new Plane[]
                    {
                        beamPlanes.TopFlange,
                        beamPlanes.TopFlangeInside,
                        beamPlanes.BottomFlange,
                        beamPlanes.BottomFlangeInside,
                        beamPlanes.BackWeb,
                        beamPlanes.BackWebInside,
                        beamPlanes.FrontWeb,
                        beamPlanes.FrontWebInside
                    };
                    break;
                case BraceProfileType.T:
                    planes = new Plane[]
                    {
                        beamPlanes.TopFlange,
                        beamPlanes.TopFlangeInside,
                        beamPlanes.BackWeb,
                        beamPlanes.BackWebInside,
                        beamPlanes.FrontWeb,
                        beamPlanes.FrontWebInside
                    };
                    break;
                case BraceProfileType.BAN:
                case BraceProfileType.HQ:
                case BraceProfileType.WELDBOX:
                    planes = new Plane[]
                    {
                        beamPlanes.BottomBound,
                        beamPlanes.BackBound,
                        beamPlanes.FrontBound,
                        beamPlanes.TopBound
                    };
                    break;
                default:
                    return null;
            }
            return planes;
        }
        /// <summary>
        /// 求给定数组中全部元素之和
        /// </summary>
        public static double SumArray(double[] arr)
        {
            if (arr == null)
                return 0.0;
            if (arr.Length == 0)
                return 0.0;
            var sum = 0.0;
            foreach (var item in arr)
            {
                sum += item;
            }
            return sum;
        }
        /// <summary>
        /// 求数组中的最大值
        /// </summary>
        public static double ArrayMax(double[] arr)
        {
            double max = double.MinValue;
            foreach(var num in arr)
            {
                if (max < num)
                    max = num;
            }
            return max;
        }
        /// <summary>
        /// 求数组中的最小值
        /// </summary>
        public static double ArrayMin(double[] arr)
        {
            double min = double.MaxValue;
            foreach (var num in arr)
            {
                if (min > num)
                    min = num;
            }
            return min;
        }
        /// <summary>
        /// 在给定平面内旋转一个三维向量指定的角度。
        /// </summary>
        /// <param name="vector">要旋转的向量。</param>
        /// <param name="planeNormal">定义旋转平面的法线向量。</param>
        /// <param name="angleDegrees">向量在平面内旋转的角度，单位为度。</param>
        /// <returns>旋转后的向量。</returns>
        public static Vec3 RotateVectorInPlane(Vec3 vector, Vec3 planeNormal, double angleDegrees)
        {
            // 将平面法线标准化，确保它是单位向量
            Vec3 normalizedPlaneNormal = planeNormal.Normalize();

            // 将角度从度转换为弧度
            double angleRadians = Math.PI / 180 * angleDegrees;

            // 根据罗德里格斯旋转公式计算旋转的各个分量
            Vec3 rotatedVector = vector * Math.Cos(angleRadians) // 向量与余弦角度的乘积
                                    + normalizedPlaneNormal.Cross(vector) * Math.Sin(angleRadians) // 法线与向量的叉乘与正弦角度的乘积
                                    + normalizedPlaneNormal * normalizedPlaneNormal.Dot(vector) * (1 - Math.Cos(angleRadians)); // 法线与法线与向量点乘的乘积，再乘以1减去余弦角度
            return rotatedVector; // 返回旋转后的向量
        }
    }
}
