﻿using WindBraceConnection110.Common;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace WindBraceConnection110.Handlers
{
    public class GraphicHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PartBase retPlate;
        private PlateProperties m_plateProps = new PlateProperties();
        private double m_thickness = double.MaxValue;
        private Plane m_baseGussetPlane;

        private List<BraceData> leftBraceData = new List<BraceData>();
        private List<BraceData> rightBraceData = new List<BraceData>();

        //构建节点板的控制点
        private List<Vec3> leftPoints = new List<Vec3>();
        private List<Vec3> rightPoints = new List<Vec3>();
        private List<Vec3> upPoints = new List<Vec3>();
        private List<Vec3> downPoints = new List<Vec3>();

        private Vec3 m_axisY = Vec3.ZERO;
        private Vec3 m_axisZ = Vec3.ZERO;

        //节点板页参数
        private int m_plateShape = 0;
        private int m_plateHole = 0;
        private double m_holeDiameter = double.MaxValue;
        private double m_holeGap = double.MaxValue;
        private int m_notch = 0;
        private double m_chamferLength = double.MaxValue;
        private double m_chamfer = double.MaxValue;
        private double m_vertNotch = double.MaxValue;
        private double m_horiNotch = double.MaxValue;

        public GraphicHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;

            //建立坐标系
            m_axisY = m_context.Y;
            m_axisZ = m_context.Z;
            m_baseGussetPlane = m_context.BaseGussetPlane;

            //执行对齐操作
            foreach(var item in m_context.BraceData)
            {
                m_context.TargetComp.CreateAlignPlane(item.Beam, item.AlignPlane, true);
            }

            //开槽
            if (m_notch == 3)
                m_context.EnableNotch = true;
            //开特殊槽口
            if (m_notch == 2)
                m_context.EnableSpecialNotch = true;
            //创建槽口
            this.CreateNotch();

            //创建节点板
            CreateGussetPlate();

            if (retPlate == null)
                return false;

            //创建螺栓组
            CreateBolts(leftBraceData.ToArray(), retPlate, false);
            CreateBolts(rightBraceData.ToArray(), retPlate, true);

            //创建孔
            CreatePlateHole();
            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.GUSSET_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);
            m_thickness = m_plateProps.Thickness;
            m_context.Thickness = m_thickness;

            //初始化上下文参数
            leftBraceData = m_context.LeftBraceData;
            rightBraceData = m_context.RightBraceData;

            leftPoints = m_context.LeftPoints;
            rightPoints = m_context.RightPoints;
            upPoints = m_context.UpPoints;
            downPoints = m_context.DownPoints;

            m_plateShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_OVERALL_SHAPE);
            m_plateHole = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_HOLE_POSITION);
            m_holeDiameter = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_HOLE_DIAMETER);
            m_holeGap = m_paramsHandler.GetDoubleValue(ParamNames.GUSSER_PLATE_HOLE_EDGE_GAP);
            m_notch = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BRACE_NOTCH);
            m_chamfer = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BRACE_CHAMFER_ANGLE);
            m_chamferLength = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BRACE_CHAMFER_LENGTH);
            m_vertNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BRACE_VERT_NOTCH_LENGTH);
            m_horiNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BRACE_HORI_NOTCH_LENGTH);

            return true;
            
        }
        private void CreateNotch()
        {
            if (m_notch < 2)
                return;
            switch (m_notch)
            {
                case 3:
                    //得到顶面
                    Plane plTop = m_baseGussetPlane.Move(0);
                    foreach (var brace in m_context.BraceData)
                    {
                        //得到侧面
                        List<Plane> plsSide = new List<Plane>();
                        plsSide.Add(brace.TopPlane);
                        plsSide.Add(brace.AlignPlane.Invert().Move(5.0));
                        plsSide.Add(brace.BottomPlane);
                        plsSide.Add(brace.GussetBorderPlane.Move(m_horiNotch));
                        m_context.TargetComp.CreatePlateCutter(brace.Beam, plTop.Move(m_thickness + m_vertNotch),
                            plTop.Invert().Move(m_vertNotch), plsSide.ToArray(), true);
                    }
                    break;
                default:
                    foreach (var brace in m_context.BraceData)
                        this.CreateHalfNotch(brace);
                    break;
            }
        }
        private void CreateHalfNotch(BraceData brace)
        {
            var type = brace.ProfileType;
            double moveLen; //切割长度
            double t;       //切割厚度
            switch (type)
            {
                case BraceProfileType.T:
                case BraceProfileType.I:
                    t = GetTCutLength(brace.BeamPlanes, false);
                    if (brace.BeamPlanes.BackWeb.Normal.IsParallelTo(m_baseGussetPlane.Normal))
                    {
                        moveLen = brace.BackPlane.Dist(m_baseGussetPlane.Origin);
                        HandlerHalfCutter(brace, t, moveLen);
                    }
                    else
                    {
                        moveLen = brace.BackPlane.Dist(brace.FrontPlane.Origin) / 2;
                        HandlerHalfCutter(brace, t, moveLen);
                    }
                    break;
                case BraceProfileType.RECTHOLLOW:
                    {
                        var eps = 5.0;
                        //得到切割体的侧面
                        List<Plane> rectSide = new List<Plane>();
                        rectSide.Add(brace.AlignPlane.Invert().Move(eps));
                        rectSide.Add(brace.TopPlane.Move(eps));
                        rectSide.Add(brace.GussetBorderPlane.Move(m_horiNotch));
                        rectSide.Add(brace.BottomPlane.Move(eps));
                        //得到顶面
                        Plane rectTop = m_baseGussetPlane.Move(0);
                        m_context.TargetComp.CreatePlateCutter(brace.Beam, rectTop.Move(m_thickness + m_vertNotch),
                            rectTop.Invert().Move(m_vertNotch), rectSide.ToArray(), true);
                    }
                    break;
                case BraceProfileType.BAN:
                case BraceProfileType.HQ:
                case BraceProfileType.WELDBOX:
                    moveLen = brace.BackPlane.Dist(brace.FrontPlane.Origin) / 2;
                    t = GetBANCutLength(brace.BeamPlanes, false);
                    HandlerHalfCutter(brace, t, moveLen);
                    break;
                default:
                    moveLen = brace.BackPlane.Dist(brace.FrontPlane.Origin) / 2;
                    t = GetLCutLength(brace.BeamPlanes, false);
                    HandlerHalfCutter(brace, t, moveLen);
                    break;
            }
        }
        //处理特殊槽口切割体
        private void HandlerHalfCutter(BraceData brace, double t, double moveLen)
        {
            var eps = 5.0;
            var len = brace.BackPlane.Dist(brace.FrontPlane.Origin);
            Plane cTop1 = brace.TopPlane.Move(eps);
            Plane cBottom1 = brace.TopPlane.Invert().Move(eps + t);
            Plane cTop2 = brace.BottomPlane.Move(eps);
            Plane cBottom2 = brace.BottomPlane.Invert().Move(eps + t);
            var borderPlane = brace.AlignPlane;
            //进行切割 Z轴负方向
            {
                if (m_baseGussetPlane.Dist(brace.BackPlane.Origin) >= len
                    || m_baseGussetPlane.Dist(brace.FrontPlane.Origin) >= len)
                    return;
                List<Plane> cSide = new List<Plane>();
                cSide.Add(borderPlane.Invert().Move(eps));
                cSide.Add(brace.BackPlane.Move(eps));
                cSide.Add(brace.GussetBorderPlane.Move(m_horiNotch));
                cSide.Add(brace.BackPlane.Invert().Move(moveLen));
                m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop1, cBottom1, cSide.ToArray(), true);
                m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop2, cBottom2, cSide.ToArray(), true);
            }
            //折角操作
            {
                if (m_chamfer <= 0 || m_chamfer >= 90 || m_chamferLength >= len - moveLen)
                    return;
                //切割长度
                var chamferLength = len - moveLen - m_chamferLength;
                //转化为弧度
                var radian = m_chamfer * Math.PI / 180;
                //顶
                {
                    var point = borderPlane.Intersect(brace.TopPlane, brace.FrontPlane);
                    //沿着X轴正方向
                    Vec3 pointX = point + brace.BraceLine.Normalize() * chamferLength * Math.Tan(radian);
                    //沿着Z轴负方向
                    Vec3 pointZ = point - brace.FrontPlane.Normal.Normalize() * chamferLength;
                    //定义Y轴方向
                    Vec3 pointY = pointZ + brace.BottomPlane.Normal.Normalize() * t;
                    List<Plane> plsSide = new List<Plane>();
                    plsSide.Add(new Plane(pointX, pointX - pointZ).Move(2 * eps));

                    var normal_1 = (pointY - pointZ).Normalize().Cross((pointX - pointZ).Normalize());
                    var normal_2 = - normal_1;

                    var normal = normal_1.Dot(brace.BraceLine) > 0 ? normal_1 : normal_2;
                    plsSide.Add(new Plane(pointZ, normal));

                    plsSide.Add(plsSide[0].Invert().Move(chamferLength / Math.Cos(radian) + 4 * eps));
                    plsSide.Add(plsSide[1].Invert().Move(chamferLength / Math.Sin(radian)));
                    m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop1, cBottom1, plsSide.ToArray(), true);
                }
                //底
                {
                    var point = borderPlane.Intersect(brace.BottomPlane, brace.FrontPlane);
                    //沿着X轴正方向
                    Vec3 pointX = point + brace.BraceLine.Normalize() * chamferLength * Math.Tan(radian);
                    //沿着Z轴负方向
                    Vec3 pointZ = point - brace.FrontPlane.Normal.Normalize() * chamferLength;
                    //定义Y轴方向
                    Vec3 pointY = pointZ + brace.TopPlane.Normal.Normalize() * t;
                    List<Plane> plsSide = new List<Plane>();
                    plsSide.Add(new Plane(pointX, pointX - pointZ).Move(2 * eps));

                    var normal_1 = (pointY - pointZ).Normalize().Cross((pointX - pointZ).Normalize());
                    var normal_2 = -normal_1;

                    var normal = normal_1.Dot(brace.BraceLine) > 0 ? normal_1 : normal_2;
                    plsSide.Add(new Plane(pointZ, normal));

                    plsSide.Add(plsSide[0].Invert().Move(chamferLength / Math.Cos(radian) + 4 * eps));
                    plsSide.Add(plsSide[1].Invert().Move(chamferLength / Math.Sin(radian)));
                    m_context.TargetComp.CreatePlateCutter(brace.Beam, cTop2, cBottom2, plsSide.ToArray(), true);
                }
            }
        }
        /// <summary>
        /// 创建节点板
        /// </summary>
        private void CreateGussetPlate()
        {
            if (m_plateShape == 2)
                retPlate = GetSquarePlate();
            else
                retPlate = GetConvexPlate();

            if (retPlate == null)
                return;

            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);
            m_context.RetPlate = retPlate;
        }
        private PartBase GetConvexPlate()
        {
            //上右下左
            List<Vec3> points = new List<Vec3>();
            foreach (var item in upPoints)
            {
                if(!points.Contains(item))
                    points.Add(item);
            }
                
            foreach (var item in rightPoints)
            {
                if (!points.Contains(item))
                    points.Add(item);
            }
            foreach (var item in downPoints)
            {
                if (!points.Contains(item))
                    points.Add(item);
            }
            foreach (var item in leftPoints)
            {
                if (!points.Contains(item))
                    points.Add(item);
            }

            var Plate = m_context.TargetComp.CreateSteelPlate(AdjustGussetDepth(points),
                m_thickness, LoongCAD.STC.Define.ObjectSide.SIDE_LEFT, true);

            return Plate;
        }
        private PartBase GetSquarePlate()
        {
            var points = new List<Vec3>();
            //上下
            foreach (var item in upPoints)
                points.Add(item);
            foreach (var item in downPoints)
                points.Add(item);
            var Plate = m_context.TargetComp.CreateSteelPlate(AdjustGussetDepth(points),
                m_thickness, LoongCAD.STC.Define.ObjectSide.SIDE_LEFT, true);

            return Plate;
        }
        /// <summary>
        /// 调整节点板深度
        /// </summary>
        private Vec3[] AdjustGussetDepth(List<Vec3> points)
        {
            var arr = points.ToArray();
            var axisX = (arr[2] - arr[1]).Normalize();
            var axisY = (arr[0] - arr[1]).Normalize();
            var axisZ = axisX.Cross(axisY);
            return axisZ.Dot(m_baseGussetPlane.Normal) > 0 ? arr : points.Reverse<Vec3>().ToArray();
        }
        /// <summary>
        /// 创建螺栓组
        /// <param name="flag">true时为右侧支撑，false时为左侧支撑</param>
        /// </summary>
        private bool CreateBolts(BraceData[] braces, PartBase retPlate, bool flag)
        {
            BoltParse boltParse = new BoltParse(m_context, m_paramsHandler);
            if (!boltParse.Execute()) return false;
            List<BoltParse.ParamData> BoltParams = boltParse.ParamDataMap;
            var target = m_context.TargetComp;
            for (int i = 0; i < braces.Length; i++)
            {
                BoltParse.ParamData boltGroup;
                var brace = braces[i];
                if (flag)
                    boltGroup = BoltParams.First();
                else
                    boltGroup = BoltParams.Last();
                if (boltGroup.HorBoltsSpace == null || boltGroup.VerBoltsSpace == null)
                    continue;

                Vec3 bolt_Y = brace.BottomPlane.Normal.Normalize();
                if (flag)
                    //统一方向
                    bolt_Y = -bolt_Y;
                Vec3 bolt_X = brace.BraceLine.Normalize();
                Vec3 bolt_Z = bolt_X.Cross(bolt_Y);
                var topPoint = brace.GussetBorderPlane.Intersect(m_baseGussetPlane, brace.TopPlane);
                var bottomPoint = brace.GussetBorderPlane.Intersect(m_baseGussetPlane, brace.BottomPlane);
                Vec3 posStart = (topPoint + bottomPoint) / 2;
                Vec3 posEnd = posStart + bolt_X * 1000;
                BoltGroup bolts = target.CreateBoltGroup(brace.Beam, retPlate, posStart, posEnd, true);
                if (bolts == null)
                    return false;
                bolts.WorkAxis = new Mat43(bolt_X, bolt_Y, bolt_Z, Vec3.ZERO);
                bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
                bolts.Rotation = 0;
                boltGroup.BoltsCfg.FillBasicProps(bolts, true);
                boltGroup.BoltsCfg.FillHoleProps(bolts);
                boltGroup.BoltsCfg.FillContentProps(bolts);
                boltGroup.BoltsCfg.FillExtendProps(bolts);
                boltGroup.BoltsCfg.FillShape(bolts);
                bolts.ArrayStrideX = boltGroup.HorBoltsSpace;
                bolts.ArrayStrideY = boltGroup.VerBoltsSpace;
                if (boltGroup.VerFarEdgeDist == double.MaxValue)
                    bolts.HeadOffsetX = 0;
                else
                    bolts.HeadOffsetX = -(boltGroup.VerFarEdgeDist + BraceTools.SumArray(bolts.ArrayStrideX));
                    
                //计算纵向偏移距离
                var dist = brace.TopPlane.Dist(brace.BottomPlane.Origin);
                var NonMiddle = BraceTools.SumArray(bolts.ArrayStrideY);
                //L型
                double vertOffset = AdjustVertOffset(boltGroup.VerMeasureMethod,dist, NonMiddle, boltGroup.VerMeasureOffset);
                bolts.VerticalOffset = vertOffset;
                //计算切割长度
                if (m_context.EnableNotch || m_context.EnableSpecialNotch)
                {
                    //开启槽口的情况
                    var CutLength = brace.BackPlane.Dist(brace.FrontPlane.Origin);
                    bolts.CutLength = CutLength + 25.0;
                }
                else
                {
                    double CutLength;
                    switch (brace.ProfileType)
                    {
                        case BraceProfileType.T:
                        case BraceProfileType.I:
                            CutLength = GetTCutLength(m_context.MainPlanes, true);
                            break;
                        case BraceProfileType.BAN:
                        case BraceProfileType.HQ:
                        case BraceProfileType.WELDBOX:
                            CutLength = GetBANCutLength(m_context.MainPlanes, true);
                            break;
                        default:
                            CutLength = GetLCutLength(m_context.MainPlanes, true);
                            break;
                    }
                    bolts.CutLength = 2 * (CutLength + m_thickness) + 1.0;
                }
                bolts.Refresh();
            }
            return true;
        }
        private void CreatePlateHole()
        {
            if (m_plateHole <= 1 || m_holeDiameter <= 0)
                return;

            // 创建螺栓坐标系
            Mat43 boltAxis;
            Vec3 startPos;
            Vec3 endPos;
            var boltY = m_axisY;
            switch (m_plateHole)
            {
                case 2:
                    {
                        //右侧打孔
                        var BasePlane = m_context.FaceRightBracePlane;
                        var boltX = -BasePlane.Normal.Normalize();
                        var boltZ = boltX.Cross(boltY);
                        boltAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
                        var offset = GetMaxpointDist(rightBraceData, BasePlane);
                        
                        var upPlane = new Plane(upPoints.Last(), m_axisY);
                        Vec3 upPoint = upPlane.Intersect(m_baseGussetPlane, BasePlane);
                        var downPlane = new Plane(downPoints.First(), m_axisY);
                        Vec3 downPoint = downPlane.Intersect(m_baseGussetPlane, BasePlane);
                        startPos = upPoint / 2 + downPoint / 2 - boltX * offset;
                        if (m_holeGap != double.MaxValue)
                        {
                            startPos += m_holeGap * boltX;
                        }
                        else
                        {
                            var point1 = rightBraceData.First().TopPoint;
                            var point2 = rightBraceData.Last().BottomPoint;
                            var plane = new Plane(point1, point2 - point1, m_axisZ);
                            double dist = startPos.Dist(plane.Intersect(m_baseGussetPlane, new Plane(startPos, m_axisY)));
                            startPos += dist * boltX;
                        }
                        endPos = startPos + boltX * 1000;
                     }
                    break;
                default:
                    {
                        //左侧打孔
                        var BasePlane = m_context.FaceLeftBracePlane;
                        var boltX = - BasePlane.Normal.Normalize();
                        var boltZ = boltX.Cross(boltY);
                        boltAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
                        var offset = GetMaxpointDist(leftBraceData, BasePlane);

                        var upPlane = new Plane(upPoints.First(), m_axisY);
                        Vec3 upPoint = upPlane.Intersect(m_baseGussetPlane, BasePlane);
                        var downPlane = new Plane(downPoints.Last(), m_axisY);
                        Vec3 downPoint = downPlane.Intersect(m_baseGussetPlane, BasePlane);
                        startPos = upPoint / 2 + downPoint / 2 - boltX * offset;
                        if (m_holeGap != double.MaxValue)
                        {
                            startPos += m_holeGap * boltX;
                        }
                        else
                        {
                            var point1 = leftBraceData.First().TopPoint;
                            var point2 = leftBraceData.Last().BottomPoint;
                            var plane = new Plane(point1, point2 - point1, m_axisZ);
                            double dist = startPos.Dist(plane.Intersect(m_baseGussetPlane, new Plane(startPos, m_axisY)));
                            startPos += dist * boltX;
                        }
                        endPos = startPos + boltX * 1000;
                    }
                    break;
            }
            BoltGroup curHoles = m_context.TargetComp.CreateBoltHoles(null, retPlate, startPos, endPos, true);
            if (curHoles == null)
                return;

            const double eps = 1.0;
            curHoles.WorkAxis = boltAxis;
            curHoles.RotationSide = Define.ObjectSide.SIDE_TOP;
            curHoles.Rotation = 0;
            curHoles.BoltSize = m_holeDiameter;
            curHoles.HoleX = 0.0;
            curHoles.HoleY = 0.0;
            curHoles.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
            curHoles.OverSizeHoleEnable0 = false;
            curHoles.OverSizeHoleEnable1 = false;
            curHoles.CutLength = m_plateProps.Thickness * 2.0 + eps;
            curHoles.Refresh();
        }
        /// <summary>
        /// 得到某侧节点板控制点到指定面距离的最大值
        /// </summary>
        /// <param name="braces"></param>
        /// <param name="plane"></param>
        /// <returns></returns>
        private double GetMaxpointDist(List<BraceData> braces, Plane plane)
        {
            double max = -1.0;
            foreach (var brace in braces)
            {
                var dist1 = plane.Dist(brace.TopPoint);
                var dist2 = plane.Dist(brace.BottomPoint);

                var temp = Math.Max(dist1, dist2);
                if (temp > max)
                    max = temp;
            }
            return max;
        }
        /// <summary>
        /// 调整螺栓纵向偏移距离的测量方式
        /// </summary>
        ///  /// <param name="type">纵向测量类型</param>
        /// <param name="dist">支撑截面高度</param>
        /// <param name="NonMiddle">纵向螺栓组总间距</param>
        /// <param name="PanelValue">螺栓面板填写值</param>
        private double AdjustVertOffset(int type, double dist, double NonMiddle, double PanelValue)
        {
            double VertOffset;
            switch (type)
            {
                case 1:
                    VertOffset = -dist / 2 + NonMiddle / 2 + PanelValue;
                    break;
                case 3:
                    VertOffset = dist / 2 - NonMiddle / 2 - PanelValue;
                    break;
                case 0:
                default:
                    VertOffset = PanelValue;
                    break;

            }
            return VertOffset;
        }
        /// <summary>
        /// 得到L型 C型 U型 CC型 []型截面的切割厚度
        /// </summary>
        /// <param name="flag">true为Z轴方向，false为Y轴方向</param>
        private double GetLCutLength(BeamPlanes beamPlanes, bool flag)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BottomFlange.Normal.Normalize(),
                beamPlanes.BackWeb.Normal.Normalize()
            };
            var dot1 = Math.Abs(m_axisZ.Dot(normals[0]));
            var dot2 = Math.Abs(m_axisZ.Dot(normals[1]));
            if (dot1 > dot2 == flag)
            {
                CutLength = beamPlanes.BottomFlange.Dist(beamPlanes.BottomFlangeInside.Origin);
                return CutLength;
            }
            CutLength = beamPlanes.BackWeb.Dist(beamPlanes.BackWebInside.Origin);
            return CutLength;
        }
        /// <summary>
        /// 得到T型 I型截面的切割厚度
        /// </summary>
        /// <param name="flag">true为Z轴方向，false为Y轴方向</param>
        private double GetTCutLength(BeamPlanes beamPlanes, bool flag)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BackWeb.Normal.Normalize(),
                beamPlanes.TopFlange.Normal.Normalize(),
            };
            var dot1 = Math.Abs(m_context.Z.Dot(normals[0]));
            var dot2 = Math.Abs(m_context.Z.Dot(normals[1]));
            if (dot1 > dot2 == flag)
            {
                CutLength = beamPlanes.BackBound.Dist(beamPlanes.FrontBound.Origin) / 2;
                return CutLength;
            }
            CutLength = beamPlanes.TopFlange.Dist(beamPlanes.TopFlangeInside.Origin);
            return CutLength;
        }
        /// <summary>
        /// 得到板 HQ型 焊接箱型截面的切割厚度
        /// </summary>
        /// <param name="flag">true为Z轴方向，false为Y轴方向</param>
        private double GetBANCutLength(BeamPlanes beamPlanes, bool flag)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BackBound.Normal.Normalize(),
                beamPlanes.TopBound.Normal.Normalize(),
            };
            var dot1 = Math.Abs(m_context.Z.Dot(normals[0]));
            var dot2 = Math.Abs(m_context.Z.Dot(normals[1]));
            if (dot1 > dot2 == flag)
            {
                CutLength = beamPlanes.BackBound.Dist(beamPlanes.FrontBound.Origin) / 2;
                return CutLength;
            }
            CutLength = beamPlanes.TopBound.Dist(beamPlanes.BottomBound.Origin) / 2;
            return CutLength;
        }
    }
}
