﻿using WindBraceConnection110.Common;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace WindBraceConnection110.Handlers
{
    public class GussetBoltHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PartBase retPlate;
        private Plane m_faceLeftBracePlane;
        private Plane m_faceRightBracePlane;
        private List<Vec3> upPoints = new List<Vec3>();
        private List<Vec3> downPoints = new List<Vec3>();

        //节点板螺栓页参数
        private int m_enabled = 0;
        private int m_horBoltNum = 0;
        private int m_verBoltNum = 0;
        private double[] m_arrayStrideX = null;
        private double[] m_arrayStrideY = null;
        private int m_horiBoltOffsetMode = 0;
        private int m_vertBoltOffsetMode = 0;
        private double m_horiBoltOffset = double.MaxValue;
        private double m_vertBoltOffset = double.MaxValue;
        private BoltsConfig BoltsCfg { get; set; } = new BoltsConfig();

        private Vec3 boltX = Vec3.ZERO;
        private Vec3 boltY = Vec3.ZERO;
        private Vec3 boltZ = Vec3.ZERO;

        private Vec3 startPos = Vec3.ZERO;
        private Vec3 endPos = Vec3.ZERO;
        private double HoriLength = double.MaxValue;

        public GussetBoltHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            if (!ParseProps())
                return;

            //初始化上下文参数
            retPlate = m_context.RetPlate;
            m_faceLeftBracePlane = m_context.FaceLeftBracePlane;
            m_faceRightBracePlane = m_context.FaceRightBracePlane;
            upPoints = m_context.UpPoints;
            downPoints = m_context.DownPoints;

            //自适应螺栓间距
            if (!AdaptBoltStride())
                return;

            //调整垂直方向
            AdjustVertOffset();

            //调整水平方向
            AdjustHoriOffset();

            CreateBoltGroup();
        }
        private bool ParseProps()
        {
            m_enabled = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_ENABLED);
            if (m_enabled != 2)
                return false;

            BoltsCfg.CopyFromBasicProps(m_paramsHandler.GetDObjectValue(ParamNames.GUSSET_BOLT_BASIC_PROPS));
            BoltsCfg.CopyFromShapeProps(m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_SHAPE));
            m_horBoltNum = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_HORI_COUNT);
            m_verBoltNum = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_VERT_COUNT);
            m_horiBoltOffsetMode = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_HORI_OFFSET_MODE);
            m_horiBoltOffset = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BOLT_HORI_OFFSET);
            m_vertBoltOffsetMode = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_VERT_OFFSET_MODE);
            m_vertBoltOffset = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BOLT_VERT_OFFSET);

            return true;
        }
        private bool AdaptBoltStride()
        {
            var arrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.GUSSET_BOLT_HORI_STRIDE);
            var arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.GUSSET_BOLT_VERT_STRIDE);
            if (m_horBoltNum <= 0 || m_verBoltNum <= 0)
                return false;

            if(arrX.Length == 0)
            {
                var startPos = GetBoltPoint(upPoints);
                var endPos = GetBoltPoint(downPoints);
                var VertLength = startPos.Dist(endPos);
                var strideX = VertLength / (m_verBoltNum + 1);
                if (strideX < 0.5)
                    return false;
                var listX = new List<double>();
                listX.AddRange(Enumerable.Repeat(strideX, m_verBoltNum - 1));
                m_arrayStrideX = listX.ToArray();
            }
            else
                m_arrayStrideX = HandlerSpacing(arrX, m_verBoltNum);

            if(arrY.Length == 0)
            {
                HoriLength = m_faceLeftBracePlane.Dist(m_faceRightBracePlane.Origin);
                var listY = new List<double>();
                var strideY = HoriLength / (m_horBoltNum + 1);
                if (strideY < 0.5)
                    return false;
                listY.AddRange(Enumerable.Repeat(strideY, m_horBoltNum - 1));
                m_arrayStrideY = listY.ToArray();
            }
            else
                m_arrayStrideY = HandlerSpacing(arrY, m_horBoltNum);
            return true;
        }
        /// <summary>
        /// 得到螺栓组的控制点
        /// </summary>
        private Vec3 GetBoltPoint(List<Vec3> points)
        {
            var ctrlPoints = points.ToArray();
            var len = ctrlPoints.Length;
            if (len == 2)
            {
                var axis = m_context.Z;
                var plane = new Plane(ctrlPoints[0], ctrlPoints[1] - ctrlPoints[0], axis);
                var pos0 = plane.Intersect(m_context.BaseGussetPlane, m_faceLeftBracePlane);
                var pos1 = plane.Intersect(m_context.BaseGussetPlane, m_faceRightBracePlane);
                return pos0 / 2 + pos1 / 2;
            }
            else if (len == 4)
                return ctrlPoints[1] / 2 + ctrlPoints[2] / 2;
            else
                return Vec3.ZERO;
        }
        private void CreateBoltGroup()
        {
            endPos = startPos + boltX * 1000;
            BoltGroup bolts = m_context.TargetComp.CreateBoltGroup
                (m_context.MainPart, retPlate, startPos, endPos, true);

            if (bolts == null)
                return;

            boltZ = m_context.Z;
            bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            BoltsCfg.FillBasicProps(bolts, true);
            BoltsCfg.FillShape(bolts);
            bolts.ArrayStrideX = m_arrayStrideX;
            bolts.ArrayStrideY = m_arrayStrideY;
            bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
            bolts.Rotation = 0;
            bolts.HeadOffsetX = m_vertBoltOffset;

            var type = BraceTools.ParseBraceProfileType(m_context.MainPart);
            double cutLength;
            switch (type)
            {
                case BraceProfileType.T:
                case BraceProfileType.I:
                    cutLength = GetTCutLength(m_context.MainPlanes);
                    break;
                case BraceProfileType.BAN:
                case BraceProfileType.OTHER:
                case BraceProfileType.HQ:
                case BraceProfileType.WELDBOX:
                    cutLength = GetBANCutLength(m_context.MainPlanes);
                    break;
                default:
                    cutLength = GetLCutLength(m_context.MainPlanes);
                    break;
            }
            bolts.CutLength = 2 * (cutLength + m_context.Thickness) + 1.0;
            bolts.Refresh();
        }
        /// <summary>
        /// 调整螺栓组垂直方向
        /// </summary>
        private void AdjustVertOffset()
        {
            switch (m_vertBoltOffsetMode)
            {
                case 1:
                    boltX = m_context.Y;
                    startPos = GetBoltPoint(upPoints);
                    break;
                case 3:
                    boltX = - m_context.Y;
                    startPos = GetBoltPoint(downPoints);
                    break;
                case 0:
                default:
                    boltX = m_context.Y;
                    startPos = GetBoltPoint(upPoints) / 2 + GetBoltPoint(downPoints) / 2;
                    m_vertBoltOffset -= BraceTools.SumArray(m_arrayStrideX) / 2;
                    break;
            }
        }
        /// <summary>
        /// 调整螺栓组水平方向
        /// </summary>
        private void AdjustHoriOffset()
        {
            double offSet;
            if(HoriLength != double.MaxValue)
            {
                offSet = HoriLength / 2 - BraceTools.SumArray(m_arrayStrideY) / 2 - m_horiBoltOffset;
            }
            else
            {
                var len = m_faceLeftBracePlane.Dist(m_faceRightBracePlane.Origin);
                offSet = len / 2 + BraceTools.SumArray(m_arrayStrideY) / 2 - m_horiBoltOffset;
            }
            switch (m_horiBoltOffsetMode)
            {
                case 1:
                    boltY = m_faceLeftBracePlane.Normal.Normalize();
                    startPos += boltY * offSet;
                    break;
                case 3:
                    boltY = m_faceRightBracePlane.Normal.Normalize();
                    startPos += boltY * offSet;
                    break;
                case 0:
                default:
                    boltY = m_faceRightBracePlane.Normal.Normalize();
                    startPos += boltY * m_horiBoltOffset;
                    break;
            }
        }
        /// <summary>
        /// 得到L型 C型 U型 CC型 []型截面的切割厚度
        /// </summary>
        private double GetLCutLength(BeamPlanes beamPlanes)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BottomFlange.Normal.Normalize(),
                beamPlanes.BackWeb.Normal.Normalize()
            };
            var dot1 = Math.Abs(m_context.Z.Dot(normals[0]));
            var dot2 = Math.Abs(m_context.Z.Dot(normals[1]));
            if (dot1 > dot2)
            {
                CutLength = beamPlanes.BottomFlange.Dist(beamPlanes.BottomFlangeInside.Origin);
                return CutLength;
            }
            CutLength = beamPlanes.BackWeb.Dist(beamPlanes.BackWebInside.Origin);
            return CutLength;
        }
        /// <summary>
        /// 得到T型 I型截面的切割厚度
        /// <summary>
        private double GetTCutLength(BeamPlanes beamPlanes)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BackWeb.Normal.Normalize(),
                beamPlanes.TopFlange.Normal.Normalize(),
            };
            var dot1 = Math.Abs(m_context.Z.Dot(normals[0]));
            var dot2 = Math.Abs(m_context.Z.Dot(normals[1]));
            if (dot1 > dot2)
            {
                CutLength = beamPlanes.BackBound.Dist(beamPlanes.FrontBound.Origin) / 2;
                return CutLength;
            }
            CutLength = beamPlanes.TopFlange.Dist(beamPlanes.TopFlangeInside.Origin);
            return CutLength;
        }
        /// <summary>
        /// 得到板 HQ型 焊接箱型截面的切割厚度
        /// </summary>
        private double GetBANCutLength(BeamPlanes beamPlanes)
        {
            double CutLength;
            Vec3[] normals = new Vec3[]
            {
                beamPlanes.BackBound.Normal.Normalize(),
                beamPlanes.TopBound.Normal.Normalize(),
            };
            var dot1 = Math.Abs(m_context.Z.Dot(normals[0]));
            var dot2 = Math.Abs(m_context.Z.Dot(normals[1]));
            if (dot1 > dot2)
            {
                CutLength = beamPlanes.BackBound.Dist(beamPlanes.FrontBound.Origin) / 2;
                return CutLength;
            }
            CutLength = beamPlanes.TopBound.Dist(beamPlanes.BottomBound.Origin) / 2;
            return CutLength;
        }
        /// <summary>
        /// 根据螺栓数目调整螺栓间距的设置
        /// </summary>
        /// <param name="arr"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        private double[] HandlerSpacing(double[] arr, int num)
        {
            var ret = new List<double>();
            for (int i = 0; i < num - 1; i++)
            {
                if (i > arr.Length - 1)
                {
                    ret.Add(arr[arr.Length - 1]);
                }
                else
                {
                    ret.Add(arr[i]);
                }
            }
            return ret.ToArray();
        }
    }
}
