﻿using WindBraceConnection110.Common;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using System;

namespace WindBraceConnection110.Handlers
{
    /// <summary>
    /// 梁零件处理
    /// </summary>
    public class PartHandler
    {
        private ComponentInstanceContext m_context;

        private List<BraceData> m_braceData = new List<BraceData>();
        private List<BraceData> m_leftBraceData = new List<BraceData>();
        private List<BraceData> m_rightBraceData = new List<BraceData>();

        private Vec3 m_axisX = Vec3.ZERO;
        private Vec3 m_axisY = Vec3.ZERO;
        private Vec3 m_axisZ = Vec3.ZERO;

        public PartHandler(ComponentInstanceContext context)
        {
            m_context = context;
        }

        public bool Execute()
        {
            //建立坐标系
            m_axisX = m_context.TargetComp.LocalMatrix.X;
            m_axisY = m_context.MainLine;
            m_axisZ = m_axisX.Cross(m_axisY);

            //解析组件的各个侧面
            if (!ParseBraceData())
                return false;
            if (!DefineBraceBuild())
                return false;
            if (!ParseLeftAndRightBrace())
                return false;

            //存储节点板构建平面
            if (!ParseBraceType())
                return false;
            if (!GetBaseGussetPlane())
                return false;

            //解析支撑的各个面
            if (!ParseBraceTopAndBottom()) 
                return false;
            if (!ParseBraceFrontAndBack())
                return false;

            //解析主零件面向支撑侧的面
            if (!PrepareLeftMainPlane())
                return false;
            if (!PrepareRightMainPlane())
                return false;

            //存储上下文变量
            m_context.LeftBraceData = m_leftBraceData;
            m_context.RightBraceData = m_rightBraceData;
            m_context.X = m_axisX;
            m_context.Y = m_axisY;
            m_context.Z = m_axisZ;
            return true;
        }
        /// <summary>
        /// 解析支撑
        /// </summary>
        private bool ParseBraceData()
        {
            var len = m_context.TargetComp.Subs.Length;
            for (int i = 0; i < len; i++)
            {
                var item = m_context.TargetComp.Subs[i];
                if (item != null)
                {
                    Beam beam = item as Beam;
                    if (beam == null)
                        return false;
                    var braceData = new BraceData
                    {
                        Beam = beam,
                        BeamPlanes = BeamPlanes.CreateFromBeam(beam),
                        ClickNumber = i
                    };
                    m_braceData.Add(braceData);
                }
            }
            if (m_braceData == null || m_braceData.Count != len)
                return false;
            m_context.BraceData = m_braceData;
            return true;
        }
        /// <summary>
        /// 定义支撑构建方向
        /// </summary>
        private bool DefineBraceBuild()
        {
            var linePoint = m_context.MainPart.CtrlPoints.First().WorldPos;
            foreach(var brace in m_braceData)
            {
                var firstCtrlPoint = brace.Beam?.CtrlPoints?.First()?.WorldPos;
                var lastCtrlPoint = brace.Beam?.CtrlPoints?.Last()?.WorldPos;
                if (!firstCtrlPoint.HasValue || !lastCtrlPoint.HasValue)
                    return false;

                var dist1 = (firstCtrlPoint.Value - linePoint).Cross(m_axisY).Length;
                var dist2 = (lastCtrlPoint.Value - linePoint).Cross(m_axisY).Length;
                if (dist2 > dist1)
                {
                    brace.BraceLine = (lastCtrlPoint.Value - firstCtrlPoint.Value).Normalize();
                    brace.FarPoint = lastCtrlPoint.Value;
                }
                else
                {
                    brace.BraceLine = -(lastCtrlPoint.Value - firstCtrlPoint.Value).Normalize();
                    brace.FarPoint = firstCtrlPoint.Value;
                }
            }

            return true;
        }
        ///<summary>
        ///解析左右侧支撑 定义组件坐标系X轴正方向为左侧
        ///</summary>
        private bool ParseLeftAndRightBrace()
        {
            var LeftBraceData = new List<BraceData>();
            var RightBraceData = new List<BraceData>();
            foreach(var brace in m_braceData)
            {
                var dot = m_axisX.Dot(brace.BraceLine);
                if (dot > 0)
                    LeftBraceData.Add(brace);
                else
                    RightBraceData.Add(brace);
            }
            if (LeftBraceData.Count < 1 || RightBraceData.Count < 1)
                return false;

            m_leftBraceData = SortBraceData(LeftBraceData);
            m_rightBraceData = SortBraceData(RightBraceData);
            return true;
        }
        /// <summary>
        /// 按照支撑构建方向与主零件构建方向对支撑进行排序
        /// </summary>
        private List<BraceData> SortBraceData(List<BraceData> braceDatas)
        {
            var len = braceDatas.Count;
            if (len < 2)
                return braceDatas;
            List<BraceData> braces = new List<BraceData>();
            int[] sort = new int[len];
            for (int i = 0; i < len; i++)
                sort[i] = i;
            //排序
            for (int i = 0; i < len - 1; i++)
            {
                for (int j = 0; j < len - i - 1; j++)
                {
                    double dot1 = braceDatas[sort[j]].BraceLine.Dot(m_axisY);
                    double dot2 = braceDatas[sort[j + 1]].BraceLine.Normalize().Dot(m_axisY);
                    if(dot2 < dot1)
                    {
                        sort[j] = sort[j] ^ sort[j + 1];
                        sort[j + 1] = sort[j] ^ sort[j + 1];
                        sort[j] = sort[j] ^ sort[j + 1];
                    }
                }
            }
            //创建新的支撑数据集合
            for(int i = 0; i < len; i++)
            {
                BraceData brace = braceDatas[sort[i]];
                if (i == 0)
                    brace.PosType = BracePosType.FIRST;
                else if (i == len - 1)
                    brace.PosType = BracePosType.MIDDLE;
                else
                    brace.PosType = BracePosType.LAST;

                braces.Add(brace);
            }
            return braces;
        }
        /// <summary>
        /// 得到节点板的构建平面
        /// </summary>
        private bool GetBaseGussetPlane()
        {
            //先找到右侧中第一个点击的支撑
            int click = int.MaxValue;
            int index = -1;
            for(int i = 0; i < m_rightBraceData.Count; i++)
            {
                if(m_rightBraceData[i].ClickNumber < click)
                {
                    click = m_rightBraceData[i].ClickNumber;
                    index = i;
                }
                    
            }
            var brace = m_rightBraceData[index];
            var type = brace.ProfileType;
            var planes = BraceTools.ParseBasePlane(type, m_axisZ, brace.BeamPlanes);
            if (planes == null)
                return false;
            m_context.BracePlane = planes[0];
            m_context.OppositeBracePlane = planes[1];

            //处理I型 T型的腹板问题
            if (type != BraceProfileType.T && type != BraceProfileType.I)
                return true;
            var flag = brace.BeamPlanes.BackWeb.Normal.Normalize().Dot(m_axisZ);
            if (flag > 1e-6)
            {
                m_context.BraceOffsetPlane = brace.BeamPlanes.BackWeb;
                m_context.OppositeBraceOffsetPlane = brace.BeamPlanes.BackWebInside;
            }
            else if (Math.Abs(flag) > 1e-6)
            {
                m_context.BraceOffsetPlane = brace.BeamPlanes.BackWebInside;
                m_context.OppositeBraceOffsetPlane = brace.BeamPlanes.BackWeb;
            }

            return true;
        }
        /// <summary>
        /// 解析支撑的顶底面 定义顶为解析面中沿着主零件构建方向的第一个面
        /// </summary>
        private bool ParseBraceTopAndBottom()
        {
            foreach(var brace in m_braceData)
            {
                var planes = BraceTools.DotExtremeBoundPlanes(brace.BeamPlanes, m_axisY, true);
                if (planes == null)
                    return false;
                var dot1 = planes[0].Normal.Normalize().Dot(m_axisY);
                var dot2 = planes[1].Normal.Normalize().Dot(m_axisY);
                if(dot1 < dot2)
                {
                    brace.TopPlane = planes[0];
                    brace.BottomPlane = planes[1];
                }
                else
                {
                    brace.TopPlane = planes[1];
                    brace.BottomPlane = planes[0];
                }
            }
            return true;
        }
        /// <summary>
        /// 解析支撑的前后面 定义前为解析面法线中与Z轴方向同向的面
        /// </summary>
        private bool ParseBraceFrontAndBack()
        {
            foreach (var brace in m_braceData)
            {
                var planes = BraceTools.DotExtremeBoundPlanes(brace.BeamPlanes, m_axisY, false);
                if (planes == null)
                    return false;
                var dot1 = planes[0].Normal.Normalize().Dot(m_axisZ);
                var dot2 = planes[1].Normal.Normalize().Dot(m_axisZ);
                if (dot1 > dot2)
                {
                    brace.FrontPlane = planes[0];
                    brace.BackPlane = planes[1];
                }
                else
                {
                    brace.FrontPlane = planes[1];
                    brace.BackPlane = planes[0];
                }
            }
            return true;
        }
        /// <summary>
        /// 解析支撑的截面类型
        /// </summary>
        private bool ParseBraceType()
        {
            foreach(var brace in m_braceData)
            {
                var type = BraceTools.ParseBraceProfileType(brace.Beam);
                if (type == BraceProfileType.OTHER)
                    return false;
                brace.ProfileType = type;
            }
            return true;
        }
        /// <summary>
        /// 准备主零件面向左侧支撑的面
        /// </summary>
        private bool PrepareLeftMainPlane()
        {
            var brace = m_leftBraceData.First();
            var plane = BraceTools.DotExtremeBoundPlane(m_context.MainPlanes, brace.BraceLine.Normalize(), true);
            if (plane == null)
                return false;
            m_context.FaceLeftBracePlane = plane;
            return true; ;
        }
        ///<summary>
        ///准备主零件面向右侧支撑的面
        ///</summary>
        private bool PrepareRightMainPlane()
        {
            var brace = m_rightBraceData.First();
            var plane = BraceTools.DotExtremeBoundPlane(m_context.MainPlanes, brace.BraceLine.Normalize(), true);
            if (plane == null)
                return false;
            m_context.FaceRightBracePlane = plane;
            return true;
        }
    }
}
