﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using WindBraceConnection110.Common;
using System.Collections.Generic;
using System;

namespace WindBraceConnection110.Handlers
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        private PartBase m_retPlate;
        private Plane m_baseGussetPlane;
        private List<BraceData> m_braceData = new List<BraceData>();

        private int m_boltEnabled = 0;
        private int m_braceWeldEnabled = 0;
        private double m_horiNotch = double.MaxValue;
        private double m_vertNotch = double.MaxValue;
        private int m_plateShape = 0;

        private bool WeldFlag = false;
        private PartBase WeldMain;

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }
        public void Execute()
        {
            if (!ParseProps())
                return;

            //初始化上下文参数
            m_baseGussetPlane = m_context.BaseGussetPlane;
            m_braceData = m_context.BraceData;
            m_retPlate = m_context.RetPlate;

            //如果未启用节点板螺栓，则创建主零件焊缝
            if (m_boltEnabled != 2 && (m_weldProps[0].SizeDown > 0 || m_weldProps[0].SizeUp > 0))
                WeldingMainAndPlate();
            if (WeldFlag)
                WeldMain = m_context.MainPart;
            else
                WeldMain = m_retPlate;

            //创建支撑的焊缝
            if (m_weldProps[1].SizeUp > 0 || m_weldProps[1].SizeDown > 0)
                CreateWelds();
        }
        private bool ParseProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }

            m_boltEnabled = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BOLT_ENABLED);
            m_braceWeldEnabled = m_paramsHandler.GetIntValue(ParamNames.GUSSET_BRACE_WELD_ENABLED);
            m_horiNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BRACE_HORI_NOTCH_LENGTH);
            m_vertNotch = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_BRACE_VERT_NOTCH_LENGTH);

            m_plateShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_OVERALL_SHAPE);
            return true;
        }
        private void CreateWelds()
        {
            if (m_braceWeldEnabled == 2)
                return;

            if (m_weldProps[1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                return;

            if (m_plateShape == 2)
            {
                WeldingSquarePlate();
                return;
            }

            foreach (var brace in m_braceData)
            {
                var planes = BraceTools.GetBracePlane(brace.BeamPlanes, brace.ProfileType);

                if (FindWeldPlane(planes, m_baseGussetPlane) != null)
                {
                    //有一个面贴合节点板的情形
                    WeldingBraceAndPlate(brace);
                    continue;
                }

                var dist = BraceTools.ArrayMax(GetPlaneDist(m_baseGussetPlane, brace.BackPlane, brace.FrontPlane));
                if (dist > brace.BackPlane.Dist(brace.FrontPlane.Origin))
                    continue;
                //节点板在支撑中间的情形
                if (m_context.EnableNotch)
                {
                    WeldingNotchBrace(brace, planes);
                }
                else if (m_context.EnableSpecialNotch)
                {
                    WeldingSpecialNotchBrace(brace, planes);
                }
            }
        }
        /// <summary>
        /// 创建支撑和节点板的焊缝
        /// </summary>
        private void WeldingBraceAndPlate(BraceData brace)
        {
            List<Vec3> weldPoints = new List<Vec3>();
            var Target = m_context.TargetComp;
            var plane = brace.AlignPlane;
            weldPoints.Add(plane.Intersect(brace.TopPlane, m_baseGussetPlane));
            weldPoints.Add(brace.GussetBorderPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
            weldPoints.Add(brace.GussetBorderPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
            weldPoints.Add(plane.Intersect(brace.BottomPlane, m_baseGussetPlane));
            for (int i = 0; i < weldPoints.Count; i++)
            {
                Weld curWeld = Target.CreatePolyWeld(WeldMain, brace.Beam,
                    new Vec3[] { weldPoints[i], weldPoints[(i + 1) % weldPoints.Count] }, true);
                m_weldProps[1].ApplyToWeld(curWeld);
            }
        }
        /// <summary>
        /// 创建方形节点板的焊缝
        /// </summary>
        private void WeldingSquarePlate()
        {
            //自动焊缝功能——————粗略版
            var Target = m_context.TargetComp;
            var upPoints = m_context.UpPoints.ToArray();
            var downPoints = m_context.DownPoints.ToArray();
            var Z = m_context.Z;

            {
                //左侧
                var upPoint = m_context.UpPoints[0];
                var len = downPoints.Length - 1;
                var downPoint = m_context.DownPoints[len];
                Plane upPlane = new Plane(upPoint, upPoints[1] - upPoint, Z);
                Plane leftPlane = new Plane(upPoint, downPoint - upPoint, Z);
                Plane downPlane = new Plane(downPoint, downPoints[len - 1] - downPoint, Z);

                foreach(var brace in m_context.LeftBraceData)
                {
                    List<Vec3> weldPoints = new List<Vec3>();
                    var plane = brace.AlignPlane;
                    if (brace.TopPlane.SignedDist(upPoint) < 0 && brace.BottomPlane.SignedDist(upPoint) < 0)
                    {
                        weldPoints.Add(upPoint);
                        weldPoints.Add(upPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
                    }
                    if (brace.TopPlane.SignedDist(upPoint) > 0 && brace.TopPlane.SignedDist(downPoint) < 0)
                        weldPoints.Add(leftPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
                    if (brace.BottomPlane.SignedDist(upPoint) > 0)
                        weldPoints.Add(upPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                    weldPoints.Add(plane.Intersect(brace.TopPlane, m_baseGussetPlane));
                    weldPoints.Add(plane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                    if (brace.BottomPlane.SignedDist(upPoint) < 0 && brace.BottomPlane.SignedDist(downPoint) > 0)
                        weldPoints.Add(leftPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                    if (brace.TopPlane.SignedDist(downPoint) < 0 && brace.BottomPlane.SignedDist(downPoint) < 0)
                    {
                        weldPoints.Add(downPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                        weldPoints.Add(downPoint);
                    }
                    for (int i = 0; i < weldPoints.Count; i++)
                    {
                        Weld curWeld = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                            new Vec3[] { weldPoints[i], weldPoints[(i + 1) % weldPoints.Count] }, true);
                        m_weldProps[1].ApplyToWeld(curWeld);
                    }
                }
            }

            {
                //右侧
                var len = upPoints.Length - 1;
                var upPoint = m_context.UpPoints[len];
                var downPoint = m_context.DownPoints[0];
                Plane upPlane = new Plane(upPoint, upPoints[len - 1] - upPoint, Z);
                Plane rightPlane = new Plane(upPoint, downPoint - upPoint, Z);
                Plane downPlane = new Plane(downPoint, downPoints[1] - downPoint, Z);

                foreach (var brace in m_context.RightBraceData)
                {
                    List<Vec3> weldPoints = new List<Vec3>();
                    var plane = brace.AlignPlane;
                    if (brace.TopPlane.SignedDist(upPoint) < 0 && brace.BottomPlane.SignedDist(upPoint) < 0)
                    {
                        weldPoints.Add(upPoint);
                        weldPoints.Add(upPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
                    }
                    if (brace.TopPlane.SignedDist(upPoint) > 0 && brace.TopPlane.SignedDist(downPoint) < 0)
                        weldPoints.Add(rightPlane.Intersect(brace.TopPlane, m_baseGussetPlane));
                    if (brace.BottomPlane.SignedDist(upPoint) > 0)
                        weldPoints.Add(upPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                    weldPoints.Add(plane.Intersect(brace.TopPlane, m_baseGussetPlane));
                    weldPoints.Add(plane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                    if (brace.BottomPlane.SignedDist(upPoint) < 0 && brace.BottomPlane.SignedDist(downPoint) > 0)
                        weldPoints.Add(rightPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                    if (brace.TopPlane.SignedDist(downPoint) < 0 && brace.BottomPlane.SignedDist(downPoint) < 0)
                    {
                        weldPoints.Add(downPlane.Intersect(brace.BottomPlane, m_baseGussetPlane));
                        weldPoints.Add(downPoint);
                    }
                    for (int i = 0; i < weldPoints.Count; i++)
                    {
                        Weld curWeld = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                            new Vec3[] { weldPoints[i], weldPoints[(i + 1) % weldPoints.Count] }, true);
                        m_weldProps[1].ApplyToWeld(curWeld);
                    }
                }
            }
        }
        /// <summary>
        /// 对开槽口的支撑进行焊接
        /// </summary>
        private void WeldingNotchBrace(BraceData brace, Plane[] planes)
        {
            var weldPlanes = FindNotchWeldPlane(planes, m_baseGussetPlane);
            var Target = m_context.TargetComp;
            var zBottom = m_baseGussetPlane.Move(m_context.Thickness);
            if (m_horiNotch == 0.0)
            {
                //焊接水平方向
               foreach(var pls in weldPlanes)
                {
                    var weldPoint1 = pls.Intersect(m_baseGussetPlane, brace.GussetBorderPlane);
                    var weldPoint2 = pls.Intersect(brace.GussetBorderPlane,zBottom);
                    Weld curWeld = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                    new Vec3[] { weldPoint1, weldPoint2 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld);
                }
            }

            if(m_vertNotch == 0.0)
            {
                var borderPlane = brace.AlignPlane;
                //焊接垂直方向
                foreach (var pls in weldPlanes)
                {
                    var weldPoint1 = pls.Intersect(m_baseGussetPlane, brace.GussetBorderPlane);
                    var weldPoint2 = pls.Intersect(m_baseGussetPlane, borderPlane);
                    Weld curWeld1 = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                    new Vec3[] { weldPoint1, weldPoint2 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld1);

                    var weldPoint3 = pls.Intersect(zBottom, brace.GussetBorderPlane);
                    var weldPoint4 = pls.Intersect(zBottom, borderPlane);
                    Weld curWeld2 = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                    new Vec3[] { weldPoint3, weldPoint4 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld2);
                }
            }
        }
        /// <summary>
        /// 焊接特殊槽口的支撑
        /// </summary>
        private void WeldingSpecialNotchBrace(BraceData brace, Plane[] planes)
        {
            var weldPlanes = FindNotchWeldPlane(planes, m_baseGussetPlane);
            var Target = m_context.TargetComp;
            var zBottom = m_baseGussetPlane.Move(m_context.Thickness);

            double dist1 = brace.FrontPlane.Dist(m_baseGussetPlane.Origin);
            double dist2 = brace.FrontPlane.Dist(zBottom.Origin);

            //沿着Z轴负方向
            var plane = dist1 > dist2 ? zBottom : m_baseGussetPlane;
            var borderPlane = brace.AlignPlane;
            //焊接垂直方向
            foreach (var pls in weldPlanes)
            {
                var weldPoint1 = pls.Intersect(plane, brace.GussetBorderPlane);
                var weldPoint2 = pls.Intersect(plane, borderPlane);
                Weld curWeld = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                new Vec3[] { weldPoint1, weldPoint2 }, true);
                m_weldProps[1].ApplyToWeld(curWeld);
            }

            if (m_horiNotch == 0.0)
            {
                //焊接水平方向
                foreach (var pls in weldPlanes)
                {
                    var weldPoint1 = pls.Intersect(m_baseGussetPlane, brace.GussetBorderPlane);
                    var weldPoint2 = pls.Intersect(brace.GussetBorderPlane, zBottom);
                    Weld curWeld = Target.CreatePolyWeld(m_context.MainPart, brace.Beam,
                    new Vec3[] { weldPoint1, weldPoint2 }, true);
                    m_weldProps[1].ApplyToWeld(curWeld);
                }
            }
        }
        /// <summary>
        /// 创建节点板和主零件的焊缝
        /// </summary>
        private void WeldingMainAndPlate()
        {
            if (m_weldProps[0].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                return;

            var type = BraceTools.ParseBraceProfileType(m_context.MainPart);
            var planes = BraceTools.GetBracePlane(m_context.MainPlanes, type);
            if (planes == null)
                return;
            if (FindWeldPlane(planes, m_baseGussetPlane) == null)
                return;
            var weldPoints = GetMainWeldPoints();
            if (weldPoints == null)
                return;
            var Target = m_context.TargetComp;
            for (int i = 0; i < weldPoints.Length; i++)
            {
                Weld curWeld = Target.CreatePolyWeld(m_context.MainPart, m_retPlate,
                    new Vec3[] { weldPoints[i], weldPoints[(i + 1) % weldPoints.Length] }, true);
                m_weldProps[0].ApplyToWeld(curWeld);
            }
            WeldFlag = true;
        }
        //得到主零件的焊接点
        private Vec3[] GetMainWeldPoints()
        {
            var upPoints = m_context.UpPoints.ToArray();
            var downPoints = m_context.DownPoints.ToArray();
            var weldPoints = new List<Vec3>();
            var faceLeftBracePlane = m_context.FaceLeftBracePlane;
            var faceRightBracePlane = m_context.FaceRightBracePlane;

            if (upPoints.Length == 4)
            {
                weldPoints.Add(upPoints[1]);
                weldPoints.Add(upPoints[2]);
            }
            else if (upPoints.Length == 2)
            {
                var upPlane = new Plane(upPoints[0], upPoints[1] - upPoints[0], m_context.Z);
                weldPoints.Add(upPlane.Intersect(m_baseGussetPlane, faceLeftBracePlane));
                weldPoints.Add(upPlane.Intersect(m_baseGussetPlane, faceRightBracePlane));
            }
            else
                return null;

            if (downPoints.Length == 4)
            {
                weldPoints.Add(downPoints[1]);
                weldPoints.Add(downPoints[2]);
            }
            else if (downPoints.Length == 2)
            {
                var downPlane = new Plane(downPoints[0], downPoints[1] - downPoints[0], m_context.Z);
                weldPoints.Add(downPlane.Intersect(m_baseGussetPlane, faceRightBracePlane));
                weldPoints.Add(downPlane.Intersect(m_baseGussetPlane, faceLeftBracePlane));
            }
            else
                return null;

            return weldPoints.ToArray();
        }
        /// <summary>
        /// 确定焊接面
        /// </summary>
        private Plane FindWeldPlane(Plane[] planes, Plane plane)
        {
            var eps = 1e-3;//阈值
            var line = plane.Normal;
            foreach(var pls in planes)
            {
                if (pls.Normal.IsParallelTo(line) && pls.Dist(plane.Origin) < eps)
                    return pls;
            }
            return null;
        }
        private Plane[] FindNotchWeldPlane(Plane[] planes, Plane plane)
        {
            var line = plane.Normal;
            var list = new List<Plane>();
            foreach(var pls in planes)
            {
                if (Math.Abs(pls.Normal.Dot(line)) < 1e-3)
                    list.Add(pls);
            }
            return list.ToArray();
        }
        /// <summary>
        /// 得到面集合中所有面与给定面的距离（均相互平行）
        /// </summary>
        private double[] GetPlaneDist(Plane plane, params Plane[] planes)
        {
            List<double> dis = new List<double>();
            foreach (var pls in planes)
            {
                if (pls == null)
                    return null;
                dis.Add(plane.Dist(pls.Origin));
            } 
            return dis.ToArray(); ;
        }
    }
}
