﻿using System;
using System.Collections.Generic;
using System.Drawing;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using stiffeners1030.Running;
using stiffeners1030.Tool;
using TargetComponentType = LoongCAD.STC.Object3D.DetailComponent;

namespace stiffeners1030.Main
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        public TargetComponentType TargetComp { get; private set; }
        public Beam MainPart { get; private set; }
        public BeamPlanes MainPlanes { get; private set; }
        public ParamsHandler ParamsHandler { get; set; }


        #region 组件内部自定义 
        public PublicParamData PublicParamData { get; set; }
        public BuildWeld BuildWeld { get; set; }
        public Vec3 MainCtrlPtA { get; private set; }
        public Vec3 MainCtrlPtB { get; private set; }
        /// <summary>
        /// 远端控制点，指向选择点
        /// </summary>
        public Vec3 FarToDetail { get; private set; }
        /// <summary>
        /// 组件的工作坐标矩阵
        /// </summary>
        public Mat43 MainMat { get; set; }
        
        /// <summary>
        /// 细部点的坐标
        /// </summary>
        public Vec3 DetailPt { get; private set; }
        /// <summary>
        /// 细部点所处的面，法线位主零件的首尾控制点的连线
        /// </summary>
        public Plane DetailPl { get; set; }
        /// <summary>
        /// 主零件前后包围面的中心面
        /// </summary>
        public Plane CentreFrtBck { get; private set; }
        /// <summary>
        /// 主零件顶底包围面的中心面
        /// </summary>
        public Plane CentreTopBtm { get; private set; }
        /// <summary>
        /// 细部中心点的坐标
        /// </summary>
        public Vec3 DetailCentrePt { get; set; }
        #endregion

        public static ComponentInstanceContext CreatInstance(TargetComponentType comp)
        {
            var compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp)) 
                return compContext;
            compContext.Clear();
            return null;
        }
        
        //具体的组件实例
        private bool Initialize(TargetComponentType comp)
        {
            if (comp == null)
            {
                Console.WriteLine(@"comp is null");
                return false;
            }
            //把传过来的组件，放到上下文
            TargetComp = comp;
            //把目标组件的主构件尝试转化为梁
            if (!(comp.Main is Beam mainPart))
            {
                Console.WriteLine(@"comp.Main is not Beam");
                return false;
            }
            //赋值给组件的主要部分（一般情况是梁）
            MainPart = mainPart;
            //解析梁
            MainPlanes = BeamPlanes.CreateFromBeam(MainPart);
            if (!PlaneHelper.CheckTypeList(MainPlanes, PlaneHelper.BOUND_PLANE_TBFB)) {
                Console.WriteLine(@"MainPlanes without bound plane");
                return false;
            }
            var ctrlP = TargetComp.CtrlPoints[0];
            if (ctrlP == null) {return false;}
            DetailPt = ctrlP.WorldPos;
            //查看主控件的控制点
            var mainCtrlPointA = TargetComp.Main.CtrlPoints[0];
            var mainCtrlPointB = TargetComp.Main.CtrlPoints[1];
            if (mainCtrlPointA == null || mainCtrlPointB == null) {return false;}
            MainCtrlPtA = mainCtrlPointA.WorldPos;
            MainCtrlPtB = mainCtrlPointB.WorldPos;
            var lineAtoB = MainCtrlPtB - MainCtrlPtA; 
            var distA = MainCtrlPtA.Dist(DetailPt);
            var distB = MainCtrlPtB.Dist(DetailPt);
            if (distA >= distB) {
                FarToDetail = -lineAtoB;
            } else { 
                FarToDetail = lineAtoB;
            }

            CentreTopBtm = PlaneHelper.GetMidPlane(MainPlanes.TopBound, MainPlanes.BottomBound);
            CentreFrtBck= PlaneHelper.GetMidPlane(MainPlanes.FrontBound, MainPlanes.BackBound);
            DetailPl = new Plane(DetailPt, lineAtoB);
            DetailCentrePt = DetailPl.Intersect(CentreTopBtm, CentreFrtBck);
            
            MainMat = TargetComp.LocalMatrix;

#if DEBUG
            PrintLocalMatrix();
#endif
            
            return true;
        }

        private void PrintLocalMatrix()
        {
            var p = DetailCentrePt;
            TargetComp.CreateLine(p,PointHelper.Move(p,MainMat.X,100.0d),0.0d,Color.Red, true);
            TargetComp.CreateLine(p,PointHelper.Move(p,MainMat.Y,100.0d),0.0d,Color.Blue, true);
            TargetComp.CreateLine(p,PointHelper.Move(p,MainMat.Z,100.0d),0.0d,Color.Yellow, true);
        }


        private void Clear()
        {
            MainPart = null;
            MainPlanes = null;
        }

    }
}
