﻿using System;
using System.Collections.Generic;
using stiffeners1030.Main;

namespace stiffeners1030.Running
{
    public interface IInit
    {
        /// <summary>
        /// 无依赖性的准备操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Init();
    }

    public interface IPrepare
    {
        /// <summary>
        /// 有依赖性的准备操作
        /// </summary>
        /// <returns>是否成功</returns>
        bool Prepare();
    }

    public interface IExecute
    {
        /// <summary>
        /// 执行操作，在所有准备结束后的操作
        /// 负责执行相关的Create函数
        /// </summary>
        /// <returns>是否成功</returns>
        bool Execute();
    }

    public class RunningBaseClass:IInit,IPrepare,IExecute
    {
        /// <summary>
        /// 缓存上下文
        /// </summary>
        protected readonly ComponentInstanceContext m_context;
        /// <summary>
        /// 公用的数据
        /// </summary>
        protected readonly PublicParamData m_pub;
        protected RunningBaseClass(ComponentInstanceContext context)
        {
            m_context = context;
            m_pub = m_context.PublicParamData;
        }

        public virtual bool Init() {
            return true;
        }

        public virtual bool Prepare() {
            return true;
        }
        public virtual bool Execute() {
            return true;
        }

    }

    public class CompositeCommand : IExecute
    {
        private readonly List<IExecute> m_commands = new List<IExecute>();

        public void AddCommand(IExecute command)
        {
            m_commands.Add(command);
        }
        
        public bool Execute()
        {
            foreach (var command in m_commands)
            {
                if (!command.Execute())
                {
                    Console.WriteLine(command.GetType().Name + @" exe fail");
                    return false;
                }
                Console.WriteLine(command.GetType().Name + @" exe Successful");
            }
            return true;
        }
    }


    /// <summary>
    /// 汇总所有的派生类执行
    /// </summary>
    public class RunningHandler
    {
        private readonly List<RunningBaseClass> m_run = new List<RunningBaseClass>();
        private readonly List<CompositeCommand> m_execute = new List<CompositeCommand>();

        public void Running(ComponentInstanceContext context)
        {
            var publicParamData = new PublicParamData(context);
            var buildWeld = new BuildWeld(context); 
            var buildStiffener = new BuildStiffener(context);
            var buildCutMain =new BuildCutMain(context);
            
            m_run.Add(publicParamData);
            m_run.Add(buildWeld);
            m_run.Add(buildCutMain);
            m_run.Add(buildStiffener);

            var commands = new CompositeCommand();
            commands.AddCommand(buildCutMain);
            commands.AddCommand(buildStiffener);
            m_execute.Add(commands);
            
            Run();
        }

        private void Run()
        {
            foreach (var run in m_run)
            {
                if (run.Init()) continue;
                Console.WriteLine(run.GetType().Name + @" init fail");
                return;
            }
            Console.WriteLine(@"all init success");
            foreach (var run in m_run)
            {
                if (run.Prepare()) continue;
                Console.WriteLine(run.GetType().Name + @" prepare fail");
                return;
            }
            Console.WriteLine(@"all prepare success");
            foreach (var exe in m_execute)
            {
                if (!exe.Execute()) {
                    return;
                }
            }
            Console.WriteLine(@"all exe success");
        }
    }

}